/* Generated by Cython 0.29.35 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "name": "zeroconf._protocol.outgoing",
        "sources": [
            "src/zeroconf/_protocol/outgoing.py"
        ]
    },
    "module_name": "zeroconf._protocol.outgoing"
}
END: Cython Metadata */

#ifndef PY_SSIZE_T_CLEAN
#define PY_SSIZE_T_CLEAN
#endif /* PY_SSIZE_T_CLEAN */
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_29_35"
#define CYTHON_HEX_VERSION 0x001D23F0
#define CYTHON_FUTURE_DIVISION 1
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_NOGIL 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #if PY_VERSION_HEX < 0x03090000
    #undef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #elif !defined(CYTHON_PEP489_MULTI_PHASE_INIT)
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_NOGIL 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
#elif defined(PY_NOGIL)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_NOGIL 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #ifndef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE 1
  #endif
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #define CYTHON_COMPILING_IN_NOGIL 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS (PY_VERSION_HEX < 0x030C00A5)
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0 || PY_VERSION_HEX >= 0x030B00A2
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #if PY_VERSION_HEX >= 0x030B00A4
    #undef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 0
  #elif !defined(CYTHON_FAST_THREAD_STATE)
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL (PY_VERSION_HEX < 0x030A0000)
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
  #ifndef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS ((PY_VERSION_HEX >= 0x030600B1) && (PY_VERSION_HEX < 0x030C00A5))
  #endif
  #if PY_VERSION_HEX >= 0x030B00A4
    #undef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK 0
  #elif !defined(CYTHON_USE_EXC_INFO_STACK)
    #define CYTHON_USE_EXC_INFO_STACK (PY_VERSION_HEX >= 0x030700A3)
  #endif
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 1
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #if PY_MAJOR_VERSION < 3
    #include "longintrepr.h"
  #endif
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_DefaultClassType PyType_Type
#if PY_VERSION_HEX >= 0x030B00A1
    static CYTHON_INLINE PyCodeObject* __Pyx_PyCode_New(int a, int k, int l, int s, int f,
                                                    PyObject *code, PyObject *c, PyObject* n, PyObject *v,
                                                    PyObject *fv, PyObject *cell, PyObject* fn,
                                                    PyObject *name, int fline, PyObject *lnos) {
        PyObject *kwds=NULL, *argcount=NULL, *posonlyargcount=NULL, *kwonlyargcount=NULL;
        PyObject *nlocals=NULL, *stacksize=NULL, *flags=NULL, *replace=NULL, *call_result=NULL, *empty=NULL;
        const char *fn_cstr=NULL;
        const char *name_cstr=NULL;
        PyCodeObject* co=NULL;
        PyObject *type, *value, *traceback;
        PyErr_Fetch(&type, &value, &traceback);
        if (!(kwds=PyDict_New())) goto end;
        if (!(argcount=PyLong_FromLong(a))) goto end;
        if (PyDict_SetItemString(kwds, "co_argcount", argcount) != 0) goto end;
        if (!(posonlyargcount=PyLong_FromLong(0))) goto end;
        if (PyDict_SetItemString(kwds, "co_posonlyargcount", posonlyargcount) != 0) goto end;
        if (!(kwonlyargcount=PyLong_FromLong(k))) goto end;
        if (PyDict_SetItemString(kwds, "co_kwonlyargcount", kwonlyargcount) != 0) goto end;
        if (!(nlocals=PyLong_FromLong(l))) goto end;
        if (PyDict_SetItemString(kwds, "co_nlocals", nlocals) != 0) goto end;
        if (!(stacksize=PyLong_FromLong(s))) goto end;
        if (PyDict_SetItemString(kwds, "co_stacksize", stacksize) != 0) goto end;
        if (!(flags=PyLong_FromLong(f))) goto end;
        if (PyDict_SetItemString(kwds, "co_flags", flags) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_code", code) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_consts", c) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_names", n) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_varnames", v) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_freevars", fv) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_cellvars", cell) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_linetable", lnos) != 0) goto end;
        if (!(fn_cstr=PyUnicode_AsUTF8AndSize(fn, NULL))) goto end;
        if (!(name_cstr=PyUnicode_AsUTF8AndSize(name, NULL))) goto end;
        if (!(co = PyCode_NewEmpty(fn_cstr, name_cstr, fline))) goto end;
        if (!(replace = PyObject_GetAttrString((PyObject*)co, "replace"))) goto cleanup_code_too;
        if (!(empty = PyTuple_New(0))) goto cleanup_code_too; // unfortunately __pyx_empty_tuple isn't available here
        if (!(call_result = PyObject_Call(replace, empty, kwds))) goto cleanup_code_too;
        Py_XDECREF((PyObject*)co);
        co = (PyCodeObject*)call_result;
        call_result = NULL;
        if (0) {
            cleanup_code_too:
            Py_XDECREF((PyObject*)co);
            co = NULL;
        }
        end:
        Py_XDECREF(kwds);
        Py_XDECREF(argcount);
        Py_XDECREF(posonlyargcount);
        Py_XDECREF(kwonlyargcount);
        Py_XDECREF(nlocals);
        Py_XDECREF(stacksize);
        Py_XDECREF(replace);
        Py_XDECREF(call_result);
        Py_XDECREF(empty);
        if (type) {
            PyErr_Restore(type, value, traceback);
        }
        return co;
    }
#else
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#if PY_VERSION_HEX <= 0x030700A3 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030400A1
  #define PyMem_RawMalloc(n)           PyMem_Malloc(n)
  #define PyMem_RawRealloc(p, n)       PyMem_Realloc(p, n)
  #define PyMem_RawFree(p)             PyMem_Free(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if PY_VERSION_HEX < 0x030700A2 && !defined(PyThread_tss_create) && !defined(Py_tss_NEEDS_INIT)
#include "pythread.h"
#define Py_tss_NEEDS_INIT 0
typedef int Py_tss_t;
static CYTHON_INLINE int PyThread_tss_create(Py_tss_t *key) {
  *key = PyThread_create_key();
  return 0;
}
static CYTHON_INLINE Py_tss_t * PyThread_tss_alloc(void) {
  Py_tss_t *key = (Py_tss_t *)PyObject_Malloc(sizeof(Py_tss_t));
  *key = Py_tss_NEEDS_INIT;
  return key;
}
static CYTHON_INLINE void PyThread_tss_free(Py_tss_t *key) {
  PyObject_Free(key);
}
static CYTHON_INLINE int PyThread_tss_is_created(Py_tss_t *key) {
  return *key != Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE void PyThread_tss_delete(Py_tss_t *key) {
  PyThread_delete_key(*key);
  *key = Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE int PyThread_tss_set(Py_tss_t *key, void *value) {
  return PyThread_set_key_value(*key, value);
}
static CYTHON_INLINE void * PyThread_tss_get(Py_tss_t *key) {
  return PyThread_get_key_value(*key);
}
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1 && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStr(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
#else
#define __Pyx_PyDict_GetItemStr(dict, name)  PyDict_GetItem(dict, name)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #if PY_VERSION_HEX >= 0x030C0000
    #define __Pyx_PyUnicode_READY(op)       (0)
  #else
    #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                                0 : _PyUnicode_Ready((PyObject *)(op)))
  #endif
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #if PY_VERSION_HEX >= 0x030C0000
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_LENGTH(u))
  #else
    #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03090000
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : ((PyCompactUnicodeObject *)(u))->wstr_length))
    #else
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
    #endif
  #endif
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None || (PyString_Check(b) && !PyString_CheckExact(b)))) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#ifndef PyObject_Unicode
  #define PyObject_Unicode             PyObject_Str
#endif
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#if PY_VERSION_HEX >= 0x030900A4
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_SET_REFCNT(obj, refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SET_SIZE(obj, size)
#else
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_REFCNT(obj) = (refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SIZE(obj) = (size)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
#else
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsHash_t
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? ((void)(klass), PyMethod_New(func, self)) : __Pyx_NewRef(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif

#if defined(_WIN32) || defined(WIN32) || defined(MS_WINDOWS)
  #if !defined(_USE_MATH_DEFINES)
    #define _USE_MATH_DEFINES
  #endif
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif

#define __PYX_MARK_ERR_POS(f_index, lineno) \
    { __pyx_filename = __pyx_f[f_index]; (void)__pyx_filename; __pyx_lineno = lineno; (void)__pyx_lineno; __pyx_clineno = __LINE__; (void)__pyx_clineno; }
#define __PYX_ERR(f_index, lineno, Ln_error) \
    { __PYX_MARK_ERR_POS(f_index, lineno) goto Ln_error; }

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__zeroconf___protocol__outgoing
#define __PYX_HAVE_API__zeroconf___protocol__outgoing
/* Early includes */
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT (PY_MAJOR_VERSION >= 3 && __PYX_DEFAULT_STRING_ENCODING_IS_UTF8)
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject*);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c) + 1);
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime = NULL;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "src/zeroconf/_protocol/outgoing.py",
  "src/zeroconf/_protocol/outgoing.pxd",
  "stringsource",
  "src/zeroconf/_dns.pxd",
  "src/zeroconf/_protocol/incoming.pxd",
};

/*--- Type declarations ---*/
struct __pyx_obj_8zeroconf_4_dns_DNSEntry;
struct __pyx_obj_8zeroconf_4_dns_DNSQuestion;
struct __pyx_obj_8zeroconf_4_dns_DNSRecord;
struct __pyx_obj_8zeroconf_4_dns_DNSAddress;
struct __pyx_obj_8zeroconf_4_dns_DNSHinfo;
struct __pyx_obj_8zeroconf_4_dns_DNSPointer;
struct __pyx_obj_8zeroconf_4_dns_DNSText;
struct __pyx_obj_8zeroconf_4_dns_DNSService;
struct __pyx_obj_8zeroconf_4_dns_DNSNsec;
struct __pyx_obj_8zeroconf_4_dns_DNSRRSet;
struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming;
struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing;
struct __pyx_obj_8zeroconf_9_protocol_8outgoing___pyx_scope_struct___write_record;
struct __pyx_obj_8zeroconf_9_protocol_8outgoing___pyx_scope_struct_1_genexpr;

/* "_dns.pxd":21
 * cdef object current_time_millis
 * 
 * cdef class DNSEntry:             # <<<<<<<<<<<<<<
 * 
 *     cdef public object key
 */
struct __pyx_obj_8zeroconf_4_dns_DNSEntry {
  PyObject_HEAD
  struct __pyx_vtabstruct_8zeroconf_4_dns_DNSEntry *__pyx_vtab;
  PyObject *key;
  PyObject *name;
  PyObject *type;
  PyObject *class_;
  PyObject *unique;
};


/* "_dns.pxd":31
 *     cdef _dns_entry_matches(self, DNSEntry other)
 * 
 * cdef class DNSQuestion(DNSEntry):             # <<<<<<<<<<<<<<
 * 
 *     cdef public cython.int _hash
 */
struct __pyx_obj_8zeroconf_4_dns_DNSQuestion {
  struct __pyx_obj_8zeroconf_4_dns_DNSEntry __pyx_base;
  int _hash;
};


/* "_dns.pxd":35
 *     cdef public cython.int _hash
 * 
 * cdef class DNSRecord(DNSEntry):             # <<<<<<<<<<<<<<
 * 
 *     cdef public object ttl
 */
struct __pyx_obj_8zeroconf_4_dns_DNSRecord {
  struct __pyx_obj_8zeroconf_4_dns_DNSEntry __pyx_base;
  PyObject *ttl;
  PyObject *created;
};


/* "_dns.pxd":43
 * 
 * 
 * cdef class DNSAddress(DNSRecord):             # <<<<<<<<<<<<<<
 * 
 *     cdef public cython.int _hash
 */
struct __pyx_obj_8zeroconf_4_dns_DNSAddress {
  struct __pyx_obj_8zeroconf_4_dns_DNSRecord __pyx_base;
  int _hash;
  PyObject *address;
  PyObject *scope_id;
};


/* "_dns.pxd":52
 * 
 * 
 * cdef class DNSHinfo(DNSRecord):             # <<<<<<<<<<<<<<
 * 
 *     cdef public cython.int _hash
 */
struct __pyx_obj_8zeroconf_4_dns_DNSHinfo {
  struct __pyx_obj_8zeroconf_4_dns_DNSRecord __pyx_base;
  int _hash;
  PyObject *cpu;
  PyObject *os;
};


/* "_dns.pxd":61
 * 
 * 
 * cdef class DNSPointer(DNSRecord):             # <<<<<<<<<<<<<<
 * 
 *     cdef public cython.int _hash
 */
struct __pyx_obj_8zeroconf_4_dns_DNSPointer {
  struct __pyx_obj_8zeroconf_4_dns_DNSRecord __pyx_base;
  int _hash;
  PyObject *alias;
  PyObject *alias_key;
};


/* "_dns.pxd":70
 * 
 * 
 * cdef class DNSText(DNSRecord):             # <<<<<<<<<<<<<<
 * 
 *     cdef public cython.int _hash
 */
struct __pyx_obj_8zeroconf_4_dns_DNSText {
  struct __pyx_obj_8zeroconf_4_dns_DNSRecord __pyx_base;
  int _hash;
  PyObject *text;
};


/* "_dns.pxd":78
 * 
 * 
 * cdef class DNSService(DNSRecord):             # <<<<<<<<<<<<<<
 * 
 *     cdef public cython.int _hash
 */
struct __pyx_obj_8zeroconf_4_dns_DNSService {
  struct __pyx_obj_8zeroconf_4_dns_DNSRecord __pyx_base;
  int _hash;
  PyObject *priority;
  PyObject *weight;
  PyObject *port;
  PyObject *server;
  PyObject *server_key;
};


/* "_dns.pxd":90
 * 
 * 
 * cdef class DNSNsec(DNSRecord):             # <<<<<<<<<<<<<<
 * 
 *     cdef public cython.int _hash
 */
struct __pyx_obj_8zeroconf_4_dns_DNSNsec {
  struct __pyx_obj_8zeroconf_4_dns_DNSRecord __pyx_base;
  int _hash;
  PyObject *next_name;
  PyObject *rdtypes;
};


/* "_dns.pxd":99
 * 
 * 
 * cdef class DNSRRSet:             # <<<<<<<<<<<<<<
 * 
 *     cdef _record_sets
 */
struct __pyx_obj_8zeroconf_4_dns_DNSRRSet {
  PyObject_HEAD
  struct __pyx_vtabstruct_8zeroconf_4_dns_DNSRRSet *__pyx_vtab;
  PyObject *_record_sets;
  PyObject *_lookup;
};


/* "incoming.pxd":48
 * 
 * 
 * cdef class DNSIncoming:             # <<<<<<<<<<<<<<
 * 
 *     cdef bint _did_read_others
 */
struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming {
  PyObject_HEAD
  struct __pyx_vtabstruct_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_vtab;
  int _did_read_others;
  unsigned int flags;
  unsigned int offset;
  PyObject *data;
  unsigned int _data_len;
  PyObject *name_cache;
  PyObject *questions;
  PyObject *_answers;
  PyObject *id;
  unsigned int num_questions;
  unsigned int num_answers;
  unsigned int num_authorities;
  unsigned int num_additionals;
  PyObject *valid;
  PyObject *now;
  PyObject *scope_id;
  PyObject *source;
};


/* "zeroconf/_protocol/outgoing.pxd":17
 * 
 * 
 * cdef class DNSOutgoing:             # <<<<<<<<<<<<<<
 * 
 *     cdef public unsigned int flags
 */
struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing {
  PyObject_HEAD
  struct __pyx_vtabstruct_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_vtab;
  unsigned int flags;
  PyObject *finished;
  PyObject *id;
  int multicast;
  PyObject *packets_data;
  PyObject *names;
  PyObject *data;
  unsigned int size;
  PyObject *allow_long;
  PyObject *state;
  PyObject *questions;
  PyObject *answers;
  PyObject *authorities;
  PyObject *additionals;
};


/* "zeroconf/_protocol/outgoing.py":298
 *         self._write_int(record.ttl if now == 0 else record.get_remaining_ttl(now))
 * 
 *     def _write_record(self, record: DNSRecord, now: float) -> bool:             # <<<<<<<<<<<<<<
 *         """Writes a record (answer, authoritative answer, additional) to
 *         the packet.  Returns True on success, or False if we did not
 */
struct __pyx_obj_8zeroconf_9_protocol_8outgoing___pyx_scope_struct___write_record {
  PyObject_HEAD
  Py_ssize_t __pyx_v_index;
  struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self;
};


/* "zeroconf/_protocol/outgoing.py":311
 *         record.write(self)
 *         # Adjust size for the short we will write before this record
 *         length = sum(len(d) for d in self.data[index + 1 :])             # <<<<<<<<<<<<<<
 *         # Here we replace the 0 length short we wrote
 *         # before with the actual length
 */
struct __pyx_obj_8zeroconf_9_protocol_8outgoing___pyx_scope_struct_1_genexpr {
  PyObject_HEAD
  struct __pyx_obj_8zeroconf_9_protocol_8outgoing___pyx_scope_struct___write_record *__pyx_outer_scope;
  PyObject *__pyx_v_d;
  PyObject *__pyx_t_0;
  Py_ssize_t __pyx_t_1;
};



/* "_dns.pxd":21
 * cdef object current_time_millis
 * 
 * cdef class DNSEntry:             # <<<<<<<<<<<<<<
 * 
 *     cdef public object key
 */

struct __pyx_vtabstruct_8zeroconf_4_dns_DNSEntry {
  PyObject *(*_dns_entry_matches)(struct __pyx_obj_8zeroconf_4_dns_DNSEntry *, struct __pyx_obj_8zeroconf_4_dns_DNSEntry *);
};
static struct __pyx_vtabstruct_8zeroconf_4_dns_DNSEntry *__pyx_vtabptr_8zeroconf_4_dns_DNSEntry;


/* "_dns.pxd":31
 *     cdef _dns_entry_matches(self, DNSEntry other)
 * 
 * cdef class DNSQuestion(DNSEntry):             # <<<<<<<<<<<<<<
 * 
 *     cdef public cython.int _hash
 */

struct __pyx_vtabstruct_8zeroconf_4_dns_DNSQuestion {
  struct __pyx_vtabstruct_8zeroconf_4_dns_DNSEntry __pyx_base;
};
static struct __pyx_vtabstruct_8zeroconf_4_dns_DNSQuestion *__pyx_vtabptr_8zeroconf_4_dns_DNSQuestion;


/* "_dns.pxd":35
 *     cdef public cython.int _hash
 * 
 * cdef class DNSRecord(DNSEntry):             # <<<<<<<<<<<<<<
 * 
 *     cdef public object ttl
 */

struct __pyx_vtabstruct_8zeroconf_4_dns_DNSRecord {
  struct __pyx_vtabstruct_8zeroconf_4_dns_DNSEntry __pyx_base;
  PyObject *(*_suppressed_by_answer)(struct __pyx_obj_8zeroconf_4_dns_DNSRecord *, struct __pyx_obj_8zeroconf_4_dns_DNSRecord *);
};
static struct __pyx_vtabstruct_8zeroconf_4_dns_DNSRecord *__pyx_vtabptr_8zeroconf_4_dns_DNSRecord;


/* "_dns.pxd":43
 * 
 * 
 * cdef class DNSAddress(DNSRecord):             # <<<<<<<<<<<<<<
 * 
 *     cdef public cython.int _hash
 */

struct __pyx_vtabstruct_8zeroconf_4_dns_DNSAddress {
  struct __pyx_vtabstruct_8zeroconf_4_dns_DNSRecord __pyx_base;
  PyObject *(*_eq)(struct __pyx_obj_8zeroconf_4_dns_DNSAddress *, struct __pyx_obj_8zeroconf_4_dns_DNSAddress *);
};
static struct __pyx_vtabstruct_8zeroconf_4_dns_DNSAddress *__pyx_vtabptr_8zeroconf_4_dns_DNSAddress;


/* "_dns.pxd":52
 * 
 * 
 * cdef class DNSHinfo(DNSRecord):             # <<<<<<<<<<<<<<
 * 
 *     cdef public cython.int _hash
 */

struct __pyx_vtabstruct_8zeroconf_4_dns_DNSHinfo {
  struct __pyx_vtabstruct_8zeroconf_4_dns_DNSRecord __pyx_base;
  PyObject *(*_eq)(struct __pyx_obj_8zeroconf_4_dns_DNSHinfo *, struct __pyx_obj_8zeroconf_4_dns_DNSHinfo *);
};
static struct __pyx_vtabstruct_8zeroconf_4_dns_DNSHinfo *__pyx_vtabptr_8zeroconf_4_dns_DNSHinfo;


/* "_dns.pxd":61
 * 
 * 
 * cdef class DNSPointer(DNSRecord):             # <<<<<<<<<<<<<<
 * 
 *     cdef public cython.int _hash
 */

struct __pyx_vtabstruct_8zeroconf_4_dns_DNSPointer {
  struct __pyx_vtabstruct_8zeroconf_4_dns_DNSRecord __pyx_base;
  PyObject *(*_eq)(struct __pyx_obj_8zeroconf_4_dns_DNSPointer *, struct __pyx_obj_8zeroconf_4_dns_DNSPointer *);
};
static struct __pyx_vtabstruct_8zeroconf_4_dns_DNSPointer *__pyx_vtabptr_8zeroconf_4_dns_DNSPointer;


/* "_dns.pxd":70
 * 
 * 
 * cdef class DNSText(DNSRecord):             # <<<<<<<<<<<<<<
 * 
 *     cdef public cython.int _hash
 */

struct __pyx_vtabstruct_8zeroconf_4_dns_DNSText {
  struct __pyx_vtabstruct_8zeroconf_4_dns_DNSRecord __pyx_base;
  PyObject *(*_eq)(struct __pyx_obj_8zeroconf_4_dns_DNSText *, struct __pyx_obj_8zeroconf_4_dns_DNSText *);
};
static struct __pyx_vtabstruct_8zeroconf_4_dns_DNSText *__pyx_vtabptr_8zeroconf_4_dns_DNSText;


/* "_dns.pxd":78
 * 
 * 
 * cdef class DNSService(DNSRecord):             # <<<<<<<<<<<<<<
 * 
 *     cdef public cython.int _hash
 */

struct __pyx_vtabstruct_8zeroconf_4_dns_DNSService {
  struct __pyx_vtabstruct_8zeroconf_4_dns_DNSRecord __pyx_base;
  PyObject *(*_eq)(struct __pyx_obj_8zeroconf_4_dns_DNSService *, struct __pyx_obj_8zeroconf_4_dns_DNSService *);
};
static struct __pyx_vtabstruct_8zeroconf_4_dns_DNSService *__pyx_vtabptr_8zeroconf_4_dns_DNSService;


/* "_dns.pxd":90
 * 
 * 
 * cdef class DNSNsec(DNSRecord):             # <<<<<<<<<<<<<<
 * 
 *     cdef public cython.int _hash
 */

struct __pyx_vtabstruct_8zeroconf_4_dns_DNSNsec {
  struct __pyx_vtabstruct_8zeroconf_4_dns_DNSRecord __pyx_base;
  PyObject *(*_eq)(struct __pyx_obj_8zeroconf_4_dns_DNSNsec *, struct __pyx_obj_8zeroconf_4_dns_DNSNsec *);
};
static struct __pyx_vtabstruct_8zeroconf_4_dns_DNSNsec *__pyx_vtabptr_8zeroconf_4_dns_DNSNsec;


/* "_dns.pxd":99
 * 
 * 
 * cdef class DNSRRSet:             # <<<<<<<<<<<<<<
 * 
 *     cdef _record_sets
 */

struct __pyx_vtabstruct_8zeroconf_4_dns_DNSRRSet {
  PyObject *(*suppresses)(struct __pyx_obj_8zeroconf_4_dns_DNSRRSet *, struct __pyx_obj_8zeroconf_4_dns_DNSRecord *, int __pyx_skip_dispatch);
  PyObject *(*_get_lookup)(struct __pyx_obj_8zeroconf_4_dns_DNSRRSet *);
};
static struct __pyx_vtabstruct_8zeroconf_4_dns_DNSRRSet *__pyx_vtabptr_8zeroconf_4_dns_DNSRRSet;


/* "incoming.pxd":48
 * 
 * 
 * cdef class DNSIncoming:             # <<<<<<<<<<<<<<
 * 
 *     cdef bint _did_read_others
 */

struct __pyx_vtabstruct_8zeroconf_9_protocol_8incoming_DNSIncoming {
  PyObject *(*_decode_labels_at_offset)(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *, unsigned int, PyObject *, PyObject *);
  PyObject *(*_read_header)(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *);
  PyObject *(*_initial_parse)(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *);
  PyObject *(*_read_others)(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *);
  PyObject *(*_read_questions)(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *);
  PyObject *(*_read_character_string)(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *);
  PyObject *(*_read_string)(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *, unsigned int);
  PyObject *(*_read_record)(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *, PyObject *, unsigned int, PyObject *, PyObject *, unsigned int);
  PyObject *(*_read_bitmap)(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *, unsigned int);
  PyObject *(*_read_name)(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *);
};
static struct __pyx_vtabstruct_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_vtabptr_8zeroconf_9_protocol_8incoming_DNSIncoming;


/* "zeroconf/_protocol/outgoing.py":49
 * 
 * 
 * class DNSOutgoing:             # <<<<<<<<<<<<<<
 * 
 *     """Object representation of an outgoing packet"""
 */

struct __pyx_vtabstruct_8zeroconf_9_protocol_8outgoing_DNSOutgoing {
  PyObject *(*_reset_for_next_packet)(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *);
  PyObject *(*_write_byte)(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *, PyObject *);
  PyObject *(*_insert_short_at_start)(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *, PyObject *);
  PyObject *(*_replace_short)(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *, PyObject *, PyObject *);
  PyObject *(*_write_int)(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *, PyObject *);
  PyObject *(*_write_question)(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *, PyObject *);
  PyObject *(*_write_record_class)(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *, PyObject *);
  PyObject *(*_check_data_limit_or_rollback)(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *, PyObject *, PyObject *);
  PyObject *(*_write_questions_from_offset)(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *, PyObject *);
  PyObject *(*_write_answers_from_offset)(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *, PyObject *);
  PyObject *(*_write_records_from_offset)(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *, PyObject *, PyObject *);
  PyObject *(*_has_more_to_add)(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *, PyObject *, PyObject *, PyObject *, PyObject *);
  PyObject *(*_packets)(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *);
};
static struct __pyx_vtabstruct_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_vtabptr_8zeroconf_9_protocol_8outgoing_DNSOutgoing;

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* PyDictVersioning.proto */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __pyx_dict_cached_value;\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  do {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
} while(0)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  do {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
} while(0)
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* ListAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely((Py_TYPE(obj) == type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#define __Pyx_BUILD_ASSERT_EXPR(cond)\
    (sizeof(char [1 - 2*!(cond)]) - 1)
#ifndef Py_MEMBER_SIZE
#define Py_MEMBER_SIZE(type, member) sizeof(((type *)0)->member)
#endif
#if CYTHON_FAST_PYCALL
  static size_t __pyx_pyframe_localsplus_offset = 0;
  #include "frameobject.h"
#if PY_VERSION_HEX >= 0x030b00a6
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
  #define __Pxy_PyFrame_Initialize_Offsets()\
    ((void)__Pyx_BUILD_ASSERT_EXPR(sizeof(PyFrameObject) == offsetof(PyFrameObject, f_localsplus) + Py_MEMBER_SIZE(PyFrameObject, f_localsplus)),\
     (void)(__pyx_pyframe_localsplus_offset = ((size_t)PyFrame_Type.tp_basicsize) - Py_MEMBER_SIZE(PyFrameObject, f_localsplus)))
  #define __Pyx_PyFrame_GetLocalsplus(frame)\
    (assert(__pyx_pyframe_localsplus_offset), (PyObject **)(((char *)(frame)) + __pyx_pyframe_localsplus_offset))
#endif // CYTHON_FAST_PYCALL
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyObjectCall2Args.proto */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2);

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* PyIntCompare.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, long intval, long inplace);

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* unicode_tailmatch.proto */
static int __Pyx_PyUnicode_Tailmatch(
    PyObject* s, PyObject* substr, Py_ssize_t start, Py_ssize_t end, int direction);

/* PyUnicode_Substring.proto */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_Substring(
            PyObject* text, Py_ssize_t start, Py_ssize_t stop);

/* SliceObject.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(
        PyObject* obj, Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** py_start, PyObject** py_stop, PyObject** py_slice,
        int has_cstart, int has_cstop, int wraparound);

/* PyIntBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_OrObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyInt_OrObjC(op1, op2, intval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceOr(op1, op2) : PyNumber_Or(op1, op2))
#endif

/* PyIntBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_RshiftObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyInt_RshiftObjC(op1, op2, intval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceRshift(op1, op2) : PyNumber_Rshift(op1, op2))
#endif

/* PyIntBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_AndObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyInt_AndObjC(op1, op2, intval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceAnd(op1, op2) : PyNumber_And(op1, op2))
#endif

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* None.proto */
static CYTHON_INLINE void __Pyx_RaiseClosureNameError(const char *varname);

/* SliceTupleAndList.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyList_GetSlice(PyObject* src, Py_ssize_t start, Py_ssize_t stop);
static CYTHON_INLINE PyObject* __Pyx_PyTuple_GetSlice(PyObject* src, Py_ssize_t start, Py_ssize_t stop);
#else
#define __Pyx_PyList_GetSlice(seq, start, stop)   PySequence_GetSlice(seq, start, stop)
#define __Pyx_PyTuple_GetSlice(seq, start, stop)  PySequence_GetSlice(seq, start, stop)
#endif

/* SliceObject.proto */
#define __Pyx_PyObject_DelSlice(obj, cstart, cstop, py_start, py_stop, py_slice, has_cstart, has_cstop, wraparound)\
    __Pyx_PyObject_SetSlice(obj, (PyObject*)NULL, cstart, cstop, py_start, py_stop, py_slice, has_cstart, has_cstop, wraparound)
static CYTHON_INLINE int __Pyx_PyObject_SetSlice(
        PyObject* obj, PyObject* value, Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** py_start, PyObject** py_stop, PyObject** py_slice,
        int has_cstart, int has_cstop, int wraparound);

/* IterFinish.proto */
static CYTHON_INLINE int __Pyx_IterFinish(void);

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* PyObjectGetMethod.proto */
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method);

/* PyObjectCallMethod0.proto */
static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name);

/* RaiseNeedMoreValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* RaiseTooManyValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* UnpackItemEndCheck.proto */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected);

/* RaiseNoneIterError.proto */
static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void);

/* UnpackTupleError.proto */
static void __Pyx_UnpackTupleError(PyObject *, Py_ssize_t index);

/* UnpackTuple2.proto */
#define __Pyx_unpack_tuple2(tuple, value1, value2, is_tuple, has_known_size, decref_tuple)\
    (likely(is_tuple || PyTuple_Check(tuple)) ?\
        (likely(has_known_size || PyTuple_GET_SIZE(tuple) == 2) ?\
            __Pyx_unpack_tuple2_exact(tuple, value1, value2, decref_tuple) :\
            (__Pyx_UnpackTupleError(tuple, 2), -1)) :\
        __Pyx_unpack_tuple2_generic(tuple, value1, value2, has_known_size, decref_tuple))
static CYTHON_INLINE int __Pyx_unpack_tuple2_exact(
    PyObject* tuple, PyObject** value1, PyObject** value2, int decref_tuple);
static int __Pyx_unpack_tuple2_generic(
    PyObject* tuple, PyObject** value1, PyObject** value2, int has_known_size, int decref_tuple);

/* dict_iter.proto */
static CYTHON_INLINE PyObject* __Pyx_dict_iterator(PyObject* dict, int is_dict, PyObject* method_name,
                                                   Py_ssize_t* p_orig_length, int* p_is_dict);
static CYTHON_INLINE int __Pyx_dict_iter_next(PyObject* dict_or_iter, Py_ssize_t orig_length, Py_ssize_t* ppos,
                                              PyObject** pkey, PyObject** pvalue, PyObject** pitem, int is_dict);

/* ListCompAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_ListComp_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len)) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_ListComp_Append(L,x) PyList_Append(L,x)
#endif

/* PyIntBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_AddObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyInt_AddObjC(op1, op2, intval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceAdd(op1, op2) : PyNumber_Add(op1, op2))
#endif

/* StringJoin.proto */
#if PY_MAJOR_VERSION < 3
#define __Pyx_PyString_Join __Pyx_PyBytes_Join
#define __Pyx_PyBaseString_Join(s, v) (PyUnicode_CheckExact(s) ? PyUnicode_Join(s, v) : __Pyx_PyBytes_Join(s, v))
#else
#define __Pyx_PyString_Join PyUnicode_Join
#define __Pyx_PyBaseString_Join PyUnicode_Join
#endif
#if CYTHON_COMPILING_IN_CPYTHON
    #if PY_MAJOR_VERSION < 3
    #define __Pyx_PyBytes_Join _PyString_Join
    #else
    #define __Pyx_PyBytes_Join _PyBytes_Join
    #endif
#else
static CYTHON_INLINE PyObject* __Pyx_PyBytes_Join(PyObject* sep, PyObject* values);
#endif

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* GetAttr.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *, PyObject *);

/* GetAttr3.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *, PyObject *, PyObject *);

/* PySequenceContains.proto */
static CYTHON_INLINE int __Pyx_PySequence_ContainsTF(PyObject* item, PyObject* seq, int eq) {
    int result = PySequence_Contains(seq, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* HasAttr.proto */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *, PyObject *);

/* IncludeStringH.proto */
#include <string.h>

/* PyObject_GenericGetAttrNoDict.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttrNoDict PyObject_GenericGetAttr
#endif

/* PyObject_GenericGetAttr.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttr PyObject_GenericGetAttr
#endif

/* SetVTable.proto */
static int __Pyx_SetVtable(PyObject *dict, void *vtable);

/* PyObjectGetAttrStrNoError.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name);

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* TypeImport.proto */
#ifndef __PYX_HAVE_RT_ImportType_proto_0_29_35
#define __PYX_HAVE_RT_ImportType_proto_0_29_35
#if __STDC_VERSION__ >= 201112L
#include <stdalign.h>
#endif
#if __STDC_VERSION__ >= 201112L || __cplusplus >= 201103L
#define __PYX_GET_STRUCT_ALIGNMENT_0_29_35(s) alignof(s)
#else
#define __PYX_GET_STRUCT_ALIGNMENT_0_29_35(s) sizeof(void*)
#endif
enum __Pyx_ImportType_CheckSize_0_29_35 {
   __Pyx_ImportType_CheckSize_Error_0_29_35 = 0,
   __Pyx_ImportType_CheckSize_Warn_0_29_35 = 1,
   __Pyx_ImportType_CheckSize_Ignore_0_29_35 = 2
};
static PyTypeObject *__Pyx_ImportType_0_29_35(PyObject* module, const char *module_name, const char *class_name, size_t size, size_t alignment, enum __Pyx_ImportType_CheckSize_0_29_35 check_size);
#endif

/* GetVTable.proto */
static void* __Pyx_GetVtable(PyObject *dict);

/* CalculateMetaclass.proto */
static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases);

/* SetNameInClass.proto */
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1
#define __Pyx_SetNameInClass(ns, name, value)\
    (likely(PyDict_CheckExact(ns)) ? _PyDict_SetItem_KnownHash(ns, name, value, ((PyASCIIObject *) name)->hash) : PyObject_SetItem(ns, name, value))
#elif CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_SetNameInClass(ns, name, value)\
    (likely(PyDict_CheckExact(ns)) ? PyDict_SetItem(ns, name, value) : PyObject_SetItem(ns, name, value))
#else
#define __Pyx_SetNameInClass(ns, name, value)  PyObject_SetItem(ns, name, value)
#endif

/* Py3ClassCreate.proto */
static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name, PyObject *qualname,
                                           PyObject *mkw, PyObject *modname, PyObject *doc);
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases, PyObject *dict,
                                      PyObject *mkw, int calculate_metaclass, int allow_py2_metaclass);

/* FetchCommonType.proto */
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type);

/* CythonFunctionShared.proto */
#define __Pyx_CyFunction_USED 1
#define __Pyx_CYFUNCTION_STATICMETHOD  0x01
#define __Pyx_CYFUNCTION_CLASSMETHOD   0x02
#define __Pyx_CYFUNCTION_CCLASS        0x04
#define __Pyx_CyFunction_GetClosure(f)\
    (((__pyx_CyFunctionObject *) (f))->func_closure)
#define __Pyx_CyFunction_GetClassObj(f)\
    (((__pyx_CyFunctionObject *) (f))->func_classobj)
#define __Pyx_CyFunction_Defaults(type, f)\
    ((type *)(((__pyx_CyFunctionObject *) (f))->defaults))
#define __Pyx_CyFunction_SetDefaultsGetter(f, g)\
    ((__pyx_CyFunctionObject *) (f))->defaults_getter = (g)
typedef struct {
    PyCFunctionObject func;
#if PY_VERSION_HEX < 0x030500A0
    PyObject *func_weakreflist;
#endif
    PyObject *func_dict;
    PyObject *func_name;
    PyObject *func_qualname;
    PyObject *func_doc;
    PyObject *func_globals;
    PyObject *func_code;
    PyObject *func_closure;
    PyObject *func_classobj;
    void *defaults;
    int defaults_pyobjects;
    size_t defaults_size;  // used by FusedFunction for copying defaults
    int flags;
    PyObject *defaults_tuple;
    PyObject *defaults_kwdict;
    PyObject *(*defaults_getter)(PyObject *);
    PyObject *func_annotations;
} __pyx_CyFunctionObject;
static PyTypeObject *__pyx_CyFunctionType = 0;
#define __Pyx_CyFunction_Check(obj)  (__Pyx_TypeCheck(obj, __pyx_CyFunctionType))
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject* op, PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *self,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *m,
                                                         size_t size,
                                                         int pyobjects);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *m,
                                                            PyObject *tuple);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *m,
                                                             PyObject *dict);
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *m,
                                                              PyObject *dict);
static int __pyx_CyFunction_init(void);

/* CythonFunction.proto */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *closure,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);

/* ObjectGetItem.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject* key);
#else
#define __Pyx_PyObject_GetItem(obj, key)  PyObject_GetItem(obj, key)
#endif

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* GCCDiagnostics.proto */
#if defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
#define __Pyx_HAS_GCC_DIAGNOSTIC
#endif

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_unsigned_int(unsigned int value);

/* CIntFromPy.proto */
static CYTHON_INLINE unsigned int __Pyx_PyInt_As_unsigned_int(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)

/* GetTopmostException.proto */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem * __Pyx_PyErr_GetTopmostException(PyThreadState *tstate);
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* SwapException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSwap(type, value, tb)  __Pyx__ExceptionSwap(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* PyObjectCallMethod1.proto */
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg);

/* CoroutineBase.proto */
typedef PyObject *(*__pyx_coroutine_body_t)(PyObject *, PyThreadState *, PyObject *);
#if CYTHON_USE_EXC_INFO_STACK
#define __Pyx_ExcInfoStruct  _PyErr_StackItem
#else
typedef struct {
    PyObject *exc_type;
    PyObject *exc_value;
    PyObject *exc_traceback;
} __Pyx_ExcInfoStruct;
#endif
typedef struct {
    PyObject_HEAD
    __pyx_coroutine_body_t body;
    PyObject *closure;
    __Pyx_ExcInfoStruct gi_exc_state;
    PyObject *gi_weakreflist;
    PyObject *classobj;
    PyObject *yieldfrom;
    PyObject *gi_name;
    PyObject *gi_qualname;
    PyObject *gi_modulename;
    PyObject *gi_code;
    PyObject *gi_frame;
    int resume_label;
    char is_running;
} __pyx_CoroutineObject;
static __pyx_CoroutineObject *__Pyx__Coroutine_New(
    PyTypeObject *type, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
    PyObject *name, PyObject *qualname, PyObject *module_name);
static __pyx_CoroutineObject *__Pyx__Coroutine_NewInit(
            __pyx_CoroutineObject *gen, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name);
static CYTHON_INLINE void __Pyx_Coroutine_ExceptionClear(__Pyx_ExcInfoStruct *self);
static int __Pyx_Coroutine_clear(PyObject *self);
static PyObject *__Pyx_Coroutine_Send(PyObject *self, PyObject *value);
static PyObject *__Pyx_Coroutine_Close(PyObject *self);
static PyObject *__Pyx_Coroutine_Throw(PyObject *gen, PyObject *args);
#if CYTHON_USE_EXC_INFO_STACK
#define __Pyx_Coroutine_SwapException(self)
#define __Pyx_Coroutine_ResetAndClearException(self)  __Pyx_Coroutine_ExceptionClear(&(self)->gi_exc_state)
#else
#define __Pyx_Coroutine_SwapException(self) {\
    __Pyx_ExceptionSwap(&(self)->gi_exc_state.exc_type, &(self)->gi_exc_state.exc_value, &(self)->gi_exc_state.exc_traceback);\
    __Pyx_Coroutine_ResetFrameBackpointer(&(self)->gi_exc_state);\
    }
#define __Pyx_Coroutine_ResetAndClearException(self) {\
    __Pyx_ExceptionReset((self)->gi_exc_state.exc_type, (self)->gi_exc_state.exc_value, (self)->gi_exc_state.exc_traceback);\
    (self)->gi_exc_state.exc_type = (self)->gi_exc_state.exc_value = (self)->gi_exc_state.exc_traceback = NULL;\
    }
#endif
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyGen_FetchStopIterationValue(pvalue)\
    __Pyx_PyGen__FetchStopIterationValue(__pyx_tstate, pvalue)
#else
#define __Pyx_PyGen_FetchStopIterationValue(pvalue)\
    __Pyx_PyGen__FetchStopIterationValue(__Pyx_PyThreadState_Current, pvalue)
#endif
static int __Pyx_PyGen__FetchStopIterationValue(PyThreadState *tstate, PyObject **pvalue);
static CYTHON_INLINE void __Pyx_Coroutine_ResetFrameBackpointer(__Pyx_ExcInfoStruct *exc_state);

/* PatchModuleWithCoroutine.proto */
static PyObject* __Pyx_Coroutine_patch_module(PyObject* module, const char* py_code);

/* PatchGeneratorABC.proto */
static int __Pyx_patch_abc(void);

/* Generator.proto */
#define __Pyx_Generator_USED
static PyTypeObject *__pyx_GeneratorType = 0;
#define __Pyx_Generator_CheckExact(obj) (Py_TYPE(obj) == __pyx_GeneratorType)
#define __Pyx_Generator_New(body, code, closure, name, qualname, module_name)\
    __Pyx__Coroutine_New(__pyx_GeneratorType, body, code, closure, name, qualname, module_name)
static PyObject *__Pyx_Generator_Next(PyObject *self);
static int __pyx_Generator_init(void);

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* PyObjectSetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
#define __Pyx_PyObject_DelAttrStr(o,n) __Pyx_PyObject_SetAttrStr(o, n, NULL)
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value);
#else
#define __Pyx_PyObject_DelAttrStr(o,n)   PyObject_DelAttr(o,n)
#define __Pyx_PyObject_SetAttrStr(o,n,v) PyObject_SetAttr(o,n,v)
#endif

/* VoidPtrExport.proto */
static int __Pyx_ExportVoidPtr(PyObject *name, void *p, const char *sig);

/* VoidPtrImport.proto */
static int __Pyx_ImportVoidPtr_0_29_35(PyObject *module, const char *name, void **p, const char *sig);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

static PyObject *__pyx_f_8zeroconf_9_protocol_8outgoing_11DNSOutgoing__reset_for_next_packet(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_8zeroconf_9_protocol_8outgoing_11DNSOutgoing__write_byte(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_value); /* proto*/
static PyObject *__pyx_f_8zeroconf_9_protocol_8outgoing_11DNSOutgoing__insert_short_at_start(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_value); /* proto*/
static PyObject *__pyx_f_8zeroconf_9_protocol_8outgoing_11DNSOutgoing__replace_short(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_index, PyObject *__pyx_v_value); /* proto*/
static PyObject *__pyx_f_8zeroconf_9_protocol_8outgoing_11DNSOutgoing__write_int(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_value); /* proto*/
static PyObject *__pyx_f_8zeroconf_9_protocol_8outgoing_11DNSOutgoing__write_question(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_question); /* proto*/
static PyObject *__pyx_f_8zeroconf_9_protocol_8outgoing_11DNSOutgoing__write_record_class(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_record); /* proto*/
static PyObject *__pyx_f_8zeroconf_9_protocol_8outgoing_11DNSOutgoing__check_data_limit_or_rollback(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_start_data_length, PyObject *__pyx_v_start_size); /* proto*/
static PyObject *__pyx_f_8zeroconf_9_protocol_8outgoing_11DNSOutgoing__write_questions_from_offset(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_questions_offset); /* proto*/
static PyObject *__pyx_f_8zeroconf_9_protocol_8outgoing_11DNSOutgoing__write_answers_from_offset(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_answer_offset); /* proto*/
static PyObject *__pyx_f_8zeroconf_9_protocol_8outgoing_11DNSOutgoing__write_records_from_offset(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_records, PyObject *__pyx_v_offset); /* proto*/
static PyObject *__pyx_f_8zeroconf_9_protocol_8outgoing_11DNSOutgoing__has_more_to_add(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_questions_offset, PyObject *__pyx_v_answer_offset, PyObject *__pyx_v_authority_offset, PyObject *__pyx_v_additional_offset); /* proto*/
static PyObject *__pyx_f_8zeroconf_9_protocol_8outgoing_11DNSOutgoing__packets(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self); /* proto*/

/* Module declarations from 'cython' */

/* Module declarations from 'zeroconf._dns' */
static PyTypeObject *__pyx_ptype_8zeroconf_4_dns_DNSEntry = 0;
static PyTypeObject *__pyx_ptype_8zeroconf_4_dns_DNSQuestion = 0;
static PyTypeObject *__pyx_ptype_8zeroconf_4_dns_DNSRecord = 0;
static PyTypeObject *__pyx_ptype_8zeroconf_4_dns_DNSAddress = 0;
static PyTypeObject *__pyx_ptype_8zeroconf_4_dns_DNSHinfo = 0;
static PyTypeObject *__pyx_ptype_8zeroconf_4_dns_DNSPointer = 0;
static PyTypeObject *__pyx_ptype_8zeroconf_4_dns_DNSText = 0;
static PyTypeObject *__pyx_ptype_8zeroconf_4_dns_DNSService = 0;
static PyTypeObject *__pyx_ptype_8zeroconf_4_dns_DNSNsec = 0;
static PyTypeObject *__pyx_ptype_8zeroconf_4_dns_DNSRRSet = 0;
static PyObject **__pyx_vp_8zeroconf_4_dns__LEN_BYTE = 0;
#define __pyx_v_8zeroconf_4_dns__LEN_BYTE (*__pyx_vp_8zeroconf_4_dns__LEN_BYTE)
static PyObject **__pyx_vp_8zeroconf_4_dns__LEN_SHORT = 0;
#define __pyx_v_8zeroconf_4_dns__LEN_SHORT (*__pyx_vp_8zeroconf_4_dns__LEN_SHORT)
static PyObject **__pyx_vp_8zeroconf_4_dns__LEN_INT = 0;
#define __pyx_v_8zeroconf_4_dns__LEN_INT (*__pyx_vp_8zeroconf_4_dns__LEN_INT)
static PyObject **__pyx_vp_8zeroconf_4_dns__NAME_COMPRESSION_MIN_SIZE = 0;
#define __pyx_v_8zeroconf_4_dns__NAME_COMPRESSION_MIN_SIZE (*__pyx_vp_8zeroconf_4_dns__NAME_COMPRESSION_MIN_SIZE)
static PyObject **__pyx_vp_8zeroconf_4_dns__BASE_MAX_SIZE = 0;
#define __pyx_v_8zeroconf_4_dns__BASE_MAX_SIZE (*__pyx_vp_8zeroconf_4_dns__BASE_MAX_SIZE)
static PyObject **__pyx_vp_8zeroconf_4_dns__EXPIRE_FULL_TIME_MS = 0;
#define __pyx_v_8zeroconf_4_dns__EXPIRE_FULL_TIME_MS (*__pyx_vp_8zeroconf_4_dns__EXPIRE_FULL_TIME_MS)
static PyObject **__pyx_vp_8zeroconf_4_dns__EXPIRE_STALE_TIME_MS = 0;
#define __pyx_v_8zeroconf_4_dns__EXPIRE_STALE_TIME_MS (*__pyx_vp_8zeroconf_4_dns__EXPIRE_STALE_TIME_MS)
static PyObject **__pyx_vp_8zeroconf_4_dns__RECENT_TIME_MS = 0;
#define __pyx_v_8zeroconf_4_dns__RECENT_TIME_MS (*__pyx_vp_8zeroconf_4_dns__RECENT_TIME_MS)
static PyObject **__pyx_vp_8zeroconf_4_dns__CLASS_UNIQUE = 0;
#define __pyx_v_8zeroconf_4_dns__CLASS_UNIQUE (*__pyx_vp_8zeroconf_4_dns__CLASS_UNIQUE)
static PyObject **__pyx_vp_8zeroconf_4_dns__CLASS_MASK = 0;
#define __pyx_v_8zeroconf_4_dns__CLASS_MASK (*__pyx_vp_8zeroconf_4_dns__CLASS_MASK)
static PyObject **__pyx_vp_8zeroconf_4_dns_current_time_millis = 0;
#define __pyx_v_8zeroconf_4_dns_current_time_millis (*__pyx_vp_8zeroconf_4_dns_current_time_millis)

/* Module declarations from 'zeroconf._protocol.incoming' */
static PyTypeObject *__pyx_ptype_8zeroconf_9_protocol_8incoming_DNSIncoming = 0;
static unsigned int *__pyx_vp_8zeroconf_9_protocol_8incoming_DNS_COMPRESSION_HEADER_LEN = 0;
#define __pyx_v_8zeroconf_9_protocol_8incoming_DNS_COMPRESSION_HEADER_LEN (*__pyx_vp_8zeroconf_9_protocol_8incoming_DNS_COMPRESSION_HEADER_LEN)
static unsigned int *__pyx_vp_8zeroconf_9_protocol_8incoming_MAX_DNS_LABELS = 0;
#define __pyx_v_8zeroconf_9_protocol_8incoming_MAX_DNS_LABELS (*__pyx_vp_8zeroconf_9_protocol_8incoming_MAX_DNS_LABELS)
static unsigned int *__pyx_vp_8zeroconf_9_protocol_8incoming_DNS_COMPRESSION_POINTER_LEN = 0;
#define __pyx_v_8zeroconf_9_protocol_8incoming_DNS_COMPRESSION_POINTER_LEN (*__pyx_vp_8zeroconf_9_protocol_8incoming_DNS_COMPRESSION_POINTER_LEN)
static unsigned int *__pyx_vp_8zeroconf_9_protocol_8incoming_MAX_NAME_LENGTH = 0;
#define __pyx_v_8zeroconf_9_protocol_8incoming_MAX_NAME_LENGTH (*__pyx_vp_8zeroconf_9_protocol_8incoming_MAX_NAME_LENGTH)
static PyObject **__pyx_vp_8zeroconf_9_protocol_8incoming_current_time_millis = 0;
#define __pyx_v_8zeroconf_9_protocol_8incoming_current_time_millis (*__pyx_vp_8zeroconf_9_protocol_8incoming_current_time_millis)
static unsigned int *__pyx_vp_8zeroconf_9_protocol_8incoming__TYPE_A = 0;
#define __pyx_v_8zeroconf_9_protocol_8incoming__TYPE_A (*__pyx_vp_8zeroconf_9_protocol_8incoming__TYPE_A)
static unsigned int *__pyx_vp_8zeroconf_9_protocol_8incoming__TYPE_CNAME = 0;
#define __pyx_v_8zeroconf_9_protocol_8incoming__TYPE_CNAME (*__pyx_vp_8zeroconf_9_protocol_8incoming__TYPE_CNAME)
static unsigned int *__pyx_vp_8zeroconf_9_protocol_8incoming__TYPE_PTR = 0;
#define __pyx_v_8zeroconf_9_protocol_8incoming__TYPE_PTR (*__pyx_vp_8zeroconf_9_protocol_8incoming__TYPE_PTR)
static unsigned int *__pyx_vp_8zeroconf_9_protocol_8incoming__TYPE_TXT = 0;
#define __pyx_v_8zeroconf_9_protocol_8incoming__TYPE_TXT (*__pyx_vp_8zeroconf_9_protocol_8incoming__TYPE_TXT)
static unsigned int *__pyx_vp_8zeroconf_9_protocol_8incoming__TYPE_SRV = 0;
#define __pyx_v_8zeroconf_9_protocol_8incoming__TYPE_SRV (*__pyx_vp_8zeroconf_9_protocol_8incoming__TYPE_SRV)
static unsigned int *__pyx_vp_8zeroconf_9_protocol_8incoming__TYPE_HINFO = 0;
#define __pyx_v_8zeroconf_9_protocol_8incoming__TYPE_HINFO (*__pyx_vp_8zeroconf_9_protocol_8incoming__TYPE_HINFO)
static unsigned int *__pyx_vp_8zeroconf_9_protocol_8incoming__TYPE_AAAA = 0;
#define __pyx_v_8zeroconf_9_protocol_8incoming__TYPE_AAAA (*__pyx_vp_8zeroconf_9_protocol_8incoming__TYPE_AAAA)
static unsigned int *__pyx_vp_8zeroconf_9_protocol_8incoming__TYPE_NSEC = 0;
#define __pyx_v_8zeroconf_9_protocol_8incoming__TYPE_NSEC (*__pyx_vp_8zeroconf_9_protocol_8incoming__TYPE_NSEC)
static unsigned int *__pyx_vp_8zeroconf_9_protocol_8incoming__FLAGS_QR_MASK = 0;
#define __pyx_v_8zeroconf_9_protocol_8incoming__FLAGS_QR_MASK (*__pyx_vp_8zeroconf_9_protocol_8incoming__FLAGS_QR_MASK)
static unsigned int *__pyx_vp_8zeroconf_9_protocol_8incoming__FLAGS_TC = 0;
#define __pyx_v_8zeroconf_9_protocol_8incoming__FLAGS_TC (*__pyx_vp_8zeroconf_9_protocol_8incoming__FLAGS_TC)
static unsigned int *__pyx_vp_8zeroconf_9_protocol_8incoming__FLAGS_QR_QUERY = 0;
#define __pyx_v_8zeroconf_9_protocol_8incoming__FLAGS_QR_QUERY (*__pyx_vp_8zeroconf_9_protocol_8incoming__FLAGS_QR_QUERY)
static unsigned int *__pyx_vp_8zeroconf_9_protocol_8incoming__FLAGS_QR_RESPONSE = 0;
#define __pyx_v_8zeroconf_9_protocol_8incoming__FLAGS_QR_RESPONSE (*__pyx_vp_8zeroconf_9_protocol_8incoming__FLAGS_QR_RESPONSE)
static PyObject **__pyx_vp_8zeroconf_9_protocol_8incoming_UNPACK_3H = 0;
#define __pyx_v_8zeroconf_9_protocol_8incoming_UNPACK_3H (*__pyx_vp_8zeroconf_9_protocol_8incoming_UNPACK_3H)
static PyObject **__pyx_vp_8zeroconf_9_protocol_8incoming_UNPACK_6H = 0;
#define __pyx_v_8zeroconf_9_protocol_8incoming_UNPACK_6H (*__pyx_vp_8zeroconf_9_protocol_8incoming_UNPACK_6H)
static PyObject **__pyx_vp_8zeroconf_9_protocol_8incoming_UNPACK_HH = 0;
#define __pyx_v_8zeroconf_9_protocol_8incoming_UNPACK_HH (*__pyx_vp_8zeroconf_9_protocol_8incoming_UNPACK_HH)
static PyObject **__pyx_vp_8zeroconf_9_protocol_8incoming_UNPACK_HHiH = 0;
#define __pyx_v_8zeroconf_9_protocol_8incoming_UNPACK_HHiH (*__pyx_vp_8zeroconf_9_protocol_8incoming_UNPACK_HHiH)
static PyObject **__pyx_vp_8zeroconf_9_protocol_8incoming_DECODE_EXCEPTIONS = 0;
#define __pyx_v_8zeroconf_9_protocol_8incoming_DECODE_EXCEPTIONS (*__pyx_vp_8zeroconf_9_protocol_8incoming_DECODE_EXCEPTIONS)
static PyObject **__pyx_vp_8zeroconf_9_protocol_8incoming_IncomingDecodeError = 0;
#define __pyx_v_8zeroconf_9_protocol_8incoming_IncomingDecodeError (*__pyx_vp_8zeroconf_9_protocol_8incoming_IncomingDecodeError)

/* Module declarations from 'zeroconf._protocol.outgoing' */
static PyTypeObject *__pyx_ptype_8zeroconf_9_protocol_8outgoing_DNSOutgoing = 0;
static PyTypeObject *__pyx_ptype_8zeroconf_9_protocol_8outgoing___pyx_scope_struct___write_record = 0;
static PyTypeObject *__pyx_ptype_8zeroconf_9_protocol_8outgoing___pyx_scope_struct_1_genexpr = 0;
static unsigned int __pyx_v_8zeroconf_9_protocol_8outgoing__CLASS_UNIQUE;
static unsigned int __pyx_v_8zeroconf_9_protocol_8outgoing__DNS_PACKET_HEADER_LEN;
static unsigned int __pyx_v_8zeroconf_9_protocol_8outgoing__FLAGS_QR_MASK;
static unsigned int __pyx_v_8zeroconf_9_protocol_8outgoing__FLAGS_QR_QUERY;
static unsigned int __pyx_v_8zeroconf_9_protocol_8outgoing__FLAGS_QR_RESPONSE;
static unsigned int __pyx_v_8zeroconf_9_protocol_8outgoing__FLAGS_TC;
static unsigned int __pyx_v_8zeroconf_9_protocol_8outgoing__MAX_MSG_ABSOLUTE;
static unsigned int __pyx_v_8zeroconf_9_protocol_8outgoing__MAX_MSG_TYPICAL;
static PyObject *__pyx_f_8zeroconf_9_protocol_8outgoing___pyx_unpickle_DNSOutgoing__set_state(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *, PyObject *); /*proto*/
#define __Pyx_MODULE_NAME "zeroconf._protocol.outgoing"
extern int __pyx_module_is_main_zeroconf___protocol__outgoing;
int __pyx_module_is_main_zeroconf___protocol__outgoing = 0;

/* Implementation of 'zeroconf._protocol.outgoing' */
static PyObject *__pyx_builtin_range;
static PyObject *__pyx_builtin_sum;
static const char __pyx_k_[] = ", ";
static const char __pyx_k_s[] = "s";
static const char __pyx_k__5[] = ".";
static const char __pyx_k__6[] = "";
static const char __pyx_k_id[] = "id_";
static const char __pyx_k_big[] = "big";
static const char __pyx_k_dns[] = "_dns";
static const char __pyx_k_doc[] = "__doc__";
static const char __pyx_k_get[] = "get";
static const char __pyx_k_inp[] = "inp";
static const char __pyx_k_int[] = "int";
static const char __pyx_k_log[] = "log";
static const char __pyx_k_new[] = "__new__";
static const char __pyx_k_now[] = "now";
static const char __pyx_k_sum[] = "sum";
static const char __pyx_k_ttl[] = "ttl";
static const char __pyx_k_Dict[] = "Dict";
static const char __pyx_k_Enum[] = "Enum";
static const char __pyx_k_List[] = "List";
static const char __pyx_k_args[] = "args";
static const char __pyx_k_data[] = "data";
static const char __pyx_k_dict[] = "__dict__";
static const char __pyx_k_enum[] = "enum";
static const char __pyx_k_id_2[] = "id";
static const char __pyx_k_init[] = "init";
static const char __pyx_k_join[] = "join";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_name[] = "name";
static const char __pyx_k_self[] = "self";
static const char __pyx_k_send[] = "send";
static const char __pyx_k_size[] = "size";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_type[] = "type_";
static const char __pyx_k_DEBUG[] = "DEBUG";
static const char __pyx_k_State[] = "State";
static const char __pyx_k_Tuple[] = "Tuple";
static const char __pyx_k_Union[] = "Union";
static const char __pyx_k_bytes[] = "bytes";
static const char __pyx_k_cache[] = "cache";
static const char __pyx_k_class[] = "class_";
static const char __pyx_k_close[] = "close";
static const char __pyx_k_const[] = "const";
static const char __pyx_k_count[] = "count";
static const char __pyx_k_debug[] = "debug";
static const char __pyx_k_flags[] = "flags";
static const char __pyx_k_float[] = "float";
static const char __pyx_k_index[] = "index";
static const char __pyx_k_items[] = "items";
static const char __pyx_k_label[] = "label";
static const char __pyx_k_names[] = "names";
static const char __pyx_k_range[] = "range";
static const char __pyx_k_slots[] = "__slots__";
static const char __pyx_k_state[] = "state";
static const char __pyx_k_throw[] = "throw";
static const char __pyx_k_value[] = "value";
static const char __pyx_k_write[] = "write";
static const char __pyx_k_dict_2[] = "_dict";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_labels[] = "labels";
static const char __pyx_k_length[] = "length";
static const char __pyx_k_logger[] = "_logger";
static const char __pyx_k_module[] = "__module__";
static const char __pyx_k_name_2[] = "__name__";
static const char __pyx_k_pickle[] = "pickle";
static const char __pyx_k_record[] = "record";
static const char __pyx_k_reduce[] = "__reduce__";
static const char __pyx_k_return[] = "return";
static const char __pyx_k_type_2[] = "type";
static const char __pyx_k_typing[] = "typing";
static const char __pyx_k_unique[] = "unique";
static const char __pyx_k_update[] = "update";
static const char __pyx_k_utfstr[] = "utfstr";
static const char __pyx_k_answers[] = "answers";
static const char __pyx_k_cache_2[] = "_cache";
static const char __pyx_k_flags_s[] = "flags=%s";
static const char __pyx_k_genexpr[] = "genexpr";
static const char __pyx_k_logging[] = "logging";
static const char __pyx_k_packets[] = "packets";
static const char __pyx_k_prepare[] = "__prepare__";
static const char __pyx_k_unicode[] = "unicode";
static const char __pyx_k_warning[] = "warning";
static const char __pyx_k_DNSCache[] = "DNSCache";
static const char __pyx_k_FLAGS_TC[] = "_FLAGS_TC";
static const char __pyx_k_Optional[] = "Optional";
static const char __pyx_k_Sequence[] = "Sequence";
static const char __pyx_k_finished[] = "finished";
static const char __pyx_k_getstate[] = "__getstate__";
static const char __pyx_k_incoming[] = "incoming";
static const char __pyx_k_is_query[] = "is_query";
static const char __pyx_k_pyx_capi[] = "__pyx_capi__";
static const char __pyx_k_pyx_type[] = "__pyx_type";
static const char __pyx_k_qualname[] = "__qualname__";
static const char __pyx_k_setstate[] = "__setstate__";
static const char __pyx_k_to_bytes[] = "to_bytes";
static const char __pyx_k_DNSRecord[] = "DNSRecord";
static const char __pyx_k_answers_s[] = "answers=%s";
static const char __pyx_k_metaclass[] = "__metaclass__";
static const char __pyx_k_multicast[] = "multicast";
static const char __pyx_k_pyx_state[] = "__pyx_state";
static const char __pyx_k_questions[] = "questions";
static const char __pyx_k_reduce_ex[] = "__reduce_ex__";
static const char __pyx_k_write_ttl[] = "_write_ttl";
static const char __pyx_k_write_utf[] = "_write_utf";
static const char __pyx_k_DNSPointer[] = "DNSPointer";
static const char __pyx_k_add_answer[] = "add_answer";
static const char __pyx_k_allow_long[] = "allow_long";
static const char __pyx_k_exceptions[] = "_exceptions";
static const char __pyx_k_is_expired[] = "is_expired";
static const char __pyx_k_pyx_result[] = "__pyx_result";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_start_size[] = "start_size";
static const char __pyx_k_write_name[] = "write_name";
static const char __pyx_k_DNSIncoming[] = "DNSIncoming";
static const char __pyx_k_DNSOutgoing[] = "DNSOutgoing";
static const char __pyx_k_DNSQuestion[] = "DNSQuestion";
static const char __pyx_k_PickleError[] = "PickleError";
static const char __pyx_k_additionals[] = "additionals";
static const char __pyx_k_authorities[] = "authorities";
static const char __pyx_k_is_response[] = "is_response";
static const char __pyx_k_multicast_s[] = "multicast=%s";
static const char __pyx_k_name_length[] = "name_length";
static const char __pyx_k_questions_s[] = "questions=%s";
static const char __pyx_k_write_short[] = "write_short";
static const char __pyx_k_CLASS_UNIQUE[] = "_CLASS_UNIQUE";
static const char __pyx_k_add_question[] = "add_question";
static const char __pyx_k_cached_entry[] = "cached_entry";
static const char __pyx_k_isEnabledFor[] = "isEnabledFor";
static const char __pyx_k_packets_data[] = "packets_data";
static const char __pyx_k_pyx_checksum[] = "__pyx_checksum";
static const char __pyx_k_stringsource[] = "stringsource";
static const char __pyx_k_use_setstate[] = "use_setstate";
static const char __pyx_k_write_record[] = "_write_record";
static const char __pyx_k_write_string[] = "write_string";
static const char __pyx_k_DNSOutgoing_s[] = "<DNSOutgoing:{%s}>";
static const char __pyx_k_FLAGS_QR_MASK[] = "_FLAGS_QR_MASK";
static const char __pyx_k_additionals_s[] = "additionals=%s";
static const char __pyx_k_authorities_s[] = "authorities=%s";
static const char __pyx_k_reduce_cython[] = "__reduce_cython__";
static const char __pyx_k_suppressed_by[] = "suppressed_by";
static const char __pyx_k_FLAGS_QR_QUERY[] = "_FLAGS_QR_QUERY";
static const char __pyx_k_cached_entries[] = "cached_entries";
static const char __pyx_k_get_by_details[] = "get_by_details";
static const char __pyx_k_MAX_MSG_TYPICAL[] = "_MAX_MSG_TYPICAL";
static const char __pyx_k_Setting_TC_flag[] = "Setting TC flag";
static const char __pyx_k_pyx_PickleError[] = "__pyx_PickleError";
static const char __pyx_k_setstate_cython[] = "__setstate_cython__";
static const char __pyx_k_MAX_MSG_ABSOLUTE[] = "_MAX_MSG_ABSOLUTE";
static const char __pyx_k_FLAGS_QR_RESPONSE[] = "_FLAGS_QR_RESPONSE";
static const char __pyx_k_get_remaining_ttl[] = "get_remaining_ttl";
static const char __pyx_k_start_data_length[] = "start_data_length";
static const char __pyx_k_add_answer_at_time[] = "add_answer_at_time";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_get_all_by_details[] = "get_all_by_details";
static const char __pyx_k_DNSOutgoing_packets[] = "DNSOutgoing.packets";
static const char __pyx_k_DNSOutgoing_is_query[] = "DNSOutgoing.is_query";
static const char __pyx_k_DNS_PACKET_HEADER_LEN[] = "_DNS_PACKET_HEADER_LEN";
static const char __pyx_k_add_additional_answer[] = "add_additional_answer";
static const char __pyx_k_DNSOutgoing__write_ttl[] = "DNSOutgoing._write_ttl";
static const char __pyx_k_DNSOutgoing__write_utf[] = "DNSOutgoing._write_utf";
static const char __pyx_k_DNSOutgoing_add_answer[] = "DNSOutgoing.add_answer";
static const char __pyx_k_DNSOutgoing_write_name[] = "DNSOutgoing.write_name";
static const char __pyx_k_add_authorative_answer[] = "add_authorative_answer";
static const char __pyx_k_write_character_string[] = "write_character_string";
static const char __pyx_k_DNSOutgoing_is_response[] = "DNSOutgoing.is_response";
static const char __pyx_k_DNSOutgoing_write_short[] = "DNSOutgoing.write_short";
static const char __pyx_k_DNSOutgoing_add_question[] = "DNSOutgoing.add_question";
static const char __pyx_k_DNSOutgoing_write_string[] = "DNSOutgoing.write_string";
static const char __pyx_k_NamePartTooLongException[] = "NamePartTooLongException";
static const char __pyx_k_pyx_unpickle_DNSOutgoing[] = "__pyx_unpickle_DNSOutgoing";
static const char __pyx_k_DNSOutgoing__write_record[] = "DNSOutgoing._write_record";
static const char __pyx_k_add_question_or_all_cache[] = "add_question_or_all_cache";
static const char __pyx_k_add_question_or_one_cache[] = "add_question_or_one_cache";
static const char __pyx_k_DNSOutgoing___reduce_cython[] = "DNSOutgoing.__reduce_cython__";
static const char __pyx_k_write_record_locals_genexpr[] = "_write_record.<locals>.genexpr";
static const char __pyx_k_zeroconf__protocol_outgoing[] = "zeroconf._protocol.outgoing";
static const char __pyx_k_DNSOutgoing___setstate_cython[] = "DNSOutgoing.__setstate_cython__";
static const char __pyx_k_DNSOutgoing_add_answer_at_time[] = "DNSOutgoing.add_answer_at_time";
static const char __pyx_k_DNSOutgoing_add_question_or_all[] = "DNSOutgoing.add_question_or_all_cache";
static const char __pyx_k_DNSOutgoing_add_question_or_one[] = "DNSOutgoing.add_question_or_one_cache";
static const char __pyx_k_Multicast_DNS_Service_Discovery[] = " Multicast DNS Service Discovery for Python, v0.14-wmcbrine\n    Copyright 2003 Paul Scott-Murphy, 2014 William McBrine\n\n    This module provides a framework for the use of DNS Service Discovery\n    using IP multicast.\n\n    This library is free software; you can redistribute it and/or\n    modify it under the terms of the GNU Lesser General Public\n    License as published by the Free Software Foundation; either\n    version 2.1 of the License, or (at your option) any later version.\n\n    This library is distributed in the hope that it will be useful,\n    but WITHOUT ANY WARRANTY; without even the implied warranty of\n    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU\n    Lesser General Public License for more details.\n\n    You should have received a copy of the GNU Lesser General Public\n    License along with this library; if not, write to the Free Software\n    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301\n    USA\n";
static const char __pyx_k_Reached_data_limit_size_d_limit[] = "Reached data limit (size=%d) > (limit=%d) - rolling back";
static const char __pyx_k_now_offsets_questions_d_answers[] = "now offsets = questions=%d, answers=%d, authorities=%d, additionals=%d";
static const char __pyx_k_packets_made_no_progress_adding[] = "packets() made no progress adding records; returning";
static const char __pyx_k_src_zeroconf__protocol_outgoing[] = "src/zeroconf/_protocol/outgoing.py";
static const char __pyx_k_DNSOutgoing_add_additional_answe[] = "DNSOutgoing.add_additional_answer";
static const char __pyx_k_DNSOutgoing_add_authorative_answ[] = "DNSOutgoing.add_authorative_answer";
static const char __pyx_k_DNSOutgoing_write_character_stri[] = "DNSOutgoing.write_character_string";
static const char __pyx_k_Incompatible_checksums_0x_x_vs_0[] = "Incompatible checksums (0x%x vs (0x3d398d7, 0xa242541, 0x5aef2cc) = (additionals, allow_long, answers, authorities, data, finished, flags, id, multicast, names, packets_data, questions, size, state))";
static const char __pyx_k_lengths_questions_d_answers_d_au[] = "lengths = questions=%d, answers=%d, authorities=%d, additionals=%d";
static const char __pyx_k_offsets_questions_d_answers_d_au[] = "offsets = questions=%d, answers=%d, authorities=%d, additionals=%d";
static PyObject *__pyx_kp_u_;
static PyObject *__pyx_n_s_CLASS_UNIQUE;
static PyObject *__pyx_n_s_DEBUG;
static PyObject *__pyx_n_s_DNSCache;
static PyObject *__pyx_n_s_DNSIncoming;
static PyObject *__pyx_n_u_DNSIncoming;
static PyObject *__pyx_n_s_DNSOutgoing;
static PyObject *__pyx_n_s_DNSOutgoing___reduce_cython;
static PyObject *__pyx_n_s_DNSOutgoing___setstate_cython;
static PyObject *__pyx_n_s_DNSOutgoing__write_record;
static PyObject *__pyx_n_s_DNSOutgoing__write_ttl;
static PyObject *__pyx_n_s_DNSOutgoing__write_utf;
static PyObject *__pyx_n_s_DNSOutgoing_add_additional_answe;
static PyObject *__pyx_n_s_DNSOutgoing_add_answer;
static PyObject *__pyx_n_s_DNSOutgoing_add_answer_at_time;
static PyObject *__pyx_n_s_DNSOutgoing_add_authorative_answ;
static PyObject *__pyx_n_s_DNSOutgoing_add_question;
static PyObject *__pyx_n_s_DNSOutgoing_add_question_or_all;
static PyObject *__pyx_n_s_DNSOutgoing_add_question_or_one;
static PyObject *__pyx_n_s_DNSOutgoing_is_query;
static PyObject *__pyx_n_s_DNSOutgoing_is_response;
static PyObject *__pyx_n_s_DNSOutgoing_packets;
static PyObject *__pyx_kp_u_DNSOutgoing_s;
static PyObject *__pyx_n_s_DNSOutgoing_write_character_stri;
static PyObject *__pyx_n_s_DNSOutgoing_write_name;
static PyObject *__pyx_n_s_DNSOutgoing_write_short;
static PyObject *__pyx_n_s_DNSOutgoing_write_string;
static PyObject *__pyx_n_s_DNSPointer;
static PyObject *__pyx_n_s_DNSQuestion;
static PyObject *__pyx_n_s_DNSRecord;
static PyObject *__pyx_n_s_DNS_PACKET_HEADER_LEN;
static PyObject *__pyx_n_s_Dict;
static PyObject *__pyx_n_s_Enum;
static PyObject *__pyx_n_s_FLAGS_QR_MASK;
static PyObject *__pyx_n_s_FLAGS_QR_QUERY;
static PyObject *__pyx_n_s_FLAGS_QR_RESPONSE;
static PyObject *__pyx_n_s_FLAGS_TC;
static PyObject *__pyx_kp_s_Incompatible_checksums_0x_x_vs_0;
static PyObject *__pyx_n_s_List;
static PyObject *__pyx_n_s_MAX_MSG_ABSOLUTE;
static PyObject *__pyx_n_s_MAX_MSG_TYPICAL;
static PyObject *__pyx_n_s_NamePartTooLongException;
static PyObject *__pyx_n_s_Optional;
static PyObject *__pyx_n_s_PickleError;
static PyObject *__pyx_kp_u_Reached_data_limit_size_d_limit;
static PyObject *__pyx_n_s_Sequence;
static PyObject *__pyx_kp_u_Setting_TC_flag;
static PyObject *__pyx_n_s_State;
static PyObject *__pyx_n_s_Tuple;
static PyObject *__pyx_n_s_Union;
static PyObject *__pyx_kp_u__5;
static PyObject *__pyx_kp_b__6;
static PyObject *__pyx_n_s_add_additional_answer;
static PyObject *__pyx_n_s_add_answer;
static PyObject *__pyx_n_s_add_answer_at_time;
static PyObject *__pyx_n_s_add_authorative_answer;
static PyObject *__pyx_n_s_add_question;
static PyObject *__pyx_n_s_add_question_or_all_cache;
static PyObject *__pyx_n_s_add_question_or_one_cache;
static PyObject *__pyx_n_u_additionals;
static PyObject *__pyx_kp_u_additionals_s;
static PyObject *__pyx_n_u_allow_long;
static PyObject *__pyx_n_u_answers;
static PyObject *__pyx_kp_u_answers_s;
static PyObject *__pyx_n_s_args;
static PyObject *__pyx_n_u_authorities;
static PyObject *__pyx_kp_u_authorities_s;
static PyObject *__pyx_n_u_big;
static PyObject *__pyx_n_u_bytes;
static PyObject *__pyx_n_s_cache;
static PyObject *__pyx_n_s_cache_2;
static PyObject *__pyx_n_s_cached_entries;
static PyObject *__pyx_n_s_cached_entry;
static PyObject *__pyx_n_s_class;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_close;
static PyObject *__pyx_n_s_const;
static PyObject *__pyx_n_s_count;
static PyObject *__pyx_n_u_data;
static PyObject *__pyx_n_s_debug;
static PyObject *__pyx_n_s_dict;
static PyObject *__pyx_n_s_dict_2;
static PyObject *__pyx_n_s_dns;
static PyObject *__pyx_n_s_doc;
static PyObject *__pyx_n_s_enum;
static PyObject *__pyx_n_s_exceptions;
static PyObject *__pyx_n_s_finished;
static PyObject *__pyx_n_u_finished;
static PyObject *__pyx_n_s_flags;
static PyObject *__pyx_n_u_flags;
static PyObject *__pyx_kp_u_flags_s;
static PyObject *__pyx_n_u_float;
static PyObject *__pyx_n_s_genexpr;
static PyObject *__pyx_n_s_get;
static PyObject *__pyx_n_s_get_all_by_details;
static PyObject *__pyx_n_s_get_by_details;
static PyObject *__pyx_n_s_get_remaining_ttl;
static PyObject *__pyx_n_s_getstate;
static PyObject *__pyx_n_s_id;
static PyObject *__pyx_n_u_id_2;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_incoming;
static PyObject *__pyx_n_s_index;
static PyObject *__pyx_n_s_init;
static PyObject *__pyx_n_s_inp;
static PyObject *__pyx_n_u_int;
static PyObject *__pyx_n_s_isEnabledFor;
static PyObject *__pyx_n_s_is_expired;
static PyObject *__pyx_n_s_is_query;
static PyObject *__pyx_n_s_is_response;
static PyObject *__pyx_n_s_items;
static PyObject *__pyx_n_s_join;
static PyObject *__pyx_n_s_label;
static PyObject *__pyx_n_s_labels;
static PyObject *__pyx_n_s_length;
static PyObject *__pyx_kp_u_lengths_questions_d_answers_d_au;
static PyObject *__pyx_n_s_log;
static PyObject *__pyx_n_s_logger;
static PyObject *__pyx_n_s_logging;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_metaclass;
static PyObject *__pyx_n_s_module;
static PyObject *__pyx_n_s_multicast;
static PyObject *__pyx_n_u_multicast;
static PyObject *__pyx_kp_u_multicast_s;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_s_name_2;
static PyObject *__pyx_n_s_name_length;
static PyObject *__pyx_n_u_names;
static PyObject *__pyx_n_s_new;
static PyObject *__pyx_n_s_now;
static PyObject *__pyx_kp_u_now_offsets_questions_d_answers;
static PyObject *__pyx_kp_u_offsets_questions_d_answers_d_au;
static PyObject *__pyx_n_s_packets;
static PyObject *__pyx_n_u_packets_data;
static PyObject *__pyx_kp_u_packets_made_no_progress_adding;
static PyObject *__pyx_n_s_pickle;
static PyObject *__pyx_n_s_prepare;
static PyObject *__pyx_n_s_pyx_PickleError;
static PyObject *__pyx_n_s_pyx_capi;
static PyObject *__pyx_n_s_pyx_checksum;
static PyObject *__pyx_n_s_pyx_result;
static PyObject *__pyx_n_s_pyx_state;
static PyObject *__pyx_n_s_pyx_type;
static PyObject *__pyx_n_s_pyx_unpickle_DNSOutgoing;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_s_qualname;
static PyObject *__pyx_n_u_questions;
static PyObject *__pyx_kp_u_questions_s;
static PyObject *__pyx_n_s_range;
static PyObject *__pyx_n_s_record;
static PyObject *__pyx_n_s_reduce;
static PyObject *__pyx_n_s_reduce_cython;
static PyObject *__pyx_n_s_reduce_ex;
static PyObject *__pyx_n_s_return;
static PyObject *__pyx_n_s_s;
static PyObject *__pyx_n_s_self;
static PyObject *__pyx_n_s_send;
static PyObject *__pyx_n_s_setstate;
static PyObject *__pyx_n_s_setstate_cython;
static PyObject *__pyx_n_u_size;
static PyObject *__pyx_n_s_slots;
static PyObject *__pyx_kp_s_src_zeroconf__protocol_outgoing;
static PyObject *__pyx_n_s_start_data_length;
static PyObject *__pyx_n_s_start_size;
static PyObject *__pyx_n_s_state;
static PyObject *__pyx_n_u_state;
static PyObject *__pyx_kp_s_stringsource;
static PyObject *__pyx_n_s_sum;
static PyObject *__pyx_n_s_suppressed_by;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_throw;
static PyObject *__pyx_n_s_to_bytes;
static PyObject *__pyx_n_s_ttl;
static PyObject *__pyx_n_s_type;
static PyObject *__pyx_n_s_type_2;
static PyObject *__pyx_n_s_typing;
static PyObject *__pyx_n_u_unicode;
static PyObject *__pyx_n_s_unique;
static PyObject *__pyx_n_s_update;
static PyObject *__pyx_n_s_use_setstate;
static PyObject *__pyx_n_s_utfstr;
static PyObject *__pyx_n_s_value;
static PyObject *__pyx_n_s_warning;
static PyObject *__pyx_n_s_write;
static PyObject *__pyx_n_s_write_character_string;
static PyObject *__pyx_n_s_write_name;
static PyObject *__pyx_n_s_write_record;
static PyObject *__pyx_n_s_write_record_locals_genexpr;
static PyObject *__pyx_n_s_write_short;
static PyObject *__pyx_n_s_write_string;
static PyObject *__pyx_n_s_write_ttl;
static PyObject *__pyx_n_s_write_utf;
static PyObject *__pyx_n_s_zeroconf__protocol_outgoing;
static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing___init__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_flags, PyObject *__pyx_v_multicast, PyObject *__pyx_v_id_); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_2is_query(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_4is_response(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_6__repr__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_8add_question(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_record); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_10add_answer(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_inp, PyObject *__pyx_v_record); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_12add_answer_at_time(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_record, PyObject *__pyx_v_now); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_14add_authorative_answer(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_record); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_16add_additional_answer(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_record); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_18add_question_or_one_cache(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_cache, double __pyx_v_now, PyObject *__pyx_v_name, PyObject *__pyx_v_type_, PyObject *__pyx_v_class_); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_20add_question_or_all_cache(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_cache, double __pyx_v_now, PyObject *__pyx_v_name, PyObject *__pyx_v_type_, PyObject *__pyx_v_class_); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_22write_short(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_24write_string(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_26_write_utf(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_s); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_28write_character_string(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_30write_name(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_name); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_32_write_ttl(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_record, double __pyx_v_now); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_13_write_record_genexpr(PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_34_write_record(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_record, double __pyx_v_now); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_36packets(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5flags___get__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self); /* proto */
static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5flags_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_8finished___get__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self); /* proto */
static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_8finished_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_8finished_4__del__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_2id___get__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self); /* proto */
static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_2id_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_2id_4__del__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_9multicast___get__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self); /* proto */
static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_9multicast_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_12packets_data___get__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self); /* proto */
static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_12packets_data_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_12packets_data_4__del__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5names___get__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self); /* proto */
static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5names_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5names_4__del__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_4data___get__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self); /* proto */
static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_4data_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_4data_4__del__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_4size___get__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self); /* proto */
static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_4size_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_10allow_long___get__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self); /* proto */
static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_10allow_long_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_10allow_long_4__del__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5state___get__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self); /* proto */
static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5state_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5state_4__del__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_9questions___get__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self); /* proto */
static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_9questions_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_9questions_4__del__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_7answers___get__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self); /* proto */
static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_7answers_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_7answers_4__del__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_11authorities___get__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self); /* proto */
static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_11authorities_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_11authorities_4__del__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_11additionals___get__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self); /* proto */
static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_11additionals_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_11additionals_4__del__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_38__reduce_cython__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_40__setstate_cython__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing___pyx_unpickle_DNSOutgoing(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_tp_new_8zeroconf_9_protocol_8outgoing_DNSOutgoing(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_8zeroconf_9_protocol_8outgoing___pyx_scope_struct___write_record(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_8zeroconf_9_protocol_8outgoing___pyx_scope_struct_1_genexpr(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_int_0;
static PyObject *__pyx_int_1;
static PyObject *__pyx_int_2;
static PyObject *__pyx_int_4;
static PyObject *__pyx_int_8;
static PyObject *__pyx_int_192;
static PyObject *__pyx_int_255;
static PyObject *__pyx_int_64198871;
static PyObject *__pyx_int_95351500;
static PyObject *__pyx_int_170140993;
static PyObject *__pyx_tuple__2;
static PyObject *__pyx_tuple__3;
static PyObject *__pyx_tuple__4;
static PyObject *__pyx_tuple__7;
static PyObject *__pyx_tuple__8;
static PyObject *__pyx_tuple__9;
static PyObject *__pyx_tuple__11;
static PyObject *__pyx_tuple__13;
static PyObject *__pyx_tuple__15;
static PyObject *__pyx_tuple__17;
static PyObject *__pyx_tuple__19;
static PyObject *__pyx_tuple__21;
static PyObject *__pyx_tuple__23;
static PyObject *__pyx_tuple__25;
static PyObject *__pyx_tuple__27;
static PyObject *__pyx_tuple__29;
static PyObject *__pyx_tuple__31;
static PyObject *__pyx_tuple__33;
static PyObject *__pyx_tuple__35;
static PyObject *__pyx_tuple__37;
static PyObject *__pyx_tuple__39;
static PyObject *__pyx_tuple__41;
static PyObject *__pyx_tuple__43;
static PyObject *__pyx_tuple__45;
static PyObject *__pyx_tuple__47;
static PyObject *__pyx_codeobj__10;
static PyObject *__pyx_codeobj__12;
static PyObject *__pyx_codeobj__14;
static PyObject *__pyx_codeobj__16;
static PyObject *__pyx_codeobj__18;
static PyObject *__pyx_codeobj__20;
static PyObject *__pyx_codeobj__22;
static PyObject *__pyx_codeobj__24;
static PyObject *__pyx_codeobj__26;
static PyObject *__pyx_codeobj__28;
static PyObject *__pyx_codeobj__30;
static PyObject *__pyx_codeobj__32;
static PyObject *__pyx_codeobj__34;
static PyObject *__pyx_codeobj__36;
static PyObject *__pyx_codeobj__38;
static PyObject *__pyx_codeobj__40;
static PyObject *__pyx_codeobj__42;
static PyObject *__pyx_codeobj__44;
static PyObject *__pyx_codeobj__46;
static PyObject *__pyx_codeobj__48;
/* Late includes */

/* "zeroconf/_protocol/outgoing.py":70
 *     )
 * 
 *     def __init__(self, flags: int, multicast: bool = True, id_: int = 0) -> None:             # <<<<<<<<<<<<<<
 *         self.flags = flags
 *         self.finished = False
 */

/* Python wrapper */
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_flags = 0;
  PyObject *__pyx_v_multicast = 0;
  PyObject *__pyx_v_id_ = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_flags,&__pyx_n_s_multicast,&__pyx_n_s_id,0};
    PyObject* values[3] = {0,0,0};
    values[1] = ((PyObject *)Py_True);
    values[2] = ((PyObject *)__pyx_int_0);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_flags)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_multicast);
          if (value) { values[1] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_id);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 70, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_flags = values[0];
    __pyx_v_multicast = values[1];
    __pyx_v_id_ = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 1, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 70, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing___init__(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self), __pyx_v_flags, __pyx_v_multicast, __pyx_v_id_);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing___init__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_flags, PyObject *__pyx_v_multicast, PyObject *__pyx_v_id_) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  unsigned int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "zeroconf/_protocol/outgoing.py":71
 * 
 *     def __init__(self, flags: int, multicast: bool = True, id_: int = 0) -> None:
 *         self.flags = flags             # <<<<<<<<<<<<<<
 *         self.finished = False
 *         self.id = id_
 */
  __pyx_t_1 = __Pyx_PyInt_As_unsigned_int(__pyx_v_flags); if (unlikely((__pyx_t_1 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(0, 71, __pyx_L1_error)
  __pyx_v_self->flags = __pyx_t_1;

  /* "zeroconf/_protocol/outgoing.py":72
 *     def __init__(self, flags: int, multicast: bool = True, id_: int = 0) -> None:
 *         self.flags = flags
 *         self.finished = False             # <<<<<<<<<<<<<<
 *         self.id = id_
 *         self.multicast = multicast
 */
  __Pyx_INCREF(Py_False);
  __Pyx_GIVEREF(Py_False);
  __Pyx_GOTREF(__pyx_v_self->finished);
  __Pyx_DECREF(__pyx_v_self->finished);
  __pyx_v_self->finished = Py_False;

  /* "zeroconf/_protocol/outgoing.py":73
 *         self.flags = flags
 *         self.finished = False
 *         self.id = id_             # <<<<<<<<<<<<<<
 *         self.multicast = multicast
 *         self.packets_data: List[bytes] = []
 */
  __Pyx_INCREF(__pyx_v_id_);
  __Pyx_GIVEREF(__pyx_v_id_);
  __Pyx_GOTREF(__pyx_v_self->id);
  __Pyx_DECREF(__pyx_v_self->id);
  __pyx_v_self->id = __pyx_v_id_;

  /* "zeroconf/_protocol/outgoing.py":74
 *         self.finished = False
 *         self.id = id_
 *         self.multicast = multicast             # <<<<<<<<<<<<<<
 *         self.packets_data: List[bytes] = []
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_multicast); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 74, __pyx_L1_error)
  __pyx_v_self->multicast = __pyx_t_2;

  /* "zeroconf/_protocol/outgoing.py":75
 *         self.id = id_
 *         self.multicast = multicast
 *         self.packets_data: List[bytes] = []             # <<<<<<<<<<<<<<
 * 
 *         # these 3 are per-packet -- see also _reset_for_next_packet()
 */
  __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 75, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->packets_data);
  __Pyx_DECREF(__pyx_v_self->packets_data);
  __pyx_v_self->packets_data = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "zeroconf/_protocol/outgoing.py":78
 * 
 *         # these 3 are per-packet -- see also _reset_for_next_packet()
 *         self.names: Dict[str, int] = {}             # <<<<<<<<<<<<<<
 *         self.data: List[bytes] = []
 *         self.size: int = _DNS_PACKET_HEADER_LEN
 */
  __pyx_t_3 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 78, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->names);
  __Pyx_DECREF(__pyx_v_self->names);
  __pyx_v_self->names = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "zeroconf/_protocol/outgoing.py":79
 *         # these 3 are per-packet -- see also _reset_for_next_packet()
 *         self.names: Dict[str, int] = {}
 *         self.data: List[bytes] = []             # <<<<<<<<<<<<<<
 *         self.size: int = _DNS_PACKET_HEADER_LEN
 *         self.allow_long: bool = True
 */
  __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->data);
  __Pyx_DECREF(__pyx_v_self->data);
  __pyx_v_self->data = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "zeroconf/_protocol/outgoing.py":80
 *         self.names: Dict[str, int] = {}
 *         self.data: List[bytes] = []
 *         self.size: int = _DNS_PACKET_HEADER_LEN             # <<<<<<<<<<<<<<
 *         self.allow_long: bool = True
 * 
 */
  __pyx_v_self->size = __pyx_v_8zeroconf_9_protocol_8outgoing__DNS_PACKET_HEADER_LEN;

  /* "zeroconf/_protocol/outgoing.py":81
 *         self.data: List[bytes] = []
 *         self.size: int = _DNS_PACKET_HEADER_LEN
 *         self.allow_long: bool = True             # <<<<<<<<<<<<<<
 * 
 *         self.state = State.init
 */
  __Pyx_INCREF(Py_True);
  __Pyx_GIVEREF(Py_True);
  __Pyx_GOTREF(__pyx_v_self->allow_long);
  __Pyx_DECREF(__pyx_v_self->allow_long);
  __pyx_v_self->allow_long = Py_True;

  /* "zeroconf/_protocol/outgoing.py":83
 *         self.allow_long: bool = True
 * 
 *         self.state = State.init             # <<<<<<<<<<<<<<
 * 
 *         self.questions: List[DNSQuestion] = []
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_State); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 83, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 83, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_GIVEREF(__pyx_t_4);
  __Pyx_GOTREF(__pyx_v_self->state);
  __Pyx_DECREF(__pyx_v_self->state);
  __pyx_v_self->state = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "zeroconf/_protocol/outgoing.py":85
 *         self.state = State.init
 * 
 *         self.questions: List[DNSQuestion] = []             # <<<<<<<<<<<<<<
 *         self.answers: List[Tuple[DNSRecord, float]] = []
 *         self.authorities: List[DNSPointer] = []
 */
  __pyx_t_4 = PyList_New(0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 85, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_4);
  __Pyx_GOTREF(__pyx_v_self->questions);
  __Pyx_DECREF(__pyx_v_self->questions);
  __pyx_v_self->questions = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "zeroconf/_protocol/outgoing.py":86
 * 
 *         self.questions: List[DNSQuestion] = []
 *         self.answers: List[Tuple[DNSRecord, float]] = []             # <<<<<<<<<<<<<<
 *         self.authorities: List[DNSPointer] = []
 *         self.additionals: List[DNSRecord] = []
 */
  __pyx_t_4 = PyList_New(0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 86, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_4);
  __Pyx_GOTREF(__pyx_v_self->answers);
  __Pyx_DECREF(__pyx_v_self->answers);
  __pyx_v_self->answers = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "zeroconf/_protocol/outgoing.py":87
 *         self.questions: List[DNSQuestion] = []
 *         self.answers: List[Tuple[DNSRecord, float]] = []
 *         self.authorities: List[DNSPointer] = []             # <<<<<<<<<<<<<<
 *         self.additionals: List[DNSRecord] = []
 * 
 */
  __pyx_t_4 = PyList_New(0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 87, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_4);
  __Pyx_GOTREF(__pyx_v_self->authorities);
  __Pyx_DECREF(__pyx_v_self->authorities);
  __pyx_v_self->authorities = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "zeroconf/_protocol/outgoing.py":88
 *         self.answers: List[Tuple[DNSRecord, float]] = []
 *         self.authorities: List[DNSPointer] = []
 *         self.additionals: List[DNSRecord] = []             # <<<<<<<<<<<<<<
 * 
 *     def is_query(self) -> bool:
 */
  __pyx_t_4 = PyList_New(0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 88, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_4);
  __Pyx_GOTREF(__pyx_v_self->additionals);
  __Pyx_DECREF(__pyx_v_self->additionals);
  __pyx_v_self->additionals = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "zeroconf/_protocol/outgoing.py":70
 *     )
 * 
 *     def __init__(self, flags: int, multicast: bool = True, id_: int = 0) -> None:             # <<<<<<<<<<<<<<
 *         self.flags = flags
 *         self.finished = False
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/outgoing.py":90
 *         self.additionals: List[DNSRecord] = []
 * 
 *     def is_query(self) -> bool:             # <<<<<<<<<<<<<<
 *         """Returns true if this is a query."""
 *         return (self.flags & _FLAGS_QR_MASK) == _FLAGS_QR_QUERY
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_3is_query(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_2is_query[] = "Returns true if this is a query.";
static PyMethodDef __pyx_mdef_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_3is_query = {"is_query", (PyCFunction)__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_3is_query, METH_NOARGS, __pyx_doc_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_2is_query};
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_3is_query(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("is_query (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_2is_query(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_2is_query(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("is_query", 0);

  /* "zeroconf/_protocol/outgoing.py":92
 *     def is_query(self) -> bool:
 *         """Returns true if this is a query."""
 *         return (self.flags & _FLAGS_QR_MASK) == _FLAGS_QR_QUERY             # <<<<<<<<<<<<<<
 * 
 *     def is_response(self) -> bool:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(((__pyx_v_self->flags & __pyx_v_8zeroconf_9_protocol_8outgoing__FLAGS_QR_MASK) == __pyx_v_8zeroconf_9_protocol_8outgoing__FLAGS_QR_QUERY)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 92, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "zeroconf/_protocol/outgoing.py":90
 *         self.additionals: List[DNSRecord] = []
 * 
 *     def is_query(self) -> bool:             # <<<<<<<<<<<<<<
 *         """Returns true if this is a query."""
 *         return (self.flags & _FLAGS_QR_MASK) == _FLAGS_QR_QUERY
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing.is_query", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/outgoing.py":94
 *         return (self.flags & _FLAGS_QR_MASK) == _FLAGS_QR_QUERY
 * 
 *     def is_response(self) -> bool:             # <<<<<<<<<<<<<<
 *         """Returns true if this is a response."""
 *         return (self.flags & _FLAGS_QR_MASK) == _FLAGS_QR_RESPONSE
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5is_response(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_4is_response[] = "Returns true if this is a response.";
static PyMethodDef __pyx_mdef_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5is_response = {"is_response", (PyCFunction)__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5is_response, METH_NOARGS, __pyx_doc_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_4is_response};
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5is_response(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("is_response (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_4is_response(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_4is_response(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("is_response", 0);

  /* "zeroconf/_protocol/outgoing.py":96
 *     def is_response(self) -> bool:
 *         """Returns true if this is a response."""
 *         return (self.flags & _FLAGS_QR_MASK) == _FLAGS_QR_RESPONSE             # <<<<<<<<<<<<<<
 * 
 *     def _reset_for_next_packet(self) -> None:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(((__pyx_v_self->flags & __pyx_v_8zeroconf_9_protocol_8outgoing__FLAGS_QR_MASK) == __pyx_v_8zeroconf_9_protocol_8outgoing__FLAGS_QR_RESPONSE)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 96, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "zeroconf/_protocol/outgoing.py":94
 *         return (self.flags & _FLAGS_QR_MASK) == _FLAGS_QR_QUERY
 * 
 *     def is_response(self) -> bool:             # <<<<<<<<<<<<<<
 *         """Returns true if this is a response."""
 *         return (self.flags & _FLAGS_QR_MASK) == _FLAGS_QR_RESPONSE
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing.is_response", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/outgoing.py":98
 *         return (self.flags & _FLAGS_QR_MASK) == _FLAGS_QR_RESPONSE
 * 
 *     def _reset_for_next_packet(self) -> None:             # <<<<<<<<<<<<<<
 *         self.names = {}
 *         self.data = []
 */

static PyObject *__pyx_f_8zeroconf_9_protocol_8outgoing_11DNSOutgoing__reset_for_next_packet(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_reset_for_next_packet", 0);

  /* "zeroconf/_protocol/outgoing.py":99
 * 
 *     def _reset_for_next_packet(self) -> None:
 *         self.names = {}             # <<<<<<<<<<<<<<
 *         self.data = []
 *         self.size = _DNS_PACKET_HEADER_LEN
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 99, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->names);
  __Pyx_DECREF(__pyx_v_self->names);
  __pyx_v_self->names = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "zeroconf/_protocol/outgoing.py":100
 *     def _reset_for_next_packet(self) -> None:
 *         self.names = {}
 *         self.data = []             # <<<<<<<<<<<<<<
 *         self.size = _DNS_PACKET_HEADER_LEN
 *         self.allow_long = True
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 100, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->data);
  __Pyx_DECREF(__pyx_v_self->data);
  __pyx_v_self->data = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "zeroconf/_protocol/outgoing.py":101
 *         self.names = {}
 *         self.data = []
 *         self.size = _DNS_PACKET_HEADER_LEN             # <<<<<<<<<<<<<<
 *         self.allow_long = True
 * 
 */
  __pyx_v_self->size = __pyx_v_8zeroconf_9_protocol_8outgoing__DNS_PACKET_HEADER_LEN;

  /* "zeroconf/_protocol/outgoing.py":102
 *         self.data = []
 *         self.size = _DNS_PACKET_HEADER_LEN
 *         self.allow_long = True             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self) -> str:
 */
  __Pyx_INCREF(Py_True);
  __Pyx_GIVEREF(Py_True);
  __Pyx_GOTREF(__pyx_v_self->allow_long);
  __Pyx_DECREF(__pyx_v_self->allow_long);
  __pyx_v_self->allow_long = Py_True;

  /* "zeroconf/_protocol/outgoing.py":98
 *         return (self.flags & _FLAGS_QR_MASK) == _FLAGS_QR_RESPONSE
 * 
 *     def _reset_for_next_packet(self) -> None:             # <<<<<<<<<<<<<<
 *         self.names = {}
 *         self.data = []
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing._reset_for_next_packet", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/outgoing.py":104
 *         self.allow_long = True
 * 
 *     def __repr__(self) -> str:             # <<<<<<<<<<<<<<
 *         return '<DNSOutgoing:{%s}>' % ', '.join(
 *             [
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_7__repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_7__repr__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_6__repr__(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_6__repr__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "zeroconf/_protocol/outgoing.py":105
 * 
 *     def __repr__(self) -> str:
 *         return '<DNSOutgoing:{%s}>' % ', '.join(             # <<<<<<<<<<<<<<
 *             [
 *                 'multicast=%s' % self.multicast,
 */
  __Pyx_XDECREF(__pyx_r);

  /* "zeroconf/_protocol/outgoing.py":107
 *         return '<DNSOutgoing:{%s}>' % ', '.join(
 *             [
 *                 'multicast=%s' % self.multicast,             # <<<<<<<<<<<<<<
 *                 'flags=%s' % self.flags,
 *                 'questions=%s' % self.questions,
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->multicast); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 107, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyUnicode_Format(__pyx_kp_u_multicast_s, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 107, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "zeroconf/_protocol/outgoing.py":108
 *             [
 *                 'multicast=%s' % self.multicast,
 *                 'flags=%s' % self.flags,             # <<<<<<<<<<<<<<
 *                 'questions=%s' % self.questions,
 *                 'answers=%s' % self.answers,
 */
  __pyx_t_1 = __Pyx_PyInt_From_unsigned_int(__pyx_v_self->flags); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 108, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyUnicode_Format(__pyx_kp_u_flags_s, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 108, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "zeroconf/_protocol/outgoing.py":109
 *                 'multicast=%s' % self.multicast,
 *                 'flags=%s' % self.flags,
 *                 'questions=%s' % self.questions,             # <<<<<<<<<<<<<<
 *                 'answers=%s' % self.answers,
 *                 'authorities=%s' % self.authorities,
 */
  __pyx_t_1 = PyUnicode_Format(__pyx_kp_u_questions_s, __pyx_v_self->questions); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 109, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);

  /* "zeroconf/_protocol/outgoing.py":110
 *                 'flags=%s' % self.flags,
 *                 'questions=%s' % self.questions,
 *                 'answers=%s' % self.answers,             # <<<<<<<<<<<<<<
 *                 'authorities=%s' % self.authorities,
 *                 'additionals=%s' % self.additionals,
 */
  __pyx_t_4 = PyUnicode_Format(__pyx_kp_u_answers_s, __pyx_v_self->answers); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 110, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);

  /* "zeroconf/_protocol/outgoing.py":111
 *                 'questions=%s' % self.questions,
 *                 'answers=%s' % self.answers,
 *                 'authorities=%s' % self.authorities,             # <<<<<<<<<<<<<<
 *                 'additionals=%s' % self.additionals,
 *             ]
 */
  __pyx_t_5 = PyUnicode_Format(__pyx_kp_u_authorities_s, __pyx_v_self->authorities); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 111, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);

  /* "zeroconf/_protocol/outgoing.py":112
 *                 'answers=%s' % self.answers,
 *                 'authorities=%s' % self.authorities,
 *                 'additionals=%s' % self.additionals,             # <<<<<<<<<<<<<<
 *             ]
 *         )
 */
  __pyx_t_6 = PyUnicode_Format(__pyx_kp_u_additionals_s, __pyx_v_self->additionals); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 112, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);

  /* "zeroconf/_protocol/outgoing.py":106
 *     def __repr__(self) -> str:
 *         return '<DNSOutgoing:{%s}>' % ', '.join(
 *             [             # <<<<<<<<<<<<<<
 *                 'multicast=%s' % self.multicast,
 *                 'flags=%s' % self.flags,
 */
  __pyx_t_7 = PyList_New(6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 106, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_2);
  PyList_SET_ITEM(__pyx_t_7, 0, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyList_SET_ITEM(__pyx_t_7, 1, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyList_SET_ITEM(__pyx_t_7, 2, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_4);
  PyList_SET_ITEM(__pyx_t_7, 3, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyList_SET_ITEM(__pyx_t_7, 4, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_6);
  PyList_SET_ITEM(__pyx_t_7, 5, __pyx_t_6);
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_1 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_t_6 = 0;

  /* "zeroconf/_protocol/outgoing.py":105
 * 
 *     def __repr__(self) -> str:
 *         return '<DNSOutgoing:{%s}>' % ', '.join(             # <<<<<<<<<<<<<<
 *             [
 *                 'multicast=%s' % self.multicast,
 */
  __pyx_t_6 = PyUnicode_Join(__pyx_kp_u_, __pyx_t_7); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 105, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = PyUnicode_Format(__pyx_kp_u_DNSOutgoing_s, __pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 105, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_r = __pyx_t_7;
  __pyx_t_7 = 0;
  goto __pyx_L0;

  /* "zeroconf/_protocol/outgoing.py":104
 *         self.allow_long = True
 * 
 *     def __repr__(self) -> str:             # <<<<<<<<<<<<<<
 *         return '<DNSOutgoing:{%s}>' % ', '.join(
 *             [
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/outgoing.py":116
 *         )
 * 
 *     def add_question(self, record: DNSQuestion) -> None:             # <<<<<<<<<<<<<<
 *         """Adds a question"""
 *         self.questions.append(record)
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_9add_question(PyObject *__pyx_v_self, PyObject *__pyx_v_record); /*proto*/
static char __pyx_doc_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_8add_question[] = "Adds a question";
static PyMethodDef __pyx_mdef_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_9add_question = {"add_question", (PyCFunction)__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_9add_question, METH_O, __pyx_doc_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_8add_question};
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_9add_question(PyObject *__pyx_v_self, PyObject *__pyx_v_record) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("add_question (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_8add_question(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self), ((PyObject *)__pyx_v_record));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_8add_question(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_record) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("add_question", 0);

  /* "zeroconf/_protocol/outgoing.py":118
 *     def add_question(self, record: DNSQuestion) -> None:
 *         """Adds a question"""
 *         self.questions.append(record)             # <<<<<<<<<<<<<<
 * 
 *     def add_answer(self, inp: DNSIncoming, record: DNSRecord) -> None:
 */
  if (unlikely(__pyx_v_self->questions == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(0, 118, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyList_Append(__pyx_v_self->questions, __pyx_v_record); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(0, 118, __pyx_L1_error)

  /* "zeroconf/_protocol/outgoing.py":116
 *         )
 * 
 *     def add_question(self, record: DNSQuestion) -> None:             # <<<<<<<<<<<<<<
 *         """Adds a question"""
 *         self.questions.append(record)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing.add_question", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/outgoing.py":120
 *         self.questions.append(record)
 * 
 *     def add_answer(self, inp: DNSIncoming, record: DNSRecord) -> None:             # <<<<<<<<<<<<<<
 *         """Adds an answer"""
 *         if not record.suppressed_by(inp):
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_11add_answer(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_10add_answer[] = "Adds an answer";
static PyMethodDef __pyx_mdef_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_11add_answer = {"add_answer", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_11add_answer, METH_VARARGS|METH_KEYWORDS, __pyx_doc_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_10add_answer};
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_11add_answer(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_inp = 0;
  PyObject *__pyx_v_record = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("add_answer (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_inp,&__pyx_n_s_record,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_inp)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_record)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("add_answer", 1, 2, 2, 1); __PYX_ERR(0, 120, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "add_answer") < 0)) __PYX_ERR(0, 120, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_inp = ((struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *)values[0]);
    __pyx_v_record = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("add_answer", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 120, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing.add_answer", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_inp), __pyx_ptype_8zeroconf_9_protocol_8incoming_DNSIncoming, 1, "inp", 0))) __PYX_ERR(0, 120, __pyx_L1_error)
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_10add_answer(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self), __pyx_v_inp, __pyx_v_record);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_10add_answer(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_inp, PyObject *__pyx_v_record) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("add_answer", 0);

  /* "zeroconf/_protocol/outgoing.py":122
 *     def add_answer(self, inp: DNSIncoming, record: DNSRecord) -> None:
 *         """Adds an answer"""
 *         if not record.suppressed_by(inp):             # <<<<<<<<<<<<<<
 *             self.add_answer_at_time(record, 0)
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_record, __pyx_n_s_suppressed_by); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 122, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, ((PyObject *)__pyx_v_inp)) : __Pyx_PyObject_CallOneArg(__pyx_t_2, ((PyObject *)__pyx_v_inp));
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 122, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 122, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = ((!__pyx_t_4) != 0);
  if (__pyx_t_5) {

    /* "zeroconf/_protocol/outgoing.py":123
 *         """Adds an answer"""
 *         if not record.suppressed_by(inp):
 *             self.add_answer_at_time(record, 0)             # <<<<<<<<<<<<<<
 * 
 *     def add_answer_at_time(self, record: Optional[DNSRecord], now: Union[float, int]) -> None:
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_add_answer_at_time); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 123, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = NULL;
    __pyx_t_6 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
        __pyx_t_6 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_record, __pyx_int_0};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 123, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_record, __pyx_int_0};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 123, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 123, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      if (__pyx_t_3) {
        __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_3); __pyx_t_3 = NULL;
      }
      __Pyx_INCREF(__pyx_v_record);
      __Pyx_GIVEREF(__pyx_v_record);
      PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_v_record);
      __Pyx_INCREF(__pyx_int_0);
      __Pyx_GIVEREF(__pyx_int_0);
      PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_int_0);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 123, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "zeroconf/_protocol/outgoing.py":122
 *     def add_answer(self, inp: DNSIncoming, record: DNSRecord) -> None:
 *         """Adds an answer"""
 *         if not record.suppressed_by(inp):             # <<<<<<<<<<<<<<
 *             self.add_answer_at_time(record, 0)
 * 
 */
  }

  /* "zeroconf/_protocol/outgoing.py":120
 *         self.questions.append(record)
 * 
 *     def add_answer(self, inp: DNSIncoming, record: DNSRecord) -> None:             # <<<<<<<<<<<<<<
 *         """Adds an answer"""
 *         if not record.suppressed_by(inp):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing.add_answer", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/outgoing.py":125
 *             self.add_answer_at_time(record, 0)
 * 
 *     def add_answer_at_time(self, record: Optional[DNSRecord], now: Union[float, int]) -> None:             # <<<<<<<<<<<<<<
 *         """Adds an answer if it does not expire by a certain time"""
 *         if record is not None and (now == 0 or not record.is_expired(now)):
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_13add_answer_at_time(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_12add_answer_at_time[] = "Adds an answer if it does not expire by a certain time";
static PyMethodDef __pyx_mdef_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_13add_answer_at_time = {"add_answer_at_time", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_13add_answer_at_time, METH_VARARGS|METH_KEYWORDS, __pyx_doc_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_12add_answer_at_time};
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_13add_answer_at_time(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_record = 0;
  PyObject *__pyx_v_now = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("add_answer_at_time (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_record,&__pyx_n_s_now,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_record)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_now)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("add_answer_at_time", 1, 2, 2, 1); __PYX_ERR(0, 125, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "add_answer_at_time") < 0)) __PYX_ERR(0, 125, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_record = values[0];
    __pyx_v_now = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("add_answer_at_time", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 125, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing.add_answer_at_time", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_12add_answer_at_time(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self), __pyx_v_record, __pyx_v_now);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_12add_answer_at_time(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_record, PyObject *__pyx_v_now) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("add_answer_at_time", 0);

  /* "zeroconf/_protocol/outgoing.py":127
 *     def add_answer_at_time(self, record: Optional[DNSRecord], now: Union[float, int]) -> None:
 *         """Adds an answer if it does not expire by a certain time"""
 *         if record is not None and (now == 0 or not record.is_expired(now)):             # <<<<<<<<<<<<<<
 *             self.answers.append((record, now))
 * 
 */
  __pyx_t_2 = (__pyx_v_record != Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_PyInt_EqObjC(__pyx_v_now, __pyx_int_0, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 127, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 127, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (!__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_record, __pyx_n_s_is_expired); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 127, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  __pyx_t_4 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_6, __pyx_v_now) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_v_now);
  __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 127, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 127, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_2 = ((!__pyx_t_3) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "zeroconf/_protocol/outgoing.py":128
 *         """Adds an answer if it does not expire by a certain time"""
 *         if record is not None and (now == 0 or not record.is_expired(now)):
 *             self.answers.append((record, now))             # <<<<<<<<<<<<<<
 * 
 *     def add_authorative_answer(self, record: DNSPointer) -> None:
 */
    if (unlikely(__pyx_v_self->answers == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
      __PYX_ERR(0, 128, __pyx_L1_error)
    }
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 128, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_v_record);
    __Pyx_GIVEREF(__pyx_v_record);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_record);
    __Pyx_INCREF(__pyx_v_now);
    __Pyx_GIVEREF(__pyx_v_now);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_v_now);
    __pyx_t_7 = __Pyx_PyList_Append(__pyx_v_self->answers, __pyx_t_4); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 128, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "zeroconf/_protocol/outgoing.py":127
 *     def add_answer_at_time(self, record: Optional[DNSRecord], now: Union[float, int]) -> None:
 *         """Adds an answer if it does not expire by a certain time"""
 *         if record is not None and (now == 0 or not record.is_expired(now)):             # <<<<<<<<<<<<<<
 *             self.answers.append((record, now))
 * 
 */
  }

  /* "zeroconf/_protocol/outgoing.py":125
 *             self.add_answer_at_time(record, 0)
 * 
 *     def add_answer_at_time(self, record: Optional[DNSRecord], now: Union[float, int]) -> None:             # <<<<<<<<<<<<<<
 *         """Adds an answer if it does not expire by a certain time"""
 *         if record is not None and (now == 0 or not record.is_expired(now)):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing.add_answer_at_time", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/outgoing.py":130
 *             self.answers.append((record, now))
 * 
 *     def add_authorative_answer(self, record: DNSPointer) -> None:             # <<<<<<<<<<<<<<
 *         """Adds an authoritative answer"""
 *         self.authorities.append(record)
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_15add_authorative_answer(PyObject *__pyx_v_self, PyObject *__pyx_v_record); /*proto*/
static char __pyx_doc_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_14add_authorative_answer[] = "Adds an authoritative answer";
static PyMethodDef __pyx_mdef_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_15add_authorative_answer = {"add_authorative_answer", (PyCFunction)__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_15add_authorative_answer, METH_O, __pyx_doc_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_14add_authorative_answer};
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_15add_authorative_answer(PyObject *__pyx_v_self, PyObject *__pyx_v_record) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("add_authorative_answer (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_14add_authorative_answer(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self), ((PyObject *)__pyx_v_record));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_14add_authorative_answer(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_record) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("add_authorative_answer", 0);

  /* "zeroconf/_protocol/outgoing.py":132
 *     def add_authorative_answer(self, record: DNSPointer) -> None:
 *         """Adds an authoritative answer"""
 *         self.authorities.append(record)             # <<<<<<<<<<<<<<
 * 
 *     def add_additional_answer(self, record: DNSRecord) -> None:
 */
  if (unlikely(__pyx_v_self->authorities == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(0, 132, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyList_Append(__pyx_v_self->authorities, __pyx_v_record); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(0, 132, __pyx_L1_error)

  /* "zeroconf/_protocol/outgoing.py":130
 *             self.answers.append((record, now))
 * 
 *     def add_authorative_answer(self, record: DNSPointer) -> None:             # <<<<<<<<<<<<<<
 *         """Adds an authoritative answer"""
 *         self.authorities.append(record)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing.add_authorative_answer", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/outgoing.py":134
 *         self.authorities.append(record)
 * 
 *     def add_additional_answer(self, record: DNSRecord) -> None:             # <<<<<<<<<<<<<<
 *         """Adds an additional answer
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_17add_additional_answer(PyObject *__pyx_v_self, PyObject *__pyx_v_record); /*proto*/
static char __pyx_doc_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_16add_additional_answer[] = "Adds an additional answer\n\n        From: RFC 6763, DNS-Based Service Discovery, February 2013\n\n        12.  DNS Additional Record Generation\n\n           DNS has an efficiency feature whereby a DNS server may place\n           additional records in the additional section of the DNS message.\n           These additional records are records that the client did not\n           explicitly request, but the server has reasonable grounds to expect\n           that the client might request them shortly, so including them can\n           save the client from having to issue additional queries.\n\n           This section recommends which additional records SHOULD be generated\n           to improve network efficiency, for both Unicast and Multicast DNS-SD\n           responses.\n\n        12.1.  PTR Records\n\n           When including a DNS-SD Service Instance Enumeration or Selective\n           Instance Enumeration (subtype) PTR record in a response packet, the\n           server/responder SHOULD include the following additional records:\n\n           o  The SRV record(s) named in the PTR rdata.\n           o  The TXT record(s) named in the PTR rdata.\n           o  All address records (type \"A\" and \"AAAA\") named in the SRV rdata.\n\n        12.2.  SRV Records\n\n           When including an SRV record in a response packet, the\n           server/responder SHOULD include the following additional records:\n\n           o  All address records (type \"A\" and \"AAAA\") named in the SRV rdata.\n\n        ";
static PyMethodDef __pyx_mdef_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_17add_additional_answer = {"add_additional_answer", (PyCFunction)__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_17add_additional_answer, METH_O, __pyx_doc_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_16add_additional_answer};
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_17add_additional_answer(PyObject *__pyx_v_self, PyObject *__pyx_v_record) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("add_additional_answer (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_16add_additional_answer(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self), ((PyObject *)__pyx_v_record));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_16add_additional_answer(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_record) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("add_additional_answer", 0);

  /* "zeroconf/_protocol/outgoing.py":170
 * 
 *         """
 *         self.additionals.append(record)             # <<<<<<<<<<<<<<
 * 
 *     def add_question_or_one_cache(
 */
  if (unlikely(__pyx_v_self->additionals == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(0, 170, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyList_Append(__pyx_v_self->additionals, __pyx_v_record); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(0, 170, __pyx_L1_error)

  /* "zeroconf/_protocol/outgoing.py":134
 *         self.authorities.append(record)
 * 
 *     def add_additional_answer(self, record: DNSRecord) -> None:             # <<<<<<<<<<<<<<
 *         """Adds an additional answer
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing.add_additional_answer", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/outgoing.py":172
 *         self.additionals.append(record)
 * 
 *     def add_question_or_one_cache(             # <<<<<<<<<<<<<<
 *         self, cache: DNSCache, now: float, name: str, type_: int, class_: int
 *     ) -> None:
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_19add_question_or_one_cache(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_18add_question_or_one_cache[] = "Add a question if it is not already cached.";
static PyMethodDef __pyx_mdef_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_19add_question_or_one_cache = {"add_question_or_one_cache", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_19add_question_or_one_cache, METH_VARARGS|METH_KEYWORDS, __pyx_doc_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_18add_question_or_one_cache};
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_19add_question_or_one_cache(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_cache = 0;
  double __pyx_v_now;
  PyObject *__pyx_v_name = 0;
  PyObject *__pyx_v_type_ = 0;
  PyObject *__pyx_v_class_ = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("add_question_or_one_cache (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_cache,&__pyx_n_s_now,&__pyx_n_s_name,&__pyx_n_s_type,&__pyx_n_s_class,0};
    PyObject* values[5] = {0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_cache)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_now)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("add_question_or_one_cache", 1, 5, 5, 1); __PYX_ERR(0, 172, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_name)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("add_question_or_one_cache", 1, 5, 5, 2); __PYX_ERR(0, 172, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_type)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("add_question_or_one_cache", 1, 5, 5, 3); __PYX_ERR(0, 172, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_class)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("add_question_or_one_cache", 1, 5, 5, 4); __PYX_ERR(0, 172, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "add_question_or_one_cache") < 0)) __PYX_ERR(0, 172, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 5) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
    }
    __pyx_v_cache = values[0];
    __pyx_v_now = __pyx_PyFloat_AsDouble(values[1]); if (unlikely((__pyx_v_now == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 173, __pyx_L3_error)
    __pyx_v_name = ((PyObject*)values[2]);
    __pyx_v_type_ = values[3];
    __pyx_v_class_ = values[4];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("add_question_or_one_cache", 1, 5, 5, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 172, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing.add_question_or_one_cache", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_name), (&PyUnicode_Type), 1, "name", 1))) __PYX_ERR(0, 173, __pyx_L1_error)
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_18add_question_or_one_cache(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self), __pyx_v_cache, __pyx_v_now, __pyx_v_name, __pyx_v_type_, __pyx_v_class_);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_18add_question_or_one_cache(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_cache, double __pyx_v_now, PyObject *__pyx_v_name, PyObject *__pyx_v_type_, PyObject *__pyx_v_class_) {
  PyObject *__pyx_v_cached_entry = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("add_question_or_one_cache", 0);

  /* "zeroconf/_protocol/outgoing.py":176
 *     ) -> None:
 *         """Add a question if it is not already cached."""
 *         cached_entry = cache.get_by_details(name, type_, class_)             # <<<<<<<<<<<<<<
 *         if not cached_entry:
 *             self.add_question(DNSQuestion(name, type_, class_))
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_cache, __pyx_n_s_get_by_details); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 176, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_3, __pyx_v_name, __pyx_v_type_, __pyx_v_class_};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 3+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 176, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_3, __pyx_v_name, __pyx_v_type_, __pyx_v_class_};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 3+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 176, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(3+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 176, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_name);
    __Pyx_GIVEREF(__pyx_v_name);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_name);
    __Pyx_INCREF(__pyx_v_type_);
    __Pyx_GIVEREF(__pyx_v_type_);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_type_);
    __Pyx_INCREF(__pyx_v_class_);
    __Pyx_GIVEREF(__pyx_v_class_);
    PyTuple_SET_ITEM(__pyx_t_5, 2+__pyx_t_4, __pyx_v_class_);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 176, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_cached_entry = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "zeroconf/_protocol/outgoing.py":177
 *         """Add a question if it is not already cached."""
 *         cached_entry = cache.get_by_details(name, type_, class_)
 *         if not cached_entry:             # <<<<<<<<<<<<<<
 *             self.add_question(DNSQuestion(name, type_, class_))
 *         else:
 */
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_v_cached_entry); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 177, __pyx_L1_error)
  __pyx_t_7 = ((!__pyx_t_6) != 0);
  if (__pyx_t_7) {

    /* "zeroconf/_protocol/outgoing.py":178
 *         cached_entry = cache.get_by_details(name, type_, class_)
 *         if not cached_entry:
 *             self.add_question(DNSQuestion(name, type_, class_))             # <<<<<<<<<<<<<<
 *         else:
 *             self.add_answer_at_time(cached_entry, now)
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_add_question); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 178, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_DNSQuestion); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 178, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_8 = NULL;
    __pyx_t_4 = 0;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
        __pyx_t_4 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[4] = {__pyx_t_8, __pyx_v_name, __pyx_v_type_, __pyx_v_class_};
      __pyx_t_5 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_4, 3+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 178, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_GOTREF(__pyx_t_5);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[4] = {__pyx_t_8, __pyx_v_name, __pyx_v_type_, __pyx_v_class_};
      __pyx_t_5 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_4, 3+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 178, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_GOTREF(__pyx_t_5);
    } else
    #endif
    {
      __pyx_t_9 = PyTuple_New(3+__pyx_t_4); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 178, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      if (__pyx_t_8) {
        __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8); __pyx_t_8 = NULL;
      }
      __Pyx_INCREF(__pyx_v_name);
      __Pyx_GIVEREF(__pyx_v_name);
      PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_4, __pyx_v_name);
      __Pyx_INCREF(__pyx_v_type_);
      __Pyx_GIVEREF(__pyx_v_type_);
      PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_4, __pyx_v_type_);
      __Pyx_INCREF(__pyx_v_class_);
      __Pyx_GIVEREF(__pyx_v_class_);
      PyTuple_SET_ITEM(__pyx_t_9, 2+__pyx_t_4, __pyx_v_class_);
      __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_9, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 178, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_5);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 178, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "zeroconf/_protocol/outgoing.py":177
 *         """Add a question if it is not already cached."""
 *         cached_entry = cache.get_by_details(name, type_, class_)
 *         if not cached_entry:             # <<<<<<<<<<<<<<
 *             self.add_question(DNSQuestion(name, type_, class_))
 *         else:
 */
    goto __pyx_L3;
  }

  /* "zeroconf/_protocol/outgoing.py":180
 *             self.add_question(DNSQuestion(name, type_, class_))
 *         else:
 *             self.add_answer_at_time(cached_entry, now)             # <<<<<<<<<<<<<<
 * 
 *     def add_question_or_all_cache(
 */
  /*else*/ {
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_add_answer_at_time); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 180, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = PyFloat_FromDouble(__pyx_v_now); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 180, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_3 = NULL;
    __pyx_t_4 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
        __pyx_t_4 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_cached_entry, __pyx_t_5};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 180, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_cached_entry, __pyx_t_5};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 180, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else
    #endif
    {
      __pyx_t_9 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 180, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      if (__pyx_t_3) {
        __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_3); __pyx_t_3 = NULL;
      }
      __Pyx_INCREF(__pyx_v_cached_entry);
      __Pyx_GIVEREF(__pyx_v_cached_entry);
      PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_4, __pyx_v_cached_entry);
      __Pyx_GIVEREF(__pyx_t_5);
      PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_4, __pyx_t_5);
      __pyx_t_5 = 0;
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_9, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 180, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __pyx_L3:;

  /* "zeroconf/_protocol/outgoing.py":172
 *         self.additionals.append(record)
 * 
 *     def add_question_or_one_cache(             # <<<<<<<<<<<<<<
 *         self, cache: DNSCache, now: float, name: str, type_: int, class_: int
 *     ) -> None:
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing.add_question_or_one_cache", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_cached_entry);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/outgoing.py":182
 *             self.add_answer_at_time(cached_entry, now)
 * 
 *     def add_question_or_all_cache(             # <<<<<<<<<<<<<<
 *         self, cache: DNSCache, now: float, name: str, type_: int, class_: int
 *     ) -> None:
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_21add_question_or_all_cache(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_20add_question_or_all_cache[] = "Add a question if it is not already cached.\n        This is currently only used for IPv6 addresses.\n        ";
static PyMethodDef __pyx_mdef_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_21add_question_or_all_cache = {"add_question_or_all_cache", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_21add_question_or_all_cache, METH_VARARGS|METH_KEYWORDS, __pyx_doc_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_20add_question_or_all_cache};
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_21add_question_or_all_cache(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_cache = 0;
  double __pyx_v_now;
  PyObject *__pyx_v_name = 0;
  PyObject *__pyx_v_type_ = 0;
  PyObject *__pyx_v_class_ = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("add_question_or_all_cache (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_cache,&__pyx_n_s_now,&__pyx_n_s_name,&__pyx_n_s_type,&__pyx_n_s_class,0};
    PyObject* values[5] = {0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_cache)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_now)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("add_question_or_all_cache", 1, 5, 5, 1); __PYX_ERR(0, 182, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_name)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("add_question_or_all_cache", 1, 5, 5, 2); __PYX_ERR(0, 182, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_type)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("add_question_or_all_cache", 1, 5, 5, 3); __PYX_ERR(0, 182, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_class)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("add_question_or_all_cache", 1, 5, 5, 4); __PYX_ERR(0, 182, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "add_question_or_all_cache") < 0)) __PYX_ERR(0, 182, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 5) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
    }
    __pyx_v_cache = values[0];
    __pyx_v_now = __pyx_PyFloat_AsDouble(values[1]); if (unlikely((__pyx_v_now == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 183, __pyx_L3_error)
    __pyx_v_name = ((PyObject*)values[2]);
    __pyx_v_type_ = values[3];
    __pyx_v_class_ = values[4];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("add_question_or_all_cache", 1, 5, 5, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 182, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing.add_question_or_all_cache", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_name), (&PyUnicode_Type), 1, "name", 1))) __PYX_ERR(0, 183, __pyx_L1_error)
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_20add_question_or_all_cache(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self), __pyx_v_cache, __pyx_v_now, __pyx_v_name, __pyx_v_type_, __pyx_v_class_);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_20add_question_or_all_cache(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_cache, double __pyx_v_now, PyObject *__pyx_v_name, PyObject *__pyx_v_type_, PyObject *__pyx_v_class_) {
  PyObject *__pyx_v_cached_entries = NULL;
  PyObject *__pyx_v_cached_entry = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  Py_ssize_t __pyx_t_10;
  PyObject *(*__pyx_t_11)(PyObject *);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("add_question_or_all_cache", 0);

  /* "zeroconf/_protocol/outgoing.py":188
 *         This is currently only used for IPv6 addresses.
 *         """
 *         cached_entries = cache.get_all_by_details(name, type_, class_)             # <<<<<<<<<<<<<<
 *         if not cached_entries:
 *             self.add_question(DNSQuestion(name, type_, class_))
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_cache, __pyx_n_s_get_all_by_details); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 188, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_3, __pyx_v_name, __pyx_v_type_, __pyx_v_class_};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 3+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 188, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_3, __pyx_v_name, __pyx_v_type_, __pyx_v_class_};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 3+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 188, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(3+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 188, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_name);
    __Pyx_GIVEREF(__pyx_v_name);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_name);
    __Pyx_INCREF(__pyx_v_type_);
    __Pyx_GIVEREF(__pyx_v_type_);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_type_);
    __Pyx_INCREF(__pyx_v_class_);
    __Pyx_GIVEREF(__pyx_v_class_);
    PyTuple_SET_ITEM(__pyx_t_5, 2+__pyx_t_4, __pyx_v_class_);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 188, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_cached_entries = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "zeroconf/_protocol/outgoing.py":189
 *         """
 *         cached_entries = cache.get_all_by_details(name, type_, class_)
 *         if not cached_entries:             # <<<<<<<<<<<<<<
 *             self.add_question(DNSQuestion(name, type_, class_))
 *             return
 */
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_v_cached_entries); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 189, __pyx_L1_error)
  __pyx_t_7 = ((!__pyx_t_6) != 0);
  if (__pyx_t_7) {

    /* "zeroconf/_protocol/outgoing.py":190
 *         cached_entries = cache.get_all_by_details(name, type_, class_)
 *         if not cached_entries:
 *             self.add_question(DNSQuestion(name, type_, class_))             # <<<<<<<<<<<<<<
 *             return
 *         for cached_entry in cached_entries:
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_add_question); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 190, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_DNSQuestion); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 190, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_8 = NULL;
    __pyx_t_4 = 0;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
        __pyx_t_4 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[4] = {__pyx_t_8, __pyx_v_name, __pyx_v_type_, __pyx_v_class_};
      __pyx_t_5 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_4, 3+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 190, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_GOTREF(__pyx_t_5);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[4] = {__pyx_t_8, __pyx_v_name, __pyx_v_type_, __pyx_v_class_};
      __pyx_t_5 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_4, 3+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 190, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_GOTREF(__pyx_t_5);
    } else
    #endif
    {
      __pyx_t_9 = PyTuple_New(3+__pyx_t_4); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 190, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      if (__pyx_t_8) {
        __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8); __pyx_t_8 = NULL;
      }
      __Pyx_INCREF(__pyx_v_name);
      __Pyx_GIVEREF(__pyx_v_name);
      PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_4, __pyx_v_name);
      __Pyx_INCREF(__pyx_v_type_);
      __Pyx_GIVEREF(__pyx_v_type_);
      PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_4, __pyx_v_type_);
      __Pyx_INCREF(__pyx_v_class_);
      __Pyx_GIVEREF(__pyx_v_class_);
      PyTuple_SET_ITEM(__pyx_t_9, 2+__pyx_t_4, __pyx_v_class_);
      __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_9, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 190, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_5);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 190, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "zeroconf/_protocol/outgoing.py":191
 *         if not cached_entries:
 *             self.add_question(DNSQuestion(name, type_, class_))
 *             return             # <<<<<<<<<<<<<<
 *         for cached_entry in cached_entries:
 *             self.add_answer_at_time(cached_entry, now)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "zeroconf/_protocol/outgoing.py":189
 *         """
 *         cached_entries = cache.get_all_by_details(name, type_, class_)
 *         if not cached_entries:             # <<<<<<<<<<<<<<
 *             self.add_question(DNSQuestion(name, type_, class_))
 *             return
 */
  }

  /* "zeroconf/_protocol/outgoing.py":192
 *             self.add_question(DNSQuestion(name, type_, class_))
 *             return
 *         for cached_entry in cached_entries:             # <<<<<<<<<<<<<<
 *             self.add_answer_at_time(cached_entry, now)
 * 
 */
  if (likely(PyList_CheckExact(__pyx_v_cached_entries)) || PyTuple_CheckExact(__pyx_v_cached_entries)) {
    __pyx_t_1 = __pyx_v_cached_entries; __Pyx_INCREF(__pyx_t_1); __pyx_t_10 = 0;
    __pyx_t_11 = NULL;
  } else {
    __pyx_t_10 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_v_cached_entries); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 192, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_11 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 192, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_11)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        if (__pyx_t_10 >= PyList_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_10); __Pyx_INCREF(__pyx_t_2); __pyx_t_10++; if (unlikely(0 < 0)) __PYX_ERR(0, 192, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_10); __pyx_t_10++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 192, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      } else {
        if (__pyx_t_10 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_10); __Pyx_INCREF(__pyx_t_2); __pyx_t_10++; if (unlikely(0 < 0)) __PYX_ERR(0, 192, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_10); __pyx_t_10++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 192, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      }
    } else {
      __pyx_t_2 = __pyx_t_11(__pyx_t_1);
      if (unlikely(!__pyx_t_2)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 192, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_XDECREF_SET(__pyx_v_cached_entry, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "zeroconf/_protocol/outgoing.py":193
 *             return
 *         for cached_entry in cached_entries:
 *             self.add_answer_at_time(cached_entry, now)             # <<<<<<<<<<<<<<
 * 
 *     def _write_byte(self, value: int) -> None:
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_add_answer_at_time); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 193, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_3 = PyFloat_FromDouble(__pyx_v_now); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 193, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_9 = NULL;
    __pyx_t_4 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
        __pyx_t_4 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[3] = {__pyx_t_9, __pyx_v_cached_entry, __pyx_t_3};
      __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 193, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[3] = {__pyx_t_9, __pyx_v_cached_entry, __pyx_t_3};
      __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 193, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    } else
    #endif
    {
      __pyx_t_8 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 193, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      if (__pyx_t_9) {
        __Pyx_GIVEREF(__pyx_t_9); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_9); __pyx_t_9 = NULL;
      }
      __Pyx_INCREF(__pyx_v_cached_entry);
      __Pyx_GIVEREF(__pyx_v_cached_entry);
      PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_4, __pyx_v_cached_entry);
      __Pyx_GIVEREF(__pyx_t_3);
      PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_4, __pyx_t_3);
      __pyx_t_3 = 0;
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_8, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 193, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "zeroconf/_protocol/outgoing.py":192
 *             self.add_question(DNSQuestion(name, type_, class_))
 *             return
 *         for cached_entry in cached_entries:             # <<<<<<<<<<<<<<
 *             self.add_answer_at_time(cached_entry, now)
 * 
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "zeroconf/_protocol/outgoing.py":182
 *             self.add_answer_at_time(cached_entry, now)
 * 
 *     def add_question_or_all_cache(             # <<<<<<<<<<<<<<
 *         self, cache: DNSCache, now: float, name: str, type_: int, class_: int
 *     ) -> None:
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing.add_question_or_all_cache", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_cached_entries);
  __Pyx_XDECREF(__pyx_v_cached_entry);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/outgoing.py":195
 *             self.add_answer_at_time(cached_entry, now)
 * 
 *     def _write_byte(self, value: int) -> None:             # <<<<<<<<<<<<<<
 *         """Writes a single byte to the packet"""
 *         self.data.append(value.to_bytes(1, 'big'))
 */

static PyObject *__pyx_f_8zeroconf_9_protocol_8outgoing_11DNSOutgoing__write_byte(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_value) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_write_byte", 0);

  /* "zeroconf/_protocol/outgoing.py":197
 *     def _write_byte(self, value: int) -> None:
 *         """Writes a single byte to the packet"""
 *         self.data.append(value.to_bytes(1, 'big'))             # <<<<<<<<<<<<<<
 *         self.size += 1
 * 
 */
  if (unlikely(__pyx_v_self->data == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(0, 197, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_value, __pyx_n_s_to_bytes); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 197, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 197, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = __Pyx_PyList_Append(__pyx_v_self->data, __pyx_t_2); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(0, 197, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "zeroconf/_protocol/outgoing.py":198
 *         """Writes a single byte to the packet"""
 *         self.data.append(value.to_bytes(1, 'big'))
 *         self.size += 1             # <<<<<<<<<<<<<<
 * 
 *     def _insert_short_at_start(self, value: int) -> None:
 */
  __pyx_v_self->size = (__pyx_v_self->size + 1);

  /* "zeroconf/_protocol/outgoing.py":195
 *             self.add_answer_at_time(cached_entry, now)
 * 
 *     def _write_byte(self, value: int) -> None:             # <<<<<<<<<<<<<<
 *         """Writes a single byte to the packet"""
 *         self.data.append(value.to_bytes(1, 'big'))
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing._write_byte", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/outgoing.py":200
 *         self.size += 1
 * 
 *     def _insert_short_at_start(self, value: int) -> None:             # <<<<<<<<<<<<<<
 *         """Inserts an unsigned short at the start of the packet"""
 *         self.data.insert(0, value.to_bytes(2, 'big'))
 */

static PyObject *__pyx_f_8zeroconf_9_protocol_8outgoing_11DNSOutgoing__insert_short_at_start(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_value) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_insert_short_at_start", 0);

  /* "zeroconf/_protocol/outgoing.py":202
 *     def _insert_short_at_start(self, value: int) -> None:
 *         """Inserts an unsigned short at the start of the packet"""
 *         self.data.insert(0, value.to_bytes(2, 'big'))             # <<<<<<<<<<<<<<
 * 
 *     def _replace_short(self, index: int, value: int) -> None:
 */
  if (unlikely(__pyx_v_self->data == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "insert");
    __PYX_ERR(0, 202, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_value, __pyx_n_s_to_bytes); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 202, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 202, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = PyList_Insert(__pyx_v_self->data, 0, __pyx_t_2); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(0, 202, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "zeroconf/_protocol/outgoing.py":200
 *         self.size += 1
 * 
 *     def _insert_short_at_start(self, value: int) -> None:             # <<<<<<<<<<<<<<
 *         """Inserts an unsigned short at the start of the packet"""
 *         self.data.insert(0, value.to_bytes(2, 'big'))
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing._insert_short_at_start", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/outgoing.py":204
 *         self.data.insert(0, value.to_bytes(2, 'big'))
 * 
 *     def _replace_short(self, index: int, value: int) -> None:             # <<<<<<<<<<<<<<
 *         """Replaces an unsigned short in a certain position in the packet"""
 *         self.data[index] = value.to_bytes(2, 'big')
 */

static PyObject *__pyx_f_8zeroconf_9_protocol_8outgoing_11DNSOutgoing__replace_short(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_index, PyObject *__pyx_v_value) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_replace_short", 0);

  /* "zeroconf/_protocol/outgoing.py":206
 *     def _replace_short(self, index: int, value: int) -> None:
 *         """Replaces an unsigned short in a certain position in the packet"""
 *         self.data[index] = value.to_bytes(2, 'big')             # <<<<<<<<<<<<<<
 * 
 *     def write_short(self, value: int) -> None:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_value, __pyx_n_s_to_bytes); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 206, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 206, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(__pyx_v_self->data == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 206, __pyx_L1_error)
  }
  if (unlikely(PyObject_SetItem(__pyx_v_self->data, __pyx_v_index, __pyx_t_2) < 0)) __PYX_ERR(0, 206, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "zeroconf/_protocol/outgoing.py":204
 *         self.data.insert(0, value.to_bytes(2, 'big'))
 * 
 *     def _replace_short(self, index: int, value: int) -> None:             # <<<<<<<<<<<<<<
 *         """Replaces an unsigned short in a certain position in the packet"""
 *         self.data[index] = value.to_bytes(2, 'big')
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing._replace_short", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/outgoing.py":208
 *         self.data[index] = value.to_bytes(2, 'big')
 * 
 *     def write_short(self, value: int) -> None:             # <<<<<<<<<<<<<<
 *         """Writes an unsigned short to the packet"""
 *         self.data.append(value.to_bytes(2, 'big'))
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_23write_short(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static char __pyx_doc_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_22write_short[] = "Writes an unsigned short to the packet";
static PyMethodDef __pyx_mdef_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_23write_short = {"write_short", (PyCFunction)__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_23write_short, METH_O, __pyx_doc_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_22write_short};
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_23write_short(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("write_short (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_22write_short(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_22write_short(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_value) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("write_short", 0);

  /* "zeroconf/_protocol/outgoing.py":210
 *     def write_short(self, value: int) -> None:
 *         """Writes an unsigned short to the packet"""
 *         self.data.append(value.to_bytes(2, 'big'))             # <<<<<<<<<<<<<<
 *         self.size += 2
 * 
 */
  if (unlikely(__pyx_v_self->data == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(0, 210, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_value, __pyx_n_s_to_bytes); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 210, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 210, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = __Pyx_PyList_Append(__pyx_v_self->data, __pyx_t_2); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(0, 210, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "zeroconf/_protocol/outgoing.py":211
 *         """Writes an unsigned short to the packet"""
 *         self.data.append(value.to_bytes(2, 'big'))
 *         self.size += 2             # <<<<<<<<<<<<<<
 * 
 *     def _write_int(self, value: Union[float, int]) -> None:
 */
  __pyx_v_self->size = (__pyx_v_self->size + 2);

  /* "zeroconf/_protocol/outgoing.py":208
 *         self.data[index] = value.to_bytes(2, 'big')
 * 
 *     def write_short(self, value: int) -> None:             # <<<<<<<<<<<<<<
 *         """Writes an unsigned short to the packet"""
 *         self.data.append(value.to_bytes(2, 'big'))
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing.write_short", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/outgoing.py":213
 *         self.size += 2
 * 
 *     def _write_int(self, value: Union[float, int]) -> None:             # <<<<<<<<<<<<<<
 *         """Writes an unsigned integer to the packet"""
 *         self.data.append(int(value).to_bytes(4, 'big'))
 */

static PyObject *__pyx_f_8zeroconf_9_protocol_8outgoing_11DNSOutgoing__write_int(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_value) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_write_int", 0);

  /* "zeroconf/_protocol/outgoing.py":215
 *     def _write_int(self, value: Union[float, int]) -> None:
 *         """Writes an unsigned integer to the packet"""
 *         self.data.append(int(value).to_bytes(4, 'big'))             # <<<<<<<<<<<<<<
 *         self.size += 4
 * 
 */
  if (unlikely(__pyx_v_self->data == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(0, 215, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyNumber_Int(__pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 215, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_to_bytes); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 215, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 215, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __Pyx_PyList_Append(__pyx_v_self->data, __pyx_t_1); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(0, 215, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "zeroconf/_protocol/outgoing.py":216
 *         """Writes an unsigned integer to the packet"""
 *         self.data.append(int(value).to_bytes(4, 'big'))
 *         self.size += 4             # <<<<<<<<<<<<<<
 * 
 *     def write_string(self, value: bytes) -> None:
 */
  __pyx_v_self->size = (__pyx_v_self->size + 4);

  /* "zeroconf/_protocol/outgoing.py":213
 *         self.size += 2
 * 
 *     def _write_int(self, value: Union[float, int]) -> None:             # <<<<<<<<<<<<<<
 *         """Writes an unsigned integer to the packet"""
 *         self.data.append(int(value).to_bytes(4, 'big'))
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing._write_int", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/outgoing.py":218
 *         self.size += 4
 * 
 *     def write_string(self, value: bytes) -> None:             # <<<<<<<<<<<<<<
 *         """Writes a string to the packet"""
 *         assert isinstance(value, bytes)
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_25write_string(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static char __pyx_doc_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_24write_string[] = "Writes a string to the packet";
static PyMethodDef __pyx_mdef_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_25write_string = {"write_string", (PyCFunction)__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_25write_string, METH_O, __pyx_doc_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_24write_string};
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_25write_string(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("write_string (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_value), (&PyBytes_Type), 1, "value", 1))) __PYX_ERR(0, 218, __pyx_L1_error)
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_24write_string(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self), ((PyObject*)__pyx_v_value));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_24write_string(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_value) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("write_string", 0);

  /* "zeroconf/_protocol/outgoing.py":220
 *     def write_string(self, value: bytes) -> None:
 *         """Writes a string to the packet"""
 *         assert isinstance(value, bytes)             # <<<<<<<<<<<<<<
 *         self.data.append(value)
 *         self.size += len(value)
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    __pyx_t_1 = PyBytes_Check(__pyx_v_value); 
    if (unlikely(!(__pyx_t_1 != 0))) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 220, __pyx_L1_error)
    }
  }
  #endif

  /* "zeroconf/_protocol/outgoing.py":221
 *         """Writes a string to the packet"""
 *         assert isinstance(value, bytes)
 *         self.data.append(value)             # <<<<<<<<<<<<<<
 *         self.size += len(value)
 * 
 */
  if (unlikely(__pyx_v_self->data == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(0, 221, __pyx_L1_error)
  }
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->data, __pyx_v_value); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 221, __pyx_L1_error)

  /* "zeroconf/_protocol/outgoing.py":222
 *         assert isinstance(value, bytes)
 *         self.data.append(value)
 *         self.size += len(value)             # <<<<<<<<<<<<<<
 * 
 *     def _write_utf(self, s: str) -> None:
 */
  if (unlikely(__pyx_v_value == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 222, __pyx_L1_error)
  }
  __pyx_t_3 = PyBytes_GET_SIZE(__pyx_v_value); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(0, 222, __pyx_L1_error)
  __pyx_v_self->size = (__pyx_v_self->size + __pyx_t_3);

  /* "zeroconf/_protocol/outgoing.py":218
 *         self.size += 4
 * 
 *     def write_string(self, value: bytes) -> None:             # <<<<<<<<<<<<<<
 *         """Writes a string to the packet"""
 *         assert isinstance(value, bytes)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing.write_string", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/outgoing.py":224
 *         self.size += len(value)
 * 
 *     def _write_utf(self, s: str) -> None:             # <<<<<<<<<<<<<<
 *         """Writes a UTF-8 string of a given length to the packet"""
 *         utfstr = s.encode('utf-8')
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_27_write_utf(PyObject *__pyx_v_self, PyObject *__pyx_v_s); /*proto*/
static char __pyx_doc_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_26_write_utf[] = "Writes a UTF-8 string of a given length to the packet";
static PyMethodDef __pyx_mdef_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_27_write_utf = {"_write_utf", (PyCFunction)__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_27_write_utf, METH_O, __pyx_doc_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_26_write_utf};
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_27_write_utf(PyObject *__pyx_v_self, PyObject *__pyx_v_s) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_write_utf (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_s), (&PyUnicode_Type), 1, "s", 1))) __PYX_ERR(0, 224, __pyx_L1_error)
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_26_write_utf(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self), ((PyObject*)__pyx_v_s));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_26_write_utf(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_s) {
  PyObject *__pyx_v_utfstr = NULL;
  Py_ssize_t __pyx_v_length;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_write_utf", 0);

  /* "zeroconf/_protocol/outgoing.py":226
 *     def _write_utf(self, s: str) -> None:
 *         """Writes a UTF-8 string of a given length to the packet"""
 *         utfstr = s.encode('utf-8')             # <<<<<<<<<<<<<<
 *         length = len(utfstr)
 *         if length > 64:
 */
  if (unlikely(__pyx_v_s == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "encode");
    __PYX_ERR(0, 226, __pyx_L1_error)
  }
  __pyx_t_1 = PyUnicode_AsUTF8String(__pyx_v_s); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 226, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_utfstr = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "zeroconf/_protocol/outgoing.py":227
 *         """Writes a UTF-8 string of a given length to the packet"""
 *         utfstr = s.encode('utf-8')
 *         length = len(utfstr)             # <<<<<<<<<<<<<<
 *         if length > 64:
 *             raise NamePartTooLongException
 */
  __pyx_t_2 = PyObject_Length(__pyx_v_utfstr); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1))) __PYX_ERR(0, 227, __pyx_L1_error)
  __pyx_v_length = __pyx_t_2;

  /* "zeroconf/_protocol/outgoing.py":228
 *         utfstr = s.encode('utf-8')
 *         length = len(utfstr)
 *         if length > 64:             # <<<<<<<<<<<<<<
 *             raise NamePartTooLongException
 *         self._write_byte(length)
 */
  __pyx_t_3 = ((__pyx_v_length > 64) != 0);
  if (unlikely(__pyx_t_3)) {

    /* "zeroconf/_protocol/outgoing.py":229
 *         length = len(utfstr)
 *         if length > 64:
 *             raise NamePartTooLongException             # <<<<<<<<<<<<<<
 *         self._write_byte(length)
 *         self.write_string(utfstr)
 */
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_NamePartTooLongException); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 229, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 229, __pyx_L1_error)

    /* "zeroconf/_protocol/outgoing.py":228
 *         utfstr = s.encode('utf-8')
 *         length = len(utfstr)
 *         if length > 64:             # <<<<<<<<<<<<<<
 *             raise NamePartTooLongException
 *         self._write_byte(length)
 */
  }

  /* "zeroconf/_protocol/outgoing.py":230
 *         if length > 64:
 *             raise NamePartTooLongException
 *         self._write_byte(length)             # <<<<<<<<<<<<<<
 *         self.write_string(utfstr)
 * 
 */
  __pyx_t_1 = PyInt_FromSsize_t(__pyx_v_length); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 230, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = ((struct __pyx_vtabstruct_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self->__pyx_vtab)->_write_byte(__pyx_v_self, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 230, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "zeroconf/_protocol/outgoing.py":231
 *             raise NamePartTooLongException
 *         self._write_byte(length)
 *         self.write_string(utfstr)             # <<<<<<<<<<<<<<
 * 
 *     def write_character_string(self, value: bytes) -> None:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_write_string); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 231, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_4 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_5, __pyx_v_utfstr) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_v_utfstr);
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 231, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "zeroconf/_protocol/outgoing.py":224
 *         self.size += len(value)
 * 
 *     def _write_utf(self, s: str) -> None:             # <<<<<<<<<<<<<<
 *         """Writes a UTF-8 string of a given length to the packet"""
 *         utfstr = s.encode('utf-8')
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing._write_utf", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_utfstr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/outgoing.py":233
 *         self.write_string(utfstr)
 * 
 *     def write_character_string(self, value: bytes) -> None:             # <<<<<<<<<<<<<<
 *         assert isinstance(value, bytes)
 *         length = len(value)
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_29write_character_string(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static PyMethodDef __pyx_mdef_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_29write_character_string = {"write_character_string", (PyCFunction)__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_29write_character_string, METH_O, 0};
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_29write_character_string(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("write_character_string (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_value), (&PyBytes_Type), 1, "value", 1))) __PYX_ERR(0, 233, __pyx_L1_error)
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_28write_character_string(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self), ((PyObject*)__pyx_v_value));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_28write_character_string(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_value) {
  Py_ssize_t __pyx_v_length;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("write_character_string", 0);

  /* "zeroconf/_protocol/outgoing.py":234
 * 
 *     def write_character_string(self, value: bytes) -> None:
 *         assert isinstance(value, bytes)             # <<<<<<<<<<<<<<
 *         length = len(value)
 *         if length > 256:
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    __pyx_t_1 = PyBytes_Check(__pyx_v_value); 
    if (unlikely(!(__pyx_t_1 != 0))) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 234, __pyx_L1_error)
    }
  }
  #endif

  /* "zeroconf/_protocol/outgoing.py":235
 *     def write_character_string(self, value: bytes) -> None:
 *         assert isinstance(value, bytes)
 *         length = len(value)             # <<<<<<<<<<<<<<
 *         if length > 256:
 *             raise NamePartTooLongException
 */
  if (unlikely(__pyx_v_value == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 235, __pyx_L1_error)
  }
  __pyx_t_2 = PyBytes_GET_SIZE(__pyx_v_value); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1))) __PYX_ERR(0, 235, __pyx_L1_error)
  __pyx_v_length = __pyx_t_2;

  /* "zeroconf/_protocol/outgoing.py":236
 *         assert isinstance(value, bytes)
 *         length = len(value)
 *         if length > 256:             # <<<<<<<<<<<<<<
 *             raise NamePartTooLongException
 *         self._write_byte(length)
 */
  __pyx_t_1 = ((__pyx_v_length > 0x100) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "zeroconf/_protocol/outgoing.py":237
 *         length = len(value)
 *         if length > 256:
 *             raise NamePartTooLongException             # <<<<<<<<<<<<<<
 *         self._write_byte(length)
 *         self.write_string(value)
 */
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_NamePartTooLongException); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 237, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 237, __pyx_L1_error)

    /* "zeroconf/_protocol/outgoing.py":236
 *         assert isinstance(value, bytes)
 *         length = len(value)
 *         if length > 256:             # <<<<<<<<<<<<<<
 *             raise NamePartTooLongException
 *         self._write_byte(length)
 */
  }

  /* "zeroconf/_protocol/outgoing.py":238
 *         if length > 256:
 *             raise NamePartTooLongException
 *         self._write_byte(length)             # <<<<<<<<<<<<<<
 *         self.write_string(value)
 * 
 */
  __pyx_t_3 = PyInt_FromSsize_t(__pyx_v_length); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 238, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = ((struct __pyx_vtabstruct_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self->__pyx_vtab)->_write_byte(__pyx_v_self, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 238, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "zeroconf/_protocol/outgoing.py":239
 *             raise NamePartTooLongException
 *         self._write_byte(length)
 *         self.write_string(value)             # <<<<<<<<<<<<<<
 * 
 *     def write_name(self, name: str) -> None:
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_write_string); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 239, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_4 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_v_value) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_value);
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 239, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "zeroconf/_protocol/outgoing.py":233
 *         self.write_string(utfstr)
 * 
 *     def write_character_string(self, value: bytes) -> None:             # <<<<<<<<<<<<<<
 *         assert isinstance(value, bytes)
 *         length = len(value)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing.write_character_string", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/outgoing.py":241
 *         self.write_string(value)
 * 
 *     def write_name(self, name: str) -> None:             # <<<<<<<<<<<<<<
 *         """
 *         Write names to packet
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_31write_name(PyObject *__pyx_v_self, PyObject *__pyx_v_name); /*proto*/
static char __pyx_doc_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_30write_name[] = "\n        Write names to packet\n\n        18.14. Name Compression\n\n        When generating Multicast DNS messages, implementations SHOULD use\n        name compression wherever possible to compress the names of resource\n        records, by replacing some or all of the resource record name with a\n        compact two-byte reference to an appearance of that data somewhere\n        earlier in the message [RFC1035].\n        ";
static PyMethodDef __pyx_mdef_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_31write_name = {"write_name", (PyCFunction)__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_31write_name, METH_O, __pyx_doc_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_30write_name};
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_31write_name(PyObject *__pyx_v_self, PyObject *__pyx_v_name) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("write_name (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_name), (&PyUnicode_Type), 1, "name", 1))) __PYX_ERR(0, 241, __pyx_L1_error)
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_30write_name(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self), ((PyObject*)__pyx_v_name));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_30write_name(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_name) {
  PyObject *__pyx_v_name_length = NULL;
  PyObject *__pyx_v_labels = NULL;
  PyObject *__pyx_v_start_size = NULL;
  Py_ssize_t __pyx_v_count;
  PyObject *__pyx_v_label = NULL;
  PyObject *__pyx_v_index = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("write_name", 0);
  __Pyx_INCREF(__pyx_v_name);

  /* "zeroconf/_protocol/outgoing.py":255
 * 
 *         # split name into each label
 *         name_length = None             # <<<<<<<<<<<<<<
 *         if name.endswith('.'):
 *             name = name[: len(name) - 1]
 */
  __Pyx_INCREF(Py_None);
  __pyx_v_name_length = Py_None;

  /* "zeroconf/_protocol/outgoing.py":256
 *         # split name into each label
 *         name_length = None
 *         if name.endswith('.'):             # <<<<<<<<<<<<<<
 *             name = name[: len(name) - 1]
 *         labels = name.split('.')
 */
  if (unlikely(__pyx_v_name == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "endswith");
    __PYX_ERR(0, 256, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyUnicode_Tailmatch(__pyx_v_name, __pyx_kp_u__5, 0, PY_SSIZE_T_MAX, 1); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(0, 256, __pyx_L1_error)
  if ((__pyx_t_1 != 0)) {

    /* "zeroconf/_protocol/outgoing.py":257
 *         name_length = None
 *         if name.endswith('.'):
 *             name = name[: len(name) - 1]             # <<<<<<<<<<<<<<
 *         labels = name.split('.')
 *         # Write each new label or a pointer to the existing
 */
    if (unlikely(__pyx_v_name == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 257, __pyx_L1_error)
    }
    if (unlikely(__pyx_v_name == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(0, 257, __pyx_L1_error)
    }
    __pyx_t_2 = __Pyx_PyUnicode_GET_LENGTH(__pyx_v_name); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1))) __PYX_ERR(0, 257, __pyx_L1_error)
    __pyx_t_3 = __Pyx_PyUnicode_Substring(__pyx_v_name, 0, (__pyx_t_2 - 1)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 257, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF_SET(__pyx_v_name, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "zeroconf/_protocol/outgoing.py":256
 *         # split name into each label
 *         name_length = None
 *         if name.endswith('.'):             # <<<<<<<<<<<<<<
 *             name = name[: len(name) - 1]
 *         labels = name.split('.')
 */
  }

  /* "zeroconf/_protocol/outgoing.py":258
 *         if name.endswith('.'):
 *             name = name[: len(name) - 1]
 *         labels = name.split('.')             # <<<<<<<<<<<<<<
 *         # Write each new label or a pointer to the existing
 *         # on in the packet
 */
  if (unlikely(__pyx_v_name == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "split");
    __PYX_ERR(0, 258, __pyx_L1_error)
  }
  __pyx_t_3 = PyUnicode_Split(__pyx_v_name, __pyx_kp_u__5, -1L); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 258, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v_labels = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "zeroconf/_protocol/outgoing.py":261
 *         # Write each new label or a pointer to the existing
 *         # on in the packet
 *         start_size = self.size             # <<<<<<<<<<<<<<
 *         for count in range(len(labels)):
 *             label = name if count == 0 else '.'.join(labels[count:])
 */
  __pyx_t_3 = __Pyx_PyInt_From_unsigned_int(__pyx_v_self->size); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 261, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v_start_size = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "zeroconf/_protocol/outgoing.py":262
 *         # on in the packet
 *         start_size = self.size
 *         for count in range(len(labels)):             # <<<<<<<<<<<<<<
 *             label = name if count == 0 else '.'.join(labels[count:])
 *             index = self.names.get(label)
 */
  __pyx_t_2 = PyObject_Length(__pyx_v_labels); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1))) __PYX_ERR(0, 262, __pyx_L1_error)
  __pyx_t_4 = __pyx_t_2;
  for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
    __pyx_v_count = __pyx_t_5;

    /* "zeroconf/_protocol/outgoing.py":263
 *         start_size = self.size
 *         for count in range(len(labels)):
 *             label = name if count == 0 else '.'.join(labels[count:])             # <<<<<<<<<<<<<<
 *             index = self.names.get(label)
 *             if index:
 */
    if (((__pyx_v_count == 0) != 0)) {
      __Pyx_INCREF(__pyx_v_name);
      __pyx_t_3 = __pyx_v_name;
    } else {
      __pyx_t_6 = __Pyx_PyObject_GetSlice(__pyx_v_labels, __pyx_v_count, 0, NULL, NULL, NULL, 1, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 263, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_7 = PyUnicode_Join(__pyx_kp_u__5, __pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 263, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_3 = __pyx_t_7;
      __pyx_t_7 = 0;
    }
    __Pyx_XDECREF_SET(__pyx_v_label, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "zeroconf/_protocol/outgoing.py":264
 *         for count in range(len(labels)):
 *             label = name if count == 0 else '.'.join(labels[count:])
 *             index = self.names.get(label)             # <<<<<<<<<<<<<<
 *             if index:
 *                 # If part of the name already exists in the packet,
 */
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->names, __pyx_n_s_get); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 264, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_6 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_3 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_6, __pyx_v_label) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_v_label);
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 264, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_XDECREF_SET(__pyx_v_index, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "zeroconf/_protocol/outgoing.py":265
 *             label = name if count == 0 else '.'.join(labels[count:])
 *             index = self.names.get(label)
 *             if index:             # <<<<<<<<<<<<<<
 *                 # If part of the name already exists in the packet,
 *                 # create a pointer to it
 */
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_index); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 265, __pyx_L1_error)
    if (__pyx_t_1) {

      /* "zeroconf/_protocol/outgoing.py":268
 *                 # If part of the name already exists in the packet,
 *                 # create a pointer to it
 *                 self._write_byte((index >> 8) | 0xC0)             # <<<<<<<<<<<<<<
 *                 self._write_byte(index & 0xFF)
 *                 return
 */
      __pyx_t_3 = __Pyx_PyInt_RshiftObjC(__pyx_v_index, __pyx_int_8, 8, 0, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 268, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_7 = __Pyx_PyInt_OrObjC(__pyx_t_3, __pyx_int_192, 0xC0, 0, 0); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 268, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = ((struct __pyx_vtabstruct_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self->__pyx_vtab)->_write_byte(__pyx_v_self, __pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 268, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "zeroconf/_protocol/outgoing.py":269
 *                 # create a pointer to it
 *                 self._write_byte((index >> 8) | 0xC0)
 *                 self._write_byte(index & 0xFF)             # <<<<<<<<<<<<<<
 *                 return
 *             if name_length is None:
 */
      __pyx_t_3 = __Pyx_PyInt_AndObjC(__pyx_v_index, __pyx_int_255, 0xFF, 0, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 269, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_7 = ((struct __pyx_vtabstruct_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self->__pyx_vtab)->_write_byte(__pyx_v_self, __pyx_t_3); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 269, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

      /* "zeroconf/_protocol/outgoing.py":270
 *                 self._write_byte((index >> 8) | 0xC0)
 *                 self._write_byte(index & 0xFF)
 *                 return             # <<<<<<<<<<<<<<
 *             if name_length is None:
 *                 name_length = len(name.encode('utf-8'))
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_r = Py_None; __Pyx_INCREF(Py_None);
      goto __pyx_L0;

      /* "zeroconf/_protocol/outgoing.py":265
 *             label = name if count == 0 else '.'.join(labels[count:])
 *             index = self.names.get(label)
 *             if index:             # <<<<<<<<<<<<<<
 *                 # If part of the name already exists in the packet,
 *                 # create a pointer to it
 */
    }

    /* "zeroconf/_protocol/outgoing.py":271
 *                 self._write_byte(index & 0xFF)
 *                 return
 *             if name_length is None:             # <<<<<<<<<<<<<<
 *                 name_length = len(name.encode('utf-8'))
 *             self.names[label] = start_size + name_length - len(label.encode('utf-8'))
 */
    __pyx_t_1 = (__pyx_v_name_length == Py_None);
    __pyx_t_8 = (__pyx_t_1 != 0);
    if (__pyx_t_8) {

      /* "zeroconf/_protocol/outgoing.py":272
 *                 return
 *             if name_length is None:
 *                 name_length = len(name.encode('utf-8'))             # <<<<<<<<<<<<<<
 *             self.names[label] = start_size + name_length - len(label.encode('utf-8'))
 *             self._write_utf(labels[count])
 */
      if (unlikely(__pyx_v_name == Py_None)) {
        PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "encode");
        __PYX_ERR(0, 272, __pyx_L1_error)
      }
      __pyx_t_7 = PyUnicode_AsUTF8String(__pyx_v_name); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 272, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_9 = PyBytes_GET_SIZE(__pyx_t_7); if (unlikely(__pyx_t_9 == ((Py_ssize_t)-1))) __PYX_ERR(0, 272, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_7 = PyInt_FromSsize_t(__pyx_t_9); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 272, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF_SET(__pyx_v_name_length, __pyx_t_7);
      __pyx_t_7 = 0;

      /* "zeroconf/_protocol/outgoing.py":271
 *                 self._write_byte(index & 0xFF)
 *                 return
 *             if name_length is None:             # <<<<<<<<<<<<<<
 *                 name_length = len(name.encode('utf-8'))
 *             self.names[label] = start_size + name_length - len(label.encode('utf-8'))
 */
    }

    /* "zeroconf/_protocol/outgoing.py":273
 *             if name_length is None:
 *                 name_length = len(name.encode('utf-8'))
 *             self.names[label] = start_size + name_length - len(label.encode('utf-8'))             # <<<<<<<<<<<<<<
 *             self._write_utf(labels[count])
 * 
 */
    __pyx_t_7 = PyNumber_Add(__pyx_v_start_size, __pyx_v_name_length); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 273, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (unlikely(__pyx_v_label == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "encode");
      __PYX_ERR(0, 273, __pyx_L1_error)
    }
    __pyx_t_3 = PyUnicode_AsUTF8String(__pyx_v_label); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 273, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_9 = PyBytes_GET_SIZE(__pyx_t_3); if (unlikely(__pyx_t_9 == ((Py_ssize_t)-1))) __PYX_ERR(0, 273, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyInt_FromSsize_t(__pyx_t_9); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 273, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_6 = PyNumber_Subtract(__pyx_t_7, __pyx_t_3); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 273, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(PyObject_SetItem(__pyx_v_self->names, __pyx_v_label, __pyx_t_6) < 0)) __PYX_ERR(0, 273, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

    /* "zeroconf/_protocol/outgoing.py":274
 *                 name_length = len(name.encode('utf-8'))
 *             self.names[label] = start_size + name_length - len(label.encode('utf-8'))
 *             self._write_utf(labels[count])             # <<<<<<<<<<<<<<
 * 
 *         # this is the end of a name
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_write_utf); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 274, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_7 = __Pyx_GetItemInt(__pyx_v_labels, __pyx_v_count, Py_ssize_t, 1, PyInt_FromSsize_t, 0, 1, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 274, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_10 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_6 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_10, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_7);
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 274, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  }

  /* "zeroconf/_protocol/outgoing.py":277
 * 
 *         # this is the end of a name
 *         self._write_byte(0)             # <<<<<<<<<<<<<<
 * 
 *     def _write_question(self, question: DNSQuestion) -> bool:
 */
  __pyx_t_6 = ((struct __pyx_vtabstruct_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self->__pyx_vtab)->_write_byte(__pyx_v_self, __pyx_int_0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 277, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "zeroconf/_protocol/outgoing.py":241
 *         self.write_string(value)
 * 
 *     def write_name(self, name: str) -> None:             # <<<<<<<<<<<<<<
 *         """
 *         Write names to packet
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing.write_name", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_name_length);
  __Pyx_XDECREF(__pyx_v_labels);
  __Pyx_XDECREF(__pyx_v_start_size);
  __Pyx_XDECREF(__pyx_v_label);
  __Pyx_XDECREF(__pyx_v_index);
  __Pyx_XDECREF(__pyx_v_name);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/outgoing.py":279
 *         self._write_byte(0)
 * 
 *     def _write_question(self, question: DNSQuestion) -> bool:             # <<<<<<<<<<<<<<
 *         """Writes a question to the packet"""
 *         start_data_length, start_size = len(self.data), self.size
 */

static PyObject *__pyx_f_8zeroconf_9_protocol_8outgoing_11DNSOutgoing__write_question(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_question) {
  Py_ssize_t __pyx_v_start_data_length;
  unsigned int __pyx_v_start_size;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  unsigned int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_write_question", 0);

  /* "zeroconf/_protocol/outgoing.py":281
 *     def _write_question(self, question: DNSQuestion) -> bool:
 *         """Writes a question to the packet"""
 *         start_data_length, start_size = len(self.data), self.size             # <<<<<<<<<<<<<<
 *         self.write_name(question.name)
 *         self.write_short(question.type)
 */
  __pyx_t_1 = __pyx_v_self->data;
  __Pyx_INCREF(__pyx_t_1);
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 281, __pyx_L1_error)
  }
  __pyx_t_2 = PyList_GET_SIZE(__pyx_t_1); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1))) __PYX_ERR(0, 281, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = __pyx_v_self->size;
  __pyx_v_start_data_length = __pyx_t_2;
  __pyx_v_start_size = __pyx_t_3;

  /* "zeroconf/_protocol/outgoing.py":282
 *         """Writes a question to the packet"""
 *         start_data_length, start_size = len(self.data), self.size
 *         self.write_name(question.name)             # <<<<<<<<<<<<<<
 *         self.write_short(question.type)
 *         self._write_record_class(question)
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_write_name); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 282, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_question, __pyx_n_s_name); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 282, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  __pyx_t_1 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_6, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 282, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "zeroconf/_protocol/outgoing.py":283
 *         start_data_length, start_size = len(self.data), self.size
 *         self.write_name(question.name)
 *         self.write_short(question.type)             # <<<<<<<<<<<<<<
 *         self._write_record_class(question)
 *         return self._check_data_limit_or_rollback(start_data_length, start_size)
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_write_short); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 283, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_question, __pyx_n_s_type_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 283, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  __pyx_t_1 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_6, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 283, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "zeroconf/_protocol/outgoing.py":284
 *         self.write_name(question.name)
 *         self.write_short(question.type)
 *         self._write_record_class(question)             # <<<<<<<<<<<<<<
 *         return self._check_data_limit_or_rollback(start_data_length, start_size)
 * 
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self->__pyx_vtab)->_write_record_class(__pyx_v_self, __pyx_v_question); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 284, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "zeroconf/_protocol/outgoing.py":285
 *         self.write_short(question.type)
 *         self._write_record_class(question)
 *         return self._check_data_limit_or_rollback(start_data_length, start_size)             # <<<<<<<<<<<<<<
 * 
 *     def _write_record_class(self, record: Union[DNSQuestion, DNSRecord]) -> None:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyInt_FromSsize_t(__pyx_v_start_data_length); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 285, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyInt_From_unsigned_int(__pyx_v_start_size); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 285, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = ((struct __pyx_vtabstruct_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self->__pyx_vtab)->_check_data_limit_or_rollback(__pyx_v_self, __pyx_t_1, __pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 285, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "zeroconf/_protocol/outgoing.py":279
 *         self._write_byte(0)
 * 
 *     def _write_question(self, question: DNSQuestion) -> bool:             # <<<<<<<<<<<<<<
 *         """Writes a question to the packet"""
 *         start_data_length, start_size = len(self.data), self.size
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing._write_question", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/outgoing.py":287
 *         return self._check_data_limit_or_rollback(start_data_length, start_size)
 * 
 *     def _write_record_class(self, record: Union[DNSQuestion, DNSRecord]) -> None:             # <<<<<<<<<<<<<<
 *         """Write out the record class including the unique/unicast (QU) bit."""
 *         if record.unique and self.multicast:
 */

static PyObject *__pyx_f_8zeroconf_9_protocol_8outgoing_11DNSOutgoing__write_record_class(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_record) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_write_record_class", 0);

  /* "zeroconf/_protocol/outgoing.py":289
 *     def _write_record_class(self, record: Union[DNSQuestion, DNSRecord]) -> None:
 *         """Write out the record class including the unique/unicast (QU) bit."""
 *         if record.unique and self.multicast:             # <<<<<<<<<<<<<<
 *             self.write_short(record.class_ | _CLASS_UNIQUE)
 *         else:
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_record, __pyx_n_s_unique); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 289, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 289, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = (__pyx_v_self->multicast != 0);
  __pyx_t_1 = __pyx_t_3;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "zeroconf/_protocol/outgoing.py":290
 *         """Write out the record class including the unique/unicast (QU) bit."""
 *         if record.unique and self.multicast:
 *             self.write_short(record.class_ | _CLASS_UNIQUE)             # <<<<<<<<<<<<<<
 *         else:
 *             self.write_short(record.class_)
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_write_short); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 290, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_record, __pyx_n_s_class); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 290, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = __Pyx_PyInt_From_unsigned_int(__pyx_v_8zeroconf_9_protocol_8outgoing__CLASS_UNIQUE); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 290, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = PyNumber_Or(__pyx_t_5, __pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 290, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    __pyx_t_2 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_6, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_7);
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 290, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "zeroconf/_protocol/outgoing.py":289
 *     def _write_record_class(self, record: Union[DNSQuestion, DNSRecord]) -> None:
 *         """Write out the record class including the unique/unicast (QU) bit."""
 *         if record.unique and self.multicast:             # <<<<<<<<<<<<<<
 *             self.write_short(record.class_ | _CLASS_UNIQUE)
 *         else:
 */
    goto __pyx_L3;
  }

  /* "zeroconf/_protocol/outgoing.py":292
 *             self.write_short(record.class_ | _CLASS_UNIQUE)
 *         else:
 *             self.write_short(record.class_)             # <<<<<<<<<<<<<<
 * 
 *     def _write_ttl(self, record: DNSRecord, now: float) -> None:
 */
  /*else*/ {
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_write_short); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 292, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_record, __pyx_n_s_class); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 292, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_6 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    __pyx_t_2 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_6, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_7);
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 292, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __pyx_L3:;

  /* "zeroconf/_protocol/outgoing.py":287
 *         return self._check_data_limit_or_rollback(start_data_length, start_size)
 * 
 *     def _write_record_class(self, record: Union[DNSQuestion, DNSRecord]) -> None:             # <<<<<<<<<<<<<<
 *         """Write out the record class including the unique/unicast (QU) bit."""
 *         if record.unique and self.multicast:
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing._write_record_class", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/outgoing.py":294
 *             self.write_short(record.class_)
 * 
 *     def _write_ttl(self, record: DNSRecord, now: float) -> None:             # <<<<<<<<<<<<<<
 *         """Write out the record ttl."""
 *         self._write_int(record.ttl if now == 0 else record.get_remaining_ttl(now))
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_33_write_ttl(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_32_write_ttl[] = "Write out the record ttl.";
static PyMethodDef __pyx_mdef_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_33_write_ttl = {"_write_ttl", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_33_write_ttl, METH_VARARGS|METH_KEYWORDS, __pyx_doc_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_32_write_ttl};
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_33_write_ttl(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_record = 0;
  double __pyx_v_now;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_write_ttl (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_record,&__pyx_n_s_now,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_record)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_now)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_write_ttl", 1, 2, 2, 1); __PYX_ERR(0, 294, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_write_ttl") < 0)) __PYX_ERR(0, 294, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_record = values[0];
    __pyx_v_now = __pyx_PyFloat_AsDouble(values[1]); if (unlikely((__pyx_v_now == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 294, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_write_ttl", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 294, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing._write_ttl", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_32_write_ttl(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self), __pyx_v_record, __pyx_v_now);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_32_write_ttl(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_record, double __pyx_v_now) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_write_ttl", 0);

  /* "zeroconf/_protocol/outgoing.py":296
 *     def _write_ttl(self, record: DNSRecord, now: float) -> None:
 *         """Write out the record ttl."""
 *         self._write_int(record.ttl if now == 0 else record.get_remaining_ttl(now))             # <<<<<<<<<<<<<<
 * 
 *     def _write_record(self, record: DNSRecord, now: float) -> bool:
 */
  if (((__pyx_v_now == 0.0) != 0)) {
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_record, __pyx_n_s_ttl); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 296, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __pyx_t_2;
    __pyx_t_2 = 0;
  } else {
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_record, __pyx_n_s_get_remaining_ttl); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 296, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyFloat_FromDouble(__pyx_v_now); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 296, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 296, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_1 = __pyx_t_2;
    __pyx_t_2 = 0;
  }
  __pyx_t_2 = ((struct __pyx_vtabstruct_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self->__pyx_vtab)->_write_int(__pyx_v_self, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 296, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "zeroconf/_protocol/outgoing.py":294
 *             self.write_short(record.class_)
 * 
 *     def _write_ttl(self, record: DNSRecord, now: float) -> None:             # <<<<<<<<<<<<<<
 *         """Write out the record ttl."""
 *         self._write_int(record.ttl if now == 0 else record.get_remaining_ttl(now))
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing._write_ttl", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/outgoing.py":298
 *         self._write_int(record.ttl if now == 0 else record.get_remaining_ttl(now))
 * 
 *     def _write_record(self, record: DNSRecord, now: float) -> bool:             # <<<<<<<<<<<<<<
 *         """Writes a record (answer, authoritative answer, additional) to
 *         the packet.  Returns True on success, or False if we did not
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_35_write_record(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_34_write_record[] = "Writes a record (answer, authoritative answer, additional) to\n        the packet.  Returns True on success, or False if we did not\n        because the packet because the record does not fit.";
static PyMethodDef __pyx_mdef_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_35_write_record = {"_write_record", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_35_write_record, METH_VARARGS|METH_KEYWORDS, __pyx_doc_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_34_write_record};
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_35_write_record(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_record = 0;
  double __pyx_v_now;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_write_record (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_record,&__pyx_n_s_now,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_record)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_now)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_write_record", 1, 2, 2, 1); __PYX_ERR(0, 298, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_write_record") < 0)) __PYX_ERR(0, 298, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_record = values[0];
    __pyx_v_now = __pyx_PyFloat_AsDouble(values[1]); if (unlikely((__pyx_v_now == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 298, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_write_record", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 298, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing._write_record", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_34_write_record(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self), __pyx_v_record, __pyx_v_now);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_13_write_record_2generator(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "zeroconf/_protocol/outgoing.py":311
 *         record.write(self)
 *         # Adjust size for the short we will write before this record
 *         length = sum(len(d) for d in self.data[index + 1 :])             # <<<<<<<<<<<<<<
 *         # Here we replace the 0 length short we wrote
 *         # before with the actual length
 */

static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_13_write_record_genexpr(PyObject *__pyx_self) {
  struct __pyx_obj_8zeroconf_9_protocol_8outgoing___pyx_scope_struct_1_genexpr *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("genexpr", 0);
  __pyx_cur_scope = (struct __pyx_obj_8zeroconf_9_protocol_8outgoing___pyx_scope_struct_1_genexpr *)__pyx_tp_new_8zeroconf_9_protocol_8outgoing___pyx_scope_struct_1_genexpr(__pyx_ptype_8zeroconf_9_protocol_8outgoing___pyx_scope_struct_1_genexpr, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_8zeroconf_9_protocol_8outgoing___pyx_scope_struct_1_genexpr *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 311, __pyx_L1_error)
  } else {
    __Pyx_GOTREF(__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_outer_scope = (struct __pyx_obj_8zeroconf_9_protocol_8outgoing___pyx_scope_struct___write_record *) __pyx_self;
  __Pyx_INCREF(((PyObject *)__pyx_cur_scope->__pyx_outer_scope));
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_outer_scope);
  {
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_13_write_record_2generator, NULL, (PyObject *) __pyx_cur_scope, __pyx_n_s_genexpr, __pyx_n_s_write_record_locals_genexpr, __pyx_n_s_zeroconf__protocol_outgoing); if (unlikely(!gen)) __PYX_ERR(0, 311, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing._write_record.genexpr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_13_write_record_2generator(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_8zeroconf_9_protocol_8outgoing___pyx_scope_struct_1_genexpr *__pyx_cur_scope = ((struct __pyx_obj_8zeroconf_9_protocol_8outgoing___pyx_scope_struct_1_genexpr *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("genexpr", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L6_resume_from_yield;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 311, __pyx_L1_error)
  if (unlikely(!__pyx_cur_scope->__pyx_outer_scope->__pyx_v_self)) { __Pyx_RaiseClosureNameError("self"); __PYX_ERR(0, 311, __pyx_L1_error) }
  if (unlikely(__pyx_cur_scope->__pyx_outer_scope->__pyx_v_self->data == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 311, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyList_GetSlice(__pyx_cur_scope->__pyx_outer_scope->__pyx_v_self->data, (__pyx_cur_scope->__pyx_outer_scope->__pyx_v_index + 1), PY_SSIZE_T_MAX); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 311, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_t_1; __Pyx_INCREF(__pyx_t_2); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  for (;;) {
    if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_2)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_1 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_1); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 311, __pyx_L1_error)
    #else
    __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 311, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    #endif
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_d);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_d, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_1);
    __pyx_t_1 = 0;
    __pyx_t_4 = PyObject_Length(__pyx_cur_scope->__pyx_v_d); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(0, 311, __pyx_L1_error)
    __pyx_t_1 = PyInt_FromSsize_t(__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 311, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    __Pyx_XGIVEREF(__pyx_t_2);
    __pyx_cur_scope->__pyx_t_0 = __pyx_t_2;
    __pyx_cur_scope->__pyx_t_1 = __pyx_t_3;
    __Pyx_XGIVEREF(__pyx_r);
    __Pyx_RefNannyFinishContext();
    __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
    /* return from generator, yielding value */
    __pyx_generator->resume_label = 1;
    return __pyx_r;
    __pyx_L6_resume_from_yield:;
    __pyx_t_2 = __pyx_cur_scope->__pyx_t_0;
    __pyx_cur_scope->__pyx_t_0 = 0;
    __Pyx_XGOTREF(__pyx_t_2);
    __pyx_t_3 = __pyx_cur_scope->__pyx_t_1;
    if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 311, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* function exit code */
  PyErr_SetNone(PyExc_StopIteration);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("genexpr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_r); __pyx_r = 0;
  #if !CYTHON_USE_EXC_INFO_STACK
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  #endif
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/outgoing.py":298
 *         self._write_int(record.ttl if now == 0 else record.get_remaining_ttl(now))
 * 
 *     def _write_record(self, record: DNSRecord, now: float) -> bool:             # <<<<<<<<<<<<<<
 *         """Writes a record (answer, authoritative answer, additional) to
 *         the packet.  Returns True on success, or False if we did not
 */

static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_34_write_record(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_record, double __pyx_v_now) {
  struct __pyx_obj_8zeroconf_9_protocol_8outgoing___pyx_scope_struct___write_record *__pyx_cur_scope;
  Py_ssize_t __pyx_v_start_data_length;
  unsigned int __pyx_v_start_size;
  PyObject *__pyx_v_length = NULL;
  PyObject *__pyx_gb_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_13_write_record_2generator = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  unsigned int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_write_record", 0);
  __pyx_cur_scope = (struct __pyx_obj_8zeroconf_9_protocol_8outgoing___pyx_scope_struct___write_record *)__pyx_tp_new_8zeroconf_9_protocol_8outgoing___pyx_scope_struct___write_record(__pyx_ptype_8zeroconf_9_protocol_8outgoing___pyx_scope_struct___write_record, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_8zeroconf_9_protocol_8outgoing___pyx_scope_struct___write_record *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 298, __pyx_L1_error)
  } else {
    __Pyx_GOTREF(__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_v_self);

  /* "zeroconf/_protocol/outgoing.py":302
 *         the packet.  Returns True on success, or False if we did not
 *         because the packet because the record does not fit."""
 *         start_data_length, start_size = len(self.data), self.size             # <<<<<<<<<<<<<<
 *         self.write_name(record.name)
 *         self.write_short(record.type)
 */
  __pyx_t_1 = __pyx_cur_scope->__pyx_v_self->data;
  __Pyx_INCREF(__pyx_t_1);
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 302, __pyx_L1_error)
  }
  __pyx_t_2 = PyList_GET_SIZE(__pyx_t_1); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1))) __PYX_ERR(0, 302, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = __pyx_cur_scope->__pyx_v_self->size;
  __pyx_v_start_data_length = __pyx_t_2;
  __pyx_v_start_size = __pyx_t_3;

  /* "zeroconf/_protocol/outgoing.py":303
 *         because the packet because the record does not fit."""
 *         start_data_length, start_size = len(self.data), self.size
 *         self.write_name(record.name)             # <<<<<<<<<<<<<<
 *         self.write_short(record.type)
 *         self._write_record_class(record)
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_cur_scope->__pyx_v_self), __pyx_n_s_write_name); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 303, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_record, __pyx_n_s_name); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 303, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  __pyx_t_1 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_6, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 303, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "zeroconf/_protocol/outgoing.py":304
 *         start_data_length, start_size = len(self.data), self.size
 *         self.write_name(record.name)
 *         self.write_short(record.type)             # <<<<<<<<<<<<<<
 *         self._write_record_class(record)
 *         self._write_ttl(record, now)
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_cur_scope->__pyx_v_self), __pyx_n_s_write_short); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 304, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_record, __pyx_n_s_type_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 304, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  __pyx_t_1 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_6, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 304, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "zeroconf/_protocol/outgoing.py":305
 *         self.write_name(record.name)
 *         self.write_short(record.type)
 *         self._write_record_class(record)             # <<<<<<<<<<<<<<
 *         self._write_ttl(record, now)
 *         index = len(self.data)
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_cur_scope->__pyx_v_self->__pyx_vtab)->_write_record_class(__pyx_cur_scope->__pyx_v_self, __pyx_v_record); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 305, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "zeroconf/_protocol/outgoing.py":306
 *         self.write_short(record.type)
 *         self._write_record_class(record)
 *         self._write_ttl(record, now)             # <<<<<<<<<<<<<<
 *         index = len(self.data)
 *         self.write_short(0)  # Will get replaced with the actual size
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_cur_scope->__pyx_v_self), __pyx_n_s_write_ttl); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 306, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyFloat_FromDouble(__pyx_v_now); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 306, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = NULL;
  __pyx_t_7 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
      __pyx_t_7 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_4)) {
    PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_v_record, __pyx_t_5};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 306, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
    PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_v_record, __pyx_t_5};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 306, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else
  #endif
  {
    __pyx_t_8 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 306, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    if (__pyx_t_6) {
      __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6); __pyx_t_6 = NULL;
    }
    __Pyx_INCREF(__pyx_v_record);
    __Pyx_GIVEREF(__pyx_v_record);
    PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_v_record);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_t_5);
    __pyx_t_5 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_8, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 306, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "zeroconf/_protocol/outgoing.py":307
 *         self._write_record_class(record)
 *         self._write_ttl(record, now)
 *         index = len(self.data)             # <<<<<<<<<<<<<<
 *         self.write_short(0)  # Will get replaced with the actual size
 *         record.write(self)
 */
  __pyx_t_1 = __pyx_cur_scope->__pyx_v_self->data;
  __Pyx_INCREF(__pyx_t_1);
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 307, __pyx_L1_error)
  }
  __pyx_t_2 = PyList_GET_SIZE(__pyx_t_1); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1))) __PYX_ERR(0, 307, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_cur_scope->__pyx_v_index = __pyx_t_2;

  /* "zeroconf/_protocol/outgoing.py":308
 *         self._write_ttl(record, now)
 *         index = len(self.data)
 *         self.write_short(0)  # Will get replaced with the actual size             # <<<<<<<<<<<<<<
 *         record.write(self)
 *         # Adjust size for the short we will write before this record
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_cur_scope->__pyx_v_self), __pyx_n_s_write_short); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 308, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_8 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_8)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_8);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  __pyx_t_1 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_8, __pyx_int_0) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_int_0);
  __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 308, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "zeroconf/_protocol/outgoing.py":309
 *         index = len(self.data)
 *         self.write_short(0)  # Will get replaced with the actual size
 *         record.write(self)             # <<<<<<<<<<<<<<
 *         # Adjust size for the short we will write before this record
 *         length = sum(len(d) for d in self.data[index + 1 :])
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_record, __pyx_n_s_write); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 309, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_8 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_8)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_8);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  __pyx_t_1 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_8, ((PyObject *)__pyx_cur_scope->__pyx_v_self)) : __Pyx_PyObject_CallOneArg(__pyx_t_4, ((PyObject *)__pyx_cur_scope->__pyx_v_self));
  __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 309, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "zeroconf/_protocol/outgoing.py":311
 *         record.write(self)
 *         # Adjust size for the short we will write before this record
 *         length = sum(len(d) for d in self.data[index + 1 :])             # <<<<<<<<<<<<<<
 *         # Here we replace the 0 length short we wrote
 *         # before with the actual length
 */
  __pyx_t_1 = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_13_write_record_genexpr(((PyObject*)__pyx_cur_scope)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 311, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_builtin_sum, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 311, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_length = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "zeroconf/_protocol/outgoing.py":314
 *         # Here we replace the 0 length short we wrote
 *         # before with the actual length
 *         self._replace_short(index, length)             # <<<<<<<<<<<<<<
 *         return self._check_data_limit_or_rollback(start_data_length, start_size)
 * 
 */
  __pyx_t_4 = PyInt_FromSsize_t(__pyx_cur_scope->__pyx_v_index); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 314, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = ((struct __pyx_vtabstruct_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_cur_scope->__pyx_v_self->__pyx_vtab)->_replace_short(__pyx_cur_scope->__pyx_v_self, __pyx_t_4, __pyx_v_length); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 314, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "zeroconf/_protocol/outgoing.py":315
 *         # before with the actual length
 *         self._replace_short(index, length)
 *         return self._check_data_limit_or_rollback(start_data_length, start_size)             # <<<<<<<<<<<<<<
 * 
 *     def _check_data_limit_or_rollback(self, start_data_length: int, start_size: int) -> bool:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyInt_FromSsize_t(__pyx_v_start_data_length); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 315, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyInt_From_unsigned_int(__pyx_v_start_size); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 315, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_8 = ((struct __pyx_vtabstruct_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_cur_scope->__pyx_v_self->__pyx_vtab)->_check_data_limit_or_rollback(__pyx_cur_scope->__pyx_v_self, __pyx_t_1, __pyx_t_4); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 315, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_r = __pyx_t_8;
  __pyx_t_8 = 0;
  goto __pyx_L0;

  /* "zeroconf/_protocol/outgoing.py":298
 *         self._write_int(record.ttl if now == 0 else record.get_remaining_ttl(now))
 * 
 *     def _write_record(self, record: DNSRecord, now: float) -> bool:             # <<<<<<<<<<<<<<
 *         """Writes a record (answer, authoritative answer, additional) to
 *         the packet.  Returns True on success, or False if we did not
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing._write_record", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_length);
  __Pyx_XDECREF(__pyx_gb_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_13_write_record_2generator);
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/outgoing.py":317
 *         return self._check_data_limit_or_rollback(start_data_length, start_size)
 * 
 *     def _check_data_limit_or_rollback(self, start_data_length: int, start_size: int) -> bool:             # <<<<<<<<<<<<<<
 *         """Check data limit, if we go over, then rollback and return False."""
 *         len_limit = _MAX_MSG_ABSOLUTE if self.allow_long else _MAX_MSG_TYPICAL
 */

static PyObject *__pyx_f_8zeroconf_9_protocol_8outgoing_11DNSOutgoing__check_data_limit_or_rollback(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_start_data_length, PyObject *__pyx_v_start_size) {
  unsigned int __pyx_v_len_limit;
  PyObject *__pyx_v_rollback_names = NULL;
  PyObject *__pyx_v_name = NULL;
  PyObject *__pyx_8genexpr1__pyx_v_name = NULL;
  PyObject *__pyx_8genexpr1__pyx_v_idx = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  unsigned int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  int __pyx_t_12;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_check_data_limit_or_rollback", 0);

  /* "zeroconf/_protocol/outgoing.py":319
 *     def _check_data_limit_or_rollback(self, start_data_length: int, start_size: int) -> bool:
 *         """Check data limit, if we go over, then rollback and return False."""
 *         len_limit = _MAX_MSG_ABSOLUTE if self.allow_long else _MAX_MSG_TYPICAL             # <<<<<<<<<<<<<<
 *         self.allow_long = False
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_self->allow_long); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 319, __pyx_L1_error)
  if (__pyx_t_2) {
    __pyx_t_1 = __pyx_v_8zeroconf_9_protocol_8outgoing__MAX_MSG_ABSOLUTE;
  } else {
    __pyx_t_1 = __pyx_v_8zeroconf_9_protocol_8outgoing__MAX_MSG_TYPICAL;
  }
  __pyx_v_len_limit = __pyx_t_1;

  /* "zeroconf/_protocol/outgoing.py":320
 *         """Check data limit, if we go over, then rollback and return False."""
 *         len_limit = _MAX_MSG_ABSOLUTE if self.allow_long else _MAX_MSG_TYPICAL
 *         self.allow_long = False             # <<<<<<<<<<<<<<
 * 
 *         if self.size <= len_limit:
 */
  __Pyx_INCREF(Py_False);
  __Pyx_GIVEREF(Py_False);
  __Pyx_GOTREF(__pyx_v_self->allow_long);
  __Pyx_DECREF(__pyx_v_self->allow_long);
  __pyx_v_self->allow_long = Py_False;

  /* "zeroconf/_protocol/outgoing.py":322
 *         self.allow_long = False
 * 
 *         if self.size <= len_limit:             # <<<<<<<<<<<<<<
 *             return True
 * 
 */
  __pyx_t_2 = ((__pyx_v_self->size <= __pyx_v_len_limit) != 0);
  if (__pyx_t_2) {

    /* "zeroconf/_protocol/outgoing.py":323
 * 
 *         if self.size <= len_limit:
 *             return True             # <<<<<<<<<<<<<<
 * 
 *         log.debug("Reached data limit (size=%d) > (limit=%d) - rolling back", self.size, len_limit)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_True);
    __pyx_r = Py_True;
    goto __pyx_L0;

    /* "zeroconf/_protocol/outgoing.py":322
 *         self.allow_long = False
 * 
 *         if self.size <= len_limit:             # <<<<<<<<<<<<<<
 *             return True
 * 
 */
  }

  /* "zeroconf/_protocol/outgoing.py":325
 *             return True
 * 
 *         log.debug("Reached data limit (size=%d) > (limit=%d) - rolling back", self.size, len_limit)             # <<<<<<<<<<<<<<
 *         del self.data[start_data_length:]
 *         self.size = start_size
 */
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_log); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 325, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_debug); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 325, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyInt_From_unsigned_int(__pyx_v_self->size); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 325, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = __Pyx_PyInt_From_unsigned_int(__pyx_v_len_limit); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 325, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = NULL;
  __pyx_t_8 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
      __pyx_t_8 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_5)) {
    PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_kp_u_Reached_data_limit_size_d_limit, __pyx_t_4, __pyx_t_6};
    __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 325, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
    PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_kp_u_Reached_data_limit_size_d_limit, __pyx_t_4, __pyx_t_6};
    __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 325, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else
  #endif
  {
    __pyx_t_9 = PyTuple_New(3+__pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 325, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    if (__pyx_t_7) {
      __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_7); __pyx_t_7 = NULL;
    }
    __Pyx_INCREF(__pyx_kp_u_Reached_data_limit_size_d_limit);
    __Pyx_GIVEREF(__pyx_kp_u_Reached_data_limit_size_d_limit);
    PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_8, __pyx_kp_u_Reached_data_limit_size_d_limit);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_8, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_9, 2+__pyx_t_8, __pyx_t_6);
    __pyx_t_4 = 0;
    __pyx_t_6 = 0;
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_9, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 325, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  }
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "zeroconf/_protocol/outgoing.py":326
 * 
 *         log.debug("Reached data limit (size=%d) > (limit=%d) - rolling back", self.size, len_limit)
 *         del self.data[start_data_length:]             # <<<<<<<<<<<<<<
 *         self.size = start_size
 * 
 */
  if (unlikely(__pyx_v_self->data == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 326, __pyx_L1_error)
  }
  __Pyx_INCREF(__pyx_v_start_data_length);
  __pyx_t_3 = __pyx_v_start_data_length;
  __pyx_t_2 = (__pyx_t_3 == Py_None);
  if (__pyx_t_2) {
    __pyx_t_10 = 0;
  } else {
    __pyx_t_11 = __Pyx_PyIndex_AsSsize_t(__pyx_t_3); if (unlikely((__pyx_t_11 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 326, __pyx_L1_error)
    __pyx_t_10 = __pyx_t_11;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__Pyx_PyObject_DelSlice(__pyx_v_self->data, __pyx_t_10, 0, NULL, NULL, NULL, 1, 0, 1) < 0) __PYX_ERR(0, 326, __pyx_L1_error)

  /* "zeroconf/_protocol/outgoing.py":327
 *         log.debug("Reached data limit (size=%d) > (limit=%d) - rolling back", self.size, len_limit)
 *         del self.data[start_data_length:]
 *         self.size = start_size             # <<<<<<<<<<<<<<
 * 
 *         rollback_names = [name for name, idx in self.names.items() if idx >= start_size]
 */
  __pyx_t_1 = __Pyx_PyInt_As_unsigned_int(__pyx_v_start_size); if (unlikely((__pyx_t_1 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(0, 327, __pyx_L1_error)
  __pyx_v_self->size = __pyx_t_1;

  /* "zeroconf/_protocol/outgoing.py":329
 *         self.size = start_size
 * 
 *         rollback_names = [name for name, idx in self.names.items() if idx >= start_size]             # <<<<<<<<<<<<<<
 *         for name in rollback_names:
 *             del self.names[name]
 */
  { /* enter inner scope */
    __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 329, __pyx_L6_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_10 = 0;
    if (unlikely(__pyx_v_self->names == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "items");
      __PYX_ERR(0, 329, __pyx_L6_error)
    }
    __pyx_t_9 = __Pyx_dict_iterator(__pyx_v_self->names, 0, __pyx_n_s_items, (&__pyx_t_11), (&__pyx_t_8)); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 329, __pyx_L6_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_XDECREF(__pyx_t_5);
    __pyx_t_5 = __pyx_t_9;
    __pyx_t_9 = 0;
    while (1) {
      __pyx_t_12 = __Pyx_dict_iter_next(__pyx_t_5, __pyx_t_11, &__pyx_t_10, &__pyx_t_9, &__pyx_t_6, NULL, __pyx_t_8);
      if (unlikely(__pyx_t_12 == 0)) break;
      if (unlikely(__pyx_t_12 == -1)) __PYX_ERR(0, 329, __pyx_L6_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_XDECREF_SET(__pyx_8genexpr1__pyx_v_name, __pyx_t_9);
      __pyx_t_9 = 0;
      __Pyx_XDECREF_SET(__pyx_8genexpr1__pyx_v_idx, __pyx_t_6);
      __pyx_t_6 = 0;
      __pyx_t_6 = PyObject_RichCompare(__pyx_8genexpr1__pyx_v_idx, __pyx_v_start_size, Py_GE); __Pyx_XGOTREF(__pyx_t_6); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 329, __pyx_L6_error)
      __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 329, __pyx_L6_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (__pyx_t_2) {
        if (unlikely(__Pyx_ListComp_Append(__pyx_t_3, (PyObject*)__pyx_8genexpr1__pyx_v_name))) __PYX_ERR(0, 329, __pyx_L6_error)
      }
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_8genexpr1__pyx_v_idx); __pyx_8genexpr1__pyx_v_idx = 0;
    __Pyx_XDECREF(__pyx_8genexpr1__pyx_v_name); __pyx_8genexpr1__pyx_v_name = 0;
    goto __pyx_L10_exit_scope;
    __pyx_L6_error:;
    __Pyx_XDECREF(__pyx_8genexpr1__pyx_v_idx); __pyx_8genexpr1__pyx_v_idx = 0;
    __Pyx_XDECREF(__pyx_8genexpr1__pyx_v_name); __pyx_8genexpr1__pyx_v_name = 0;
    goto __pyx_L1_error;
    __pyx_L10_exit_scope:;
  } /* exit inner scope */
  __pyx_v_rollback_names = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "zeroconf/_protocol/outgoing.py":330
 * 
 *         rollback_names = [name for name, idx in self.names.items() if idx >= start_size]
 *         for name in rollback_names:             # <<<<<<<<<<<<<<
 *             del self.names[name]
 *         return False
 */
  __pyx_t_3 = __pyx_v_rollback_names; __Pyx_INCREF(__pyx_t_3); __pyx_t_11 = 0;
  for (;;) {
    if (__pyx_t_11 >= PyList_GET_SIZE(__pyx_t_3)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_5 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_11); __Pyx_INCREF(__pyx_t_5); __pyx_t_11++; if (unlikely(0 < 0)) __PYX_ERR(0, 330, __pyx_L1_error)
    #else
    __pyx_t_5 = PySequence_ITEM(__pyx_t_3, __pyx_t_11); __pyx_t_11++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 330, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    #endif
    __Pyx_XDECREF_SET(__pyx_v_name, __pyx_t_5);
    __pyx_t_5 = 0;

    /* "zeroconf/_protocol/outgoing.py":331
 *         rollback_names = [name for name, idx in self.names.items() if idx >= start_size]
 *         for name in rollback_names:
 *             del self.names[name]             # <<<<<<<<<<<<<<
 *         return False
 * 
 */
    if (unlikely(PyObject_DelItem(__pyx_v_self->names, __pyx_v_name) < 0)) __PYX_ERR(0, 331, __pyx_L1_error)

    /* "zeroconf/_protocol/outgoing.py":330
 * 
 *         rollback_names = [name for name, idx in self.names.items() if idx >= start_size]
 *         for name in rollback_names:             # <<<<<<<<<<<<<<
 *             del self.names[name]
 *         return False
 */
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "zeroconf/_protocol/outgoing.py":332
 *         for name in rollback_names:
 *             del self.names[name]
 *         return False             # <<<<<<<<<<<<<<
 * 
 *     def _write_questions_from_offset(self, questions_offset: int) -> int:
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(Py_False);
  __pyx_r = Py_False;
  goto __pyx_L0;

  /* "zeroconf/_protocol/outgoing.py":317
 *         return self._check_data_limit_or_rollback(start_data_length, start_size)
 * 
 *     def _check_data_limit_or_rollback(self, start_data_length: int, start_size: int) -> bool:             # <<<<<<<<<<<<<<
 *         """Check data limit, if we go over, then rollback and return False."""
 *         len_limit = _MAX_MSG_ABSOLUTE if self.allow_long else _MAX_MSG_TYPICAL
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing._check_data_limit_or_rollback", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_rollback_names);
  __Pyx_XDECREF(__pyx_v_name);
  __Pyx_XDECREF(__pyx_8genexpr1__pyx_v_name);
  __Pyx_XDECREF(__pyx_8genexpr1__pyx_v_idx);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/outgoing.py":334
 *         return False
 * 
 *     def _write_questions_from_offset(self, questions_offset: int) -> int:             # <<<<<<<<<<<<<<
 *         questions_written = 0
 *         for question in self.questions[questions_offset:]:
 */

static PyObject *__pyx_f_8zeroconf_9_protocol_8outgoing_11DNSOutgoing__write_questions_from_offset(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_questions_offset) {
  PyObject *__pyx_v_questions_written = NULL;
  PyObject *__pyx_v_question = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_write_questions_from_offset", 0);

  /* "zeroconf/_protocol/outgoing.py":335
 * 
 *     def _write_questions_from_offset(self, questions_offset: int) -> int:
 *         questions_written = 0             # <<<<<<<<<<<<<<
 *         for question in self.questions[questions_offset:]:
 *             if not self._write_question(question):
 */
  __Pyx_INCREF(__pyx_int_0);
  __pyx_v_questions_written = __pyx_int_0;

  /* "zeroconf/_protocol/outgoing.py":336
 *     def _write_questions_from_offset(self, questions_offset: int) -> int:
 *         questions_written = 0
 *         for question in self.questions[questions_offset:]:             # <<<<<<<<<<<<<<
 *             if not self._write_question(question):
 *                 break
 */
  if (unlikely(__pyx_v_self->questions == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 336, __pyx_L1_error)
  }
  __Pyx_INCREF(__pyx_v_questions_offset);
  __pyx_t_1 = __pyx_v_questions_offset;
  __pyx_t_3 = (__pyx_t_1 == Py_None);
  if (__pyx_t_3) {
    __pyx_t_2 = 0;
  } else {
    __pyx_t_4 = __Pyx_PyIndex_AsSsize_t(__pyx_t_1); if (unlikely((__pyx_t_4 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 336, __pyx_L1_error)
    __pyx_t_2 = __pyx_t_4;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyList_GetSlice(__pyx_v_self->questions, __pyx_t_2, PY_SSIZE_T_MAX); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 336, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = __pyx_t_1; __Pyx_INCREF(__pyx_t_5); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  for (;;) {
    if (__pyx_t_2 >= PyList_GET_SIZE(__pyx_t_5)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_1 = PyList_GET_ITEM(__pyx_t_5, __pyx_t_2); __Pyx_INCREF(__pyx_t_1); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 336, __pyx_L1_error)
    #else
    __pyx_t_1 = PySequence_ITEM(__pyx_t_5, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 336, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    #endif
    __Pyx_XDECREF_SET(__pyx_v_question, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "zeroconf/_protocol/outgoing.py":337
 *         questions_written = 0
 *         for question in self.questions[questions_offset:]:
 *             if not self._write_question(question):             # <<<<<<<<<<<<<<
 *                 break
 *             questions_written += 1
 */
    __pyx_t_1 = ((struct __pyx_vtabstruct_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self->__pyx_vtab)->_write_question(__pyx_v_self, __pyx_v_question); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 337, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 337, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_6 = ((!__pyx_t_3) != 0);
    if (__pyx_t_6) {

      /* "zeroconf/_protocol/outgoing.py":338
 *         for question in self.questions[questions_offset:]:
 *             if not self._write_question(question):
 *                 break             # <<<<<<<<<<<<<<
 *             questions_written += 1
 *         return questions_written
 */
      goto __pyx_L4_break;

      /* "zeroconf/_protocol/outgoing.py":337
 *         questions_written = 0
 *         for question in self.questions[questions_offset:]:
 *             if not self._write_question(question):             # <<<<<<<<<<<<<<
 *                 break
 *             questions_written += 1
 */
    }

    /* "zeroconf/_protocol/outgoing.py":339
 *             if not self._write_question(question):
 *                 break
 *             questions_written += 1             # <<<<<<<<<<<<<<
 *         return questions_written
 * 
 */
    __pyx_t_1 = __Pyx_PyInt_AddObjC(__pyx_v_questions_written, __pyx_int_1, 1, 1, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 339, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_questions_written, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "zeroconf/_protocol/outgoing.py":336
 *     def _write_questions_from_offset(self, questions_offset: int) -> int:
 *         questions_written = 0
 *         for question in self.questions[questions_offset:]:             # <<<<<<<<<<<<<<
 *             if not self._write_question(question):
 *                 break
 */
  }
  __pyx_L4_break:;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "zeroconf/_protocol/outgoing.py":340
 *                 break
 *             questions_written += 1
 *         return questions_written             # <<<<<<<<<<<<<<
 * 
 *     def _write_answers_from_offset(self, answer_offset: int) -> int:
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_questions_written);
  __pyx_r = __pyx_v_questions_written;
  goto __pyx_L0;

  /* "zeroconf/_protocol/outgoing.py":334
 *         return False
 * 
 *     def _write_questions_from_offset(self, questions_offset: int) -> int:             # <<<<<<<<<<<<<<
 *         questions_written = 0
 *         for question in self.questions[questions_offset:]:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing._write_questions_from_offset", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_questions_written);
  __Pyx_XDECREF(__pyx_v_question);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/outgoing.py":342
 *         return questions_written
 * 
 *     def _write_answers_from_offset(self, answer_offset: int) -> int:             # <<<<<<<<<<<<<<
 *         answers_written = 0
 *         for answer, time_ in self.answers[answer_offset:]:
 */

static PyObject *__pyx_f_8zeroconf_9_protocol_8outgoing_11DNSOutgoing__write_answers_from_offset(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_answer_offset) {
  PyObject *__pyx_v_answers_written = NULL;
  PyObject *__pyx_v_answer = NULL;
  PyObject *__pyx_v_time_ = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *(*__pyx_t_9)(PyObject *);
  int __pyx_t_10;
  int __pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_write_answers_from_offset", 0);

  /* "zeroconf/_protocol/outgoing.py":343
 * 
 *     def _write_answers_from_offset(self, answer_offset: int) -> int:
 *         answers_written = 0             # <<<<<<<<<<<<<<
 *         for answer, time_ in self.answers[answer_offset:]:
 *             if not self._write_record(answer, time_):
 */
  __Pyx_INCREF(__pyx_int_0);
  __pyx_v_answers_written = __pyx_int_0;

  /* "zeroconf/_protocol/outgoing.py":344
 *     def _write_answers_from_offset(self, answer_offset: int) -> int:
 *         answers_written = 0
 *         for answer, time_ in self.answers[answer_offset:]:             # <<<<<<<<<<<<<<
 *             if not self._write_record(answer, time_):
 *                 break
 */
  if (unlikely(__pyx_v_self->answers == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 344, __pyx_L1_error)
  }
  __Pyx_INCREF(__pyx_v_answer_offset);
  __pyx_t_1 = __pyx_v_answer_offset;
  __pyx_t_3 = (__pyx_t_1 == Py_None);
  if (__pyx_t_3) {
    __pyx_t_2 = 0;
  } else {
    __pyx_t_4 = __Pyx_PyIndex_AsSsize_t(__pyx_t_1); if (unlikely((__pyx_t_4 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 344, __pyx_L1_error)
    __pyx_t_2 = __pyx_t_4;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyList_GetSlice(__pyx_v_self->answers, __pyx_t_2, PY_SSIZE_T_MAX); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 344, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = __pyx_t_1; __Pyx_INCREF(__pyx_t_5); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  for (;;) {
    if (__pyx_t_2 >= PyList_GET_SIZE(__pyx_t_5)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_1 = PyList_GET_ITEM(__pyx_t_5, __pyx_t_2); __Pyx_INCREF(__pyx_t_1); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 344, __pyx_L1_error)
    #else
    __pyx_t_1 = PySequence_ITEM(__pyx_t_5, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 344, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    #endif
    if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
      PyObject* sequence = __pyx_t_1;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 344, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_6 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_7 = PyTuple_GET_ITEM(sequence, 1); 
      } else {
        __pyx_t_6 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_7 = PyList_GET_ITEM(sequence, 1); 
      }
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_7);
      #else
      __pyx_t_6 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 344, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_7 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 344, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_8 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 344, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_9 = Py_TYPE(__pyx_t_8)->tp_iternext;
      index = 0; __pyx_t_6 = __pyx_t_9(__pyx_t_8); if (unlikely(!__pyx_t_6)) goto __pyx_L5_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_6);
      index = 1; __pyx_t_7 = __pyx_t_9(__pyx_t_8); if (unlikely(!__pyx_t_7)) goto __pyx_L5_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_7);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_9(__pyx_t_8), 2) < 0) __PYX_ERR(0, 344, __pyx_L1_error)
      __pyx_t_9 = NULL;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      goto __pyx_L6_unpacking_done;
      __pyx_L5_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_9 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 344, __pyx_L1_error)
      __pyx_L6_unpacking_done:;
    }
    __Pyx_XDECREF_SET(__pyx_v_answer, __pyx_t_6);
    __pyx_t_6 = 0;
    __Pyx_XDECREF_SET(__pyx_v_time_, __pyx_t_7);
    __pyx_t_7 = 0;

    /* "zeroconf/_protocol/outgoing.py":345
 *         answers_written = 0
 *         for answer, time_ in self.answers[answer_offset:]:
 *             if not self._write_record(answer, time_):             # <<<<<<<<<<<<<<
 *                 break
 *             answers_written += 1
 */
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_write_record); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 345, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_6 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_7)) {
      PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_v_answer, __pyx_v_time_};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 345, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
      PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_v_answer, __pyx_v_time_};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 345, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_8 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 345, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      if (__pyx_t_6) {
        __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6); __pyx_t_6 = NULL;
      }
      __Pyx_INCREF(__pyx_v_answer);
      __Pyx_GIVEREF(__pyx_v_answer);
      PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_10, __pyx_v_answer);
      __Pyx_INCREF(__pyx_v_time_);
      __Pyx_GIVEREF(__pyx_v_time_);
      PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_10, __pyx_v_time_);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_8, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 345, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    }
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 345, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_11 = ((!__pyx_t_3) != 0);
    if (__pyx_t_11) {

      /* "zeroconf/_protocol/outgoing.py":346
 *         for answer, time_ in self.answers[answer_offset:]:
 *             if not self._write_record(answer, time_):
 *                 break             # <<<<<<<<<<<<<<
 *             answers_written += 1
 *         return answers_written
 */
      goto __pyx_L4_break;

      /* "zeroconf/_protocol/outgoing.py":345
 *         answers_written = 0
 *         for answer, time_ in self.answers[answer_offset:]:
 *             if not self._write_record(answer, time_):             # <<<<<<<<<<<<<<
 *                 break
 *             answers_written += 1
 */
    }

    /* "zeroconf/_protocol/outgoing.py":347
 *             if not self._write_record(answer, time_):
 *                 break
 *             answers_written += 1             # <<<<<<<<<<<<<<
 *         return answers_written
 * 
 */
    __pyx_t_1 = __Pyx_PyInt_AddObjC(__pyx_v_answers_written, __pyx_int_1, 1, 1, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 347, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_answers_written, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "zeroconf/_protocol/outgoing.py":344
 *     def _write_answers_from_offset(self, answer_offset: int) -> int:
 *         answers_written = 0
 *         for answer, time_ in self.answers[answer_offset:]:             # <<<<<<<<<<<<<<
 *             if not self._write_record(answer, time_):
 *                 break
 */
  }
  __pyx_L4_break:;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "zeroconf/_protocol/outgoing.py":348
 *                 break
 *             answers_written += 1
 *         return answers_written             # <<<<<<<<<<<<<<
 * 
 *     def _write_records_from_offset(self, records: Sequence[DNSRecord], offset: int) -> int:
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_answers_written);
  __pyx_r = __pyx_v_answers_written;
  goto __pyx_L0;

  /* "zeroconf/_protocol/outgoing.py":342
 *         return questions_written
 * 
 *     def _write_answers_from_offset(self, answer_offset: int) -> int:             # <<<<<<<<<<<<<<
 *         answers_written = 0
 *         for answer, time_ in self.answers[answer_offset:]:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing._write_answers_from_offset", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_answers_written);
  __Pyx_XDECREF(__pyx_v_answer);
  __Pyx_XDECREF(__pyx_v_time_);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/outgoing.py":350
 *         return answers_written
 * 
 *     def _write_records_from_offset(self, records: Sequence[DNSRecord], offset: int) -> int:             # <<<<<<<<<<<<<<
 *         records_written = 0
 *         for record in records[offset:]:
 */

static PyObject *__pyx_f_8zeroconf_9_protocol_8outgoing_11DNSOutgoing__write_records_from_offset(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_records, PyObject *__pyx_v_offset) {
  PyObject *__pyx_v_records_written = NULL;
  PyObject *__pyx_v_record = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  PyObject *(*__pyx_t_4)(PyObject *);
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  int __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_write_records_from_offset", 0);

  /* "zeroconf/_protocol/outgoing.py":351
 * 
 *     def _write_records_from_offset(self, records: Sequence[DNSRecord], offset: int) -> int:
 *         records_written = 0             # <<<<<<<<<<<<<<
 *         for record in records[offset:]:
 *             if not self._write_record(record, 0):
 */
  __Pyx_INCREF(__pyx_int_0);
  __pyx_v_records_written = __pyx_int_0;

  /* "zeroconf/_protocol/outgoing.py":352
 *     def _write_records_from_offset(self, records: Sequence[DNSRecord], offset: int) -> int:
 *         records_written = 0
 *         for record in records[offset:]:             # <<<<<<<<<<<<<<
 *             if not self._write_record(record, 0):
 *                 break
 */
  __pyx_t_1 = __Pyx_PyObject_GetSlice(__pyx_v_records, 0, 0, &__pyx_v_offset, NULL, NULL, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 352, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
    __pyx_t_2 = __pyx_t_1; __Pyx_INCREF(__pyx_t_2); __pyx_t_3 = 0;
    __pyx_t_4 = NULL;
  } else {
    __pyx_t_3 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 352, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 352, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  for (;;) {
    if (likely(!__pyx_t_4)) {
      if (likely(PyList_CheckExact(__pyx_t_2))) {
        if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_1); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 352, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 352, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      } else {
        if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_1); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 352, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 352, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      }
    } else {
      __pyx_t_1 = __pyx_t_4(__pyx_t_2);
      if (unlikely(!__pyx_t_1)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 352, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_XDECREF_SET(__pyx_v_record, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "zeroconf/_protocol/outgoing.py":353
 *         records_written = 0
 *         for record in records[offset:]:
 *             if not self._write_record(record, 0):             # <<<<<<<<<<<<<<
 *                 break
 *             records_written += 1
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_write_record); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 353, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = NULL;
    __pyx_t_7 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
        __pyx_t_7 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_v_record, __pyx_int_0};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 353, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_v_record, __pyx_int_0};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 353, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_8 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 353, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      if (__pyx_t_6) {
        __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6); __pyx_t_6 = NULL;
      }
      __Pyx_INCREF(__pyx_v_record);
      __Pyx_GIVEREF(__pyx_v_record);
      PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_v_record);
      __Pyx_INCREF(__pyx_int_0);
      __Pyx_GIVEREF(__pyx_int_0);
      PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_int_0);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_8, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 353, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 353, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_10 = ((!__pyx_t_9) != 0);
    if (__pyx_t_10) {

      /* "zeroconf/_protocol/outgoing.py":354
 *         for record in records[offset:]:
 *             if not self._write_record(record, 0):
 *                 break             # <<<<<<<<<<<<<<
 *             records_written += 1
 *         return records_written
 */
      goto __pyx_L4_break;

      /* "zeroconf/_protocol/outgoing.py":353
 *         records_written = 0
 *         for record in records[offset:]:
 *             if not self._write_record(record, 0):             # <<<<<<<<<<<<<<
 *                 break
 *             records_written += 1
 */
    }

    /* "zeroconf/_protocol/outgoing.py":355
 *             if not self._write_record(record, 0):
 *                 break
 *             records_written += 1             # <<<<<<<<<<<<<<
 *         return records_written
 * 
 */
    __pyx_t_1 = __Pyx_PyInt_AddObjC(__pyx_v_records_written, __pyx_int_1, 1, 1, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 355, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_records_written, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "zeroconf/_protocol/outgoing.py":352
 *     def _write_records_from_offset(self, records: Sequence[DNSRecord], offset: int) -> int:
 *         records_written = 0
 *         for record in records[offset:]:             # <<<<<<<<<<<<<<
 *             if not self._write_record(record, 0):
 *                 break
 */
  }
  __pyx_L4_break:;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "zeroconf/_protocol/outgoing.py":356
 *                 break
 *             records_written += 1
 *         return records_written             # <<<<<<<<<<<<<<
 * 
 *     def _has_more_to_add(
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_records_written);
  __pyx_r = __pyx_v_records_written;
  goto __pyx_L0;

  /* "zeroconf/_protocol/outgoing.py":350
 *         return answers_written
 * 
 *     def _write_records_from_offset(self, records: Sequence[DNSRecord], offset: int) -> int:             # <<<<<<<<<<<<<<
 *         records_written = 0
 *         for record in records[offset:]:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing._write_records_from_offset", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_records_written);
  __Pyx_XDECREF(__pyx_v_record);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/outgoing.py":358
 *         return records_written
 * 
 *     def _has_more_to_add(             # <<<<<<<<<<<<<<
 *         self, questions_offset: int, answer_offset: int, authority_offset: int, additional_offset: int
 *     ) -> bool:
 */

static PyObject *__pyx_f_8zeroconf_9_protocol_8outgoing_11DNSOutgoing__has_more_to_add(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_questions_offset, PyObject *__pyx_v_answer_offset, PyObject *__pyx_v_authority_offset, PyObject *__pyx_v_additional_offset) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_has_more_to_add", 0);

  /* "zeroconf/_protocol/outgoing.py":362
 *     ) -> bool:
 *         """Check if all questions, answers, authority, and additionals have been written to the packet."""
 *         return (             # <<<<<<<<<<<<<<
 *             questions_offset < len(self.questions)
 *             or answer_offset < len(self.answers)
 */
  __Pyx_XDECREF(__pyx_r);

  /* "zeroconf/_protocol/outgoing.py":363
 *         """Check if all questions, answers, authority, and additionals have been written to the packet."""
 *         return (
 *             questions_offset < len(self.questions)             # <<<<<<<<<<<<<<
 *             or answer_offset < len(self.answers)
 *             or authority_offset < len(self.authorities)
 */
  __pyx_t_2 = __pyx_v_self->questions;
  __Pyx_INCREF(__pyx_t_2);
  if (unlikely(__pyx_t_2 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 363, __pyx_L1_error)
  }
  __pyx_t_3 = PyList_GET_SIZE(__pyx_t_2); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(0, 363, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyInt_FromSsize_t(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 363, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = PyObject_RichCompare(__pyx_v_questions_offset, __pyx_t_2, Py_LT); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 363, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 363, __pyx_L1_error)
  if (!__pyx_t_5) {
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else {
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_1 = __pyx_t_4;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    goto __pyx_L3_bool_binop_done;
  }

  /* "zeroconf/_protocol/outgoing.py":364
 *         return (
 *             questions_offset < len(self.questions)
 *             or answer_offset < len(self.answers)             # <<<<<<<<<<<<<<
 *             or authority_offset < len(self.authorities)
 *             or additional_offset < len(self.additionals)
 */
  __pyx_t_4 = __pyx_v_self->answers;
  __Pyx_INCREF(__pyx_t_4);
  if (unlikely(__pyx_t_4 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 364, __pyx_L1_error)
  }
  __pyx_t_3 = PyList_GET_SIZE(__pyx_t_4); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(0, 364, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyInt_FromSsize_t(__pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 364, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = PyObject_RichCompare(__pyx_v_answer_offset, __pyx_t_4, Py_LT); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 364, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 364, __pyx_L1_error)
  if (!__pyx_t_5) {
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  } else {
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_1 = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    goto __pyx_L3_bool_binop_done;
  }

  /* "zeroconf/_protocol/outgoing.py":365
 *             questions_offset < len(self.questions)
 *             or answer_offset < len(self.answers)
 *             or authority_offset < len(self.authorities)             # <<<<<<<<<<<<<<
 *             or additional_offset < len(self.additionals)
 *         )
 */
  __pyx_t_2 = __pyx_v_self->authorities;
  __Pyx_INCREF(__pyx_t_2);
  if (unlikely(__pyx_t_2 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 365, __pyx_L1_error)
  }
  __pyx_t_3 = PyList_GET_SIZE(__pyx_t_2); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(0, 365, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyInt_FromSsize_t(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 365, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = PyObject_RichCompare(__pyx_v_authority_offset, __pyx_t_2, Py_LT); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 365, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 365, __pyx_L1_error)
  if (!__pyx_t_5) {
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else {
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_1 = __pyx_t_4;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    goto __pyx_L3_bool_binop_done;
  }

  /* "zeroconf/_protocol/outgoing.py":366
 *             or answer_offset < len(self.answers)
 *             or authority_offset < len(self.authorities)
 *             or additional_offset < len(self.additionals)             # <<<<<<<<<<<<<<
 *         )
 * 
 */
  __pyx_t_4 = __pyx_v_self->additionals;
  __Pyx_INCREF(__pyx_t_4);
  if (unlikely(__pyx_t_4 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 366, __pyx_L1_error)
  }
  __pyx_t_3 = PyList_GET_SIZE(__pyx_t_4); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(0, 366, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyInt_FromSsize_t(__pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 366, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = PyObject_RichCompare(__pyx_v_additional_offset, __pyx_t_4, Py_LT); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 366, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_1 = __pyx_t_2;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_L3_bool_binop_done:;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "zeroconf/_protocol/outgoing.py":358
 *         return records_written
 * 
 *     def _has_more_to_add(             # <<<<<<<<<<<<<<
 *         self, questions_offset: int, answer_offset: int, authority_offset: int, additional_offset: int
 *     ) -> bool:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing._has_more_to_add", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/outgoing.py":369
 *         )
 * 
 *     def packets(self) -> List[bytes]:             # <<<<<<<<<<<<<<
 *         """Returns a list of bytestrings containing the packets' bytes
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_37packets(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_36packets[] = "Returns a list of bytestrings containing the packets' bytes\n\n        No further parts should be added to the packet once this\n        is done.  The packets are each restricted to _MAX_MSG_TYPICAL\n        or less in length, except for the case of a single answer which\n        will be written out to a single oversized packet no more than\n        _MAX_MSG_ABSOLUTE in length (and hence will be subject to IP\n        fragmentation potentially).";
static PyMethodDef __pyx_mdef_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_37packets = {"packets", (PyCFunction)__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_37packets, METH_NOARGS, __pyx_doc_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_36packets};
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_37packets(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("packets (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_36packets(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_36packets(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("packets", 0);

  /* "zeroconf/_protocol/outgoing.py":378
 *         _MAX_MSG_ABSOLUTE in length (and hence will be subject to IP
 *         fragmentation potentially)."""
 *         return self._packets()             # <<<<<<<<<<<<<<
 * 
 *     def _packets(self) -> List[bytes]:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self->__pyx_vtab)->_packets(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 378, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "zeroconf/_protocol/outgoing.py":369
 *         )
 * 
 *     def packets(self) -> List[bytes]:             # <<<<<<<<<<<<<<
 *         """Returns a list of bytestrings containing the packets' bytes
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing.packets", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/outgoing.py":380
 *         return self._packets()
 * 
 *     def _packets(self) -> List[bytes]:             # <<<<<<<<<<<<<<
 *         if self.state == State.finished:
 *             return self.packets_data
 */

static PyObject *__pyx_f_8zeroconf_9_protocol_8outgoing_11DNSOutgoing__packets(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self) {
  unsigned int __pyx_v_questions_offset;
  unsigned int __pyx_v_answer_offset;
  unsigned int __pyx_v_authority_offset;
  unsigned int __pyx_v_additional_offset;
  unsigned int __pyx_v_questions_written;
  unsigned int __pyx_v_answers_written;
  unsigned int __pyx_v_authorities_written;
  unsigned int __pyx_v_additionals_written;
  int __pyx_v_first_time;
  PyObject *__pyx_v_debug_enable = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  Py_ssize_t __pyx_t_12;
  unsigned int __pyx_t_13;
  int __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_packets", 0);

  /* "zeroconf/_protocol/outgoing.py":381
 * 
 *     def _packets(self) -> List[bytes]:
 *         if self.state == State.finished:             # <<<<<<<<<<<<<<
 *             return self.packets_data
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_State); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 381, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_finished); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 381, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyObject_RichCompare(__pyx_v_self->state, __pyx_t_2, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 381, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 381, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_3) {

    /* "zeroconf/_protocol/outgoing.py":382
 *     def _packets(self) -> List[bytes]:
 *         if self.state == State.finished:
 *             return self.packets_data             # <<<<<<<<<<<<<<
 * 
 *         questions_offset = 0
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_self->packets_data);
    __pyx_r = __pyx_v_self->packets_data;
    goto __pyx_L0;

    /* "zeroconf/_protocol/outgoing.py":381
 * 
 *     def _packets(self) -> List[bytes]:
 *         if self.state == State.finished:             # <<<<<<<<<<<<<<
 *             return self.packets_data
 * 
 */
  }

  /* "zeroconf/_protocol/outgoing.py":384
 *             return self.packets_data
 * 
 *         questions_offset = 0             # <<<<<<<<<<<<<<
 *         answer_offset = 0
 *         authority_offset = 0
 */
  __pyx_v_questions_offset = 0;

  /* "zeroconf/_protocol/outgoing.py":385
 * 
 *         questions_offset = 0
 *         answer_offset = 0             # <<<<<<<<<<<<<<
 *         authority_offset = 0
 *         additional_offset = 0
 */
  __pyx_v_answer_offset = 0;

  /* "zeroconf/_protocol/outgoing.py":386
 *         questions_offset = 0
 *         answer_offset = 0
 *         authority_offset = 0             # <<<<<<<<<<<<<<
 *         additional_offset = 0
 *         # we have to at least write out the question
 */
  __pyx_v_authority_offset = 0;

  /* "zeroconf/_protocol/outgoing.py":387
 *         answer_offset = 0
 *         authority_offset = 0
 *         additional_offset = 0             # <<<<<<<<<<<<<<
 *         # we have to at least write out the question
 *         first_time = True
 */
  __pyx_v_additional_offset = 0;

  /* "zeroconf/_protocol/outgoing.py":389
 *         additional_offset = 0
 *         # we have to at least write out the question
 *         first_time = True             # <<<<<<<<<<<<<<
 *         debug_enable = log.isEnabledFor(logging.DEBUG)
 * 
 */
  __pyx_v_first_time = 1;

  /* "zeroconf/_protocol/outgoing.py":390
 *         # we have to at least write out the question
 *         first_time = True
 *         debug_enable = log.isEnabledFor(logging.DEBUG)             # <<<<<<<<<<<<<<
 * 
 *         while first_time or self._has_more_to_add(
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_log); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 390, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_isEnabledFor); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 390, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_logging); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 390, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_DEBUG); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 390, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_2, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 390, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_debug_enable = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "zeroconf/_protocol/outgoing.py":392
 *         debug_enable = log.isEnabledFor(logging.DEBUG)
 * 
 *         while first_time or self._has_more_to_add(             # <<<<<<<<<<<<<<
 *             questions_offset, answer_offset, authority_offset, additional_offset
 *         ):
 */
  while (1) {
    __pyx_t_6 = (__pyx_v_first_time != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_3 = __pyx_t_6;
      goto __pyx_L6_bool_binop_done;
    }

    /* "zeroconf/_protocol/outgoing.py":393
 * 
 *         while first_time or self._has_more_to_add(
 *             questions_offset, answer_offset, authority_offset, additional_offset             # <<<<<<<<<<<<<<
 *         ):
 *             first_time = False
 */
    __pyx_t_1 = __Pyx_PyInt_From_unsigned_int(__pyx_v_questions_offset); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 393, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = __Pyx_PyInt_From_unsigned_int(__pyx_v_answer_offset); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 393, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyInt_From_unsigned_int(__pyx_v_authority_offset); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 393, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_2 = __Pyx_PyInt_From_unsigned_int(__pyx_v_additional_offset); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 393, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);

    /* "zeroconf/_protocol/outgoing.py":392
 *         debug_enable = log.isEnabledFor(logging.DEBUG)
 * 
 *         while first_time or self._has_more_to_add(             # <<<<<<<<<<<<<<
 *             questions_offset, answer_offset, authority_offset, additional_offset
 *         ):
 */
    __pyx_t_7 = ((struct __pyx_vtabstruct_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self->__pyx_vtab)->_has_more_to_add(__pyx_v_self, __pyx_t_1, __pyx_t_4, __pyx_t_5, __pyx_t_2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 392, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_7); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 392, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_3 = __pyx_t_6;
    __pyx_L6_bool_binop_done:;
    if (!__pyx_t_3) break;

    /* "zeroconf/_protocol/outgoing.py":395
 *             questions_offset, answer_offset, authority_offset, additional_offset
 *         ):
 *             first_time = False             # <<<<<<<<<<<<<<
 *             if debug_enable:
 *                 log.debug(
 */
    __pyx_v_first_time = 0;

    /* "zeroconf/_protocol/outgoing.py":396
 *         ):
 *             first_time = False
 *             if debug_enable:             # <<<<<<<<<<<<<<
 *                 log.debug(
 *                     "offsets = questions=%d, answers=%d, authorities=%d, additionals=%d",
 */
    __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_v_debug_enable); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 396, __pyx_L1_error)
    if (__pyx_t_3) {

      /* "zeroconf/_protocol/outgoing.py":397
 *             first_time = False
 *             if debug_enable:
 *                 log.debug(             # <<<<<<<<<<<<<<
 *                     "offsets = questions=%d, answers=%d, authorities=%d, additionals=%d",
 *                     questions_offset,
 */
      __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_log); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 397, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_debug); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 397, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "zeroconf/_protocol/outgoing.py":399
 *                 log.debug(
 *                     "offsets = questions=%d, answers=%d, authorities=%d, additionals=%d",
 *                     questions_offset,             # <<<<<<<<<<<<<<
 *                     answer_offset,
 *                     authority_offset,
 */
      __pyx_t_2 = __Pyx_PyInt_From_unsigned_int(__pyx_v_questions_offset); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 399, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);

      /* "zeroconf/_protocol/outgoing.py":400
 *                     "offsets = questions=%d, answers=%d, authorities=%d, additionals=%d",
 *                     questions_offset,
 *                     answer_offset,             # <<<<<<<<<<<<<<
 *                     authority_offset,
 *                     additional_offset,
 */
      __pyx_t_4 = __Pyx_PyInt_From_unsigned_int(__pyx_v_answer_offset); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 400, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);

      /* "zeroconf/_protocol/outgoing.py":401
 *                     questions_offset,
 *                     answer_offset,
 *                     authority_offset,             # <<<<<<<<<<<<<<
 *                     additional_offset,
 *                 )
 */
      __pyx_t_1 = __Pyx_PyInt_From_unsigned_int(__pyx_v_authority_offset); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 401, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);

      /* "zeroconf/_protocol/outgoing.py":402
 *                     answer_offset,
 *                     authority_offset,
 *                     additional_offset,             # <<<<<<<<<<<<<<
 *                 )
 *                 log.debug(
 */
      __pyx_t_8 = __Pyx_PyInt_From_unsigned_int(__pyx_v_additional_offset); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 402, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_9 = NULL;
      __pyx_t_10 = 0;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_9)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_9);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
          __pyx_t_10 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_5)) {
        PyObject *__pyx_temp[6] = {__pyx_t_9, __pyx_kp_u_offsets_questions_d_answers_d_au, __pyx_t_2, __pyx_t_4, __pyx_t_1, __pyx_t_8};
        __pyx_t_7 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_10, 5+__pyx_t_10); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 397, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
        PyObject *__pyx_temp[6] = {__pyx_t_9, __pyx_kp_u_offsets_questions_d_answers_d_au, __pyx_t_2, __pyx_t_4, __pyx_t_1, __pyx_t_8};
        __pyx_t_7 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_10, 5+__pyx_t_10); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 397, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      } else
      #endif
      {
        __pyx_t_11 = PyTuple_New(5+__pyx_t_10); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 397, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        if (__pyx_t_9) {
          __Pyx_GIVEREF(__pyx_t_9); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_9); __pyx_t_9 = NULL;
        }
        __Pyx_INCREF(__pyx_kp_u_offsets_questions_d_answers_d_au);
        __Pyx_GIVEREF(__pyx_kp_u_offsets_questions_d_answers_d_au);
        PyTuple_SET_ITEM(__pyx_t_11, 0+__pyx_t_10, __pyx_kp_u_offsets_questions_d_answers_d_au);
        __Pyx_GIVEREF(__pyx_t_2);
        PyTuple_SET_ITEM(__pyx_t_11, 1+__pyx_t_10, __pyx_t_2);
        __Pyx_GIVEREF(__pyx_t_4);
        PyTuple_SET_ITEM(__pyx_t_11, 2+__pyx_t_10, __pyx_t_4);
        __Pyx_GIVEREF(__pyx_t_1);
        PyTuple_SET_ITEM(__pyx_t_11, 3+__pyx_t_10, __pyx_t_1);
        __Pyx_GIVEREF(__pyx_t_8);
        PyTuple_SET_ITEM(__pyx_t_11, 4+__pyx_t_10, __pyx_t_8);
        __pyx_t_2 = 0;
        __pyx_t_4 = 0;
        __pyx_t_1 = 0;
        __pyx_t_8 = 0;
        __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_11, NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 397, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      }
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

      /* "zeroconf/_protocol/outgoing.py":404
 *                     additional_offset,
 *                 )
 *                 log.debug(             # <<<<<<<<<<<<<<
 *                     "lengths = questions=%d, answers=%d, authorities=%d, additionals=%d",
 *                     len(self.questions),
 */
      __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_log); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 404, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_debug); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 404, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

      /* "zeroconf/_protocol/outgoing.py":406
 *                 log.debug(
 *                     "lengths = questions=%d, answers=%d, authorities=%d, additionals=%d",
 *                     len(self.questions),             # <<<<<<<<<<<<<<
 *                     len(self.answers),
 *                     len(self.authorities),
 */
      __pyx_t_5 = __pyx_v_self->questions;
      __Pyx_INCREF(__pyx_t_5);
      if (unlikely(__pyx_t_5 == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
        __PYX_ERR(0, 406, __pyx_L1_error)
      }
      __pyx_t_12 = PyList_GET_SIZE(__pyx_t_5); if (unlikely(__pyx_t_12 == ((Py_ssize_t)-1))) __PYX_ERR(0, 406, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = PyInt_FromSsize_t(__pyx_t_12); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 406, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);

      /* "zeroconf/_protocol/outgoing.py":407
 *                     "lengths = questions=%d, answers=%d, authorities=%d, additionals=%d",
 *                     len(self.questions),
 *                     len(self.answers),             # <<<<<<<<<<<<<<
 *                     len(self.authorities),
 *                     len(self.additionals),
 */
      __pyx_t_8 = __pyx_v_self->answers;
      __Pyx_INCREF(__pyx_t_8);
      if (unlikely(__pyx_t_8 == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
        __PYX_ERR(0, 407, __pyx_L1_error)
      }
      __pyx_t_12 = PyList_GET_SIZE(__pyx_t_8); if (unlikely(__pyx_t_12 == ((Py_ssize_t)-1))) __PYX_ERR(0, 407, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_8 = PyInt_FromSsize_t(__pyx_t_12); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 407, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);

      /* "zeroconf/_protocol/outgoing.py":408
 *                     len(self.questions),
 *                     len(self.answers),
 *                     len(self.authorities),             # <<<<<<<<<<<<<<
 *                     len(self.additionals),
 *                 )
 */
      __pyx_t_1 = __pyx_v_self->authorities;
      __Pyx_INCREF(__pyx_t_1);
      if (unlikely(__pyx_t_1 == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
        __PYX_ERR(0, 408, __pyx_L1_error)
      }
      __pyx_t_12 = PyList_GET_SIZE(__pyx_t_1); if (unlikely(__pyx_t_12 == ((Py_ssize_t)-1))) __PYX_ERR(0, 408, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = PyInt_FromSsize_t(__pyx_t_12); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 408, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);

      /* "zeroconf/_protocol/outgoing.py":409
 *                     len(self.answers),
 *                     len(self.authorities),
 *                     len(self.additionals),             # <<<<<<<<<<<<<<
 *                 )
 * 
 */
      __pyx_t_4 = __pyx_v_self->additionals;
      __Pyx_INCREF(__pyx_t_4);
      if (unlikely(__pyx_t_4 == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
        __PYX_ERR(0, 409, __pyx_L1_error)
      }
      __pyx_t_12 = PyList_GET_SIZE(__pyx_t_4); if (unlikely(__pyx_t_12 == ((Py_ssize_t)-1))) __PYX_ERR(0, 409, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = PyInt_FromSsize_t(__pyx_t_12); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 409, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_2 = NULL;
      __pyx_t_10 = 0;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_11))) {
        __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_11);
        if (likely(__pyx_t_2)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_11);
          __Pyx_INCREF(__pyx_t_2);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_11, function);
          __pyx_t_10 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_11)) {
        PyObject *__pyx_temp[6] = {__pyx_t_2, __pyx_kp_u_lengths_questions_d_answers_d_au, __pyx_t_5, __pyx_t_8, __pyx_t_1, __pyx_t_4};
        __pyx_t_7 = __Pyx_PyFunction_FastCall(__pyx_t_11, __pyx_temp+1-__pyx_t_10, 5+__pyx_t_10); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 404, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_11)) {
        PyObject *__pyx_temp[6] = {__pyx_t_2, __pyx_kp_u_lengths_questions_d_answers_d_au, __pyx_t_5, __pyx_t_8, __pyx_t_1, __pyx_t_4};
        __pyx_t_7 = __Pyx_PyCFunction_FastCall(__pyx_t_11, __pyx_temp+1-__pyx_t_10, 5+__pyx_t_10); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 404, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else
      #endif
      {
        __pyx_t_9 = PyTuple_New(5+__pyx_t_10); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 404, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        if (__pyx_t_2) {
          __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_2); __pyx_t_2 = NULL;
        }
        __Pyx_INCREF(__pyx_kp_u_lengths_questions_d_answers_d_au);
        __Pyx_GIVEREF(__pyx_kp_u_lengths_questions_d_answers_d_au);
        PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_10, __pyx_kp_u_lengths_questions_d_answers_d_au);
        __Pyx_GIVEREF(__pyx_t_5);
        PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_10, __pyx_t_5);
        __Pyx_GIVEREF(__pyx_t_8);
        PyTuple_SET_ITEM(__pyx_t_9, 2+__pyx_t_10, __pyx_t_8);
        __Pyx_GIVEREF(__pyx_t_1);
        PyTuple_SET_ITEM(__pyx_t_9, 3+__pyx_t_10, __pyx_t_1);
        __Pyx_GIVEREF(__pyx_t_4);
        PyTuple_SET_ITEM(__pyx_t_9, 4+__pyx_t_10, __pyx_t_4);
        __pyx_t_5 = 0;
        __pyx_t_8 = 0;
        __pyx_t_1 = 0;
        __pyx_t_4 = 0;
        __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_11, __pyx_t_9, NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 404, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      }
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

      /* "zeroconf/_protocol/outgoing.py":396
 *         ):
 *             first_time = False
 *             if debug_enable:             # <<<<<<<<<<<<<<
 *                 log.debug(
 *                     "offsets = questions=%d, answers=%d, authorities=%d, additionals=%d",
 */
    }

    /* "zeroconf/_protocol/outgoing.py":412
 *                 )
 * 
 *             questions_written = self._write_questions_from_offset(questions_offset)             # <<<<<<<<<<<<<<
 *             answers_written = self._write_answers_from_offset(answer_offset)
 *             authorities_written = self._write_records_from_offset(self.authorities, authority_offset)
 */
    __pyx_t_7 = __Pyx_PyInt_From_unsigned_int(__pyx_v_questions_offset); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 412, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_11 = ((struct __pyx_vtabstruct_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self->__pyx_vtab)->_write_questions_from_offset(__pyx_v_self, __pyx_t_7); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 412, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_13 = __Pyx_PyInt_As_unsigned_int(__pyx_t_11); if (unlikely((__pyx_t_13 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(0, 412, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __pyx_v_questions_written = __pyx_t_13;

    /* "zeroconf/_protocol/outgoing.py":413
 * 
 *             questions_written = self._write_questions_from_offset(questions_offset)
 *             answers_written = self._write_answers_from_offset(answer_offset)             # <<<<<<<<<<<<<<
 *             authorities_written = self._write_records_from_offset(self.authorities, authority_offset)
 *             additionals_written = self._write_records_from_offset(self.additionals, additional_offset)
 */
    __pyx_t_11 = __Pyx_PyInt_From_unsigned_int(__pyx_v_answer_offset); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 413, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __pyx_t_7 = ((struct __pyx_vtabstruct_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self->__pyx_vtab)->_write_answers_from_offset(__pyx_v_self, __pyx_t_11); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 413, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __pyx_t_13 = __Pyx_PyInt_As_unsigned_int(__pyx_t_7); if (unlikely((__pyx_t_13 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(0, 413, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_v_answers_written = __pyx_t_13;

    /* "zeroconf/_protocol/outgoing.py":414
 *             questions_written = self._write_questions_from_offset(questions_offset)
 *             answers_written = self._write_answers_from_offset(answer_offset)
 *             authorities_written = self._write_records_from_offset(self.authorities, authority_offset)             # <<<<<<<<<<<<<<
 *             additionals_written = self._write_records_from_offset(self.additionals, additional_offset)
 * 
 */
    __pyx_t_7 = __pyx_v_self->authorities;
    __Pyx_INCREF(__pyx_t_7);
    __pyx_t_11 = __Pyx_PyInt_From_unsigned_int(__pyx_v_authority_offset); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 414, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __pyx_t_9 = ((struct __pyx_vtabstruct_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self->__pyx_vtab)->_write_records_from_offset(__pyx_v_self, __pyx_t_7, __pyx_t_11); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 414, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __pyx_t_13 = __Pyx_PyInt_As_unsigned_int(__pyx_t_9); if (unlikely((__pyx_t_13 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(0, 414, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_v_authorities_written = __pyx_t_13;

    /* "zeroconf/_protocol/outgoing.py":415
 *             answers_written = self._write_answers_from_offset(answer_offset)
 *             authorities_written = self._write_records_from_offset(self.authorities, authority_offset)
 *             additionals_written = self._write_records_from_offset(self.additionals, additional_offset)             # <<<<<<<<<<<<<<
 * 
 *             self._insert_short_at_start(additionals_written)
 */
    __pyx_t_9 = __pyx_v_self->additionals;
    __Pyx_INCREF(__pyx_t_9);
    __pyx_t_11 = __Pyx_PyInt_From_unsigned_int(__pyx_v_additional_offset); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 415, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __pyx_t_7 = ((struct __pyx_vtabstruct_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self->__pyx_vtab)->_write_records_from_offset(__pyx_v_self, __pyx_t_9, __pyx_t_11); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 415, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __pyx_t_13 = __Pyx_PyInt_As_unsigned_int(__pyx_t_7); if (unlikely((__pyx_t_13 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(0, 415, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_v_additionals_written = __pyx_t_13;

    /* "zeroconf/_protocol/outgoing.py":417
 *             additionals_written = self._write_records_from_offset(self.additionals, additional_offset)
 * 
 *             self._insert_short_at_start(additionals_written)             # <<<<<<<<<<<<<<
 *             self._insert_short_at_start(authorities_written)
 *             self._insert_short_at_start(answers_written)
 */
    __pyx_t_7 = __Pyx_PyInt_From_unsigned_int(__pyx_v_additionals_written); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 417, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_11 = ((struct __pyx_vtabstruct_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self->__pyx_vtab)->_insert_short_at_start(__pyx_v_self, __pyx_t_7); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 417, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

    /* "zeroconf/_protocol/outgoing.py":418
 * 
 *             self._insert_short_at_start(additionals_written)
 *             self._insert_short_at_start(authorities_written)             # <<<<<<<<<<<<<<
 *             self._insert_short_at_start(answers_written)
 *             self._insert_short_at_start(questions_written)
 */
    __pyx_t_11 = __Pyx_PyInt_From_unsigned_int(__pyx_v_authorities_written); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 418, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __pyx_t_7 = ((struct __pyx_vtabstruct_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self->__pyx_vtab)->_insert_short_at_start(__pyx_v_self, __pyx_t_11); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 418, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

    /* "zeroconf/_protocol/outgoing.py":419
 *             self._insert_short_at_start(additionals_written)
 *             self._insert_short_at_start(authorities_written)
 *             self._insert_short_at_start(answers_written)             # <<<<<<<<<<<<<<
 *             self._insert_short_at_start(questions_written)
 * 
 */
    __pyx_t_7 = __Pyx_PyInt_From_unsigned_int(__pyx_v_answers_written); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 419, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_11 = ((struct __pyx_vtabstruct_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self->__pyx_vtab)->_insert_short_at_start(__pyx_v_self, __pyx_t_7); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 419, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

    /* "zeroconf/_protocol/outgoing.py":420
 *             self._insert_short_at_start(authorities_written)
 *             self._insert_short_at_start(answers_written)
 *             self._insert_short_at_start(questions_written)             # <<<<<<<<<<<<<<
 * 
 *             questions_offset += questions_written
 */
    __pyx_t_11 = __Pyx_PyInt_From_unsigned_int(__pyx_v_questions_written); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 420, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __pyx_t_7 = ((struct __pyx_vtabstruct_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self->__pyx_vtab)->_insert_short_at_start(__pyx_v_self, __pyx_t_11); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 420, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

    /* "zeroconf/_protocol/outgoing.py":422
 *             self._insert_short_at_start(questions_written)
 * 
 *             questions_offset += questions_written             # <<<<<<<<<<<<<<
 *             answer_offset += answers_written
 *             authority_offset += authorities_written
 */
    __pyx_v_questions_offset = (__pyx_v_questions_offset + __pyx_v_questions_written);

    /* "zeroconf/_protocol/outgoing.py":423
 * 
 *             questions_offset += questions_written
 *             answer_offset += answers_written             # <<<<<<<<<<<<<<
 *             authority_offset += authorities_written
 *             additional_offset += additionals_written
 */
    __pyx_v_answer_offset = (__pyx_v_answer_offset + __pyx_v_answers_written);

    /* "zeroconf/_protocol/outgoing.py":424
 *             questions_offset += questions_written
 *             answer_offset += answers_written
 *             authority_offset += authorities_written             # <<<<<<<<<<<<<<
 *             additional_offset += additionals_written
 *             if debug_enable:
 */
    __pyx_v_authority_offset = (__pyx_v_authority_offset + __pyx_v_authorities_written);

    /* "zeroconf/_protocol/outgoing.py":425
 *             answer_offset += answers_written
 *             authority_offset += authorities_written
 *             additional_offset += additionals_written             # <<<<<<<<<<<<<<
 *             if debug_enable:
 *                 log.debug(
 */
    __pyx_v_additional_offset = (__pyx_v_additional_offset + __pyx_v_additionals_written);

    /* "zeroconf/_protocol/outgoing.py":426
 *             authority_offset += authorities_written
 *             additional_offset += additionals_written
 *             if debug_enable:             # <<<<<<<<<<<<<<
 *                 log.debug(
 *                     "now offsets = questions=%d, answers=%d, authorities=%d, additionals=%d",
 */
    __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_v_debug_enable); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 426, __pyx_L1_error)
    if (__pyx_t_3) {

      /* "zeroconf/_protocol/outgoing.py":427
 *             additional_offset += additionals_written
 *             if debug_enable:
 *                 log.debug(             # <<<<<<<<<<<<<<
 *                     "now offsets = questions=%d, answers=%d, authorities=%d, additionals=%d",
 *                     questions_offset,
 */
      __Pyx_GetModuleGlobalName(__pyx_t_11, __pyx_n_s_log); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 427, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_11, __pyx_n_s_debug); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 427, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

      /* "zeroconf/_protocol/outgoing.py":429
 *                 log.debug(
 *                     "now offsets = questions=%d, answers=%d, authorities=%d, additionals=%d",
 *                     questions_offset,             # <<<<<<<<<<<<<<
 *                     answer_offset,
 *                     authority_offset,
 */
      __pyx_t_11 = __Pyx_PyInt_From_unsigned_int(__pyx_v_questions_offset); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 429, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);

      /* "zeroconf/_protocol/outgoing.py":430
 *                     "now offsets = questions=%d, answers=%d, authorities=%d, additionals=%d",
 *                     questions_offset,
 *                     answer_offset,             # <<<<<<<<<<<<<<
 *                     authority_offset,
 *                     additional_offset,
 */
      __pyx_t_4 = __Pyx_PyInt_From_unsigned_int(__pyx_v_answer_offset); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 430, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);

      /* "zeroconf/_protocol/outgoing.py":431
 *                     questions_offset,
 *                     answer_offset,
 *                     authority_offset,             # <<<<<<<<<<<<<<
 *                     additional_offset,
 *                 )
 */
      __pyx_t_1 = __Pyx_PyInt_From_unsigned_int(__pyx_v_authority_offset); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 431, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);

      /* "zeroconf/_protocol/outgoing.py":432
 *                     answer_offset,
 *                     authority_offset,
 *                     additional_offset,             # <<<<<<<<<<<<<<
 *                 )
 * 
 */
      __pyx_t_8 = __Pyx_PyInt_From_unsigned_int(__pyx_v_additional_offset); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 432, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_5 = NULL;
      __pyx_t_10 = 0;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_9))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_9);
        if (likely(__pyx_t_5)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
          __Pyx_INCREF(__pyx_t_5);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_9, function);
          __pyx_t_10 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_9)) {
        PyObject *__pyx_temp[6] = {__pyx_t_5, __pyx_kp_u_now_offsets_questions_d_answers, __pyx_t_11, __pyx_t_4, __pyx_t_1, __pyx_t_8};
        __pyx_t_7 = __Pyx_PyFunction_FastCall(__pyx_t_9, __pyx_temp+1-__pyx_t_10, 5+__pyx_t_10); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 427, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_9)) {
        PyObject *__pyx_temp[6] = {__pyx_t_5, __pyx_kp_u_now_offsets_questions_d_answers, __pyx_t_11, __pyx_t_4, __pyx_t_1, __pyx_t_8};
        __pyx_t_7 = __Pyx_PyCFunction_FastCall(__pyx_t_9, __pyx_temp+1-__pyx_t_10, 5+__pyx_t_10); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 427, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      } else
      #endif
      {
        __pyx_t_2 = PyTuple_New(5+__pyx_t_10); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 427, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        if (__pyx_t_5) {
          __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_5); __pyx_t_5 = NULL;
        }
        __Pyx_INCREF(__pyx_kp_u_now_offsets_questions_d_answers);
        __Pyx_GIVEREF(__pyx_kp_u_now_offsets_questions_d_answers);
        PyTuple_SET_ITEM(__pyx_t_2, 0+__pyx_t_10, __pyx_kp_u_now_offsets_questions_d_answers);
        __Pyx_GIVEREF(__pyx_t_11);
        PyTuple_SET_ITEM(__pyx_t_2, 1+__pyx_t_10, __pyx_t_11);
        __Pyx_GIVEREF(__pyx_t_4);
        PyTuple_SET_ITEM(__pyx_t_2, 2+__pyx_t_10, __pyx_t_4);
        __Pyx_GIVEREF(__pyx_t_1);
        PyTuple_SET_ITEM(__pyx_t_2, 3+__pyx_t_10, __pyx_t_1);
        __Pyx_GIVEREF(__pyx_t_8);
        PyTuple_SET_ITEM(__pyx_t_2, 4+__pyx_t_10, __pyx_t_8);
        __pyx_t_11 = 0;
        __pyx_t_4 = 0;
        __pyx_t_1 = 0;
        __pyx_t_8 = 0;
        __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_t_2, NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 427, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      }
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

      /* "zeroconf/_protocol/outgoing.py":426
 *             authority_offset += authorities_written
 *             additional_offset += additionals_written
 *             if debug_enable:             # <<<<<<<<<<<<<<
 *                 log.debug(
 *                     "now offsets = questions=%d, answers=%d, authorities=%d, additionals=%d",
 */
    }

    /* "zeroconf/_protocol/outgoing.py":435
 *                 )
 * 
 *             if self.is_query() and self._has_more_to_add(             # <<<<<<<<<<<<<<
 *                 questions_offset, answer_offset, authority_offset, additional_offset
 *             ):
 */
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_is_query); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 435, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_2 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_9);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_9, function);
      }
    }
    __pyx_t_7 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_9);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 435, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_7); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 435, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (__pyx_t_6) {
    } else {
      __pyx_t_3 = __pyx_t_6;
      goto __pyx_L11_bool_binop_done;
    }

    /* "zeroconf/_protocol/outgoing.py":436
 * 
 *             if self.is_query() and self._has_more_to_add(
 *                 questions_offset, answer_offset, authority_offset, additional_offset             # <<<<<<<<<<<<<<
 *             ):
 *                 # https://datatracker.ietf.org/doc/html/rfc6762#section-7.2
 */
    __pyx_t_7 = __Pyx_PyInt_From_unsigned_int(__pyx_v_questions_offset); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 436, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = __Pyx_PyInt_From_unsigned_int(__pyx_v_answer_offset); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 436, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_2 = __Pyx_PyInt_From_unsigned_int(__pyx_v_authority_offset); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 436, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_8 = __Pyx_PyInt_From_unsigned_int(__pyx_v_additional_offset); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 436, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);

    /* "zeroconf/_protocol/outgoing.py":435
 *                 )
 * 
 *             if self.is_query() and self._has_more_to_add(             # <<<<<<<<<<<<<<
 *                 questions_offset, answer_offset, authority_offset, additional_offset
 *             ):
 */
    __pyx_t_1 = ((struct __pyx_vtabstruct_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self->__pyx_vtab)->_has_more_to_add(__pyx_v_self, __pyx_t_7, __pyx_t_9, __pyx_t_2, __pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 435, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 435, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_3 = __pyx_t_6;
    __pyx_L11_bool_binop_done:;
    if (__pyx_t_3) {

      /* "zeroconf/_protocol/outgoing.py":439
 *             ):
 *                 # https://datatracker.ietf.org/doc/html/rfc6762#section-7.2
 *                 log.debug("Setting TC flag")             # <<<<<<<<<<<<<<
 *                 self._insert_short_at_start(self.flags | _FLAGS_TC)
 *             else:
 */
      __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_n_s_log); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 439, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_debug); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 439, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_8 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_8)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_8);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      __pyx_t_1 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_8, __pyx_kp_u_Setting_TC_flag) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_kp_u_Setting_TC_flag);
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 439, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "zeroconf/_protocol/outgoing.py":440
 *                 # https://datatracker.ietf.org/doc/html/rfc6762#section-7.2
 *                 log.debug("Setting TC flag")
 *                 self._insert_short_at_start(self.flags | _FLAGS_TC)             # <<<<<<<<<<<<<<
 *             else:
 *                 self._insert_short_at_start(self.flags)
 */
      __pyx_t_1 = __Pyx_PyInt_From_unsigned_int((__pyx_v_self->flags | __pyx_v_8zeroconf_9_protocol_8outgoing__FLAGS_TC)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 440, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_2 = ((struct __pyx_vtabstruct_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self->__pyx_vtab)->_insert_short_at_start(__pyx_v_self, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 440, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "zeroconf/_protocol/outgoing.py":435
 *                 )
 * 
 *             if self.is_query() and self._has_more_to_add(             # <<<<<<<<<<<<<<
 *                 questions_offset, answer_offset, authority_offset, additional_offset
 *             ):
 */
      goto __pyx_L10;
    }

    /* "zeroconf/_protocol/outgoing.py":442
 *                 self._insert_short_at_start(self.flags | _FLAGS_TC)
 *             else:
 *                 self._insert_short_at_start(self.flags)             # <<<<<<<<<<<<<<
 * 
 *             if self.multicast:
 */
    /*else*/ {
      __pyx_t_2 = __Pyx_PyInt_From_unsigned_int(__pyx_v_self->flags); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 442, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_1 = ((struct __pyx_vtabstruct_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self->__pyx_vtab)->_insert_short_at_start(__pyx_v_self, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 442, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    }
    __pyx_L10:;

    /* "zeroconf/_protocol/outgoing.py":444
 *                 self._insert_short_at_start(self.flags)
 * 
 *             if self.multicast:             # <<<<<<<<<<<<<<
 *                 self._insert_short_at_start(0)
 *             else:
 */
    __pyx_t_3 = (__pyx_v_self->multicast != 0);
    if (__pyx_t_3) {

      /* "zeroconf/_protocol/outgoing.py":445
 * 
 *             if self.multicast:
 *                 self._insert_short_at_start(0)             # <<<<<<<<<<<<<<
 *             else:
 *                 self._insert_short_at_start(self.id)
 */
      __pyx_t_1 = ((struct __pyx_vtabstruct_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self->__pyx_vtab)->_insert_short_at_start(__pyx_v_self, __pyx_int_0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 445, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "zeroconf/_protocol/outgoing.py":444
 *                 self._insert_short_at_start(self.flags)
 * 
 *             if self.multicast:             # <<<<<<<<<<<<<<
 *                 self._insert_short_at_start(0)
 *             else:
 */
      goto __pyx_L13;
    }

    /* "zeroconf/_protocol/outgoing.py":447
 *                 self._insert_short_at_start(0)
 *             else:
 *                 self._insert_short_at_start(self.id)             # <<<<<<<<<<<<<<
 * 
 *             self.packets_data.append(b''.join(self.data))
 */
    /*else*/ {
      __pyx_t_1 = __pyx_v_self->id;
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_2 = ((struct __pyx_vtabstruct_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self->__pyx_vtab)->_insert_short_at_start(__pyx_v_self, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 447, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }
    __pyx_L13:;

    /* "zeroconf/_protocol/outgoing.py":449
 *                 self._insert_short_at_start(self.id)
 * 
 *             self.packets_data.append(b''.join(self.data))             # <<<<<<<<<<<<<<
 *             self._reset_for_next_packet()
 * 
 */
    if (unlikely(__pyx_v_self->packets_data == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
      __PYX_ERR(0, 449, __pyx_L1_error)
    }
    __pyx_t_2 = __pyx_v_self->data;
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_1 = __Pyx_PyBytes_Join(__pyx_kp_b__6, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 449, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_14 = __Pyx_PyList_Append(__pyx_v_self->packets_data, __pyx_t_1); if (unlikely(__pyx_t_14 == ((int)-1))) __PYX_ERR(0, 449, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "zeroconf/_protocol/outgoing.py":450
 * 
 *             self.packets_data.append(b''.join(self.data))
 *             self._reset_for_next_packet()             # <<<<<<<<<<<<<<
 * 
 *             if (questions_written + answers_written + authorities_written + additionals_written) == 0 and (
 */
    __pyx_t_1 = ((struct __pyx_vtabstruct_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self->__pyx_vtab)->_reset_for_next_packet(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 450, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "zeroconf/_protocol/outgoing.py":452
 *             self._reset_for_next_packet()
 * 
 *             if (questions_written + answers_written + authorities_written + additionals_written) == 0 and (             # <<<<<<<<<<<<<<
 *                 len(self.questions) + len(self.answers) + len(self.authorities) + len(self.additionals)
 *             ) > 0:
 */
    __pyx_t_6 = (((((__pyx_v_questions_written + __pyx_v_answers_written) + __pyx_v_authorities_written) + __pyx_v_additionals_written) == 0) != 0);
    if (__pyx_t_6) {
    } else {
      __pyx_t_3 = __pyx_t_6;
      goto __pyx_L15_bool_binop_done;
    }

    /* "zeroconf/_protocol/outgoing.py":453
 * 
 *             if (questions_written + answers_written + authorities_written + additionals_written) == 0 and (
 *                 len(self.questions) + len(self.answers) + len(self.authorities) + len(self.additionals)             # <<<<<<<<<<<<<<
 *             ) > 0:
 *                 log.warning("packets() made no progress adding records; returning")
 */
    __pyx_t_1 = __pyx_v_self->questions;
    __Pyx_INCREF(__pyx_t_1);
    if (unlikely(__pyx_t_1 == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(0, 453, __pyx_L1_error)
    }
    __pyx_t_12 = PyList_GET_SIZE(__pyx_t_1); if (unlikely(__pyx_t_12 == ((Py_ssize_t)-1))) __PYX_ERR(0, 453, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __pyx_v_self->answers;
    __Pyx_INCREF(__pyx_t_1);
    if (unlikely(__pyx_t_1 == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(0, 453, __pyx_L1_error)
    }
    __pyx_t_15 = PyList_GET_SIZE(__pyx_t_1); if (unlikely(__pyx_t_15 == ((Py_ssize_t)-1))) __PYX_ERR(0, 453, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __pyx_v_self->authorities;
    __Pyx_INCREF(__pyx_t_1);
    if (unlikely(__pyx_t_1 == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(0, 453, __pyx_L1_error)
    }
    __pyx_t_16 = PyList_GET_SIZE(__pyx_t_1); if (unlikely(__pyx_t_16 == ((Py_ssize_t)-1))) __PYX_ERR(0, 453, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __pyx_v_self->additionals;
    __Pyx_INCREF(__pyx_t_1);
    if (unlikely(__pyx_t_1 == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(0, 453, __pyx_L1_error)
    }
    __pyx_t_17 = PyList_GET_SIZE(__pyx_t_1); if (unlikely(__pyx_t_17 == ((Py_ssize_t)-1))) __PYX_ERR(0, 453, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "zeroconf/_protocol/outgoing.py":454
 *             if (questions_written + answers_written + authorities_written + additionals_written) == 0 and (
 *                 len(self.questions) + len(self.answers) + len(self.authorities) + len(self.additionals)
 *             ) > 0:             # <<<<<<<<<<<<<<
 *                 log.warning("packets() made no progress adding records; returning")
 *                 break
 */
    __pyx_t_6 = (((((__pyx_t_12 + __pyx_t_15) + __pyx_t_16) + __pyx_t_17) > 0) != 0);
    __pyx_t_3 = __pyx_t_6;
    __pyx_L15_bool_binop_done:;

    /* "zeroconf/_protocol/outgoing.py":452
 *             self._reset_for_next_packet()
 * 
 *             if (questions_written + answers_written + authorities_written + additionals_written) == 0 and (             # <<<<<<<<<<<<<<
 *                 len(self.questions) + len(self.answers) + len(self.authorities) + len(self.additionals)
 *             ) > 0:
 */
    if (__pyx_t_3) {

      /* "zeroconf/_protocol/outgoing.py":455
 *                 len(self.questions) + len(self.answers) + len(self.authorities) + len(self.additionals)
 *             ) > 0:
 *                 log.warning("packets() made no progress adding records; returning")             # <<<<<<<<<<<<<<
 *                 break
 *         self.state = State.finished
 */
      __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_log); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 455, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_warning); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 455, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_8))) {
        __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_8);
        if (likely(__pyx_t_2)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
          __Pyx_INCREF(__pyx_t_2);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_8, function);
        }
      }
      __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_8, __pyx_t_2, __pyx_kp_u_packets_made_no_progress_adding) : __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_kp_u_packets_made_no_progress_adding);
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 455, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "zeroconf/_protocol/outgoing.py":456
 *             ) > 0:
 *                 log.warning("packets() made no progress adding records; returning")
 *                 break             # <<<<<<<<<<<<<<
 *         self.state = State.finished
 *         return self.packets_data
 */
      goto __pyx_L5_break;

      /* "zeroconf/_protocol/outgoing.py":452
 *             self._reset_for_next_packet()
 * 
 *             if (questions_written + answers_written + authorities_written + additionals_written) == 0 and (             # <<<<<<<<<<<<<<
 *                 len(self.questions) + len(self.answers) + len(self.authorities) + len(self.additionals)
 *             ) > 0:
 */
    }
  }
  __pyx_L5_break:;

  /* "zeroconf/_protocol/outgoing.py":457
 *                 log.warning("packets() made no progress adding records; returning")
 *                 break
 *         self.state = State.finished             # <<<<<<<<<<<<<<
 *         return self.packets_data
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_State); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 457, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_finished); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 457, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_GIVEREF(__pyx_t_8);
  __Pyx_GOTREF(__pyx_v_self->state);
  __Pyx_DECREF(__pyx_v_self->state);
  __pyx_v_self->state = __pyx_t_8;
  __pyx_t_8 = 0;

  /* "zeroconf/_protocol/outgoing.py":458
 *                 break
 *         self.state = State.finished
 *         return self.packets_data             # <<<<<<<<<<<<<<
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->packets_data);
  __pyx_r = __pyx_v_self->packets_data;
  goto __pyx_L0;

  /* "zeroconf/_protocol/outgoing.py":380
 *         return self._packets()
 * 
 *     def _packets(self) -> List[bytes]:             # <<<<<<<<<<<<<<
 *         if self.state == State.finished:
 *             return self.packets_data
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing._packets", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_debug_enable);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/outgoing.pxd":19
 * cdef class DNSOutgoing:
 * 
 *     cdef public unsigned int flags             # <<<<<<<<<<<<<<
 *     cdef public object finished
 *     cdef public object id
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5flags_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5flags_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5flags___get__(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5flags___get__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_unsigned_int(__pyx_v_self->flags); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 19, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing.flags.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5flags_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5flags_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5flags_2__set__(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5flags_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  unsigned int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_unsigned_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(1, 19, __pyx_L1_error)
  __pyx_v_self->flags = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing.flags.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/outgoing.pxd":20
 * 
 *     cdef public unsigned int flags
 *     cdef public object finished             # <<<<<<<<<<<<<<
 *     cdef public object id
 *     cdef public bint multicast
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_8finished_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_8finished_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_8finished___get__(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_8finished___get__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->finished);
  __pyx_r = __pyx_v_self->finished;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_8finished_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_8finished_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_8finished_2__set__(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_8finished_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->finished);
  __Pyx_DECREF(__pyx_v_self->finished);
  __pyx_v_self->finished = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_8finished_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_8finished_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_8finished_4__del__(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_8finished_4__del__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->finished);
  __Pyx_DECREF(__pyx_v_self->finished);
  __pyx_v_self->finished = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/outgoing.pxd":21
 *     cdef public unsigned int flags
 *     cdef public object finished
 *     cdef public object id             # <<<<<<<<<<<<<<
 *     cdef public bint multicast
 *     cdef public cython.list packets_data
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_2id_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_2id_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_2id___get__(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_2id___get__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->id);
  __pyx_r = __pyx_v_self->id;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_2id_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_2id_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_2id_2__set__(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_2id_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->id);
  __Pyx_DECREF(__pyx_v_self->id);
  __pyx_v_self->id = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_2id_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_2id_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_2id_4__del__(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_2id_4__del__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->id);
  __Pyx_DECREF(__pyx_v_self->id);
  __pyx_v_self->id = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/outgoing.pxd":22
 *     cdef public object finished
 *     cdef public object id
 *     cdef public bint multicast             # <<<<<<<<<<<<<<
 *     cdef public cython.list packets_data
 *     cdef public object names
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_9multicast_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_9multicast_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_9multicast___get__(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_9multicast___get__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->multicast); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 22, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing.multicast.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_9multicast_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_9multicast_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_9multicast_2__set__(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_9multicast_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 22, __pyx_L1_error)
  __pyx_v_self->multicast = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing.multicast.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/outgoing.pxd":23
 *     cdef public object id
 *     cdef public bint multicast
 *     cdef public cython.list packets_data             # <<<<<<<<<<<<<<
 *     cdef public object names
 *     cdef public cython.list data
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_12packets_data_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_12packets_data_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_12packets_data___get__(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_12packets_data___get__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->packets_data);
  __pyx_r = __pyx_v_self->packets_data;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_12packets_data_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_12packets_data_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_12packets_data_2__set__(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_12packets_data_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyList_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||((void)PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(1, 23, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->packets_data);
  __Pyx_DECREF(__pyx_v_self->packets_data);
  __pyx_v_self->packets_data = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing.packets_data.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_12packets_data_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_12packets_data_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_12packets_data_4__del__(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_12packets_data_4__del__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->packets_data);
  __Pyx_DECREF(__pyx_v_self->packets_data);
  __pyx_v_self->packets_data = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/outgoing.pxd":24
 *     cdef public bint multicast
 *     cdef public cython.list packets_data
 *     cdef public object names             # <<<<<<<<<<<<<<
 *     cdef public cython.list data
 *     cdef public unsigned int size
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5names_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5names_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5names___get__(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5names___get__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->names);
  __pyx_r = __pyx_v_self->names;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5names_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5names_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5names_2__set__(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5names_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->names);
  __Pyx_DECREF(__pyx_v_self->names);
  __pyx_v_self->names = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5names_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5names_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5names_4__del__(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5names_4__del__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->names);
  __Pyx_DECREF(__pyx_v_self->names);
  __pyx_v_self->names = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/outgoing.pxd":25
 *     cdef public cython.list packets_data
 *     cdef public object names
 *     cdef public cython.list data             # <<<<<<<<<<<<<<
 *     cdef public unsigned int size
 *     cdef public object allow_long
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_4data_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_4data_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_4data___get__(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_4data___get__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->data);
  __pyx_r = __pyx_v_self->data;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_4data_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_4data_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_4data_2__set__(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_4data_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyList_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||((void)PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(1, 25, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->data);
  __Pyx_DECREF(__pyx_v_self->data);
  __pyx_v_self->data = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing.data.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_4data_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_4data_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_4data_4__del__(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_4data_4__del__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->data);
  __Pyx_DECREF(__pyx_v_self->data);
  __pyx_v_self->data = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/outgoing.pxd":26
 *     cdef public object names
 *     cdef public cython.list data
 *     cdef public unsigned int size             # <<<<<<<<<<<<<<
 *     cdef public object allow_long
 *     cdef public object state
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_4size_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_4size_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_4size___get__(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_4size___get__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_unsigned_int(__pyx_v_self->size); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing.size.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_4size_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_4size_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_4size_2__set__(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_4size_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  unsigned int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_unsigned_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(1, 26, __pyx_L1_error)
  __pyx_v_self->size = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing.size.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/outgoing.pxd":27
 *     cdef public cython.list data
 *     cdef public unsigned int size
 *     cdef public object allow_long             # <<<<<<<<<<<<<<
 *     cdef public object state
 *     cdef public cython.list questions
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_10allow_long_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_10allow_long_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_10allow_long___get__(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_10allow_long___get__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->allow_long);
  __pyx_r = __pyx_v_self->allow_long;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_10allow_long_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_10allow_long_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_10allow_long_2__set__(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_10allow_long_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->allow_long);
  __Pyx_DECREF(__pyx_v_self->allow_long);
  __pyx_v_self->allow_long = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_10allow_long_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_10allow_long_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_10allow_long_4__del__(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_10allow_long_4__del__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->allow_long);
  __Pyx_DECREF(__pyx_v_self->allow_long);
  __pyx_v_self->allow_long = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/outgoing.pxd":28
 *     cdef public unsigned int size
 *     cdef public object allow_long
 *     cdef public object state             # <<<<<<<<<<<<<<
 *     cdef public cython.list questions
 *     cdef public cython.list answers
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5state_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5state_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5state___get__(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5state___get__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->state);
  __pyx_r = __pyx_v_self->state;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5state_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5state_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5state_2__set__(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5state_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->state);
  __Pyx_DECREF(__pyx_v_self->state);
  __pyx_v_self->state = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5state_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5state_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5state_4__del__(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5state_4__del__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->state);
  __Pyx_DECREF(__pyx_v_self->state);
  __pyx_v_self->state = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/outgoing.pxd":29
 *     cdef public object allow_long
 *     cdef public object state
 *     cdef public cython.list questions             # <<<<<<<<<<<<<<
 *     cdef public cython.list answers
 *     cdef public cython.list authorities
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_9questions_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_9questions_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_9questions___get__(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_9questions___get__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->questions);
  __pyx_r = __pyx_v_self->questions;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_9questions_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_9questions_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_9questions_2__set__(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_9questions_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyList_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||((void)PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(1, 29, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->questions);
  __Pyx_DECREF(__pyx_v_self->questions);
  __pyx_v_self->questions = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing.questions.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_9questions_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_9questions_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_9questions_4__del__(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_9questions_4__del__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->questions);
  __Pyx_DECREF(__pyx_v_self->questions);
  __pyx_v_self->questions = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/outgoing.pxd":30
 *     cdef public object state
 *     cdef public cython.list questions
 *     cdef public cython.list answers             # <<<<<<<<<<<<<<
 *     cdef public cython.list authorities
 *     cdef public cython.list additionals
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_7answers_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_7answers_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_7answers___get__(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_7answers___get__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->answers);
  __pyx_r = __pyx_v_self->answers;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_7answers_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_7answers_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_7answers_2__set__(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_7answers_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyList_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||((void)PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(1, 30, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->answers);
  __Pyx_DECREF(__pyx_v_self->answers);
  __pyx_v_self->answers = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing.answers.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_7answers_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_7answers_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_7answers_4__del__(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_7answers_4__del__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->answers);
  __Pyx_DECREF(__pyx_v_self->answers);
  __pyx_v_self->answers = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/outgoing.pxd":31
 *     cdef public cython.list questions
 *     cdef public cython.list answers
 *     cdef public cython.list authorities             # <<<<<<<<<<<<<<
 *     cdef public cython.list additionals
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_11authorities_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_11authorities_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_11authorities___get__(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_11authorities___get__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->authorities);
  __pyx_r = __pyx_v_self->authorities;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_11authorities_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_11authorities_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_11authorities_2__set__(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_11authorities_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyList_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||((void)PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(1, 31, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->authorities);
  __Pyx_DECREF(__pyx_v_self->authorities);
  __pyx_v_self->authorities = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing.authorities.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_11authorities_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_11authorities_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_11authorities_4__del__(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_11authorities_4__del__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->authorities);
  __Pyx_DECREF(__pyx_v_self->authorities);
  __pyx_v_self->authorities = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/outgoing.pxd":32
 *     cdef public cython.list answers
 *     cdef public cython.list authorities
 *     cdef public cython.list additionals             # <<<<<<<<<<<<<<
 * 
 *     cdef _reset_for_next_packet(self)
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_11additionals_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_11additionals_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_11additionals___get__(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_11additionals___get__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->additionals);
  __pyx_r = __pyx_v_self->additionals;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_11additionals_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_11additionals_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_11additionals_2__set__(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_11additionals_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyList_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||((void)PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(1, 32, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->additionals);
  __Pyx_DECREF(__pyx_v_self->additionals);
  __pyx_v_self->additionals = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing.additionals.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_11additionals_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_11additionals_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_11additionals_4__del__(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_11additionals_4__del__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->additionals);
  __Pyx_DECREF(__pyx_v_self->additionals);
  __pyx_v_self->additionals = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_39__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_39__reduce_cython__ = {"__reduce_cython__", (PyCFunction)__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_39__reduce_cython__, METH_NOARGS, 0};
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_39__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_38__reduce_cython__(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_38__reduce_cython__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.additionals, self.allow_long, self.answers, self.authorities, self.data, self.finished, self.flags, self.id, self.multicast, self.names, self.packets_data, self.questions, self.size, self.state)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyInt_From_unsigned_int(__pyx_v_self->flags); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->multicast); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_unsigned_int(__pyx_v_self->size); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(14); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_v_self->additionals);
  __Pyx_GIVEREF(__pyx_v_self->additionals);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_self->additionals);
  __Pyx_INCREF(__pyx_v_self->allow_long);
  __Pyx_GIVEREF(__pyx_v_self->allow_long);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_v_self->allow_long);
  __Pyx_INCREF(__pyx_v_self->answers);
  __Pyx_GIVEREF(__pyx_v_self->answers);
  PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_self->answers);
  __Pyx_INCREF(__pyx_v_self->authorities);
  __Pyx_GIVEREF(__pyx_v_self->authorities);
  PyTuple_SET_ITEM(__pyx_t_4, 3, __pyx_v_self->authorities);
  __Pyx_INCREF(__pyx_v_self->data);
  __Pyx_GIVEREF(__pyx_v_self->data);
  PyTuple_SET_ITEM(__pyx_t_4, 4, __pyx_v_self->data);
  __Pyx_INCREF(__pyx_v_self->finished);
  __Pyx_GIVEREF(__pyx_v_self->finished);
  PyTuple_SET_ITEM(__pyx_t_4, 5, __pyx_v_self->finished);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 6, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->id);
  __Pyx_GIVEREF(__pyx_v_self->id);
  PyTuple_SET_ITEM(__pyx_t_4, 7, __pyx_v_self->id);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 8, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->names);
  __Pyx_GIVEREF(__pyx_v_self->names);
  PyTuple_SET_ITEM(__pyx_t_4, 9, __pyx_v_self->names);
  __Pyx_INCREF(__pyx_v_self->packets_data);
  __Pyx_GIVEREF(__pyx_v_self->packets_data);
  PyTuple_SET_ITEM(__pyx_t_4, 10, __pyx_v_self->packets_data);
  __Pyx_INCREF(__pyx_v_self->questions);
  __Pyx_GIVEREF(__pyx_v_self->questions);
  PyTuple_SET_ITEM(__pyx_t_4, 11, __pyx_v_self->questions);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 12, __pyx_t_3);
  __Pyx_INCREF(__pyx_v_self->state);
  __Pyx_GIVEREF(__pyx_v_self->state);
  PyTuple_SET_ITEM(__pyx_t_4, 13, __pyx_v_self->state);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.additionals, self.allow_long, self.answers, self.authorities, self.data, self.finished, self.flags, self.id, self.multicast, self.names, self.packets_data, self.questions, self.size, self.state)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_4 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_v__dict = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "(tree fragment)":7
 *     state = (self.additionals, self.allow_long, self.answers, self.authorities, self.data, self.finished, self.flags, self.id, self.multicast, self.names, self.packets_data, self.questions, self.size, self.state)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_5 = (__pyx_v__dict != Py_None);
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v__dict);
    __pyx_t_3 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.additionals is not None or self.allow_long is not None or self.answers is not None or self.authorities is not None or self.data is not None or self.finished is not None or self.id is not None or self.names is not None or self.packets_data is not None or self.questions is not None or self.state is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.additionals, self.allow_long, self.answers, self.authorities, self.data, self.finished, self.flags, self.id, self.multicast, self.names, self.packets_data, self.questions, self.size, self.state)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.additionals is not None or self.allow_long is not None or self.answers is not None or self.authorities is not None or self.data is not None or self.finished is not None or self.id is not None or self.names is not None or self.packets_data is not None or self.questions is not None or self.state is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_DNSOutgoing, (type(self), 0x3d398d7, None), state
 */
  /*else*/ {
    __pyx_t_5 = (__pyx_v_self->additionals != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_5 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->allow_long != Py_None);
    __pyx_t_5 = (__pyx_t_7 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_6 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (__pyx_v_self->answers != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_5 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->authorities != ((PyObject*)Py_None));
    __pyx_t_5 = (__pyx_t_7 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_6 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (__pyx_v_self->data != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_5 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->finished != Py_None);
    __pyx_t_5 = (__pyx_t_7 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_6 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (__pyx_v_self->id != Py_None);
    __pyx_t_7 = (__pyx_t_5 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->names != Py_None);
    __pyx_t_5 = (__pyx_t_7 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_6 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (__pyx_v_self->packets_data != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_5 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->questions != ((PyObject*)Py_None));
    __pyx_t_5 = (__pyx_t_7 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_6 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (__pyx_v_self->state != Py_None);
    __pyx_t_7 = (__pyx_t_5 != 0);
    __pyx_t_6 = __pyx_t_7;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_6;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.additionals is not None or self.allow_long is not None or self.answers is not None or self.authorities is not None or self.data is not None or self.finished is not None or self.id is not None or self.names is not None or self.packets_data is not None or self.questions is not None or self.state is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_DNSOutgoing, (type(self), 0x3d398d7, None), state
 *     else:
 */
  __pyx_t_6 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":13
 *         use_setstate = self.additionals is not None or self.allow_long is not None or self.answers is not None or self.authorities is not None or self.data is not None or self.finished is not None or self.id is not None or self.names is not None or self.packets_data is not None or self.questions is not None or self.state is not None
 *     if use_setstate:
 *         return __pyx_unpickle_DNSOutgoing, (type(self), 0x3d398d7, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_DNSOutgoing, (type(self), 0x3d398d7, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_pyx_unpickle_DNSOutgoing); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_64198871);
    __Pyx_GIVEREF(__pyx_int_64198871);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_int_64198871);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_4, 2, Py_None);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_4);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_state);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.additionals is not None or self.allow_long is not None or self.answers is not None or self.authorities is not None or self.data is not None or self.finished is not None or self.id is not None or self.names is not None or self.packets_data is not None or self.questions is not None or self.state is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_DNSOutgoing, (type(self), 0x3d398d7, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_DNSOutgoing, (type(self), 0x3d398d7, None), state
 *     else:
 *         return __pyx_unpickle_DNSOutgoing, (type(self), 0x3d398d7, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_DNSOutgoing__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_pyx_unpickle_DNSOutgoing); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_64198871);
    __Pyx_GIVEREF(__pyx_int_64198871);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_int_64198871);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_4);
    __pyx_t_2 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_DNSOutgoing, (type(self), 0x3d398d7, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_DNSOutgoing__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_41__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyMethodDef __pyx_mdef_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_41__setstate_cython__ = {"__setstate_cython__", (PyCFunction)__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_41__setstate_cython__, METH_O, 0};
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_41__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_40__setstate_cython__(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_40__setstate_cython__(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_DNSOutgoing, (type(self), 0x3d398d7, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_DNSOutgoing__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||((void)PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_8zeroconf_9_protocol_8outgoing___pyx_unpickle_DNSOutgoing__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_DNSOutgoing, (type(self), 0x3d398d7, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_DNSOutgoing__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.DNSOutgoing.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_DNSOutgoing(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_1__pyx_unpickle_DNSOutgoing(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_8zeroconf_9_protocol_8outgoing_1__pyx_unpickle_DNSOutgoing = {"__pyx_unpickle_DNSOutgoing", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_8zeroconf_9_protocol_8outgoing_1__pyx_unpickle_DNSOutgoing, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_8zeroconf_9_protocol_8outgoing_1__pyx_unpickle_DNSOutgoing(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_DNSOutgoing (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_DNSOutgoing", 1, 3, 3, 1); __PYX_ERR(2, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_DNSOutgoing", 1, 3, 3, 2); __PYX_ERR(2, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_DNSOutgoing") < 0)) __PYX_ERR(2, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_DNSOutgoing", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.__pyx_unpickle_DNSOutgoing", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8outgoing___pyx_unpickle_DNSOutgoing(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8outgoing___pyx_unpickle_DNSOutgoing(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_DNSOutgoing", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0x3d398d7, 0xa242541, 0x5aef2cc):             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0x3d398d7, 0xa242541, 0x5aef2cc) = (additionals, allow_long, answers, authorities, data, finished, flags, id, multicast, names, packets_data, questions, size, state))" % __pyx_checksum)
 */
  __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = (__Pyx_PySequence_ContainsTF(__pyx_t_1, __pyx_tuple__7, Py_NE)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(2, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0x3d398d7, 0xa242541, 0x5aef2cc):
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0x3d398d7, 0xa242541, 0x5aef2cc) = (additionals, allow_long, answers, authorities, data, finished, flags, id, multicast, names, packets_data, questions, size, state))" % __pyx_checksum)
 *     __pyx_result = DNSOutgoing.__new__(__pyx_type)
 */
    __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_PickleError);
    __pyx_t_4 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_1, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_4, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_t_1);
    __pyx_v___pyx_PickleError = __pyx_t_1;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum not in (0x3d398d7, 0xa242541, 0x5aef2cc):
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0x3d398d7, 0xa242541, 0x5aef2cc) = (additionals, allow_long, answers, authorities, data, finished, flags, id, multicast, names, packets_data, questions, size, state))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = DNSOutgoing.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_0x_x_vs_0, __pyx_t_1); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_1 = __pyx_v___pyx_PickleError; __pyx_t_6 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
      }
    }
    __pyx_t_4 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_6, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_5);
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(2, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0x3d398d7, 0xa242541, 0x5aef2cc):             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0x3d398d7, 0xa242541, 0x5aef2cc) = (additionals, allow_long, answers, authorities, data, finished, flags, id, multicast, names, packets_data, questions, size, state))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0x3d398d7, 0xa242541, 0x5aef2cc) = (additionals, allow_long, answers, authorities, data, finished, flags, id, multicast, names, packets_data, questions, size, state))" % __pyx_checksum)
 *     __pyx_result = DNSOutgoing.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_DNSOutgoing__set_state(<DNSOutgoing> __pyx_result, __pyx_state)
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_8zeroconf_9_protocol_8outgoing_DNSOutgoing), __pyx_n_s_new); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_4 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_5, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0x3d398d7, 0xa242541, 0x5aef2cc) = (additionals, allow_long, answers, authorities, data, finished, flags, id, multicast, names, packets_data, questions, size, state))" % __pyx_checksum)
 *     __pyx_result = DNSOutgoing.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_DNSOutgoing__set_state(<DNSOutgoing> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_3 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {

    /* "(tree fragment)":9
 *     __pyx_result = DNSOutgoing.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_DNSOutgoing__set_state(<DNSOutgoing> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_DNSOutgoing__set_state(DNSOutgoing __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||((void)PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 9, __pyx_L1_error)
    __pyx_t_4 = __pyx_f_8zeroconf_9_protocol_8outgoing___pyx_unpickle_DNSOutgoing__set_state(((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0x3d398d7, 0xa242541, 0x5aef2cc) = (additionals, allow_long, answers, authorities, data, finished, flags, id, multicast, names, packets_data, questions, size, state))" % __pyx_checksum)
 *     __pyx_result = DNSOutgoing.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_DNSOutgoing__set_state(<DNSOutgoing> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_DNSOutgoing__set_state(<DNSOutgoing> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_DNSOutgoing__set_state(DNSOutgoing __pyx_result, tuple __pyx_state):
 *     __pyx_result.additionals = __pyx_state[0]; __pyx_result.allow_long = __pyx_state[1]; __pyx_result.answers = __pyx_state[2]; __pyx_result.authorities = __pyx_state[3]; __pyx_result.data = __pyx_state[4]; __pyx_result.finished = __pyx_state[5]; __pyx_result.flags = __pyx_state[6]; __pyx_result.id = __pyx_state[7]; __pyx_result.multicast = __pyx_state[8]; __pyx_result.names = __pyx_state[9]; __pyx_result.packets_data = __pyx_state[10]; __pyx_result.questions = __pyx_state[11]; __pyx_result.size = __pyx_state[12]; __pyx_result.state = __pyx_state[13]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_DNSOutgoing(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.__pyx_unpickle_DNSOutgoing", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_DNSOutgoing__set_state(<DNSOutgoing> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_DNSOutgoing__set_state(DNSOutgoing __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.additionals = __pyx_state[0]; __pyx_result.allow_long = __pyx_state[1]; __pyx_result.answers = __pyx_state[2]; __pyx_result.authorities = __pyx_state[3]; __pyx_result.data = __pyx_state[4]; __pyx_result.finished = __pyx_state[5]; __pyx_result.flags = __pyx_state[6]; __pyx_result.id = __pyx_state[7]; __pyx_result.multicast = __pyx_state[8]; __pyx_result.names = __pyx_state[9]; __pyx_result.packets_data = __pyx_state[10]; __pyx_result.questions = __pyx_state[11]; __pyx_result.size = __pyx_state[12]; __pyx_result.state = __pyx_state[13]
 *     if len(__pyx_state) > 14 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_8zeroconf_9_protocol_8outgoing___pyx_unpickle_DNSOutgoing__set_state(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  unsigned int __pyx_t_2;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_DNSOutgoing__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_DNSOutgoing__set_state(DNSOutgoing __pyx_result, tuple __pyx_state):
 *     __pyx_result.additionals = __pyx_state[0]; __pyx_result.allow_long = __pyx_state[1]; __pyx_result.answers = __pyx_state[2]; __pyx_result.authorities = __pyx_state[3]; __pyx_result.data = __pyx_state[4]; __pyx_result.finished = __pyx_state[5]; __pyx_result.flags = __pyx_state[6]; __pyx_result.id = __pyx_state[7]; __pyx_result.multicast = __pyx_state[8]; __pyx_result.names = __pyx_state[9]; __pyx_result.packets_data = __pyx_state[10]; __pyx_result.questions = __pyx_state[11]; __pyx_result.size = __pyx_state[12]; __pyx_result.state = __pyx_state[13]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 14 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[14])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||((void)PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->additionals);
  __Pyx_DECREF(__pyx_v___pyx_result->additionals);
  __pyx_v___pyx_result->additionals = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->allow_long);
  __Pyx_DECREF(__pyx_v___pyx_result->allow_long);
  __pyx_v___pyx_result->allow_long = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||((void)PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->answers);
  __Pyx_DECREF(__pyx_v___pyx_result->answers);
  __pyx_v___pyx_result->answers = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||((void)PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->authorities);
  __Pyx_DECREF(__pyx_v___pyx_result->authorities);
  __pyx_v___pyx_result->authorities = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||((void)PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->data);
  __Pyx_DECREF(__pyx_v___pyx_result->data);
  __pyx_v___pyx_result->data = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->finished);
  __Pyx_DECREF(__pyx_v___pyx_result->finished);
  __pyx_v___pyx_result->finished = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_unsigned_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->flags = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->id);
  __Pyx_DECREF(__pyx_v___pyx_result->id);
  __pyx_v___pyx_result->id = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->multicast = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->names);
  __Pyx_DECREF(__pyx_v___pyx_result->names);
  __pyx_v___pyx_result->names = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 10, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||((void)PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->packets_data);
  __Pyx_DECREF(__pyx_v___pyx_result->packets_data);
  __pyx_v___pyx_result->packets_data = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 11, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||((void)PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->questions);
  __Pyx_DECREF(__pyx_v___pyx_result->questions);
  __pyx_v___pyx_result->questions = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 12, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_unsigned_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->size = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 13, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->state);
  __Pyx_DECREF(__pyx_v___pyx_result->state);
  __pyx_v___pyx_result->state = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_DNSOutgoing__set_state(DNSOutgoing __pyx_result, tuple __pyx_state):
 *     __pyx_result.additionals = __pyx_state[0]; __pyx_result.allow_long = __pyx_state[1]; __pyx_result.answers = __pyx_state[2]; __pyx_result.authorities = __pyx_state[3]; __pyx_result.data = __pyx_state[4]; __pyx_result.finished = __pyx_state[5]; __pyx_result.flags = __pyx_state[6]; __pyx_result.id = __pyx_state[7]; __pyx_result.multicast = __pyx_state[8]; __pyx_result.names = __pyx_state[9]; __pyx_result.packets_data = __pyx_state[10]; __pyx_result.questions = __pyx_state[11]; __pyx_result.size = __pyx_state[12]; __pyx_result.state = __pyx_state[13]
 *     if len(__pyx_state) > 14 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[14])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 13, __pyx_L1_error)
  }
  __pyx_t_4 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_5 = ((__pyx_t_4 > 14) != 0);
  if (__pyx_t_5) {
  } else {
    __pyx_t_3 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_6 = (__pyx_t_5 != 0);
  __pyx_t_3 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_3) {

    /* "(tree fragment)":14
 *     __pyx_result.additionals = __pyx_state[0]; __pyx_result.allow_long = __pyx_state[1]; __pyx_result.answers = __pyx_state[2]; __pyx_result.authorities = __pyx_state[3]; __pyx_result.data = __pyx_state[4]; __pyx_result.finished = __pyx_state[5]; __pyx_result.flags = __pyx_state[6]; __pyx_result.id = __pyx_state[7]; __pyx_result.multicast = __pyx_state[8]; __pyx_result.names = __pyx_state[9]; __pyx_result.packets_data = __pyx_state[10]; __pyx_result.questions = __pyx_state[11]; __pyx_result.size = __pyx_state[12]; __pyx_result.state = __pyx_state[13]
 *     if len(__pyx_state) > 14 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[14])             # <<<<<<<<<<<<<<
 */
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_update); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 14, __pyx_L1_error)
    }
    __pyx_t_7 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 14, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
      }
    }
    __pyx_t_1 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_8, __pyx_t_9, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_7);
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_DNSOutgoing__set_state(DNSOutgoing __pyx_result, tuple __pyx_state):
 *     __pyx_result.additionals = __pyx_state[0]; __pyx_result.allow_long = __pyx_state[1]; __pyx_result.answers = __pyx_state[2]; __pyx_result.authorities = __pyx_state[3]; __pyx_result.data = __pyx_state[4]; __pyx_result.finished = __pyx_state[5]; __pyx_result.flags = __pyx_state[6]; __pyx_result.id = __pyx_state[7]; __pyx_result.multicast = __pyx_state[8]; __pyx_result.names = __pyx_state[9]; __pyx_result.packets_data = __pyx_state[10]; __pyx_result.questions = __pyx_state[11]; __pyx_result.size = __pyx_state[12]; __pyx_result.state = __pyx_state[13]
 *     if len(__pyx_state) > 14 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[14])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_DNSOutgoing__set_state(<DNSOutgoing> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_DNSOutgoing__set_state(DNSOutgoing __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.additionals = __pyx_state[0]; __pyx_result.allow_long = __pyx_state[1]; __pyx_result.answers = __pyx_state[2]; __pyx_result.authorities = __pyx_state[3]; __pyx_result.data = __pyx_state[4]; __pyx_result.finished = __pyx_state[5]; __pyx_result.flags = __pyx_state[6]; __pyx_result.id = __pyx_state[7]; __pyx_result.multicast = __pyx_state[8]; __pyx_result.names = __pyx_state[9]; __pyx_result.packets_data = __pyx_state[10]; __pyx_result.questions = __pyx_state[11]; __pyx_result.size = __pyx_state[12]; __pyx_result.state = __pyx_state[13]
 *     if len(__pyx_state) > 14 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("zeroconf._protocol.outgoing.__pyx_unpickle_DNSOutgoing__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static struct __pyx_vtabstruct_8zeroconf_9_protocol_8outgoing_DNSOutgoing __pyx_vtable_8zeroconf_9_protocol_8outgoing_DNSOutgoing;

static PyObject *__pyx_tp_new_8zeroconf_9_protocol_8outgoing_DNSOutgoing(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)o);
  p->__pyx_vtab = __pyx_vtabptr_8zeroconf_9_protocol_8outgoing_DNSOutgoing;
  p->finished = Py_None; Py_INCREF(Py_None);
  p->id = Py_None; Py_INCREF(Py_None);
  p->packets_data = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->names = Py_None; Py_INCREF(Py_None);
  p->data = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->allow_long = Py_None; Py_INCREF(Py_None);
  p->state = Py_None; Py_INCREF(Py_None);
  p->questions = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->answers = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->authorities = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->additionals = ((PyObject*)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_8zeroconf_9_protocol_8outgoing_DNSOutgoing(PyObject *o) {
  struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *p = (struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->finished);
  Py_CLEAR(p->id);
  Py_CLEAR(p->packets_data);
  Py_CLEAR(p->names);
  Py_CLEAR(p->data);
  Py_CLEAR(p->allow_long);
  Py_CLEAR(p->state);
  Py_CLEAR(p->questions);
  Py_CLEAR(p->answers);
  Py_CLEAR(p->authorities);
  Py_CLEAR(p->additionals);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_8zeroconf_9_protocol_8outgoing_DNSOutgoing(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *p = (struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)o;
  if (p->finished) {
    e = (*v)(p->finished, a); if (e) return e;
  }
  if (p->id) {
    e = (*v)(p->id, a); if (e) return e;
  }
  if (p->packets_data) {
    e = (*v)(p->packets_data, a); if (e) return e;
  }
  if (p->names) {
    e = (*v)(p->names, a); if (e) return e;
  }
  if (p->data) {
    e = (*v)(p->data, a); if (e) return e;
  }
  if (p->allow_long) {
    e = (*v)(p->allow_long, a); if (e) return e;
  }
  if (p->state) {
    e = (*v)(p->state, a); if (e) return e;
  }
  if (p->questions) {
    e = (*v)(p->questions, a); if (e) return e;
  }
  if (p->answers) {
    e = (*v)(p->answers, a); if (e) return e;
  }
  if (p->authorities) {
    e = (*v)(p->authorities, a); if (e) return e;
  }
  if (p->additionals) {
    e = (*v)(p->additionals, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_8zeroconf_9_protocol_8outgoing_DNSOutgoing(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *p = (struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)o;
  tmp = ((PyObject*)p->finished);
  p->finished = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->id);
  p->id = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->packets_data);
  p->packets_data = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->names);
  p->names = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->data);
  p->data = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->allow_long);
  p->allow_long = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->state);
  p->state = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->questions);
  p->questions = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->answers);
  p->answers = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->authorities);
  p->authorities = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->additionals);
  p->additionals = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_flags(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5flags_1__get__(o);
}

static int __pyx_setprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_flags(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5flags_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_finished(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_8finished_1__get__(o);
}

static int __pyx_setprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_finished(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_8finished_3__set__(o, v);
  }
  else {
    return __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_8finished_5__del__(o);
  }
}

static PyObject *__pyx_getprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_id(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_2id_1__get__(o);
}

static int __pyx_setprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_id(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_2id_3__set__(o, v);
  }
  else {
    return __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_2id_5__del__(o);
  }
}

static PyObject *__pyx_getprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_multicast(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_9multicast_1__get__(o);
}

static int __pyx_setprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_multicast(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_9multicast_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_packets_data(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_12packets_data_1__get__(o);
}

static int __pyx_setprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_packets_data(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_12packets_data_3__set__(o, v);
  }
  else {
    return __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_12packets_data_5__del__(o);
  }
}

static PyObject *__pyx_getprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_names(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5names_1__get__(o);
}

static int __pyx_setprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_names(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5names_3__set__(o, v);
  }
  else {
    return __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5names_5__del__(o);
  }
}

static PyObject *__pyx_getprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_data(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_4data_1__get__(o);
}

static int __pyx_setprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_data(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_4data_3__set__(o, v);
  }
  else {
    return __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_4data_5__del__(o);
  }
}

static PyObject *__pyx_getprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_size(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_4size_1__get__(o);
}

static int __pyx_setprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_size(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_4size_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_allow_long(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_10allow_long_1__get__(o);
}

static int __pyx_setprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_allow_long(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_10allow_long_3__set__(o, v);
  }
  else {
    return __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_10allow_long_5__del__(o);
  }
}

static PyObject *__pyx_getprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_state(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5state_1__get__(o);
}

static int __pyx_setprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_state(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5state_3__set__(o, v);
  }
  else {
    return __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5state_5__del__(o);
  }
}

static PyObject *__pyx_getprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_questions(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_9questions_1__get__(o);
}

static int __pyx_setprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_questions(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_9questions_3__set__(o, v);
  }
  else {
    return __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_9questions_5__del__(o);
  }
}

static PyObject *__pyx_getprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_answers(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_7answers_1__get__(o);
}

static int __pyx_setprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_answers(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_7answers_3__set__(o, v);
  }
  else {
    return __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_7answers_5__del__(o);
  }
}

static PyObject *__pyx_getprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_authorities(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_11authorities_1__get__(o);
}

static int __pyx_setprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_authorities(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_11authorities_3__set__(o, v);
  }
  else {
    return __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_11authorities_5__del__(o);
  }
}

static PyObject *__pyx_getprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_additionals(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_11additionals_1__get__(o);
}

static int __pyx_setprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_additionals(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_11additionals_3__set__(o, v);
  }
  else {
    return __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_11additionals_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_8zeroconf_9_protocol_8outgoing_DNSOutgoing[] = {
  {"is_query", (PyCFunction)__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_3is_query, METH_NOARGS, __pyx_doc_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_2is_query},
  {"is_response", (PyCFunction)__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5is_response, METH_NOARGS, __pyx_doc_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_4is_response},
  {"add_question", (PyCFunction)__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_9add_question, METH_O, __pyx_doc_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_8add_question},
  {"add_answer", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_11add_answer, METH_VARARGS|METH_KEYWORDS, __pyx_doc_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_10add_answer},
  {"add_answer_at_time", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_13add_answer_at_time, METH_VARARGS|METH_KEYWORDS, __pyx_doc_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_12add_answer_at_time},
  {"add_authorative_answer", (PyCFunction)__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_15add_authorative_answer, METH_O, __pyx_doc_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_14add_authorative_answer},
  {"add_additional_answer", (PyCFunction)__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_17add_additional_answer, METH_O, __pyx_doc_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_16add_additional_answer},
  {"add_question_or_one_cache", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_19add_question_or_one_cache, METH_VARARGS|METH_KEYWORDS, __pyx_doc_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_18add_question_or_one_cache},
  {"add_question_or_all_cache", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_21add_question_or_all_cache, METH_VARARGS|METH_KEYWORDS, __pyx_doc_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_20add_question_or_all_cache},
  {"write_short", (PyCFunction)__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_23write_short, METH_O, __pyx_doc_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_22write_short},
  {"write_string", (PyCFunction)__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_25write_string, METH_O, __pyx_doc_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_24write_string},
  {"_write_utf", (PyCFunction)__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_27_write_utf, METH_O, __pyx_doc_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_26_write_utf},
  {"write_character_string", (PyCFunction)__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_29write_character_string, METH_O, 0},
  {"write_name", (PyCFunction)__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_31write_name, METH_O, __pyx_doc_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_30write_name},
  {"_write_ttl", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_33_write_ttl, METH_VARARGS|METH_KEYWORDS, __pyx_doc_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_32_write_ttl},
  {"_write_record", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_35_write_record, METH_VARARGS|METH_KEYWORDS, __pyx_doc_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_34_write_record},
  {"packets", (PyCFunction)__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_37packets, METH_NOARGS, __pyx_doc_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_36packets},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_39__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_41__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_8zeroconf_9_protocol_8outgoing_DNSOutgoing[] = {
  {(char *)"flags", __pyx_getprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_flags, __pyx_setprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_flags, (char *)0, 0},
  {(char *)"finished", __pyx_getprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_finished, __pyx_setprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_finished, (char *)0, 0},
  {(char *)"id", __pyx_getprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_id, __pyx_setprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_id, (char *)0, 0},
  {(char *)"multicast", __pyx_getprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_multicast, __pyx_setprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_multicast, (char *)0, 0},
  {(char *)"packets_data", __pyx_getprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_packets_data, __pyx_setprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_packets_data, (char *)0, 0},
  {(char *)"names", __pyx_getprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_names, __pyx_setprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_names, (char *)0, 0},
  {(char *)"data", __pyx_getprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_data, __pyx_setprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_data, (char *)0, 0},
  {(char *)"size", __pyx_getprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_size, __pyx_setprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_size, (char *)0, 0},
  {(char *)"allow_long", __pyx_getprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_allow_long, __pyx_setprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_allow_long, (char *)0, 0},
  {(char *)"state", __pyx_getprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_state, __pyx_setprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_state, (char *)0, 0},
  {(char *)"questions", __pyx_getprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_questions, __pyx_setprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_questions, (char *)0, 0},
  {(char *)"answers", __pyx_getprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_answers, __pyx_setprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_answers, (char *)0, 0},
  {(char *)"authorities", __pyx_getprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_authorities, __pyx_setprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_authorities, (char *)0, 0},
  {(char *)"additionals", __pyx_getprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_additionals, __pyx_setprop_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_additionals, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_8zeroconf_9_protocol_8outgoing_DNSOutgoing = {
  PyVarObject_HEAD_INIT(0, 0)
  "zeroconf._protocol.outgoing.DNSOutgoing", /*tp_name*/
  sizeof(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_8zeroconf_9_protocol_8outgoing_DNSOutgoing, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_7__repr__, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "Object representation of an outgoing packet", /*tp_doc*/
  __pyx_tp_traverse_8zeroconf_9_protocol_8outgoing_DNSOutgoing, /*tp_traverse*/
  __pyx_tp_clear_8zeroconf_9_protocol_8outgoing_DNSOutgoing, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_8zeroconf_9_protocol_8outgoing_DNSOutgoing, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_8zeroconf_9_protocol_8outgoing_DNSOutgoing, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_8zeroconf_9_protocol_8outgoing_DNSOutgoing, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};

static struct __pyx_obj_8zeroconf_9_protocol_8outgoing___pyx_scope_struct___write_record *__pyx_freelist_8zeroconf_9_protocol_8outgoing___pyx_scope_struct___write_record[8];
static int __pyx_freecount_8zeroconf_9_protocol_8outgoing___pyx_scope_struct___write_record = 0;

static PyObject *__pyx_tp_new_8zeroconf_9_protocol_8outgoing___pyx_scope_struct___write_record(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_8zeroconf_9_protocol_8outgoing___pyx_scope_struct___write_record > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_8zeroconf_9_protocol_8outgoing___pyx_scope_struct___write_record)))) {
    o = (PyObject*)__pyx_freelist_8zeroconf_9_protocol_8outgoing___pyx_scope_struct___write_record[--__pyx_freecount_8zeroconf_9_protocol_8outgoing___pyx_scope_struct___write_record];
    memset(o, 0, sizeof(struct __pyx_obj_8zeroconf_9_protocol_8outgoing___pyx_scope_struct___write_record));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_8zeroconf_9_protocol_8outgoing___pyx_scope_struct___write_record(PyObject *o) {
  struct __pyx_obj_8zeroconf_9_protocol_8outgoing___pyx_scope_struct___write_record *p = (struct __pyx_obj_8zeroconf_9_protocol_8outgoing___pyx_scope_struct___write_record *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_self);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_8zeroconf_9_protocol_8outgoing___pyx_scope_struct___write_record < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_8zeroconf_9_protocol_8outgoing___pyx_scope_struct___write_record)))) {
    __pyx_freelist_8zeroconf_9_protocol_8outgoing___pyx_scope_struct___write_record[__pyx_freecount_8zeroconf_9_protocol_8outgoing___pyx_scope_struct___write_record++] = ((struct __pyx_obj_8zeroconf_9_protocol_8outgoing___pyx_scope_struct___write_record *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_8zeroconf_9_protocol_8outgoing___pyx_scope_struct___write_record(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_8zeroconf_9_protocol_8outgoing___pyx_scope_struct___write_record *p = (struct __pyx_obj_8zeroconf_9_protocol_8outgoing___pyx_scope_struct___write_record *)o;
  if (p->__pyx_v_self) {
    e = (*v)(((PyObject *)p->__pyx_v_self), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_8zeroconf_9_protocol_8outgoing___pyx_scope_struct___write_record(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_8zeroconf_9_protocol_8outgoing___pyx_scope_struct___write_record *p = (struct __pyx_obj_8zeroconf_9_protocol_8outgoing___pyx_scope_struct___write_record *)o;
  tmp = ((PyObject*)p->__pyx_v_self);
  p->__pyx_v_self = ((struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyTypeObject __pyx_type_8zeroconf_9_protocol_8outgoing___pyx_scope_struct___write_record = {
  PyVarObject_HEAD_INIT(0, 0)
  "zeroconf._protocol.outgoing.__pyx_scope_struct___write_record", /*tp_name*/
  sizeof(struct __pyx_obj_8zeroconf_9_protocol_8outgoing___pyx_scope_struct___write_record), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_8zeroconf_9_protocol_8outgoing___pyx_scope_struct___write_record, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_8zeroconf_9_protocol_8outgoing___pyx_scope_struct___write_record, /*tp_traverse*/
  __pyx_tp_clear_8zeroconf_9_protocol_8outgoing___pyx_scope_struct___write_record, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_8zeroconf_9_protocol_8outgoing___pyx_scope_struct___write_record, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};

static struct __pyx_obj_8zeroconf_9_protocol_8outgoing___pyx_scope_struct_1_genexpr *__pyx_freelist_8zeroconf_9_protocol_8outgoing___pyx_scope_struct_1_genexpr[8];
static int __pyx_freecount_8zeroconf_9_protocol_8outgoing___pyx_scope_struct_1_genexpr = 0;

static PyObject *__pyx_tp_new_8zeroconf_9_protocol_8outgoing___pyx_scope_struct_1_genexpr(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_8zeroconf_9_protocol_8outgoing___pyx_scope_struct_1_genexpr > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_8zeroconf_9_protocol_8outgoing___pyx_scope_struct_1_genexpr)))) {
    o = (PyObject*)__pyx_freelist_8zeroconf_9_protocol_8outgoing___pyx_scope_struct_1_genexpr[--__pyx_freecount_8zeroconf_9_protocol_8outgoing___pyx_scope_struct_1_genexpr];
    memset(o, 0, sizeof(struct __pyx_obj_8zeroconf_9_protocol_8outgoing___pyx_scope_struct_1_genexpr));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_8zeroconf_9_protocol_8outgoing___pyx_scope_struct_1_genexpr(PyObject *o) {
  struct __pyx_obj_8zeroconf_9_protocol_8outgoing___pyx_scope_struct_1_genexpr *p = (struct __pyx_obj_8zeroconf_9_protocol_8outgoing___pyx_scope_struct_1_genexpr *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_outer_scope);
  Py_CLEAR(p->__pyx_v_d);
  Py_CLEAR(p->__pyx_t_0);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_8zeroconf_9_protocol_8outgoing___pyx_scope_struct_1_genexpr < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_8zeroconf_9_protocol_8outgoing___pyx_scope_struct_1_genexpr)))) {
    __pyx_freelist_8zeroconf_9_protocol_8outgoing___pyx_scope_struct_1_genexpr[__pyx_freecount_8zeroconf_9_protocol_8outgoing___pyx_scope_struct_1_genexpr++] = ((struct __pyx_obj_8zeroconf_9_protocol_8outgoing___pyx_scope_struct_1_genexpr *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_8zeroconf_9_protocol_8outgoing___pyx_scope_struct_1_genexpr(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_8zeroconf_9_protocol_8outgoing___pyx_scope_struct_1_genexpr *p = (struct __pyx_obj_8zeroconf_9_protocol_8outgoing___pyx_scope_struct_1_genexpr *)o;
  if (p->__pyx_outer_scope) {
    e = (*v)(((PyObject *)p->__pyx_outer_scope), a); if (e) return e;
  }
  if (p->__pyx_v_d) {
    e = (*v)(p->__pyx_v_d, a); if (e) return e;
  }
  if (p->__pyx_t_0) {
    e = (*v)(p->__pyx_t_0, a); if (e) return e;
  }
  return 0;
}

static PyTypeObject __pyx_type_8zeroconf_9_protocol_8outgoing___pyx_scope_struct_1_genexpr = {
  PyVarObject_HEAD_INIT(0, 0)
  "zeroconf._protocol.outgoing.__pyx_scope_struct_1_genexpr", /*tp_name*/
  sizeof(struct __pyx_obj_8zeroconf_9_protocol_8outgoing___pyx_scope_struct_1_genexpr), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_8zeroconf_9_protocol_8outgoing___pyx_scope_struct_1_genexpr, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_8zeroconf_9_protocol_8outgoing___pyx_scope_struct_1_genexpr, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_8zeroconf_9_protocol_8outgoing___pyx_scope_struct_1_genexpr, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_outgoing(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_outgoing},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "outgoing",
    __pyx_k_Multicast_DNS_Service_Discovery, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_kp_u_, __pyx_k_, sizeof(__pyx_k_), 0, 1, 0, 0},
  {&__pyx_n_s_CLASS_UNIQUE, __pyx_k_CLASS_UNIQUE, sizeof(__pyx_k_CLASS_UNIQUE), 0, 0, 1, 1},
  {&__pyx_n_s_DEBUG, __pyx_k_DEBUG, sizeof(__pyx_k_DEBUG), 0, 0, 1, 1},
  {&__pyx_n_s_DNSCache, __pyx_k_DNSCache, sizeof(__pyx_k_DNSCache), 0, 0, 1, 1},
  {&__pyx_n_s_DNSIncoming, __pyx_k_DNSIncoming, sizeof(__pyx_k_DNSIncoming), 0, 0, 1, 1},
  {&__pyx_n_u_DNSIncoming, __pyx_k_DNSIncoming, sizeof(__pyx_k_DNSIncoming), 0, 1, 0, 1},
  {&__pyx_n_s_DNSOutgoing, __pyx_k_DNSOutgoing, sizeof(__pyx_k_DNSOutgoing), 0, 0, 1, 1},
  {&__pyx_n_s_DNSOutgoing___reduce_cython, __pyx_k_DNSOutgoing___reduce_cython, sizeof(__pyx_k_DNSOutgoing___reduce_cython), 0, 0, 1, 1},
  {&__pyx_n_s_DNSOutgoing___setstate_cython, __pyx_k_DNSOutgoing___setstate_cython, sizeof(__pyx_k_DNSOutgoing___setstate_cython), 0, 0, 1, 1},
  {&__pyx_n_s_DNSOutgoing__write_record, __pyx_k_DNSOutgoing__write_record, sizeof(__pyx_k_DNSOutgoing__write_record), 0, 0, 1, 1},
  {&__pyx_n_s_DNSOutgoing__write_ttl, __pyx_k_DNSOutgoing__write_ttl, sizeof(__pyx_k_DNSOutgoing__write_ttl), 0, 0, 1, 1},
  {&__pyx_n_s_DNSOutgoing__write_utf, __pyx_k_DNSOutgoing__write_utf, sizeof(__pyx_k_DNSOutgoing__write_utf), 0, 0, 1, 1},
  {&__pyx_n_s_DNSOutgoing_add_additional_answe, __pyx_k_DNSOutgoing_add_additional_answe, sizeof(__pyx_k_DNSOutgoing_add_additional_answe), 0, 0, 1, 1},
  {&__pyx_n_s_DNSOutgoing_add_answer, __pyx_k_DNSOutgoing_add_answer, sizeof(__pyx_k_DNSOutgoing_add_answer), 0, 0, 1, 1},
  {&__pyx_n_s_DNSOutgoing_add_answer_at_time, __pyx_k_DNSOutgoing_add_answer_at_time, sizeof(__pyx_k_DNSOutgoing_add_answer_at_time), 0, 0, 1, 1},
  {&__pyx_n_s_DNSOutgoing_add_authorative_answ, __pyx_k_DNSOutgoing_add_authorative_answ, sizeof(__pyx_k_DNSOutgoing_add_authorative_answ), 0, 0, 1, 1},
  {&__pyx_n_s_DNSOutgoing_add_question, __pyx_k_DNSOutgoing_add_question, sizeof(__pyx_k_DNSOutgoing_add_question), 0, 0, 1, 1},
  {&__pyx_n_s_DNSOutgoing_add_question_or_all, __pyx_k_DNSOutgoing_add_question_or_all, sizeof(__pyx_k_DNSOutgoing_add_question_or_all), 0, 0, 1, 1},
  {&__pyx_n_s_DNSOutgoing_add_question_or_one, __pyx_k_DNSOutgoing_add_question_or_one, sizeof(__pyx_k_DNSOutgoing_add_question_or_one), 0, 0, 1, 1},
  {&__pyx_n_s_DNSOutgoing_is_query, __pyx_k_DNSOutgoing_is_query, sizeof(__pyx_k_DNSOutgoing_is_query), 0, 0, 1, 1},
  {&__pyx_n_s_DNSOutgoing_is_response, __pyx_k_DNSOutgoing_is_response, sizeof(__pyx_k_DNSOutgoing_is_response), 0, 0, 1, 1},
  {&__pyx_n_s_DNSOutgoing_packets, __pyx_k_DNSOutgoing_packets, sizeof(__pyx_k_DNSOutgoing_packets), 0, 0, 1, 1},
  {&__pyx_kp_u_DNSOutgoing_s, __pyx_k_DNSOutgoing_s, sizeof(__pyx_k_DNSOutgoing_s), 0, 1, 0, 0},
  {&__pyx_n_s_DNSOutgoing_write_character_stri, __pyx_k_DNSOutgoing_write_character_stri, sizeof(__pyx_k_DNSOutgoing_write_character_stri), 0, 0, 1, 1},
  {&__pyx_n_s_DNSOutgoing_write_name, __pyx_k_DNSOutgoing_write_name, sizeof(__pyx_k_DNSOutgoing_write_name), 0, 0, 1, 1},
  {&__pyx_n_s_DNSOutgoing_write_short, __pyx_k_DNSOutgoing_write_short, sizeof(__pyx_k_DNSOutgoing_write_short), 0, 0, 1, 1},
  {&__pyx_n_s_DNSOutgoing_write_string, __pyx_k_DNSOutgoing_write_string, sizeof(__pyx_k_DNSOutgoing_write_string), 0, 0, 1, 1},
  {&__pyx_n_s_DNSPointer, __pyx_k_DNSPointer, sizeof(__pyx_k_DNSPointer), 0, 0, 1, 1},
  {&__pyx_n_s_DNSQuestion, __pyx_k_DNSQuestion, sizeof(__pyx_k_DNSQuestion), 0, 0, 1, 1},
  {&__pyx_n_s_DNSRecord, __pyx_k_DNSRecord, sizeof(__pyx_k_DNSRecord), 0, 0, 1, 1},
  {&__pyx_n_s_DNS_PACKET_HEADER_LEN, __pyx_k_DNS_PACKET_HEADER_LEN, sizeof(__pyx_k_DNS_PACKET_HEADER_LEN), 0, 0, 1, 1},
  {&__pyx_n_s_Dict, __pyx_k_Dict, sizeof(__pyx_k_Dict), 0, 0, 1, 1},
  {&__pyx_n_s_Enum, __pyx_k_Enum, sizeof(__pyx_k_Enum), 0, 0, 1, 1},
  {&__pyx_n_s_FLAGS_QR_MASK, __pyx_k_FLAGS_QR_MASK, sizeof(__pyx_k_FLAGS_QR_MASK), 0, 0, 1, 1},
  {&__pyx_n_s_FLAGS_QR_QUERY, __pyx_k_FLAGS_QR_QUERY, sizeof(__pyx_k_FLAGS_QR_QUERY), 0, 0, 1, 1},
  {&__pyx_n_s_FLAGS_QR_RESPONSE, __pyx_k_FLAGS_QR_RESPONSE, sizeof(__pyx_k_FLAGS_QR_RESPONSE), 0, 0, 1, 1},
  {&__pyx_n_s_FLAGS_TC, __pyx_k_FLAGS_TC, sizeof(__pyx_k_FLAGS_TC), 0, 0, 1, 1},
  {&__pyx_kp_s_Incompatible_checksums_0x_x_vs_0, __pyx_k_Incompatible_checksums_0x_x_vs_0, sizeof(__pyx_k_Incompatible_checksums_0x_x_vs_0), 0, 0, 1, 0},
  {&__pyx_n_s_List, __pyx_k_List, sizeof(__pyx_k_List), 0, 0, 1, 1},
  {&__pyx_n_s_MAX_MSG_ABSOLUTE, __pyx_k_MAX_MSG_ABSOLUTE, sizeof(__pyx_k_MAX_MSG_ABSOLUTE), 0, 0, 1, 1},
  {&__pyx_n_s_MAX_MSG_TYPICAL, __pyx_k_MAX_MSG_TYPICAL, sizeof(__pyx_k_MAX_MSG_TYPICAL), 0, 0, 1, 1},
  {&__pyx_n_s_NamePartTooLongException, __pyx_k_NamePartTooLongException, sizeof(__pyx_k_NamePartTooLongException), 0, 0, 1, 1},
  {&__pyx_n_s_Optional, __pyx_k_Optional, sizeof(__pyx_k_Optional), 0, 0, 1, 1},
  {&__pyx_n_s_PickleError, __pyx_k_PickleError, sizeof(__pyx_k_PickleError), 0, 0, 1, 1},
  {&__pyx_kp_u_Reached_data_limit_size_d_limit, __pyx_k_Reached_data_limit_size_d_limit, sizeof(__pyx_k_Reached_data_limit_size_d_limit), 0, 1, 0, 0},
  {&__pyx_n_s_Sequence, __pyx_k_Sequence, sizeof(__pyx_k_Sequence), 0, 0, 1, 1},
  {&__pyx_kp_u_Setting_TC_flag, __pyx_k_Setting_TC_flag, sizeof(__pyx_k_Setting_TC_flag), 0, 1, 0, 0},
  {&__pyx_n_s_State, __pyx_k_State, sizeof(__pyx_k_State), 0, 0, 1, 1},
  {&__pyx_n_s_Tuple, __pyx_k_Tuple, sizeof(__pyx_k_Tuple), 0, 0, 1, 1},
  {&__pyx_n_s_Union, __pyx_k_Union, sizeof(__pyx_k_Union), 0, 0, 1, 1},
  {&__pyx_kp_u__5, __pyx_k__5, sizeof(__pyx_k__5), 0, 1, 0, 0},
  {&__pyx_kp_b__6, __pyx_k__6, sizeof(__pyx_k__6), 0, 0, 0, 0},
  {&__pyx_n_s_add_additional_answer, __pyx_k_add_additional_answer, sizeof(__pyx_k_add_additional_answer), 0, 0, 1, 1},
  {&__pyx_n_s_add_answer, __pyx_k_add_answer, sizeof(__pyx_k_add_answer), 0, 0, 1, 1},
  {&__pyx_n_s_add_answer_at_time, __pyx_k_add_answer_at_time, sizeof(__pyx_k_add_answer_at_time), 0, 0, 1, 1},
  {&__pyx_n_s_add_authorative_answer, __pyx_k_add_authorative_answer, sizeof(__pyx_k_add_authorative_answer), 0, 0, 1, 1},
  {&__pyx_n_s_add_question, __pyx_k_add_question, sizeof(__pyx_k_add_question), 0, 0, 1, 1},
  {&__pyx_n_s_add_question_or_all_cache, __pyx_k_add_question_or_all_cache, sizeof(__pyx_k_add_question_or_all_cache), 0, 0, 1, 1},
  {&__pyx_n_s_add_question_or_one_cache, __pyx_k_add_question_or_one_cache, sizeof(__pyx_k_add_question_or_one_cache), 0, 0, 1, 1},
  {&__pyx_n_u_additionals, __pyx_k_additionals, sizeof(__pyx_k_additionals), 0, 1, 0, 1},
  {&__pyx_kp_u_additionals_s, __pyx_k_additionals_s, sizeof(__pyx_k_additionals_s), 0, 1, 0, 0},
  {&__pyx_n_u_allow_long, __pyx_k_allow_long, sizeof(__pyx_k_allow_long), 0, 1, 0, 1},
  {&__pyx_n_u_answers, __pyx_k_answers, sizeof(__pyx_k_answers), 0, 1, 0, 1},
  {&__pyx_kp_u_answers_s, __pyx_k_answers_s, sizeof(__pyx_k_answers_s), 0, 1, 0, 0},
  {&__pyx_n_s_args, __pyx_k_args, sizeof(__pyx_k_args), 0, 0, 1, 1},
  {&__pyx_n_u_authorities, __pyx_k_authorities, sizeof(__pyx_k_authorities), 0, 1, 0, 1},
  {&__pyx_kp_u_authorities_s, __pyx_k_authorities_s, sizeof(__pyx_k_authorities_s), 0, 1, 0, 0},
  {&__pyx_n_u_big, __pyx_k_big, sizeof(__pyx_k_big), 0, 1, 0, 1},
  {&__pyx_n_u_bytes, __pyx_k_bytes, sizeof(__pyx_k_bytes), 0, 1, 0, 1},
  {&__pyx_n_s_cache, __pyx_k_cache, sizeof(__pyx_k_cache), 0, 0, 1, 1},
  {&__pyx_n_s_cache_2, __pyx_k_cache_2, sizeof(__pyx_k_cache_2), 0, 0, 1, 1},
  {&__pyx_n_s_cached_entries, __pyx_k_cached_entries, sizeof(__pyx_k_cached_entries), 0, 0, 1, 1},
  {&__pyx_n_s_cached_entry, __pyx_k_cached_entry, sizeof(__pyx_k_cached_entry), 0, 0, 1, 1},
  {&__pyx_n_s_class, __pyx_k_class, sizeof(__pyx_k_class), 0, 0, 1, 1},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_close, __pyx_k_close, sizeof(__pyx_k_close), 0, 0, 1, 1},
  {&__pyx_n_s_const, __pyx_k_const, sizeof(__pyx_k_const), 0, 0, 1, 1},
  {&__pyx_n_s_count, __pyx_k_count, sizeof(__pyx_k_count), 0, 0, 1, 1},
  {&__pyx_n_u_data, __pyx_k_data, sizeof(__pyx_k_data), 0, 1, 0, 1},
  {&__pyx_n_s_debug, __pyx_k_debug, sizeof(__pyx_k_debug), 0, 0, 1, 1},
  {&__pyx_n_s_dict, __pyx_k_dict, sizeof(__pyx_k_dict), 0, 0, 1, 1},
  {&__pyx_n_s_dict_2, __pyx_k_dict_2, sizeof(__pyx_k_dict_2), 0, 0, 1, 1},
  {&__pyx_n_s_dns, __pyx_k_dns, sizeof(__pyx_k_dns), 0, 0, 1, 1},
  {&__pyx_n_s_doc, __pyx_k_doc, sizeof(__pyx_k_doc), 0, 0, 1, 1},
  {&__pyx_n_s_enum, __pyx_k_enum, sizeof(__pyx_k_enum), 0, 0, 1, 1},
  {&__pyx_n_s_exceptions, __pyx_k_exceptions, sizeof(__pyx_k_exceptions), 0, 0, 1, 1},
  {&__pyx_n_s_finished, __pyx_k_finished, sizeof(__pyx_k_finished), 0, 0, 1, 1},
  {&__pyx_n_u_finished, __pyx_k_finished, sizeof(__pyx_k_finished), 0, 1, 0, 1},
  {&__pyx_n_s_flags, __pyx_k_flags, sizeof(__pyx_k_flags), 0, 0, 1, 1},
  {&__pyx_n_u_flags, __pyx_k_flags, sizeof(__pyx_k_flags), 0, 1, 0, 1},
  {&__pyx_kp_u_flags_s, __pyx_k_flags_s, sizeof(__pyx_k_flags_s), 0, 1, 0, 0},
  {&__pyx_n_u_float, __pyx_k_float, sizeof(__pyx_k_float), 0, 1, 0, 1},
  {&__pyx_n_s_genexpr, __pyx_k_genexpr, sizeof(__pyx_k_genexpr), 0, 0, 1, 1},
  {&__pyx_n_s_get, __pyx_k_get, sizeof(__pyx_k_get), 0, 0, 1, 1},
  {&__pyx_n_s_get_all_by_details, __pyx_k_get_all_by_details, sizeof(__pyx_k_get_all_by_details), 0, 0, 1, 1},
  {&__pyx_n_s_get_by_details, __pyx_k_get_by_details, sizeof(__pyx_k_get_by_details), 0, 0, 1, 1},
  {&__pyx_n_s_get_remaining_ttl, __pyx_k_get_remaining_ttl, sizeof(__pyx_k_get_remaining_ttl), 0, 0, 1, 1},
  {&__pyx_n_s_getstate, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
  {&__pyx_n_s_id, __pyx_k_id, sizeof(__pyx_k_id), 0, 0, 1, 1},
  {&__pyx_n_u_id_2, __pyx_k_id_2, sizeof(__pyx_k_id_2), 0, 1, 0, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_incoming, __pyx_k_incoming, sizeof(__pyx_k_incoming), 0, 0, 1, 1},
  {&__pyx_n_s_index, __pyx_k_index, sizeof(__pyx_k_index), 0, 0, 1, 1},
  {&__pyx_n_s_init, __pyx_k_init, sizeof(__pyx_k_init), 0, 0, 1, 1},
  {&__pyx_n_s_inp, __pyx_k_inp, sizeof(__pyx_k_inp), 0, 0, 1, 1},
  {&__pyx_n_u_int, __pyx_k_int, sizeof(__pyx_k_int), 0, 1, 0, 1},
  {&__pyx_n_s_isEnabledFor, __pyx_k_isEnabledFor, sizeof(__pyx_k_isEnabledFor), 0, 0, 1, 1},
  {&__pyx_n_s_is_expired, __pyx_k_is_expired, sizeof(__pyx_k_is_expired), 0, 0, 1, 1},
  {&__pyx_n_s_is_query, __pyx_k_is_query, sizeof(__pyx_k_is_query), 0, 0, 1, 1},
  {&__pyx_n_s_is_response, __pyx_k_is_response, sizeof(__pyx_k_is_response), 0, 0, 1, 1},
  {&__pyx_n_s_items, __pyx_k_items, sizeof(__pyx_k_items), 0, 0, 1, 1},
  {&__pyx_n_s_join, __pyx_k_join, sizeof(__pyx_k_join), 0, 0, 1, 1},
  {&__pyx_n_s_label, __pyx_k_label, sizeof(__pyx_k_label), 0, 0, 1, 1},
  {&__pyx_n_s_labels, __pyx_k_labels, sizeof(__pyx_k_labels), 0, 0, 1, 1},
  {&__pyx_n_s_length, __pyx_k_length, sizeof(__pyx_k_length), 0, 0, 1, 1},
  {&__pyx_kp_u_lengths_questions_d_answers_d_au, __pyx_k_lengths_questions_d_answers_d_au, sizeof(__pyx_k_lengths_questions_d_answers_d_au), 0, 1, 0, 0},
  {&__pyx_n_s_log, __pyx_k_log, sizeof(__pyx_k_log), 0, 0, 1, 1},
  {&__pyx_n_s_logger, __pyx_k_logger, sizeof(__pyx_k_logger), 0, 0, 1, 1},
  {&__pyx_n_s_logging, __pyx_k_logging, sizeof(__pyx_k_logging), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_metaclass, __pyx_k_metaclass, sizeof(__pyx_k_metaclass), 0, 0, 1, 1},
  {&__pyx_n_s_module, __pyx_k_module, sizeof(__pyx_k_module), 0, 0, 1, 1},
  {&__pyx_n_s_multicast, __pyx_k_multicast, sizeof(__pyx_k_multicast), 0, 0, 1, 1},
  {&__pyx_n_u_multicast, __pyx_k_multicast, sizeof(__pyx_k_multicast), 0, 1, 0, 1},
  {&__pyx_kp_u_multicast_s, __pyx_k_multicast_s, sizeof(__pyx_k_multicast_s), 0, 1, 0, 0},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_s_name_2, __pyx_k_name_2, sizeof(__pyx_k_name_2), 0, 0, 1, 1},
  {&__pyx_n_s_name_length, __pyx_k_name_length, sizeof(__pyx_k_name_length), 0, 0, 1, 1},
  {&__pyx_n_u_names, __pyx_k_names, sizeof(__pyx_k_names), 0, 1, 0, 1},
  {&__pyx_n_s_new, __pyx_k_new, sizeof(__pyx_k_new), 0, 0, 1, 1},
  {&__pyx_n_s_now, __pyx_k_now, sizeof(__pyx_k_now), 0, 0, 1, 1},
  {&__pyx_kp_u_now_offsets_questions_d_answers, __pyx_k_now_offsets_questions_d_answers, sizeof(__pyx_k_now_offsets_questions_d_answers), 0, 1, 0, 0},
  {&__pyx_kp_u_offsets_questions_d_answers_d_au, __pyx_k_offsets_questions_d_answers_d_au, sizeof(__pyx_k_offsets_questions_d_answers_d_au), 0, 1, 0, 0},
  {&__pyx_n_s_packets, __pyx_k_packets, sizeof(__pyx_k_packets), 0, 0, 1, 1},
  {&__pyx_n_u_packets_data, __pyx_k_packets_data, sizeof(__pyx_k_packets_data), 0, 1, 0, 1},
  {&__pyx_kp_u_packets_made_no_progress_adding, __pyx_k_packets_made_no_progress_adding, sizeof(__pyx_k_packets_made_no_progress_adding), 0, 1, 0, 0},
  {&__pyx_n_s_pickle, __pyx_k_pickle, sizeof(__pyx_k_pickle), 0, 0, 1, 1},
  {&__pyx_n_s_prepare, __pyx_k_prepare, sizeof(__pyx_k_prepare), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_PickleError, __pyx_k_pyx_PickleError, sizeof(__pyx_k_pyx_PickleError), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_capi, __pyx_k_pyx_capi, sizeof(__pyx_k_pyx_capi), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_checksum, __pyx_k_pyx_checksum, sizeof(__pyx_k_pyx_checksum), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_result, __pyx_k_pyx_result, sizeof(__pyx_k_pyx_result), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_state, __pyx_k_pyx_state, sizeof(__pyx_k_pyx_state), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_type, __pyx_k_pyx_type, sizeof(__pyx_k_pyx_type), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_DNSOutgoing, __pyx_k_pyx_unpickle_DNSOutgoing, sizeof(__pyx_k_pyx_unpickle_DNSOutgoing), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_s_qualname, __pyx_k_qualname, sizeof(__pyx_k_qualname), 0, 0, 1, 1},
  {&__pyx_n_u_questions, __pyx_k_questions, sizeof(__pyx_k_questions), 0, 1, 0, 1},
  {&__pyx_kp_u_questions_s, __pyx_k_questions_s, sizeof(__pyx_k_questions_s), 0, 1, 0, 0},
  {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
  {&__pyx_n_s_record, __pyx_k_record, sizeof(__pyx_k_record), 0, 0, 1, 1},
  {&__pyx_n_s_reduce, __pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_cython, __pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_ex, __pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 0, 1, 1},
  {&__pyx_n_s_return, __pyx_k_return, sizeof(__pyx_k_return), 0, 0, 1, 1},
  {&__pyx_n_s_s, __pyx_k_s, sizeof(__pyx_k_s), 0, 0, 1, 1},
  {&__pyx_n_s_self, __pyx_k_self, sizeof(__pyx_k_self), 0, 0, 1, 1},
  {&__pyx_n_s_send, __pyx_k_send, sizeof(__pyx_k_send), 0, 0, 1, 1},
  {&__pyx_n_s_setstate, __pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 0, 1, 1},
  {&__pyx_n_s_setstate_cython, __pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 0, 1, 1},
  {&__pyx_n_u_size, __pyx_k_size, sizeof(__pyx_k_size), 0, 1, 0, 1},
  {&__pyx_n_s_slots, __pyx_k_slots, sizeof(__pyx_k_slots), 0, 0, 1, 1},
  {&__pyx_kp_s_src_zeroconf__protocol_outgoing, __pyx_k_src_zeroconf__protocol_outgoing, sizeof(__pyx_k_src_zeroconf__protocol_outgoing), 0, 0, 1, 0},
  {&__pyx_n_s_start_data_length, __pyx_k_start_data_length, sizeof(__pyx_k_start_data_length), 0, 0, 1, 1},
  {&__pyx_n_s_start_size, __pyx_k_start_size, sizeof(__pyx_k_start_size), 0, 0, 1, 1},
  {&__pyx_n_s_state, __pyx_k_state, sizeof(__pyx_k_state), 0, 0, 1, 1},
  {&__pyx_n_u_state, __pyx_k_state, sizeof(__pyx_k_state), 0, 1, 0, 1},
  {&__pyx_kp_s_stringsource, __pyx_k_stringsource, sizeof(__pyx_k_stringsource), 0, 0, 1, 0},
  {&__pyx_n_s_sum, __pyx_k_sum, sizeof(__pyx_k_sum), 0, 0, 1, 1},
  {&__pyx_n_s_suppressed_by, __pyx_k_suppressed_by, sizeof(__pyx_k_suppressed_by), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_throw, __pyx_k_throw, sizeof(__pyx_k_throw), 0, 0, 1, 1},
  {&__pyx_n_s_to_bytes, __pyx_k_to_bytes, sizeof(__pyx_k_to_bytes), 0, 0, 1, 1},
  {&__pyx_n_s_ttl, __pyx_k_ttl, sizeof(__pyx_k_ttl), 0, 0, 1, 1},
  {&__pyx_n_s_type, __pyx_k_type, sizeof(__pyx_k_type), 0, 0, 1, 1},
  {&__pyx_n_s_type_2, __pyx_k_type_2, sizeof(__pyx_k_type_2), 0, 0, 1, 1},
  {&__pyx_n_s_typing, __pyx_k_typing, sizeof(__pyx_k_typing), 0, 0, 1, 1},
  {&__pyx_n_u_unicode, __pyx_k_unicode, sizeof(__pyx_k_unicode), 0, 1, 0, 1},
  {&__pyx_n_s_unique, __pyx_k_unique, sizeof(__pyx_k_unique), 0, 0, 1, 1},
  {&__pyx_n_s_update, __pyx_k_update, sizeof(__pyx_k_update), 0, 0, 1, 1},
  {&__pyx_n_s_use_setstate, __pyx_k_use_setstate, sizeof(__pyx_k_use_setstate), 0, 0, 1, 1},
  {&__pyx_n_s_utfstr, __pyx_k_utfstr, sizeof(__pyx_k_utfstr), 0, 0, 1, 1},
  {&__pyx_n_s_value, __pyx_k_value, sizeof(__pyx_k_value), 0, 0, 1, 1},
  {&__pyx_n_s_warning, __pyx_k_warning, sizeof(__pyx_k_warning), 0, 0, 1, 1},
  {&__pyx_n_s_write, __pyx_k_write, sizeof(__pyx_k_write), 0, 0, 1, 1},
  {&__pyx_n_s_write_character_string, __pyx_k_write_character_string, sizeof(__pyx_k_write_character_string), 0, 0, 1, 1},
  {&__pyx_n_s_write_name, __pyx_k_write_name, sizeof(__pyx_k_write_name), 0, 0, 1, 1},
  {&__pyx_n_s_write_record, __pyx_k_write_record, sizeof(__pyx_k_write_record), 0, 0, 1, 1},
  {&__pyx_n_s_write_record_locals_genexpr, __pyx_k_write_record_locals_genexpr, sizeof(__pyx_k_write_record_locals_genexpr), 0, 0, 1, 1},
  {&__pyx_n_s_write_short, __pyx_k_write_short, sizeof(__pyx_k_write_short), 0, 0, 1, 1},
  {&__pyx_n_s_write_string, __pyx_k_write_string, sizeof(__pyx_k_write_string), 0, 0, 1, 1},
  {&__pyx_n_s_write_ttl, __pyx_k_write_ttl, sizeof(__pyx_k_write_ttl), 0, 0, 1, 1},
  {&__pyx_n_s_write_utf, __pyx_k_write_utf, sizeof(__pyx_k_write_utf), 0, 0, 1, 1},
  {&__pyx_n_s_zeroconf__protocol_outgoing, __pyx_k_zeroconf__protocol_outgoing, sizeof(__pyx_k_zeroconf__protocol_outgoing), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_range) __PYX_ERR(0, 262, __pyx_L1_error)
  __pyx_builtin_sum = __Pyx_GetBuiltinName(__pyx_n_s_sum); if (!__pyx_builtin_sum) __PYX_ERR(0, 311, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "zeroconf/_protocol/outgoing.py":197
 *     def _write_byte(self, value: int) -> None:
 *         """Writes a single byte to the packet"""
 *         self.data.append(value.to_bytes(1, 'big'))             # <<<<<<<<<<<<<<
 *         self.size += 1
 * 
 */
  __pyx_tuple__2 = PyTuple_Pack(2, __pyx_int_1, __pyx_n_u_big); if (unlikely(!__pyx_tuple__2)) __PYX_ERR(0, 197, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__2);
  __Pyx_GIVEREF(__pyx_tuple__2);

  /* "zeroconf/_protocol/outgoing.py":202
 *     def _insert_short_at_start(self, value: int) -> None:
 *         """Inserts an unsigned short at the start of the packet"""
 *         self.data.insert(0, value.to_bytes(2, 'big'))             # <<<<<<<<<<<<<<
 * 
 *     def _replace_short(self, index: int, value: int) -> None:
 */
  __pyx_tuple__3 = PyTuple_Pack(2, __pyx_int_2, __pyx_n_u_big); if (unlikely(!__pyx_tuple__3)) __PYX_ERR(0, 202, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);

  /* "zeroconf/_protocol/outgoing.py":215
 *     def _write_int(self, value: Union[float, int]) -> None:
 *         """Writes an unsigned integer to the packet"""
 *         self.data.append(int(value).to_bytes(4, 'big'))             # <<<<<<<<<<<<<<
 *         self.size += 4
 * 
 */
  __pyx_tuple__4 = PyTuple_Pack(2, __pyx_int_4, __pyx_n_u_big); if (unlikely(!__pyx_tuple__4)) __PYX_ERR(0, 215, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0x3d398d7, 0xa242541, 0x5aef2cc):             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0x3d398d7, 0xa242541, 0x5aef2cc) = (additionals, allow_long, answers, authorities, data, finished, flags, id, multicast, names, packets_data, questions, size, state))" % __pyx_checksum)
 */
  __pyx_tuple__7 = PyTuple_Pack(3, __pyx_int_64198871, __pyx_int_170140993, __pyx_int_95351500); if (unlikely(!__pyx_tuple__7)) __PYX_ERR(2, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__7);
  __Pyx_GIVEREF(__pyx_tuple__7);

  /* "zeroconf/_protocol/outgoing.py":54
 * 
 *     __slots__ = (
 *         'flags',             # <<<<<<<<<<<<<<
 *         'finished',
 *         'id',
 */
  __pyx_tuple__8 = PyTuple_Pack(14, __pyx_n_u_flags, __pyx_n_u_finished, __pyx_n_u_id_2, __pyx_n_u_multicast, __pyx_n_u_packets_data, __pyx_n_u_names, __pyx_n_u_data, __pyx_n_u_size, __pyx_n_u_allow_long, __pyx_n_u_state, __pyx_n_u_questions, __pyx_n_u_answers, __pyx_n_u_authorities, __pyx_n_u_additionals); if (unlikely(!__pyx_tuple__8)) __PYX_ERR(0, 54, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__8);
  __Pyx_GIVEREF(__pyx_tuple__8);

  /* "zeroconf/_protocol/outgoing.py":90
 *         self.additionals: List[DNSRecord] = []
 * 
 *     def is_query(self) -> bool:             # <<<<<<<<<<<<<<
 *         """Returns true if this is a query."""
 *         return (self.flags & _FLAGS_QR_MASK) == _FLAGS_QR_QUERY
 */
  __pyx_tuple__9 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__9)) __PYX_ERR(0, 90, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__9);
  __Pyx_GIVEREF(__pyx_tuple__9);
  __pyx_codeobj__10 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__9, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_zeroconf__protocol_outgoing, __pyx_n_s_is_query, 90, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__10)) __PYX_ERR(0, 90, __pyx_L1_error)

  /* "zeroconf/_protocol/outgoing.py":94
 *         return (self.flags & _FLAGS_QR_MASK) == _FLAGS_QR_QUERY
 * 
 *     def is_response(self) -> bool:             # <<<<<<<<<<<<<<
 *         """Returns true if this is a response."""
 *         return (self.flags & _FLAGS_QR_MASK) == _FLAGS_QR_RESPONSE
 */
  __pyx_tuple__11 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__11)) __PYX_ERR(0, 94, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__11);
  __Pyx_GIVEREF(__pyx_tuple__11);
  __pyx_codeobj__12 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__11, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_zeroconf__protocol_outgoing, __pyx_n_s_is_response, 94, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__12)) __PYX_ERR(0, 94, __pyx_L1_error)

  /* "zeroconf/_protocol/outgoing.py":116
 *         )
 * 
 *     def add_question(self, record: DNSQuestion) -> None:             # <<<<<<<<<<<<<<
 *         """Adds a question"""
 *         self.questions.append(record)
 */
  __pyx_tuple__13 = PyTuple_Pack(2, __pyx_n_s_self, __pyx_n_s_record); if (unlikely(!__pyx_tuple__13)) __PYX_ERR(0, 116, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__13);
  __Pyx_GIVEREF(__pyx_tuple__13);
  __pyx_codeobj__14 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__13, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_zeroconf__protocol_outgoing, __pyx_n_s_add_question, 116, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__14)) __PYX_ERR(0, 116, __pyx_L1_error)

  /* "zeroconf/_protocol/outgoing.py":120
 *         self.questions.append(record)
 * 
 *     def add_answer(self, inp: DNSIncoming, record: DNSRecord) -> None:             # <<<<<<<<<<<<<<
 *         """Adds an answer"""
 *         if not record.suppressed_by(inp):
 */
  __pyx_tuple__15 = PyTuple_Pack(3, __pyx_n_s_self, __pyx_n_s_inp, __pyx_n_s_record); if (unlikely(!__pyx_tuple__15)) __PYX_ERR(0, 120, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__15);
  __Pyx_GIVEREF(__pyx_tuple__15);
  __pyx_codeobj__16 = (PyObject*)__Pyx_PyCode_New(3, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__15, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_zeroconf__protocol_outgoing, __pyx_n_s_add_answer, 120, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__16)) __PYX_ERR(0, 120, __pyx_L1_error)

  /* "zeroconf/_protocol/outgoing.py":125
 *             self.add_answer_at_time(record, 0)
 * 
 *     def add_answer_at_time(self, record: Optional[DNSRecord], now: Union[float, int]) -> None:             # <<<<<<<<<<<<<<
 *         """Adds an answer if it does not expire by a certain time"""
 *         if record is not None and (now == 0 or not record.is_expired(now)):
 */
  __pyx_tuple__17 = PyTuple_Pack(3, __pyx_n_s_self, __pyx_n_s_record, __pyx_n_s_now); if (unlikely(!__pyx_tuple__17)) __PYX_ERR(0, 125, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__17);
  __Pyx_GIVEREF(__pyx_tuple__17);
  __pyx_codeobj__18 = (PyObject*)__Pyx_PyCode_New(3, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__17, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_zeroconf__protocol_outgoing, __pyx_n_s_add_answer_at_time, 125, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__18)) __PYX_ERR(0, 125, __pyx_L1_error)

  /* "zeroconf/_protocol/outgoing.py":130
 *             self.answers.append((record, now))
 * 
 *     def add_authorative_answer(self, record: DNSPointer) -> None:             # <<<<<<<<<<<<<<
 *         """Adds an authoritative answer"""
 *         self.authorities.append(record)
 */
  __pyx_tuple__19 = PyTuple_Pack(2, __pyx_n_s_self, __pyx_n_s_record); if (unlikely(!__pyx_tuple__19)) __PYX_ERR(0, 130, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__19);
  __Pyx_GIVEREF(__pyx_tuple__19);
  __pyx_codeobj__20 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__19, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_zeroconf__protocol_outgoing, __pyx_n_s_add_authorative_answer, 130, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__20)) __PYX_ERR(0, 130, __pyx_L1_error)

  /* "zeroconf/_protocol/outgoing.py":134
 *         self.authorities.append(record)
 * 
 *     def add_additional_answer(self, record: DNSRecord) -> None:             # <<<<<<<<<<<<<<
 *         """Adds an additional answer
 * 
 */
  __pyx_tuple__21 = PyTuple_Pack(2, __pyx_n_s_self, __pyx_n_s_record); if (unlikely(!__pyx_tuple__21)) __PYX_ERR(0, 134, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__21);
  __Pyx_GIVEREF(__pyx_tuple__21);
  __pyx_codeobj__22 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__21, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_zeroconf__protocol_outgoing, __pyx_n_s_add_additional_answer, 134, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__22)) __PYX_ERR(0, 134, __pyx_L1_error)

  /* "zeroconf/_protocol/outgoing.py":172
 *         self.additionals.append(record)
 * 
 *     def add_question_or_one_cache(             # <<<<<<<<<<<<<<
 *         self, cache: DNSCache, now: float, name: str, type_: int, class_: int
 *     ) -> None:
 */
  __pyx_tuple__23 = PyTuple_Pack(7, __pyx_n_s_self, __pyx_n_s_cache, __pyx_n_s_now, __pyx_n_s_name, __pyx_n_s_type, __pyx_n_s_class, __pyx_n_s_cached_entry); if (unlikely(!__pyx_tuple__23)) __PYX_ERR(0, 172, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__23);
  __Pyx_GIVEREF(__pyx_tuple__23);
  __pyx_codeobj__24 = (PyObject*)__Pyx_PyCode_New(6, 0, 7, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__23, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_zeroconf__protocol_outgoing, __pyx_n_s_add_question_or_one_cache, 172, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__24)) __PYX_ERR(0, 172, __pyx_L1_error)

  /* "zeroconf/_protocol/outgoing.py":182
 *             self.add_answer_at_time(cached_entry, now)
 * 
 *     def add_question_or_all_cache(             # <<<<<<<<<<<<<<
 *         self, cache: DNSCache, now: float, name: str, type_: int, class_: int
 *     ) -> None:
 */
  __pyx_tuple__25 = PyTuple_Pack(8, __pyx_n_s_self, __pyx_n_s_cache, __pyx_n_s_now, __pyx_n_s_name, __pyx_n_s_type, __pyx_n_s_class, __pyx_n_s_cached_entries, __pyx_n_s_cached_entry); if (unlikely(!__pyx_tuple__25)) __PYX_ERR(0, 182, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__25);
  __Pyx_GIVEREF(__pyx_tuple__25);
  __pyx_codeobj__26 = (PyObject*)__Pyx_PyCode_New(6, 0, 8, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__25, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_zeroconf__protocol_outgoing, __pyx_n_s_add_question_or_all_cache, 182, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__26)) __PYX_ERR(0, 182, __pyx_L1_error)

  /* "zeroconf/_protocol/outgoing.py":208
 *         self.data[index] = value.to_bytes(2, 'big')
 * 
 *     def write_short(self, value: int) -> None:             # <<<<<<<<<<<<<<
 *         """Writes an unsigned short to the packet"""
 *         self.data.append(value.to_bytes(2, 'big'))
 */
  __pyx_tuple__27 = PyTuple_Pack(2, __pyx_n_s_self, __pyx_n_s_value); if (unlikely(!__pyx_tuple__27)) __PYX_ERR(0, 208, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__27);
  __Pyx_GIVEREF(__pyx_tuple__27);
  __pyx_codeobj__28 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__27, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_zeroconf__protocol_outgoing, __pyx_n_s_write_short, 208, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__28)) __PYX_ERR(0, 208, __pyx_L1_error)

  /* "zeroconf/_protocol/outgoing.py":218
 *         self.size += 4
 * 
 *     def write_string(self, value: bytes) -> None:             # <<<<<<<<<<<<<<
 *         """Writes a string to the packet"""
 *         assert isinstance(value, bytes)
 */
  __pyx_tuple__29 = PyTuple_Pack(2, __pyx_n_s_self, __pyx_n_s_value); if (unlikely(!__pyx_tuple__29)) __PYX_ERR(0, 218, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__29);
  __Pyx_GIVEREF(__pyx_tuple__29);
  __pyx_codeobj__30 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__29, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_zeroconf__protocol_outgoing, __pyx_n_s_write_string, 218, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__30)) __PYX_ERR(0, 218, __pyx_L1_error)

  /* "zeroconf/_protocol/outgoing.py":224
 *         self.size += len(value)
 * 
 *     def _write_utf(self, s: str) -> None:             # <<<<<<<<<<<<<<
 *         """Writes a UTF-8 string of a given length to the packet"""
 *         utfstr = s.encode('utf-8')
 */
  __pyx_tuple__31 = PyTuple_Pack(4, __pyx_n_s_self, __pyx_n_s_s, __pyx_n_s_utfstr, __pyx_n_s_length); if (unlikely(!__pyx_tuple__31)) __PYX_ERR(0, 224, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__31);
  __Pyx_GIVEREF(__pyx_tuple__31);
  __pyx_codeobj__32 = (PyObject*)__Pyx_PyCode_New(2, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__31, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_zeroconf__protocol_outgoing, __pyx_n_s_write_utf, 224, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__32)) __PYX_ERR(0, 224, __pyx_L1_error)

  /* "zeroconf/_protocol/outgoing.py":233
 *         self.write_string(utfstr)
 * 
 *     def write_character_string(self, value: bytes) -> None:             # <<<<<<<<<<<<<<
 *         assert isinstance(value, bytes)
 *         length = len(value)
 */
  __pyx_tuple__33 = PyTuple_Pack(3, __pyx_n_s_self, __pyx_n_s_value, __pyx_n_s_length); if (unlikely(!__pyx_tuple__33)) __PYX_ERR(0, 233, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__33);
  __Pyx_GIVEREF(__pyx_tuple__33);
  __pyx_codeobj__34 = (PyObject*)__Pyx_PyCode_New(2, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__33, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_zeroconf__protocol_outgoing, __pyx_n_s_write_character_string, 233, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__34)) __PYX_ERR(0, 233, __pyx_L1_error)

  /* "zeroconf/_protocol/outgoing.py":241
 *         self.write_string(value)
 * 
 *     def write_name(self, name: str) -> None:             # <<<<<<<<<<<<<<
 *         """
 *         Write names to packet
 */
  __pyx_tuple__35 = PyTuple_Pack(8, __pyx_n_s_self, __pyx_n_s_name, __pyx_n_s_name_length, __pyx_n_s_labels, __pyx_n_s_start_size, __pyx_n_s_count, __pyx_n_s_label, __pyx_n_s_index); if (unlikely(!__pyx_tuple__35)) __PYX_ERR(0, 241, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__35);
  __Pyx_GIVEREF(__pyx_tuple__35);
  __pyx_codeobj__36 = (PyObject*)__Pyx_PyCode_New(2, 0, 8, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__35, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_zeroconf__protocol_outgoing, __pyx_n_s_write_name, 241, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__36)) __PYX_ERR(0, 241, __pyx_L1_error)

  /* "zeroconf/_protocol/outgoing.py":294
 *             self.write_short(record.class_)
 * 
 *     def _write_ttl(self, record: DNSRecord, now: float) -> None:             # <<<<<<<<<<<<<<
 *         """Write out the record ttl."""
 *         self._write_int(record.ttl if now == 0 else record.get_remaining_ttl(now))
 */
  __pyx_tuple__37 = PyTuple_Pack(3, __pyx_n_s_self, __pyx_n_s_record, __pyx_n_s_now); if (unlikely(!__pyx_tuple__37)) __PYX_ERR(0, 294, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__37);
  __Pyx_GIVEREF(__pyx_tuple__37);
  __pyx_codeobj__38 = (PyObject*)__Pyx_PyCode_New(3, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__37, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_zeroconf__protocol_outgoing, __pyx_n_s_write_ttl, 294, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__38)) __PYX_ERR(0, 294, __pyx_L1_error)

  /* "zeroconf/_protocol/outgoing.py":298
 *         self._write_int(record.ttl if now == 0 else record.get_remaining_ttl(now))
 * 
 *     def _write_record(self, record: DNSRecord, now: float) -> bool:             # <<<<<<<<<<<<<<
 *         """Writes a record (answer, authoritative answer, additional) to
 *         the packet.  Returns True on success, or False if we did not
 */
  __pyx_tuple__39 = PyTuple_Pack(9, __pyx_n_s_self, __pyx_n_s_record, __pyx_n_s_now, __pyx_n_s_start_data_length, __pyx_n_s_start_size, __pyx_n_s_index, __pyx_n_s_length, __pyx_n_s_genexpr, __pyx_n_s_genexpr); if (unlikely(!__pyx_tuple__39)) __PYX_ERR(0, 298, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__39);
  __Pyx_GIVEREF(__pyx_tuple__39);
  __pyx_codeobj__40 = (PyObject*)__Pyx_PyCode_New(3, 0, 9, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__39, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_zeroconf__protocol_outgoing, __pyx_n_s_write_record, 298, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__40)) __PYX_ERR(0, 298, __pyx_L1_error)

  /* "zeroconf/_protocol/outgoing.py":369
 *         )
 * 
 *     def packets(self) -> List[bytes]:             # <<<<<<<<<<<<<<
 *         """Returns a list of bytestrings containing the packets' bytes
 * 
 */
  __pyx_tuple__41 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__41)) __PYX_ERR(0, 369, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__41);
  __Pyx_GIVEREF(__pyx_tuple__41);
  __pyx_codeobj__42 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__41, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_zeroconf__protocol_outgoing, __pyx_n_s_packets, 369, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__42)) __PYX_ERR(0, 369, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */
  __pyx_tuple__43 = PyTuple_Pack(4, __pyx_n_s_self, __pyx_n_s_state, __pyx_n_s_dict_2, __pyx_n_s_use_setstate); if (unlikely(!__pyx_tuple__43)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__43);
  __Pyx_GIVEREF(__pyx_tuple__43);
  __pyx_codeobj__44 = (PyObject*)__Pyx_PyCode_New(1, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__43, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_reduce_cython, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__44)) __PYX_ERR(2, 1, __pyx_L1_error)

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_DNSOutgoing, (type(self), 0x3d398d7, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_DNSOutgoing__set_state(self, __pyx_state)
 */
  __pyx_tuple__45 = PyTuple_Pack(2, __pyx_n_s_self, __pyx_n_s_pyx_state); if (unlikely(!__pyx_tuple__45)) __PYX_ERR(2, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__45);
  __Pyx_GIVEREF(__pyx_tuple__45);
  __pyx_codeobj__46 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__45, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_setstate_cython, 16, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__46)) __PYX_ERR(2, 16, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __pyx_unpickle_DNSOutgoing(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_tuple__47 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__47)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__47);
  __Pyx_GIVEREF(__pyx_tuple__47);
  __pyx_codeobj__48 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__47, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_DNSOutgoing, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__48)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_1 = PyInt_FromLong(1); if (unlikely(!__pyx_int_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_2 = PyInt_FromLong(2); if (unlikely(!__pyx_int_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_4 = PyInt_FromLong(4); if (unlikely(!__pyx_int_4)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_8 = PyInt_FromLong(8); if (unlikely(!__pyx_int_8)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_192 = PyInt_FromLong(192); if (unlikely(!__pyx_int_192)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_255 = PyInt_FromLong(255); if (unlikely(!__pyx_int_255)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_64198871 = PyInt_FromLong(64198871L); if (unlikely(!__pyx_int_64198871)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_95351500 = PyInt_FromLong(95351500L); if (unlikely(!__pyx_int_95351500)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_170140993 = PyInt_FromLong(170140993L); if (unlikely(!__pyx_int_170140993)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(void); /*proto*/

static int __Pyx_modinit_global_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(void) {
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  if (__Pyx_ExportVoidPtr(__pyx_n_s_CLASS_UNIQUE, (void *)&__pyx_v_8zeroconf_9_protocol_8outgoing__CLASS_UNIQUE, "unsigned int") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_DNS_PACKET_HEADER_LEN, (void *)&__pyx_v_8zeroconf_9_protocol_8outgoing__DNS_PACKET_HEADER_LEN, "unsigned int") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_FLAGS_QR_MASK, (void *)&__pyx_v_8zeroconf_9_protocol_8outgoing__FLAGS_QR_MASK, "unsigned int") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_FLAGS_QR_QUERY, (void *)&__pyx_v_8zeroconf_9_protocol_8outgoing__FLAGS_QR_QUERY, "unsigned int") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_FLAGS_QR_RESPONSE, (void *)&__pyx_v_8zeroconf_9_protocol_8outgoing__FLAGS_QR_RESPONSE, "unsigned int") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_FLAGS_TC, (void *)&__pyx_v_8zeroconf_9_protocol_8outgoing__FLAGS_TC, "unsigned int") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_MAX_MSG_ABSOLUTE, (void *)&__pyx_v_8zeroconf_9_protocol_8outgoing__MAX_MSG_ABSOLUTE, "unsigned int") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_MAX_MSG_TYPICAL, (void *)&__pyx_v_8zeroconf_9_protocol_8outgoing__MAX_MSG_TYPICAL, "unsigned int") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_function_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_init_code(void) {
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  __pyx_vtabptr_8zeroconf_9_protocol_8outgoing_DNSOutgoing = &__pyx_vtable_8zeroconf_9_protocol_8outgoing_DNSOutgoing;
  __pyx_vtable_8zeroconf_9_protocol_8outgoing_DNSOutgoing._reset_for_next_packet = (PyObject *(*)(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *))__pyx_f_8zeroconf_9_protocol_8outgoing_11DNSOutgoing__reset_for_next_packet;
  __pyx_vtable_8zeroconf_9_protocol_8outgoing_DNSOutgoing._write_byte = (PyObject *(*)(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *, PyObject *))__pyx_f_8zeroconf_9_protocol_8outgoing_11DNSOutgoing__write_byte;
  __pyx_vtable_8zeroconf_9_protocol_8outgoing_DNSOutgoing._insert_short_at_start = (PyObject *(*)(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *, PyObject *))__pyx_f_8zeroconf_9_protocol_8outgoing_11DNSOutgoing__insert_short_at_start;
  __pyx_vtable_8zeroconf_9_protocol_8outgoing_DNSOutgoing._replace_short = (PyObject *(*)(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *, PyObject *, PyObject *))__pyx_f_8zeroconf_9_protocol_8outgoing_11DNSOutgoing__replace_short;
  __pyx_vtable_8zeroconf_9_protocol_8outgoing_DNSOutgoing._write_int = (PyObject *(*)(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *, PyObject *))__pyx_f_8zeroconf_9_protocol_8outgoing_11DNSOutgoing__write_int;
  __pyx_vtable_8zeroconf_9_protocol_8outgoing_DNSOutgoing._write_question = (PyObject *(*)(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *, PyObject *))__pyx_f_8zeroconf_9_protocol_8outgoing_11DNSOutgoing__write_question;
  __pyx_vtable_8zeroconf_9_protocol_8outgoing_DNSOutgoing._write_record_class = (PyObject *(*)(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *, PyObject *))__pyx_f_8zeroconf_9_protocol_8outgoing_11DNSOutgoing__write_record_class;
  __pyx_vtable_8zeroconf_9_protocol_8outgoing_DNSOutgoing._check_data_limit_or_rollback = (PyObject *(*)(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *, PyObject *, PyObject *))__pyx_f_8zeroconf_9_protocol_8outgoing_11DNSOutgoing__check_data_limit_or_rollback;
  __pyx_vtable_8zeroconf_9_protocol_8outgoing_DNSOutgoing._write_questions_from_offset = (PyObject *(*)(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *, PyObject *))__pyx_f_8zeroconf_9_protocol_8outgoing_11DNSOutgoing__write_questions_from_offset;
  __pyx_vtable_8zeroconf_9_protocol_8outgoing_DNSOutgoing._write_answers_from_offset = (PyObject *(*)(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *, PyObject *))__pyx_f_8zeroconf_9_protocol_8outgoing_11DNSOutgoing__write_answers_from_offset;
  __pyx_vtable_8zeroconf_9_protocol_8outgoing_DNSOutgoing._write_records_from_offset = (PyObject *(*)(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *, PyObject *, PyObject *))__pyx_f_8zeroconf_9_protocol_8outgoing_11DNSOutgoing__write_records_from_offset;
  __pyx_vtable_8zeroconf_9_protocol_8outgoing_DNSOutgoing._has_more_to_add = (PyObject *(*)(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *, PyObject *, PyObject *, PyObject *, PyObject *))__pyx_f_8zeroconf_9_protocol_8outgoing_11DNSOutgoing__has_more_to_add;
  __pyx_vtable_8zeroconf_9_protocol_8outgoing_DNSOutgoing._packets = (PyObject *(*)(struct __pyx_obj_8zeroconf_9_protocol_8outgoing_DNSOutgoing *))__pyx_f_8zeroconf_9_protocol_8outgoing_11DNSOutgoing__packets;
  if (PyType_Ready(&__pyx_type_8zeroconf_9_protocol_8outgoing_DNSOutgoing) < 0) __PYX_ERR(0, 49, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_8zeroconf_9_protocol_8outgoing_DNSOutgoing.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_8zeroconf_9_protocol_8outgoing_DNSOutgoing.tp_dictoffset && __pyx_type_8zeroconf_9_protocol_8outgoing_DNSOutgoing.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_8zeroconf_9_protocol_8outgoing_DNSOutgoing.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_8zeroconf_9_protocol_8outgoing_DNSOutgoing.tp_dict, __pyx_vtabptr_8zeroconf_9_protocol_8outgoing_DNSOutgoing) < 0) __PYX_ERR(0, 49, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_DNSOutgoing, (PyObject *)&__pyx_type_8zeroconf_9_protocol_8outgoing_DNSOutgoing) < 0) __PYX_ERR(0, 49, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_8zeroconf_9_protocol_8outgoing_DNSOutgoing) < 0) __PYX_ERR(0, 49, __pyx_L1_error)
  __pyx_ptype_8zeroconf_9_protocol_8outgoing_DNSOutgoing = &__pyx_type_8zeroconf_9_protocol_8outgoing_DNSOutgoing;
  if (PyType_Ready(&__pyx_type_8zeroconf_9_protocol_8outgoing___pyx_scope_struct___write_record) < 0) __PYX_ERR(0, 298, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_8zeroconf_9_protocol_8outgoing___pyx_scope_struct___write_record.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_8zeroconf_9_protocol_8outgoing___pyx_scope_struct___write_record.tp_dictoffset && __pyx_type_8zeroconf_9_protocol_8outgoing___pyx_scope_struct___write_record.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_8zeroconf_9_protocol_8outgoing___pyx_scope_struct___write_record.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  __pyx_ptype_8zeroconf_9_protocol_8outgoing___pyx_scope_struct___write_record = &__pyx_type_8zeroconf_9_protocol_8outgoing___pyx_scope_struct___write_record;
  if (PyType_Ready(&__pyx_type_8zeroconf_9_protocol_8outgoing___pyx_scope_struct_1_genexpr) < 0) __PYX_ERR(0, 311, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_8zeroconf_9_protocol_8outgoing___pyx_scope_struct_1_genexpr.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_8zeroconf_9_protocol_8outgoing___pyx_scope_struct_1_genexpr.tp_dictoffset && __pyx_type_8zeroconf_9_protocol_8outgoing___pyx_scope_struct_1_genexpr.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_8zeroconf_9_protocol_8outgoing___pyx_scope_struct_1_genexpr.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  __pyx_ptype_8zeroconf_9_protocol_8outgoing___pyx_scope_struct_1_genexpr = &__pyx_type_8zeroconf_9_protocol_8outgoing___pyx_scope_struct_1_genexpr;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __pyx_t_1 = PyImport_ImportModule("zeroconf._dns"); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 21, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_8zeroconf_4_dns_DNSEntry = __Pyx_ImportType_0_29_35(__pyx_t_1, "zeroconf._dns", "DNSEntry", sizeof(struct __pyx_obj_8zeroconf_4_dns_DNSEntry), __PYX_GET_STRUCT_ALIGNMENT_0_29_35(struct __pyx_obj_8zeroconf_4_dns_DNSEntry),__Pyx_ImportType_CheckSize_Warn_0_29_35); if (!__pyx_ptype_8zeroconf_4_dns_DNSEntry) __PYX_ERR(3, 21, __pyx_L1_error)
  __pyx_vtabptr_8zeroconf_4_dns_DNSEntry = (struct __pyx_vtabstruct_8zeroconf_4_dns_DNSEntry*)__Pyx_GetVtable(__pyx_ptype_8zeroconf_4_dns_DNSEntry->tp_dict); if (unlikely(!__pyx_vtabptr_8zeroconf_4_dns_DNSEntry)) __PYX_ERR(3, 21, __pyx_L1_error)
  __pyx_ptype_8zeroconf_4_dns_DNSQuestion = __Pyx_ImportType_0_29_35(__pyx_t_1, "zeroconf._dns", "DNSQuestion", sizeof(struct __pyx_obj_8zeroconf_4_dns_DNSQuestion), __PYX_GET_STRUCT_ALIGNMENT_0_29_35(struct __pyx_obj_8zeroconf_4_dns_DNSQuestion),__Pyx_ImportType_CheckSize_Warn_0_29_35); if (!__pyx_ptype_8zeroconf_4_dns_DNSQuestion) __PYX_ERR(3, 31, __pyx_L1_error)
  __pyx_vtabptr_8zeroconf_4_dns_DNSQuestion = (struct __pyx_vtabstruct_8zeroconf_4_dns_DNSQuestion*)__Pyx_GetVtable(__pyx_ptype_8zeroconf_4_dns_DNSQuestion->tp_dict); if (unlikely(!__pyx_vtabptr_8zeroconf_4_dns_DNSQuestion)) __PYX_ERR(3, 31, __pyx_L1_error)
  __pyx_ptype_8zeroconf_4_dns_DNSRecord = __Pyx_ImportType_0_29_35(__pyx_t_1, "zeroconf._dns", "DNSRecord", sizeof(struct __pyx_obj_8zeroconf_4_dns_DNSRecord), __PYX_GET_STRUCT_ALIGNMENT_0_29_35(struct __pyx_obj_8zeroconf_4_dns_DNSRecord),__Pyx_ImportType_CheckSize_Warn_0_29_35); if (!__pyx_ptype_8zeroconf_4_dns_DNSRecord) __PYX_ERR(3, 35, __pyx_L1_error)
  __pyx_vtabptr_8zeroconf_4_dns_DNSRecord = (struct __pyx_vtabstruct_8zeroconf_4_dns_DNSRecord*)__Pyx_GetVtable(__pyx_ptype_8zeroconf_4_dns_DNSRecord->tp_dict); if (unlikely(!__pyx_vtabptr_8zeroconf_4_dns_DNSRecord)) __PYX_ERR(3, 35, __pyx_L1_error)
  __pyx_ptype_8zeroconf_4_dns_DNSAddress = __Pyx_ImportType_0_29_35(__pyx_t_1, "zeroconf._dns", "DNSAddress", sizeof(struct __pyx_obj_8zeroconf_4_dns_DNSAddress), __PYX_GET_STRUCT_ALIGNMENT_0_29_35(struct __pyx_obj_8zeroconf_4_dns_DNSAddress),__Pyx_ImportType_CheckSize_Warn_0_29_35); if (!__pyx_ptype_8zeroconf_4_dns_DNSAddress) __PYX_ERR(3, 43, __pyx_L1_error)
  __pyx_vtabptr_8zeroconf_4_dns_DNSAddress = (struct __pyx_vtabstruct_8zeroconf_4_dns_DNSAddress*)__Pyx_GetVtable(__pyx_ptype_8zeroconf_4_dns_DNSAddress->tp_dict); if (unlikely(!__pyx_vtabptr_8zeroconf_4_dns_DNSAddress)) __PYX_ERR(3, 43, __pyx_L1_error)
  __pyx_ptype_8zeroconf_4_dns_DNSHinfo = __Pyx_ImportType_0_29_35(__pyx_t_1, "zeroconf._dns", "DNSHinfo", sizeof(struct __pyx_obj_8zeroconf_4_dns_DNSHinfo), __PYX_GET_STRUCT_ALIGNMENT_0_29_35(struct __pyx_obj_8zeroconf_4_dns_DNSHinfo),__Pyx_ImportType_CheckSize_Warn_0_29_35); if (!__pyx_ptype_8zeroconf_4_dns_DNSHinfo) __PYX_ERR(3, 52, __pyx_L1_error)
  __pyx_vtabptr_8zeroconf_4_dns_DNSHinfo = (struct __pyx_vtabstruct_8zeroconf_4_dns_DNSHinfo*)__Pyx_GetVtable(__pyx_ptype_8zeroconf_4_dns_DNSHinfo->tp_dict); if (unlikely(!__pyx_vtabptr_8zeroconf_4_dns_DNSHinfo)) __PYX_ERR(3, 52, __pyx_L1_error)
  __pyx_ptype_8zeroconf_4_dns_DNSPointer = __Pyx_ImportType_0_29_35(__pyx_t_1, "zeroconf._dns", "DNSPointer", sizeof(struct __pyx_obj_8zeroconf_4_dns_DNSPointer), __PYX_GET_STRUCT_ALIGNMENT_0_29_35(struct __pyx_obj_8zeroconf_4_dns_DNSPointer),__Pyx_ImportType_CheckSize_Warn_0_29_35); if (!__pyx_ptype_8zeroconf_4_dns_DNSPointer) __PYX_ERR(3, 61, __pyx_L1_error)
  __pyx_vtabptr_8zeroconf_4_dns_DNSPointer = (struct __pyx_vtabstruct_8zeroconf_4_dns_DNSPointer*)__Pyx_GetVtable(__pyx_ptype_8zeroconf_4_dns_DNSPointer->tp_dict); if (unlikely(!__pyx_vtabptr_8zeroconf_4_dns_DNSPointer)) __PYX_ERR(3, 61, __pyx_L1_error)
  __pyx_ptype_8zeroconf_4_dns_DNSText = __Pyx_ImportType_0_29_35(__pyx_t_1, "zeroconf._dns", "DNSText", sizeof(struct __pyx_obj_8zeroconf_4_dns_DNSText), __PYX_GET_STRUCT_ALIGNMENT_0_29_35(struct __pyx_obj_8zeroconf_4_dns_DNSText),__Pyx_ImportType_CheckSize_Warn_0_29_35); if (!__pyx_ptype_8zeroconf_4_dns_DNSText) __PYX_ERR(3, 70, __pyx_L1_error)
  __pyx_vtabptr_8zeroconf_4_dns_DNSText = (struct __pyx_vtabstruct_8zeroconf_4_dns_DNSText*)__Pyx_GetVtable(__pyx_ptype_8zeroconf_4_dns_DNSText->tp_dict); if (unlikely(!__pyx_vtabptr_8zeroconf_4_dns_DNSText)) __PYX_ERR(3, 70, __pyx_L1_error)
  __pyx_ptype_8zeroconf_4_dns_DNSService = __Pyx_ImportType_0_29_35(__pyx_t_1, "zeroconf._dns", "DNSService", sizeof(struct __pyx_obj_8zeroconf_4_dns_DNSService), __PYX_GET_STRUCT_ALIGNMENT_0_29_35(struct __pyx_obj_8zeroconf_4_dns_DNSService),__Pyx_ImportType_CheckSize_Warn_0_29_35); if (!__pyx_ptype_8zeroconf_4_dns_DNSService) __PYX_ERR(3, 78, __pyx_L1_error)
  __pyx_vtabptr_8zeroconf_4_dns_DNSService = (struct __pyx_vtabstruct_8zeroconf_4_dns_DNSService*)__Pyx_GetVtable(__pyx_ptype_8zeroconf_4_dns_DNSService->tp_dict); if (unlikely(!__pyx_vtabptr_8zeroconf_4_dns_DNSService)) __PYX_ERR(3, 78, __pyx_L1_error)
  __pyx_ptype_8zeroconf_4_dns_DNSNsec = __Pyx_ImportType_0_29_35(__pyx_t_1, "zeroconf._dns", "DNSNsec", sizeof(struct __pyx_obj_8zeroconf_4_dns_DNSNsec), __PYX_GET_STRUCT_ALIGNMENT_0_29_35(struct __pyx_obj_8zeroconf_4_dns_DNSNsec),__Pyx_ImportType_CheckSize_Warn_0_29_35); if (!__pyx_ptype_8zeroconf_4_dns_DNSNsec) __PYX_ERR(3, 90, __pyx_L1_error)
  __pyx_vtabptr_8zeroconf_4_dns_DNSNsec = (struct __pyx_vtabstruct_8zeroconf_4_dns_DNSNsec*)__Pyx_GetVtable(__pyx_ptype_8zeroconf_4_dns_DNSNsec->tp_dict); if (unlikely(!__pyx_vtabptr_8zeroconf_4_dns_DNSNsec)) __PYX_ERR(3, 90, __pyx_L1_error)
  __pyx_ptype_8zeroconf_4_dns_DNSRRSet = __Pyx_ImportType_0_29_35(__pyx_t_1, "zeroconf._dns", "DNSRRSet", sizeof(struct __pyx_obj_8zeroconf_4_dns_DNSRRSet), __PYX_GET_STRUCT_ALIGNMENT_0_29_35(struct __pyx_obj_8zeroconf_4_dns_DNSRRSet),__Pyx_ImportType_CheckSize_Warn_0_29_35); if (!__pyx_ptype_8zeroconf_4_dns_DNSRRSet) __PYX_ERR(3, 99, __pyx_L1_error)
  __pyx_vtabptr_8zeroconf_4_dns_DNSRRSet = (struct __pyx_vtabstruct_8zeroconf_4_dns_DNSRRSet*)__Pyx_GetVtable(__pyx_ptype_8zeroconf_4_dns_DNSRRSet->tp_dict); if (unlikely(!__pyx_vtabptr_8zeroconf_4_dns_DNSRRSet)) __PYX_ERR(3, 99, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("zeroconf._protocol.incoming"); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 48, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_8zeroconf_9_protocol_8incoming_DNSIncoming = __Pyx_ImportType_0_29_35(__pyx_t_1, "zeroconf._protocol.incoming", "DNSIncoming", sizeof(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming), __PYX_GET_STRUCT_ALIGNMENT_0_29_35(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming),__Pyx_ImportType_CheckSize_Warn_0_29_35); if (!__pyx_ptype_8zeroconf_9_protocol_8incoming_DNSIncoming) __PYX_ERR(4, 48, __pyx_L1_error)
  __pyx_vtabptr_8zeroconf_9_protocol_8incoming_DNSIncoming = (struct __pyx_vtabstruct_8zeroconf_9_protocol_8incoming_DNSIncoming*)__Pyx_GetVtable(__pyx_ptype_8zeroconf_9_protocol_8incoming_DNSIncoming->tp_dict); if (unlikely(!__pyx_vtabptr_8zeroconf_9_protocol_8incoming_DNSIncoming)) __PYX_ERR(4, 48, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_variable_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __pyx_t_1 = PyImport_ImportModule("zeroconf._dns"); if (!__pyx_t_1) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_ImportVoidPtr_0_29_35(__pyx_t_1, "_LEN_BYTE", (void **)&__pyx_vp_8zeroconf_4_dns__LEN_BYTE, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_0_29_35(__pyx_t_1, "_LEN_SHORT", (void **)&__pyx_vp_8zeroconf_4_dns__LEN_SHORT, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_0_29_35(__pyx_t_1, "_LEN_INT", (void **)&__pyx_vp_8zeroconf_4_dns__LEN_INT, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_0_29_35(__pyx_t_1, "_NAME_COMPRESSION_MIN_SIZE", (void **)&__pyx_vp_8zeroconf_4_dns__NAME_COMPRESSION_MIN_SIZE, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_0_29_35(__pyx_t_1, "_BASE_MAX_SIZE", (void **)&__pyx_vp_8zeroconf_4_dns__BASE_MAX_SIZE, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_0_29_35(__pyx_t_1, "_EXPIRE_FULL_TIME_MS", (void **)&__pyx_vp_8zeroconf_4_dns__EXPIRE_FULL_TIME_MS, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_0_29_35(__pyx_t_1, "_EXPIRE_STALE_TIME_MS", (void **)&__pyx_vp_8zeroconf_4_dns__EXPIRE_STALE_TIME_MS, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_0_29_35(__pyx_t_1, "_RECENT_TIME_MS", (void **)&__pyx_vp_8zeroconf_4_dns__RECENT_TIME_MS, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_0_29_35(__pyx_t_1, "_CLASS_UNIQUE", (void **)&__pyx_vp_8zeroconf_4_dns__CLASS_UNIQUE, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_0_29_35(__pyx_t_1, "_CLASS_MASK", (void **)&__pyx_vp_8zeroconf_4_dns__CLASS_MASK, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_0_29_35(__pyx_t_1, "current_time_millis", (void **)&__pyx_vp_8zeroconf_4_dns_current_time_millis, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("zeroconf._protocol.incoming"); if (!__pyx_t_1) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_ImportVoidPtr_0_29_35(__pyx_t_1, "DNS_COMPRESSION_HEADER_LEN", (void **)&__pyx_vp_8zeroconf_9_protocol_8incoming_DNS_COMPRESSION_HEADER_LEN, "unsigned int") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_0_29_35(__pyx_t_1, "MAX_DNS_LABELS", (void **)&__pyx_vp_8zeroconf_9_protocol_8incoming_MAX_DNS_LABELS, "unsigned int") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_0_29_35(__pyx_t_1, "DNS_COMPRESSION_POINTER_LEN", (void **)&__pyx_vp_8zeroconf_9_protocol_8incoming_DNS_COMPRESSION_POINTER_LEN, "unsigned int") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_0_29_35(__pyx_t_1, "MAX_NAME_LENGTH", (void **)&__pyx_vp_8zeroconf_9_protocol_8incoming_MAX_NAME_LENGTH, "unsigned int") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_0_29_35(__pyx_t_1, "current_time_millis", (void **)&__pyx_vp_8zeroconf_9_protocol_8incoming_current_time_millis, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_0_29_35(__pyx_t_1, "_TYPE_A", (void **)&__pyx_vp_8zeroconf_9_protocol_8incoming__TYPE_A, "unsigned int") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_0_29_35(__pyx_t_1, "_TYPE_CNAME", (void **)&__pyx_vp_8zeroconf_9_protocol_8incoming__TYPE_CNAME, "unsigned int") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_0_29_35(__pyx_t_1, "_TYPE_PTR", (void **)&__pyx_vp_8zeroconf_9_protocol_8incoming__TYPE_PTR, "unsigned int") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_0_29_35(__pyx_t_1, "_TYPE_TXT", (void **)&__pyx_vp_8zeroconf_9_protocol_8incoming__TYPE_TXT, "unsigned int") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_0_29_35(__pyx_t_1, "_TYPE_SRV", (void **)&__pyx_vp_8zeroconf_9_protocol_8incoming__TYPE_SRV, "unsigned int") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_0_29_35(__pyx_t_1, "_TYPE_HINFO", (void **)&__pyx_vp_8zeroconf_9_protocol_8incoming__TYPE_HINFO, "unsigned int") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_0_29_35(__pyx_t_1, "_TYPE_AAAA", (void **)&__pyx_vp_8zeroconf_9_protocol_8incoming__TYPE_AAAA, "unsigned int") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_0_29_35(__pyx_t_1, "_TYPE_NSEC", (void **)&__pyx_vp_8zeroconf_9_protocol_8incoming__TYPE_NSEC, "unsigned int") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_0_29_35(__pyx_t_1, "_FLAGS_QR_MASK", (void **)&__pyx_vp_8zeroconf_9_protocol_8incoming__FLAGS_QR_MASK, "unsigned int") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_0_29_35(__pyx_t_1, "_FLAGS_TC", (void **)&__pyx_vp_8zeroconf_9_protocol_8incoming__FLAGS_TC, "unsigned int") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_0_29_35(__pyx_t_1, "_FLAGS_QR_QUERY", (void **)&__pyx_vp_8zeroconf_9_protocol_8incoming__FLAGS_QR_QUERY, "unsigned int") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_0_29_35(__pyx_t_1, "_FLAGS_QR_RESPONSE", (void **)&__pyx_vp_8zeroconf_9_protocol_8incoming__FLAGS_QR_RESPONSE, "unsigned int") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_0_29_35(__pyx_t_1, "UNPACK_3H", (void **)&__pyx_vp_8zeroconf_9_protocol_8incoming_UNPACK_3H, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_0_29_35(__pyx_t_1, "UNPACK_6H", (void **)&__pyx_vp_8zeroconf_9_protocol_8incoming_UNPACK_6H, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_0_29_35(__pyx_t_1, "UNPACK_HH", (void **)&__pyx_vp_8zeroconf_9_protocol_8incoming_UNPACK_HH, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_0_29_35(__pyx_t_1, "UNPACK_HHiH", (void **)&__pyx_vp_8zeroconf_9_protocol_8incoming_UNPACK_HHiH, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_0_29_35(__pyx_t_1, "DECODE_EXCEPTIONS", (void **)&__pyx_vp_8zeroconf_9_protocol_8incoming_DECODE_EXCEPTIONS, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_0_29_35(__pyx_t_1, "IncomingDecodeError", (void **)&__pyx_vp_8zeroconf_9_protocol_8incoming_IncomingDecodeError, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_function_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}


#ifndef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#elif PY_MAJOR_VERSION < 3
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" void
#else
#define __Pyx_PyMODINIT_FUNC void
#endif
#else
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" PyObject *
#else
#define __Pyx_PyMODINIT_FUNC PyObject *
#endif
#endif


#if PY_MAJOR_VERSION < 3
__Pyx_PyMODINIT_FUNC initoutgoing(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC initoutgoing(void)
#else
__Pyx_PyMODINIT_FUNC PyInit_outgoing(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit_outgoing(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    #if PY_VERSION_HEX >= 0x030700A1
    static PY_INT64_T main_interpreter_id = -1;
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return (unlikely(current_id == -1)) ? -1 : 0;
    } else if (unlikely(main_interpreter_id != current_id))
    #else
    static PyInterpreterState *main_interpreter = NULL;
    PyInterpreterState *current_interpreter = PyThreadState_Get()->interp;
    if (!main_interpreter) {
        main_interpreter = current_interpreter;
    } else if (unlikely(main_interpreter != current_interpreter))
    #endif
    {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
            result = PyDict_SetItemString(moddict, to_name, value);
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__Pyx_check_single_interpreter())
        return NULL;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec_outgoing(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  unsigned int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module 'outgoing' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #elif PY_MAJOR_VERSION >= 3
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif
  __Pyx_RefNannySetupContext("__Pyx_PyMODINIT_FUNC PyInit_outgoing(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pxy_PyFrame_Initialize_Offsets
  __Pxy_PyFrame_Initialize_Offsets();
  #endif
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(WITH_THREAD) && PY_VERSION_HEX < 0x030700F0 && defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  PyEval_InitThreads();
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("outgoing", __pyx_methods, __pyx_k_Multicast_DNS_Service_Discovery, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_b);
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_cython_runtime);
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_zeroconf___protocol__outgoing) {
    if (PyObject_SetAttr(__pyx_m, __pyx_n_s_name_2, __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "zeroconf._protocol.outgoing")) {
      if (unlikely(PyDict_SetItemString(modules, "zeroconf._protocol.outgoing", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code();
  if (unlikely(__Pyx_modinit_variable_export_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  (void)__Pyx_modinit_function_export_code();
  if (unlikely(__Pyx_modinit_type_init_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely(__Pyx_modinit_type_import_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely(__Pyx_modinit_variable_import_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  (void)__Pyx_modinit_function_import_code();
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "zeroconf/_protocol/outgoing.py":23
 * """
 * 
 * import enum             # <<<<<<<<<<<<<<
 * import logging
 * from typing import Dict, List, Optional, Sequence, Tuple, Union
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_enum, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 23, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_enum, __pyx_t_1) < 0) __PYX_ERR(0, 23, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "zeroconf/_protocol/outgoing.py":24
 * 
 * import enum
 * import logging             # <<<<<<<<<<<<<<
 * from typing import Dict, List, Optional, Sequence, Tuple, Union
 * 
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_logging, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_logging, __pyx_t_1) < 0) __PYX_ERR(0, 24, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "zeroconf/_protocol/outgoing.py":25
 * import enum
 * import logging
 * from typing import Dict, List, Optional, Sequence, Tuple, Union             # <<<<<<<<<<<<<<
 * 
 * from .._cache import DNSCache
 */
  __pyx_t_1 = PyList_New(6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_Dict);
  __Pyx_GIVEREF(__pyx_n_s_Dict);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_Dict);
  __Pyx_INCREF(__pyx_n_s_List);
  __Pyx_GIVEREF(__pyx_n_s_List);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_s_List);
  __Pyx_INCREF(__pyx_n_s_Optional);
  __Pyx_GIVEREF(__pyx_n_s_Optional);
  PyList_SET_ITEM(__pyx_t_1, 2, __pyx_n_s_Optional);
  __Pyx_INCREF(__pyx_n_s_Sequence);
  __Pyx_GIVEREF(__pyx_n_s_Sequence);
  PyList_SET_ITEM(__pyx_t_1, 3, __pyx_n_s_Sequence);
  __Pyx_INCREF(__pyx_n_s_Tuple);
  __Pyx_GIVEREF(__pyx_n_s_Tuple);
  PyList_SET_ITEM(__pyx_t_1, 4, __pyx_n_s_Tuple);
  __Pyx_INCREF(__pyx_n_s_Union);
  __Pyx_GIVEREF(__pyx_n_s_Union);
  PyList_SET_ITEM(__pyx_t_1, 5, __pyx_n_s_Union);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_typing, __pyx_t_1, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_Dict); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Dict, __pyx_t_1) < 0) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_List); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_List, __pyx_t_1) < 0) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_Optional); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Optional, __pyx_t_1) < 0) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_Sequence); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Sequence, __pyx_t_1) < 0) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_Tuple); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Tuple, __pyx_t_1) < 0) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_Union); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Union, __pyx_t_1) < 0) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "zeroconf/_protocol/outgoing.py":27
 * from typing import Dict, List, Optional, Sequence, Tuple, Union
 * 
 * from .._cache import DNSCache             # <<<<<<<<<<<<<<
 * from .._dns import DNSPointer, DNSQuestion, DNSRecord
 * from .._exceptions import NamePartTooLongException
 */
  __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_DNSCache);
  __Pyx_GIVEREF(__pyx_n_s_DNSCache);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_DNSCache);
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_cache_2, __pyx_t_2, 2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_DNSCache); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_DNSCache, __pyx_t_2) < 0) __PYX_ERR(0, 27, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "zeroconf/_protocol/outgoing.py":28
 * 
 * from .._cache import DNSCache
 * from .._dns import DNSPointer, DNSQuestion, DNSRecord             # <<<<<<<<<<<<<<
 * from .._exceptions import NamePartTooLongException
 * from .._logger import log
 */
  __pyx_t_1 = PyList_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_DNSPointer);
  __Pyx_GIVEREF(__pyx_n_s_DNSPointer);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_DNSPointer);
  __Pyx_INCREF(__pyx_n_s_DNSQuestion);
  __Pyx_GIVEREF(__pyx_n_s_DNSQuestion);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_s_DNSQuestion);
  __Pyx_INCREF(__pyx_n_s_DNSRecord);
  __Pyx_GIVEREF(__pyx_n_s_DNSRecord);
  PyList_SET_ITEM(__pyx_t_1, 2, __pyx_n_s_DNSRecord);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_dns, __pyx_t_1, 2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_DNSPointer); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_DNSPointer, __pyx_t_1) < 0) __PYX_ERR(0, 28, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_DNSQuestion); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_DNSQuestion, __pyx_t_1) < 0) __PYX_ERR(0, 28, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_DNSRecord); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_DNSRecord, __pyx_t_1) < 0) __PYX_ERR(0, 28, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "zeroconf/_protocol/outgoing.py":29
 * from .._cache import DNSCache
 * from .._dns import DNSPointer, DNSQuestion, DNSRecord
 * from .._exceptions import NamePartTooLongException             # <<<<<<<<<<<<<<
 * from .._logger import log
 * from ..const import (
 */
  __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 29, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_NamePartTooLongException);
  __Pyx_GIVEREF(__pyx_n_s_NamePartTooLongException);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_NamePartTooLongException);
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_exceptions, __pyx_t_2, 2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 29, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_NamePartTooLongException); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 29, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_NamePartTooLongException, __pyx_t_2) < 0) __PYX_ERR(0, 29, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "zeroconf/_protocol/outgoing.py":30
 * from .._dns import DNSPointer, DNSQuestion, DNSRecord
 * from .._exceptions import NamePartTooLongException
 * from .._logger import log             # <<<<<<<<<<<<<<
 * from ..const import (
 *     _CLASS_UNIQUE,
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 30, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_log);
  __Pyx_GIVEREF(__pyx_n_s_log);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_log);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_logger, __pyx_t_1, 2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 30, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_log); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 30, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_log, __pyx_t_1) < 0) __PYX_ERR(0, 30, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "zeroconf/_protocol/outgoing.py":32
 * from .._logger import log
 * from ..const import (
 *     _CLASS_UNIQUE,             # <<<<<<<<<<<<<<
 *     _DNS_PACKET_HEADER_LEN,
 *     _FLAGS_QR_MASK,
 */
  __pyx_t_2 = PyList_New(8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 32, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_CLASS_UNIQUE);
  __Pyx_GIVEREF(__pyx_n_s_CLASS_UNIQUE);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_CLASS_UNIQUE);
  __Pyx_INCREF(__pyx_n_s_DNS_PACKET_HEADER_LEN);
  __Pyx_GIVEREF(__pyx_n_s_DNS_PACKET_HEADER_LEN);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_n_s_DNS_PACKET_HEADER_LEN);
  __Pyx_INCREF(__pyx_n_s_FLAGS_QR_MASK);
  __Pyx_GIVEREF(__pyx_n_s_FLAGS_QR_MASK);
  PyList_SET_ITEM(__pyx_t_2, 2, __pyx_n_s_FLAGS_QR_MASK);
  __Pyx_INCREF(__pyx_n_s_FLAGS_QR_QUERY);
  __Pyx_GIVEREF(__pyx_n_s_FLAGS_QR_QUERY);
  PyList_SET_ITEM(__pyx_t_2, 3, __pyx_n_s_FLAGS_QR_QUERY);
  __Pyx_INCREF(__pyx_n_s_FLAGS_QR_RESPONSE);
  __Pyx_GIVEREF(__pyx_n_s_FLAGS_QR_RESPONSE);
  PyList_SET_ITEM(__pyx_t_2, 4, __pyx_n_s_FLAGS_QR_RESPONSE);
  __Pyx_INCREF(__pyx_n_s_FLAGS_TC);
  __Pyx_GIVEREF(__pyx_n_s_FLAGS_TC);
  PyList_SET_ITEM(__pyx_t_2, 5, __pyx_n_s_FLAGS_TC);
  __Pyx_INCREF(__pyx_n_s_MAX_MSG_ABSOLUTE);
  __Pyx_GIVEREF(__pyx_n_s_MAX_MSG_ABSOLUTE);
  PyList_SET_ITEM(__pyx_t_2, 6, __pyx_n_s_MAX_MSG_ABSOLUTE);
  __Pyx_INCREF(__pyx_n_s_MAX_MSG_TYPICAL);
  __Pyx_GIVEREF(__pyx_n_s_MAX_MSG_TYPICAL);
  PyList_SET_ITEM(__pyx_t_2, 7, __pyx_n_s_MAX_MSG_TYPICAL);

  /* "zeroconf/_protocol/outgoing.py":31
 * from .._exceptions import NamePartTooLongException
 * from .._logger import log
 * from ..const import (             # <<<<<<<<<<<<<<
 *     _CLASS_UNIQUE,
 *     _DNS_PACKET_HEADER_LEN,
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_const, __pyx_t_2, 2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_CLASS_UNIQUE); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_As_unsigned_int(__pyx_t_2); if (unlikely((__pyx_t_3 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(0, 31, __pyx_L1_error)
  __pyx_v_8zeroconf_9_protocol_8outgoing__CLASS_UNIQUE = __pyx_t_3;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_DNS_PACKET_HEADER_LEN); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_As_unsigned_int(__pyx_t_2); if (unlikely((__pyx_t_3 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(0, 31, __pyx_L1_error)
  __pyx_v_8zeroconf_9_protocol_8outgoing__DNS_PACKET_HEADER_LEN = __pyx_t_3;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_FLAGS_QR_MASK); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_As_unsigned_int(__pyx_t_2); if (unlikely((__pyx_t_3 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(0, 31, __pyx_L1_error)
  __pyx_v_8zeroconf_9_protocol_8outgoing__FLAGS_QR_MASK = __pyx_t_3;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_FLAGS_QR_QUERY); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_As_unsigned_int(__pyx_t_2); if (unlikely((__pyx_t_3 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(0, 31, __pyx_L1_error)
  __pyx_v_8zeroconf_9_protocol_8outgoing__FLAGS_QR_QUERY = __pyx_t_3;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_FLAGS_QR_RESPONSE); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_As_unsigned_int(__pyx_t_2); if (unlikely((__pyx_t_3 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(0, 31, __pyx_L1_error)
  __pyx_v_8zeroconf_9_protocol_8outgoing__FLAGS_QR_RESPONSE = __pyx_t_3;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_FLAGS_TC); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_As_unsigned_int(__pyx_t_2); if (unlikely((__pyx_t_3 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(0, 31, __pyx_L1_error)
  __pyx_v_8zeroconf_9_protocol_8outgoing__FLAGS_TC = __pyx_t_3;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_MAX_MSG_ABSOLUTE); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_As_unsigned_int(__pyx_t_2); if (unlikely((__pyx_t_3 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(0, 31, __pyx_L1_error)
  __pyx_v_8zeroconf_9_protocol_8outgoing__MAX_MSG_ABSOLUTE = __pyx_t_3;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_MAX_MSG_TYPICAL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_As_unsigned_int(__pyx_t_2); if (unlikely((__pyx_t_3 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(0, 31, __pyx_L1_error)
  __pyx_v_8zeroconf_9_protocol_8outgoing__MAX_MSG_TYPICAL = __pyx_t_3;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "zeroconf/_protocol/outgoing.py":41
 *     _MAX_MSG_TYPICAL,
 * )
 * from .incoming import DNSIncoming             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 41, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_DNSIncoming);
  __Pyx_GIVEREF(__pyx_n_s_DNSIncoming);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_DNSIncoming);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_incoming, __pyx_t_1, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 41, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "zeroconf/_protocol/outgoing.py":44
 * 
 * 
 * class State(enum.Enum):             # <<<<<<<<<<<<<<
 *     init = 0
 *     finished = 1
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_enum); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 44, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_Enum); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 44, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 44, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_CalculateMetaclass(NULL, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 44, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_Py3MetaclassPrepare(__pyx_t_1, __pyx_t_2, __pyx_n_s_State, __pyx_n_s_State, (PyObject *) NULL, __pyx_n_s_zeroconf__protocol_outgoing, (PyObject *) NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 44, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);

  /* "zeroconf/_protocol/outgoing.py":45
 * 
 * class State(enum.Enum):
 *     init = 0             # <<<<<<<<<<<<<<
 *     finished = 1
 * 
 */
  if (__Pyx_SetNameInClass(__pyx_t_4, __pyx_n_s_init, __pyx_int_0) < 0) __PYX_ERR(0, 45, __pyx_L1_error)

  /* "zeroconf/_protocol/outgoing.py":46
 * class State(enum.Enum):
 *     init = 0
 *     finished = 1             # <<<<<<<<<<<<<<
 * 
 * 
 */
  if (__Pyx_SetNameInClass(__pyx_t_4, __pyx_n_s_finished, __pyx_int_1) < 0) __PYX_ERR(0, 46, __pyx_L1_error)

  /* "zeroconf/_protocol/outgoing.py":44
 * 
 * 
 * class State(enum.Enum):             # <<<<<<<<<<<<<<
 *     init = 0
 *     finished = 1
 */
  __pyx_t_5 = __Pyx_Py3ClassCreate(__pyx_t_1, __pyx_n_s_State, __pyx_t_2, __pyx_t_4, NULL, 0, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 44, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_State, __pyx_t_5) < 0) __PYX_ERR(0, 44, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "zeroconf/_protocol/outgoing.py":54
 * 
 *     __slots__ = (
 *         'flags',             # <<<<<<<<<<<<<<
 *         'finished',
 *         'id',
 */
  if (PyDict_SetItem((PyObject *)__pyx_ptype_8zeroconf_9_protocol_8outgoing_DNSOutgoing->tp_dict, __pyx_n_s_slots, __pyx_tuple__8) < 0) __PYX_ERR(0, 53, __pyx_L1_error)
  PyType_Modified(__pyx_ptype_8zeroconf_9_protocol_8outgoing_DNSOutgoing);

  /* "zeroconf/_protocol/outgoing.py":90
 *         self.additionals: List[DNSRecord] = []
 * 
 *     def is_query(self) -> bool:             # <<<<<<<<<<<<<<
 *         """Returns true if this is a query."""
 *         return (self.flags & _FLAGS_QR_MASK) == _FLAGS_QR_QUERY
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 90, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_return, ((PyObject*)&PyBool_Type)) < 0) __PYX_ERR(0, 90, __pyx_L1_error)
  __pyx_t_1 = __Pyx_CyFunction_New(&__pyx_mdef_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_3is_query, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_DNSOutgoing_is_query, NULL, __pyx_n_s_zeroconf__protocol_outgoing, __pyx_d, ((PyObject *)__pyx_codeobj__10)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 90, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_1, __pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_8zeroconf_9_protocol_8outgoing_DNSOutgoing->tp_dict, __pyx_n_s_is_query, __pyx_t_1) < 0) __PYX_ERR(0, 90, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_8zeroconf_9_protocol_8outgoing_DNSOutgoing);

  /* "zeroconf/_protocol/outgoing.py":94
 *         return (self.flags & _FLAGS_QR_MASK) == _FLAGS_QR_QUERY
 * 
 *     def is_response(self) -> bool:             # <<<<<<<<<<<<<<
 *         """Returns true if this is a response."""
 *         return (self.flags & _FLAGS_QR_MASK) == _FLAGS_QR_RESPONSE
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 94, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_return, ((PyObject*)&PyBool_Type)) < 0) __PYX_ERR(0, 94, __pyx_L1_error)
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_5is_response, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_DNSOutgoing_is_response, NULL, __pyx_n_s_zeroconf__protocol_outgoing, __pyx_d, ((PyObject *)__pyx_codeobj__12)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 94, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_2, __pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_8zeroconf_9_protocol_8outgoing_DNSOutgoing->tp_dict, __pyx_n_s_is_response, __pyx_t_2) < 0) __PYX_ERR(0, 94, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_8zeroconf_9_protocol_8outgoing_DNSOutgoing);

  /* "zeroconf/_protocol/outgoing.py":116
 *         )
 * 
 *     def add_question(self, record: DNSQuestion) -> None:             # <<<<<<<<<<<<<<
 *         """Adds a question"""
 *         self.questions.append(record)
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 116, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_DNSQuestion); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 116, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_record, __pyx_t_1) < 0) __PYX_ERR(0, 116, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_return, Py_None) < 0) __PYX_ERR(0, 116, __pyx_L1_error)
  __pyx_t_1 = __Pyx_CyFunction_New(&__pyx_mdef_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_9add_question, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_DNSOutgoing_add_question, NULL, __pyx_n_s_zeroconf__protocol_outgoing, __pyx_d, ((PyObject *)__pyx_codeobj__14)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 116, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_1, __pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_8zeroconf_9_protocol_8outgoing_DNSOutgoing->tp_dict, __pyx_n_s_add_question, __pyx_t_1) < 0) __PYX_ERR(0, 116, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_8zeroconf_9_protocol_8outgoing_DNSOutgoing);

  /* "zeroconf/_protocol/outgoing.py":120
 *         self.questions.append(record)
 * 
 *     def add_answer(self, inp: DNSIncoming, record: DNSRecord) -> None:             # <<<<<<<<<<<<<<
 *         """Adds an answer"""
 *         if not record.suppressed_by(inp):
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 120, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_inp, __pyx_n_u_DNSIncoming) < 0) __PYX_ERR(0, 120, __pyx_L1_error)
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_DNSRecord); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 120, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_record, __pyx_t_2) < 0) __PYX_ERR(0, 120, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_return, Py_None) < 0) __PYX_ERR(0, 120, __pyx_L1_error)
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_11add_answer, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_DNSOutgoing_add_answer, NULL, __pyx_n_s_zeroconf__protocol_outgoing, __pyx_d, ((PyObject *)__pyx_codeobj__16)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 120, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_2, __pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_8zeroconf_9_protocol_8outgoing_DNSOutgoing->tp_dict, __pyx_n_s_add_answer, __pyx_t_2) < 0) __PYX_ERR(0, 120, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_8zeroconf_9_protocol_8outgoing_DNSOutgoing);

  /* "zeroconf/_protocol/outgoing.py":125
 *             self.add_answer_at_time(record, 0)
 * 
 *     def add_answer_at_time(self, record: Optional[DNSRecord], now: Union[float, int]) -> None:             # <<<<<<<<<<<<<<
 *         """Adds an answer if it does not expire by a certain time"""
 *         if record is not None and (now == 0 or not record.is_expired(now)):
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 125, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_Optional); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 125, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_DNSRecord); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 125, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetItem(__pyx_t_1, __pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 125, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_record, __pyx_t_5) < 0) __PYX_ERR(0, 125, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_Union); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 125, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 125, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(((PyObject *)(&PyFloat_Type)));
  __Pyx_GIVEREF(((PyObject *)(&PyFloat_Type)));
  PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)(&PyFloat_Type)));
  __Pyx_INCREF(((PyObject *)(&PyInt_Type)));
  __Pyx_GIVEREF(((PyObject *)(&PyInt_Type)));
  PyTuple_SET_ITEM(__pyx_t_4, 1, ((PyObject *)(&PyInt_Type)));
  __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_t_5, __pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 125, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_now, __pyx_t_1) < 0) __PYX_ERR(0, 125, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_return, Py_None) < 0) __PYX_ERR(0, 125, __pyx_L1_error)
  __pyx_t_1 = __Pyx_CyFunction_New(&__pyx_mdef_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_13add_answer_at_time, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_DNSOutgoing_add_answer_at_time, NULL, __pyx_n_s_zeroconf__protocol_outgoing, __pyx_d, ((PyObject *)__pyx_codeobj__18)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 125, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_1, __pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_8zeroconf_9_protocol_8outgoing_DNSOutgoing->tp_dict, __pyx_n_s_add_answer_at_time, __pyx_t_1) < 0) __PYX_ERR(0, 125, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_8zeroconf_9_protocol_8outgoing_DNSOutgoing);

  /* "zeroconf/_protocol/outgoing.py":130
 *             self.answers.append((record, now))
 * 
 *     def add_authorative_answer(self, record: DNSPointer) -> None:             # <<<<<<<<<<<<<<
 *         """Adds an authoritative answer"""
 *         self.authorities.append(record)
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 130, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_DNSPointer); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 130, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_record, __pyx_t_2) < 0) __PYX_ERR(0, 130, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_return, Py_None) < 0) __PYX_ERR(0, 130, __pyx_L1_error)
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_15add_authorative_answer, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_DNSOutgoing_add_authorative_answ, NULL, __pyx_n_s_zeroconf__protocol_outgoing, __pyx_d, ((PyObject *)__pyx_codeobj__20)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 130, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_2, __pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_8zeroconf_9_protocol_8outgoing_DNSOutgoing->tp_dict, __pyx_n_s_add_authorative_answer, __pyx_t_2) < 0) __PYX_ERR(0, 130, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_8zeroconf_9_protocol_8outgoing_DNSOutgoing);

  /* "zeroconf/_protocol/outgoing.py":134
 *         self.authorities.append(record)
 * 
 *     def add_additional_answer(self, record: DNSRecord) -> None:             # <<<<<<<<<<<<<<
 *         """Adds an additional answer
 * 
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 134, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_DNSRecord); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 134, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_record, __pyx_t_1) < 0) __PYX_ERR(0, 134, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_return, Py_None) < 0) __PYX_ERR(0, 134, __pyx_L1_error)
  __pyx_t_1 = __Pyx_CyFunction_New(&__pyx_mdef_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_17add_additional_answer, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_DNSOutgoing_add_additional_answe, NULL, __pyx_n_s_zeroconf__protocol_outgoing, __pyx_d, ((PyObject *)__pyx_codeobj__22)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 134, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_1, __pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_8zeroconf_9_protocol_8outgoing_DNSOutgoing->tp_dict, __pyx_n_s_add_additional_answer, __pyx_t_1) < 0) __PYX_ERR(0, 134, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_8zeroconf_9_protocol_8outgoing_DNSOutgoing);

  /* "zeroconf/_protocol/outgoing.py":172
 *         self.additionals.append(record)
 * 
 *     def add_question_or_one_cache(             # <<<<<<<<<<<<<<
 *         self, cache: DNSCache, now: float, name: str, type_: int, class_: int
 *     ) -> None:
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 172, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);

  /* "zeroconf/_protocol/outgoing.py":173
 * 
 *     def add_question_or_one_cache(
 *         self, cache: DNSCache, now: float, name: str, type_: int, class_: int             # <<<<<<<<<<<<<<
 *     ) -> None:
 *         """Add a question if it is not already cached."""
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_DNSCache); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 173, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_cache, __pyx_t_2) < 0) __PYX_ERR(0, 172, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_now, __pyx_n_u_float) < 0) __PYX_ERR(0, 172, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_name, __pyx_n_u_unicode) < 0) __PYX_ERR(0, 172, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_type, __pyx_n_u_int) < 0) __PYX_ERR(0, 172, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_class, __pyx_n_u_int) < 0) __PYX_ERR(0, 172, __pyx_L1_error)

  /* "zeroconf/_protocol/outgoing.py":174
 *     def add_question_or_one_cache(
 *         self, cache: DNSCache, now: float, name: str, type_: int, class_: int
 *     ) -> None:             # <<<<<<<<<<<<<<
 *         """Add a question if it is not already cached."""
 *         cached_entry = cache.get_by_details(name, type_, class_)
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_return, Py_None) < 0) __PYX_ERR(0, 172, __pyx_L1_error)

  /* "zeroconf/_protocol/outgoing.py":172
 *         self.additionals.append(record)
 * 
 *     def add_question_or_one_cache(             # <<<<<<<<<<<<<<
 *         self, cache: DNSCache, now: float, name: str, type_: int, class_: int
 *     ) -> None:
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_19add_question_or_one_cache, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_DNSOutgoing_add_question_or_one, NULL, __pyx_n_s_zeroconf__protocol_outgoing, __pyx_d, ((PyObject *)__pyx_codeobj__24)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 172, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_2, __pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_8zeroconf_9_protocol_8outgoing_DNSOutgoing->tp_dict, __pyx_n_s_add_question_or_one_cache, __pyx_t_2) < 0) __PYX_ERR(0, 172, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_8zeroconf_9_protocol_8outgoing_DNSOutgoing);

  /* "zeroconf/_protocol/outgoing.py":182
 *             self.add_answer_at_time(cached_entry, now)
 * 
 *     def add_question_or_all_cache(             # <<<<<<<<<<<<<<
 *         self, cache: DNSCache, now: float, name: str, type_: int, class_: int
 *     ) -> None:
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 182, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "zeroconf/_protocol/outgoing.py":183
 * 
 *     def add_question_or_all_cache(
 *         self, cache: DNSCache, now: float, name: str, type_: int, class_: int             # <<<<<<<<<<<<<<
 *     ) -> None:
 *         """Add a question if it is not already cached.
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_DNSCache); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 183, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_cache, __pyx_t_1) < 0) __PYX_ERR(0, 182, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_now, __pyx_n_u_float) < 0) __PYX_ERR(0, 182, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_name, __pyx_n_u_unicode) < 0) __PYX_ERR(0, 182, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_type, __pyx_n_u_int) < 0) __PYX_ERR(0, 182, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_class, __pyx_n_u_int) < 0) __PYX_ERR(0, 182, __pyx_L1_error)

  /* "zeroconf/_protocol/outgoing.py":184
 *     def add_question_or_all_cache(
 *         self, cache: DNSCache, now: float, name: str, type_: int, class_: int
 *     ) -> None:             # <<<<<<<<<<<<<<
 *         """Add a question if it is not already cached.
 *         This is currently only used for IPv6 addresses.
 */
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_return, Py_None) < 0) __PYX_ERR(0, 182, __pyx_L1_error)

  /* "zeroconf/_protocol/outgoing.py":182
 *             self.add_answer_at_time(cached_entry, now)
 * 
 *     def add_question_or_all_cache(             # <<<<<<<<<<<<<<
 *         self, cache: DNSCache, now: float, name: str, type_: int, class_: int
 *     ) -> None:
 */
  __pyx_t_1 = __Pyx_CyFunction_New(&__pyx_mdef_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_21add_question_or_all_cache, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_DNSOutgoing_add_question_or_all, NULL, __pyx_n_s_zeroconf__protocol_outgoing, __pyx_d, ((PyObject *)__pyx_codeobj__26)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 182, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_1, __pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_8zeroconf_9_protocol_8outgoing_DNSOutgoing->tp_dict, __pyx_n_s_add_question_or_all_cache, __pyx_t_1) < 0) __PYX_ERR(0, 182, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_8zeroconf_9_protocol_8outgoing_DNSOutgoing);

  /* "zeroconf/_protocol/outgoing.py":208
 *         self.data[index] = value.to_bytes(2, 'big')
 * 
 *     def write_short(self, value: int) -> None:             # <<<<<<<<<<<<<<
 *         """Writes an unsigned short to the packet"""
 *         self.data.append(value.to_bytes(2, 'big'))
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 208, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_value, __pyx_n_u_int) < 0) __PYX_ERR(0, 208, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_return, Py_None) < 0) __PYX_ERR(0, 208, __pyx_L1_error)
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_23write_short, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_DNSOutgoing_write_short, NULL, __pyx_n_s_zeroconf__protocol_outgoing, __pyx_d, ((PyObject *)__pyx_codeobj__28)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 208, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_2, __pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_8zeroconf_9_protocol_8outgoing_DNSOutgoing->tp_dict, __pyx_n_s_write_short, __pyx_t_2) < 0) __PYX_ERR(0, 208, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_8zeroconf_9_protocol_8outgoing_DNSOutgoing);

  /* "zeroconf/_protocol/outgoing.py":218
 *         self.size += 4
 * 
 *     def write_string(self, value: bytes) -> None:             # <<<<<<<<<<<<<<
 *         """Writes a string to the packet"""
 *         assert isinstance(value, bytes)
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 218, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_value, __pyx_n_u_bytes) < 0) __PYX_ERR(0, 218, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_return, Py_None) < 0) __PYX_ERR(0, 218, __pyx_L1_error)
  __pyx_t_1 = __Pyx_CyFunction_New(&__pyx_mdef_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_25write_string, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_DNSOutgoing_write_string, NULL, __pyx_n_s_zeroconf__protocol_outgoing, __pyx_d, ((PyObject *)__pyx_codeobj__30)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 218, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_1, __pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_8zeroconf_9_protocol_8outgoing_DNSOutgoing->tp_dict, __pyx_n_s_write_string, __pyx_t_1) < 0) __PYX_ERR(0, 218, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_8zeroconf_9_protocol_8outgoing_DNSOutgoing);

  /* "zeroconf/_protocol/outgoing.py":224
 *         self.size += len(value)
 * 
 *     def _write_utf(self, s: str) -> None:             # <<<<<<<<<<<<<<
 *         """Writes a UTF-8 string of a given length to the packet"""
 *         utfstr = s.encode('utf-8')
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 224, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_s, __pyx_n_u_unicode) < 0) __PYX_ERR(0, 224, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_return, Py_None) < 0) __PYX_ERR(0, 224, __pyx_L1_error)
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_27_write_utf, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_DNSOutgoing__write_utf, NULL, __pyx_n_s_zeroconf__protocol_outgoing, __pyx_d, ((PyObject *)__pyx_codeobj__32)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 224, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_2, __pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_8zeroconf_9_protocol_8outgoing_DNSOutgoing->tp_dict, __pyx_n_s_write_utf, __pyx_t_2) < 0) __PYX_ERR(0, 224, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_8zeroconf_9_protocol_8outgoing_DNSOutgoing);

  /* "zeroconf/_protocol/outgoing.py":233
 *         self.write_string(utfstr)
 * 
 *     def write_character_string(self, value: bytes) -> None:             # <<<<<<<<<<<<<<
 *         assert isinstance(value, bytes)
 *         length = len(value)
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 233, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_value, __pyx_n_u_bytes) < 0) __PYX_ERR(0, 233, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_return, Py_None) < 0) __PYX_ERR(0, 233, __pyx_L1_error)
  __pyx_t_1 = __Pyx_CyFunction_New(&__pyx_mdef_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_29write_character_string, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_DNSOutgoing_write_character_stri, NULL, __pyx_n_s_zeroconf__protocol_outgoing, __pyx_d, ((PyObject *)__pyx_codeobj__34)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 233, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_1, __pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_8zeroconf_9_protocol_8outgoing_DNSOutgoing->tp_dict, __pyx_n_s_write_character_string, __pyx_t_1) < 0) __PYX_ERR(0, 233, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_8zeroconf_9_protocol_8outgoing_DNSOutgoing);

  /* "zeroconf/_protocol/outgoing.py":241
 *         self.write_string(value)
 * 
 *     def write_name(self, name: str) -> None:             # <<<<<<<<<<<<<<
 *         """
 *         Write names to packet
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 241, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_name, __pyx_n_u_unicode) < 0) __PYX_ERR(0, 241, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_return, Py_None) < 0) __PYX_ERR(0, 241, __pyx_L1_error)
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_31write_name, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_DNSOutgoing_write_name, NULL, __pyx_n_s_zeroconf__protocol_outgoing, __pyx_d, ((PyObject *)__pyx_codeobj__36)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 241, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_2, __pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_8zeroconf_9_protocol_8outgoing_DNSOutgoing->tp_dict, __pyx_n_s_write_name, __pyx_t_2) < 0) __PYX_ERR(0, 241, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_8zeroconf_9_protocol_8outgoing_DNSOutgoing);

  /* "zeroconf/_protocol/outgoing.py":294
 *             self.write_short(record.class_)
 * 
 *     def _write_ttl(self, record: DNSRecord, now: float) -> None:             # <<<<<<<<<<<<<<
 *         """Write out the record ttl."""
 *         self._write_int(record.ttl if now == 0 else record.get_remaining_ttl(now))
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 294, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_DNSRecord); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 294, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_record, __pyx_t_1) < 0) __PYX_ERR(0, 294, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_now, __pyx_n_u_float) < 0) __PYX_ERR(0, 294, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_return, Py_None) < 0) __PYX_ERR(0, 294, __pyx_L1_error)
  __pyx_t_1 = __Pyx_CyFunction_New(&__pyx_mdef_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_33_write_ttl, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_DNSOutgoing__write_ttl, NULL, __pyx_n_s_zeroconf__protocol_outgoing, __pyx_d, ((PyObject *)__pyx_codeobj__38)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 294, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_1, __pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_8zeroconf_9_protocol_8outgoing_DNSOutgoing->tp_dict, __pyx_n_s_write_ttl, __pyx_t_1) < 0) __PYX_ERR(0, 294, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_8zeroconf_9_protocol_8outgoing_DNSOutgoing);

  /* "zeroconf/_protocol/outgoing.py":298
 *         self._write_int(record.ttl if now == 0 else record.get_remaining_ttl(now))
 * 
 *     def _write_record(self, record: DNSRecord, now: float) -> bool:             # <<<<<<<<<<<<<<
 *         """Writes a record (answer, authoritative answer, additional) to
 *         the packet.  Returns True on success, or False if we did not
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 298, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_DNSRecord); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 298, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_record, __pyx_t_2) < 0) __PYX_ERR(0, 298, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_now, __pyx_n_u_float) < 0) __PYX_ERR(0, 298, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_return, ((PyObject*)&PyBool_Type)) < 0) __PYX_ERR(0, 298, __pyx_L1_error)
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_35_write_record, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_DNSOutgoing__write_record, NULL, __pyx_n_s_zeroconf__protocol_outgoing, __pyx_d, ((PyObject *)__pyx_codeobj__40)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 298, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_2, __pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_8zeroconf_9_protocol_8outgoing_DNSOutgoing->tp_dict, __pyx_n_s_write_record, __pyx_t_2) < 0) __PYX_ERR(0, 298, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_8zeroconf_9_protocol_8outgoing_DNSOutgoing);

  /* "zeroconf/_protocol/outgoing.py":369
 *         )
 * 
 *     def packets(self) -> List[bytes]:             # <<<<<<<<<<<<<<
 *         """Returns a list of bytestrings containing the packets' bytes
 * 
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 369, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_List); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 369, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyObject_GetItem(__pyx_t_1, ((PyObject *)(&PyBytes_Type))); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 369, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_return, __pyx_t_4) < 0) __PYX_ERR(0, 369, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_37packets, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_DNSOutgoing_packets, NULL, __pyx_n_s_zeroconf__protocol_outgoing, __pyx_d, ((PyObject *)__pyx_codeobj__42)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 369, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_4, __pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_8zeroconf_9_protocol_8outgoing_DNSOutgoing->tp_dict, __pyx_n_s_packets, __pyx_t_4) < 0) __PYX_ERR(0, 369, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  PyType_Modified(__pyx_ptype_8zeroconf_9_protocol_8outgoing_DNSOutgoing);

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_39__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_DNSOutgoing___reduce_cython, NULL, __pyx_n_s_zeroconf__protocol_outgoing, __pyx_d, ((PyObject *)__pyx_codeobj__44)); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_8zeroconf_9_protocol_8outgoing_DNSOutgoing->tp_dict, __pyx_n_s_reduce_cython, __pyx_t_4) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  PyType_Modified(__pyx_ptype_8zeroconf_9_protocol_8outgoing_DNSOutgoing);

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_DNSOutgoing, (type(self), 0x3d398d7, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_DNSOutgoing__set_state(self, __pyx_state)
 */
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_8zeroconf_9_protocol_8outgoing_11DNSOutgoing_41__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_DNSOutgoing___setstate_cython, NULL, __pyx_n_s_zeroconf__protocol_outgoing, __pyx_d, ((PyObject *)__pyx_codeobj__46)); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_8zeroconf_9_protocol_8outgoing_DNSOutgoing->tp_dict, __pyx_n_s_setstate_cython, __pyx_t_4) < 0) __PYX_ERR(2, 16, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  PyType_Modified(__pyx_ptype_8zeroconf_9_protocol_8outgoing_DNSOutgoing);

  /* "(tree fragment)":1
 * def __pyx_unpickle_DNSOutgoing(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_8zeroconf_9_protocol_8outgoing_1__pyx_unpickle_DNSOutgoing, 0, __pyx_n_s_pyx_unpickle_DNSOutgoing, NULL, __pyx_n_s_zeroconf__protocol_outgoing, __pyx_d, ((PyObject *)__pyx_codeobj__48)); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_DNSOutgoing, __pyx_t_4) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "zeroconf/_protocol/outgoing.py":1
 * """ Multicast DNS Service Discovery for Python, v0.14-wmcbrine             # <<<<<<<<<<<<<<
 *     Copyright 2003 Paul Scott-Murphy, 2014 William McBrine
 * 
 */
  __pyx_t_4 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_4) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init zeroconf._protocol.outgoing", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_CLEAR(__pyx_m);
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init zeroconf._protocol.outgoing");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (__Pyx_PyUnicode_GET_LENGTH(**name) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (__Pyx_PyUnicode_GET_LENGTH(**argname) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* PyDictVersioning */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* GetModuleGlobalName */
#if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1
    result = _PyDict_GetItem_KnownHash(__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    } else if (unlikely(PyErr_Occurred())) {
        return NULL;
    }
#else
    result = PyDict_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
#endif
#else
    result = PyObject_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* ArgTypeTest */
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (exact) {
        #if PY_MAJOR_VERSION == 2
        if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    }
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
    return 0;
}

/* PyCFunctionFastCall */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)(void*)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)(void*)meth)) (self, args, nargs);
    }
}
#endif

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = __Pyx_PyFrame_GetLocalsplus(f);
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = Py_TYPE(func)->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCall2Args */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2) {
    PyObject *args, *result = NULL;
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyFunction_FastCall(function, args, 2);
    }
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyCFunction_FastCall(function, args, 2);
    }
    #endif
    args = PyTuple_New(2);
    if (unlikely(!args)) goto done;
    Py_INCREF(arg1);
    PyTuple_SET_ITEM(args, 0, arg1);
    Py_INCREF(arg2);
    PyTuple_SET_ITEM(args, 1, arg2);
    Py_INCREF(function);
    result = __Pyx_PyObject_Call(function, args, NULL);
    Py_DECREF(args);
    Py_DECREF(function);
done:
    return result;
}

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallOneArg */
#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (__Pyx_PyFastCFunction_Check(func)) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* PyIntCompare */
static CYTHON_INLINE PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, CYTHON_UNUSED long inplace) {
    if (op1 == op2) {
        Py_RETURN_TRUE;
    }
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long a = PyInt_AS_LONG(op1);
        if (a == b) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        int unequal;
        unsigned long uintval;
        Py_ssize_t size = Py_SIZE(op1);
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        if (intval == 0) {
            if (size == 0) Py_RETURN_TRUE; else Py_RETURN_FALSE;
        } else if (intval < 0) {
            if (size >= 0)
                Py_RETURN_FALSE;
            intval = -intval;
            size = -size;
        } else {
            if (size <= 0)
                Py_RETURN_FALSE;
        }
        uintval = (unsigned long) intval;
#if PyLong_SHIFT * 4 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 4)) {
            unequal = (size != 5) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[4] != ((uintval >> (4 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 3 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 3)) {
            unequal = (size != 4) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 2 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 2)) {
            unequal = (size != 3) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 1 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 1)) {
            unequal = (size != 2) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
            unequal = (size != 1) || (((unsigned long) digits[0]) != (uintval & (unsigned long) PyLong_MASK));
        if (unequal == 0) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
        if ((double)a == (double)b) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    return (
        PyObject_RichCompare(op1, op2, Py_EQ));
}

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* RaiseException */
#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_FAST_THREAD_STATE
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#else
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* unicode_tailmatch */
static int __Pyx_PyUnicode_TailmatchTuple(PyObject* s, PyObject* substrings,
                                          Py_ssize_t start, Py_ssize_t end, int direction) {
    Py_ssize_t i, count = PyTuple_GET_SIZE(substrings);
    for (i = 0; i < count; i++) {
        Py_ssize_t result;
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        result = PyUnicode_Tailmatch(s, PyTuple_GET_ITEM(substrings, i),
                                     start, end, direction);
#else
        PyObject* sub = PySequence_ITEM(substrings, i);
        if (unlikely(!sub)) return -1;
        result = PyUnicode_Tailmatch(s, sub, start, end, direction);
        Py_DECREF(sub);
#endif
        if (result) {
            return (int) result;
        }
    }
    return 0;
}
static int __Pyx_PyUnicode_Tailmatch(PyObject* s, PyObject* substr,
                                     Py_ssize_t start, Py_ssize_t end, int direction) {
    if (unlikely(PyTuple_Check(substr))) {
        return __Pyx_PyUnicode_TailmatchTuple(s, substr, start, end, direction);
    }
    return (int) PyUnicode_Tailmatch(s, substr, start, end, direction);
}

/* PyUnicode_Substring */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_Substring(
            PyObject* text, Py_ssize_t start, Py_ssize_t stop) {
    Py_ssize_t length;
    if (unlikely(__Pyx_PyUnicode_READY(text) == -1)) return NULL;
    length = __Pyx_PyUnicode_GET_LENGTH(text);
    if (start < 0) {
        start += length;
        if (start < 0)
            start = 0;
    }
    if (stop < 0)
        stop += length;
    else if (stop > length)
        stop = length;
    if (stop <= start)
        return __Pyx_NewRef(__pyx_empty_unicode);
#if CYTHON_PEP393_ENABLED
    return PyUnicode_FromKindAndData(PyUnicode_KIND(text),
        PyUnicode_1BYTE_DATA(text) + start*PyUnicode_KIND(text), stop-start);
#else
    return PyUnicode_FromUnicode(PyUnicode_AS_UNICODE(text)+start, stop-start);
#endif
}

/* SliceObject */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(PyObject* obj,
        Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** _py_start, PyObject** _py_stop, PyObject** _py_slice,
        int has_cstart, int has_cstop, CYTHON_UNUSED int wraparound) {
#if CYTHON_USE_TYPE_SLOTS
    PyMappingMethods* mp;
#if PY_MAJOR_VERSION < 3
    PySequenceMethods* ms = Py_TYPE(obj)->tp_as_sequence;
    if (likely(ms && ms->sq_slice)) {
        if (!has_cstart) {
            if (_py_start && (*_py_start != Py_None)) {
                cstart = __Pyx_PyIndex_AsSsize_t(*_py_start);
                if ((cstart == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstart = 0;
        }
        if (!has_cstop) {
            if (_py_stop && (*_py_stop != Py_None)) {
                cstop = __Pyx_PyIndex_AsSsize_t(*_py_stop);
                if ((cstop == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstop = PY_SSIZE_T_MAX;
        }
        if (wraparound && unlikely((cstart < 0) | (cstop < 0)) && likely(ms->sq_length)) {
            Py_ssize_t l = ms->sq_length(obj);
            if (likely(l >= 0)) {
                if (cstop < 0) {
                    cstop += l;
                    if (cstop < 0) cstop = 0;
                }
                if (cstart < 0) {
                    cstart += l;
                    if (cstart < 0) cstart = 0;
                }
            } else {
                if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                    goto bad;
                PyErr_Clear();
            }
        }
        return ms->sq_slice(obj, cstart, cstop);
    }
#endif
    mp = Py_TYPE(obj)->tp_as_mapping;
    if (likely(mp && mp->mp_subscript))
#endif
    {
        PyObject* result;
        PyObject *py_slice, *py_start, *py_stop;
        if (_py_slice) {
            py_slice = *_py_slice;
        } else {
            PyObject* owned_start = NULL;
            PyObject* owned_stop = NULL;
            if (_py_start) {
                py_start = *_py_start;
            } else {
                if (has_cstart) {
                    owned_start = py_start = PyInt_FromSsize_t(cstart);
                    if (unlikely(!py_start)) goto bad;
                } else
                    py_start = Py_None;
            }
            if (_py_stop) {
                py_stop = *_py_stop;
            } else {
                if (has_cstop) {
                    owned_stop = py_stop = PyInt_FromSsize_t(cstop);
                    if (unlikely(!py_stop)) {
                        Py_XDECREF(owned_start);
                        goto bad;
                    }
                } else
                    py_stop = Py_None;
            }
            py_slice = PySlice_New(py_start, py_stop, Py_None);
            Py_XDECREF(owned_start);
            Py_XDECREF(owned_stop);
            if (unlikely(!py_slice)) goto bad;
        }
#if CYTHON_USE_TYPE_SLOTS
        result = mp->mp_subscript(obj, py_slice);
#else
        result = PyObject_GetItem(obj, py_slice);
#endif
        if (!_py_slice) {
            Py_DECREF(py_slice);
        }
        return result;
    }
    PyErr_Format(PyExc_TypeError,
        "'%.200s' object is unsliceable", Py_TYPE(obj)->tp_name);
bad:
    return NULL;
}

/* PyIntBinop */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_OrObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, int inplace, int zerodivision_check) {
    (void)inplace;
    (void)zerodivision_check;
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long a = PyInt_AS_LONG(op1);
            return PyInt_FromLong(a | b);
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a, x;
#ifdef HAVE_LONG_LONG
        const PY_LONG_LONG llb = intval;
        PY_LONG_LONG lla, llx;
#endif
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        if (likely(__Pyx_sst_abs(size) <= 1)) {
            a = likely(size) ? digits[0] : 0;
            if (size == -1) a = -a;
        } else {
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                default: return PyLong_Type.tp_as_number->nb_or(op1, op2);
            }
        }
                x = a | b;
            return PyLong_FromLong(x);
#ifdef HAVE_LONG_LONG
        long_long:
                llx = lla | llb;
            return PyLong_FromLongLong(llx);
#endif
        
        
    }
    #endif
    return (inplace ? PyNumber_InPlaceOr : PyNumber_Or)(op1, op2);
}
#endif

/* PyIntBinop */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_RshiftObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, int inplace, int zerodivision_check) {
    (void)inplace;
    (void)zerodivision_check;
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long a = PyInt_AS_LONG(op1);
            return PyInt_FromLong(a >> b);
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a, x;
#ifdef HAVE_LONG_LONG
        const PY_LONG_LONG llb = intval;
        PY_LONG_LONG lla, llx;
#endif
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        if (likely(__Pyx_sst_abs(size) <= 1)) {
            a = likely(size) ? digits[0] : 0;
            if (size == -1) a = -a;
        } else {
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                default: return PyLong_Type.tp_as_number->nb_rshift(op1, op2);
            }
        }
                x = a >> b;
            return PyLong_FromLong(x);
#ifdef HAVE_LONG_LONG
        long_long:
                llx = lla >> llb;
            return PyLong_FromLongLong(llx);
#endif
        
        
    }
    #endif
    return (inplace ? PyNumber_InPlaceRshift : PyNumber_Rshift)(op1, op2);
}
#endif

/* PyIntBinop */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_AndObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, int inplace, int zerodivision_check) {
    (void)inplace;
    (void)zerodivision_check;
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long a = PyInt_AS_LONG(op1);
            return PyInt_FromLong(a & b);
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a, x;
#ifdef HAVE_LONG_LONG
        const PY_LONG_LONG llb = intval;
        PY_LONG_LONG lla, llx;
#endif
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        if (likely(__Pyx_sst_abs(size) <= 1)) {
            a = likely(size) ? digits[0] : 0;
            if (size == -1) a = -a;
        } else {
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                default: return PyLong_Type.tp_as_number->nb_and(op1, op2);
            }
        }
                x = a & b;
            return PyLong_FromLong(x);
#ifdef HAVE_LONG_LONG
        long_long:
                llx = lla & llb;
            return PyLong_FromLongLong(llx);
#endif
        
        
    }
    #endif
    return (inplace ? PyNumber_InPlaceAnd : PyNumber_And)(op1, op2);
}
#endif

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* None */
static CYTHON_INLINE void __Pyx_RaiseClosureNameError(const char *varname) {
    PyErr_Format(PyExc_NameError, "free variable '%s' referenced before assignment in enclosing scope", varname);
}

/* SliceTupleAndList */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE void __Pyx_crop_slice(Py_ssize_t* _start, Py_ssize_t* _stop, Py_ssize_t* _length) {
    Py_ssize_t start = *_start, stop = *_stop, length = *_length;
    if (start < 0) {
        start += length;
        if (start < 0)
            start = 0;
    }
    if (stop < 0)
        stop += length;
    else if (stop > length)
        stop = length;
    *_length = stop - start;
    *_start = start;
    *_stop = stop;
}
static CYTHON_INLINE void __Pyx_copy_object_array(PyObject** CYTHON_RESTRICT src, PyObject** CYTHON_RESTRICT dest, Py_ssize_t length) {
    PyObject *v;
    Py_ssize_t i;
    for (i = 0; i < length; i++) {
        v = dest[i] = src[i];
        Py_INCREF(v);
    }
}
static CYTHON_INLINE PyObject* __Pyx_PyList_GetSlice(
            PyObject* src, Py_ssize_t start, Py_ssize_t stop) {
    PyObject* dest;
    Py_ssize_t length = PyList_GET_SIZE(src);
    __Pyx_crop_slice(&start, &stop, &length);
    if (unlikely(length <= 0))
        return PyList_New(0);
    dest = PyList_New(length);
    if (unlikely(!dest))
        return NULL;
    __Pyx_copy_object_array(
        ((PyListObject*)src)->ob_item + start,
        ((PyListObject*)dest)->ob_item,
        length);
    return dest;
}
static CYTHON_INLINE PyObject* __Pyx_PyTuple_GetSlice(
            PyObject* src, Py_ssize_t start, Py_ssize_t stop) {
    PyObject* dest;
    Py_ssize_t length = PyTuple_GET_SIZE(src);
    __Pyx_crop_slice(&start, &stop, &length);
    if (unlikely(length <= 0))
        return PyTuple_New(0);
    dest = PyTuple_New(length);
    if (unlikely(!dest))
        return NULL;
    __Pyx_copy_object_array(
        ((PyTupleObject*)src)->ob_item + start,
        ((PyTupleObject*)dest)->ob_item,
        length);
    return dest;
}
#endif

/* SliceObject */
static CYTHON_INLINE int __Pyx_PyObject_SetSlice(PyObject* obj, PyObject* value,
        Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** _py_start, PyObject** _py_stop, PyObject** _py_slice,
        int has_cstart, int has_cstop, CYTHON_UNUSED int wraparound) {
#if CYTHON_USE_TYPE_SLOTS
    PyMappingMethods* mp;
#if PY_MAJOR_VERSION < 3
    PySequenceMethods* ms = Py_TYPE(obj)->tp_as_sequence;
    if (likely(ms && ms->sq_ass_slice)) {
        if (!has_cstart) {
            if (_py_start && (*_py_start != Py_None)) {
                cstart = __Pyx_PyIndex_AsSsize_t(*_py_start);
                if ((cstart == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstart = 0;
        }
        if (!has_cstop) {
            if (_py_stop && (*_py_stop != Py_None)) {
                cstop = __Pyx_PyIndex_AsSsize_t(*_py_stop);
                if ((cstop == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstop = PY_SSIZE_T_MAX;
        }
        if (wraparound && unlikely((cstart < 0) | (cstop < 0)) && likely(ms->sq_length)) {
            Py_ssize_t l = ms->sq_length(obj);
            if (likely(l >= 0)) {
                if (cstop < 0) {
                    cstop += l;
                    if (cstop < 0) cstop = 0;
                }
                if (cstart < 0) {
                    cstart += l;
                    if (cstart < 0) cstart = 0;
                }
            } else {
                if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                    goto bad;
                PyErr_Clear();
            }
        }
        return ms->sq_ass_slice(obj, cstart, cstop, value);
    }
#endif
    mp = Py_TYPE(obj)->tp_as_mapping;
    if (likely(mp && mp->mp_ass_subscript))
#endif
    {
        int result;
        PyObject *py_slice, *py_start, *py_stop;
        if (_py_slice) {
            py_slice = *_py_slice;
        } else {
            PyObject* owned_start = NULL;
            PyObject* owned_stop = NULL;
            if (_py_start) {
                py_start = *_py_start;
            } else {
                if (has_cstart) {
                    owned_start = py_start = PyInt_FromSsize_t(cstart);
                    if (unlikely(!py_start)) goto bad;
                } else
                    py_start = Py_None;
            }
            if (_py_stop) {
                py_stop = *_py_stop;
            } else {
                if (has_cstop) {
                    owned_stop = py_stop = PyInt_FromSsize_t(cstop);
                    if (unlikely(!py_stop)) {
                        Py_XDECREF(owned_start);
                        goto bad;
                    }
                } else
                    py_stop = Py_None;
            }
            py_slice = PySlice_New(py_start, py_stop, Py_None);
            Py_XDECREF(owned_start);
            Py_XDECREF(owned_stop);
            if (unlikely(!py_slice)) goto bad;
        }
#if CYTHON_USE_TYPE_SLOTS
        result = mp->mp_ass_subscript(obj, py_slice, value);
#else
        result = value ? PyObject_SetItem(obj, py_slice, value) : PyObject_DelItem(obj, py_slice);
#endif
        if (!_py_slice) {
            Py_DECREF(py_slice);
        }
        return result;
    }
    PyErr_Format(PyExc_TypeError,
        "'%.200s' object does not support slice %.10s",
        Py_TYPE(obj)->tp_name, value ? "assignment" : "deletion");
bad:
    return -1;
}

/* IterFinish */
static CYTHON_INLINE int __Pyx_IterFinish(void) {
#if CYTHON_FAST_THREAD_STATE
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject* exc_type = tstate->curexc_type;
    if (unlikely(exc_type)) {
        if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) {
            PyObject *exc_value, *exc_tb;
            exc_value = tstate->curexc_value;
            exc_tb = tstate->curexc_traceback;
            tstate->curexc_type = 0;
            tstate->curexc_value = 0;
            tstate->curexc_traceback = 0;
            Py_DECREF(exc_type);
            Py_XDECREF(exc_value);
            Py_XDECREF(exc_tb);
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#else
    if (unlikely(PyErr_Occurred())) {
        if (likely(PyErr_ExceptionMatches(PyExc_StopIteration))) {
            PyErr_Clear();
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#endif
}

/* PyObjectCallNoArg */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#if defined(__Pyx_CyFunction_USED) && defined(NDEBUG)
    if (likely(PyCFunction_Check(func) || __Pyx_CyFunction_Check(func)))
#else
    if (likely(PyCFunction_Check(func)))
#endif
    {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* PyObjectGetMethod */
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method) {
    PyObject *attr;
#if CYTHON_UNPACK_METHODS && CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_PYTYPE_LOOKUP
    PyTypeObject *tp = Py_TYPE(obj);
    PyObject *descr;
    descrgetfunc f = NULL;
    PyObject **dictptr, *dict;
    int meth_found = 0;
    assert (*method == NULL);
    if (unlikely(tp->tp_getattro != PyObject_GenericGetAttr)) {
        attr = __Pyx_PyObject_GetAttrStr(obj, name);
        goto try_unpack;
    }
    if (unlikely(tp->tp_dict == NULL) && unlikely(PyType_Ready(tp) < 0)) {
        return 0;
    }
    descr = _PyType_Lookup(tp, name);
    if (likely(descr != NULL)) {
        Py_INCREF(descr);
#if PY_MAJOR_VERSION >= 3
        #ifdef __Pyx_CyFunction_USED
        if (likely(PyFunction_Check(descr) || (Py_TYPE(descr) == &PyMethodDescr_Type) || __Pyx_CyFunction_Check(descr)))
        #else
        if (likely(PyFunction_Check(descr) || (Py_TYPE(descr) == &PyMethodDescr_Type)))
        #endif
#else
        #ifdef __Pyx_CyFunction_USED
        if (likely(PyFunction_Check(descr) || __Pyx_CyFunction_Check(descr)))
        #else
        if (likely(PyFunction_Check(descr)))
        #endif
#endif
        {
            meth_found = 1;
        } else {
            f = Py_TYPE(descr)->tp_descr_get;
            if (f != NULL && PyDescr_IsData(descr)) {
                attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
                Py_DECREF(descr);
                goto try_unpack;
            }
        }
    }
    dictptr = _PyObject_GetDictPtr(obj);
    if (dictptr != NULL && (dict = *dictptr) != NULL) {
        Py_INCREF(dict);
        attr = __Pyx_PyDict_GetItemStr(dict, name);
        if (attr != NULL) {
            Py_INCREF(attr);
            Py_DECREF(dict);
            Py_XDECREF(descr);
            goto try_unpack;
        }
        Py_DECREF(dict);
    }
    if (meth_found) {
        *method = descr;
        return 1;
    }
    if (f != NULL) {
        attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
        Py_DECREF(descr);
        goto try_unpack;
    }
    if (descr != NULL) {
        *method = descr;
        return 0;
    }
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'%.50s' object has no attribute '%U'",
                 tp->tp_name, name);
#else
                 "'%.50s' object has no attribute '%.400s'",
                 tp->tp_name, PyString_AS_STRING(name));
#endif
    return 0;
#else
    attr = __Pyx_PyObject_GetAttrStr(obj, name);
    goto try_unpack;
#endif
try_unpack:
#if CYTHON_UNPACK_METHODS
    if (likely(attr) && PyMethod_Check(attr) && likely(PyMethod_GET_SELF(attr) == obj)) {
        PyObject *function = PyMethod_GET_FUNCTION(attr);
        Py_INCREF(function);
        Py_DECREF(attr);
        *method = function;
        return 1;
    }
#endif
    *method = attr;
    return 0;
}

/* PyObjectCallMethod0 */
static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name) {
    PyObject *method = NULL, *result = NULL;
    int is_method = __Pyx_PyObject_GetMethod(obj, method_name, &method);
    if (likely(is_method)) {
        result = __Pyx_PyObject_CallOneArg(method, obj);
        Py_DECREF(method);
        return result;
    }
    if (unlikely(!method)) goto bad;
    result = __Pyx_PyObject_CallNoArg(method);
    Py_DECREF(method);
bad:
    return result;
}

/* RaiseNeedMoreValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* RaiseTooManyValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* UnpackItemEndCheck */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected) {
    if (unlikely(retval)) {
        Py_DECREF(retval);
        __Pyx_RaiseTooManyValuesError(expected);
        return -1;
    }
    return __Pyx_IterFinish();
}

/* RaiseNoneIterError */
static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
}

/* UnpackTupleError */
static void __Pyx_UnpackTupleError(PyObject *t, Py_ssize_t index) {
    if (t == Py_None) {
      __Pyx_RaiseNoneNotIterableError();
    } else if (PyTuple_GET_SIZE(t) < index) {
      __Pyx_RaiseNeedMoreValuesError(PyTuple_GET_SIZE(t));
    } else {
      __Pyx_RaiseTooManyValuesError(index);
    }
}

/* UnpackTuple2 */
static CYTHON_INLINE int __Pyx_unpack_tuple2_exact(
        PyObject* tuple, PyObject** pvalue1, PyObject** pvalue2, int decref_tuple) {
    PyObject *value1 = NULL, *value2 = NULL;
#if CYTHON_COMPILING_IN_PYPY
    value1 = PySequence_ITEM(tuple, 0);  if (unlikely(!value1)) goto bad;
    value2 = PySequence_ITEM(tuple, 1);  if (unlikely(!value2)) goto bad;
#else
    value1 = PyTuple_GET_ITEM(tuple, 0);  Py_INCREF(value1);
    value2 = PyTuple_GET_ITEM(tuple, 1);  Py_INCREF(value2);
#endif
    if (decref_tuple) {
        Py_DECREF(tuple);
    }
    *pvalue1 = value1;
    *pvalue2 = value2;
    return 0;
#if CYTHON_COMPILING_IN_PYPY
bad:
    Py_XDECREF(value1);
    Py_XDECREF(value2);
    if (decref_tuple) { Py_XDECREF(tuple); }
    return -1;
#endif
}
static int __Pyx_unpack_tuple2_generic(PyObject* tuple, PyObject** pvalue1, PyObject** pvalue2,
                                       int has_known_size, int decref_tuple) {
    Py_ssize_t index;
    PyObject *value1 = NULL, *value2 = NULL, *iter = NULL;
    iternextfunc iternext;
    iter = PyObject_GetIter(tuple);
    if (unlikely(!iter)) goto bad;
    if (decref_tuple) { Py_DECREF(tuple); tuple = NULL; }
    iternext = Py_TYPE(iter)->tp_iternext;
    value1 = iternext(iter); if (unlikely(!value1)) { index = 0; goto unpacking_failed; }
    value2 = iternext(iter); if (unlikely(!value2)) { index = 1; goto unpacking_failed; }
    if (!has_known_size && unlikely(__Pyx_IternextUnpackEndCheck(iternext(iter), 2))) goto bad;
    Py_DECREF(iter);
    *pvalue1 = value1;
    *pvalue2 = value2;
    return 0;
unpacking_failed:
    if (!has_known_size && __Pyx_IterFinish() == 0)
        __Pyx_RaiseNeedMoreValuesError(index);
bad:
    Py_XDECREF(iter);
    Py_XDECREF(value1);
    Py_XDECREF(value2);
    if (decref_tuple) { Py_XDECREF(tuple); }
    return -1;
}

/* dict_iter */
static CYTHON_INLINE PyObject* __Pyx_dict_iterator(PyObject* iterable, int is_dict, PyObject* method_name,
                                                   Py_ssize_t* p_orig_length, int* p_source_is_dict) {
    is_dict = is_dict || likely(PyDict_CheckExact(iterable));
    *p_source_is_dict = is_dict;
    if (is_dict) {
#if !CYTHON_COMPILING_IN_PYPY
        *p_orig_length = PyDict_Size(iterable);
        Py_INCREF(iterable);
        return iterable;
#elif PY_MAJOR_VERSION >= 3
        static PyObject *py_items = NULL, *py_keys = NULL, *py_values = NULL;
        PyObject **pp = NULL;
        if (method_name) {
            const char *name = PyUnicode_AsUTF8(method_name);
            if (strcmp(name, "iteritems") == 0) pp = &py_items;
            else if (strcmp(name, "iterkeys") == 0) pp = &py_keys;
            else if (strcmp(name, "itervalues") == 0) pp = &py_values;
            if (pp) {
                if (!*pp) {
                    *pp = PyUnicode_FromString(name + 4);
                    if (!*pp)
                        return NULL;
                }
                method_name = *pp;
            }
        }
#endif
    }
    *p_orig_length = 0;
    if (method_name) {
        PyObject* iter;
        iterable = __Pyx_PyObject_CallMethod0(iterable, method_name);
        if (!iterable)
            return NULL;
#if !CYTHON_COMPILING_IN_PYPY
        if (PyTuple_CheckExact(iterable) || PyList_CheckExact(iterable))
            return iterable;
#endif
        iter = PyObject_GetIter(iterable);
        Py_DECREF(iterable);
        return iter;
    }
    return PyObject_GetIter(iterable);
}
static CYTHON_INLINE int __Pyx_dict_iter_next(
        PyObject* iter_obj, CYTHON_NCP_UNUSED Py_ssize_t orig_length, CYTHON_NCP_UNUSED Py_ssize_t* ppos,
        PyObject** pkey, PyObject** pvalue, PyObject** pitem, int source_is_dict) {
    PyObject* next_item;
#if !CYTHON_COMPILING_IN_PYPY
    if (source_is_dict) {
        PyObject *key, *value;
        if (unlikely(orig_length != PyDict_Size(iter_obj))) {
            PyErr_SetString(PyExc_RuntimeError, "dictionary changed size during iteration");
            return -1;
        }
        if (unlikely(!PyDict_Next(iter_obj, ppos, &key, &value))) {
            return 0;
        }
        if (pitem) {
            PyObject* tuple = PyTuple_New(2);
            if (unlikely(!tuple)) {
                return -1;
            }
            Py_INCREF(key);
            Py_INCREF(value);
            PyTuple_SET_ITEM(tuple, 0, key);
            PyTuple_SET_ITEM(tuple, 1, value);
            *pitem = tuple;
        } else {
            if (pkey) {
                Py_INCREF(key);
                *pkey = key;
            }
            if (pvalue) {
                Py_INCREF(value);
                *pvalue = value;
            }
        }
        return 1;
    } else if (PyTuple_CheckExact(iter_obj)) {
        Py_ssize_t pos = *ppos;
        if (unlikely(pos >= PyTuple_GET_SIZE(iter_obj))) return 0;
        *ppos = pos + 1;
        next_item = PyTuple_GET_ITEM(iter_obj, pos);
        Py_INCREF(next_item);
    } else if (PyList_CheckExact(iter_obj)) {
        Py_ssize_t pos = *ppos;
        if (unlikely(pos >= PyList_GET_SIZE(iter_obj))) return 0;
        *ppos = pos + 1;
        next_item = PyList_GET_ITEM(iter_obj, pos);
        Py_INCREF(next_item);
    } else
#endif
    {
        next_item = PyIter_Next(iter_obj);
        if (unlikely(!next_item)) {
            return __Pyx_IterFinish();
        }
    }
    if (pitem) {
        *pitem = next_item;
    } else if (pkey && pvalue) {
        if (__Pyx_unpack_tuple2(next_item, pkey, pvalue, source_is_dict, source_is_dict, 1))
            return -1;
    } else if (pkey) {
        *pkey = next_item;
    } else {
        *pvalue = next_item;
    }
    return 1;
}

/* PyIntBinop */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_AddObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, int inplace, int zerodivision_check) {
    (void)inplace;
    (void)zerodivision_check;
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long x;
        long a = PyInt_AS_LONG(op1);
            x = (long)((unsigned long)a + b);
            if (likely((x^a) >= 0 || (x^b) >= 0))
                return PyInt_FromLong(x);
            return PyLong_Type.tp_as_number->nb_add(op1, op2);
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a, x;
#ifdef HAVE_LONG_LONG
        const PY_LONG_LONG llb = intval;
        PY_LONG_LONG lla, llx;
#endif
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        if (likely(__Pyx_sst_abs(size) <= 1)) {
            a = likely(size) ? digits[0] : 0;
            if (size == -1) a = -a;
        } else {
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                default: return PyLong_Type.tp_as_number->nb_add(op1, op2);
            }
        }
                x = a + b;
            return PyLong_FromLong(x);
#ifdef HAVE_LONG_LONG
        long_long:
                llx = lla + llb;
            return PyLong_FromLongLong(llx);
#endif
        
        
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
            double result;
            PyFPE_START_PROTECT("add", return NULL)
            result = ((double)a) + (double)b;
            PyFPE_END_PROTECT(result)
            return PyFloat_FromDouble(result);
    }
    return (inplace ? PyNumber_InPlaceAdd : PyNumber_Add)(op1, op2);
}
#endif

/* StringJoin */
#if !CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyBytes_Join(PyObject* sep, PyObject* values) {
    return PyObject_CallMethodObjArgs(sep, __pyx_n_s_join, values, NULL);
}
#endif

/* PyErrExceptionMatches */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    PyObject *exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
    if (unlikely(PyTuple_Check(err)))
        return __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    return __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
}
#endif

/* GetAttr */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *o, PyObject *n) {
#if CYTHON_USE_TYPE_SLOTS
#if PY_MAJOR_VERSION >= 3
    if (likely(PyUnicode_Check(n)))
#else
    if (likely(PyString_Check(n)))
#endif
        return __Pyx_PyObject_GetAttrStr(o, n);
#endif
    return PyObject_GetAttr(o, n);
}

/* GetAttr3 */
static PyObject *__Pyx_GetAttr3Default(PyObject *d) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (unlikely(!__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        return NULL;
    __Pyx_PyErr_Clear();
    Py_INCREF(d);
    return d;
}
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *o, PyObject *n, PyObject *d) {
    PyObject *r = __Pyx_GetAttr(o, n);
    return (likely(r)) ? r : __Pyx_GetAttr3Default(d);
}

/* Import */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if ((1) && (strchr(__Pyx_MODULE_NAME, '.'))) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, (PyObject *)NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* ImportFrom */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* HasAttr */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *o, PyObject *n) {
    PyObject *r;
    if (unlikely(!__Pyx_PyBaseString_Check(n))) {
        PyErr_SetString(PyExc_TypeError,
                        "hasattr(): attribute name must be string");
        return -1;
    }
    r = __Pyx_GetAttr(o, n);
    if (unlikely(!r)) {
        PyErr_Clear();
        return 0;
    } else {
        Py_DECREF(r);
        return 1;
    }
}

/* PyObject_GenericGetAttrNoDict */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject *__Pyx_RaiseGenericGetAttributeError(PyTypeObject *tp, PyObject *attr_name) {
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'%.50s' object has no attribute '%U'",
                 tp->tp_name, attr_name);
#else
                 "'%.50s' object has no attribute '%.400s'",
                 tp->tp_name, PyString_AS_STRING(attr_name));
#endif
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name) {
    PyObject *descr;
    PyTypeObject *tp = Py_TYPE(obj);
    if (unlikely(!PyString_Check(attr_name))) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    assert(!tp->tp_dictoffset);
    descr = _PyType_Lookup(tp, attr_name);
    if (unlikely(!descr)) {
        return __Pyx_RaiseGenericGetAttributeError(tp, attr_name);
    }
    Py_INCREF(descr);
    #if PY_MAJOR_VERSION < 3
    if (likely(PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_HAVE_CLASS)))
    #endif
    {
        descrgetfunc f = Py_TYPE(descr)->tp_descr_get;
        if (unlikely(f)) {
            PyObject *res = f(descr, obj, (PyObject *)tp);
            Py_DECREF(descr);
            return res;
        }
    }
    return descr;
}
#endif

/* PyObject_GenericGetAttr */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name) {
    if (unlikely(Py_TYPE(obj)->tp_dictoffset)) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    return __Pyx_PyObject_GenericGetAttrNoDict(obj, attr_name);
}
#endif

/* SetVTable */
static int __Pyx_SetVtable(PyObject *dict, void *vtable) {
#if PY_VERSION_HEX >= 0x02070000
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
#else
    PyObject *ob = PyCObject_FromVoidPtr(vtable, 0);
#endif
    if (!ob)
        goto bad;
    if (PyDict_SetItem(dict, __pyx_n_s_pyx_vtable, ob) < 0)
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* PyObjectGetAttrStrNoError */
static void __Pyx_PyObject_GetAttrStr_ClearAttributeError(void) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (likely(__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        __Pyx_PyErr_Clear();
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name) {
    PyObject *result;
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_TYPE_SLOTS && PY_VERSION_HEX >= 0x030700B1
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro == PyObject_GenericGetAttr)) {
        return _PyObject_GenericGetAttrWithDict(obj, attr_name, NULL, 1);
    }
#endif
    result = __Pyx_PyObject_GetAttrStr(obj, attr_name);
    if (unlikely(!result)) {
        __Pyx_PyObject_GetAttrStr_ClearAttributeError();
    }
    return result;
}

/* SetupReduce */
static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStr(meth, __pyx_n_s_name_2);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_getstate = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
    PyObject *getstate = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    getstate = _PyType_Lookup((PyTypeObject*)type_obj, __pyx_n_s_getstate);
#else
    getstate = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_getstate);
    if (!getstate && PyErr_Occurred()) {
        goto __PYX_BAD;
    }
#endif
    if (getstate) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_getstate = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_getstate);
#else
        object_getstate = __Pyx_PyObject_GetAttrStrNoError((PyObject*)&PyBaseObject_Type, __pyx_n_s_getstate);
        if (!object_getstate && PyErr_Occurred()) {
            goto __PYX_BAD;
        }
#endif
        if (object_getstate != getstate) {
            goto __PYX_GOOD;
        }
    }
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_ex); if (unlikely(!reduce_ex)) goto __PYX_BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce); if (unlikely(!reduce)) goto __PYX_BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_n_s_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_reduce_cython);
            if (likely(reduce_cython)) {
                ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce, reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
            } else if (reduce == object_reduce || PyErr_Occurred()) {
                goto __PYX_BAD;
            }
            setstate = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_n_s_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_setstate_cython);
                if (likely(setstate_cython)) {
                    ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate, setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                    ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                } else if (!setstate || PyErr_Occurred()) {
                    goto __PYX_BAD;
                }
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto __PYX_GOOD;
__PYX_BAD:
    if (!PyErr_Occurred())
        PyErr_Format(PyExc_RuntimeError, "Unable to initialize pickling for %s", ((PyTypeObject*)type_obj)->tp_name);
    ret = -1;
__PYX_GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
    Py_XDECREF(object_getstate);
    Py_XDECREF(getstate);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* TypeImport */
#ifndef __PYX_HAVE_RT_ImportType_0_29_35
#define __PYX_HAVE_RT_ImportType_0_29_35
static PyTypeObject *__Pyx_ImportType_0_29_35(PyObject *module, const char *module_name, const char *class_name,
    size_t size, size_t alignment, enum __Pyx_ImportType_CheckSize_0_29_35 check_size)
{
    PyObject *result = 0;
    char warning[200];
    Py_ssize_t basicsize;
    Py_ssize_t itemsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
    PyObject *py_itemsize;
#endif
    result = PyObject_GetAttrString(module, class_name);
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
    itemsize = ((PyTypeObject *)result)->tp_itemsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
    py_itemsize = PyObject_GetAttrString(result, "__itemsize__");
    if (!py_itemsize)
        goto bad;
    itemsize = PyLong_AsSsize_t(py_itemsize);
    Py_DECREF(py_itemsize);
    py_itemsize = 0;
    if (itemsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if (itemsize) {
        if (size % alignment) {
            alignment = size % alignment;
        }
        if (itemsize < (Py_ssize_t)alignment)
            itemsize = (Py_ssize_t)alignment;
    }
    if ((size_t)(basicsize + itemsize) < size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    if (check_size == __Pyx_ImportType_CheckSize_Error_0_29_35 && (size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    else if (check_size == __Pyx_ImportType_CheckSize_Warn_0_29_35 && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(result);
    return NULL;
}
#endif

/* GetVTable */
static void* __Pyx_GetVtable(PyObject *dict) {
    void* ptr;
    PyObject *ob = PyObject_GetItem(dict, __pyx_n_s_pyx_vtable);
    if (!ob)
        goto bad;
#if PY_VERSION_HEX >= 0x02070000
    ptr = PyCapsule_GetPointer(ob, 0);
#else
    ptr = PyCObject_AsVoidPtr(ob);
#endif
    if (!ptr && !PyErr_Occurred())
        PyErr_SetString(PyExc_RuntimeError, "invalid vtable found for imported type");
    Py_DECREF(ob);
    return ptr;
bad:
    Py_XDECREF(ob);
    return NULL;
}

/* CalculateMetaclass */
static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases) {
    Py_ssize_t i, nbases = PyTuple_GET_SIZE(bases);
    for (i=0; i < nbases; i++) {
        PyTypeObject *tmptype;
        PyObject *tmp = PyTuple_GET_ITEM(bases, i);
        tmptype = Py_TYPE(tmp);
#if PY_MAJOR_VERSION < 3
        if (tmptype == &PyClass_Type)
            continue;
#endif
        if (!metaclass) {
            metaclass = tmptype;
            continue;
        }
        if (PyType_IsSubtype(metaclass, tmptype))
            continue;
        if (PyType_IsSubtype(tmptype, metaclass)) {
            metaclass = tmptype;
            continue;
        }
        PyErr_SetString(PyExc_TypeError,
                        "metaclass conflict: "
                        "the metaclass of a derived class "
                        "must be a (non-strict) subclass "
                        "of the metaclasses of all its bases");
        return NULL;
    }
    if (!metaclass) {
#if PY_MAJOR_VERSION < 3
        metaclass = &PyClass_Type;
#else
        metaclass = &PyType_Type;
#endif
    }
    Py_INCREF((PyObject*) metaclass);
    return (PyObject*) metaclass;
}

/* Py3ClassCreate */
static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name,
                                           PyObject *qualname, PyObject *mkw, PyObject *modname, PyObject *doc) {
    PyObject *ns;
    if (metaclass) {
        PyObject *prep = __Pyx_PyObject_GetAttrStr(metaclass, __pyx_n_s_prepare);
        if (prep) {
            PyObject *pargs = PyTuple_Pack(2, name, bases);
            if (unlikely(!pargs)) {
                Py_DECREF(prep);
                return NULL;
            }
            ns = PyObject_Call(prep, pargs, mkw);
            Py_DECREF(prep);
            Py_DECREF(pargs);
        } else {
            if (unlikely(!PyErr_ExceptionMatches(PyExc_AttributeError)))
                return NULL;
            PyErr_Clear();
            ns = PyDict_New();
        }
    } else {
        ns = PyDict_New();
    }
    if (unlikely(!ns))
        return NULL;
    if (unlikely(PyObject_SetItem(ns, __pyx_n_s_module, modname) < 0)) goto bad;
    if (unlikely(PyObject_SetItem(ns, __pyx_n_s_qualname, qualname) < 0)) goto bad;
    if (unlikely(doc && PyObject_SetItem(ns, __pyx_n_s_doc, doc) < 0)) goto bad;
    return ns;
bad:
    Py_DECREF(ns);
    return NULL;
}
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases,
                                      PyObject *dict, PyObject *mkw,
                                      int calculate_metaclass, int allow_py2_metaclass) {
    PyObject *result, *margs;
    PyObject *owned_metaclass = NULL;
    if (allow_py2_metaclass) {
        owned_metaclass = PyObject_GetItem(dict, __pyx_n_s_metaclass);
        if (owned_metaclass) {
            metaclass = owned_metaclass;
        } else if (likely(PyErr_ExceptionMatches(PyExc_KeyError))) {
            PyErr_Clear();
        } else {
            return NULL;
        }
    }
    if (calculate_metaclass && (!metaclass || PyType_Check(metaclass))) {
        metaclass = __Pyx_CalculateMetaclass((PyTypeObject*) metaclass, bases);
        Py_XDECREF(owned_metaclass);
        if (unlikely(!metaclass))
            return NULL;
        owned_metaclass = metaclass;
    }
    margs = PyTuple_Pack(3, name, bases, dict);
    if (unlikely(!margs)) {
        result = NULL;
    } else {
        result = PyObject_Call(metaclass, margs, mkw);
        Py_DECREF(margs);
    }
    Py_XDECREF(owned_metaclass);
    return result;
}

/* FetchCommonType */
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type) {
    PyObject* fake_module;
    PyTypeObject* cached_type = NULL;
    fake_module = PyImport_AddModule((char*) "_cython_" CYTHON_ABI);
    if (!fake_module) return NULL;
    Py_INCREF(fake_module);
    cached_type = (PyTypeObject*) PyObject_GetAttrString(fake_module, type->tp_name);
    if (cached_type) {
        if (!PyType_Check((PyObject*)cached_type)) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s is not a type object",
                type->tp_name);
            goto bad;
        }
        if (cached_type->tp_basicsize != type->tp_basicsize) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s has the wrong size, try recompiling",
                type->tp_name);
            goto bad;
        }
    } else {
        if (!PyErr_ExceptionMatches(PyExc_AttributeError)) goto bad;
        PyErr_Clear();
        if (PyType_Ready(type) < 0) goto bad;
        if (PyObject_SetAttrString(fake_module, type->tp_name, (PyObject*) type) < 0)
            goto bad;
        Py_INCREF(type);
        cached_type = type;
    }
done:
    Py_DECREF(fake_module);
    return cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}

/* CythonFunctionShared */
#include <structmember.h>
static PyObject *
__Pyx_CyFunction_get_doc(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *closure)
{
    if (unlikely(op->func_doc == NULL)) {
        if (op->func.m_ml->ml_doc) {
#if PY_MAJOR_VERSION >= 3
            op->func_doc = PyUnicode_FromString(op->func.m_ml->ml_doc);
#else
            op->func_doc = PyString_FromString(op->func.m_ml->ml_doc);
#endif
            if (unlikely(op->func_doc == NULL))
                return NULL;
        } else {
            Py_INCREF(Py_None);
            return Py_None;
        }
    }
    Py_INCREF(op->func_doc);
    return op->func_doc;
}
static int
__Pyx_CyFunction_set_doc(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp = op->func_doc;
    if (value == NULL) {
        value = Py_None;
    }
    Py_INCREF(value);
    op->func_doc = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_name(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    if (unlikely(op->func_name == NULL)) {
#if PY_MAJOR_VERSION >= 3
        op->func_name = PyUnicode_InternFromString(op->func.m_ml->ml_name);
#else
        op->func_name = PyString_InternFromString(op->func.m_ml->ml_name);
#endif
        if (unlikely(op->func_name == NULL))
            return NULL;
    }
    Py_INCREF(op->func_name);
    return op->func_name;
}
static int
__Pyx_CyFunction_set_name(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    tmp = op->func_name;
    Py_INCREF(value);
    op->func_name = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_qualname(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    Py_INCREF(op->func_qualname);
    return op->func_qualname;
}
static int
__Pyx_CyFunction_set_qualname(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    tmp = op->func_qualname;
    Py_INCREF(value);
    op->func_qualname = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_self(__pyx_CyFunctionObject *m, CYTHON_UNUSED void *closure)
{
    PyObject *self;
    self = m->func_closure;
    if (self == NULL)
        self = Py_None;
    Py_INCREF(self);
    return self;
}
static PyObject *
__Pyx_CyFunction_get_dict(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    if (unlikely(op->func_dict == NULL)) {
        op->func_dict = PyDict_New();
        if (unlikely(op->func_dict == NULL))
            return NULL;
    }
    Py_INCREF(op->func_dict);
    return op->func_dict;
}
static int
__Pyx_CyFunction_set_dict(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
    if (unlikely(value == NULL)) {
        PyErr_SetString(PyExc_TypeError,
               "function's dictionary may not be deleted");
        return -1;
    }
    if (unlikely(!PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
               "setting function's dictionary to a non-dict");
        return -1;
    }
    tmp = op->func_dict;
    Py_INCREF(value);
    op->func_dict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_globals(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    Py_INCREF(op->func_globals);
    return op->func_globals;
}
static PyObject *
__Pyx_CyFunction_get_closure(CYTHON_UNUSED __pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    Py_INCREF(Py_None);
    return Py_None;
}
static PyObject *
__Pyx_CyFunction_get_code(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    PyObject* result = (op->func_code) ? op->func_code : Py_None;
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_init_defaults(__pyx_CyFunctionObject *op) {
    int result = 0;
    PyObject *res = op->defaults_getter((PyObject *) op);
    if (unlikely(!res))
        return -1;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    op->defaults_tuple = PyTuple_GET_ITEM(res, 0);
    Py_INCREF(op->defaults_tuple);
    op->defaults_kwdict = PyTuple_GET_ITEM(res, 1);
    Py_INCREF(op->defaults_kwdict);
    #else
    op->defaults_tuple = PySequence_ITEM(res, 0);
    if (unlikely(!op->defaults_tuple)) result = -1;
    else {
        op->defaults_kwdict = PySequence_ITEM(res, 1);
        if (unlikely(!op->defaults_kwdict)) result = -1;
    }
    #endif
    Py_DECREF(res);
    return result;
}
static int
__Pyx_CyFunction_set_defaults(__pyx_CyFunctionObject *op, PyObject* value, CYTHON_UNUSED void *context) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyTuple_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__defaults__ must be set to a tuple object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_tuple;
    op->defaults_tuple = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_defaults(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context) {
    PyObject* result = op->defaults_tuple;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_tuple;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_kwdefaults(__pyx_CyFunctionObject *op, PyObject* value, CYTHON_UNUSED void *context) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__kwdefaults__ must be set to a dict object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_kwdict;
    op->defaults_kwdict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_kwdefaults(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context) {
    PyObject* result = op->defaults_kwdict;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_kwdict;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_annotations(__pyx_CyFunctionObject *op, PyObject* value, CYTHON_UNUSED void *context) {
    PyObject* tmp;
    if (!value || value == Py_None) {
        value = NULL;
    } else if (!PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__annotations__ must be set to a dict object");
        return -1;
    }
    Py_XINCREF(value);
    tmp = op->func_annotations;
    op->func_annotations = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_annotations(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context) {
    PyObject* result = op->func_annotations;
    if (unlikely(!result)) {
        result = PyDict_New();
        if (unlikely(!result)) return NULL;
        op->func_annotations = result;
    }
    Py_INCREF(result);
    return result;
}
static PyGetSetDef __pyx_CyFunction_getsets[] = {
    {(char *) "func_doc", (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "__doc__",  (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "func_name", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__name__", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__qualname__", (getter)__Pyx_CyFunction_get_qualname, (setter)__Pyx_CyFunction_set_qualname, 0, 0},
    {(char *) "__self__", (getter)__Pyx_CyFunction_get_self, 0, 0, 0},
    {(char *) "func_dict", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "__dict__", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "func_globals", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "__globals__", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "func_closure", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "__closure__", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "func_code", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "__code__", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "func_defaults", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__defaults__", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__kwdefaults__", (getter)__Pyx_CyFunction_get_kwdefaults, (setter)__Pyx_CyFunction_set_kwdefaults, 0, 0},
    {(char *) "__annotations__", (getter)__Pyx_CyFunction_get_annotations, (setter)__Pyx_CyFunction_set_annotations, 0, 0},
    {0, 0, 0, 0, 0}
};
static PyMemberDef __pyx_CyFunction_members[] = {
    {(char *) "__module__", T_OBJECT, offsetof(PyCFunctionObject, m_module), PY_WRITE_RESTRICTED, 0},
    {0, 0, 0,  0, 0}
};
static PyObject *
__Pyx_CyFunction_reduce(__pyx_CyFunctionObject *m, CYTHON_UNUSED PyObject *args)
{
#if PY_MAJOR_VERSION >= 3
    Py_INCREF(m->func_qualname);
    return m->func_qualname;
#else
    return PyString_FromString(m->func.m_ml->ml_name);
#endif
}
static PyMethodDef __pyx_CyFunction_methods[] = {
    {"__reduce__", (PyCFunction)__Pyx_CyFunction_reduce, METH_VARARGS, 0},
    {0, 0, 0, 0}
};
#if PY_VERSION_HEX < 0x030500A0
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func_weakreflist)
#else
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func.m_weakreflist)
#endif
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject *op, PyMethodDef *ml, int flags, PyObject* qualname,
                                       PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    if (unlikely(op == NULL))
        return NULL;
    op->flags = flags;
    __Pyx_CyFunction_weakreflist(op) = NULL;
    op->func.m_ml = ml;
    op->func.m_self = (PyObject *) op;
    Py_XINCREF(closure);
    op->func_closure = closure;
    Py_XINCREF(module);
    op->func.m_module = module;
    op->func_dict = NULL;
    op->func_name = NULL;
    Py_INCREF(qualname);
    op->func_qualname = qualname;
    op->func_doc = NULL;
    op->func_classobj = NULL;
    op->func_globals = globals;
    Py_INCREF(op->func_globals);
    Py_XINCREF(code);
    op->func_code = code;
    op->defaults_pyobjects = 0;
    op->defaults_size = 0;
    op->defaults = NULL;
    op->defaults_tuple = NULL;
    op->defaults_kwdict = NULL;
    op->defaults_getter = NULL;
    op->func_annotations = NULL;
    return (PyObject *) op;
}
static int
__Pyx_CyFunction_clear(__pyx_CyFunctionObject *m)
{
    Py_CLEAR(m->func_closure);
    Py_CLEAR(m->func.m_module);
    Py_CLEAR(m->func_dict);
    Py_CLEAR(m->func_name);
    Py_CLEAR(m->func_qualname);
    Py_CLEAR(m->func_doc);
    Py_CLEAR(m->func_globals);
    Py_CLEAR(m->func_code);
    Py_CLEAR(m->func_classobj);
    Py_CLEAR(m->defaults_tuple);
    Py_CLEAR(m->defaults_kwdict);
    Py_CLEAR(m->func_annotations);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_XDECREF(pydefaults[i]);
        PyObject_Free(m->defaults);
        m->defaults = NULL;
    }
    return 0;
}
static void __Pyx__CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    if (__Pyx_CyFunction_weakreflist(m) != NULL)
        PyObject_ClearWeakRefs((PyObject *) m);
    __Pyx_CyFunction_clear(m);
    PyObject_GC_Del(m);
}
static void __Pyx_CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    PyObject_GC_UnTrack(m);
    __Pyx__CyFunction_dealloc(m);
}
static int __Pyx_CyFunction_traverse(__pyx_CyFunctionObject *m, visitproc visit, void *arg)
{
    Py_VISIT(m->func_closure);
    Py_VISIT(m->func.m_module);
    Py_VISIT(m->func_dict);
    Py_VISIT(m->func_name);
    Py_VISIT(m->func_qualname);
    Py_VISIT(m->func_doc);
    Py_VISIT(m->func_globals);
    Py_VISIT(m->func_code);
    Py_VISIT(m->func_classobj);
    Py_VISIT(m->defaults_tuple);
    Py_VISIT(m->defaults_kwdict);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_VISIT(pydefaults[i]);
    }
    return 0;
}
static PyObject *__Pyx_CyFunction_descr_get(PyObject *func, PyObject *obj, PyObject *type)
{
#if PY_MAJOR_VERSION < 3
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    if (m->flags & __Pyx_CYFUNCTION_STATICMETHOD) {
        Py_INCREF(func);
        return func;
    }
    if (m->flags & __Pyx_CYFUNCTION_CLASSMETHOD) {
        if (type == NULL)
            type = (PyObject *)(Py_TYPE(obj));
        return __Pyx_PyMethod_New(func, type, (PyObject *)(Py_TYPE(type)));
    }
    if (obj == Py_None)
        obj = NULL;
#endif
    return __Pyx_PyMethod_New(func, obj, type);
}
static PyObject*
__Pyx_CyFunction_repr(__pyx_CyFunctionObject *op)
{
#if PY_MAJOR_VERSION >= 3
    return PyUnicode_FromFormat("<cyfunction %U at %p>",
                                op->func_qualname, (void *)op);
#else
    return PyString_FromFormat("<cyfunction %s at %p>",
                               PyString_AsString(op->func_qualname), (void *)op);
#endif
}
static PyObject * __Pyx_CyFunction_CallMethod(PyObject *func, PyObject *self, PyObject *arg, PyObject *kw) {
    PyCFunctionObject* f = (PyCFunctionObject*)func;
    PyCFunction meth = f->m_ml->ml_meth;
    Py_ssize_t size;
    switch (f->m_ml->ml_flags & (METH_VARARGS | METH_KEYWORDS | METH_NOARGS | METH_O)) {
    case METH_VARARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0))
            return (*meth)(self, arg);
        break;
    case METH_VARARGS | METH_KEYWORDS:
        return (*(PyCFunctionWithKeywords)(void*)meth)(self, arg, kw);
    case METH_NOARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 0))
                return (*meth)(self, NULL);
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes no arguments (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    case METH_O:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 1)) {
                PyObject *result, *arg0;
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                arg0 = PyTuple_GET_ITEM(arg, 0);
                #else
                arg0 = PySequence_ITEM(arg, 0); if (unlikely(!arg0)) return NULL;
                #endif
                result = (*meth)(self, arg0);
                #if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
                Py_DECREF(arg0);
                #endif
                return result;
            }
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes exactly one argument (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags in "
                        "__Pyx_CyFunction_Call. METH_OLDARGS is no "
                        "longer supported!");
        return NULL;
    }
    PyErr_Format(PyExc_TypeError, "%.200s() takes no keyword arguments",
                 f->m_ml->ml_name);
    return NULL;
}
static CYTHON_INLINE PyObject *__Pyx_CyFunction_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    return __Pyx_CyFunction_CallMethod(func, ((PyCFunctionObject*)func)->m_self, arg, kw);
}
static PyObject *__Pyx_CyFunction_CallAsMethod(PyObject *func, PyObject *args, PyObject *kw) {
    PyObject *result;
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *) func;
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        Py_ssize_t argc;
        PyObject *new_args;
        PyObject *self;
        argc = PyTuple_GET_SIZE(args);
        new_args = PyTuple_GetSlice(args, 1, argc);
        if (unlikely(!new_args))
            return NULL;
        self = PyTuple_GetItem(args, 0);
        if (unlikely(!self)) {
            Py_DECREF(new_args);
#if PY_MAJOR_VERSION > 2
            PyErr_Format(PyExc_TypeError,
                         "unbound method %.200S() needs an argument",
                         cyfunc->func_qualname);
#else
            PyErr_SetString(PyExc_TypeError,
                            "unbound method needs an argument");
#endif
            return NULL;
        }
        result = __Pyx_CyFunction_CallMethod(func, self, new_args, kw);
        Py_DECREF(new_args);
    } else {
        result = __Pyx_CyFunction_Call(func, args, kw);
    }
    return result;
}
static PyTypeObject __pyx_CyFunctionType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    "cython_function_or_method",
    sizeof(__pyx_CyFunctionObject),
    0,
    (destructor) __Pyx_CyFunction_dealloc,
    0,
    0,
    0,
#if PY_MAJOR_VERSION < 3
    0,
#else
    0,
#endif
    (reprfunc) __Pyx_CyFunction_repr,
    0,
    0,
    0,
    0,
    __Pyx_CyFunction_CallAsMethod,
    0,
    0,
    0,
    0,
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,
    0,
    (traverseproc) __Pyx_CyFunction_traverse,
    (inquiry) __Pyx_CyFunction_clear,
    0,
#if PY_VERSION_HEX < 0x030500A0
    offsetof(__pyx_CyFunctionObject, func_weakreflist),
#else
    offsetof(PyCFunctionObject, m_weakreflist),
#endif
    0,
    0,
    __pyx_CyFunction_methods,
    __pyx_CyFunction_members,
    __pyx_CyFunction_getsets,
    0,
    0,
    __Pyx_CyFunction_descr_get,
    0,
    offsetof(__pyx_CyFunctionObject, func_dict),
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if PY_VERSION_HEX >= 0x030400a1
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
    0,
#endif
#if PY_VERSION_HEX >= 0x030C0000
    0,
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
    0,
#endif
};
static int __pyx_CyFunction_init(void) {
    __pyx_CyFunctionType = __Pyx_FetchCommonType(&__pyx_CyFunctionType_type);
    if (unlikely(__pyx_CyFunctionType == NULL)) {
        return -1;
    }
    return 0;
}
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *func, size_t size, int pyobjects) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults = PyObject_Malloc(size);
    if (unlikely(!m->defaults))
        return PyErr_NoMemory();
    memset(m->defaults, 0, size);
    m->defaults_pyobjects = pyobjects;
    m->defaults_size = size;
    return m->defaults;
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *func, PyObject *tuple) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_tuple = tuple;
    Py_INCREF(tuple);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_kwdict = dict;
    Py_INCREF(dict);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->func_annotations = dict;
    Py_INCREF(dict);
}

/* CythonFunction */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml, int flags, PyObject* qualname,
                                      PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    PyObject *op = __Pyx_CyFunction_Init(
        PyObject_GC_New(__pyx_CyFunctionObject, __pyx_CyFunctionType),
        ml, flags, qualname, closure, module, globals, code
    );
    if (likely(op)) {
        PyObject_GC_Track(op);
    }
    return op;
}

/* ObjectGetItem */
#if CYTHON_USE_TYPE_SLOTS
static PyObject *__Pyx_PyObject_GetIndex(PyObject *obj, PyObject* index) {
    PyObject *runerr = NULL;
    Py_ssize_t key_value;
    PySequenceMethods *m = Py_TYPE(obj)->tp_as_sequence;
    if (unlikely(!(m && m->sq_item))) {
        PyErr_Format(PyExc_TypeError, "'%.200s' object is not subscriptable", Py_TYPE(obj)->tp_name);
        return NULL;
    }
    key_value = __Pyx_PyIndex_AsSsize_t(index);
    if (likely(key_value != -1 || !(runerr = PyErr_Occurred()))) {
        return __Pyx_GetItemInt_Fast(obj, key_value, 0, 1, 1);
    }
    if (PyErr_GivenExceptionMatches(runerr, PyExc_OverflowError)) {
        PyErr_Clear();
        PyErr_Format(PyExc_IndexError, "cannot fit '%.200s' into an index-sized integer", Py_TYPE(index)->tp_name);
    }
    return NULL;
}
static PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject* key) {
    PyMappingMethods *m = Py_TYPE(obj)->tp_as_mapping;
    if (likely(m && m->mp_subscript)) {
        return m->mp_subscript(obj, key);
    }
    return __Pyx_PyObject_GetIndex(obj, key);
}
#endif

/* CLineInTraceback */
#ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(CYTHON_UNUSED PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    if (unlikely(!__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, *cython_runtime_dict,
            __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_n_s_cline_in_traceback))
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        (void) PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, ((size_t)new_max) * sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
#if PY_VERSION_HEX >= 0x030b00a6
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = NULL;
    PyObject *py_funcname = NULL;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_srcfile = NULL;
    py_srcfile = PyString_FromString(filename);
    if (!py_srcfile) goto bad;
    #endif
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        funcname = PyUnicode_AsUTF8(py_funcname);
        if (!funcname) goto bad;
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        if (!py_funcname) goto bad;
        #endif
    }
    #if PY_MAJOR_VERSION < 3
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    #else
    py_code = PyCode_NewEmpty(filename, funcname, py_line);
    #endif
    Py_XDECREF(py_funcname);  // XDECREF since it's only set on Py3 if cline
    return py_code;
bad:
    Py_XDECREF(py_funcname);
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_srcfile);
    #endif
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject *ptype, *pvalue, *ptraceback;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) {
            /* If the code object creation fails, then we should clear the
               fetched exception references and propagate the new exception */
            Py_XDECREF(ptype);
            Py_XDECREF(pvalue);
            Py_XDECREF(ptraceback);
            goto bad;
        }
        __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntFromPyVerify */
#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_unsigned_int(unsigned int value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const unsigned int neg_one = (unsigned int) -1, const_zero = (unsigned int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(unsigned int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(unsigned int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(unsigned int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(unsigned int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(unsigned int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(unsigned int),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
static CYTHON_INLINE unsigned int __Pyx_PyInt_As_unsigned_int(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const unsigned int neg_one = (unsigned int) -1, const_zero = (unsigned int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(unsigned int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(unsigned int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (unsigned int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (unsigned int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(unsigned int, digit, digits[0])
                case 2:
                    if (8 * sizeof(unsigned int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) >= 2 * PyLong_SHIFT) {
                            return (unsigned int) (((((unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(unsigned int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) >= 3 * PyLong_SHIFT) {
                            return (unsigned int) (((((((unsigned int)digits[2]) << PyLong_SHIFT) | (unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(unsigned int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) >= 4 * PyLong_SHIFT) {
                            return (unsigned int) (((((((((unsigned int)digits[3]) << PyLong_SHIFT) | (unsigned int)digits[2]) << PyLong_SHIFT) | (unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (unsigned int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(unsigned int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(unsigned int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (unsigned int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(unsigned int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(unsigned int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(unsigned int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) - 1 > 2 * PyLong_SHIFT) {
                            return (unsigned int) (((unsigned int)-1)*(((((unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(unsigned int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) - 1 > 2 * PyLong_SHIFT) {
                            return (unsigned int) ((((((unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(unsigned int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) - 1 > 3 * PyLong_SHIFT) {
                            return (unsigned int) (((unsigned int)-1)*(((((((unsigned int)digits[2]) << PyLong_SHIFT) | (unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(unsigned int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) - 1 > 3 * PyLong_SHIFT) {
                            return (unsigned int) ((((((((unsigned int)digits[2]) << PyLong_SHIFT) | (unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(unsigned int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) - 1 > 4 * PyLong_SHIFT) {
                            return (unsigned int) (((unsigned int)-1)*(((((((((unsigned int)digits[3]) << PyLong_SHIFT) | (unsigned int)digits[2]) << PyLong_SHIFT) | (unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(unsigned int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) - 1 > 4 * PyLong_SHIFT) {
                            return (unsigned int) ((((((((((unsigned int)digits[3]) << PyLong_SHIFT) | (unsigned int)digits[2]) << PyLong_SHIFT) | (unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(unsigned int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(unsigned int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            unsigned int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (unsigned int) -1;
        }
    } else {
        unsigned int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (unsigned int) -1;
        val = __Pyx_PyInt_As_unsigned_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to unsigned int");
    return (unsigned int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to unsigned int");
    return (unsigned int) -1;
}

/* CIntFromPy */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* FastTypeChecks */
#if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        #if PY_MAJOR_VERSION < 3
        if (likely(exc_type == t)) return 1;
        #endif
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* GetTopmostException */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem *
__Pyx_PyErr_GetTopmostException(PyThreadState *tstate)
{
    _PyErr_StackItem *exc_info = tstate->exc_info;
    while ((exc_info->exc_type == NULL || exc_info->exc_type == Py_None) &&
           exc_info->previous_item != NULL)
    {
        exc_info = exc_info->previous_item;
    }
    return exc_info;
}
#endif

/* SaveResetException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    *type = exc_info->exc_type;
    *value = exc_info->exc_value;
    *tb = exc_info->exc_traceback;
    #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    #endif
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = type;
    exc_info->exc_value = value;
    exc_info->exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
#endif

/* SwapException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = *type;
    exc_info->exc_value = *value;
    exc_info->exc_traceback = *tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = *type;
    tstate->exc_value = *value;
    tstate->exc_traceback = *tb;
    #endif
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyErr_GetExcInfo(&tmp_type, &tmp_value, &tmp_tb);
    PyErr_SetExcInfo(*type, *value, *tb);
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#endif

/* PyObjectCallMethod1 */
static PyObject* __Pyx__PyObject_CallMethod1(PyObject* method, PyObject* arg) {
    PyObject *result = __Pyx_PyObject_CallOneArg(method, arg);
    Py_DECREF(method);
    return result;
}
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg) {
    PyObject *method = NULL, *result;
    int is_method = __Pyx_PyObject_GetMethod(obj, method_name, &method);
    if (likely(is_method)) {
        result = __Pyx_PyObject_Call2Args(method, obj, arg);
        Py_DECREF(method);
        return result;
    }
    if (unlikely(!method)) return NULL;
    return __Pyx__PyObject_CallMethod1(method, arg);
}

/* CoroutineBase */
#include <structmember.h>
#include <frameobject.h>
#if PY_VERSION_HEX >= 0x030b00a6
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
#define __Pyx_Coroutine_Undelegate(gen) Py_CLEAR((gen)->yieldfrom)
static int __Pyx_PyGen__FetchStopIterationValue(CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject **pvalue) {
    PyObject *et, *ev, *tb;
    PyObject *value = NULL;
    __Pyx_ErrFetch(&et, &ev, &tb);
    if (!et) {
        Py_XDECREF(tb);
        Py_XDECREF(ev);
        Py_INCREF(Py_None);
        *pvalue = Py_None;
        return 0;
    }
    if (likely(et == PyExc_StopIteration)) {
        if (!ev) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#if PY_VERSION_HEX >= 0x030300A0
        else if (Py_TYPE(ev) == (PyTypeObject*)PyExc_StopIteration) {
            value = ((PyStopIterationObject *)ev)->value;
            Py_INCREF(value);
            Py_DECREF(ev);
        }
#endif
        else if (unlikely(PyTuple_Check(ev))) {
            if (PyTuple_GET_SIZE(ev) >= 1) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                value = PyTuple_GET_ITEM(ev, 0);
                Py_INCREF(value);
#else
                value = PySequence_ITEM(ev, 0);
#endif
            } else {
                Py_INCREF(Py_None);
                value = Py_None;
            }
            Py_DECREF(ev);
        }
        else if (!__Pyx_TypeCheck(ev, (PyTypeObject*)PyExc_StopIteration)) {
            value = ev;
        }
        if (likely(value)) {
            Py_XDECREF(tb);
            Py_DECREF(et);
            *pvalue = value;
            return 0;
        }
    } else if (!__Pyx_PyErr_GivenExceptionMatches(et, PyExc_StopIteration)) {
        __Pyx_ErrRestore(et, ev, tb);
        return -1;
    }
    PyErr_NormalizeException(&et, &ev, &tb);
    if (unlikely(!PyObject_TypeCheck(ev, (PyTypeObject*)PyExc_StopIteration))) {
        __Pyx_ErrRestore(et, ev, tb);
        return -1;
    }
    Py_XDECREF(tb);
    Py_DECREF(et);
#if PY_VERSION_HEX >= 0x030300A0
    value = ((PyStopIterationObject *)ev)->value;
    Py_INCREF(value);
    Py_DECREF(ev);
#else
    {
        PyObject* args = __Pyx_PyObject_GetAttrStr(ev, __pyx_n_s_args);
        Py_DECREF(ev);
        if (likely(args)) {
            value = PySequence_GetItem(args, 0);
            Py_DECREF(args);
        }
        if (unlikely(!value)) {
            __Pyx_ErrRestore(NULL, NULL, NULL);
            Py_INCREF(Py_None);
            value = Py_None;
        }
    }
#endif
    *pvalue = value;
    return 0;
}
static CYTHON_INLINE
void __Pyx_Coroutine_ExceptionClear(__Pyx_ExcInfoStruct *exc_state) {
    PyObject *t, *v, *tb;
    t = exc_state->exc_type;
    v = exc_state->exc_value;
    tb = exc_state->exc_traceback;
    exc_state->exc_type = NULL;
    exc_state->exc_value = NULL;
    exc_state->exc_traceback = NULL;
    Py_XDECREF(t);
    Py_XDECREF(v);
    Py_XDECREF(tb);
}
#define __Pyx_Coroutine_AlreadyRunningError(gen)  (__Pyx__Coroutine_AlreadyRunningError(gen), (PyObject*)NULL)
static void __Pyx__Coroutine_AlreadyRunningError(CYTHON_UNUSED __pyx_CoroutineObject *gen) {
    const char *msg;
    if ((0)) {
    #ifdef __Pyx_Coroutine_USED
    } else if (__Pyx_Coroutine_Check((PyObject*)gen)) {
        msg = "coroutine already executing";
    #endif
    #ifdef __Pyx_AsyncGen_USED
    } else if (__Pyx_AsyncGen_CheckExact((PyObject*)gen)) {
        msg = "async generator already executing";
    #endif
    } else {
        msg = "generator already executing";
    }
    PyErr_SetString(PyExc_ValueError, msg);
}
#define __Pyx_Coroutine_NotStartedError(gen)  (__Pyx__Coroutine_NotStartedError(gen), (PyObject*)NULL)
static void __Pyx__Coroutine_NotStartedError(CYTHON_UNUSED PyObject *gen) {
    const char *msg;
    if ((0)) {
    #ifdef __Pyx_Coroutine_USED
    } else if (__Pyx_Coroutine_Check(gen)) {
        msg = "can't send non-None value to a just-started coroutine";
    #endif
    #ifdef __Pyx_AsyncGen_USED
    } else if (__Pyx_AsyncGen_CheckExact(gen)) {
        msg = "can't send non-None value to a just-started async generator";
    #endif
    } else {
        msg = "can't send non-None value to a just-started generator";
    }
    PyErr_SetString(PyExc_TypeError, msg);
}
#define __Pyx_Coroutine_AlreadyTerminatedError(gen, value, closing)  (__Pyx__Coroutine_AlreadyTerminatedError(gen, value, closing), (PyObject*)NULL)
static void __Pyx__Coroutine_AlreadyTerminatedError(CYTHON_UNUSED PyObject *gen, PyObject *value, CYTHON_UNUSED int closing) {
    #ifdef __Pyx_Coroutine_USED
    if (!closing && __Pyx_Coroutine_Check(gen)) {
        PyErr_SetString(PyExc_RuntimeError, "cannot reuse already awaited coroutine");
    } else
    #endif
    if (value) {
        #ifdef __Pyx_AsyncGen_USED
        if (__Pyx_AsyncGen_CheckExact(gen))
            PyErr_SetNone(__Pyx_PyExc_StopAsyncIteration);
        else
        #endif
        PyErr_SetNone(PyExc_StopIteration);
    }
}
static
PyObject *__Pyx_Coroutine_SendEx(__pyx_CoroutineObject *self, PyObject *value, int closing) {
    __Pyx_PyThreadState_declare
    PyThreadState *tstate;
    __Pyx_ExcInfoStruct *exc_state;
    PyObject *retval;
    assert(!self->is_running);
    if (unlikely(self->resume_label == 0)) {
        if (unlikely(value && value != Py_None)) {
            return __Pyx_Coroutine_NotStartedError((PyObject*)self);
        }
    }
    if (unlikely(self->resume_label == -1)) {
        return __Pyx_Coroutine_AlreadyTerminatedError((PyObject*)self, value, closing);
    }
#if CYTHON_FAST_THREAD_STATE
    __Pyx_PyThreadState_assign
    tstate = __pyx_tstate;
#else
    tstate = __Pyx_PyThreadState_Current;
#endif
    exc_state = &self->gi_exc_state;
    if (exc_state->exc_type) {
        #if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_PYSTON
        #else
        if (exc_state->exc_traceback) {
            PyTracebackObject *tb = (PyTracebackObject *) exc_state->exc_traceback;
            PyFrameObject *f = tb->tb_frame;
            assert(f->f_back == NULL);
            #if PY_VERSION_HEX >= 0x030B00A1
            f->f_back = PyThreadState_GetFrame(tstate);
            #else
            Py_XINCREF(tstate->frame);
            f->f_back = tstate->frame;
            #endif
        }
        #endif
    }
#if CYTHON_USE_EXC_INFO_STACK
    exc_state->previous_item = tstate->exc_info;
    tstate->exc_info = exc_state;
#else
    if (exc_state->exc_type) {
        __Pyx_ExceptionSwap(&exc_state->exc_type, &exc_state->exc_value, &exc_state->exc_traceback);
    } else {
        __Pyx_Coroutine_ExceptionClear(exc_state);
        __Pyx_ExceptionSave(&exc_state->exc_type, &exc_state->exc_value, &exc_state->exc_traceback);
    }
#endif
    self->is_running = 1;
    retval = self->body((PyObject *) self, tstate, value);
    self->is_running = 0;
#if CYTHON_USE_EXC_INFO_STACK
    exc_state = &self->gi_exc_state;
    tstate->exc_info = exc_state->previous_item;
    exc_state->previous_item = NULL;
    __Pyx_Coroutine_ResetFrameBackpointer(exc_state);
#endif
    return retval;
}
static CYTHON_INLINE void __Pyx_Coroutine_ResetFrameBackpointer(__Pyx_ExcInfoStruct *exc_state) {
    PyObject *exc_tb = exc_state->exc_traceback;
    if (likely(exc_tb)) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_PYSTON
#else
        PyTracebackObject *tb = (PyTracebackObject *) exc_tb;
        PyFrameObject *f = tb->tb_frame;
        Py_CLEAR(f->f_back);
#endif
    }
}
static CYTHON_INLINE
PyObject *__Pyx_Coroutine_MethodReturn(CYTHON_UNUSED PyObject* gen, PyObject *retval) {
    if (unlikely(!retval)) {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        if (!__Pyx_PyErr_Occurred()) {
            PyObject *exc = PyExc_StopIteration;
            #ifdef __Pyx_AsyncGen_USED
            if (__Pyx_AsyncGen_CheckExact(gen))
                exc = __Pyx_PyExc_StopAsyncIteration;
            #endif
            __Pyx_PyErr_SetNone(exc);
        }
    }
    return retval;
}
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03030000 && (defined(__linux__) || PY_VERSION_HEX >= 0x030600B3)
static CYTHON_INLINE
PyObject *__Pyx_PyGen_Send(PyGenObject *gen, PyObject *arg) {
#if PY_VERSION_HEX <= 0x030A00A1
    return _PyGen_Send(gen, arg);
#else
    PyObject *result;
    if (PyIter_Send((PyObject*)gen, arg ? arg : Py_None, &result) == PYGEN_RETURN) {
        if (PyAsyncGen_CheckExact(gen)) {
            assert(result == Py_None);
            PyErr_SetNone(PyExc_StopAsyncIteration);
        }
        else if (result == Py_None) {
            PyErr_SetNone(PyExc_StopIteration);
        }
        else {
            _PyGen_SetStopIterationValue(result);
        }
        Py_CLEAR(result);
    }
    return result;
#endif
}
#endif
static CYTHON_INLINE
PyObject *__Pyx_Coroutine_FinishDelegation(__pyx_CoroutineObject *gen) {
    PyObject *ret;
    PyObject *val = NULL;
    __Pyx_Coroutine_Undelegate(gen);
    __Pyx_PyGen__FetchStopIterationValue(__Pyx_PyThreadState_Current, &val);
    ret = __Pyx_Coroutine_SendEx(gen, val, 0);
    Py_XDECREF(val);
    return ret;
}
static PyObject *__Pyx_Coroutine_Send(PyObject *self, PyObject *value) {
    PyObject *retval;
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject*) self;
    PyObject *yf = gen->yieldfrom;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        PyObject *ret;
        gen->is_running = 1;
        #ifdef __Pyx_Generator_USED
        if (__Pyx_Generator_CheckExact(yf)) {
            ret = __Pyx_Coroutine_Send(yf, value);
        } else
        #endif
        #ifdef __Pyx_Coroutine_USED
        if (__Pyx_Coroutine_Check(yf)) {
            ret = __Pyx_Coroutine_Send(yf, value);
        } else
        #endif
        #ifdef __Pyx_AsyncGen_USED
        if (__pyx_PyAsyncGenASend_CheckExact(yf)) {
            ret = __Pyx_async_gen_asend_send(yf, value);
        } else
        #endif
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03030000 && (defined(__linux__) || PY_VERSION_HEX >= 0x030600B3)
        if (PyGen_CheckExact(yf)) {
            ret = __Pyx_PyGen_Send((PyGenObject*)yf, value == Py_None ? NULL : value);
        } else
        #endif
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03050000 && defined(PyCoro_CheckExact) && (defined(__linux__) || PY_VERSION_HEX >= 0x030600B3)
        if (PyCoro_CheckExact(yf)) {
            ret = __Pyx_PyGen_Send((PyGenObject*)yf, value == Py_None ? NULL : value);
        } else
        #endif
        {
            if (value == Py_None)
                ret = Py_TYPE(yf)->tp_iternext(yf);
            else
                ret = __Pyx_PyObject_CallMethod1(yf, __pyx_n_s_send, value);
        }
        gen->is_running = 0;
        if (likely(ret)) {
            return ret;
        }
        retval = __Pyx_Coroutine_FinishDelegation(gen);
    } else {
        retval = __Pyx_Coroutine_SendEx(gen, value, 0);
    }
    return __Pyx_Coroutine_MethodReturn(self, retval);
}
static int __Pyx_Coroutine_CloseIter(__pyx_CoroutineObject *gen, PyObject *yf) {
    PyObject *retval = NULL;
    int err = 0;
    #ifdef __Pyx_Generator_USED
    if (__Pyx_Generator_CheckExact(yf)) {
        retval = __Pyx_Coroutine_Close(yf);
        if (!retval)
            return -1;
    } else
    #endif
    #ifdef __Pyx_Coroutine_USED
    if (__Pyx_Coroutine_Check(yf)) {
        retval = __Pyx_Coroutine_Close(yf);
        if (!retval)
            return -1;
    } else
    if (__Pyx_CoroutineAwait_CheckExact(yf)) {
        retval = __Pyx_CoroutineAwait_Close((__pyx_CoroutineAwaitObject*)yf, NULL);
        if (!retval)
            return -1;
    } else
    #endif
    #ifdef __Pyx_AsyncGen_USED
    if (__pyx_PyAsyncGenASend_CheckExact(yf)) {
        retval = __Pyx_async_gen_asend_close(yf, NULL);
    } else
    if (__pyx_PyAsyncGenAThrow_CheckExact(yf)) {
        retval = __Pyx_async_gen_athrow_close(yf, NULL);
    } else
    #endif
    {
        PyObject *meth;
        gen->is_running = 1;
        meth = __Pyx_PyObject_GetAttrStr(yf, __pyx_n_s_close);
        if (unlikely(!meth)) {
            if (!PyErr_ExceptionMatches(PyExc_AttributeError)) {
                PyErr_WriteUnraisable(yf);
            }
            PyErr_Clear();
        } else {
            retval = PyObject_CallFunction(meth, NULL);
            Py_DECREF(meth);
            if (!retval)
                err = -1;
        }
        gen->is_running = 0;
    }
    Py_XDECREF(retval);
    return err;
}
static PyObject *__Pyx_Generator_Next(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject*) self;
    PyObject *yf = gen->yieldfrom;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        PyObject *ret;
        gen->is_running = 1;
        #ifdef __Pyx_Generator_USED
        if (__Pyx_Generator_CheckExact(yf)) {
            ret = __Pyx_Generator_Next(yf);
        } else
        #endif
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03030000 && (defined(__linux__) || PY_VERSION_HEX >= 0x030600B3)
        if (PyGen_CheckExact(yf)) {
            ret = __Pyx_PyGen_Send((PyGenObject*)yf, NULL);
        } else
        #endif
        #ifdef __Pyx_Coroutine_USED
        if (__Pyx_Coroutine_Check(yf)) {
            ret = __Pyx_Coroutine_Send(yf, Py_None);
        } else
        #endif
            ret = Py_TYPE(yf)->tp_iternext(yf);
        gen->is_running = 0;
        if (likely(ret)) {
            return ret;
        }
        return __Pyx_Coroutine_FinishDelegation(gen);
    }
    return __Pyx_Coroutine_SendEx(gen, Py_None, 0);
}
static PyObject *__Pyx_Coroutine_Close_Method(PyObject *self, CYTHON_UNUSED PyObject *arg) {
    return __Pyx_Coroutine_Close(self);
}
static PyObject *__Pyx_Coroutine_Close(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject *retval, *raised_exception;
    PyObject *yf = gen->yieldfrom;
    int err = 0;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        Py_INCREF(yf);
        err = __Pyx_Coroutine_CloseIter(gen, yf);
        __Pyx_Coroutine_Undelegate(gen);
        Py_DECREF(yf);
    }
    if (err == 0)
        PyErr_SetNone(PyExc_GeneratorExit);
    retval = __Pyx_Coroutine_SendEx(gen, NULL, 1);
    if (unlikely(retval)) {
        const char *msg;
        Py_DECREF(retval);
        if ((0)) {
        #ifdef __Pyx_Coroutine_USED
        } else if (__Pyx_Coroutine_Check(self)) {
            msg = "coroutine ignored GeneratorExit";
        #endif
        #ifdef __Pyx_AsyncGen_USED
        } else if (__Pyx_AsyncGen_CheckExact(self)) {
#if PY_VERSION_HEX < 0x03060000
            msg = "async generator ignored GeneratorExit - might require Python 3.6+ finalisation (PEP 525)";
#else
            msg = "async generator ignored GeneratorExit";
#endif
        #endif
        } else {
            msg = "generator ignored GeneratorExit";
        }
        PyErr_SetString(PyExc_RuntimeError, msg);
        return NULL;
    }
    raised_exception = PyErr_Occurred();
    if (likely(!raised_exception || __Pyx_PyErr_GivenExceptionMatches2(raised_exception, PyExc_GeneratorExit, PyExc_StopIteration))) {
        if (raised_exception) PyErr_Clear();
        Py_INCREF(Py_None);
        return Py_None;
    }
    return NULL;
}
static PyObject *__Pyx__Coroutine_Throw(PyObject *self, PyObject *typ, PyObject *val, PyObject *tb,
                                        PyObject *args, int close_on_genexit) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject *yf = gen->yieldfrom;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        PyObject *ret;
        Py_INCREF(yf);
        if (__Pyx_PyErr_GivenExceptionMatches(typ, PyExc_GeneratorExit) && close_on_genexit) {
            int err = __Pyx_Coroutine_CloseIter(gen, yf);
            Py_DECREF(yf);
            __Pyx_Coroutine_Undelegate(gen);
            if (err < 0)
                return __Pyx_Coroutine_MethodReturn(self, __Pyx_Coroutine_SendEx(gen, NULL, 0));
            goto throw_here;
        }
        gen->is_running = 1;
        if (0
        #ifdef __Pyx_Generator_USED
            || __Pyx_Generator_CheckExact(yf)
        #endif
        #ifdef __Pyx_Coroutine_USED
            || __Pyx_Coroutine_Check(yf)
        #endif
            ) {
            ret = __Pyx__Coroutine_Throw(yf, typ, val, tb, args, close_on_genexit);
        #ifdef __Pyx_Coroutine_USED
        } else if (__Pyx_CoroutineAwait_CheckExact(yf)) {
            ret = __Pyx__Coroutine_Throw(((__pyx_CoroutineAwaitObject*)yf)->coroutine, typ, val, tb, args, close_on_genexit);
        #endif
        } else {
            PyObject *meth = __Pyx_PyObject_GetAttrStr(yf, __pyx_n_s_throw);
            if (unlikely(!meth)) {
                Py_DECREF(yf);
                if (!PyErr_ExceptionMatches(PyExc_AttributeError)) {
                    gen->is_running = 0;
                    return NULL;
                }
                PyErr_Clear();
                __Pyx_Coroutine_Undelegate(gen);
                gen->is_running = 0;
                goto throw_here;
            }
            if (likely(args)) {
                ret = PyObject_CallObject(meth, args);
            } else {
                ret = PyObject_CallFunctionObjArgs(meth, typ, val, tb, NULL);
            }
            Py_DECREF(meth);
        }
        gen->is_running = 0;
        Py_DECREF(yf);
        if (!ret) {
            ret = __Pyx_Coroutine_FinishDelegation(gen);
        }
        return __Pyx_Coroutine_MethodReturn(self, ret);
    }
throw_here:
    __Pyx_Raise(typ, val, tb, NULL);
    return __Pyx_Coroutine_MethodReturn(self, __Pyx_Coroutine_SendEx(gen, NULL, 0));
}
static PyObject *__Pyx_Coroutine_Throw(PyObject *self, PyObject *args) {
    PyObject *typ;
    PyObject *val = NULL;
    PyObject *tb = NULL;
    if (!PyArg_UnpackTuple(args, (char *)"throw", 1, 3, &typ, &val, &tb))
        return NULL;
    return __Pyx__Coroutine_Throw(self, typ, val, tb, args, 1);
}
static CYTHON_INLINE int __Pyx_Coroutine_traverse_excstate(__Pyx_ExcInfoStruct *exc_state, visitproc visit, void *arg) {
    Py_VISIT(exc_state->exc_type);
    Py_VISIT(exc_state->exc_value);
    Py_VISIT(exc_state->exc_traceback);
    return 0;
}
static int __Pyx_Coroutine_traverse(__pyx_CoroutineObject *gen, visitproc visit, void *arg) {
    Py_VISIT(gen->closure);
    Py_VISIT(gen->classobj);
    Py_VISIT(gen->yieldfrom);
    return __Pyx_Coroutine_traverse_excstate(&gen->gi_exc_state, visit, arg);
}
static int __Pyx_Coroutine_clear(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    Py_CLEAR(gen->closure);
    Py_CLEAR(gen->classobj);
    Py_CLEAR(gen->yieldfrom);
    __Pyx_Coroutine_ExceptionClear(&gen->gi_exc_state);
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        Py_CLEAR(((__pyx_PyAsyncGenObject*)gen)->ag_finalizer);
    }
#endif
    Py_CLEAR(gen->gi_code);
    Py_CLEAR(gen->gi_frame);
    Py_CLEAR(gen->gi_name);
    Py_CLEAR(gen->gi_qualname);
    Py_CLEAR(gen->gi_modulename);
    return 0;
}
static void __Pyx_Coroutine_dealloc(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject_GC_UnTrack(gen);
    if (gen->gi_weakreflist != NULL)
        PyObject_ClearWeakRefs(self);
    if (gen->resume_label >= 0) {
        PyObject_GC_Track(self);
#if PY_VERSION_HEX >= 0x030400a1 && CYTHON_USE_TP_FINALIZE
        if (PyObject_CallFinalizerFromDealloc(self))
#else
        Py_TYPE(gen)->tp_del(self);
        if (Py_REFCNT(self) > 0)
#endif
        {
            return;
        }
        PyObject_GC_UnTrack(self);
    }
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        /* We have to handle this case for asynchronous generators
           right here, because this code has to be between UNTRACK
           and GC_Del. */
        Py_CLEAR(((__pyx_PyAsyncGenObject*)self)->ag_finalizer);
    }
#endif
    __Pyx_Coroutine_clear(self);
    PyObject_GC_Del(gen);
}
static void __Pyx_Coroutine_del(PyObject *self) {
    PyObject *error_type, *error_value, *error_traceback;
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    __Pyx_PyThreadState_declare
    if (gen->resume_label < 0) {
        return;
    }
#if !CYTHON_USE_TP_FINALIZE
    assert(self->ob_refcnt == 0);
    __Pyx_SET_REFCNT(self, 1);
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&error_type, &error_value, &error_traceback);
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        __pyx_PyAsyncGenObject *agen = (__pyx_PyAsyncGenObject*)self;
        PyObject *finalizer = agen->ag_finalizer;
        if (finalizer && !agen->ag_closed) {
            PyObject *res = __Pyx_PyObject_CallOneArg(finalizer, self);
            if (unlikely(!res)) {
                PyErr_WriteUnraisable(self);
            } else {
                Py_DECREF(res);
            }
            __Pyx_ErrRestore(error_type, error_value, error_traceback);
            return;
        }
    }
#endif
    if (unlikely(gen->resume_label == 0 && !error_value)) {
#ifdef __Pyx_Coroutine_USED
#ifdef __Pyx_Generator_USED
    if (!__Pyx_Generator_CheckExact(self))
#endif
        {
        PyObject_GC_UnTrack(self);
#if PY_MAJOR_VERSION >= 3  || defined(PyErr_WarnFormat)
        if (unlikely(PyErr_WarnFormat(PyExc_RuntimeWarning, 1, "coroutine '%.50S' was never awaited", gen->gi_qualname) < 0))
            PyErr_WriteUnraisable(self);
#else
        {PyObject *msg;
        char *cmsg;
        #if CYTHON_COMPILING_IN_PYPY
        msg = NULL;
        cmsg = (char*) "coroutine was never awaited";
        #else
        char *cname;
        PyObject *qualname;
        qualname = gen->gi_qualname;
        cname = PyString_AS_STRING(qualname);
        msg = PyString_FromFormat("coroutine '%.50s' was never awaited", cname);
        if (unlikely(!msg)) {
            PyErr_Clear();
            cmsg = (char*) "coroutine was never awaited";
        } else {
            cmsg = PyString_AS_STRING(msg);
        }
        #endif
        if (unlikely(PyErr_WarnEx(PyExc_RuntimeWarning, cmsg, 1) < 0))
            PyErr_WriteUnraisable(self);
        Py_XDECREF(msg);}
#endif
        PyObject_GC_Track(self);
        }
#endif
    } else {
        PyObject *res = __Pyx_Coroutine_Close(self);
        if (unlikely(!res)) {
            if (PyErr_Occurred())
                PyErr_WriteUnraisable(self);
        } else {
            Py_DECREF(res);
        }
    }
    __Pyx_ErrRestore(error_type, error_value, error_traceback);
#if !CYTHON_USE_TP_FINALIZE
    assert(Py_REFCNT(self) > 0);
    if (--self->ob_refcnt == 0) {
        return;
    }
    {
        Py_ssize_t refcnt = Py_REFCNT(self);
        _Py_NewReference(self);
        __Pyx_SET_REFCNT(self, refcnt);
    }
#if CYTHON_COMPILING_IN_CPYTHON
    assert(PyType_IS_GC(Py_TYPE(self)) &&
           _Py_AS_GC(self)->gc.gc_refs != _PyGC_REFS_UNTRACKED);
    _Py_DEC_REFTOTAL;
#endif
#ifdef COUNT_ALLOCS
    --Py_TYPE(self)->tp_frees;
    --Py_TYPE(self)->tp_allocs;
#endif
#endif
}
static PyObject *
__Pyx_Coroutine_get_name(__pyx_CoroutineObject *self, CYTHON_UNUSED void *context)
{
    PyObject *name = self->gi_name;
    if (unlikely(!name)) name = Py_None;
    Py_INCREF(name);
    return name;
}
static int
__Pyx_Coroutine_set_name(__pyx_CoroutineObject *self, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    tmp = self->gi_name;
    Py_INCREF(value);
    self->gi_name = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_Coroutine_get_qualname(__pyx_CoroutineObject *self, CYTHON_UNUSED void *context)
{
    PyObject *name = self->gi_qualname;
    if (unlikely(!name)) name = Py_None;
    Py_INCREF(name);
    return name;
}
static int
__Pyx_Coroutine_set_qualname(__pyx_CoroutineObject *self, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    tmp = self->gi_qualname;
    Py_INCREF(value);
    self->gi_qualname = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_Coroutine_get_frame(__pyx_CoroutineObject *self, CYTHON_UNUSED void *context)
{
    PyObject *frame = self->gi_frame;
    if (!frame) {
        if (unlikely(!self->gi_code)) {
            Py_RETURN_NONE;
        }
        frame = (PyObject *) PyFrame_New(
            PyThreadState_Get(),            /*PyThreadState *tstate,*/
            (PyCodeObject*) self->gi_code,  /*PyCodeObject *code,*/
            __pyx_d,                 /*PyObject *globals,*/
            0                               /*PyObject *locals*/
        );
        if (unlikely(!frame))
            return NULL;
        self->gi_frame = frame;
    }
    Py_INCREF(frame);
    return frame;
}
static __pyx_CoroutineObject *__Pyx__Coroutine_New(
            PyTypeObject* type, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name) {
    __pyx_CoroutineObject *gen = PyObject_GC_New(__pyx_CoroutineObject, type);
    if (unlikely(!gen))
        return NULL;
    return __Pyx__Coroutine_NewInit(gen, body, code, closure, name, qualname, module_name);
}
static __pyx_CoroutineObject *__Pyx__Coroutine_NewInit(
            __pyx_CoroutineObject *gen, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name) {
    gen->body = body;
    gen->closure = closure;
    Py_XINCREF(closure);
    gen->is_running = 0;
    gen->resume_label = 0;
    gen->classobj = NULL;
    gen->yieldfrom = NULL;
    gen->gi_exc_state.exc_type = NULL;
    gen->gi_exc_state.exc_value = NULL;
    gen->gi_exc_state.exc_traceback = NULL;
#if CYTHON_USE_EXC_INFO_STACK
    gen->gi_exc_state.previous_item = NULL;
#endif
    gen->gi_weakreflist = NULL;
    Py_XINCREF(qualname);
    gen->gi_qualname = qualname;
    Py_XINCREF(name);
    gen->gi_name = name;
    Py_XINCREF(module_name);
    gen->gi_modulename = module_name;
    Py_XINCREF(code);
    gen->gi_code = code;
    gen->gi_frame = NULL;
    PyObject_GC_Track(gen);
    return gen;
}

/* PatchModuleWithCoroutine */
static PyObject* __Pyx_Coroutine_patch_module(PyObject* module, const char* py_code) {
#if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
    int result;
    PyObject *globals, *result_obj;
    globals = PyDict_New();  if (unlikely(!globals)) goto ignore;
    result = PyDict_SetItemString(globals, "_cython_coroutine_type",
    #ifdef __Pyx_Coroutine_USED
        (PyObject*)__pyx_CoroutineType);
    #else
        Py_None);
    #endif
    if (unlikely(result < 0)) goto ignore;
    result = PyDict_SetItemString(globals, "_cython_generator_type",
    #ifdef __Pyx_Generator_USED
        (PyObject*)__pyx_GeneratorType);
    #else
        Py_None);
    #endif
    if (unlikely(result < 0)) goto ignore;
    if (unlikely(PyDict_SetItemString(globals, "_module", module) < 0)) goto ignore;
    if (unlikely(PyDict_SetItemString(globals, "__builtins__", __pyx_b) < 0)) goto ignore;
    result_obj = PyRun_String(py_code, Py_file_input, globals, globals);
    if (unlikely(!result_obj)) goto ignore;
    Py_DECREF(result_obj);
    Py_DECREF(globals);
    return module;
ignore:
    Py_XDECREF(globals);
    PyErr_WriteUnraisable(module);
    if (unlikely(PyErr_WarnEx(PyExc_RuntimeWarning, "Cython module failed to patch module with custom type", 1) < 0)) {
        Py_DECREF(module);
        module = NULL;
    }
#else
    py_code++;
#endif
    return module;
}

/* PatchGeneratorABC */
#ifndef CYTHON_REGISTER_ABCS
#define CYTHON_REGISTER_ABCS 1
#endif
#if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
static PyObject* __Pyx_patch_abc_module(PyObject *module);
static PyObject* __Pyx_patch_abc_module(PyObject *module) {
    module = __Pyx_Coroutine_patch_module(
        module, ""
"if _cython_generator_type is not None:\n"
"    try: Generator = _module.Generator\n"
"    except AttributeError: pass\n"
"    else: Generator.register(_cython_generator_type)\n"
"if _cython_coroutine_type is not None:\n"
"    try: Coroutine = _module.Coroutine\n"
"    except AttributeError: pass\n"
"    else: Coroutine.register(_cython_coroutine_type)\n"
    );
    return module;
}
#endif
static int __Pyx_patch_abc(void) {
#if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
    static int abc_patched = 0;
    if (CYTHON_REGISTER_ABCS && !abc_patched) {
        PyObject *module;
        module = PyImport_ImportModule((PY_MAJOR_VERSION >= 3) ? "collections.abc" : "collections");
        if (!module) {
            PyErr_WriteUnraisable(NULL);
            if (unlikely(PyErr_WarnEx(PyExc_RuntimeWarning,
                    ((PY_MAJOR_VERSION >= 3) ?
                        "Cython module failed to register with collections.abc module" :
                        "Cython module failed to register with collections module"), 1) < 0)) {
                return -1;
            }
        } else {
            module = __Pyx_patch_abc_module(module);
            abc_patched = 1;
            if (unlikely(!module))
                return -1;
            Py_DECREF(module);
        }
        module = PyImport_ImportModule("backports_abc");
        if (module) {
            module = __Pyx_patch_abc_module(module);
            Py_XDECREF(module);
        }
        if (!module) {
            PyErr_Clear();
        }
    }
#else
    if ((0)) __Pyx_Coroutine_patch_module(NULL, NULL);
#endif
    return 0;
}

/* Generator */
static PyMethodDef __pyx_Generator_methods[] = {
    {"send", (PyCFunction) __Pyx_Coroutine_Send, METH_O,
     (char*) PyDoc_STR("send(arg) -> send 'arg' into generator,\nreturn next yielded value or raise StopIteration.")},
    {"throw", (PyCFunction) __Pyx_Coroutine_Throw, METH_VARARGS,
     (char*) PyDoc_STR("throw(typ[,val[,tb]]) -> raise exception in generator,\nreturn next yielded value or raise StopIteration.")},
    {"close", (PyCFunction) __Pyx_Coroutine_Close_Method, METH_NOARGS,
     (char*) PyDoc_STR("close() -> raise GeneratorExit inside generator.")},
    {0, 0, 0, 0}
};
static PyMemberDef __pyx_Generator_memberlist[] = {
    {(char *) "gi_running", T_BOOL, offsetof(__pyx_CoroutineObject, is_running), READONLY, NULL},
    {(char*) "gi_yieldfrom", T_OBJECT, offsetof(__pyx_CoroutineObject, yieldfrom), READONLY,
     (char*) PyDoc_STR("object being iterated by 'yield from', or None")},
    {(char*) "gi_code", T_OBJECT, offsetof(__pyx_CoroutineObject, gi_code), READONLY, NULL},
    {0, 0, 0, 0, 0}
};
static PyGetSetDef __pyx_Generator_getsets[] = {
    {(char *) "__name__", (getter)__Pyx_Coroutine_get_name, (setter)__Pyx_Coroutine_set_name,
     (char*) PyDoc_STR("name of the generator"), 0},
    {(char *) "__qualname__", (getter)__Pyx_Coroutine_get_qualname, (setter)__Pyx_Coroutine_set_qualname,
     (char*) PyDoc_STR("qualified name of the generator"), 0},
    {(char *) "gi_frame", (getter)__Pyx_Coroutine_get_frame, NULL,
     (char*) PyDoc_STR("Frame of the generator"), 0},
    {0, 0, 0, 0, 0}
};
static PyTypeObject __pyx_GeneratorType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    "generator",
    sizeof(__pyx_CoroutineObject),
    0,
    (destructor) __Pyx_Coroutine_dealloc,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_HAVE_FINALIZE,
    0,
    (traverseproc) __Pyx_Coroutine_traverse,
    0,
    0,
    offsetof(__pyx_CoroutineObject, gi_weakreflist),
    0,
    (iternextfunc) __Pyx_Generator_Next,
    __pyx_Generator_methods,
    __pyx_Generator_memberlist,
    __pyx_Generator_getsets,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if CYTHON_USE_TP_FINALIZE
    0,
#else
    __Pyx_Coroutine_del,
#endif
    0,
#if CYTHON_USE_TP_FINALIZE
    __Pyx_Coroutine_del,
#elif PY_VERSION_HEX >= 0x030400a1
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
    0,
#endif
#if PY_VERSION_HEX >= 0x030C0000
    0,
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
    0,
#endif
};
static int __pyx_Generator_init(void) {
    __pyx_GeneratorType_type.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
    __pyx_GeneratorType_type.tp_iter = PyObject_SelfIter;
    __pyx_GeneratorType = __Pyx_FetchCommonType(&__pyx_GeneratorType_type);
    if (unlikely(!__pyx_GeneratorType)) {
        return -1;
    }
    return 0;
}

/* CheckBinaryVersion */
static int __Pyx_check_binary_version(void) {
    char ctversion[5];
    int same=1, i, found_dot;
    const char* rt_from_call = Py_GetVersion();
    PyOS_snprintf(ctversion, 5, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    found_dot = 0;
    for (i = 0; i < 4; i++) {
        if (!ctversion[i]) {
            same = (rt_from_call[i] < '0' || rt_from_call[i] > '9');
            break;
        }
        if (rt_from_call[i] != ctversion[i]) {
            same = 0;
            break;
        }
    }
    if (!same) {
        char rtversion[5] = {'\0'};
        char message[200];
        for (i=0; i<4; ++i) {
            if (rt_from_call[i] == '.') {
                if (found_dot) break;
                found_dot = 1;
            } else if (rt_from_call[i] < '0' || rt_from_call[i] > '9') {
                break;
            }
            rtversion[i] = rt_from_call[i];
        }
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* PyObjectSetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_setattro))
        return tp->tp_setattro(obj, attr_name, value);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_setattr))
        return tp->tp_setattr(obj, PyString_AS_STRING(attr_name), value);
#endif
    return PyObject_SetAttr(obj, attr_name, value);
}
#endif

/* VoidPtrExport */
static int __Pyx_ExportVoidPtr(PyObject *name, void *p, const char *sig) {
    PyObject *d;
    PyObject *cobj = 0;
    d = PyDict_GetItem(__pyx_d, __pyx_n_s_pyx_capi);
    Py_XINCREF(d);
    if (!d) {
        d = PyDict_New();
        if (!d)
            goto bad;
        if (__Pyx_PyObject_SetAttrStr(__pyx_m, __pyx_n_s_pyx_capi, d) < 0)
            goto bad;
    }
#if PY_VERSION_HEX >= 0x02070000
    cobj = PyCapsule_New(p, sig, 0);
#else
    cobj = PyCObject_FromVoidPtrAndDesc(p, (void *)sig, 0);
#endif
    if (!cobj)
        goto bad;
    if (PyDict_SetItem(d, name, cobj) < 0)
        goto bad;
    Py_DECREF(cobj);
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(cobj);
    Py_XDECREF(d);
    return -1;
}

/* VoidPtrImport */
#ifndef __PYX_HAVE_RT_ImportVoidPtr_0_29_35
#define __PYX_HAVE_RT_ImportVoidPtr_0_29_35
static int __Pyx_ImportVoidPtr_0_29_35(PyObject *module, const char *name, void **p, const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    d = PyObject_GetAttrString(module, (char *)"__pyx_capi__");
    if (!d)
        goto bad;
    cobj = PyDict_GetItemString(d, name);
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%.200s does not export expected C variable %.200s",
                PyModule_GetName(module), name);
        goto bad;
    }
#if PY_VERSION_HEX >= 0x02070000
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C variable %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), name, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    *p = PyCapsule_GetPointer(cobj, sig);
#else
    {const char *desc, *s1, *s2;
    desc = (const char *)PyCObject_GetDesc(cobj);
    if (!desc)
        goto bad;
    s1 = desc; s2 = sig;
    while (*s1 != '\0' && *s1 == *s2) { s1++; s2++; }
    if (*s1 != *s2) {
        PyErr_Format(PyExc_TypeError,
            "C variable %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), name, sig, desc);
        goto bad;
    }
    *p = PyCObject_AsVoidPtr(cobj);}
#endif
    if (!(*p))
        goto bad;
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(d);
    return -1;
}
#endif

/* InitStrings */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(b);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject* o) {
  if (sizeof(Py_hash_t) == sizeof(Py_ssize_t)) {
    return (Py_hash_t) __Pyx_PyIndex_AsSsize_t(o);
#if PY_MAJOR_VERSION < 3
  } else if (likely(PyInt_CheckExact(o))) {
    return PyInt_AS_LONG(o);
#endif
  } else {
    Py_ssize_t ival;
    PyObject *x;
    x = PyNumber_Index(o);
    if (!x) return -1;
    ival = PyInt_AsLong(x);
    Py_DECREF(x);
    return ival;
  }
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return b ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
