/* Generated by Cython 0.29.35 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "name": "zeroconf._protocol.incoming",
        "sources": [
            "src/zeroconf/_protocol/incoming.py"
        ]
    },
    "module_name": "zeroconf._protocol.incoming"
}
END: Cython Metadata */

#ifndef PY_SSIZE_T_CLEAN
#define PY_SSIZE_T_CLEAN
#endif /* PY_SSIZE_T_CLEAN */
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_29_35"
#define CYTHON_HEX_VERSION 0x001D23F0
#define CYTHON_FUTURE_DIVISION 1
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_NOGIL 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #if PY_VERSION_HEX < 0x03090000
    #undef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #elif !defined(CYTHON_PEP489_MULTI_PHASE_INIT)
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_NOGIL 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
#elif defined(PY_NOGIL)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_NOGIL 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #ifndef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE 1
  #endif
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #define CYTHON_COMPILING_IN_NOGIL 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS (PY_VERSION_HEX < 0x030C00A5)
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0 || PY_VERSION_HEX >= 0x030B00A2
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #if PY_VERSION_HEX >= 0x030B00A4
    #undef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 0
  #elif !defined(CYTHON_FAST_THREAD_STATE)
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL (PY_VERSION_HEX < 0x030A0000)
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
  #ifndef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS ((PY_VERSION_HEX >= 0x030600B1) && (PY_VERSION_HEX < 0x030C00A5))
  #endif
  #if PY_VERSION_HEX >= 0x030B00A4
    #undef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK 0
  #elif !defined(CYTHON_USE_EXC_INFO_STACK)
    #define CYTHON_USE_EXC_INFO_STACK (PY_VERSION_HEX >= 0x030700A3)
  #endif
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 1
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #if PY_MAJOR_VERSION < 3
    #include "longintrepr.h"
  #endif
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_DefaultClassType PyType_Type
#if PY_VERSION_HEX >= 0x030B00A1
    static CYTHON_INLINE PyCodeObject* __Pyx_PyCode_New(int a, int k, int l, int s, int f,
                                                    PyObject *code, PyObject *c, PyObject* n, PyObject *v,
                                                    PyObject *fv, PyObject *cell, PyObject* fn,
                                                    PyObject *name, int fline, PyObject *lnos) {
        PyObject *kwds=NULL, *argcount=NULL, *posonlyargcount=NULL, *kwonlyargcount=NULL;
        PyObject *nlocals=NULL, *stacksize=NULL, *flags=NULL, *replace=NULL, *call_result=NULL, *empty=NULL;
        const char *fn_cstr=NULL;
        const char *name_cstr=NULL;
        PyCodeObject* co=NULL;
        PyObject *type, *value, *traceback;
        PyErr_Fetch(&type, &value, &traceback);
        if (!(kwds=PyDict_New())) goto end;
        if (!(argcount=PyLong_FromLong(a))) goto end;
        if (PyDict_SetItemString(kwds, "co_argcount", argcount) != 0) goto end;
        if (!(posonlyargcount=PyLong_FromLong(0))) goto end;
        if (PyDict_SetItemString(kwds, "co_posonlyargcount", posonlyargcount) != 0) goto end;
        if (!(kwonlyargcount=PyLong_FromLong(k))) goto end;
        if (PyDict_SetItemString(kwds, "co_kwonlyargcount", kwonlyargcount) != 0) goto end;
        if (!(nlocals=PyLong_FromLong(l))) goto end;
        if (PyDict_SetItemString(kwds, "co_nlocals", nlocals) != 0) goto end;
        if (!(stacksize=PyLong_FromLong(s))) goto end;
        if (PyDict_SetItemString(kwds, "co_stacksize", stacksize) != 0) goto end;
        if (!(flags=PyLong_FromLong(f))) goto end;
        if (PyDict_SetItemString(kwds, "co_flags", flags) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_code", code) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_consts", c) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_names", n) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_varnames", v) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_freevars", fv) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_cellvars", cell) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_linetable", lnos) != 0) goto end;
        if (!(fn_cstr=PyUnicode_AsUTF8AndSize(fn, NULL))) goto end;
        if (!(name_cstr=PyUnicode_AsUTF8AndSize(name, NULL))) goto end;
        if (!(co = PyCode_NewEmpty(fn_cstr, name_cstr, fline))) goto end;
        if (!(replace = PyObject_GetAttrString((PyObject*)co, "replace"))) goto cleanup_code_too;
        if (!(empty = PyTuple_New(0))) goto cleanup_code_too; // unfortunately __pyx_empty_tuple isn't available here
        if (!(call_result = PyObject_Call(replace, empty, kwds))) goto cleanup_code_too;
        Py_XDECREF((PyObject*)co);
        co = (PyCodeObject*)call_result;
        call_result = NULL;
        if (0) {
            cleanup_code_too:
            Py_XDECREF((PyObject*)co);
            co = NULL;
        }
        end:
        Py_XDECREF(kwds);
        Py_XDECREF(argcount);
        Py_XDECREF(posonlyargcount);
        Py_XDECREF(kwonlyargcount);
        Py_XDECREF(nlocals);
        Py_XDECREF(stacksize);
        Py_XDECREF(replace);
        Py_XDECREF(call_result);
        Py_XDECREF(empty);
        if (type) {
            PyErr_Restore(type, value, traceback);
        }
        return co;
    }
#else
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#if PY_VERSION_HEX <= 0x030700A3 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030400A1
  #define PyMem_RawMalloc(n)           PyMem_Malloc(n)
  #define PyMem_RawRealloc(p, n)       PyMem_Realloc(p, n)
  #define PyMem_RawFree(p)             PyMem_Free(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if PY_VERSION_HEX < 0x030700A2 && !defined(PyThread_tss_create) && !defined(Py_tss_NEEDS_INIT)
#include "pythread.h"
#define Py_tss_NEEDS_INIT 0
typedef int Py_tss_t;
static CYTHON_INLINE int PyThread_tss_create(Py_tss_t *key) {
  *key = PyThread_create_key();
  return 0;
}
static CYTHON_INLINE Py_tss_t * PyThread_tss_alloc(void) {
  Py_tss_t *key = (Py_tss_t *)PyObject_Malloc(sizeof(Py_tss_t));
  *key = Py_tss_NEEDS_INIT;
  return key;
}
static CYTHON_INLINE void PyThread_tss_free(Py_tss_t *key) {
  PyObject_Free(key);
}
static CYTHON_INLINE int PyThread_tss_is_created(Py_tss_t *key) {
  return *key != Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE void PyThread_tss_delete(Py_tss_t *key) {
  PyThread_delete_key(*key);
  *key = Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE int PyThread_tss_set(Py_tss_t *key, void *value) {
  return PyThread_set_key_value(*key, value);
}
static CYTHON_INLINE void * PyThread_tss_get(Py_tss_t *key) {
  return PyThread_get_key_value(*key);
}
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1 && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStr(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
#else
#define __Pyx_PyDict_GetItemStr(dict, name)  PyDict_GetItem(dict, name)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #if PY_VERSION_HEX >= 0x030C0000
    #define __Pyx_PyUnicode_READY(op)       (0)
  #else
    #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                                0 : _PyUnicode_Ready((PyObject *)(op)))
  #endif
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #if PY_VERSION_HEX >= 0x030C0000
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_LENGTH(u))
  #else
    #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03090000
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : ((PyCompactUnicodeObject *)(u))->wstr_length))
    #else
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
    #endif
  #endif
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None || (PyString_Check(b) && !PyString_CheckExact(b)))) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#ifndef PyObject_Unicode
  #define PyObject_Unicode             PyObject_Str
#endif
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#if PY_VERSION_HEX >= 0x030900A4
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_SET_REFCNT(obj, refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SET_SIZE(obj, size)
#else
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_REFCNT(obj) = (refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SIZE(obj) = (size)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
#else
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsHash_t
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? ((void)(klass), PyMethod_New(func, self)) : __Pyx_NewRef(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif

#if defined(_WIN32) || defined(WIN32) || defined(MS_WINDOWS)
  #if !defined(_USE_MATH_DEFINES)
    #define _USE_MATH_DEFINES
  #endif
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif

#define __PYX_MARK_ERR_POS(f_index, lineno) \
    { __pyx_filename = __pyx_f[f_index]; (void)__pyx_filename; __pyx_lineno = lineno; (void)__pyx_lineno; __pyx_clineno = __LINE__; (void)__pyx_clineno; }
#define __PYX_ERR(f_index, lineno, Ln_error) \
    { __PYX_MARK_ERR_POS(f_index, lineno) goto Ln_error; }

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__zeroconf___protocol__incoming
#define __PYX_HAVE_API__zeroconf___protocol__incoming
/* Early includes */
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT (PY_MAJOR_VERSION >= 3 && __PYX_DEFAULT_STRING_ENCODING_IS_UTF8)
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject*);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c) + 1);
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime = NULL;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "src/zeroconf/_protocol/incoming.py",
  "src/zeroconf/_protocol/incoming.pxd",
  "stringsource",
  "src/zeroconf/_dns.pxd",
};

/*--- Type declarations ---*/
struct __pyx_obj_8zeroconf_4_dns_DNSEntry;
struct __pyx_obj_8zeroconf_4_dns_DNSQuestion;
struct __pyx_obj_8zeroconf_4_dns_DNSRecord;
struct __pyx_obj_8zeroconf_4_dns_DNSAddress;
struct __pyx_obj_8zeroconf_4_dns_DNSHinfo;
struct __pyx_obj_8zeroconf_4_dns_DNSPointer;
struct __pyx_obj_8zeroconf_4_dns_DNSText;
struct __pyx_obj_8zeroconf_4_dns_DNSService;
struct __pyx_obj_8zeroconf_4_dns_DNSNsec;
struct __pyx_obj_8zeroconf_4_dns_DNSRRSet;
struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming;

/* "_dns.pxd":21
 * cdef object current_time_millis
 * 
 * cdef class DNSEntry:             # <<<<<<<<<<<<<<
 * 
 *     cdef public object key
 */
struct __pyx_obj_8zeroconf_4_dns_DNSEntry {
  PyObject_HEAD
  struct __pyx_vtabstruct_8zeroconf_4_dns_DNSEntry *__pyx_vtab;
  PyObject *key;
  PyObject *name;
  PyObject *type;
  PyObject *class_;
  PyObject *unique;
};


/* "_dns.pxd":31
 *     cdef _dns_entry_matches(self, DNSEntry other)
 * 
 * cdef class DNSQuestion(DNSEntry):             # <<<<<<<<<<<<<<
 * 
 *     cdef public cython.int _hash
 */
struct __pyx_obj_8zeroconf_4_dns_DNSQuestion {
  struct __pyx_obj_8zeroconf_4_dns_DNSEntry __pyx_base;
  int _hash;
};


/* "_dns.pxd":35
 *     cdef public cython.int _hash
 * 
 * cdef class DNSRecord(DNSEntry):             # <<<<<<<<<<<<<<
 * 
 *     cdef public object ttl
 */
struct __pyx_obj_8zeroconf_4_dns_DNSRecord {
  struct __pyx_obj_8zeroconf_4_dns_DNSEntry __pyx_base;
  PyObject *ttl;
  PyObject *created;
};


/* "_dns.pxd":43
 * 
 * 
 * cdef class DNSAddress(DNSRecord):             # <<<<<<<<<<<<<<
 * 
 *     cdef public cython.int _hash
 */
struct __pyx_obj_8zeroconf_4_dns_DNSAddress {
  struct __pyx_obj_8zeroconf_4_dns_DNSRecord __pyx_base;
  int _hash;
  PyObject *address;
  PyObject *scope_id;
};


/* "_dns.pxd":52
 * 
 * 
 * cdef class DNSHinfo(DNSRecord):             # <<<<<<<<<<<<<<
 * 
 *     cdef public cython.int _hash
 */
struct __pyx_obj_8zeroconf_4_dns_DNSHinfo {
  struct __pyx_obj_8zeroconf_4_dns_DNSRecord __pyx_base;
  int _hash;
  PyObject *cpu;
  PyObject *os;
};


/* "_dns.pxd":61
 * 
 * 
 * cdef class DNSPointer(DNSRecord):             # <<<<<<<<<<<<<<
 * 
 *     cdef public cython.int _hash
 */
struct __pyx_obj_8zeroconf_4_dns_DNSPointer {
  struct __pyx_obj_8zeroconf_4_dns_DNSRecord __pyx_base;
  int _hash;
  PyObject *alias;
  PyObject *alias_key;
};


/* "_dns.pxd":70
 * 
 * 
 * cdef class DNSText(DNSRecord):             # <<<<<<<<<<<<<<
 * 
 *     cdef public cython.int _hash
 */
struct __pyx_obj_8zeroconf_4_dns_DNSText {
  struct __pyx_obj_8zeroconf_4_dns_DNSRecord __pyx_base;
  int _hash;
  PyObject *text;
};


/* "_dns.pxd":78
 * 
 * 
 * cdef class DNSService(DNSRecord):             # <<<<<<<<<<<<<<
 * 
 *     cdef public cython.int _hash
 */
struct __pyx_obj_8zeroconf_4_dns_DNSService {
  struct __pyx_obj_8zeroconf_4_dns_DNSRecord __pyx_base;
  int _hash;
  PyObject *priority;
  PyObject *weight;
  PyObject *port;
  PyObject *server;
  PyObject *server_key;
};


/* "_dns.pxd":90
 * 
 * 
 * cdef class DNSNsec(DNSRecord):             # <<<<<<<<<<<<<<
 * 
 *     cdef public cython.int _hash
 */
struct __pyx_obj_8zeroconf_4_dns_DNSNsec {
  struct __pyx_obj_8zeroconf_4_dns_DNSRecord __pyx_base;
  int _hash;
  PyObject *next_name;
  PyObject *rdtypes;
};


/* "_dns.pxd":99
 * 
 * 
 * cdef class DNSRRSet:             # <<<<<<<<<<<<<<
 * 
 *     cdef _record_sets
 */
struct __pyx_obj_8zeroconf_4_dns_DNSRRSet {
  PyObject_HEAD
  struct __pyx_vtabstruct_8zeroconf_4_dns_DNSRRSet *__pyx_vtab;
  PyObject *_record_sets;
  PyObject *_lookup;
};


/* "zeroconf/_protocol/incoming.pxd":48
 * 
 * 
 * cdef class DNSIncoming:             # <<<<<<<<<<<<<<
 * 
 *     cdef bint _did_read_others
 */
struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming {
  PyObject_HEAD
  struct __pyx_vtabstruct_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_vtab;
  int _did_read_others;
  unsigned int flags;
  unsigned int offset;
  PyObject *data;
  unsigned int _data_len;
  PyObject *name_cache;
  PyObject *questions;
  PyObject *_answers;
  PyObject *id;
  unsigned int num_questions;
  unsigned int num_answers;
  unsigned int num_authorities;
  unsigned int num_additionals;
  PyObject *valid;
  PyObject *now;
  PyObject *scope_id;
  PyObject *source;
};



/* "_dns.pxd":21
 * cdef object current_time_millis
 * 
 * cdef class DNSEntry:             # <<<<<<<<<<<<<<
 * 
 *     cdef public object key
 */

struct __pyx_vtabstruct_8zeroconf_4_dns_DNSEntry {
  PyObject *(*_dns_entry_matches)(struct __pyx_obj_8zeroconf_4_dns_DNSEntry *, struct __pyx_obj_8zeroconf_4_dns_DNSEntry *);
};
static struct __pyx_vtabstruct_8zeroconf_4_dns_DNSEntry *__pyx_vtabptr_8zeroconf_4_dns_DNSEntry;


/* "_dns.pxd":31
 *     cdef _dns_entry_matches(self, DNSEntry other)
 * 
 * cdef class DNSQuestion(DNSEntry):             # <<<<<<<<<<<<<<
 * 
 *     cdef public cython.int _hash
 */

struct __pyx_vtabstruct_8zeroconf_4_dns_DNSQuestion {
  struct __pyx_vtabstruct_8zeroconf_4_dns_DNSEntry __pyx_base;
};
static struct __pyx_vtabstruct_8zeroconf_4_dns_DNSQuestion *__pyx_vtabptr_8zeroconf_4_dns_DNSQuestion;


/* "_dns.pxd":35
 *     cdef public cython.int _hash
 * 
 * cdef class DNSRecord(DNSEntry):             # <<<<<<<<<<<<<<
 * 
 *     cdef public object ttl
 */

struct __pyx_vtabstruct_8zeroconf_4_dns_DNSRecord {
  struct __pyx_vtabstruct_8zeroconf_4_dns_DNSEntry __pyx_base;
  PyObject *(*_suppressed_by_answer)(struct __pyx_obj_8zeroconf_4_dns_DNSRecord *, struct __pyx_obj_8zeroconf_4_dns_DNSRecord *);
};
static struct __pyx_vtabstruct_8zeroconf_4_dns_DNSRecord *__pyx_vtabptr_8zeroconf_4_dns_DNSRecord;


/* "_dns.pxd":43
 * 
 * 
 * cdef class DNSAddress(DNSRecord):             # <<<<<<<<<<<<<<
 * 
 *     cdef public cython.int _hash
 */

struct __pyx_vtabstruct_8zeroconf_4_dns_DNSAddress {
  struct __pyx_vtabstruct_8zeroconf_4_dns_DNSRecord __pyx_base;
  PyObject *(*_eq)(struct __pyx_obj_8zeroconf_4_dns_DNSAddress *, struct __pyx_obj_8zeroconf_4_dns_DNSAddress *);
};
static struct __pyx_vtabstruct_8zeroconf_4_dns_DNSAddress *__pyx_vtabptr_8zeroconf_4_dns_DNSAddress;


/* "_dns.pxd":52
 * 
 * 
 * cdef class DNSHinfo(DNSRecord):             # <<<<<<<<<<<<<<
 * 
 *     cdef public cython.int _hash
 */

struct __pyx_vtabstruct_8zeroconf_4_dns_DNSHinfo {
  struct __pyx_vtabstruct_8zeroconf_4_dns_DNSRecord __pyx_base;
  PyObject *(*_eq)(struct __pyx_obj_8zeroconf_4_dns_DNSHinfo *, struct __pyx_obj_8zeroconf_4_dns_DNSHinfo *);
};
static struct __pyx_vtabstruct_8zeroconf_4_dns_DNSHinfo *__pyx_vtabptr_8zeroconf_4_dns_DNSHinfo;


/* "_dns.pxd":61
 * 
 * 
 * cdef class DNSPointer(DNSRecord):             # <<<<<<<<<<<<<<
 * 
 *     cdef public cython.int _hash
 */

struct __pyx_vtabstruct_8zeroconf_4_dns_DNSPointer {
  struct __pyx_vtabstruct_8zeroconf_4_dns_DNSRecord __pyx_base;
  PyObject *(*_eq)(struct __pyx_obj_8zeroconf_4_dns_DNSPointer *, struct __pyx_obj_8zeroconf_4_dns_DNSPointer *);
};
static struct __pyx_vtabstruct_8zeroconf_4_dns_DNSPointer *__pyx_vtabptr_8zeroconf_4_dns_DNSPointer;


/* "_dns.pxd":70
 * 
 * 
 * cdef class DNSText(DNSRecord):             # <<<<<<<<<<<<<<
 * 
 *     cdef public cython.int _hash
 */

struct __pyx_vtabstruct_8zeroconf_4_dns_DNSText {
  struct __pyx_vtabstruct_8zeroconf_4_dns_DNSRecord __pyx_base;
  PyObject *(*_eq)(struct __pyx_obj_8zeroconf_4_dns_DNSText *, struct __pyx_obj_8zeroconf_4_dns_DNSText *);
};
static struct __pyx_vtabstruct_8zeroconf_4_dns_DNSText *__pyx_vtabptr_8zeroconf_4_dns_DNSText;


/* "_dns.pxd":78
 * 
 * 
 * cdef class DNSService(DNSRecord):             # <<<<<<<<<<<<<<
 * 
 *     cdef public cython.int _hash
 */

struct __pyx_vtabstruct_8zeroconf_4_dns_DNSService {
  struct __pyx_vtabstruct_8zeroconf_4_dns_DNSRecord __pyx_base;
  PyObject *(*_eq)(struct __pyx_obj_8zeroconf_4_dns_DNSService *, struct __pyx_obj_8zeroconf_4_dns_DNSService *);
};
static struct __pyx_vtabstruct_8zeroconf_4_dns_DNSService *__pyx_vtabptr_8zeroconf_4_dns_DNSService;


/* "_dns.pxd":90
 * 
 * 
 * cdef class DNSNsec(DNSRecord):             # <<<<<<<<<<<<<<
 * 
 *     cdef public cython.int _hash
 */

struct __pyx_vtabstruct_8zeroconf_4_dns_DNSNsec {
  struct __pyx_vtabstruct_8zeroconf_4_dns_DNSRecord __pyx_base;
  PyObject *(*_eq)(struct __pyx_obj_8zeroconf_4_dns_DNSNsec *, struct __pyx_obj_8zeroconf_4_dns_DNSNsec *);
};
static struct __pyx_vtabstruct_8zeroconf_4_dns_DNSNsec *__pyx_vtabptr_8zeroconf_4_dns_DNSNsec;


/* "_dns.pxd":99
 * 
 * 
 * cdef class DNSRRSet:             # <<<<<<<<<<<<<<
 * 
 *     cdef _record_sets
 */

struct __pyx_vtabstruct_8zeroconf_4_dns_DNSRRSet {
  PyObject *(*suppresses)(struct __pyx_obj_8zeroconf_4_dns_DNSRRSet *, struct __pyx_obj_8zeroconf_4_dns_DNSRecord *, int __pyx_skip_dispatch);
  PyObject *(*_get_lookup)(struct __pyx_obj_8zeroconf_4_dns_DNSRRSet *);
};
static struct __pyx_vtabstruct_8zeroconf_4_dns_DNSRRSet *__pyx_vtabptr_8zeroconf_4_dns_DNSRRSet;


/* "zeroconf/_protocol/incoming.py":72
 * 
 * 
 * class DNSIncoming:             # <<<<<<<<<<<<<<
 *     """Object representation of an incoming DNS packet"""
 * 
 */

struct __pyx_vtabstruct_8zeroconf_9_protocol_8incoming_DNSIncoming {
  PyObject *(*_decode_labels_at_offset)(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *, unsigned int, PyObject *, PyObject *);
  PyObject *(*_read_header)(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *);
  PyObject *(*_initial_parse)(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *);
  PyObject *(*_read_others)(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *);
  PyObject *(*_read_questions)(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *);
  PyObject *(*_read_character_string)(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *);
  PyObject *(*_read_string)(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *, unsigned int);
  PyObject *(*_read_record)(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *, PyObject *, unsigned int, PyObject *, PyObject *, unsigned int);
  PyObject *(*_read_bitmap)(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *, unsigned int);
  PyObject *(*_read_name)(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *);
};
static struct __pyx_vtabstruct_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_vtabptr_8zeroconf_9_protocol_8incoming_DNSIncoming;

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely((Py_TYPE(obj) == type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#define __Pyx_BUILD_ASSERT_EXPR(cond)\
    (sizeof(char [1 - 2*!(cond)]) - 1)
#ifndef Py_MEMBER_SIZE
#define Py_MEMBER_SIZE(type, member) sizeof(((type *)0)->member)
#endif
#if CYTHON_FAST_PYCALL
  static size_t __pyx_pyframe_localsplus_offset = 0;
  #include "frameobject.h"
#if PY_VERSION_HEX >= 0x030b00a6
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
  #define __Pxy_PyFrame_Initialize_Offsets()\
    ((void)__Pyx_BUILD_ASSERT_EXPR(sizeof(PyFrameObject) == offsetof(PyFrameObject, f_localsplus) + Py_MEMBER_SIZE(PyFrameObject, f_localsplus)),\
     (void)(__pyx_pyframe_localsplus_offset = ((size_t)PyFrame_Type.tp_basicsize) - Py_MEMBER_SIZE(PyFrameObject, f_localsplus)))
  #define __Pyx_PyFrame_GetLocalsplus(frame)\
    (assert(__pyx_pyframe_localsplus_offset), (PyObject **)(((char *)(frame)) + __pyx_pyframe_localsplus_offset))
#endif // CYTHON_FAST_PYCALL
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* GetTopmostException.proto */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem * __Pyx_PyErr_GetTopmostException(PyThreadState *tstate);
#endif

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* KeywordStringCheck.proto */
static int __Pyx_CheckKeywordStrings(PyObject *kwdict, const char* function_name, int kw_allowed);

/* PyDictVersioning.proto */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __pyx_dict_cached_value;\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  do {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
} while(0)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  do {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
} while(0)
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* PySequenceContains.proto */
static CYTHON_INLINE int __Pyx_PySequence_ContainsTF(PyObject* item, PyObject* seq, int eq) {
    int result = PySequence_Contains(seq, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* PyObjectCall2Args.proto */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2);

/* RaiseTooManyValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* RaiseNeedMoreValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* IterFinish.proto */
static CYTHON_INLINE int __Pyx_IterFinish(void);

/* UnpackItemEndCheck.proto */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected);

/* ListAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

/* decode_c_string_utf16.proto */
static CYTHON_INLINE PyObject *__Pyx_PyUnicode_DecodeUTF16(const char *s, Py_ssize_t size, const char *errors) {
    int byteorder = 0;
    return PyUnicode_DecodeUTF16(s, size, errors, &byteorder);
}
static CYTHON_INLINE PyObject *__Pyx_PyUnicode_DecodeUTF16LE(const char *s, Py_ssize_t size, const char *errors) {
    int byteorder = -1;
    return PyUnicode_DecodeUTF16(s, size, errors, &byteorder);
}
static CYTHON_INLINE PyObject *__Pyx_PyUnicode_DecodeUTF16BE(const char *s, Py_ssize_t size, const char *errors) {
    int byteorder = 1;
    return PyUnicode_DecodeUTF16(s, size, errors, &byteorder);
}

/* decode_c_bytes.proto */
static CYTHON_INLINE PyObject* __Pyx_decode_c_bytes(
         const char* cstring, Py_ssize_t length, Py_ssize_t start, Py_ssize_t stop,
         const char* encoding, const char* errors,
         PyObject* (*decode_func)(const char *s, Py_ssize_t size, const char *errors));

/* decode_bytes.proto */
static CYTHON_INLINE PyObject* __Pyx_decode_bytes(
         PyObject* string, Py_ssize_t start, Py_ssize_t stop,
         const char* encoding, const char* errors,
         PyObject* (*decode_func)(const char *s, Py_ssize_t size, const char *errors)) {
    return __Pyx_decode_c_bytes(
        PyBytes_AS_STRING(string), PyBytes_GET_SIZE(string),
        start, stop, encoding, errors, decode_func);
}

/* GCCDiagnostics.proto */
#if defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
#define __Pyx_HAS_GCC_DIAGNOSTIC
#endif

/* BuildPyUnicode.proto */
static PyObject* __Pyx_PyUnicode_BuildFromAscii(Py_ssize_t ulength, char* chars, int clength,
                                                int prepend_sign, char padding_char);

/* IncludeStringH.proto */
#include <string.h>

/* CIntToPyUnicode.proto */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_From_unsigned_int(unsigned int value, Py_ssize_t width, char padding_char, char format_char);

/* PyObjectFormatSimple.proto */
#if CYTHON_COMPILING_IN_PYPY
    #define __Pyx_PyObject_FormatSimple(s, f) (\
        likely(PyUnicode_CheckExact(s)) ? (Py_INCREF(s), s) :\
        PyObject_Format(s, f))
#elif PY_MAJOR_VERSION < 3
    #define __Pyx_PyObject_FormatSimple(s, f) (\
        likely(PyUnicode_CheckExact(s)) ? (Py_INCREF(s), s) :\
        likely(PyString_CheckExact(s)) ? PyUnicode_FromEncodedObject(s, NULL, "strict") :\
        PyObject_Format(s, f))
#elif CYTHON_USE_TYPE_SLOTS
    #define __Pyx_PyObject_FormatSimple(s, f) (\
        likely(PyUnicode_CheckExact(s)) ? (Py_INCREF(s), s) :\
        likely(PyLong_CheckExact(s)) ? PyLong_Type.tp_str(s) :\
        likely(PyFloat_CheckExact(s)) ? PyFloat_Type.tp_str(s) :\
        PyObject_Format(s, f))
#else
    #define __Pyx_PyObject_FormatSimple(s, f) (\
        likely(PyUnicode_CheckExact(s)) ? (Py_INCREF(s), s) :\
        PyObject_Format(s, f))
#endif

/* JoinPyUnicode.proto */
static PyObject* __Pyx_PyUnicode_Join(PyObject* value_tuple, Py_ssize_t value_count, Py_ssize_t result_ulength,
                                      Py_UCS4 max_char);

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* pyfrozenset_new.proto */
static CYTHON_INLINE PyObject* __Pyx_PyFrozenSet_New(PyObject* it);

/* PySetContains.proto */
static CYTHON_INLINE int __Pyx_PySet_ContainsTF(PyObject* key, PyObject* set, int eq);

/* dict_getitem_default.proto */
static PyObject* __Pyx_PyDict_GetItemDefault(PyObject* d, PyObject* key, PyObject* default_value);

/* UnpackUnboundCMethod.proto */
typedef struct {
    PyObject *type;
    PyObject **method_name;
    PyCFunction func;
    PyObject *method;
    int flag;
} __Pyx_CachedCFunction;

/* CallUnboundCMethod1.proto */
static PyObject* __Pyx__CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg);
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg);
#else
#define __Pyx_CallUnboundCMethod1(cfunc, self, arg)  __Pyx__CallUnboundCMethod1(cfunc, self, arg)
#endif

/* CallUnboundCMethod2.proto */
static PyObject* __Pyx__CallUnboundCMethod2(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg1, PyObject* arg2);
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030600B1
static CYTHON_INLINE PyObject *__Pyx_CallUnboundCMethod2(__Pyx_CachedCFunction *cfunc, PyObject *self, PyObject *arg1, PyObject *arg2);
#else
#define __Pyx_CallUnboundCMethod2(cfunc, self, arg1, arg2)  __Pyx__CallUnboundCMethod2(cfunc, self, arg1, arg2)
#endif

/* ListExtend.proto */
static CYTHON_INLINE int __Pyx_PyList_Extend(PyObject* L, PyObject* v) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject* none = _PyList_Extend((PyListObject*)L, v);
    if (unlikely(!none))
        return -1;
    Py_DECREF(none);
    return 0;
#else
    return PyList_SetSlice(L, PY_SSIZE_T_MAX, PY_SSIZE_T_MAX, v);
#endif
}

/* GetAttr.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *, PyObject *);

/* GetAttr3.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *, PyObject *, PyObject *);

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* HasAttr.proto */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *, PyObject *);

/* PyObject_GenericGetAttrNoDict.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttrNoDict PyObject_GenericGetAttr
#endif

/* PyObject_GenericGetAttr.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttr PyObject_GenericGetAttr
#endif

/* SetVTable.proto */
static int __Pyx_SetVtable(PyObject *dict, void *vtable);

/* PyObjectGetAttrStrNoError.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name);

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* TypeImport.proto */
#ifndef __PYX_HAVE_RT_ImportType_proto_0_29_35
#define __PYX_HAVE_RT_ImportType_proto_0_29_35
#if __STDC_VERSION__ >= 201112L
#include <stdalign.h>
#endif
#if __STDC_VERSION__ >= 201112L || __cplusplus >= 201103L
#define __PYX_GET_STRUCT_ALIGNMENT_0_29_35(s) alignof(s)
#else
#define __PYX_GET_STRUCT_ALIGNMENT_0_29_35(s) sizeof(void*)
#endif
enum __Pyx_ImportType_CheckSize_0_29_35 {
   __Pyx_ImportType_CheckSize_Error_0_29_35 = 0,
   __Pyx_ImportType_CheckSize_Warn_0_29_35 = 1,
   __Pyx_ImportType_CheckSize_Ignore_0_29_35 = 2
};
static PyTypeObject *__Pyx_ImportType_0_29_35(PyObject* module, const char *module_name, const char *class_name, size_t size, size_t alignment, enum __Pyx_ImportType_CheckSize_0_29_35 check_size);
#endif

/* GetVTable.proto */
static void* __Pyx_GetVtable(PyObject *dict);

/* FetchCommonType.proto */
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type);

/* CythonFunctionShared.proto */
#define __Pyx_CyFunction_USED 1
#define __Pyx_CYFUNCTION_STATICMETHOD  0x01
#define __Pyx_CYFUNCTION_CLASSMETHOD   0x02
#define __Pyx_CYFUNCTION_CCLASS        0x04
#define __Pyx_CyFunction_GetClosure(f)\
    (((__pyx_CyFunctionObject *) (f))->func_closure)
#define __Pyx_CyFunction_GetClassObj(f)\
    (((__pyx_CyFunctionObject *) (f))->func_classobj)
#define __Pyx_CyFunction_Defaults(type, f)\
    ((type *)(((__pyx_CyFunctionObject *) (f))->defaults))
#define __Pyx_CyFunction_SetDefaultsGetter(f, g)\
    ((__pyx_CyFunctionObject *) (f))->defaults_getter = (g)
typedef struct {
    PyCFunctionObject func;
#if PY_VERSION_HEX < 0x030500A0
    PyObject *func_weakreflist;
#endif
    PyObject *func_dict;
    PyObject *func_name;
    PyObject *func_qualname;
    PyObject *func_doc;
    PyObject *func_globals;
    PyObject *func_code;
    PyObject *func_closure;
    PyObject *func_classobj;
    void *defaults;
    int defaults_pyobjects;
    size_t defaults_size;  // used by FusedFunction for copying defaults
    int flags;
    PyObject *defaults_tuple;
    PyObject *defaults_kwdict;
    PyObject *(*defaults_getter)(PyObject *);
    PyObject *func_annotations;
} __pyx_CyFunctionObject;
static PyTypeObject *__pyx_CyFunctionType = 0;
#define __Pyx_CyFunction_Check(obj)  (__Pyx_TypeCheck(obj, __pyx_CyFunctionType))
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject* op, PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *self,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *m,
                                                         size_t size,
                                                         int pyobjects);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *m,
                                                            PyObject *tuple);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *m,
                                                             PyObject *dict);
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *m,
                                                              PyObject *dict);
static int __pyx_CyFunction_init(void);

/* CythonFunction.proto */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *closure,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);

/* ClassMethod.proto */
#include "descrobject.h"
static CYTHON_UNUSED PyObject* __Pyx_Method_ClassMethod(PyObject *method);

/* GetNameInClass.proto */
#define __Pyx_GetNameInClass(var, nmspace, name)  (var) = __Pyx__GetNameInClass(nmspace, name)
static PyObject *__Pyx__GetNameInClass(PyObject *nmspace, PyObject *name);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_unsigned_int(unsigned int value);

/* CIntFromPy.proto */
static CYTHON_INLINE unsigned int __Pyx_PyInt_As_unsigned_int(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* PyObjectSetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
#define __Pyx_PyObject_DelAttrStr(o,n) __Pyx_PyObject_SetAttrStr(o, n, NULL)
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value);
#else
#define __Pyx_PyObject_DelAttrStr(o,n)   PyObject_DelAttr(o,n)
#define __Pyx_PyObject_SetAttrStr(o,n,v) PyObject_SetAttr(o,n,v)
#endif

/* VoidPtrExport.proto */
static int __Pyx_ExportVoidPtr(PyObject *name, void *p, const char *sig);

/* VoidPtrImport.proto */
static int __Pyx_ImportVoidPtr_0_29_35(PyObject *module, const char *name, void **p, const char *sig);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

static PyObject *__pyx_f_8zeroconf_9_protocol_8incoming_11DNSIncoming__initial_parse(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_8zeroconf_9_protocol_8incoming_11DNSIncoming__read_header(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_8zeroconf_9_protocol_8incoming_11DNSIncoming__read_questions(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_8zeroconf_9_protocol_8incoming_11DNSIncoming__read_character_string(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_8zeroconf_9_protocol_8incoming_11DNSIncoming__read_string(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self, unsigned int __pyx_v_length); /* proto*/
static PyObject *__pyx_f_8zeroconf_9_protocol_8incoming_11DNSIncoming__read_others(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_8zeroconf_9_protocol_8incoming_11DNSIncoming__read_record(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self, PyObject *__pyx_v_domain, unsigned int __pyx_v_type_, PyObject *__pyx_v_class_, PyObject *__pyx_v_ttl, unsigned int __pyx_v_length); /* proto*/
static PyObject *__pyx_f_8zeroconf_9_protocol_8incoming_11DNSIncoming__read_bitmap(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self, unsigned int __pyx_v_end); /* proto*/
static PyObject *__pyx_f_8zeroconf_9_protocol_8incoming_11DNSIncoming__read_name(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_8zeroconf_9_protocol_8incoming_11DNSIncoming__decode_labels_at_offset(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self, unsigned int __pyx_v_off, PyObject *__pyx_v_labels, PyObject *__pyx_v_seen_pointers); /* proto*/

/* Module declarations from 'cython' */

/* Module declarations from 'zeroconf._dns' */
static PyTypeObject *__pyx_ptype_8zeroconf_4_dns_DNSEntry = 0;
static PyTypeObject *__pyx_ptype_8zeroconf_4_dns_DNSQuestion = 0;
static PyTypeObject *__pyx_ptype_8zeroconf_4_dns_DNSRecord = 0;
static PyTypeObject *__pyx_ptype_8zeroconf_4_dns_DNSAddress = 0;
static PyTypeObject *__pyx_ptype_8zeroconf_4_dns_DNSHinfo = 0;
static PyTypeObject *__pyx_ptype_8zeroconf_4_dns_DNSPointer = 0;
static PyTypeObject *__pyx_ptype_8zeroconf_4_dns_DNSText = 0;
static PyTypeObject *__pyx_ptype_8zeroconf_4_dns_DNSService = 0;
static PyTypeObject *__pyx_ptype_8zeroconf_4_dns_DNSNsec = 0;
static PyTypeObject *__pyx_ptype_8zeroconf_4_dns_DNSRRSet = 0;
static PyObject **__pyx_vp_8zeroconf_4_dns__LEN_BYTE = 0;
#define __pyx_v_8zeroconf_4_dns__LEN_BYTE (*__pyx_vp_8zeroconf_4_dns__LEN_BYTE)
static PyObject **__pyx_vp_8zeroconf_4_dns__LEN_SHORT = 0;
#define __pyx_v_8zeroconf_4_dns__LEN_SHORT (*__pyx_vp_8zeroconf_4_dns__LEN_SHORT)
static PyObject **__pyx_vp_8zeroconf_4_dns__LEN_INT = 0;
#define __pyx_v_8zeroconf_4_dns__LEN_INT (*__pyx_vp_8zeroconf_4_dns__LEN_INT)
static PyObject **__pyx_vp_8zeroconf_4_dns__NAME_COMPRESSION_MIN_SIZE = 0;
#define __pyx_v_8zeroconf_4_dns__NAME_COMPRESSION_MIN_SIZE (*__pyx_vp_8zeroconf_4_dns__NAME_COMPRESSION_MIN_SIZE)
static PyObject **__pyx_vp_8zeroconf_4_dns__BASE_MAX_SIZE = 0;
#define __pyx_v_8zeroconf_4_dns__BASE_MAX_SIZE (*__pyx_vp_8zeroconf_4_dns__BASE_MAX_SIZE)
static PyObject **__pyx_vp_8zeroconf_4_dns__EXPIRE_FULL_TIME_MS = 0;
#define __pyx_v_8zeroconf_4_dns__EXPIRE_FULL_TIME_MS (*__pyx_vp_8zeroconf_4_dns__EXPIRE_FULL_TIME_MS)
static PyObject **__pyx_vp_8zeroconf_4_dns__EXPIRE_STALE_TIME_MS = 0;
#define __pyx_v_8zeroconf_4_dns__EXPIRE_STALE_TIME_MS (*__pyx_vp_8zeroconf_4_dns__EXPIRE_STALE_TIME_MS)
static PyObject **__pyx_vp_8zeroconf_4_dns__RECENT_TIME_MS = 0;
#define __pyx_v_8zeroconf_4_dns__RECENT_TIME_MS (*__pyx_vp_8zeroconf_4_dns__RECENT_TIME_MS)
static PyObject **__pyx_vp_8zeroconf_4_dns__CLASS_UNIQUE = 0;
#define __pyx_v_8zeroconf_4_dns__CLASS_UNIQUE (*__pyx_vp_8zeroconf_4_dns__CLASS_UNIQUE)
static PyObject **__pyx_vp_8zeroconf_4_dns__CLASS_MASK = 0;
#define __pyx_v_8zeroconf_4_dns__CLASS_MASK (*__pyx_vp_8zeroconf_4_dns__CLASS_MASK)
static PyObject **__pyx_vp_8zeroconf_4_dns_current_time_millis = 0;
#define __pyx_v_8zeroconf_4_dns_current_time_millis (*__pyx_vp_8zeroconf_4_dns_current_time_millis)

/* Module declarations from 'zeroconf._protocol.incoming' */
static PyTypeObject *__pyx_ptype_8zeroconf_9_protocol_8incoming_DNSIncoming = 0;
static unsigned int __pyx_v_8zeroconf_9_protocol_8incoming_DNS_COMPRESSION_HEADER_LEN;
static unsigned int __pyx_v_8zeroconf_9_protocol_8incoming_MAX_DNS_LABELS;
static unsigned int __pyx_v_8zeroconf_9_protocol_8incoming_DNS_COMPRESSION_POINTER_LEN;
static unsigned int __pyx_v_8zeroconf_9_protocol_8incoming_MAX_NAME_LENGTH;
static PyObject *__pyx_v_8zeroconf_9_protocol_8incoming_current_time_millis = 0;
static unsigned int __pyx_v_8zeroconf_9_protocol_8incoming__TYPE_A;
static unsigned int __pyx_v_8zeroconf_9_protocol_8incoming__TYPE_CNAME;
static unsigned int __pyx_v_8zeroconf_9_protocol_8incoming__TYPE_PTR;
static unsigned int __pyx_v_8zeroconf_9_protocol_8incoming__TYPE_TXT;
static unsigned int __pyx_v_8zeroconf_9_protocol_8incoming__TYPE_SRV;
static unsigned int __pyx_v_8zeroconf_9_protocol_8incoming__TYPE_HINFO;
static unsigned int __pyx_v_8zeroconf_9_protocol_8incoming__TYPE_AAAA;
static unsigned int __pyx_v_8zeroconf_9_protocol_8incoming__TYPE_NSEC;
static unsigned int __pyx_v_8zeroconf_9_protocol_8incoming__FLAGS_QR_MASK;
static unsigned int __pyx_v_8zeroconf_9_protocol_8incoming__FLAGS_TC;
static unsigned int __pyx_v_8zeroconf_9_protocol_8incoming__FLAGS_QR_QUERY;
static unsigned int __pyx_v_8zeroconf_9_protocol_8incoming__FLAGS_QR_RESPONSE;
static PyObject *__pyx_v_8zeroconf_9_protocol_8incoming_UNPACK_3H = 0;
static PyObject *__pyx_v_8zeroconf_9_protocol_8incoming_UNPACK_6H = 0;
static PyObject *__pyx_v_8zeroconf_9_protocol_8incoming_UNPACK_HH = 0;
static PyObject *__pyx_v_8zeroconf_9_protocol_8incoming_UNPACK_HHiH = 0;
static PyObject *__pyx_v_8zeroconf_9_protocol_8incoming_DECODE_EXCEPTIONS = 0;
static PyObject *__pyx_v_8zeroconf_9_protocol_8incoming_IncomingDecodeError = 0;
static PyObject *__pyx_f_8zeroconf_9_protocol_8incoming___pyx_unpickle_DNSIncoming__set_state(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *, PyObject *); /*proto*/
#define __Pyx_MODULE_NAME "zeroconf._protocol.incoming"
extern int __pyx_module_is_main_zeroconf___protocol__incoming;
int __pyx_module_is_main_zeroconf___protocol__incoming = 0;

/* Implementation of 'zeroconf._protocol.incoming' */
static PyObject *__pyx_builtin_IndexError;
static PyObject *__pyx_builtin_range;
static PyObject *__pyx_builtin_enumerate;
static const char __pyx_k_[] = ", ";
static const char __pyx_k_3H[] = "!3H";
static const char __pyx_k_6H[] = "!6H";
static const char __pyx_k_HH[] = "!HH";
static const char __pyx_k__2[] = ".";
static const char __pyx_k_id[] = "id";
static const char __pyx_k_Any[] = "Any";
static const char __pyx_k_Set[] = "Set";
static const char __pyx_k_cls[] = "cls";
static const char __pyx_k_dns[] = "_dns";
static const char __pyx_k_get[] = "get";
static const char __pyx_k_log[] = "log";
static const char __pyx_k_new[] = "__new__";
static const char __pyx_k_now[] = "now";
static const char __pyx_k_str[] = "_str";
static const char __pyx_k_sys[] = "sys";
static const char __pyx_k_Dict[] = "Dict";
static const char __pyx_k_HHiH[] = "!HHiH";
static const char __pyx_k_List[] = "List";
static const char __pyx_k_data[] = "data";
static const char __pyx_k_dict[] = "__dict__";
static const char __pyx_k_from[] = " from ";
static const char __pyx_k_id_s[] = "id=%s";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_name[] = "__name__";
static const char __pyx_k_self[] = "self";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_TYPES[] = "_TYPES";
static const char __pyx_k_Tuple[] = "Tuple";
static const char __pyx_k_Union[] = "Union";
static const char __pyx_k_const[] = "const";
static const char __pyx_k_debug[] = "debug";
static const char __pyx_k_error[] = "error";
static const char __pyx_k_flags[] = "flags";
static const char __pyx_k_n_q_s[] = "n_q=%s";
static const char __pyx_k_range[] = "range";
static const char __pyx_k_slots[] = "__slots__";
static const char __pyx_k_state[] = "state";
static const char __pyx_k_valid[] = "valid";
static const char __pyx_k_Struct[] = "Struct";
static const char __pyx_k_TYPE_A[] = "_TYPE_A";
static const char __pyx_k_dict_2[] = "_dict";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_logger[] = "_logger";
static const char __pyx_k_offset[] = "offset";
static const char __pyx_k_pickle[] = "pickle";
static const char __pyx_k_reduce[] = "__reduce__";
static const char __pyx_k_return[] = "return";
static const char __pyx_k_source[] = "source";
static const char __pyx_k_struct[] = "struct";
static const char __pyx_k_typing[] = "typing";
static const char __pyx_k_update[] = "update";
static const char __pyx_k_DNSNsec[] = "DNSNsec";
static const char __pyx_k_DNSText[] = "DNSText";
static const char __pyx_k_answers[] = "answers";
static const char __pyx_k_exc_str[] = "exc_str";
static const char __pyx_k_flags_s[] = "flags=%s";
static const char __pyx_k_n_add_s[] = "n_add=%s";
static const char __pyx_k_n_ans_s[] = "n_ans=%s";
static const char __pyx_k_DNSHinfo[] = "DNSHinfo";
static const char __pyx_k_DNS_name[] = "DNS name ";
static const char __pyx_k_FLAGS_TC[] = "_FLAGS_TC";
static const char __pyx_k_Optional[] = "Optional";
static const char __pyx_k_TYPE_PTR[] = "_TYPE_PTR";
static const char __pyx_k_TYPE_SRV[] = "_TYPE_SRV";
static const char __pyx_k_TYPE_TXT[] = "_TYPE_TXT";
static const char __pyx_k_data_len[] = "_data_len";
static const char __pyx_k_exc_info[] = "exc_info";
static const char __pyx_k_getstate[] = "__getstate__";
static const char __pyx_k_is_query[] = "is_query";
static const char __pyx_k_n_auth_s[] = "n_auth=%s";
static const char __pyx_k_pyx_capi[] = "__pyx_capi__";
static const char __pyx_k_pyx_type[] = "__pyx_type";
static const char __pyx_k_scope_id[] = "scope_id";
static const char __pyx_k_setstate[] = "__setstate__";
static const char __pyx_k_DNSRecord[] = "DNSRecord";
static const char __pyx_k_TYPE_AAAA[] = "_TYPE_AAAA";
static const char __pyx_k_TYPE_NSEC[] = "_TYPE_NSEC";
static const char __pyx_k_UNPACK_3H[] = "UNPACK_3H";
static const char __pyx_k_UNPACK_6H[] = "UNPACK_6H";
static const char __pyx_k_UNPACK_HH[] = "UNPACK_HH";
static const char __pyx_k_answers_2[] = "_answers";
static const char __pyx_k_answers_s[] = "answers=%s";
static const char __pyx_k_enumerate[] = "enumerate";
static const char __pyx_k_points_to[] = " points to ";
static const char __pyx_k_pyx_state[] = "__pyx_state";
static const char __pyx_k_questions[] = "questions";
static const char __pyx_k_reduce_ex[] = "__reduce_ex__";
static const char __pyx_k_seen_logs[] = "_seen_logs";
static const char __pyx_k_truncated[] = "truncated";
static const char __pyx_k_DNSAddress[] = "DNSAddress";
static const char __pyx_k_DNSPointer[] = "DNSPointer";
static const char __pyx_k_DNSService[] = "DNSService";
static const char __pyx_k_IndexError[] = "IndexError";
static const char __pyx_k_TYPE_CNAME[] = "_TYPE_CNAME";
static const char __pyx_k_TYPE_HINFO[] = "_TYPE_HINFO";
static const char __pyx_k_exceptions[] = "_exceptions";
static const char __pyx_k_name_cache[] = "name_cache";
static const char __pyx_k_pyx_result[] = "__pyx_result";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_utils_time[] = "_utils.time";
static const char __pyx_k_DNSIncoming[] = "DNSIncoming";
static const char __pyx_k_DNSQuestion[] = "DNSQuestion";
static const char __pyx_k_PickleError[] = "PickleError";
static const char __pyx_k_UNPACK_HHiH[] = "UNPACK_HHiH";
static const char __pyx_k_is_response[] = "is_response";
static const char __pyx_k_logger_data[] = "logger_data";
static const char __pyx_k_num_answers[] = "num_answers";
static const char __pyx_k_questions_s[] = "questions=%s";
static const char __pyx_k_truncated_s[] = "truncated=%s";
static const char __pyx_k_unpack_from[] = "unpack_from";
static const char __pyx_k_log_exc_info[] = "log_exc_info";
static const char __pyx_k_pyx_checksum[] = "__pyx_checksum";
static const char __pyx_k_stringsource[] = "stringsource";
static const char __pyx_k_use_setstate[] = "use_setstate";
static const char __pyx_k_DNSIncoming_s[] = "<DNSIncoming:{%s}>";
static const char __pyx_k_FLAGS_QR_MASK[] = "_FLAGS_QR_MASK";
static const char __pyx_k_is_unknown_at[] = " is unknown at ";
static const char __pyx_k_num_questions[] = "num_questions";
static const char __pyx_k_reduce_cython[] = "__reduce_cython__";
static const char __pyx_k_FLAGS_QR_QUERY[] = "_FLAGS_QR_QUERY";
static const char __pyx_k_MAX_DNS_LABELS[] = "MAX_DNS_LABELS";
static const char __pyx_k_MAX_NAME_LENGTH[] = "MAX_NAME_LENGTH";
static const char __pyx_k_did_read_others[] = "_did_read_others";
static const char __pyx_k_num_additionals[] = "num_additionals";
static const char __pyx_k_num_authorities[] = "num_authorities";
static const char __pyx_k_pyx_PickleError[] = "__pyx_PickleError";
static const char __pyx_k_setstate_cython[] = "__setstate_cython__";
static const char __pyx_k_DECODE_EXCEPTIONS[] = "DECODE_EXCEPTIONS";
static const char __pyx_k_FLAGS_QR_RESPONSE[] = "_FLAGS_QR_RESPONSE";
static const char __pyx_k_Exception_occurred[] = "Exception occurred";
static const char __pyx_k_beyond_packet_from[] = " beyond packet from ";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_IncomingDecodeError[] = "IncomingDecodeError";
static const char __pyx_k_current_time_millis[] = "current_time_millis";
static const char __pyx_k_log_exception_debug[] = "_log_exception_debug";
static const char __pyx_k_was_seen_again_from[] = " was seen again from ";
static const char __pyx_k_DNSIncoming_is_query[] = "DNSIncoming.is_query";
static const char __pyx_k_DNS_compression_type[] = "DNS compression type ";
static const char __pyx_k_points_to_itself_from[] = " points to itself from ";
static const char __pyx_k_DNSIncoming_is_response[] = "DNSIncoming.is_response";
static const char __pyx_k_pyx_unpickle_DNSIncoming[] = "__pyx_unpickle_DNSIncoming";
static const char __pyx_k_exceeds_maximum_length_of[] = " exceeds maximum length of ";
static const char __pyx_k_DNS_COMPRESSION_HEADER_LEN[] = "DNS_COMPRESSION_HEADER_LEN";
static const char __pyx_k_DNS_compression_pointer_at[] = "DNS compression pointer at ";
static const char __pyx_k_DNSIncoming___reduce_cython[] = "DNSIncoming.__reduce_cython__";
static const char __pyx_k_DNS_COMPRESSION_POINTER_LEN[] = "DNS_COMPRESSION_POINTER_LEN";
static const char __pyx_k_zeroconf__protocol_incoming[] = "zeroconf._protocol.incoming";
static const char __pyx_k_DNSIncoming___setstate_cython[] = "DNSIncoming.__setstate_cython__";
static const char __pyx_k_Multicast_DNS_Service_Discovery[] = " Multicast DNS Service Discovery for Python, v0.14-wmcbrine\n    Copyright 2003 Paul Scott-Murphy, 2014 William McBrine\n\n    This module provides a framework for the use of DNS Service Discovery\n    using IP multicast.\n\n    This library is free software; you can redistribute it and/or\n    modify it under the terms of the GNU Lesser General Public\n    License as published by the Free Software Foundation; either\n    version 2.1 of the License, or (at your option) any later version.\n\n    This library is distributed in the hope that it will be useful,\n    but WITHOUT ANY WARRANTY; without even the implied warranty of\n    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU\n    Lesser General Public License for more details.\n\n    You should have received a copy of the GNU Lesser General Public\n    License along with this library; if not, write to the Free Software\n    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301\n    USA\n";
static const char __pyx_k_Unable_to_parse_skipping_record[] = "Unable to parse; skipping record for %s with type %s at offset %d while unpacking %r";
static const char __pyx_k_src_zeroconf__protocol_incoming[] = "src/zeroconf/_protocol/incoming.py";
static const char __pyx_k_Corrupt_packet_received_while_de[] = "Corrupt packet received while decoding name from ";
static const char __pyx_k_DNSIncoming__log_exception_debug[] = "DNSIncoming._log_exception_debug";
static const char __pyx_k_Incompatible_checksums_0x_x_vs_0[] = "Incompatible checksums (0x%x vs (0x69f643b, 0xcd74736, 0xd8a590c) = (_answers, _data_len, _did_read_others, data, flags, id, name_cache, now, num_additionals, num_answers, num_authorities, num_questions, offset, questions, scope_id, source, valid))";
static const char __pyx_k_Maximum_dns_labels_reached_while[] = "Maximum dns labels reached while processing pointer at ";
static const char __pyx_k_Received_invalid_packet_from_s_a[] = "Received invalid packet from %s at offset %d while unpacking %r";
static PyObject *__pyx_kp_u_;
static PyObject *__pyx_kp_b_3H;
static PyObject *__pyx_kp_b_6H;
static PyObject *__pyx_n_s_Any;
static PyObject *__pyx_kp_u_Corrupt_packet_received_while_de;
static PyObject *__pyx_n_s_DECODE_EXCEPTIONS;
static PyObject *__pyx_n_s_DNSAddress;
static PyObject *__pyx_n_s_DNSHinfo;
static PyObject *__pyx_n_s_DNSIncoming;
static PyObject *__pyx_n_s_DNSIncoming___reduce_cython;
static PyObject *__pyx_n_s_DNSIncoming___setstate_cython;
static PyObject *__pyx_n_s_DNSIncoming__log_exception_debug;
static PyObject *__pyx_n_s_DNSIncoming_is_query;
static PyObject *__pyx_n_s_DNSIncoming_is_response;
static PyObject *__pyx_kp_u_DNSIncoming_s;
static PyObject *__pyx_n_s_DNSNsec;
static PyObject *__pyx_n_s_DNSPointer;
static PyObject *__pyx_n_s_DNSQuestion;
static PyObject *__pyx_n_s_DNSRecord;
static PyObject *__pyx_n_s_DNSService;
static PyObject *__pyx_n_s_DNSText;
static PyObject *__pyx_n_s_DNS_COMPRESSION_HEADER_LEN;
static PyObject *__pyx_n_s_DNS_COMPRESSION_POINTER_LEN;
static PyObject *__pyx_kp_u_DNS_compression_pointer_at;
static PyObject *__pyx_kp_u_DNS_compression_type;
static PyObject *__pyx_kp_u_DNS_name;
static PyObject *__pyx_n_s_Dict;
static PyObject *__pyx_kp_u_Exception_occurred;
static PyObject *__pyx_n_s_FLAGS_QR_MASK;
static PyObject *__pyx_n_s_FLAGS_QR_QUERY;
static PyObject *__pyx_n_s_FLAGS_QR_RESPONSE;
static PyObject *__pyx_n_s_FLAGS_TC;
static PyObject *__pyx_kp_b_HH;
static PyObject *__pyx_kp_b_HHiH;
static PyObject *__pyx_n_s_IncomingDecodeError;
static PyObject *__pyx_kp_s_Incompatible_checksums_0x_x_vs_0;
static PyObject *__pyx_n_s_IndexError;
static PyObject *__pyx_n_s_List;
static PyObject *__pyx_n_s_MAX_DNS_LABELS;
static PyObject *__pyx_n_s_MAX_NAME_LENGTH;
static PyObject *__pyx_kp_u_Maximum_dns_labels_reached_while;
static PyObject *__pyx_n_s_Optional;
static PyObject *__pyx_n_s_PickleError;
static PyObject *__pyx_kp_u_Received_invalid_packet_from_s_a;
static PyObject *__pyx_n_s_Set;
static PyObject *__pyx_n_s_Struct;
static PyObject *__pyx_n_s_TYPES;
static PyObject *__pyx_n_s_TYPE_A;
static PyObject *__pyx_n_s_TYPE_AAAA;
static PyObject *__pyx_n_s_TYPE_CNAME;
static PyObject *__pyx_n_s_TYPE_HINFO;
static PyObject *__pyx_n_s_TYPE_NSEC;
static PyObject *__pyx_n_s_TYPE_PTR;
static PyObject *__pyx_n_s_TYPE_SRV;
static PyObject *__pyx_n_s_TYPE_TXT;
static PyObject *__pyx_n_s_Tuple;
static PyObject *__pyx_n_s_UNPACK_3H;
static PyObject *__pyx_n_s_UNPACK_6H;
static PyObject *__pyx_n_s_UNPACK_HH;
static PyObject *__pyx_n_s_UNPACK_HHiH;
static PyObject *__pyx_kp_u_Unable_to_parse_skipping_record;
static PyObject *__pyx_n_s_Union;
static PyObject *__pyx_kp_u__2;
static PyObject *__pyx_n_s_answers;
static PyObject *__pyx_n_u_answers_2;
static PyObject *__pyx_kp_u_answers_s;
static PyObject *__pyx_kp_u_beyond_packet_from;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_cls;
static PyObject *__pyx_n_s_const;
static PyObject *__pyx_n_s_current_time_millis;
static PyObject *__pyx_n_s_data;
static PyObject *__pyx_n_u_data;
static PyObject *__pyx_n_u_data_len;
static PyObject *__pyx_n_s_debug;
static PyObject *__pyx_n_s_dict;
static PyObject *__pyx_n_s_dict_2;
static PyObject *__pyx_n_u_did_read_others;
static PyObject *__pyx_n_s_dns;
static PyObject *__pyx_n_s_enumerate;
static PyObject *__pyx_n_s_error;
static PyObject *__pyx_n_s_exc_info;
static PyObject *__pyx_n_s_exc_str;
static PyObject *__pyx_kp_u_exceeds_maximum_length_of;
static PyObject *__pyx_n_s_exceptions;
static PyObject *__pyx_n_u_flags;
static PyObject *__pyx_kp_u_flags_s;
static PyObject *__pyx_kp_u_from;
static PyObject *__pyx_n_s_get;
static PyObject *__pyx_n_s_getstate;
static PyObject *__pyx_n_u_id;
static PyObject *__pyx_kp_u_id_s;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_is_query;
static PyObject *__pyx_n_s_is_response;
static PyObject *__pyx_kp_u_is_unknown_at;
static PyObject *__pyx_n_s_log;
static PyObject *__pyx_n_s_log_exc_info;
static PyObject *__pyx_n_s_log_exception_debug;
static PyObject *__pyx_n_s_logger;
static PyObject *__pyx_n_s_logger_data;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_kp_u_n_add_s;
static PyObject *__pyx_kp_u_n_ans_s;
static PyObject *__pyx_kp_u_n_auth_s;
static PyObject *__pyx_kp_u_n_q_s;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_u_name_cache;
static PyObject *__pyx_n_s_new;
static PyObject *__pyx_n_s_now;
static PyObject *__pyx_n_u_now;
static PyObject *__pyx_n_u_num_additionals;
static PyObject *__pyx_n_u_num_answers;
static PyObject *__pyx_n_u_num_authorities;
static PyObject *__pyx_n_u_num_questions;
static PyObject *__pyx_n_u_offset;
static PyObject *__pyx_n_s_pickle;
static PyObject *__pyx_kp_u_points_to;
static PyObject *__pyx_kp_u_points_to_itself_from;
static PyObject *__pyx_n_s_pyx_PickleError;
static PyObject *__pyx_n_s_pyx_capi;
static PyObject *__pyx_n_s_pyx_checksum;
static PyObject *__pyx_n_s_pyx_result;
static PyObject *__pyx_n_s_pyx_state;
static PyObject *__pyx_n_s_pyx_type;
static PyObject *__pyx_n_s_pyx_unpickle_DNSIncoming;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_u_questions;
static PyObject *__pyx_kp_u_questions_s;
static PyObject *__pyx_n_s_range;
static PyObject *__pyx_n_s_reduce;
static PyObject *__pyx_n_s_reduce_cython;
static PyObject *__pyx_n_s_reduce_ex;
static PyObject *__pyx_n_s_return;
static PyObject *__pyx_n_s_scope_id;
static PyObject *__pyx_n_u_scope_id;
static PyObject *__pyx_n_s_seen_logs;
static PyObject *__pyx_n_s_self;
static PyObject *__pyx_n_s_setstate;
static PyObject *__pyx_n_s_setstate_cython;
static PyObject *__pyx_n_s_slots;
static PyObject *__pyx_n_s_source;
static PyObject *__pyx_n_u_source;
static PyObject *__pyx_kp_s_src_zeroconf__protocol_incoming;
static PyObject *__pyx_n_s_state;
static PyObject *__pyx_n_s_str;
static PyObject *__pyx_kp_s_stringsource;
static PyObject *__pyx_n_s_struct;
static PyObject *__pyx_n_s_sys;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_truncated;
static PyObject *__pyx_kp_u_truncated_s;
static PyObject *__pyx_n_s_typing;
static PyObject *__pyx_n_s_unpack_from;
static PyObject *__pyx_n_s_update;
static PyObject *__pyx_n_s_use_setstate;
static PyObject *__pyx_n_s_utils_time;
static PyObject *__pyx_n_u_valid;
static PyObject *__pyx_kp_u_was_seen_again_from;
static PyObject *__pyx_n_s_zeroconf__protocol_incoming;
static int __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming___init__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_source, PyObject *__pyx_v_scope_id, PyObject *__pyx_v_now); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_2is_query(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_4is_response(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_9truncated___get__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_6_log_exception_debug(CYTHON_UNUSED PyTypeObject *__pyx_v_cls, PyObject *__pyx_v_logger_data); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_7answers___get__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_8is_probe___get__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_8__repr__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_5flags___get__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self); /* proto */
static int __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_5flags_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_4data___get__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self); /* proto */
static int __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_4data_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_4data_4__del__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_10name_cache___get__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self); /* proto */
static int __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_10name_cache_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_10name_cache_4__del__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_9questions___get__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self); /* proto */
static int __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_9questions_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_9questions_4__del__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_2id___get__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self); /* proto */
static int __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_2id_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_2id_4__del__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_13num_questions___get__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self); /* proto */
static int __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_13num_questions_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_11num_answers___get__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self); /* proto */
static int __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_11num_answers_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_15num_authorities___get__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self); /* proto */
static int __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_15num_authorities_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_15num_additionals___get__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self); /* proto */
static int __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_15num_additionals_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_5valid___get__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self); /* proto */
static int __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_5valid_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_5valid_4__del__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_3now___get__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self); /* proto */
static int __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_3now_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_3now_4__del__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_8scope_id___get__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self); /* proto */
static int __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_8scope_id_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_8scope_id_4__del__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_6source___get__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self); /* proto */
static int __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_6source_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_6source_4__del__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_10__reduce_cython__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_12__setstate_cython__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_8zeroconf_9_protocol_8incoming___pyx_unpickle_DNSIncoming(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_tp_new_8zeroconf_9_protocol_8incoming_DNSIncoming(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_get = {0, &__pyx_n_s_get, 0, 0, 0};
static PyObject *__pyx_int_0;
static PyObject *__pyx_int_111109179;
static PyObject *__pyx_int_215435062;
static PyObject *__pyx_int_227170572;
static PyObject *__pyx_tuple__3;
static PyObject *__pyx_tuple__4;
static PyObject *__pyx_tuple__5;
static PyObject *__pyx_tuple__6;
static PyObject *__pyx_tuple__7;
static PyObject *__pyx_tuple__8;
static PyObject *__pyx_tuple__9;
static PyObject *__pyx_tuple__11;
static PyObject *__pyx_tuple__13;
static PyObject *__pyx_tuple__15;
static PyObject *__pyx_tuple__17;
static PyObject *__pyx_tuple__19;
static PyObject *__pyx_codeobj__10;
static PyObject *__pyx_codeobj__12;
static PyObject *__pyx_codeobj__14;
static PyObject *__pyx_codeobj__16;
static PyObject *__pyx_codeobj__18;
static PyObject *__pyx_codeobj__20;
/* Late includes */

/* "zeroconf/_protocol/incoming.py":95
 *     )
 * 
 *     def __init__(             # <<<<<<<<<<<<<<
 *         self,
 *         data: bytes,
 */

/* Python wrapper */
static int __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_8zeroconf_9_protocol_8incoming_11DNSIncoming___init__[] = "Constructor from string holding bytes of packet";
#if CYTHON_UPDATE_DESCRIPTOR_DOC
struct wrapperbase __pyx_wrapperbase_8zeroconf_9_protocol_8incoming_11DNSIncoming___init__;
#endif
static int __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_v_source = 0;
  PyObject *__pyx_v_scope_id = 0;
  PyObject *__pyx_v_now = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_data,&__pyx_n_s_source,&__pyx_n_s_scope_id,&__pyx_n_s_now,0};
    PyObject* values[4] = {0,0,0,0};

    /* "zeroconf/_protocol/incoming.py":98
 *         self,
 *         data: bytes,
 *         source: Optional[Tuple[str, int]] = None,             # <<<<<<<<<<<<<<
 *         scope_id: Optional[int] = None,
 *         now: Optional[float] = None,
 */
    values[1] = ((PyObject *)Py_None);

    /* "zeroconf/_protocol/incoming.py":99
 *         data: bytes,
 *         source: Optional[Tuple[str, int]] = None,
 *         scope_id: Optional[int] = None,             # <<<<<<<<<<<<<<
 *         now: Optional[float] = None,
 *     ) -> None:
 */
    values[2] = ((PyObject *)Py_None);

    /* "zeroconf/_protocol/incoming.py":100
 *         source: Optional[Tuple[str, int]] = None,
 *         scope_id: Optional[int] = None,
 *         now: Optional[float] = None,             # <<<<<<<<<<<<<<
 *     ) -> None:
 *         """Constructor from string holding bytes of packet"""
 */
    values[3] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_data)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_source);
          if (value) { values[1] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_scope_id);
          if (value) { values[2] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_now);
          if (value) { values[3] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 95, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_data = ((PyObject*)values[0]);
    __pyx_v_source = values[1];
    __pyx_v_scope_id = values[2];
    __pyx_v_now = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 1, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 95, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("zeroconf._protocol.incoming.DNSIncoming.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_data), (&PyBytes_Type), 1, "data", 1))) __PYX_ERR(0, 97, __pyx_L1_error)
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming___init__(((struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self), __pyx_v_data, __pyx_v_source, __pyx_v_scope_id, __pyx_v_now);

  /* "zeroconf/_protocol/incoming.py":95
 *     )
 * 
 *     def __init__(             # <<<<<<<<<<<<<<
 *         self,
 *         data: bytes,
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming___init__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_source, PyObject *__pyx_v_scope_id, PyObject *__pyx_v_now) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "zeroconf/_protocol/incoming.py":103
 *     ) -> None:
 *         """Constructor from string holding bytes of packet"""
 *         self.flags = 0             # <<<<<<<<<<<<<<
 *         self.offset = 0
 *         self.data = data
 */
  __pyx_v_self->flags = 0;

  /* "zeroconf/_protocol/incoming.py":104
 *         """Constructor from string holding bytes of packet"""
 *         self.flags = 0
 *         self.offset = 0             # <<<<<<<<<<<<<<
 *         self.data = data
 *         self._data_len = len(data)
 */
  __pyx_v_self->offset = 0;

  /* "zeroconf/_protocol/incoming.py":105
 *         self.flags = 0
 *         self.offset = 0
 *         self.data = data             # <<<<<<<<<<<<<<
 *         self._data_len = len(data)
 *         self.name_cache: Dict[int, List[str]] = {}
 */
  __Pyx_INCREF(__pyx_v_data);
  __Pyx_GIVEREF(__pyx_v_data);
  __Pyx_GOTREF(__pyx_v_self->data);
  __Pyx_DECREF(__pyx_v_self->data);
  __pyx_v_self->data = __pyx_v_data;

  /* "zeroconf/_protocol/incoming.py":106
 *         self.offset = 0
 *         self.data = data
 *         self._data_len = len(data)             # <<<<<<<<<<<<<<
 *         self.name_cache: Dict[int, List[str]] = {}
 *         self.questions: List[DNSQuestion] = []
 */
  if (unlikely(__pyx_v_data == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 106, __pyx_L1_error)
  }
  __pyx_t_1 = PyBytes_GET_SIZE(__pyx_v_data); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 106, __pyx_L1_error)
  __pyx_v_self->_data_len = __pyx_t_1;

  /* "zeroconf/_protocol/incoming.py":107
 *         self.data = data
 *         self._data_len = len(data)
 *         self.name_cache: Dict[int, List[str]] = {}             # <<<<<<<<<<<<<<
 *         self.questions: List[DNSQuestion] = []
 *         self._answers: List[DNSRecord] = []
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 107, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->name_cache);
  __Pyx_DECREF(__pyx_v_self->name_cache);
  __pyx_v_self->name_cache = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "zeroconf/_protocol/incoming.py":108
 *         self._data_len = len(data)
 *         self.name_cache: Dict[int, List[str]] = {}
 *         self.questions: List[DNSQuestion] = []             # <<<<<<<<<<<<<<
 *         self._answers: List[DNSRecord] = []
 *         self.id = 0
 */
  __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 108, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->questions);
  __Pyx_DECREF(__pyx_v_self->questions);
  __pyx_v_self->questions = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "zeroconf/_protocol/incoming.py":109
 *         self.name_cache: Dict[int, List[str]] = {}
 *         self.questions: List[DNSQuestion] = []
 *         self._answers: List[DNSRecord] = []             # <<<<<<<<<<<<<<
 *         self.id = 0
 *         self.num_questions = 0
 */
  __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 109, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->_answers);
  __Pyx_DECREF(__pyx_v_self->_answers);
  __pyx_v_self->_answers = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "zeroconf/_protocol/incoming.py":110
 *         self.questions: List[DNSQuestion] = []
 *         self._answers: List[DNSRecord] = []
 *         self.id = 0             # <<<<<<<<<<<<<<
 *         self.num_questions = 0
 *         self.num_answers = 0
 */
  __Pyx_INCREF(__pyx_int_0);
  __Pyx_GIVEREF(__pyx_int_0);
  __Pyx_GOTREF(__pyx_v_self->id);
  __Pyx_DECREF(__pyx_v_self->id);
  __pyx_v_self->id = __pyx_int_0;

  /* "zeroconf/_protocol/incoming.py":111
 *         self._answers: List[DNSRecord] = []
 *         self.id = 0
 *         self.num_questions = 0             # <<<<<<<<<<<<<<
 *         self.num_answers = 0
 *         self.num_authorities = 0
 */
  __pyx_v_self->num_questions = 0;

  /* "zeroconf/_protocol/incoming.py":112
 *         self.id = 0
 *         self.num_questions = 0
 *         self.num_answers = 0             # <<<<<<<<<<<<<<
 *         self.num_authorities = 0
 *         self.num_additionals = 0
 */
  __pyx_v_self->num_answers = 0;

  /* "zeroconf/_protocol/incoming.py":113
 *         self.num_questions = 0
 *         self.num_answers = 0
 *         self.num_authorities = 0             # <<<<<<<<<<<<<<
 *         self.num_additionals = 0
 *         self.valid = False
 */
  __pyx_v_self->num_authorities = 0;

  /* "zeroconf/_protocol/incoming.py":114
 *         self.num_answers = 0
 *         self.num_authorities = 0
 *         self.num_additionals = 0             # <<<<<<<<<<<<<<
 *         self.valid = False
 *         self._did_read_others = False
 */
  __pyx_v_self->num_additionals = 0;

  /* "zeroconf/_protocol/incoming.py":115
 *         self.num_authorities = 0
 *         self.num_additionals = 0
 *         self.valid = False             # <<<<<<<<<<<<<<
 *         self._did_read_others = False
 *         self.now = now or current_time_millis()
 */
  __Pyx_INCREF(Py_False);
  __Pyx_GIVEREF(Py_False);
  __Pyx_GOTREF(__pyx_v_self->valid);
  __Pyx_DECREF(__pyx_v_self->valid);
  __pyx_v_self->valid = Py_False;

  /* "zeroconf/_protocol/incoming.py":116
 *         self.num_additionals = 0
 *         self.valid = False
 *         self._did_read_others = False             # <<<<<<<<<<<<<<
 *         self.now = now or current_time_millis()
 *         self.source = source
 */
  __pyx_v_self->_did_read_others = 0;

  /* "zeroconf/_protocol/incoming.py":117
 *         self.valid = False
 *         self._did_read_others = False
 *         self.now = now or current_time_millis()             # <<<<<<<<<<<<<<
 *         self.source = source
 *         self.scope_id = scope_id
 */
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_v_now); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 117, __pyx_L1_error)
  if (!__pyx_t_3) {
  } else {
    __Pyx_INCREF(__pyx_v_now);
    __pyx_t_2 = __pyx_v_now;
    goto __pyx_L3_bool_binop_done;
  }
  __Pyx_INCREF(__pyx_v_8zeroconf_9_protocol_8incoming_current_time_millis);
  __pyx_t_5 = __pyx_v_8zeroconf_9_protocol_8incoming_current_time_millis; __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  __pyx_t_4 = (__pyx_t_6) ? __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_6) : __Pyx_PyObject_CallNoArg(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 117, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_INCREF(__pyx_t_4);
  __pyx_t_2 = __pyx_t_4;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_L3_bool_binop_done:;
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->now);
  __Pyx_DECREF(__pyx_v_self->now);
  __pyx_v_self->now = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "zeroconf/_protocol/incoming.py":118
 *         self._did_read_others = False
 *         self.now = now or current_time_millis()
 *         self.source = source             # <<<<<<<<<<<<<<
 *         self.scope_id = scope_id
 *         try:
 */
  __Pyx_INCREF(__pyx_v_source);
  __Pyx_GIVEREF(__pyx_v_source);
  __Pyx_GOTREF(__pyx_v_self->source);
  __Pyx_DECREF(__pyx_v_self->source);
  __pyx_v_self->source = __pyx_v_source;

  /* "zeroconf/_protocol/incoming.py":119
 *         self.now = now or current_time_millis()
 *         self.source = source
 *         self.scope_id = scope_id             # <<<<<<<<<<<<<<
 *         try:
 *             self._initial_parse()
 */
  __Pyx_INCREF(__pyx_v_scope_id);
  __Pyx_GIVEREF(__pyx_v_scope_id);
  __Pyx_GOTREF(__pyx_v_self->scope_id);
  __Pyx_DECREF(__pyx_v_self->scope_id);
  __pyx_v_self->scope_id = __pyx_v_scope_id;

  /* "zeroconf/_protocol/incoming.py":120
 *         self.source = source
 *         self.scope_id = scope_id
 *         try:             # <<<<<<<<<<<<<<
 *             self._initial_parse()
 *         except DECODE_EXCEPTIONS:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_7, &__pyx_t_8, &__pyx_t_9);
    __Pyx_XGOTREF(__pyx_t_7);
    __Pyx_XGOTREF(__pyx_t_8);
    __Pyx_XGOTREF(__pyx_t_9);
    /*try:*/ {

      /* "zeroconf/_protocol/incoming.py":121
 *         self.scope_id = scope_id
 *         try:
 *             self._initial_parse()             # <<<<<<<<<<<<<<
 *         except DECODE_EXCEPTIONS:
 *             self._log_exception_debug(
 */
      __pyx_t_2 = ((struct __pyx_vtabstruct_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self->__pyx_vtab)->_initial_parse(__pyx_v_self); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 121, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "zeroconf/_protocol/incoming.py":120
 *         self.source = source
 *         self.scope_id = scope_id
 *         try:             # <<<<<<<<<<<<<<
 *             self._initial_parse()
 *         except DECODE_EXCEPTIONS:
 */
    }
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    goto __pyx_L10_try_end;
    __pyx_L5_error:;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;

    /* "zeroconf/_protocol/incoming.py":122
 *         try:
 *             self._initial_parse()
 *         except DECODE_EXCEPTIONS:             # <<<<<<<<<<<<<<
 *             self._log_exception_debug(
 *                 'Received invalid packet from %s at offset %d while unpacking %r',
 */
    __pyx_t_10 = __Pyx_PyErr_ExceptionMatches(__pyx_v_8zeroconf_9_protocol_8incoming_DECODE_EXCEPTIONS);
    if (__pyx_t_10) {
      __Pyx_AddTraceback("zeroconf._protocol.incoming.DNSIncoming.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_2, &__pyx_t_4, &__pyx_t_5) < 0) __PYX_ERR(0, 122, __pyx_L7_except_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GOTREF(__pyx_t_5);

      /* "zeroconf/_protocol/incoming.py":123
 *             self._initial_parse()
 *         except DECODE_EXCEPTIONS:
 *             self._log_exception_debug(             # <<<<<<<<<<<<<<
 *                 'Received invalid packet from %s at offset %d while unpacking %r',
 *                 self.source,
 */
      __pyx_t_11 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_log_exception_debug); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 123, __pyx_L7_except_error)
      __Pyx_GOTREF(__pyx_t_11);

      /* "zeroconf/_protocol/incoming.py":126
 *                 'Received invalid packet from %s at offset %d while unpacking %r',
 *                 self.source,
 *                 self.offset,             # <<<<<<<<<<<<<<
 *                 self.data,
 *             )
 */
      __pyx_t_12 = __Pyx_PyInt_From_unsigned_int(__pyx_v_self->offset); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 126, __pyx_L7_except_error)
      __Pyx_GOTREF(__pyx_t_12);

      /* "zeroconf/_protocol/incoming.py":127
 *                 self.source,
 *                 self.offset,
 *                 self.data,             # <<<<<<<<<<<<<<
 *             )
 * 
 */
      __pyx_t_13 = NULL;
      __pyx_t_10 = 0;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_11))) {
        __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_11);
        if (likely(__pyx_t_13)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_11);
          __Pyx_INCREF(__pyx_t_13);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_11, function);
          __pyx_t_10 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_11)) {
        PyObject *__pyx_temp[5] = {__pyx_t_13, __pyx_kp_u_Received_invalid_packet_from_s_a, __pyx_v_self->source, __pyx_t_12, __pyx_v_self->data};
        __pyx_t_6 = __Pyx_PyFunction_FastCall(__pyx_t_11, __pyx_temp+1-__pyx_t_10, 4+__pyx_t_10); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 123, __pyx_L7_except_error)
        __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_11)) {
        PyObject *__pyx_temp[5] = {__pyx_t_13, __pyx_kp_u_Received_invalid_packet_from_s_a, __pyx_v_self->source, __pyx_t_12, __pyx_v_self->data};
        __pyx_t_6 = __Pyx_PyCFunction_FastCall(__pyx_t_11, __pyx_temp+1-__pyx_t_10, 4+__pyx_t_10); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 123, __pyx_L7_except_error)
        __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      } else
      #endif
      {
        __pyx_t_14 = PyTuple_New(4+__pyx_t_10); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 123, __pyx_L7_except_error)
        __Pyx_GOTREF(__pyx_t_14);
        if (__pyx_t_13) {
          __Pyx_GIVEREF(__pyx_t_13); PyTuple_SET_ITEM(__pyx_t_14, 0, __pyx_t_13); __pyx_t_13 = NULL;
        }
        __Pyx_INCREF(__pyx_kp_u_Received_invalid_packet_from_s_a);
        __Pyx_GIVEREF(__pyx_kp_u_Received_invalid_packet_from_s_a);
        PyTuple_SET_ITEM(__pyx_t_14, 0+__pyx_t_10, __pyx_kp_u_Received_invalid_packet_from_s_a);
        __Pyx_INCREF(__pyx_v_self->source);
        __Pyx_GIVEREF(__pyx_v_self->source);
        PyTuple_SET_ITEM(__pyx_t_14, 1+__pyx_t_10, __pyx_v_self->source);
        __Pyx_GIVEREF(__pyx_t_12);
        PyTuple_SET_ITEM(__pyx_t_14, 2+__pyx_t_10, __pyx_t_12);
        __Pyx_INCREF(__pyx_v_self->data);
        __Pyx_GIVEREF(__pyx_v_self->data);
        PyTuple_SET_ITEM(__pyx_t_14, 3+__pyx_t_10, __pyx_v_self->data);
        __pyx_t_12 = 0;
        __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_11, __pyx_t_14, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 123, __pyx_L7_except_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      }
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      goto __pyx_L6_exception_handled;
    }
    goto __pyx_L7_except_error;
    __pyx_L7_except_error:;

    /* "zeroconf/_protocol/incoming.py":120
 *         self.source = source
 *         self.scope_id = scope_id
 *         try:             # <<<<<<<<<<<<<<
 *             self._initial_parse()
 *         except DECODE_EXCEPTIONS:
 */
    __Pyx_XGIVEREF(__pyx_t_7);
    __Pyx_XGIVEREF(__pyx_t_8);
    __Pyx_XGIVEREF(__pyx_t_9);
    __Pyx_ExceptionReset(__pyx_t_7, __pyx_t_8, __pyx_t_9);
    goto __pyx_L1_error;
    __pyx_L6_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_7);
    __Pyx_XGIVEREF(__pyx_t_8);
    __Pyx_XGIVEREF(__pyx_t_9);
    __Pyx_ExceptionReset(__pyx_t_7, __pyx_t_8, __pyx_t_9);
    __pyx_L10_try_end:;
  }

  /* "zeroconf/_protocol/incoming.py":95
 *     )
 * 
 *     def __init__(             # <<<<<<<<<<<<<<
 *         self,
 *         data: bytes,
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_AddTraceback("zeroconf._protocol.incoming.DNSIncoming.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/incoming.py":130
 *             )
 * 
 *     def is_query(self) -> bool:             # <<<<<<<<<<<<<<
 *         """Returns true if this is a query."""
 *         return (self.flags & _FLAGS_QR_MASK) == _FLAGS_QR_QUERY
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_3is_query(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_8zeroconf_9_protocol_8incoming_11DNSIncoming_2is_query[] = "Returns true if this is a query.";
static PyMethodDef __pyx_mdef_8zeroconf_9_protocol_8incoming_11DNSIncoming_3is_query = {"is_query", (PyCFunction)__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_3is_query, METH_NOARGS, __pyx_doc_8zeroconf_9_protocol_8incoming_11DNSIncoming_2is_query};
static PyObject *__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_3is_query(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("is_query (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_2is_query(((struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_2is_query(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("is_query", 0);

  /* "zeroconf/_protocol/incoming.py":132
 *     def is_query(self) -> bool:
 *         """Returns true if this is a query."""
 *         return (self.flags & _FLAGS_QR_MASK) == _FLAGS_QR_QUERY             # <<<<<<<<<<<<<<
 * 
 *     def is_response(self) -> bool:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(((__pyx_v_self->flags & __pyx_v_8zeroconf_9_protocol_8incoming__FLAGS_QR_MASK) == __pyx_v_8zeroconf_9_protocol_8incoming__FLAGS_QR_QUERY)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 132, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "zeroconf/_protocol/incoming.py":130
 *             )
 * 
 *     def is_query(self) -> bool:             # <<<<<<<<<<<<<<
 *         """Returns true if this is a query."""
 *         return (self.flags & _FLAGS_QR_MASK) == _FLAGS_QR_QUERY
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("zeroconf._protocol.incoming.DNSIncoming.is_query", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/incoming.py":134
 *         return (self.flags & _FLAGS_QR_MASK) == _FLAGS_QR_QUERY
 * 
 *     def is_response(self) -> bool:             # <<<<<<<<<<<<<<
 *         """Returns true if this is a response."""
 *         return (self.flags & _FLAGS_QR_MASK) == _FLAGS_QR_RESPONSE
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_5is_response(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_8zeroconf_9_protocol_8incoming_11DNSIncoming_4is_response[] = "Returns true if this is a response.";
static PyMethodDef __pyx_mdef_8zeroconf_9_protocol_8incoming_11DNSIncoming_5is_response = {"is_response", (PyCFunction)__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_5is_response, METH_NOARGS, __pyx_doc_8zeroconf_9_protocol_8incoming_11DNSIncoming_4is_response};
static PyObject *__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_5is_response(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("is_response (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_4is_response(((struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_4is_response(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("is_response", 0);

  /* "zeroconf/_protocol/incoming.py":136
 *     def is_response(self) -> bool:
 *         """Returns true if this is a response."""
 *         return (self.flags & _FLAGS_QR_MASK) == _FLAGS_QR_RESPONSE             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(((__pyx_v_self->flags & __pyx_v_8zeroconf_9_protocol_8incoming__FLAGS_QR_MASK) == __pyx_v_8zeroconf_9_protocol_8incoming__FLAGS_QR_RESPONSE)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 136, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "zeroconf/_protocol/incoming.py":134
 *         return (self.flags & _FLAGS_QR_MASK) == _FLAGS_QR_QUERY
 * 
 *     def is_response(self) -> bool:             # <<<<<<<<<<<<<<
 *         """Returns true if this is a response."""
 *         return (self.flags & _FLAGS_QR_MASK) == _FLAGS_QR_RESPONSE
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("zeroconf._protocol.incoming.DNSIncoming.is_response", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/incoming.py":139
 * 
 *     @property
 *     def truncated(self) -> bool:             # <<<<<<<<<<<<<<
 *         """Returns true if this is a truncated."""
 *         return (self.flags & _FLAGS_TC) == _FLAGS_TC
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_9truncated_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_9truncated_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_9truncated___get__(((struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_9truncated___get__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "zeroconf/_protocol/incoming.py":141
 *     def truncated(self) -> bool:
 *         """Returns true if this is a truncated."""
 *         return (self.flags & _FLAGS_TC) == _FLAGS_TC             # <<<<<<<<<<<<<<
 * 
 *     def _initial_parse(self) -> None:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(((__pyx_v_self->flags & __pyx_v_8zeroconf_9_protocol_8incoming__FLAGS_TC) == __pyx_v_8zeroconf_9_protocol_8incoming__FLAGS_TC)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 141, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "zeroconf/_protocol/incoming.py":139
 * 
 *     @property
 *     def truncated(self) -> bool:             # <<<<<<<<<<<<<<
 *         """Returns true if this is a truncated."""
 *         return (self.flags & _FLAGS_TC) == _FLAGS_TC
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("zeroconf._protocol.incoming.DNSIncoming.truncated.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/incoming.py":143
 *         return (self.flags & _FLAGS_TC) == _FLAGS_TC
 * 
 *     def _initial_parse(self) -> None:             # <<<<<<<<<<<<<<
 *         """Parse the data needed to initalize the packet object."""
 *         self._read_header()
 */

static PyObject *__pyx_f_8zeroconf_9_protocol_8incoming_11DNSIncoming__initial_parse(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_initial_parse", 0);

  /* "zeroconf/_protocol/incoming.py":145
 *     def _initial_parse(self) -> None:
 *         """Parse the data needed to initalize the packet object."""
 *         self._read_header()             # <<<<<<<<<<<<<<
 *         self._read_questions()
 *         if not self.num_questions:
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self->__pyx_vtab)->_read_header(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 145, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "zeroconf/_protocol/incoming.py":146
 *         """Parse the data needed to initalize the packet object."""
 *         self._read_header()
 *         self._read_questions()             # <<<<<<<<<<<<<<
 *         if not self.num_questions:
 *             self._read_others()
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self->__pyx_vtab)->_read_questions(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 146, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "zeroconf/_protocol/incoming.py":147
 *         self._read_header()
 *         self._read_questions()
 *         if not self.num_questions:             # <<<<<<<<<<<<<<
 *             self._read_others()
 *         self.valid = True
 */
  __pyx_t_2 = ((!(__pyx_v_self->num_questions != 0)) != 0);
  if (__pyx_t_2) {

    /* "zeroconf/_protocol/incoming.py":148
 *         self._read_questions()
 *         if not self.num_questions:
 *             self._read_others()             # <<<<<<<<<<<<<<
 *         self.valid = True
 * 
 */
    __pyx_t_1 = ((struct __pyx_vtabstruct_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self->__pyx_vtab)->_read_others(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 148, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "zeroconf/_protocol/incoming.py":147
 *         self._read_header()
 *         self._read_questions()
 *         if not self.num_questions:             # <<<<<<<<<<<<<<
 *             self._read_others()
 *         self.valid = True
 */
  }

  /* "zeroconf/_protocol/incoming.py":149
 *         if not self.num_questions:
 *             self._read_others()
 *         self.valid = True             # <<<<<<<<<<<<<<
 * 
 *     @classmethod
 */
  __Pyx_INCREF(Py_True);
  __Pyx_GIVEREF(Py_True);
  __Pyx_GOTREF(__pyx_v_self->valid);
  __Pyx_DECREF(__pyx_v_self->valid);
  __pyx_v_self->valid = Py_True;

  /* "zeroconf/_protocol/incoming.py":143
 *         return (self.flags & _FLAGS_TC) == _FLAGS_TC
 * 
 *     def _initial_parse(self) -> None:             # <<<<<<<<<<<<<<
 *         """Parse the data needed to initalize the packet object."""
 *         self._read_header()
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("zeroconf._protocol.incoming.DNSIncoming._initial_parse", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/incoming.py":152
 * 
 *     @classmethod
 *     def _log_exception_debug(cls, *logger_data: Any) -> None:             # <<<<<<<<<<<<<<
 *         log_exc_info = False
 *         exc_info = sys.exc_info()
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_7_log_exception_debug(PyObject *__pyx_v_cls, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_8zeroconf_9_protocol_8incoming_11DNSIncoming_7_log_exception_debug = {"_log_exception_debug", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_7_log_exception_debug, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_7_log_exception_debug(PyObject *__pyx_v_cls, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_logger_data = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_log_exception_debug (wrapper)", 0);
  if (unlikely(__pyx_kwds) && unlikely(PyDict_Size(__pyx_kwds) > 0) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "_log_exception_debug", 0))) return NULL;
  __Pyx_INCREF(__pyx_args);
  __pyx_v_logger_data = __pyx_args;
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_6_log_exception_debug(((PyTypeObject*)__pyx_v_cls), __pyx_v_logger_data);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_logger_data);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_6_log_exception_debug(CYTHON_UNUSED PyTypeObject *__pyx_v_cls, PyObject *__pyx_v_logger_data) {
  int __pyx_v_log_exc_info;
  PyObject *__pyx_v_exc_info = NULL;
  PyObject *__pyx_v_exc_str = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_log_exception_debug", 0);

  /* "zeroconf/_protocol/incoming.py":153
 *     @classmethod
 *     def _log_exception_debug(cls, *logger_data: Any) -> None:
 *         log_exc_info = False             # <<<<<<<<<<<<<<
 *         exc_info = sys.exc_info()
 *         exc_str = str(exc_info[1])
 */
  __pyx_v_log_exc_info = 0;

  /* "zeroconf/_protocol/incoming.py":154
 *     def _log_exception_debug(cls, *logger_data: Any) -> None:
 *         log_exc_info = False
 *         exc_info = sys.exc_info()             # <<<<<<<<<<<<<<
 *         exc_str = str(exc_info[1])
 *         if exc_str not in _seen_logs:
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_sys); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 154, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_exc_info); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 154, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 154, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_exc_info = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "zeroconf/_protocol/incoming.py":155
 *         log_exc_info = False
 *         exc_info = sys.exc_info()
 *         exc_str = str(exc_info[1])             # <<<<<<<<<<<<<<
 *         if exc_str not in _seen_logs:
 *             # log the trace only on the first time
 */
  __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_exc_info, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 155, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyUnicode_Type)), __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 155, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_exc_str = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "zeroconf/_protocol/incoming.py":156
 *         exc_info = sys.exc_info()
 *         exc_str = str(exc_info[1])
 *         if exc_str not in _seen_logs:             # <<<<<<<<<<<<<<
 *             # log the trace only on the first time
 *             _seen_logs[exc_str] = exc_info
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_seen_logs); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 156, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = (__Pyx_PySequence_ContainsTF(__pyx_v_exc_str, __pyx_t_3, Py_NE)); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 156, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_5 = (__pyx_t_4 != 0);
  if (__pyx_t_5) {

    /* "zeroconf/_protocol/incoming.py":158
 *         if exc_str not in _seen_logs:
 *             # log the trace only on the first time
 *             _seen_logs[exc_str] = exc_info             # <<<<<<<<<<<<<<
 *             log_exc_info = True
 *         log.debug(*(logger_data or ['Exception occurred']), exc_info=log_exc_info)
 */
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_seen_logs); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 158, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_v_exc_str, __pyx_v_exc_info) < 0)) __PYX_ERR(0, 158, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "zeroconf/_protocol/incoming.py":159
 *             # log the trace only on the first time
 *             _seen_logs[exc_str] = exc_info
 *             log_exc_info = True             # <<<<<<<<<<<<<<
 *         log.debug(*(logger_data or ['Exception occurred']), exc_info=log_exc_info)
 * 
 */
    __pyx_v_log_exc_info = 1;

    /* "zeroconf/_protocol/incoming.py":156
 *         exc_info = sys.exc_info()
 *         exc_str = str(exc_info[1])
 *         if exc_str not in _seen_logs:             # <<<<<<<<<<<<<<
 *             # log the trace only on the first time
 *             _seen_logs[exc_str] = exc_info
 */
  }

  /* "zeroconf/_protocol/incoming.py":160
 *             _seen_logs[exc_str] = exc_info
 *             log_exc_info = True
 *         log.debug(*(logger_data or ['Exception occurred']), exc_info=log_exc_info)             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_log); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 160, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_debug); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 160, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_v_logger_data); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 160, __pyx_L1_error)
  if (!__pyx_t_5) {
  } else {
    __Pyx_INCREF(__pyx_v_logger_data);
    __pyx_t_3 = __pyx_v_logger_data;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 160, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_kp_u_Exception_occurred);
  __Pyx_GIVEREF(__pyx_kp_u_Exception_occurred);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_kp_u_Exception_occurred);
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = __pyx_t_2;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_L4_bool_binop_done:;
  __pyx_t_2 = __Pyx_PySequence_Tuple(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 160, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 160, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_6 = __Pyx_PyBool_FromLong(__pyx_v_log_exc_info); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 160, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_exc_info, __pyx_t_6) < 0) __PYX_ERR(0, 160, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 160, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "zeroconf/_protocol/incoming.py":152
 * 
 *     @classmethod
 *     def _log_exception_debug(cls, *logger_data: Any) -> None:             # <<<<<<<<<<<<<<
 *         log_exc_info = False
 *         exc_info = sys.exc_info()
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("zeroconf._protocol.incoming.DNSIncoming._log_exception_debug", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_exc_info);
  __Pyx_XDECREF(__pyx_v_exc_str);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/incoming.py":163
 * 
 *     @property
 *     def answers(self) -> List[DNSRecord]:             # <<<<<<<<<<<<<<
 *         """Answers in the packet."""
 *         if not self._did_read_others:
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_7answers_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_7answers_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_7answers___get__(((struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_7answers___get__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "zeroconf/_protocol/incoming.py":165
 *     def answers(self) -> List[DNSRecord]:
 *         """Answers in the packet."""
 *         if not self._did_read_others:             # <<<<<<<<<<<<<<
 *             try:
 *                 self._read_others()
 */
  __pyx_t_1 = ((!(__pyx_v_self->_did_read_others != 0)) != 0);
  if (__pyx_t_1) {

    /* "zeroconf/_protocol/incoming.py":166
 *         """Answers in the packet."""
 *         if not self._did_read_others:
 *             try:             # <<<<<<<<<<<<<<
 *                 self._read_others()
 *             except DECODE_EXCEPTIONS:
 */
    {
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __Pyx_ExceptionSave(&__pyx_t_2, &__pyx_t_3, &__pyx_t_4);
      __Pyx_XGOTREF(__pyx_t_2);
      __Pyx_XGOTREF(__pyx_t_3);
      __Pyx_XGOTREF(__pyx_t_4);
      /*try:*/ {

        /* "zeroconf/_protocol/incoming.py":167
 *         if not self._did_read_others:
 *             try:
 *                 self._read_others()             # <<<<<<<<<<<<<<
 *             except DECODE_EXCEPTIONS:
 *                 self._log_exception_debug(
 */
        __pyx_t_5 = ((struct __pyx_vtabstruct_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self->__pyx_vtab)->_read_others(__pyx_v_self); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 167, __pyx_L4_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

        /* "zeroconf/_protocol/incoming.py":166
 *         """Answers in the packet."""
 *         if not self._did_read_others:
 *             try:             # <<<<<<<<<<<<<<
 *                 self._read_others()
 *             except DECODE_EXCEPTIONS:
 */
      }
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      goto __pyx_L9_try_end;
      __pyx_L4_error:;
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

      /* "zeroconf/_protocol/incoming.py":168
 *             try:
 *                 self._read_others()
 *             except DECODE_EXCEPTIONS:             # <<<<<<<<<<<<<<
 *                 self._log_exception_debug(
 *                     'Received invalid packet from %s at offset %d while unpacking %r',
 */
      __pyx_t_6 = __Pyx_PyErr_ExceptionMatches(__pyx_v_8zeroconf_9_protocol_8incoming_DECODE_EXCEPTIONS);
      if (__pyx_t_6) {
        __Pyx_AddTraceback("zeroconf._protocol.incoming.DNSIncoming.answers.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
        if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_7, &__pyx_t_8) < 0) __PYX_ERR(0, 168, __pyx_L6_except_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_GOTREF(__pyx_t_8);

        /* "zeroconf/_protocol/incoming.py":169
 *                 self._read_others()
 *             except DECODE_EXCEPTIONS:
 *                 self._log_exception_debug(             # <<<<<<<<<<<<<<
 *                     'Received invalid packet from %s at offset %d while unpacking %r',
 *                     self.source,
 */
        __pyx_t_10 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_log_exception_debug); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 169, __pyx_L6_except_error)
        __Pyx_GOTREF(__pyx_t_10);

        /* "zeroconf/_protocol/incoming.py":172
 *                     'Received invalid packet from %s at offset %d while unpacking %r',
 *                     self.source,
 *                     self.offset,             # <<<<<<<<<<<<<<
 *                     self.data,
 *                 )
 */
        __pyx_t_11 = __Pyx_PyInt_From_unsigned_int(__pyx_v_self->offset); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 172, __pyx_L6_except_error)
        __Pyx_GOTREF(__pyx_t_11);

        /* "zeroconf/_protocol/incoming.py":173
 *                     self.source,
 *                     self.offset,
 *                     self.data,             # <<<<<<<<<<<<<<
 *                 )
 *         return self._answers
 */
        __pyx_t_12 = NULL;
        __pyx_t_6 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_10))) {
          __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_10);
          if (likely(__pyx_t_12)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
            __Pyx_INCREF(__pyx_t_12);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_10, function);
            __pyx_t_6 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_10)) {
          PyObject *__pyx_temp[5] = {__pyx_t_12, __pyx_kp_u_Received_invalid_packet_from_s_a, __pyx_v_self->source, __pyx_t_11, __pyx_v_self->data};
          __pyx_t_9 = __Pyx_PyFunction_FastCall(__pyx_t_10, __pyx_temp+1-__pyx_t_6, 4+__pyx_t_6); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 169, __pyx_L6_except_error)
          __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_GOTREF(__pyx_t_9);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_10)) {
          PyObject *__pyx_temp[5] = {__pyx_t_12, __pyx_kp_u_Received_invalid_packet_from_s_a, __pyx_v_self->source, __pyx_t_11, __pyx_v_self->data};
          __pyx_t_9 = __Pyx_PyCFunction_FastCall(__pyx_t_10, __pyx_temp+1-__pyx_t_6, 4+__pyx_t_6); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 169, __pyx_L6_except_error)
          __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_GOTREF(__pyx_t_9);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        } else
        #endif
        {
          __pyx_t_13 = PyTuple_New(4+__pyx_t_6); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 169, __pyx_L6_except_error)
          __Pyx_GOTREF(__pyx_t_13);
          if (__pyx_t_12) {
            __Pyx_GIVEREF(__pyx_t_12); PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_12); __pyx_t_12 = NULL;
          }
          __Pyx_INCREF(__pyx_kp_u_Received_invalid_packet_from_s_a);
          __Pyx_GIVEREF(__pyx_kp_u_Received_invalid_packet_from_s_a);
          PyTuple_SET_ITEM(__pyx_t_13, 0+__pyx_t_6, __pyx_kp_u_Received_invalid_packet_from_s_a);
          __Pyx_INCREF(__pyx_v_self->source);
          __Pyx_GIVEREF(__pyx_v_self->source);
          PyTuple_SET_ITEM(__pyx_t_13, 1+__pyx_t_6, __pyx_v_self->source);
          __Pyx_GIVEREF(__pyx_t_11);
          PyTuple_SET_ITEM(__pyx_t_13, 2+__pyx_t_6, __pyx_t_11);
          __Pyx_INCREF(__pyx_v_self->data);
          __Pyx_GIVEREF(__pyx_v_self->data);
          PyTuple_SET_ITEM(__pyx_t_13, 3+__pyx_t_6, __pyx_v_self->data);
          __pyx_t_11 = 0;
          __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_10, __pyx_t_13, NULL); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 169, __pyx_L6_except_error)
          __Pyx_GOTREF(__pyx_t_9);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        }
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        goto __pyx_L5_exception_handled;
      }
      goto __pyx_L6_except_error;
      __pyx_L6_except_error:;

      /* "zeroconf/_protocol/incoming.py":166
 *         """Answers in the packet."""
 *         if not self._did_read_others:
 *             try:             # <<<<<<<<<<<<<<
 *                 self._read_others()
 *             except DECODE_EXCEPTIONS:
 */
      __Pyx_XGIVEREF(__pyx_t_2);
      __Pyx_XGIVEREF(__pyx_t_3);
      __Pyx_XGIVEREF(__pyx_t_4);
      __Pyx_ExceptionReset(__pyx_t_2, __pyx_t_3, __pyx_t_4);
      goto __pyx_L1_error;
      __pyx_L5_exception_handled:;
      __Pyx_XGIVEREF(__pyx_t_2);
      __Pyx_XGIVEREF(__pyx_t_3);
      __Pyx_XGIVEREF(__pyx_t_4);
      __Pyx_ExceptionReset(__pyx_t_2, __pyx_t_3, __pyx_t_4);
      __pyx_L9_try_end:;
    }

    /* "zeroconf/_protocol/incoming.py":165
 *     def answers(self) -> List[DNSRecord]:
 *         """Answers in the packet."""
 *         if not self._did_read_others:             # <<<<<<<<<<<<<<
 *             try:
 *                 self._read_others()
 */
  }

  /* "zeroconf/_protocol/incoming.py":175
 *                     self.data,
 *                 )
 *         return self._answers             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_answers);
  __pyx_r = __pyx_v_self->_answers;
  goto __pyx_L0;

  /* "zeroconf/_protocol/incoming.py":163
 * 
 *     @property
 *     def answers(self) -> List[DNSRecord]:             # <<<<<<<<<<<<<<
 *         """Answers in the packet."""
 *         if not self._did_read_others:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_AddTraceback("zeroconf._protocol.incoming.DNSIncoming.answers.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/incoming.py":178
 * 
 *     @property
 *     def is_probe(self) -> bool:             # <<<<<<<<<<<<<<
 *         """Returns true if this is a probe."""
 *         return self.num_authorities > 0
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_8is_probe_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_8is_probe_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_8is_probe___get__(((struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_8is_probe___get__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "zeroconf/_protocol/incoming.py":180
 *     def is_probe(self) -> bool:
 *         """Returns true if this is a probe."""
 *         return self.num_authorities > 0             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self) -> str:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong((__pyx_v_self->num_authorities > 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 180, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "zeroconf/_protocol/incoming.py":178
 * 
 *     @property
 *     def is_probe(self) -> bool:             # <<<<<<<<<<<<<<
 *         """Returns true if this is a probe."""
 *         return self.num_authorities > 0
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("zeroconf._protocol.incoming.DNSIncoming.is_probe.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/incoming.py":182
 *         return self.num_authorities > 0
 * 
 *     def __repr__(self) -> str:             # <<<<<<<<<<<<<<
 *         return '<DNSIncoming:{%s}>' % ', '.join(
 *             [
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_9__repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_9__repr__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_8__repr__(((struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_8__repr__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "zeroconf/_protocol/incoming.py":183
 * 
 *     def __repr__(self) -> str:
 *         return '<DNSIncoming:{%s}>' % ', '.join(             # <<<<<<<<<<<<<<
 *             [
 *                 'id=%s' % self.id,
 */
  __Pyx_XDECREF(__pyx_r);

  /* "zeroconf/_protocol/incoming.py":185
 *         return '<DNSIncoming:{%s}>' % ', '.join(
 *             [
 *                 'id=%s' % self.id,             # <<<<<<<<<<<<<<
 *                 'flags=%s' % self.flags,
 *                 'truncated=%s' % self.truncated,
 */
  __pyx_t_1 = __Pyx_PyUnicode_FormatSafe(__pyx_kp_u_id_s, __pyx_v_self->id); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 185, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);

  /* "zeroconf/_protocol/incoming.py":186
 *             [
 *                 'id=%s' % self.id,
 *                 'flags=%s' % self.flags,             # <<<<<<<<<<<<<<
 *                 'truncated=%s' % self.truncated,
 *                 'n_q=%s' % self.num_questions,
 */
  __pyx_t_2 = __Pyx_PyInt_From_unsigned_int(__pyx_v_self->flags); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 186, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyUnicode_Format(__pyx_kp_u_flags_s, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 186, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "zeroconf/_protocol/incoming.py":187
 *                 'id=%s' % self.id,
 *                 'flags=%s' % self.flags,
 *                 'truncated=%s' % self.truncated,             # <<<<<<<<<<<<<<
 *                 'n_q=%s' % self.num_questions,
 *                 'n_ans=%s' % self.num_answers,
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_truncated); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 187, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyUnicode_FormatSafe(__pyx_kp_u_truncated_s, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 187, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "zeroconf/_protocol/incoming.py":188
 *                 'flags=%s' % self.flags,
 *                 'truncated=%s' % self.truncated,
 *                 'n_q=%s' % self.num_questions,             # <<<<<<<<<<<<<<
 *                 'n_ans=%s' % self.num_answers,
 *                 'n_auth=%s' % self.num_authorities,
 */
  __pyx_t_2 = __Pyx_PyInt_From_unsigned_int(__pyx_v_self->num_questions); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 188, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = PyUnicode_Format(__pyx_kp_u_n_q_s, __pyx_t_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 188, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "zeroconf/_protocol/incoming.py":189
 *                 'truncated=%s' % self.truncated,
 *                 'n_q=%s' % self.num_questions,
 *                 'n_ans=%s' % self.num_answers,             # <<<<<<<<<<<<<<
 *                 'n_auth=%s' % self.num_authorities,
 *                 'n_add=%s' % self.num_additionals,
 */
  __pyx_t_2 = __Pyx_PyInt_From_unsigned_int(__pyx_v_self->num_answers); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 189, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = PyUnicode_Format(__pyx_kp_u_n_ans_s, __pyx_t_2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 189, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "zeroconf/_protocol/incoming.py":190
 *                 'n_q=%s' % self.num_questions,
 *                 'n_ans=%s' % self.num_answers,
 *                 'n_auth=%s' % self.num_authorities,             # <<<<<<<<<<<<<<
 *                 'n_add=%s' % self.num_additionals,
 *                 'questions=%s' % self.questions,
 */
  __pyx_t_2 = __Pyx_PyInt_From_unsigned_int(__pyx_v_self->num_authorities); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 190, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_7 = PyUnicode_Format(__pyx_kp_u_n_auth_s, __pyx_t_2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 190, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "zeroconf/_protocol/incoming.py":191
 *                 'n_ans=%s' % self.num_answers,
 *                 'n_auth=%s' % self.num_authorities,
 *                 'n_add=%s' % self.num_additionals,             # <<<<<<<<<<<<<<
 *                 'questions=%s' % self.questions,
 *                 'answers=%s' % self.answers,
 */
  __pyx_t_2 = __Pyx_PyInt_From_unsigned_int(__pyx_v_self->num_additionals); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 191, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_8 = PyUnicode_Format(__pyx_kp_u_n_add_s, __pyx_t_2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 191, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "zeroconf/_protocol/incoming.py":192
 *                 'n_auth=%s' % self.num_authorities,
 *                 'n_add=%s' % self.num_additionals,
 *                 'questions=%s' % self.questions,             # <<<<<<<<<<<<<<
 *                 'answers=%s' % self.answers,
 *             ]
 */
  __pyx_t_2 = PyUnicode_Format(__pyx_kp_u_questions_s, __pyx_v_self->questions); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 192, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "zeroconf/_protocol/incoming.py":193
 *                 'n_add=%s' % self.num_additionals,
 *                 'questions=%s' % self.questions,
 *                 'answers=%s' % self.answers,             # <<<<<<<<<<<<<<
 *             ]
 *         )
 */
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_answers); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 193, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_10 = __Pyx_PyUnicode_FormatSafe(__pyx_kp_u_answers_s, __pyx_t_9); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 193, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

  /* "zeroconf/_protocol/incoming.py":184
 *     def __repr__(self) -> str:
 *         return '<DNSIncoming:{%s}>' % ', '.join(
 *             [             # <<<<<<<<<<<<<<
 *                 'id=%s' % self.id,
 *                 'flags=%s' % self.flags,
 */
  __pyx_t_9 = PyList_New(9); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 184, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_GIVEREF(__pyx_t_1);
  PyList_SET_ITEM(__pyx_t_9, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_3);
  PyList_SET_ITEM(__pyx_t_9, 1, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyList_SET_ITEM(__pyx_t_9, 2, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyList_SET_ITEM(__pyx_t_9, 3, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_6);
  PyList_SET_ITEM(__pyx_t_9, 4, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_7);
  PyList_SET_ITEM(__pyx_t_9, 5, __pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_8);
  PyList_SET_ITEM(__pyx_t_9, 6, __pyx_t_8);
  __Pyx_GIVEREF(__pyx_t_2);
  PyList_SET_ITEM(__pyx_t_9, 7, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_10);
  PyList_SET_ITEM(__pyx_t_9, 8, __pyx_t_10);
  __pyx_t_1 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_t_6 = 0;
  __pyx_t_7 = 0;
  __pyx_t_8 = 0;
  __pyx_t_2 = 0;
  __pyx_t_10 = 0;

  /* "zeroconf/_protocol/incoming.py":183
 * 
 *     def __repr__(self) -> str:
 *         return '<DNSIncoming:{%s}>' % ', '.join(             # <<<<<<<<<<<<<<
 *             [
 *                 'id=%s' % self.id,
 */
  __pyx_t_10 = PyUnicode_Join(__pyx_kp_u_, __pyx_t_9); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 183, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_9 = PyUnicode_Format(__pyx_kp_u_DNSIncoming_s, __pyx_t_10); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 183, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_r = __pyx_t_9;
  __pyx_t_9 = 0;
  goto __pyx_L0;

  /* "zeroconf/_protocol/incoming.py":182
 *         return self.num_authorities > 0
 * 
 *     def __repr__(self) -> str:             # <<<<<<<<<<<<<<
 *         return '<DNSIncoming:{%s}>' % ', '.join(
 *             [
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("zeroconf._protocol.incoming.DNSIncoming.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/incoming.py":197
 *         )
 * 
 *     def _read_header(self) -> None:             # <<<<<<<<<<<<<<
 *         """Reads header portion of packet"""
 *         (
 */

static PyObject *__pyx_f_8zeroconf_9_protocol_8incoming_11DNSIncoming__read_header(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *(*__pyx_t_9)(PyObject *);
  unsigned int __pyx_t_10;
  unsigned int __pyx_t_11;
  unsigned int __pyx_t_12;
  unsigned int __pyx_t_13;
  unsigned int __pyx_t_14;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_read_header", 0);

  /* "zeroconf/_protocol/incoming.py":206
 *             self.num_authorities,
 *             self.num_additionals,
 *         ) = UNPACK_6H(self.data)             # <<<<<<<<<<<<<<
 *         self.offset += 12
 * 
 */
  __Pyx_INCREF(__pyx_v_8zeroconf_9_protocol_8incoming_UNPACK_6H);
  __pyx_t_2 = __pyx_v_8zeroconf_9_protocol_8incoming_UNPACK_6H; __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_v_self->data) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_self->data);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 206, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
    PyObject* sequence = __pyx_t_1;
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
    if (unlikely(size != 6)) {
      if (size > 6) __Pyx_RaiseTooManyValuesError(6);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 200, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_2 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 1); 
      __pyx_t_4 = PyTuple_GET_ITEM(sequence, 2); 
      __pyx_t_5 = PyTuple_GET_ITEM(sequence, 3); 
      __pyx_t_6 = PyTuple_GET_ITEM(sequence, 4); 
      __pyx_t_7 = PyTuple_GET_ITEM(sequence, 5); 
    } else {
      __pyx_t_2 = PyList_GET_ITEM(sequence, 0); 
      __pyx_t_3 = PyList_GET_ITEM(sequence, 1); 
      __pyx_t_4 = PyList_GET_ITEM(sequence, 2); 
      __pyx_t_5 = PyList_GET_ITEM(sequence, 3); 
      __pyx_t_6 = PyList_GET_ITEM(sequence, 4); 
      __pyx_t_7 = PyList_GET_ITEM(sequence, 5); 
    }
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_t_5);
    __Pyx_INCREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_t_7);
    #else
    {
      Py_ssize_t i;
      PyObject** temps[6] = {&__pyx_t_2,&__pyx_t_3,&__pyx_t_4,&__pyx_t_5,&__pyx_t_6,&__pyx_t_7};
      for (i=0; i < 6; i++) {
        PyObject* item = PySequence_ITEM(sequence, i); if (unlikely(!item)) __PYX_ERR(0, 200, __pyx_L1_error)
        __Pyx_GOTREF(item);
        *(temps[i]) = item;
      }
    }
    #endif
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else {
    Py_ssize_t index = -1;
    PyObject** temps[6] = {&__pyx_t_2,&__pyx_t_3,&__pyx_t_4,&__pyx_t_5,&__pyx_t_6,&__pyx_t_7};
    __pyx_t_8 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 200, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_9 = Py_TYPE(__pyx_t_8)->tp_iternext;
    for (index=0; index < 6; index++) {
      PyObject* item = __pyx_t_9(__pyx_t_8); if (unlikely(!item)) goto __pyx_L3_unpacking_failed;
      __Pyx_GOTREF(item);
      *(temps[index]) = item;
    }
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_9(__pyx_t_8), 6) < 0) __PYX_ERR(0, 200, __pyx_L1_error)
    __pyx_t_9 = NULL;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    goto __pyx_L4_unpacking_done;
    __pyx_L3_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_9 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 200, __pyx_L1_error)
    __pyx_L4_unpacking_done:;
  }

  /* "zeroconf/_protocol/incoming.py":200
 *         """Reads header portion of packet"""
 *         (
 *             self.id,             # <<<<<<<<<<<<<<
 *             self.flags,
 *             self.num_questions,
 */
  __pyx_t_10 = __Pyx_PyInt_As_unsigned_int(__pyx_t_3); if (unlikely((__pyx_t_10 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(0, 200, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_11 = __Pyx_PyInt_As_unsigned_int(__pyx_t_4); if (unlikely((__pyx_t_11 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(0, 200, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_12 = __Pyx_PyInt_As_unsigned_int(__pyx_t_5); if (unlikely((__pyx_t_12 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(0, 200, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_13 = __Pyx_PyInt_As_unsigned_int(__pyx_t_6); if (unlikely((__pyx_t_13 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(0, 200, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_14 = __Pyx_PyInt_As_unsigned_int(__pyx_t_7); if (unlikely((__pyx_t_14 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(0, 200, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->id);
  __Pyx_DECREF(__pyx_v_self->id);
  __pyx_v_self->id = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "zeroconf/_protocol/incoming.py":201
 *         (
 *             self.id,
 *             self.flags,             # <<<<<<<<<<<<<<
 *             self.num_questions,
 *             self.num_answers,
 */
  __pyx_v_self->flags = __pyx_t_10;

  /* "zeroconf/_protocol/incoming.py":202
 *             self.id,
 *             self.flags,
 *             self.num_questions,             # <<<<<<<<<<<<<<
 *             self.num_answers,
 *             self.num_authorities,
 */
  __pyx_v_self->num_questions = __pyx_t_11;

  /* "zeroconf/_protocol/incoming.py":203
 *             self.flags,
 *             self.num_questions,
 *             self.num_answers,             # <<<<<<<<<<<<<<
 *             self.num_authorities,
 *             self.num_additionals,
 */
  __pyx_v_self->num_answers = __pyx_t_12;

  /* "zeroconf/_protocol/incoming.py":204
 *             self.num_questions,
 *             self.num_answers,
 *             self.num_authorities,             # <<<<<<<<<<<<<<
 *             self.num_additionals,
 *         ) = UNPACK_6H(self.data)
 */
  __pyx_v_self->num_authorities = __pyx_t_13;

  /* "zeroconf/_protocol/incoming.py":205
 *             self.num_answers,
 *             self.num_authorities,
 *             self.num_additionals,             # <<<<<<<<<<<<<<
 *         ) = UNPACK_6H(self.data)
 *         self.offset += 12
 */
  __pyx_v_self->num_additionals = __pyx_t_14;

  /* "zeroconf/_protocol/incoming.py":207
 *             self.num_additionals,
 *         ) = UNPACK_6H(self.data)
 *         self.offset += 12             # <<<<<<<<<<<<<<
 * 
 *     def _read_questions(self) -> None:
 */
  __pyx_v_self->offset = (__pyx_v_self->offset + 12);

  /* "zeroconf/_protocol/incoming.py":197
 *         )
 * 
 *     def _read_header(self) -> None:             # <<<<<<<<<<<<<<
 *         """Reads header portion of packet"""
 *         (
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("zeroconf._protocol.incoming.DNSIncoming._read_header", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/incoming.py":209
 *         self.offset += 12
 * 
 *     def _read_questions(self) -> None:             # <<<<<<<<<<<<<<
 *         """Reads questions section of packet"""
 *         for _ in range(self.num_questions):
 */

static PyObject *__pyx_f_8zeroconf_9_protocol_8incoming_11DNSIncoming__read_questions(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self) {
  CYTHON_UNUSED unsigned int __pyx_v__;
  PyObject *__pyx_v_name = NULL;
  PyObject *__pyx_v_type_ = NULL;
  PyObject *__pyx_v_class_ = NULL;
  struct __pyx_obj_8zeroconf_4_dns_DNSQuestion *__pyx_v_question = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  unsigned int __pyx_t_1;
  unsigned int __pyx_t_2;
  unsigned int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  PyObject *(*__pyx_t_10)(PyObject *);
  int __pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_read_questions", 0);

  /* "zeroconf/_protocol/incoming.py":211
 *     def _read_questions(self) -> None:
 *         """Reads questions section of packet"""
 *         for _ in range(self.num_questions):             # <<<<<<<<<<<<<<
 *             name = self._read_name()
 *             type_, class_ = UNPACK_HH(self.data, self.offset)
 */
  __pyx_t_1 = __pyx_v_self->num_questions;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v__ = __pyx_t_3;

    /* "zeroconf/_protocol/incoming.py":212
 *         """Reads questions section of packet"""
 *         for _ in range(self.num_questions):
 *             name = self._read_name()             # <<<<<<<<<<<<<<
 *             type_, class_ = UNPACK_HH(self.data, self.offset)
 *             self.offset += 4
 */
    __pyx_t_4 = ((struct __pyx_vtabstruct_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self->__pyx_vtab)->_read_name(__pyx_v_self); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 212, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_XDECREF_SET(__pyx_v_name, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "zeroconf/_protocol/incoming.py":213
 *         for _ in range(self.num_questions):
 *             name = self._read_name()
 *             type_, class_ = UNPACK_HH(self.data, self.offset)             # <<<<<<<<<<<<<<
 *             self.offset += 4
 *             question = DNSQuestion(name, type_, class_)
 */
    __pyx_t_5 = __Pyx_PyInt_From_unsigned_int(__pyx_v_self->offset); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 213, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_INCREF(__pyx_v_8zeroconf_9_protocol_8incoming_UNPACK_HH);
    __pyx_t_6 = __pyx_v_8zeroconf_9_protocol_8incoming_UNPACK_HH; __pyx_t_7 = NULL;
    __pyx_t_8 = 0;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
      if (likely(__pyx_t_7)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_6, function);
        __pyx_t_8 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_6)) {
      PyObject *__pyx_temp[3] = {__pyx_t_7, __pyx_v_self->data, __pyx_t_5};
      __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 213, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
      PyObject *__pyx_temp[3] = {__pyx_t_7, __pyx_v_self->data, __pyx_t_5};
      __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 213, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else
    #endif
    {
      __pyx_t_9 = PyTuple_New(2+__pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 213, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      if (__pyx_t_7) {
        __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_7); __pyx_t_7 = NULL;
      }
      __Pyx_INCREF(__pyx_v_self->data);
      __Pyx_GIVEREF(__pyx_v_self->data);
      PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_8, __pyx_v_self->data);
      __Pyx_GIVEREF(__pyx_t_5);
      PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_8, __pyx_t_5);
      __pyx_t_5 = 0;
      __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_9, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 213, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    }
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if ((likely(PyTuple_CheckExact(__pyx_t_4))) || (PyList_CheckExact(__pyx_t_4))) {
      PyObject* sequence = __pyx_t_4;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 213, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_6 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_9 = PyTuple_GET_ITEM(sequence, 1); 
      } else {
        __pyx_t_6 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_9 = PyList_GET_ITEM(sequence, 1); 
      }
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_9);
      #else
      __pyx_t_6 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 213, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_9 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 213, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      #endif
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_5 = PyObject_GetIter(__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 213, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_10 = Py_TYPE(__pyx_t_5)->tp_iternext;
      index = 0; __pyx_t_6 = __pyx_t_10(__pyx_t_5); if (unlikely(!__pyx_t_6)) goto __pyx_L5_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_6);
      index = 1; __pyx_t_9 = __pyx_t_10(__pyx_t_5); if (unlikely(!__pyx_t_9)) goto __pyx_L5_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_9);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_10(__pyx_t_5), 2) < 0) __PYX_ERR(0, 213, __pyx_L1_error)
      __pyx_t_10 = NULL;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      goto __pyx_L6_unpacking_done;
      __pyx_L5_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_10 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 213, __pyx_L1_error)
      __pyx_L6_unpacking_done:;
    }
    __Pyx_XDECREF_SET(__pyx_v_type_, __pyx_t_6);
    __pyx_t_6 = 0;
    __Pyx_XDECREF_SET(__pyx_v_class_, __pyx_t_9);
    __pyx_t_9 = 0;

    /* "zeroconf/_protocol/incoming.py":214
 *             name = self._read_name()
 *             type_, class_ = UNPACK_HH(self.data, self.offset)
 *             self.offset += 4             # <<<<<<<<<<<<<<
 *             question = DNSQuestion(name, type_, class_)
 *             self.questions.append(question)
 */
    __pyx_v_self->offset = (__pyx_v_self->offset + 4);

    /* "zeroconf/_protocol/incoming.py":215
 *             type_, class_ = UNPACK_HH(self.data, self.offset)
 *             self.offset += 4
 *             question = DNSQuestion(name, type_, class_)             # <<<<<<<<<<<<<<
 *             self.questions.append(question)
 * 
 */
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 215, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_v_name);
    __Pyx_GIVEREF(__pyx_v_name);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_name);
    __Pyx_INCREF(__pyx_v_type_);
    __Pyx_GIVEREF(__pyx_v_type_);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_v_type_);
    __Pyx_INCREF(__pyx_v_class_);
    __Pyx_GIVEREF(__pyx_v_class_);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_class_);
    __pyx_t_9 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_8zeroconf_4_dns_DNSQuestion), __pyx_t_4, NULL); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 215, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF_SET(__pyx_v_question, ((struct __pyx_obj_8zeroconf_4_dns_DNSQuestion *)__pyx_t_9));
    __pyx_t_9 = 0;

    /* "zeroconf/_protocol/incoming.py":216
 *             self.offset += 4
 *             question = DNSQuestion(name, type_, class_)
 *             self.questions.append(question)             # <<<<<<<<<<<<<<
 * 
 *     def _read_character_string(self) -> bytes:
 */
    if (unlikely(__pyx_v_self->questions == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
      __PYX_ERR(0, 216, __pyx_L1_error)
    }
    __pyx_t_11 = __Pyx_PyList_Append(__pyx_v_self->questions, ((PyObject *)__pyx_v_question)); if (unlikely(__pyx_t_11 == ((int)-1))) __PYX_ERR(0, 216, __pyx_L1_error)
  }

  /* "zeroconf/_protocol/incoming.py":209
 *         self.offset += 12
 * 
 *     def _read_questions(self) -> None:             # <<<<<<<<<<<<<<
 *         """Reads questions section of packet"""
 *         for _ in range(self.num_questions):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("zeroconf._protocol.incoming.DNSIncoming._read_questions", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_name);
  __Pyx_XDECREF(__pyx_v_type_);
  __Pyx_XDECREF(__pyx_v_class_);
  __Pyx_XDECREF((PyObject *)__pyx_v_question);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/incoming.py":218
 *             self.questions.append(question)
 * 
 *     def _read_character_string(self) -> bytes:             # <<<<<<<<<<<<<<
 *         """Reads a character string from the packet"""
 *         length = self.data[self.offset]
 */

static PyObject *__pyx_f_8zeroconf_9_protocol_8incoming_11DNSIncoming__read_character_string(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self) {
  PyObject *__pyx_v_length = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  unsigned int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_read_character_string", 0);

  /* "zeroconf/_protocol/incoming.py":220
 *     def _read_character_string(self) -> bytes:
 *         """Reads a character string from the packet"""
 *         length = self.data[self.offset]             # <<<<<<<<<<<<<<
 *         self.offset += 1
 *         return self._read_string(length)
 */
  __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_self->data, __pyx_v_self->offset, unsigned int, 0, __Pyx_PyInt_From_unsigned_int, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 220, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_length = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "zeroconf/_protocol/incoming.py":221
 *         """Reads a character string from the packet"""
 *         length = self.data[self.offset]
 *         self.offset += 1             # <<<<<<<<<<<<<<
 *         return self._read_string(length)
 * 
 */
  __pyx_v_self->offset = (__pyx_v_self->offset + 1);

  /* "zeroconf/_protocol/incoming.py":222
 *         length = self.data[self.offset]
 *         self.offset += 1
 *         return self._read_string(length)             # <<<<<<<<<<<<<<
 * 
 *     def _read_string(self, length: int) -> bytes:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyInt_As_unsigned_int(__pyx_v_length); if (unlikely((__pyx_t_2 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(0, 222, __pyx_L1_error)
  __pyx_t_1 = ((struct __pyx_vtabstruct_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self->__pyx_vtab)->_read_string(__pyx_v_self, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 222, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||((void)PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 222, __pyx_L1_error)
  __pyx_r = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "zeroconf/_protocol/incoming.py":218
 *             self.questions.append(question)
 * 
 *     def _read_character_string(self) -> bytes:             # <<<<<<<<<<<<<<
 *         """Reads a character string from the packet"""
 *         length = self.data[self.offset]
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("zeroconf._protocol.incoming.DNSIncoming._read_character_string", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_length);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/incoming.py":224
 *         return self._read_string(length)
 * 
 *     def _read_string(self, length: int) -> bytes:             # <<<<<<<<<<<<<<
 *         """Reads a string of a given length from the packet"""
 *         info = self.data[self.offset : self.offset + length]
 */

static PyObject *__pyx_f_8zeroconf_9_protocol_8incoming_11DNSIncoming__read_string(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self, unsigned int __pyx_v_length) {
  PyObject *__pyx_v_info = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_read_string", 0);

  /* "zeroconf/_protocol/incoming.py":226
 *     def _read_string(self, length: int) -> bytes:
 *         """Reads a string of a given length from the packet"""
 *         info = self.data[self.offset : self.offset + length]             # <<<<<<<<<<<<<<
 *         self.offset += length
 *         return info
 */
  if (unlikely(__pyx_v_self->data == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 226, __pyx_L1_error)
  }
  __pyx_t_1 = PySequence_GetSlice(__pyx_v_self->data, __pyx_v_self->offset, (__pyx_v_self->offset + __pyx_v_length)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 226, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_info = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "zeroconf/_protocol/incoming.py":227
 *         """Reads a string of a given length from the packet"""
 *         info = self.data[self.offset : self.offset + length]
 *         self.offset += length             # <<<<<<<<<<<<<<
 *         return info
 * 
 */
  __pyx_v_self->offset = (__pyx_v_self->offset + __pyx_v_length);

  /* "zeroconf/_protocol/incoming.py":228
 *         info = self.data[self.offset : self.offset + length]
 *         self.offset += length
 *         return info             # <<<<<<<<<<<<<<
 * 
 *     def _read_others(self) -> None:
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_info);
  __pyx_r = __pyx_v_info;
  goto __pyx_L0;

  /* "zeroconf/_protocol/incoming.py":224
 *         return self._read_string(length)
 * 
 *     def _read_string(self, length: int) -> bytes:             # <<<<<<<<<<<<<<
 *         """Reads a string of a given length from the packet"""
 *         info = self.data[self.offset : self.offset + length]
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("zeroconf._protocol.incoming.DNSIncoming._read_string", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_info);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/incoming.py":230
 *         return info
 * 
 *     def _read_others(self) -> None:             # <<<<<<<<<<<<<<
 *         """Reads the answers, authorities and additionals section of the
 *         packet"""
 */

static PyObject *__pyx_f_8zeroconf_9_protocol_8incoming_11DNSIncoming__read_others(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self) {
  unsigned int __pyx_v_end;
  unsigned int __pyx_v_length;
  unsigned int __pyx_v_n;
  CYTHON_UNUSED unsigned int __pyx_v__;
  PyObject *__pyx_v_domain = NULL;
  PyObject *__pyx_v_type_ = NULL;
  PyObject *__pyx_v_class_ = NULL;
  PyObject *__pyx_v_ttl = NULL;
  PyObject *__pyx_v_rec = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  unsigned int __pyx_t_1;
  unsigned int __pyx_t_2;
  unsigned int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *(*__pyx_t_11)(PyObject *);
  unsigned int __pyx_t_12;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  int __pyx_t_18;
  int __pyx_t_19;
  int __pyx_t_20;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_read_others", 0);

  /* "zeroconf/_protocol/incoming.py":233
 *         """Reads the answers, authorities and additionals section of the
 *         packet"""
 *         self._did_read_others = True             # <<<<<<<<<<<<<<
 *         n = self.num_answers + self.num_authorities + self.num_additionals
 *         for _ in range(n):
 */
  __pyx_v_self->_did_read_others = 1;

  /* "zeroconf/_protocol/incoming.py":234
 *         packet"""
 *         self._did_read_others = True
 *         n = self.num_answers + self.num_authorities + self.num_additionals             # <<<<<<<<<<<<<<
 *         for _ in range(n):
 *             domain = self._read_name()
 */
  __pyx_v_n = ((__pyx_v_self->num_answers + __pyx_v_self->num_authorities) + __pyx_v_self->num_additionals);

  /* "zeroconf/_protocol/incoming.py":235
 *         self._did_read_others = True
 *         n = self.num_answers + self.num_authorities + self.num_additionals
 *         for _ in range(n):             # <<<<<<<<<<<<<<
 *             domain = self._read_name()
 *             type_, class_, ttl, length = UNPACK_HHiH(self.data, self.offset)
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v__ = __pyx_t_3;

    /* "zeroconf/_protocol/incoming.py":236
 *         n = self.num_answers + self.num_authorities + self.num_additionals
 *         for _ in range(n):
 *             domain = self._read_name()             # <<<<<<<<<<<<<<
 *             type_, class_, ttl, length = UNPACK_HHiH(self.data, self.offset)
 *             self.offset += 10
 */
    __pyx_t_4 = ((struct __pyx_vtabstruct_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self->__pyx_vtab)->_read_name(__pyx_v_self); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 236, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_XDECREF_SET(__pyx_v_domain, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "zeroconf/_protocol/incoming.py":237
 *         for _ in range(n):
 *             domain = self._read_name()
 *             type_, class_, ttl, length = UNPACK_HHiH(self.data, self.offset)             # <<<<<<<<<<<<<<
 *             self.offset += 10
 *             end = self.offset + length
 */
    __pyx_t_5 = __Pyx_PyInt_From_unsigned_int(__pyx_v_self->offset); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 237, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_INCREF(__pyx_v_8zeroconf_9_protocol_8incoming_UNPACK_HHiH);
    __pyx_t_6 = __pyx_v_8zeroconf_9_protocol_8incoming_UNPACK_HHiH; __pyx_t_7 = NULL;
    __pyx_t_8 = 0;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
      if (likely(__pyx_t_7)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_6, function);
        __pyx_t_8 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_6)) {
      PyObject *__pyx_temp[3] = {__pyx_t_7, __pyx_v_self->data, __pyx_t_5};
      __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 237, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
      PyObject *__pyx_temp[3] = {__pyx_t_7, __pyx_v_self->data, __pyx_t_5};
      __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 237, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else
    #endif
    {
      __pyx_t_9 = PyTuple_New(2+__pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 237, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      if (__pyx_t_7) {
        __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_7); __pyx_t_7 = NULL;
      }
      __Pyx_INCREF(__pyx_v_self->data);
      __Pyx_GIVEREF(__pyx_v_self->data);
      PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_8, __pyx_v_self->data);
      __Pyx_GIVEREF(__pyx_t_5);
      PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_8, __pyx_t_5);
      __pyx_t_5 = 0;
      __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_9, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 237, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    }
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if ((likely(PyTuple_CheckExact(__pyx_t_4))) || (PyList_CheckExact(__pyx_t_4))) {
      PyObject* sequence = __pyx_t_4;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 4)) {
        if (size > 4) __Pyx_RaiseTooManyValuesError(4);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 237, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_6 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_9 = PyTuple_GET_ITEM(sequence, 1); 
        __pyx_t_5 = PyTuple_GET_ITEM(sequence, 2); 
        __pyx_t_7 = PyTuple_GET_ITEM(sequence, 3); 
      } else {
        __pyx_t_6 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_9 = PyList_GET_ITEM(sequence, 1); 
        __pyx_t_5 = PyList_GET_ITEM(sequence, 2); 
        __pyx_t_7 = PyList_GET_ITEM(sequence, 3); 
      }
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_9);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_7);
      #else
      {
        Py_ssize_t i;
        PyObject** temps[4] = {&__pyx_t_6,&__pyx_t_9,&__pyx_t_5,&__pyx_t_7};
        for (i=0; i < 4; i++) {
          PyObject* item = PySequence_ITEM(sequence, i); if (unlikely(!item)) __PYX_ERR(0, 237, __pyx_L1_error)
          __Pyx_GOTREF(item);
          *(temps[i]) = item;
        }
      }
      #endif
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else {
      Py_ssize_t index = -1;
      PyObject** temps[4] = {&__pyx_t_6,&__pyx_t_9,&__pyx_t_5,&__pyx_t_7};
      __pyx_t_10 = PyObject_GetIter(__pyx_t_4); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 237, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_11 = Py_TYPE(__pyx_t_10)->tp_iternext;
      for (index=0; index < 4; index++) {
        PyObject* item = __pyx_t_11(__pyx_t_10); if (unlikely(!item)) goto __pyx_L5_unpacking_failed;
        __Pyx_GOTREF(item);
        *(temps[index]) = item;
      }
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_11(__pyx_t_10), 4) < 0) __PYX_ERR(0, 237, __pyx_L1_error)
      __pyx_t_11 = NULL;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      goto __pyx_L6_unpacking_done;
      __pyx_L5_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __pyx_t_11 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 237, __pyx_L1_error)
      __pyx_L6_unpacking_done:;
    }
    __pyx_t_12 = __Pyx_PyInt_As_unsigned_int(__pyx_t_7); if (unlikely((__pyx_t_12 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(0, 237, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_XDECREF_SET(__pyx_v_type_, __pyx_t_6);
    __pyx_t_6 = 0;
    __Pyx_XDECREF_SET(__pyx_v_class_, __pyx_t_9);
    __pyx_t_9 = 0;
    __Pyx_XDECREF_SET(__pyx_v_ttl, __pyx_t_5);
    __pyx_t_5 = 0;
    __pyx_v_length = __pyx_t_12;

    /* "zeroconf/_protocol/incoming.py":238
 *             domain = self._read_name()
 *             type_, class_, ttl, length = UNPACK_HHiH(self.data, self.offset)
 *             self.offset += 10             # <<<<<<<<<<<<<<
 *             end = self.offset + length
 *             rec = None
 */
    __pyx_v_self->offset = (__pyx_v_self->offset + 10);

    /* "zeroconf/_protocol/incoming.py":239
 *             type_, class_, ttl, length = UNPACK_HHiH(self.data, self.offset)
 *             self.offset += 10
 *             end = self.offset + length             # <<<<<<<<<<<<<<
 *             rec = None
 *             try:
 */
    __pyx_v_end = (__pyx_v_self->offset + __pyx_v_length);

    /* "zeroconf/_protocol/incoming.py":240
 *             self.offset += 10
 *             end = self.offset + length
 *             rec = None             # <<<<<<<<<<<<<<
 *             try:
 *                 rec = self._read_record(domain, type_, class_, ttl, length)
 */
    __Pyx_INCREF(Py_None);
    __Pyx_XDECREF_SET(__pyx_v_rec, Py_None);

    /* "zeroconf/_protocol/incoming.py":241
 *             end = self.offset + length
 *             rec = None
 *             try:             # <<<<<<<<<<<<<<
 *                 rec = self._read_record(domain, type_, class_, ttl, length)
 *             except DECODE_EXCEPTIONS:
 */
    {
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __Pyx_ExceptionSave(&__pyx_t_13, &__pyx_t_14, &__pyx_t_15);
      __Pyx_XGOTREF(__pyx_t_13);
      __Pyx_XGOTREF(__pyx_t_14);
      __Pyx_XGOTREF(__pyx_t_15);
      /*try:*/ {

        /* "zeroconf/_protocol/incoming.py":242
 *             rec = None
 *             try:
 *                 rec = self._read_record(domain, type_, class_, ttl, length)             # <<<<<<<<<<<<<<
 *             except DECODE_EXCEPTIONS:
 *                 # Skip records that fail to decode if we know the length
 */
        __pyx_t_12 = __Pyx_PyInt_As_unsigned_int(__pyx_v_type_); if (unlikely((__pyx_t_12 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(0, 242, __pyx_L7_error)
        __pyx_t_4 = ((struct __pyx_vtabstruct_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self->__pyx_vtab)->_read_record(__pyx_v_self, __pyx_v_domain, __pyx_t_12, __pyx_v_class_, __pyx_v_ttl, __pyx_v_length); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 242, __pyx_L7_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF_SET(__pyx_v_rec, __pyx_t_4);
        __pyx_t_4 = 0;

        /* "zeroconf/_protocol/incoming.py":241
 *             end = self.offset + length
 *             rec = None
 *             try:             # <<<<<<<<<<<<<<
 *                 rec = self._read_record(domain, type_, class_, ttl, length)
 *             except DECODE_EXCEPTIONS:
 */
      }
      __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
      __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
      __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
      goto __pyx_L14_try_end;
      __pyx_L7_error:;
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;

      /* "zeroconf/_protocol/incoming.py":243
 *             try:
 *                 rec = self._read_record(domain, type_, class_, ttl, length)
 *             except DECODE_EXCEPTIONS:             # <<<<<<<<<<<<<<
 *                 # Skip records that fail to decode if we know the length
 *                 # If the packet is really corrupt read_name and the unpack
 */
      __pyx_t_8 = __Pyx_PyErr_ExceptionMatches(__pyx_v_8zeroconf_9_protocol_8incoming_DECODE_EXCEPTIONS);
      if (__pyx_t_8) {
        __Pyx_AddTraceback("zeroconf._protocol.incoming.DNSIncoming._read_others", __pyx_clineno, __pyx_lineno, __pyx_filename);
        if (__Pyx_GetException(&__pyx_t_4, &__pyx_t_7, &__pyx_t_5) < 0) __PYX_ERR(0, 243, __pyx_L9_except_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_GOTREF(__pyx_t_5);

        /* "zeroconf/_protocol/incoming.py":247
 *                 # If the packet is really corrupt read_name and the unpack
 *                 # above would fail and hit the exception catch in read_others
 *                 self.offset = end             # <<<<<<<<<<<<<<
 *                 log.debug(
 *                     'Unable to parse; skipping record for %s with type %s at offset %d while unpacking %r',
 */
        __pyx_v_self->offset = __pyx_v_end;

        /* "zeroconf/_protocol/incoming.py":248
 *                 # above would fail and hit the exception catch in read_others
 *                 self.offset = end
 *                 log.debug(             # <<<<<<<<<<<<<<
 *                     'Unable to parse; skipping record for %s with type %s at offset %d while unpacking %r',
 *                     domain,
 */
        __Pyx_GetModuleGlobalName(__pyx_t_9, __pyx_n_s_log); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 248, __pyx_L9_except_error)
        __Pyx_GOTREF(__pyx_t_9);
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_n_s_debug); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 248, __pyx_L9_except_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

        /* "zeroconf/_protocol/incoming.py":251
 *                     'Unable to parse; skipping record for %s with type %s at offset %d while unpacking %r',
 *                     domain,
 *                     _TYPES.get(type_, type_),             # <<<<<<<<<<<<<<
 *                     self.offset,
 *                     self.data,
 */
        __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_n_s_TYPES); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 251, __pyx_L9_except_error)
        __Pyx_GOTREF(__pyx_t_10);
        __pyx_t_16 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_get); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 251, __pyx_L9_except_error)
        __Pyx_GOTREF(__pyx_t_16);
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        __pyx_t_10 = NULL;
        __pyx_t_8 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_16))) {
          __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_16);
          if (likely(__pyx_t_10)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_16);
            __Pyx_INCREF(__pyx_t_10);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_16, function);
            __pyx_t_8 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_16)) {
          PyObject *__pyx_temp[3] = {__pyx_t_10, __pyx_v_type_, __pyx_v_type_};
          __pyx_t_9 = __Pyx_PyFunction_FastCall(__pyx_t_16, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 251, __pyx_L9_except_error)
          __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_GOTREF(__pyx_t_9);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_16)) {
          PyObject *__pyx_temp[3] = {__pyx_t_10, __pyx_v_type_, __pyx_v_type_};
          __pyx_t_9 = __Pyx_PyCFunction_FastCall(__pyx_t_16, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 251, __pyx_L9_except_error)
          __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_GOTREF(__pyx_t_9);
        } else
        #endif
        {
          __pyx_t_17 = PyTuple_New(2+__pyx_t_8); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 251, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_17);
          if (__pyx_t_10) {
            __Pyx_GIVEREF(__pyx_t_10); PyTuple_SET_ITEM(__pyx_t_17, 0, __pyx_t_10); __pyx_t_10 = NULL;
          }
          __Pyx_INCREF(__pyx_v_type_);
          __Pyx_GIVEREF(__pyx_v_type_);
          PyTuple_SET_ITEM(__pyx_t_17, 0+__pyx_t_8, __pyx_v_type_);
          __Pyx_INCREF(__pyx_v_type_);
          __Pyx_GIVEREF(__pyx_v_type_);
          PyTuple_SET_ITEM(__pyx_t_17, 1+__pyx_t_8, __pyx_v_type_);
          __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_16, __pyx_t_17, NULL); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 251, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_9);
          __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
        }
        __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;

        /* "zeroconf/_protocol/incoming.py":252
 *                     domain,
 *                     _TYPES.get(type_, type_),
 *                     self.offset,             # <<<<<<<<<<<<<<
 *                     self.data,
 *                     exc_info=True,
 */
        __pyx_t_16 = __Pyx_PyInt_From_unsigned_int(__pyx_v_self->offset); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 252, __pyx_L9_except_error)
        __Pyx_GOTREF(__pyx_t_16);

        /* "zeroconf/_protocol/incoming.py":248
 *                 # above would fail and hit the exception catch in read_others
 *                 self.offset = end
 *                 log.debug(             # <<<<<<<<<<<<<<
 *                     'Unable to parse; skipping record for %s with type %s at offset %d while unpacking %r',
 *                     domain,
 */
        __pyx_t_17 = PyTuple_New(5); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 248, __pyx_L9_except_error)
        __Pyx_GOTREF(__pyx_t_17);
        __Pyx_INCREF(__pyx_kp_u_Unable_to_parse_skipping_record);
        __Pyx_GIVEREF(__pyx_kp_u_Unable_to_parse_skipping_record);
        PyTuple_SET_ITEM(__pyx_t_17, 0, __pyx_kp_u_Unable_to_parse_skipping_record);
        __Pyx_INCREF(__pyx_v_domain);
        __Pyx_GIVEREF(__pyx_v_domain);
        PyTuple_SET_ITEM(__pyx_t_17, 1, __pyx_v_domain);
        __Pyx_GIVEREF(__pyx_t_9);
        PyTuple_SET_ITEM(__pyx_t_17, 2, __pyx_t_9);
        __Pyx_GIVEREF(__pyx_t_16);
        PyTuple_SET_ITEM(__pyx_t_17, 3, __pyx_t_16);
        __Pyx_INCREF(__pyx_v_self->data);
        __Pyx_GIVEREF(__pyx_v_self->data);
        PyTuple_SET_ITEM(__pyx_t_17, 4, __pyx_v_self->data);
        __pyx_t_9 = 0;
        __pyx_t_16 = 0;

        /* "zeroconf/_protocol/incoming.py":254
 *                     self.offset,
 *                     self.data,
 *                     exc_info=True,             # <<<<<<<<<<<<<<
 *                 )
 *             if rec is not None:
 */
        __pyx_t_16 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 254, __pyx_L9_except_error)
        __Pyx_GOTREF(__pyx_t_16);
        if (PyDict_SetItem(__pyx_t_16, __pyx_n_s_exc_info, Py_True) < 0) __PYX_ERR(0, 254, __pyx_L9_except_error)

        /* "zeroconf/_protocol/incoming.py":248
 *                 # above would fail and hit the exception catch in read_others
 *                 self.offset = end
 *                 log.debug(             # <<<<<<<<<<<<<<
 *                     'Unable to parse; skipping record for %s with type %s at offset %d while unpacking %r',
 *                     domain,
 */
        __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_17, __pyx_t_16); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 248, __pyx_L9_except_error)
        __Pyx_GOTREF(__pyx_t_9);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
        __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        goto __pyx_L8_exception_handled;
      }
      goto __pyx_L9_except_error;
      __pyx_L9_except_error:;

      /* "zeroconf/_protocol/incoming.py":241
 *             end = self.offset + length
 *             rec = None
 *             try:             # <<<<<<<<<<<<<<
 *                 rec = self._read_record(domain, type_, class_, ttl, length)
 *             except DECODE_EXCEPTIONS:
 */
      __Pyx_XGIVEREF(__pyx_t_13);
      __Pyx_XGIVEREF(__pyx_t_14);
      __Pyx_XGIVEREF(__pyx_t_15);
      __Pyx_ExceptionReset(__pyx_t_13, __pyx_t_14, __pyx_t_15);
      goto __pyx_L1_error;
      __pyx_L8_exception_handled:;
      __Pyx_XGIVEREF(__pyx_t_13);
      __Pyx_XGIVEREF(__pyx_t_14);
      __Pyx_XGIVEREF(__pyx_t_15);
      __Pyx_ExceptionReset(__pyx_t_13, __pyx_t_14, __pyx_t_15);
      __pyx_L14_try_end:;
    }

    /* "zeroconf/_protocol/incoming.py":256
 *                     exc_info=True,
 *                 )
 *             if rec is not None:             # <<<<<<<<<<<<<<
 *                 self._answers.append(rec)
 * 
 */
    __pyx_t_18 = (__pyx_v_rec != Py_None);
    __pyx_t_19 = (__pyx_t_18 != 0);
    if (__pyx_t_19) {

      /* "zeroconf/_protocol/incoming.py":257
 *                 )
 *             if rec is not None:
 *                 self._answers.append(rec)             # <<<<<<<<<<<<<<
 * 
 *     def _read_record(
 */
      if (unlikely(__pyx_v_self->_answers == Py_None)) {
        PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
        __PYX_ERR(0, 257, __pyx_L1_error)
      }
      __pyx_t_20 = __Pyx_PyList_Append(__pyx_v_self->_answers, __pyx_v_rec); if (unlikely(__pyx_t_20 == ((int)-1))) __PYX_ERR(0, 257, __pyx_L1_error)

      /* "zeroconf/_protocol/incoming.py":256
 *                     exc_info=True,
 *                 )
 *             if rec is not None:             # <<<<<<<<<<<<<<
 *                 self._answers.append(rec)
 * 
 */
    }
  }

  /* "zeroconf/_protocol/incoming.py":230
 *         return info
 * 
 *     def _read_others(self) -> None:             # <<<<<<<<<<<<<<
 *         """Reads the answers, authorities and additionals section of the
 *         packet"""
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_AddTraceback("zeroconf._protocol.incoming.DNSIncoming._read_others", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_domain);
  __Pyx_XDECREF(__pyx_v_type_);
  __Pyx_XDECREF(__pyx_v_class_);
  __Pyx_XDECREF(__pyx_v_ttl);
  __Pyx_XDECREF(__pyx_v_rec);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/incoming.py":259
 *                 self._answers.append(rec)
 * 
 *     def _read_record(             # <<<<<<<<<<<<<<
 *         self, domain: _str, type_: int, class_: int, ttl: int, length: int
 *     ) -> Optional[DNSRecord]:
 */

static PyObject *__pyx_f_8zeroconf_9_protocol_8incoming_11DNSIncoming__read_record(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self, PyObject *__pyx_v_domain, unsigned int __pyx_v_type_, PyObject *__pyx_v_class_, PyObject *__pyx_v_ttl, unsigned int __pyx_v_length) {
  unsigned int __pyx_v_name_start;
  struct __pyx_obj_8zeroconf_4_dns_DNSAddress *__pyx_v_dns_address = NULL;
  PyObject *__pyx_v_priority = NULL;
  PyObject *__pyx_v_weight = NULL;
  PyObject *__pyx_v_port = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  unsigned int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  PyObject *(*__pyx_t_10)(PyObject *);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_read_record", 0);

  /* "zeroconf/_protocol/incoming.py":263
 *     ) -> Optional[DNSRecord]:
 *         """Read known records types and skip unknown ones."""
 *         if type_ == _TYPE_A:             # <<<<<<<<<<<<<<
 *             dns_address = DNSAddress(domain, type_, class_, ttl, self._read_string(4))
 *             dns_address.created = self.now
 */
  __pyx_t_1 = ((__pyx_v_type_ == __pyx_v_8zeroconf_9_protocol_8incoming__TYPE_A) != 0);
  if (__pyx_t_1) {

    /* "zeroconf/_protocol/incoming.py":264
 *         """Read known records types and skip unknown ones."""
 *         if type_ == _TYPE_A:
 *             dns_address = DNSAddress(domain, type_, class_, ttl, self._read_string(4))             # <<<<<<<<<<<<<<
 *             dns_address.created = self.now
 *             return dns_address
 */
    __pyx_t_2 = __Pyx_PyInt_From_unsigned_int(__pyx_v_type_); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 264, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = ((struct __pyx_vtabstruct_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self->__pyx_vtab)->_read_string(__pyx_v_self, 4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 264, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_New(5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 264, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_v_domain);
    __Pyx_GIVEREF(__pyx_v_domain);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_domain);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_2);
    __Pyx_INCREF(__pyx_v_class_);
    __Pyx_GIVEREF(__pyx_v_class_);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_class_);
    __Pyx_INCREF(__pyx_v_ttl);
    __Pyx_GIVEREF(__pyx_v_ttl);
    PyTuple_SET_ITEM(__pyx_t_4, 3, __pyx_v_ttl);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_4, 4, __pyx_t_3);
    __pyx_t_2 = 0;
    __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_8zeroconf_4_dns_DNSAddress), __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 264, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_dns_address = ((struct __pyx_obj_8zeroconf_4_dns_DNSAddress *)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "zeroconf/_protocol/incoming.py":265
 *         if type_ == _TYPE_A:
 *             dns_address = DNSAddress(domain, type_, class_, ttl, self._read_string(4))
 *             dns_address.created = self.now             # <<<<<<<<<<<<<<
 *             return dns_address
 *         if type_ in (_TYPE_CNAME, _TYPE_PTR):
 */
    __pyx_t_3 = __pyx_v_self->now;
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    __Pyx_GOTREF(__pyx_v_dns_address->__pyx_base.created);
    __Pyx_DECREF(__pyx_v_dns_address->__pyx_base.created);
    __pyx_v_dns_address->__pyx_base.created = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "zeroconf/_protocol/incoming.py":266
 *             dns_address = DNSAddress(domain, type_, class_, ttl, self._read_string(4))
 *             dns_address.created = self.now
 *             return dns_address             # <<<<<<<<<<<<<<
 *         if type_ in (_TYPE_CNAME, _TYPE_PTR):
 *             return DNSPointer(domain, type_, class_, ttl, self._read_name(), self.now)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject *)__pyx_v_dns_address));
    __pyx_r = ((PyObject *)__pyx_v_dns_address);
    goto __pyx_L0;

    /* "zeroconf/_protocol/incoming.py":263
 *     ) -> Optional[DNSRecord]:
 *         """Read known records types and skip unknown ones."""
 *         if type_ == _TYPE_A:             # <<<<<<<<<<<<<<
 *             dns_address = DNSAddress(domain, type_, class_, ttl, self._read_string(4))
 *             dns_address.created = self.now
 */
  }

  /* "zeroconf/_protocol/incoming.py":267
 *             dns_address.created = self.now
 *             return dns_address
 *         if type_ in (_TYPE_CNAME, _TYPE_PTR):             # <<<<<<<<<<<<<<
 *             return DNSPointer(domain, type_, class_, ttl, self._read_name(), self.now)
 *         if type_ == _TYPE_TXT:
 */
  __pyx_t_5 = __pyx_v_type_;
  __pyx_t_6 = ((__pyx_t_5 == __pyx_v_8zeroconf_9_protocol_8incoming__TYPE_CNAME) != 0);
  if (!__pyx_t_6) {
  } else {
    __pyx_t_1 = __pyx_t_6;
    goto __pyx_L5_bool_binop_done;
  }
  __pyx_t_6 = ((__pyx_t_5 == __pyx_v_8zeroconf_9_protocol_8incoming__TYPE_PTR) != 0);
  __pyx_t_1 = __pyx_t_6;
  __pyx_L5_bool_binop_done:;
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "zeroconf/_protocol/incoming.py":268
 *             return dns_address
 *         if type_ in (_TYPE_CNAME, _TYPE_PTR):
 *             return DNSPointer(domain, type_, class_, ttl, self._read_name(), self.now)             # <<<<<<<<<<<<<<
 *         if type_ == _TYPE_TXT:
 *             return DNSText(domain, type_, class_, ttl, self._read_string(length), self.now)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyInt_From_unsigned_int(__pyx_v_type_); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 268, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = ((struct __pyx_vtabstruct_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self->__pyx_vtab)->_read_name(__pyx_v_self); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 268, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_2 = PyTuple_New(6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 268, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_v_domain);
    __Pyx_GIVEREF(__pyx_v_domain);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_domain);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_3);
    __Pyx_INCREF(__pyx_v_class_);
    __Pyx_GIVEREF(__pyx_v_class_);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_class_);
    __Pyx_INCREF(__pyx_v_ttl);
    __Pyx_GIVEREF(__pyx_v_ttl);
    PyTuple_SET_ITEM(__pyx_t_2, 3, __pyx_v_ttl);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_2, 4, __pyx_t_4);
    __Pyx_INCREF(__pyx_v_self->now);
    __Pyx_GIVEREF(__pyx_v_self->now);
    PyTuple_SET_ITEM(__pyx_t_2, 5, __pyx_v_self->now);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_8zeroconf_4_dns_DNSPointer), __pyx_t_2, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 268, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "zeroconf/_protocol/incoming.py":267
 *             dns_address.created = self.now
 *             return dns_address
 *         if type_ in (_TYPE_CNAME, _TYPE_PTR):             # <<<<<<<<<<<<<<
 *             return DNSPointer(domain, type_, class_, ttl, self._read_name(), self.now)
 *         if type_ == _TYPE_TXT:
 */
  }

  /* "zeroconf/_protocol/incoming.py":269
 *         if type_ in (_TYPE_CNAME, _TYPE_PTR):
 *             return DNSPointer(domain, type_, class_, ttl, self._read_name(), self.now)
 *         if type_ == _TYPE_TXT:             # <<<<<<<<<<<<<<
 *             return DNSText(domain, type_, class_, ttl, self._read_string(length), self.now)
 *         if type_ == _TYPE_SRV:
 */
  __pyx_t_6 = ((__pyx_v_type_ == __pyx_v_8zeroconf_9_protocol_8incoming__TYPE_TXT) != 0);
  if (__pyx_t_6) {

    /* "zeroconf/_protocol/incoming.py":270
 *             return DNSPointer(domain, type_, class_, ttl, self._read_name(), self.now)
 *         if type_ == _TYPE_TXT:
 *             return DNSText(domain, type_, class_, ttl, self._read_string(length), self.now)             # <<<<<<<<<<<<<<
 *         if type_ == _TYPE_SRV:
 *             priority, weight, port = UNPACK_3H(self.data, self.offset)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_4 = __Pyx_PyInt_From_unsigned_int(__pyx_v_type_); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 270, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_2 = ((struct __pyx_vtabstruct_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self->__pyx_vtab)->_read_string(__pyx_v_self, __pyx_v_length); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 270, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyTuple_New(6); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 270, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v_domain);
    __Pyx_GIVEREF(__pyx_v_domain);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_domain);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_4);
    __Pyx_INCREF(__pyx_v_class_);
    __Pyx_GIVEREF(__pyx_v_class_);
    PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_v_class_);
    __Pyx_INCREF(__pyx_v_ttl);
    __Pyx_GIVEREF(__pyx_v_ttl);
    PyTuple_SET_ITEM(__pyx_t_3, 3, __pyx_v_ttl);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_3, 4, __pyx_t_2);
    __Pyx_INCREF(__pyx_v_self->now);
    __Pyx_GIVEREF(__pyx_v_self->now);
    PyTuple_SET_ITEM(__pyx_t_3, 5, __pyx_v_self->now);
    __pyx_t_4 = 0;
    __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_8zeroconf_4_dns_DNSText), __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 270, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "zeroconf/_protocol/incoming.py":269
 *         if type_ in (_TYPE_CNAME, _TYPE_PTR):
 *             return DNSPointer(domain, type_, class_, ttl, self._read_name(), self.now)
 *         if type_ == _TYPE_TXT:             # <<<<<<<<<<<<<<
 *             return DNSText(domain, type_, class_, ttl, self._read_string(length), self.now)
 *         if type_ == _TYPE_SRV:
 */
  }

  /* "zeroconf/_protocol/incoming.py":271
 *         if type_ == _TYPE_TXT:
 *             return DNSText(domain, type_, class_, ttl, self._read_string(length), self.now)
 *         if type_ == _TYPE_SRV:             # <<<<<<<<<<<<<<
 *             priority, weight, port = UNPACK_3H(self.data, self.offset)
 *             self.offset += 6
 */
  __pyx_t_6 = ((__pyx_v_type_ == __pyx_v_8zeroconf_9_protocol_8incoming__TYPE_SRV) != 0);
  if (__pyx_t_6) {

    /* "zeroconf/_protocol/incoming.py":272
 *             return DNSText(domain, type_, class_, ttl, self._read_string(length), self.now)
 *         if type_ == _TYPE_SRV:
 *             priority, weight, port = UNPACK_3H(self.data, self.offset)             # <<<<<<<<<<<<<<
 *             self.offset += 6
 *             return DNSService(
 */
    __pyx_t_3 = __Pyx_PyInt_From_unsigned_int(__pyx_v_self->offset); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 272, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v_8zeroconf_9_protocol_8incoming_UNPACK_3H);
    __pyx_t_4 = __pyx_v_8zeroconf_9_protocol_8incoming_UNPACK_3H; __pyx_t_7 = NULL;
    __pyx_t_8 = 0;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_7)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
        __pyx_t_8 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[3] = {__pyx_t_7, __pyx_v_self->data, __pyx_t_3};
      __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 272, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[3] = {__pyx_t_7, __pyx_v_self->data, __pyx_t_3};
      __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 272, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    } else
    #endif
    {
      __pyx_t_9 = PyTuple_New(2+__pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 272, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      if (__pyx_t_7) {
        __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_7); __pyx_t_7 = NULL;
      }
      __Pyx_INCREF(__pyx_v_self->data);
      __Pyx_GIVEREF(__pyx_v_self->data);
      PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_8, __pyx_v_self->data);
      __Pyx_GIVEREF(__pyx_t_3);
      PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_8, __pyx_t_3);
      __pyx_t_3 = 0;
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_9, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 272, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if ((likely(PyTuple_CheckExact(__pyx_t_2))) || (PyList_CheckExact(__pyx_t_2))) {
      PyObject* sequence = __pyx_t_2;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 3)) {
        if (size > 3) __Pyx_RaiseTooManyValuesError(3);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 272, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_4 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_9 = PyTuple_GET_ITEM(sequence, 1); 
        __pyx_t_3 = PyTuple_GET_ITEM(sequence, 2); 
      } else {
        __pyx_t_4 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_9 = PyList_GET_ITEM(sequence, 1); 
        __pyx_t_3 = PyList_GET_ITEM(sequence, 2); 
      }
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_9);
      __Pyx_INCREF(__pyx_t_3);
      #else
      __pyx_t_4 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 272, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_9 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 272, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_3 = PySequence_ITEM(sequence, 2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 272, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      #endif
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_7 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 272, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_10 = Py_TYPE(__pyx_t_7)->tp_iternext;
      index = 0; __pyx_t_4 = __pyx_t_10(__pyx_t_7); if (unlikely(!__pyx_t_4)) goto __pyx_L9_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_4);
      index = 1; __pyx_t_9 = __pyx_t_10(__pyx_t_7); if (unlikely(!__pyx_t_9)) goto __pyx_L9_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_9);
      index = 2; __pyx_t_3 = __pyx_t_10(__pyx_t_7); if (unlikely(!__pyx_t_3)) goto __pyx_L9_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_3);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_10(__pyx_t_7), 3) < 0) __PYX_ERR(0, 272, __pyx_L1_error)
      __pyx_t_10 = NULL;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      goto __pyx_L10_unpacking_done;
      __pyx_L9_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_10 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 272, __pyx_L1_error)
      __pyx_L10_unpacking_done:;
    }
    __pyx_v_priority = __pyx_t_4;
    __pyx_t_4 = 0;
    __pyx_v_weight = __pyx_t_9;
    __pyx_t_9 = 0;
    __pyx_v_port = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "zeroconf/_protocol/incoming.py":273
 *         if type_ == _TYPE_SRV:
 *             priority, weight, port = UNPACK_3H(self.data, self.offset)
 *             self.offset += 6             # <<<<<<<<<<<<<<
 *             return DNSService(
 *                 domain,
 */
    __pyx_v_self->offset = (__pyx_v_self->offset + 6);

    /* "zeroconf/_protocol/incoming.py":274
 *             priority, weight, port = UNPACK_3H(self.data, self.offset)
 *             self.offset += 6
 *             return DNSService(             # <<<<<<<<<<<<<<
 *                 domain,
 *                 type_,
 */
    __Pyx_XDECREF(__pyx_r);

    /* "zeroconf/_protocol/incoming.py":276
 *             return DNSService(
 *                 domain,
 *                 type_,             # <<<<<<<<<<<<<<
 *                 class_,
 *                 ttl,
 */
    __pyx_t_2 = __Pyx_PyInt_From_unsigned_int(__pyx_v_type_); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 276, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);

    /* "zeroconf/_protocol/incoming.py":282
 *                 weight,
 *                 port,
 *                 self._read_name(),             # <<<<<<<<<<<<<<
 *                 self.now,
 *             )
 */
    __pyx_t_3 = ((struct __pyx_vtabstruct_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self->__pyx_vtab)->_read_name(__pyx_v_self); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 282, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);

    /* "zeroconf/_protocol/incoming.py":274
 *             priority, weight, port = UNPACK_3H(self.data, self.offset)
 *             self.offset += 6
 *             return DNSService(             # <<<<<<<<<<<<<<
 *                 domain,
 *                 type_,
 */
    __pyx_t_9 = PyTuple_New(9); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 274, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_INCREF(__pyx_v_domain);
    __Pyx_GIVEREF(__pyx_v_domain);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_v_domain);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_t_2);
    __Pyx_INCREF(__pyx_v_class_);
    __Pyx_GIVEREF(__pyx_v_class_);
    PyTuple_SET_ITEM(__pyx_t_9, 2, __pyx_v_class_);
    __Pyx_INCREF(__pyx_v_ttl);
    __Pyx_GIVEREF(__pyx_v_ttl);
    PyTuple_SET_ITEM(__pyx_t_9, 3, __pyx_v_ttl);
    __Pyx_INCREF(__pyx_v_priority);
    __Pyx_GIVEREF(__pyx_v_priority);
    PyTuple_SET_ITEM(__pyx_t_9, 4, __pyx_v_priority);
    __Pyx_INCREF(__pyx_v_weight);
    __Pyx_GIVEREF(__pyx_v_weight);
    PyTuple_SET_ITEM(__pyx_t_9, 5, __pyx_v_weight);
    __Pyx_INCREF(__pyx_v_port);
    __Pyx_GIVEREF(__pyx_v_port);
    PyTuple_SET_ITEM(__pyx_t_9, 6, __pyx_v_port);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_9, 7, __pyx_t_3);
    __Pyx_INCREF(__pyx_v_self->now);
    __Pyx_GIVEREF(__pyx_v_self->now);
    PyTuple_SET_ITEM(__pyx_t_9, 8, __pyx_v_self->now);
    __pyx_t_2 = 0;
    __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_8zeroconf_4_dns_DNSService), __pyx_t_9, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 274, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "zeroconf/_protocol/incoming.py":271
 *         if type_ == _TYPE_TXT:
 *             return DNSText(domain, type_, class_, ttl, self._read_string(length), self.now)
 *         if type_ == _TYPE_SRV:             # <<<<<<<<<<<<<<
 *             priority, weight, port = UNPACK_3H(self.data, self.offset)
 *             self.offset += 6
 */
  }

  /* "zeroconf/_protocol/incoming.py":285
 *                 self.now,
 *             )
 *         if type_ == _TYPE_HINFO:             # <<<<<<<<<<<<<<
 *             return DNSHinfo(
 *                 domain,
 */
  __pyx_t_6 = ((__pyx_v_type_ == __pyx_v_8zeroconf_9_protocol_8incoming__TYPE_HINFO) != 0);
  if (__pyx_t_6) {

    /* "zeroconf/_protocol/incoming.py":286
 *             )
 *         if type_ == _TYPE_HINFO:
 *             return DNSHinfo(             # <<<<<<<<<<<<<<
 *                 domain,
 *                 type_,
 */
    __Pyx_XDECREF(__pyx_r);

    /* "zeroconf/_protocol/incoming.py":288
 *             return DNSHinfo(
 *                 domain,
 *                 type_,             # <<<<<<<<<<<<<<
 *                 class_,
 *                 ttl,
 */
    __pyx_t_3 = __Pyx_PyInt_From_unsigned_int(__pyx_v_type_); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 288, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);

    /* "zeroconf/_protocol/incoming.py":291
 *                 class_,
 *                 ttl,
 *                 self._read_character_string().decode('utf-8', 'replace'),             # <<<<<<<<<<<<<<
 *                 self._read_character_string().decode('utf-8', 'replace'),
 *                 self.now,
 */
    __pyx_t_9 = ((struct __pyx_vtabstruct_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self->__pyx_vtab)->_read_character_string(__pyx_v_self); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 291, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    if (unlikely(__pyx_t_9 == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "decode");
      __PYX_ERR(0, 291, __pyx_L1_error)
    }
    __pyx_t_2 = __Pyx_decode_bytes(__pyx_t_9, 0, PY_SSIZE_T_MAX, NULL, ((char const *)"replace"), PyUnicode_DecodeUTF8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 291, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

    /* "zeroconf/_protocol/incoming.py":292
 *                 ttl,
 *                 self._read_character_string().decode('utf-8', 'replace'),
 *                 self._read_character_string().decode('utf-8', 'replace'),             # <<<<<<<<<<<<<<
 *                 self.now,
 *             )
 */
    __pyx_t_9 = ((struct __pyx_vtabstruct_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self->__pyx_vtab)->_read_character_string(__pyx_v_self); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 292, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    if (unlikely(__pyx_t_9 == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "decode");
      __PYX_ERR(0, 292, __pyx_L1_error)
    }
    __pyx_t_4 = __Pyx_decode_bytes(__pyx_t_9, 0, PY_SSIZE_T_MAX, NULL, ((char const *)"replace"), PyUnicode_DecodeUTF8); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 292, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

    /* "zeroconf/_protocol/incoming.py":286
 *             )
 *         if type_ == _TYPE_HINFO:
 *             return DNSHinfo(             # <<<<<<<<<<<<<<
 *                 domain,
 *                 type_,
 */
    __pyx_t_9 = PyTuple_New(7); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 286, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_INCREF(__pyx_v_domain);
    __Pyx_GIVEREF(__pyx_v_domain);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_v_domain);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_t_3);
    __Pyx_INCREF(__pyx_v_class_);
    __Pyx_GIVEREF(__pyx_v_class_);
    PyTuple_SET_ITEM(__pyx_t_9, 2, __pyx_v_class_);
    __Pyx_INCREF(__pyx_v_ttl);
    __Pyx_GIVEREF(__pyx_v_ttl);
    PyTuple_SET_ITEM(__pyx_t_9, 3, __pyx_v_ttl);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_9, 4, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_9, 5, __pyx_t_4);
    __Pyx_INCREF(__pyx_v_self->now);
    __Pyx_GIVEREF(__pyx_v_self->now);
    PyTuple_SET_ITEM(__pyx_t_9, 6, __pyx_v_self->now);
    __pyx_t_3 = 0;
    __pyx_t_2 = 0;
    __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_8zeroconf_4_dns_DNSHinfo), __pyx_t_9, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 286, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "zeroconf/_protocol/incoming.py":285
 *                 self.now,
 *             )
 *         if type_ == _TYPE_HINFO:             # <<<<<<<<<<<<<<
 *             return DNSHinfo(
 *                 domain,
 */
  }

  /* "zeroconf/_protocol/incoming.py":295
 *                 self.now,
 *             )
 *         if type_ == _TYPE_AAAA:             # <<<<<<<<<<<<<<
 *             dns_address = DNSAddress(domain, type_, class_, ttl, self._read_string(16))
 *             dns_address.created = self.now
 */
  __pyx_t_6 = ((__pyx_v_type_ == __pyx_v_8zeroconf_9_protocol_8incoming__TYPE_AAAA) != 0);
  if (__pyx_t_6) {

    /* "zeroconf/_protocol/incoming.py":296
 *             )
 *         if type_ == _TYPE_AAAA:
 *             dns_address = DNSAddress(domain, type_, class_, ttl, self._read_string(16))             # <<<<<<<<<<<<<<
 *             dns_address.created = self.now
 *             dns_address.scope_id = self.scope_id
 */
    __pyx_t_4 = __Pyx_PyInt_From_unsigned_int(__pyx_v_type_); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 296, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_9 = ((struct __pyx_vtabstruct_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self->__pyx_vtab)->_read_string(__pyx_v_self, 16); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 296, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_2 = PyTuple_New(5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 296, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_v_domain);
    __Pyx_GIVEREF(__pyx_v_domain);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_domain);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_4);
    __Pyx_INCREF(__pyx_v_class_);
    __Pyx_GIVEREF(__pyx_v_class_);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_class_);
    __Pyx_INCREF(__pyx_v_ttl);
    __Pyx_GIVEREF(__pyx_v_ttl);
    PyTuple_SET_ITEM(__pyx_t_2, 3, __pyx_v_ttl);
    __Pyx_GIVEREF(__pyx_t_9);
    PyTuple_SET_ITEM(__pyx_t_2, 4, __pyx_t_9);
    __pyx_t_4 = 0;
    __pyx_t_9 = 0;
    __pyx_t_9 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_8zeroconf_4_dns_DNSAddress), __pyx_t_2, NULL); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 296, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_dns_address = ((struct __pyx_obj_8zeroconf_4_dns_DNSAddress *)__pyx_t_9);
    __pyx_t_9 = 0;

    /* "zeroconf/_protocol/incoming.py":297
 *         if type_ == _TYPE_AAAA:
 *             dns_address = DNSAddress(domain, type_, class_, ttl, self._read_string(16))
 *             dns_address.created = self.now             # <<<<<<<<<<<<<<
 *             dns_address.scope_id = self.scope_id
 *             return dns_address
 */
    __pyx_t_9 = __pyx_v_self->now;
    __Pyx_INCREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_9);
    __Pyx_GOTREF(__pyx_v_dns_address->__pyx_base.created);
    __Pyx_DECREF(__pyx_v_dns_address->__pyx_base.created);
    __pyx_v_dns_address->__pyx_base.created = __pyx_t_9;
    __pyx_t_9 = 0;

    /* "zeroconf/_protocol/incoming.py":298
 *             dns_address = DNSAddress(domain, type_, class_, ttl, self._read_string(16))
 *             dns_address.created = self.now
 *             dns_address.scope_id = self.scope_id             # <<<<<<<<<<<<<<
 *             return dns_address
 *         if type_ == _TYPE_NSEC:
 */
    __pyx_t_9 = __pyx_v_self->scope_id;
    __Pyx_INCREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_9);
    __Pyx_GOTREF(__pyx_v_dns_address->scope_id);
    __Pyx_DECREF(__pyx_v_dns_address->scope_id);
    __pyx_v_dns_address->scope_id = __pyx_t_9;
    __pyx_t_9 = 0;

    /* "zeroconf/_protocol/incoming.py":299
 *             dns_address.created = self.now
 *             dns_address.scope_id = self.scope_id
 *             return dns_address             # <<<<<<<<<<<<<<
 *         if type_ == _TYPE_NSEC:
 *             name_start = self.offset
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject *)__pyx_v_dns_address));
    __pyx_r = ((PyObject *)__pyx_v_dns_address);
    goto __pyx_L0;

    /* "zeroconf/_protocol/incoming.py":295
 *                 self.now,
 *             )
 *         if type_ == _TYPE_AAAA:             # <<<<<<<<<<<<<<
 *             dns_address = DNSAddress(domain, type_, class_, ttl, self._read_string(16))
 *             dns_address.created = self.now
 */
  }

  /* "zeroconf/_protocol/incoming.py":300
 *             dns_address.scope_id = self.scope_id
 *             return dns_address
 *         if type_ == _TYPE_NSEC:             # <<<<<<<<<<<<<<
 *             name_start = self.offset
 *             return DNSNsec(
 */
  __pyx_t_6 = ((__pyx_v_type_ == __pyx_v_8zeroconf_9_protocol_8incoming__TYPE_NSEC) != 0);
  if (__pyx_t_6) {

    /* "zeroconf/_protocol/incoming.py":301
 *             return dns_address
 *         if type_ == _TYPE_NSEC:
 *             name_start = self.offset             # <<<<<<<<<<<<<<
 *             return DNSNsec(
 *                 domain,
 */
    __pyx_t_5 = __pyx_v_self->offset;
    __pyx_v_name_start = __pyx_t_5;

    /* "zeroconf/_protocol/incoming.py":302
 *         if type_ == _TYPE_NSEC:
 *             name_start = self.offset
 *             return DNSNsec(             # <<<<<<<<<<<<<<
 *                 domain,
 *                 type_,
 */
    __Pyx_XDECREF(__pyx_r);

    /* "zeroconf/_protocol/incoming.py":304
 *             return DNSNsec(
 *                 domain,
 *                 type_,             # <<<<<<<<<<<<<<
 *                 class_,
 *                 ttl,
 */
    __pyx_t_9 = __Pyx_PyInt_From_unsigned_int(__pyx_v_type_); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 304, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);

    /* "zeroconf/_protocol/incoming.py":307
 *                 class_,
 *                 ttl,
 *                 self._read_name(),             # <<<<<<<<<<<<<<
 *                 self._read_bitmap(name_start + length),
 *                 self.now,
 */
    __pyx_t_2 = ((struct __pyx_vtabstruct_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self->__pyx_vtab)->_read_name(__pyx_v_self); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 307, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);

    /* "zeroconf/_protocol/incoming.py":308
 *                 ttl,
 *                 self._read_name(),
 *                 self._read_bitmap(name_start + length),             # <<<<<<<<<<<<<<
 *                 self.now,
 *             )
 */
    __pyx_t_4 = ((struct __pyx_vtabstruct_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self->__pyx_vtab)->_read_bitmap(__pyx_v_self, (__pyx_v_name_start + __pyx_v_length)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 308, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);

    /* "zeroconf/_protocol/incoming.py":302
 *         if type_ == _TYPE_NSEC:
 *             name_start = self.offset
 *             return DNSNsec(             # <<<<<<<<<<<<<<
 *                 domain,
 *                 type_,
 */
    __pyx_t_3 = PyTuple_New(7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 302, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v_domain);
    __Pyx_GIVEREF(__pyx_v_domain);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_domain);
    __Pyx_GIVEREF(__pyx_t_9);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_9);
    __Pyx_INCREF(__pyx_v_class_);
    __Pyx_GIVEREF(__pyx_v_class_);
    PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_v_class_);
    __Pyx_INCREF(__pyx_v_ttl);
    __Pyx_GIVEREF(__pyx_v_ttl);
    PyTuple_SET_ITEM(__pyx_t_3, 3, __pyx_v_ttl);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_3, 4, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_3, 5, __pyx_t_4);
    __Pyx_INCREF(__pyx_v_self->now);
    __Pyx_GIVEREF(__pyx_v_self->now);
    PyTuple_SET_ITEM(__pyx_t_3, 6, __pyx_v_self->now);
    __pyx_t_9 = 0;
    __pyx_t_2 = 0;
    __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_8zeroconf_4_dns_DNSNsec), __pyx_t_3, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 302, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "zeroconf/_protocol/incoming.py":300
 *             dns_address.scope_id = self.scope_id
 *             return dns_address
 *         if type_ == _TYPE_NSEC:             # <<<<<<<<<<<<<<
 *             name_start = self.offset
 *             return DNSNsec(
 */
  }

  /* "zeroconf/_protocol/incoming.py":314
 *         # Skip the payload for the resource record so the next
 *         # records can be parsed correctly
 *         self.offset += length             # <<<<<<<<<<<<<<
 *         return None
 * 
 */
  __pyx_v_self->offset = (__pyx_v_self->offset + __pyx_v_length);

  /* "zeroconf/_protocol/incoming.py":315
 *         # records can be parsed correctly
 *         self.offset += length
 *         return None             # <<<<<<<<<<<<<<
 * 
 *     def _read_bitmap(self, end: int) -> List[int]:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;

  /* "zeroconf/_protocol/incoming.py":259
 *                 self._answers.append(rec)
 * 
 *     def _read_record(             # <<<<<<<<<<<<<<
 *         self, domain: _str, type_: int, class_: int, ttl: int, length: int
 *     ) -> Optional[DNSRecord]:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("zeroconf._protocol.incoming.DNSIncoming._read_record", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_dns_address);
  __Pyx_XDECREF(__pyx_v_priority);
  __Pyx_XDECREF(__pyx_v_weight);
  __Pyx_XDECREF(__pyx_v_port);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/incoming.py":317
 *         return None
 * 
 *     def _read_bitmap(self, end: int) -> List[int]:             # <<<<<<<<<<<<<<
 *         """Reads an NSEC bitmap from the packet."""
 *         rdtypes = []
 */

static PyObject *__pyx_f_8zeroconf_9_protocol_8incoming_11DNSIncoming__read_bitmap(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self, unsigned int __pyx_v_end) {
  unsigned int __pyx_v_offset;
  unsigned int __pyx_v_offset_plus_one;
  unsigned int __pyx_v_offset_plus_two;
  unsigned int __pyx_v_window;
  unsigned int __pyx_v_bit;
  unsigned int __pyx_v_byte;
  unsigned int __pyx_v_i;
  unsigned int __pyx_v_bitmap_length;
  PyObject *__pyx_v_rdtypes = NULL;
  unsigned int __pyx_v_bitmap_end;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  unsigned int __pyx_t_3;
  char *__pyx_t_4;
  char *__pyx_t_5;
  char *__pyx_t_6;
  char *__pyx_t_7;
  unsigned int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_read_bitmap", 0);

  /* "zeroconf/_protocol/incoming.py":319
 *     def _read_bitmap(self, end: int) -> List[int]:
 *         """Reads an NSEC bitmap from the packet."""
 *         rdtypes = []             # <<<<<<<<<<<<<<
 *         while self.offset < end:
 *             offset = self.offset
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 319, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_rdtypes = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "zeroconf/_protocol/incoming.py":320
 *         """Reads an NSEC bitmap from the packet."""
 *         rdtypes = []
 *         while self.offset < end:             # <<<<<<<<<<<<<<
 *             offset = self.offset
 *             offset_plus_one = offset + 1
 */
  while (1) {
    __pyx_t_2 = ((__pyx_v_self->offset < __pyx_v_end) != 0);
    if (!__pyx_t_2) break;

    /* "zeroconf/_protocol/incoming.py":321
 *         rdtypes = []
 *         while self.offset < end:
 *             offset = self.offset             # <<<<<<<<<<<<<<
 *             offset_plus_one = offset + 1
 *             offset_plus_two = offset + 2
 */
    __pyx_t_3 = __pyx_v_self->offset;
    __pyx_v_offset = __pyx_t_3;

    /* "zeroconf/_protocol/incoming.py":322
 *         while self.offset < end:
 *             offset = self.offset
 *             offset_plus_one = offset + 1             # <<<<<<<<<<<<<<
 *             offset_plus_two = offset + 2
 *             window = self.data[offset]
 */
    __pyx_v_offset_plus_one = (__pyx_v_offset + 1);

    /* "zeroconf/_protocol/incoming.py":323
 *             offset = self.offset
 *             offset_plus_one = offset + 1
 *             offset_plus_two = offset + 2             # <<<<<<<<<<<<<<
 *             window = self.data[offset]
 *             bitmap_length = self.data[offset_plus_one]
 */
    __pyx_v_offset_plus_two = (__pyx_v_offset + 2);

    /* "zeroconf/_protocol/incoming.py":324
 *             offset_plus_one = offset + 1
 *             offset_plus_two = offset + 2
 *             window = self.data[offset]             # <<<<<<<<<<<<<<
 *             bitmap_length = self.data[offset_plus_one]
 *             bitmap_end = offset_plus_two + bitmap_length
 */
    __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_self->data, __pyx_v_offset, unsigned int, 0, __Pyx_PyInt_From_unsigned_int, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 324, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyInt_As_unsigned_int(__pyx_t_1); if (unlikely((__pyx_t_3 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(0, 324, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_window = __pyx_t_3;

    /* "zeroconf/_protocol/incoming.py":325
 *             offset_plus_two = offset + 2
 *             window = self.data[offset]
 *             bitmap_length = self.data[offset_plus_one]             # <<<<<<<<<<<<<<
 *             bitmap_end = offset_plus_two + bitmap_length
 *             for i, byte in enumerate(self.data[offset_plus_two:bitmap_end]):
 */
    __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_self->data, __pyx_v_offset_plus_one, unsigned int, 0, __Pyx_PyInt_From_unsigned_int, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 325, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyInt_As_unsigned_int(__pyx_t_1); if (unlikely((__pyx_t_3 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(0, 325, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_bitmap_length = __pyx_t_3;

    /* "zeroconf/_protocol/incoming.py":326
 *             window = self.data[offset]
 *             bitmap_length = self.data[offset_plus_one]
 *             bitmap_end = offset_plus_two + bitmap_length             # <<<<<<<<<<<<<<
 *             for i, byte in enumerate(self.data[offset_plus_two:bitmap_end]):
 *                 for bit in range(0, 8):
 */
    __pyx_v_bitmap_end = (__pyx_v_offset_plus_two + __pyx_v_bitmap_length);

    /* "zeroconf/_protocol/incoming.py":327
 *             bitmap_length = self.data[offset_plus_one]
 *             bitmap_end = offset_plus_two + bitmap_length
 *             for i, byte in enumerate(self.data[offset_plus_two:bitmap_end]):             # <<<<<<<<<<<<<<
 *                 for bit in range(0, 8):
 *                     if byte & (0x80 >> bit):
 */
    __pyx_t_3 = 0;
    if (unlikely(__pyx_v_self->data == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 327, __pyx_L1_error)
    }
    __pyx_t_1 = PySequence_GetSlice(__pyx_v_self->data, __pyx_v_offset_plus_two, __pyx_v_bitmap_end); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 327, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = PyBytes_AS_STRING(__pyx_t_1);
    __pyx_t_6 = (__pyx_t_5 + PyBytes_GET_SIZE(__pyx_t_1));
    for (__pyx_t_7 = __pyx_t_5; __pyx_t_7 < __pyx_t_6; __pyx_t_7++) {
      __pyx_t_4 = __pyx_t_7;
      __pyx_v_byte = (__pyx_t_4[0]);

      /* "zeroconf/_protocol/incoming.py":328
 *             bitmap_end = offset_plus_two + bitmap_length
 *             for i, byte in enumerate(self.data[offset_plus_two:bitmap_end]):
 *                 for bit in range(0, 8):             # <<<<<<<<<<<<<<
 *                     if byte & (0x80 >> bit):
 *                         rdtypes.append(bit + window * 256 + i * 8)
 */
      __pyx_v_i = __pyx_t_3;

      /* "zeroconf/_protocol/incoming.py":327
 *             bitmap_length = self.data[offset_plus_one]
 *             bitmap_end = offset_plus_two + bitmap_length
 *             for i, byte in enumerate(self.data[offset_plus_two:bitmap_end]):             # <<<<<<<<<<<<<<
 *                 for bit in range(0, 8):
 *                     if byte & (0x80 >> bit):
 */
      __pyx_t_3 = (__pyx_t_3 + 1);

      /* "zeroconf/_protocol/incoming.py":328
 *             bitmap_end = offset_plus_two + bitmap_length
 *             for i, byte in enumerate(self.data[offset_plus_two:bitmap_end]):
 *                 for bit in range(0, 8):             # <<<<<<<<<<<<<<
 *                     if byte & (0x80 >> bit):
 *                         rdtypes.append(bit + window * 256 + i * 8)
 */
      for (__pyx_t_8 = 0; __pyx_t_8 < 8; __pyx_t_8+=1) {
        __pyx_v_bit = __pyx_t_8;

        /* "zeroconf/_protocol/incoming.py":329
 *             for i, byte in enumerate(self.data[offset_plus_two:bitmap_end]):
 *                 for bit in range(0, 8):
 *                     if byte & (0x80 >> bit):             # <<<<<<<<<<<<<<
 *                         rdtypes.append(bit + window * 256 + i * 8)
 *             self.offset += 2 + bitmap_length
 */
        __pyx_t_2 = ((__pyx_v_byte & (0x80 >> __pyx_v_bit)) != 0);
        if (__pyx_t_2) {

          /* "zeroconf/_protocol/incoming.py":330
 *                 for bit in range(0, 8):
 *                     if byte & (0x80 >> bit):
 *                         rdtypes.append(bit + window * 256 + i * 8)             # <<<<<<<<<<<<<<
 *             self.offset += 2 + bitmap_length
 *         return rdtypes
 */
          __pyx_t_9 = __Pyx_PyInt_From_long(((__pyx_v_bit + (__pyx_v_window * 0x100)) + (__pyx_v_i * 8))); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 330, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_9);
          __pyx_t_10 = __Pyx_PyList_Append(__pyx_v_rdtypes, __pyx_t_9); if (unlikely(__pyx_t_10 == ((int)-1))) __PYX_ERR(0, 330, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

          /* "zeroconf/_protocol/incoming.py":329
 *             for i, byte in enumerate(self.data[offset_plus_two:bitmap_end]):
 *                 for bit in range(0, 8):
 *                     if byte & (0x80 >> bit):             # <<<<<<<<<<<<<<
 *                         rdtypes.append(bit + window * 256 + i * 8)
 *             self.offset += 2 + bitmap_length
 */
        }
      }
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "zeroconf/_protocol/incoming.py":331
 *                     if byte & (0x80 >> bit):
 *                         rdtypes.append(bit + window * 256 + i * 8)
 *             self.offset += 2 + bitmap_length             # <<<<<<<<<<<<<<
 *         return rdtypes
 * 
 */
    __pyx_v_self->offset = (__pyx_v_self->offset + (2 + __pyx_v_bitmap_length));
  }

  /* "zeroconf/_protocol/incoming.py":332
 *                         rdtypes.append(bit + window * 256 + i * 8)
 *             self.offset += 2 + bitmap_length
 *         return rdtypes             # <<<<<<<<<<<<<<
 * 
 *     def _read_name(self) -> str:
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_rdtypes);
  __pyx_r = __pyx_v_rdtypes;
  goto __pyx_L0;

  /* "zeroconf/_protocol/incoming.py":317
 *         return None
 * 
 *     def _read_bitmap(self, end: int) -> List[int]:             # <<<<<<<<<<<<<<
 *         """Reads an NSEC bitmap from the packet."""
 *         rdtypes = []
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("zeroconf._protocol.incoming.DNSIncoming._read_bitmap", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_rdtypes);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/incoming.py":334
 *         return rdtypes
 * 
 *     def _read_name(self) -> str:             # <<<<<<<<<<<<<<
 *         """Reads a domain name from the packet."""
 *         labels: List[str] = []
 */

static PyObject *__pyx_f_8zeroconf_9_protocol_8incoming_11DNSIncoming__read_name(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self) {
  PyObject *__pyx_v_labels = NULL;
  PyObject *__pyx_v_seen_pointers = NULL;
  unsigned int __pyx_v_original_offset;
  PyObject *__pyx_v_name = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  unsigned int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t __pyx_t_4;
  int __pyx_t_5;
  Py_UCS4 __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_read_name", 0);

  /* "zeroconf/_protocol/incoming.py":336
 *     def _read_name(self) -> str:
 *         """Reads a domain name from the packet."""
 *         labels: List[str] = []             # <<<<<<<<<<<<<<
 *         seen_pointers: Set[int] = set()
 *         original_offset = self.offset
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 336, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_labels = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "zeroconf/_protocol/incoming.py":337
 *         """Reads a domain name from the packet."""
 *         labels: List[str] = []
 *         seen_pointers: Set[int] = set()             # <<<<<<<<<<<<<<
 *         original_offset = self.offset
 *         self.offset = self._decode_labels_at_offset(original_offset, labels, seen_pointers)
 */
  __pyx_t_1 = PySet_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 337, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_seen_pointers = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "zeroconf/_protocol/incoming.py":338
 *         labels: List[str] = []
 *         seen_pointers: Set[int] = set()
 *         original_offset = self.offset             # <<<<<<<<<<<<<<
 *         self.offset = self._decode_labels_at_offset(original_offset, labels, seen_pointers)
 *         self.name_cache[original_offset] = labels
 */
  __pyx_t_2 = __pyx_v_self->offset;
  __pyx_v_original_offset = __pyx_t_2;

  /* "zeroconf/_protocol/incoming.py":339
 *         seen_pointers: Set[int] = set()
 *         original_offset = self.offset
 *         self.offset = self._decode_labels_at_offset(original_offset, labels, seen_pointers)             # <<<<<<<<<<<<<<
 *         self.name_cache[original_offset] = labels
 *         name = ".".join(labels) + "."
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self->__pyx_vtab)->_decode_labels_at_offset(__pyx_v_self, __pyx_v_original_offset, __pyx_v_labels, __pyx_v_seen_pointers); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 339, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_unsigned_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(0, 339, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_self->offset = __pyx_t_2;

  /* "zeroconf/_protocol/incoming.py":340
 *         original_offset = self.offset
 *         self.offset = self._decode_labels_at_offset(original_offset, labels, seen_pointers)
 *         self.name_cache[original_offset] = labels             # <<<<<<<<<<<<<<
 *         name = ".".join(labels) + "."
 *         if len(name) > MAX_NAME_LENGTH:
 */
  if (unlikely(__pyx_v_self->name_cache == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 340, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyInt_From_unsigned_int(__pyx_v_original_offset); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 340, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (unlikely(PyDict_SetItem(__pyx_v_self->name_cache, __pyx_t_1, __pyx_v_labels) < 0)) __PYX_ERR(0, 340, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "zeroconf/_protocol/incoming.py":341
 *         self.offset = self._decode_labels_at_offset(original_offset, labels, seen_pointers)
 *         self.name_cache[original_offset] = labels
 *         name = ".".join(labels) + "."             # <<<<<<<<<<<<<<
 *         if len(name) > MAX_NAME_LENGTH:
 *             raise IncomingDecodeError(
 */
  __pyx_t_1 = PyUnicode_Join(__pyx_kp_u__2, __pyx_v_labels); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 341, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyUnicode_Concat(__pyx_t_1, __pyx_kp_u__2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 341, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_name = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "zeroconf/_protocol/incoming.py":342
 *         self.name_cache[original_offset] = labels
 *         name = ".".join(labels) + "."
 *         if len(name) > MAX_NAME_LENGTH:             # <<<<<<<<<<<<<<
 *             raise IncomingDecodeError(
 *                 f"DNS name {name} exceeds maximum length of {MAX_NAME_LENGTH} from {self.source}"
 */
  __pyx_t_4 = __Pyx_PyUnicode_GET_LENGTH(__pyx_v_name); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(0, 342, __pyx_L1_error)
  __pyx_t_5 = ((__pyx_t_4 > __pyx_v_8zeroconf_9_protocol_8incoming_MAX_NAME_LENGTH) != 0);
  if (unlikely(__pyx_t_5)) {

    /* "zeroconf/_protocol/incoming.py":344
 *         if len(name) > MAX_NAME_LENGTH:
 *             raise IncomingDecodeError(
 *                 f"DNS name {name} exceeds maximum length of {MAX_NAME_LENGTH} from {self.source}"             # <<<<<<<<<<<<<<
 *             )
 *         return name
 */
    __pyx_t_1 = PyTuple_New(6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 344, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = 0;
    __pyx_t_6 = 127;
    __Pyx_INCREF(__pyx_kp_u_DNS_name);
    __pyx_t_4 += 9;
    __Pyx_GIVEREF(__pyx_kp_u_DNS_name);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_kp_u_DNS_name);
    __Pyx_INCREF(__pyx_v_name);
    __pyx_t_6 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_v_name) > __pyx_t_6) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_v_name) : __pyx_t_6;
    __pyx_t_4 += __Pyx_PyUnicode_GET_LENGTH(__pyx_v_name);
    __Pyx_GIVEREF(__pyx_v_name);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_name);
    __Pyx_INCREF(__pyx_kp_u_exceeds_maximum_length_of);
    __pyx_t_4 += 27;
    __Pyx_GIVEREF(__pyx_kp_u_exceeds_maximum_length_of);
    PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_kp_u_exceeds_maximum_length_of);
    __pyx_t_7 = __Pyx_PyUnicode_From_unsigned_int(__pyx_v_8zeroconf_9_protocol_8incoming_MAX_NAME_LENGTH, 0, ' ', 'd'); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 344, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_4 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_7);
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_1, 3, __pyx_t_7);
    __pyx_t_7 = 0;
    __Pyx_INCREF(__pyx_kp_u_from);
    __pyx_t_4 += 6;
    __Pyx_GIVEREF(__pyx_kp_u_from);
    PyTuple_SET_ITEM(__pyx_t_1, 4, __pyx_kp_u_from);
    __pyx_t_7 = __Pyx_PyObject_FormatSimple(__pyx_v_self->source, __pyx_empty_unicode); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 344, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_6 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_7) > __pyx_t_6) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_7) : __pyx_t_6;
    __pyx_t_4 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_7);
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_1, 5, __pyx_t_7);
    __pyx_t_7 = 0;
    __pyx_t_7 = __Pyx_PyUnicode_Join(__pyx_t_1, 6, __pyx_t_4, __pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 344, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_INCREF(__pyx_v_8zeroconf_9_protocol_8incoming_IncomingDecodeError);
    __pyx_t_1 = __pyx_v_8zeroconf_9_protocol_8incoming_IncomingDecodeError; __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
      }
    }
    __pyx_t_3 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_8, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_7);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 343, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 343, __pyx_L1_error)

    /* "zeroconf/_protocol/incoming.py":342
 *         self.name_cache[original_offset] = labels
 *         name = ".".join(labels) + "."
 *         if len(name) > MAX_NAME_LENGTH:             # <<<<<<<<<<<<<<
 *             raise IncomingDecodeError(
 *                 f"DNS name {name} exceeds maximum length of {MAX_NAME_LENGTH} from {self.source}"
 */
  }

  /* "zeroconf/_protocol/incoming.py":346
 *                 f"DNS name {name} exceeds maximum length of {MAX_NAME_LENGTH} from {self.source}"
 *             )
 *         return name             # <<<<<<<<<<<<<<
 * 
 *     def _decode_labels_at_offset(self, off: int, labels: List[str], seen_pointers: Set[int]) -> int:
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_name);
  __pyx_r = __pyx_v_name;
  goto __pyx_L0;

  /* "zeroconf/_protocol/incoming.py":334
 *         return rdtypes
 * 
 *     def _read_name(self) -> str:             # <<<<<<<<<<<<<<
 *         """Reads a domain name from the packet."""
 *         labels: List[str] = []
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("zeroconf._protocol.incoming.DNSIncoming._read_name", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_labels);
  __Pyx_XDECREF(__pyx_v_seen_pointers);
  __Pyx_XDECREF(__pyx_v_name);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/incoming.py":348
 *         return name
 * 
 *     def _decode_labels_at_offset(self, off: int, labels: List[str], seen_pointers: Set[int]) -> int:             # <<<<<<<<<<<<<<
 *         # This is a tight loop that is called frequently, small optimizations can make a difference.
 *         while off < self._data_len:
 */

static PyObject *__pyx_f_8zeroconf_9_protocol_8incoming_11DNSIncoming__decode_labels_at_offset(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self, unsigned int __pyx_v_off, PyObject *__pyx_v_labels, PyObject *__pyx_v_seen_pointers) {
  unsigned int __pyx_v_label_idx;
  unsigned int __pyx_v_length;
  unsigned int __pyx_v_link;
  unsigned int __pyx_v_link_data;
  PyObject *__pyx_v_lint_int = NULL;
  PyObject *__pyx_v_linked_labels = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  unsigned int __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  Py_ssize_t __pyx_t_6;
  Py_UCS4 __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  long __pyx_t_10;
  int __pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_decode_labels_at_offset", 0);

  /* "zeroconf/_protocol/incoming.py":350
 *     def _decode_labels_at_offset(self, off: int, labels: List[str], seen_pointers: Set[int]) -> int:
 *         # This is a tight loop that is called frequently, small optimizations can make a difference.
 *         while off < self._data_len:             # <<<<<<<<<<<<<<
 *             length = self.data[off]
 *             if length == 0:
 */
  while (1) {
    __pyx_t_1 = ((__pyx_v_off < __pyx_v_self->_data_len) != 0);
    if (!__pyx_t_1) break;

    /* "zeroconf/_protocol/incoming.py":351
 *         # This is a tight loop that is called frequently, small optimizations can make a difference.
 *         while off < self._data_len:
 *             length = self.data[off]             # <<<<<<<<<<<<<<
 *             if length == 0:
 *                 return off + DNS_COMPRESSION_HEADER_LEN
 */
    __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_self->data, __pyx_v_off, unsigned int, 0, __Pyx_PyInt_From_unsigned_int, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 351, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyInt_As_unsigned_int(__pyx_t_2); if (unlikely((__pyx_t_3 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(0, 351, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_length = __pyx_t_3;

    /* "zeroconf/_protocol/incoming.py":352
 *         while off < self._data_len:
 *             length = self.data[off]
 *             if length == 0:             # <<<<<<<<<<<<<<
 *                 return off + DNS_COMPRESSION_HEADER_LEN
 * 
 */
    __pyx_t_1 = ((__pyx_v_length == 0) != 0);
    if (__pyx_t_1) {

      /* "zeroconf/_protocol/incoming.py":353
 *             length = self.data[off]
 *             if length == 0:
 *                 return off + DNS_COMPRESSION_HEADER_LEN             # <<<<<<<<<<<<<<
 * 
 *             if length < 0x40:
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_2 = __Pyx_PyInt_From_unsigned_int((__pyx_v_off + __pyx_v_8zeroconf_9_protocol_8incoming_DNS_COMPRESSION_HEADER_LEN)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 353, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      goto __pyx_L0;

      /* "zeroconf/_protocol/incoming.py":352
 *         while off < self._data_len:
 *             length = self.data[off]
 *             if length == 0:             # <<<<<<<<<<<<<<
 *                 return off + DNS_COMPRESSION_HEADER_LEN
 * 
 */
    }

    /* "zeroconf/_protocol/incoming.py":355
 *                 return off + DNS_COMPRESSION_HEADER_LEN
 * 
 *             if length < 0x40:             # <<<<<<<<<<<<<<
 *                 label_idx = off + DNS_COMPRESSION_HEADER_LEN
 *                 labels.append(self.data[label_idx : label_idx + length].decode('utf-8', 'replace'))
 */
    __pyx_t_1 = ((__pyx_v_length < 0x40) != 0);
    if (__pyx_t_1) {

      /* "zeroconf/_protocol/incoming.py":356
 * 
 *             if length < 0x40:
 *                 label_idx = off + DNS_COMPRESSION_HEADER_LEN             # <<<<<<<<<<<<<<
 *                 labels.append(self.data[label_idx : label_idx + length].decode('utf-8', 'replace'))
 *                 off += DNS_COMPRESSION_HEADER_LEN + length
 */
      __pyx_v_label_idx = (__pyx_v_off + __pyx_v_8zeroconf_9_protocol_8incoming_DNS_COMPRESSION_HEADER_LEN);

      /* "zeroconf/_protocol/incoming.py":357
 *             if length < 0x40:
 *                 label_idx = off + DNS_COMPRESSION_HEADER_LEN
 *                 labels.append(self.data[label_idx : label_idx + length].decode('utf-8', 'replace'))             # <<<<<<<<<<<<<<
 *                 off += DNS_COMPRESSION_HEADER_LEN + length
 *                 continue
 */
      if (unlikely(__pyx_v_labels == Py_None)) {
        PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
        __PYX_ERR(0, 357, __pyx_L1_error)
      }
      if (unlikely(__pyx_v_self->data == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 357, __pyx_L1_error)
      }
      __pyx_t_2 = __Pyx_decode_bytes(__pyx_v_self->data, __pyx_v_label_idx, (__pyx_v_label_idx + __pyx_v_length), NULL, ((char const *)"replace"), PyUnicode_DecodeUTF8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 357, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = __Pyx_PyList_Append(__pyx_v_labels, __pyx_t_2); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 357, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "zeroconf/_protocol/incoming.py":358
 *                 label_idx = off + DNS_COMPRESSION_HEADER_LEN
 *                 labels.append(self.data[label_idx : label_idx + length].decode('utf-8', 'replace'))
 *                 off += DNS_COMPRESSION_HEADER_LEN + length             # <<<<<<<<<<<<<<
 *                 continue
 * 
 */
      __pyx_v_off = (__pyx_v_off + (__pyx_v_8zeroconf_9_protocol_8incoming_DNS_COMPRESSION_HEADER_LEN + __pyx_v_length));

      /* "zeroconf/_protocol/incoming.py":359
 *                 labels.append(self.data[label_idx : label_idx + length].decode('utf-8', 'replace'))
 *                 off += DNS_COMPRESSION_HEADER_LEN + length
 *                 continue             # <<<<<<<<<<<<<<
 * 
 *             if length < 0xC0:
 */
      goto __pyx_L3_continue;

      /* "zeroconf/_protocol/incoming.py":355
 *                 return off + DNS_COMPRESSION_HEADER_LEN
 * 
 *             if length < 0x40:             # <<<<<<<<<<<<<<
 *                 label_idx = off + DNS_COMPRESSION_HEADER_LEN
 *                 labels.append(self.data[label_idx : label_idx + length].decode('utf-8', 'replace'))
 */
    }

    /* "zeroconf/_protocol/incoming.py":361
 *                 continue
 * 
 *             if length < 0xC0:             # <<<<<<<<<<<<<<
 *                 raise IncomingDecodeError(
 *                     f"DNS compression type {length} is unknown at {off} from {self.source}"
 */
    __pyx_t_1 = ((__pyx_v_length < 0xC0) != 0);
    if (unlikely(__pyx_t_1)) {

      /* "zeroconf/_protocol/incoming.py":363
 *             if length < 0xC0:
 *                 raise IncomingDecodeError(
 *                     f"DNS compression type {length} is unknown at {off} from {self.source}"             # <<<<<<<<<<<<<<
 *                 )
 * 
 */
      __pyx_t_5 = PyTuple_New(6); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 363, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = 0;
      __pyx_t_7 = 127;
      __Pyx_INCREF(__pyx_kp_u_DNS_compression_type);
      __pyx_t_6 += 21;
      __Pyx_GIVEREF(__pyx_kp_u_DNS_compression_type);
      PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_kp_u_DNS_compression_type);
      __pyx_t_8 = __Pyx_PyUnicode_From_unsigned_int(__pyx_v_length, 0, ' ', 'd'); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 363, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_6 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_8);
      __Pyx_GIVEREF(__pyx_t_8);
      PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_8);
      __pyx_t_8 = 0;
      __Pyx_INCREF(__pyx_kp_u_is_unknown_at);
      __pyx_t_6 += 15;
      __Pyx_GIVEREF(__pyx_kp_u_is_unknown_at);
      PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_kp_u_is_unknown_at);
      __pyx_t_8 = __Pyx_PyUnicode_From_unsigned_int(__pyx_v_off, 0, ' ', 'd'); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 363, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_6 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_8);
      __Pyx_GIVEREF(__pyx_t_8);
      PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_8);
      __pyx_t_8 = 0;
      __Pyx_INCREF(__pyx_kp_u_from);
      __pyx_t_6 += 6;
      __Pyx_GIVEREF(__pyx_kp_u_from);
      PyTuple_SET_ITEM(__pyx_t_5, 4, __pyx_kp_u_from);
      __pyx_t_8 = __Pyx_PyObject_FormatSimple(__pyx_v_self->source, __pyx_empty_unicode); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 363, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_7 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_8) > __pyx_t_7) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_8) : __pyx_t_7;
      __pyx_t_6 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_8);
      __Pyx_GIVEREF(__pyx_t_8);
      PyTuple_SET_ITEM(__pyx_t_5, 5, __pyx_t_8);
      __pyx_t_8 = 0;
      __pyx_t_8 = __Pyx_PyUnicode_Join(__pyx_t_5, 6, __pyx_t_6, __pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 363, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_INCREF(__pyx_v_8zeroconf_9_protocol_8incoming_IncomingDecodeError);
      __pyx_t_5 = __pyx_v_8zeroconf_9_protocol_8incoming_IncomingDecodeError; __pyx_t_9 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_9)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_9);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
        }
      }
      __pyx_t_2 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_9, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_8);
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 362, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_Raise(__pyx_t_2, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __PYX_ERR(0, 362, __pyx_L1_error)

      /* "zeroconf/_protocol/incoming.py":361
 *                 continue
 * 
 *             if length < 0xC0:             # <<<<<<<<<<<<<<
 *                 raise IncomingDecodeError(
 *                     f"DNS compression type {length} is unknown at {off} from {self.source}"
 */
    }

    /* "zeroconf/_protocol/incoming.py":367
 * 
 *             # We have a DNS compression pointer
 *             link_data = self.data[off + 1]             # <<<<<<<<<<<<<<
 *             link = (length & 0x3F) * 256 + link_data
 *             lint_int = int(link)
 */
    __pyx_t_10 = (__pyx_v_off + 1);
    __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_self->data, __pyx_t_10, long, 1, __Pyx_PyInt_From_long, 0, 1, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 367, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyInt_As_unsigned_int(__pyx_t_2); if (unlikely((__pyx_t_3 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(0, 367, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_link_data = __pyx_t_3;

    /* "zeroconf/_protocol/incoming.py":368
 *             # We have a DNS compression pointer
 *             link_data = self.data[off + 1]
 *             link = (length & 0x3F) * 256 + link_data             # <<<<<<<<<<<<<<
 *             lint_int = int(link)
 *             if link > self._data_len:
 */
    __pyx_v_link = (((__pyx_v_length & 0x3F) * 0x100) + __pyx_v_link_data);

    /* "zeroconf/_protocol/incoming.py":369
 *             link_data = self.data[off + 1]
 *             link = (length & 0x3F) * 256 + link_data
 *             lint_int = int(link)             # <<<<<<<<<<<<<<
 *             if link > self._data_len:
 *                 raise IncomingDecodeError(
 */
    __pyx_t_2 = __Pyx_PyInt_From_unsigned_int(__pyx_v_link); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 369, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyInt_Type)), __pyx_t_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 369, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_lint_int = __pyx_t_5;
    __pyx_t_5 = 0;

    /* "zeroconf/_protocol/incoming.py":370
 *             link = (length & 0x3F) * 256 + link_data
 *             lint_int = int(link)
 *             if link > self._data_len:             # <<<<<<<<<<<<<<
 *                 raise IncomingDecodeError(
 *                     f"DNS compression pointer at {off} points to {link} beyond packet from {self.source}"
 */
    __pyx_t_1 = ((__pyx_v_link > __pyx_v_self->_data_len) != 0);
    if (unlikely(__pyx_t_1)) {

      /* "zeroconf/_protocol/incoming.py":372
 *             if link > self._data_len:
 *                 raise IncomingDecodeError(
 *                     f"DNS compression pointer at {off} points to {link} beyond packet from {self.source}"             # <<<<<<<<<<<<<<
 *                 )
 *             if link == off:
 */
      __pyx_t_2 = PyTuple_New(6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 372, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_6 = 0;
      __pyx_t_7 = 127;
      __Pyx_INCREF(__pyx_kp_u_DNS_compression_pointer_at);
      __pyx_t_6 += 27;
      __Pyx_GIVEREF(__pyx_kp_u_DNS_compression_pointer_at);
      PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_kp_u_DNS_compression_pointer_at);
      __pyx_t_8 = __Pyx_PyUnicode_From_unsigned_int(__pyx_v_off, 0, ' ', 'd'); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 372, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_6 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_8);
      __Pyx_GIVEREF(__pyx_t_8);
      PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_8);
      __pyx_t_8 = 0;
      __Pyx_INCREF(__pyx_kp_u_points_to);
      __pyx_t_6 += 11;
      __Pyx_GIVEREF(__pyx_kp_u_points_to);
      PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_kp_u_points_to);
      __pyx_t_8 = __Pyx_PyUnicode_From_unsigned_int(__pyx_v_link, 0, ' ', 'd'); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 372, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_6 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_8);
      __Pyx_GIVEREF(__pyx_t_8);
      PyTuple_SET_ITEM(__pyx_t_2, 3, __pyx_t_8);
      __pyx_t_8 = 0;
      __Pyx_INCREF(__pyx_kp_u_beyond_packet_from);
      __pyx_t_6 += 20;
      __Pyx_GIVEREF(__pyx_kp_u_beyond_packet_from);
      PyTuple_SET_ITEM(__pyx_t_2, 4, __pyx_kp_u_beyond_packet_from);
      __pyx_t_8 = __Pyx_PyObject_FormatSimple(__pyx_v_self->source, __pyx_empty_unicode); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 372, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_7 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_8) > __pyx_t_7) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_8) : __pyx_t_7;
      __pyx_t_6 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_8);
      __Pyx_GIVEREF(__pyx_t_8);
      PyTuple_SET_ITEM(__pyx_t_2, 5, __pyx_t_8);
      __pyx_t_8 = 0;
      __pyx_t_8 = __Pyx_PyUnicode_Join(__pyx_t_2, 6, __pyx_t_6, __pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 372, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_INCREF(__pyx_v_8zeroconf_9_protocol_8incoming_IncomingDecodeError);
      __pyx_t_2 = __pyx_v_8zeroconf_9_protocol_8incoming_IncomingDecodeError; __pyx_t_9 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_9)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_9);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      __pyx_t_5 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_9, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_8);
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 371, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_Raise(__pyx_t_5, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __PYX_ERR(0, 371, __pyx_L1_error)

      /* "zeroconf/_protocol/incoming.py":370
 *             link = (length & 0x3F) * 256 + link_data
 *             lint_int = int(link)
 *             if link > self._data_len:             # <<<<<<<<<<<<<<
 *                 raise IncomingDecodeError(
 *                     f"DNS compression pointer at {off} points to {link} beyond packet from {self.source}"
 */
    }

    /* "zeroconf/_protocol/incoming.py":374
 *                     f"DNS compression pointer at {off} points to {link} beyond packet from {self.source}"
 *                 )
 *             if link == off:             # <<<<<<<<<<<<<<
 *                 raise IncomingDecodeError(
 *                     f"DNS compression pointer at {off} points to itself from {self.source}"
 */
    __pyx_t_1 = ((__pyx_v_link == __pyx_v_off) != 0);
    if (unlikely(__pyx_t_1)) {

      /* "zeroconf/_protocol/incoming.py":376
 *             if link == off:
 *                 raise IncomingDecodeError(
 *                     f"DNS compression pointer at {off} points to itself from {self.source}"             # <<<<<<<<<<<<<<
 *                 )
 *             if lint_int in seen_pointers:
 */
      __pyx_t_2 = PyTuple_New(4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 376, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_6 = 0;
      __pyx_t_7 = 127;
      __Pyx_INCREF(__pyx_kp_u_DNS_compression_pointer_at);
      __pyx_t_6 += 27;
      __Pyx_GIVEREF(__pyx_kp_u_DNS_compression_pointer_at);
      PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_kp_u_DNS_compression_pointer_at);
      __pyx_t_8 = __Pyx_PyUnicode_From_unsigned_int(__pyx_v_off, 0, ' ', 'd'); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 376, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_6 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_8);
      __Pyx_GIVEREF(__pyx_t_8);
      PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_8);
      __pyx_t_8 = 0;
      __Pyx_INCREF(__pyx_kp_u_points_to_itself_from);
      __pyx_t_6 += 23;
      __Pyx_GIVEREF(__pyx_kp_u_points_to_itself_from);
      PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_kp_u_points_to_itself_from);
      __pyx_t_8 = __Pyx_PyObject_FormatSimple(__pyx_v_self->source, __pyx_empty_unicode); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 376, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_7 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_8) > __pyx_t_7) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_8) : __pyx_t_7;
      __pyx_t_6 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_8);
      __Pyx_GIVEREF(__pyx_t_8);
      PyTuple_SET_ITEM(__pyx_t_2, 3, __pyx_t_8);
      __pyx_t_8 = 0;
      __pyx_t_8 = __Pyx_PyUnicode_Join(__pyx_t_2, 4, __pyx_t_6, __pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 376, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_INCREF(__pyx_v_8zeroconf_9_protocol_8incoming_IncomingDecodeError);
      __pyx_t_2 = __pyx_v_8zeroconf_9_protocol_8incoming_IncomingDecodeError; __pyx_t_9 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_9)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_9);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      __pyx_t_5 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_9, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_8);
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 375, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_Raise(__pyx_t_5, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __PYX_ERR(0, 375, __pyx_L1_error)

      /* "zeroconf/_protocol/incoming.py":374
 *                     f"DNS compression pointer at {off} points to {link} beyond packet from {self.source}"
 *                 )
 *             if link == off:             # <<<<<<<<<<<<<<
 *                 raise IncomingDecodeError(
 *                     f"DNS compression pointer at {off} points to itself from {self.source}"
 */
    }

    /* "zeroconf/_protocol/incoming.py":378
 *                     f"DNS compression pointer at {off} points to itself from {self.source}"
 *                 )
 *             if lint_int in seen_pointers:             # <<<<<<<<<<<<<<
 *                 raise IncomingDecodeError(
 *                     f"DNS compression pointer at {off} was seen again from {self.source}"
 */
    if (unlikely(__pyx_v_seen_pointers == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
      __PYX_ERR(0, 378, __pyx_L1_error)
    }
    __pyx_t_1 = (__Pyx_PySet_ContainsTF(__pyx_v_lint_int, __pyx_v_seen_pointers, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 378, __pyx_L1_error)
    __pyx_t_11 = (__pyx_t_1 != 0);
    if (unlikely(__pyx_t_11)) {

      /* "zeroconf/_protocol/incoming.py":380
 *             if lint_int in seen_pointers:
 *                 raise IncomingDecodeError(
 *                     f"DNS compression pointer at {off} was seen again from {self.source}"             # <<<<<<<<<<<<<<
 *                 )
 *             linked_labels = self.name_cache.get(lint_int)
 */
      __pyx_t_2 = PyTuple_New(4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 380, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_6 = 0;
      __pyx_t_7 = 127;
      __Pyx_INCREF(__pyx_kp_u_DNS_compression_pointer_at);
      __pyx_t_6 += 27;
      __Pyx_GIVEREF(__pyx_kp_u_DNS_compression_pointer_at);
      PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_kp_u_DNS_compression_pointer_at);
      __pyx_t_8 = __Pyx_PyUnicode_From_unsigned_int(__pyx_v_off, 0, ' ', 'd'); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 380, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_6 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_8);
      __Pyx_GIVEREF(__pyx_t_8);
      PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_8);
      __pyx_t_8 = 0;
      __Pyx_INCREF(__pyx_kp_u_was_seen_again_from);
      __pyx_t_6 += 21;
      __Pyx_GIVEREF(__pyx_kp_u_was_seen_again_from);
      PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_kp_u_was_seen_again_from);
      __pyx_t_8 = __Pyx_PyObject_FormatSimple(__pyx_v_self->source, __pyx_empty_unicode); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 380, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_7 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_8) > __pyx_t_7) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_8) : __pyx_t_7;
      __pyx_t_6 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_8);
      __Pyx_GIVEREF(__pyx_t_8);
      PyTuple_SET_ITEM(__pyx_t_2, 3, __pyx_t_8);
      __pyx_t_8 = 0;
      __pyx_t_8 = __Pyx_PyUnicode_Join(__pyx_t_2, 4, __pyx_t_6, __pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 380, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_INCREF(__pyx_v_8zeroconf_9_protocol_8incoming_IncomingDecodeError);
      __pyx_t_2 = __pyx_v_8zeroconf_9_protocol_8incoming_IncomingDecodeError; __pyx_t_9 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_9)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_9);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      __pyx_t_5 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_9, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_8);
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 379, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_Raise(__pyx_t_5, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __PYX_ERR(0, 379, __pyx_L1_error)

      /* "zeroconf/_protocol/incoming.py":378
 *                     f"DNS compression pointer at {off} points to itself from {self.source}"
 *                 )
 *             if lint_int in seen_pointers:             # <<<<<<<<<<<<<<
 *                 raise IncomingDecodeError(
 *                     f"DNS compression pointer at {off} was seen again from {self.source}"
 */
    }

    /* "zeroconf/_protocol/incoming.py":382
 *                     f"DNS compression pointer at {off} was seen again from {self.source}"
 *                 )
 *             linked_labels = self.name_cache.get(lint_int)             # <<<<<<<<<<<<<<
 *             if not linked_labels:
 *                 linked_labels = []
 */
    if (unlikely(__pyx_v_self->name_cache == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "get");
      __PYX_ERR(0, 382, __pyx_L1_error)
    }
    __pyx_t_5 = __Pyx_PyDict_GetItemDefault(__pyx_v_self->name_cache, __pyx_v_lint_int, Py_None); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 382, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_v_linked_labels = __pyx_t_5;
    __pyx_t_5 = 0;

    /* "zeroconf/_protocol/incoming.py":383
 *                 )
 *             linked_labels = self.name_cache.get(lint_int)
 *             if not linked_labels:             # <<<<<<<<<<<<<<
 *                 linked_labels = []
 *                 seen_pointers.add(lint_int)
 */
    __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_v_linked_labels); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 383, __pyx_L1_error)
    __pyx_t_1 = ((!__pyx_t_11) != 0);
    if (__pyx_t_1) {

      /* "zeroconf/_protocol/incoming.py":384
 *             linked_labels = self.name_cache.get(lint_int)
 *             if not linked_labels:
 *                 linked_labels = []             # <<<<<<<<<<<<<<
 *                 seen_pointers.add(lint_int)
 *                 self._decode_labels_at_offset(link, linked_labels, seen_pointers)
 */
      __pyx_t_5 = PyList_New(0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 384, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF_SET(__pyx_v_linked_labels, __pyx_t_5);
      __pyx_t_5 = 0;

      /* "zeroconf/_protocol/incoming.py":385
 *             if not linked_labels:
 *                 linked_labels = []
 *                 seen_pointers.add(lint_int)             # <<<<<<<<<<<<<<
 *                 self._decode_labels_at_offset(link, linked_labels, seen_pointers)
 *                 self.name_cache[lint_int] = linked_labels
 */
      if (unlikely(__pyx_v_seen_pointers == Py_None)) {
        PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "add");
        __PYX_ERR(0, 385, __pyx_L1_error)
      }
      __pyx_t_4 = PySet_Add(__pyx_v_seen_pointers, __pyx_v_lint_int); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 385, __pyx_L1_error)

      /* "zeroconf/_protocol/incoming.py":386
 *                 linked_labels = []
 *                 seen_pointers.add(lint_int)
 *                 self._decode_labels_at_offset(link, linked_labels, seen_pointers)             # <<<<<<<<<<<<<<
 *                 self.name_cache[lint_int] = linked_labels
 *             labels.extend(linked_labels)
 */
      if (!(likely(PyList_CheckExact(__pyx_v_linked_labels))||((void)PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_v_linked_labels)->tp_name), 0))) __PYX_ERR(0, 386, __pyx_L1_error)
      __pyx_t_5 = ((struct __pyx_vtabstruct_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self->__pyx_vtab)->_decode_labels_at_offset(__pyx_v_self, __pyx_v_link, ((PyObject*)__pyx_v_linked_labels), __pyx_v_seen_pointers); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 386, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

      /* "zeroconf/_protocol/incoming.py":387
 *                 seen_pointers.add(lint_int)
 *                 self._decode_labels_at_offset(link, linked_labels, seen_pointers)
 *                 self.name_cache[lint_int] = linked_labels             # <<<<<<<<<<<<<<
 *             labels.extend(linked_labels)
 *             if len(labels) > MAX_DNS_LABELS:
 */
      if (unlikely(__pyx_v_self->name_cache == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 387, __pyx_L1_error)
      }
      if (unlikely(PyDict_SetItem(__pyx_v_self->name_cache, __pyx_v_lint_int, __pyx_v_linked_labels) < 0)) __PYX_ERR(0, 387, __pyx_L1_error)

      /* "zeroconf/_protocol/incoming.py":383
 *                 )
 *             linked_labels = self.name_cache.get(lint_int)
 *             if not linked_labels:             # <<<<<<<<<<<<<<
 *                 linked_labels = []
 *                 seen_pointers.add(lint_int)
 */
    }

    /* "zeroconf/_protocol/incoming.py":388
 *                 self._decode_labels_at_offset(link, linked_labels, seen_pointers)
 *                 self.name_cache[lint_int] = linked_labels
 *             labels.extend(linked_labels)             # <<<<<<<<<<<<<<
 *             if len(labels) > MAX_DNS_LABELS:
 *                 raise IncomingDecodeError(
 */
    if (unlikely(__pyx_v_labels == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "extend");
      __PYX_ERR(0, 388, __pyx_L1_error)
    }
    __pyx_t_4 = __Pyx_PyList_Extend(__pyx_v_labels, __pyx_v_linked_labels); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 388, __pyx_L1_error)

    /* "zeroconf/_protocol/incoming.py":389
 *                 self.name_cache[lint_int] = linked_labels
 *             labels.extend(linked_labels)
 *             if len(labels) > MAX_DNS_LABELS:             # <<<<<<<<<<<<<<
 *                 raise IncomingDecodeError(
 *                     f"Maximum dns labels reached while processing pointer at {off} from {self.source}"
 */
    if (unlikely(__pyx_v_labels == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(0, 389, __pyx_L1_error)
    }
    __pyx_t_6 = PyList_GET_SIZE(__pyx_v_labels); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1))) __PYX_ERR(0, 389, __pyx_L1_error)
    __pyx_t_1 = ((__pyx_t_6 > __pyx_v_8zeroconf_9_protocol_8incoming_MAX_DNS_LABELS) != 0);
    if (unlikely(__pyx_t_1)) {

      /* "zeroconf/_protocol/incoming.py":391
 *             if len(labels) > MAX_DNS_LABELS:
 *                 raise IncomingDecodeError(
 *                     f"Maximum dns labels reached while processing pointer at {off} from {self.source}"             # <<<<<<<<<<<<<<
 *                 )
 *             return off + DNS_COMPRESSION_POINTER_LEN
 */
      __pyx_t_2 = PyTuple_New(4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 391, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_6 = 0;
      __pyx_t_7 = 127;
      __Pyx_INCREF(__pyx_kp_u_Maximum_dns_labels_reached_while);
      __pyx_t_6 += 55;
      __Pyx_GIVEREF(__pyx_kp_u_Maximum_dns_labels_reached_while);
      PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_kp_u_Maximum_dns_labels_reached_while);
      __pyx_t_8 = __Pyx_PyUnicode_From_unsigned_int(__pyx_v_off, 0, ' ', 'd'); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 391, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_6 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_8);
      __Pyx_GIVEREF(__pyx_t_8);
      PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_8);
      __pyx_t_8 = 0;
      __Pyx_INCREF(__pyx_kp_u_from);
      __pyx_t_6 += 6;
      __Pyx_GIVEREF(__pyx_kp_u_from);
      PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_kp_u_from);
      __pyx_t_8 = __Pyx_PyObject_FormatSimple(__pyx_v_self->source, __pyx_empty_unicode); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 391, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_7 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_8) > __pyx_t_7) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_8) : __pyx_t_7;
      __pyx_t_6 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_8);
      __Pyx_GIVEREF(__pyx_t_8);
      PyTuple_SET_ITEM(__pyx_t_2, 3, __pyx_t_8);
      __pyx_t_8 = 0;
      __pyx_t_8 = __Pyx_PyUnicode_Join(__pyx_t_2, 4, __pyx_t_6, __pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 391, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_INCREF(__pyx_v_8zeroconf_9_protocol_8incoming_IncomingDecodeError);
      __pyx_t_2 = __pyx_v_8zeroconf_9_protocol_8incoming_IncomingDecodeError; __pyx_t_9 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_9)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_9);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      __pyx_t_5 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_9, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_8);
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 390, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_Raise(__pyx_t_5, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __PYX_ERR(0, 390, __pyx_L1_error)

      /* "zeroconf/_protocol/incoming.py":389
 *                 self.name_cache[lint_int] = linked_labels
 *             labels.extend(linked_labels)
 *             if len(labels) > MAX_DNS_LABELS:             # <<<<<<<<<<<<<<
 *                 raise IncomingDecodeError(
 *                     f"Maximum dns labels reached while processing pointer at {off} from {self.source}"
 */
    }

    /* "zeroconf/_protocol/incoming.py":393
 *                     f"Maximum dns labels reached while processing pointer at {off} from {self.source}"
 *                 )
 *             return off + DNS_COMPRESSION_POINTER_LEN             # <<<<<<<<<<<<<<
 * 
 *         raise IncomingDecodeError(f"Corrupt packet received while decoding name from {self.source}")
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_5 = __Pyx_PyInt_From_unsigned_int((__pyx_v_off + __pyx_v_8zeroconf_9_protocol_8incoming_DNS_COMPRESSION_POINTER_LEN)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 393, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;
    __pyx_L3_continue:;
  }

  /* "zeroconf/_protocol/incoming.py":395
 *             return off + DNS_COMPRESSION_POINTER_LEN
 * 
 *         raise IncomingDecodeError(f"Corrupt packet received while decoding name from {self.source}")             # <<<<<<<<<<<<<<
 */
  __pyx_t_2 = __Pyx_PyObject_FormatSimple(__pyx_v_self->source, __pyx_empty_unicode); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 395, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_8 = __Pyx_PyUnicode_Concat(__pyx_kp_u_Corrupt_packet_received_while_de, __pyx_t_2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 395, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_INCREF(__pyx_v_8zeroconf_9_protocol_8incoming_IncomingDecodeError);
  __pyx_t_2 = __pyx_v_8zeroconf_9_protocol_8incoming_IncomingDecodeError; __pyx_t_9 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_9)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_9);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_5 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_9, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 395, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_Raise(__pyx_t_5, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __PYX_ERR(0, 395, __pyx_L1_error)

  /* "zeroconf/_protocol/incoming.py":348
 *         return name
 * 
 *     def _decode_labels_at_offset(self, off: int, labels: List[str], seen_pointers: Set[int]) -> int:             # <<<<<<<<<<<<<<
 *         # This is a tight loop that is called frequently, small optimizations can make a difference.
 *         while off < self._data_len:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("zeroconf._protocol.incoming.DNSIncoming._decode_labels_at_offset", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_lint_int);
  __Pyx_XDECREF(__pyx_v_linked_labels);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/incoming.pxd":51
 * 
 *     cdef bint _did_read_others
 *     cdef public unsigned int flags             # <<<<<<<<<<<<<<
 *     cdef cython.uint offset
 *     cdef public bytes data
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_5flags_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_5flags_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_5flags___get__(((struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_5flags___get__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_unsigned_int(__pyx_v_self->flags); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 51, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("zeroconf._protocol.incoming.DNSIncoming.flags.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_5flags_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_5flags_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_5flags_2__set__(((struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_5flags_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  unsigned int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_unsigned_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(1, 51, __pyx_L1_error)
  __pyx_v_self->flags = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("zeroconf._protocol.incoming.DNSIncoming.flags.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/incoming.pxd":53
 *     cdef public unsigned int flags
 *     cdef cython.uint offset
 *     cdef public bytes data             # <<<<<<<<<<<<<<
 *     cdef unsigned int _data_len
 *     cdef public cython.dict name_cache
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_4data_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_4data_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_4data___get__(((struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_4data___get__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->data);
  __pyx_r = __pyx_v_self->data;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_4data_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_4data_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_4data_2__set__(((struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_4data_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyBytes_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||((void)PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(1, 53, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->data);
  __Pyx_DECREF(__pyx_v_self->data);
  __pyx_v_self->data = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("zeroconf._protocol.incoming.DNSIncoming.data.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_4data_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_4data_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_4data_4__del__(((struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_4data_4__del__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->data);
  __Pyx_DECREF(__pyx_v_self->data);
  __pyx_v_self->data = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/incoming.pxd":55
 *     cdef public bytes data
 *     cdef unsigned int _data_len
 *     cdef public cython.dict name_cache             # <<<<<<<<<<<<<<
 *     cdef public cython.list questions
 *     cdef cython.list _answers
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_10name_cache_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_10name_cache_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_10name_cache___get__(((struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_10name_cache___get__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->name_cache);
  __pyx_r = __pyx_v_self->name_cache;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_10name_cache_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_10name_cache_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_10name_cache_2__set__(((struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_10name_cache_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyDict_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||((void)PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(1, 55, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->name_cache);
  __Pyx_DECREF(__pyx_v_self->name_cache);
  __pyx_v_self->name_cache = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("zeroconf._protocol.incoming.DNSIncoming.name_cache.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_10name_cache_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_10name_cache_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_10name_cache_4__del__(((struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_10name_cache_4__del__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->name_cache);
  __Pyx_DECREF(__pyx_v_self->name_cache);
  __pyx_v_self->name_cache = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/incoming.pxd":56
 *     cdef unsigned int _data_len
 *     cdef public cython.dict name_cache
 *     cdef public cython.list questions             # <<<<<<<<<<<<<<
 *     cdef cython.list _answers
 *     cdef public object id
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_9questions_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_9questions_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_9questions___get__(((struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_9questions___get__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->questions);
  __pyx_r = __pyx_v_self->questions;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_9questions_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_9questions_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_9questions_2__set__(((struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_9questions_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyList_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||((void)PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(1, 56, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->questions);
  __Pyx_DECREF(__pyx_v_self->questions);
  __pyx_v_self->questions = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("zeroconf._protocol.incoming.DNSIncoming.questions.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_9questions_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_9questions_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_9questions_4__del__(((struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_9questions_4__del__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->questions);
  __Pyx_DECREF(__pyx_v_self->questions);
  __pyx_v_self->questions = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/incoming.pxd":58
 *     cdef public cython.list questions
 *     cdef cython.list _answers
 *     cdef public object id             # <<<<<<<<<<<<<<
 *     cdef public cython.uint num_questions
 *     cdef public cython.uint num_answers
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_2id_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_2id_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_2id___get__(((struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_2id___get__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->id);
  __pyx_r = __pyx_v_self->id;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_2id_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_2id_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_2id_2__set__(((struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_2id_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->id);
  __Pyx_DECREF(__pyx_v_self->id);
  __pyx_v_self->id = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_2id_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_2id_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_2id_4__del__(((struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_2id_4__del__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->id);
  __Pyx_DECREF(__pyx_v_self->id);
  __pyx_v_self->id = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/incoming.pxd":59
 *     cdef cython.list _answers
 *     cdef public object id
 *     cdef public cython.uint num_questions             # <<<<<<<<<<<<<<
 *     cdef public cython.uint num_answers
 *     cdef public cython.uint num_authorities
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_13num_questions_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_13num_questions_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_13num_questions___get__(((struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_13num_questions___get__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_unsigned_int(__pyx_v_self->num_questions); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 59, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("zeroconf._protocol.incoming.DNSIncoming.num_questions.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_13num_questions_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_13num_questions_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_13num_questions_2__set__(((struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_13num_questions_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  unsigned int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_unsigned_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(1, 59, __pyx_L1_error)
  __pyx_v_self->num_questions = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("zeroconf._protocol.incoming.DNSIncoming.num_questions.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/incoming.pxd":60
 *     cdef public object id
 *     cdef public cython.uint num_questions
 *     cdef public cython.uint num_answers             # <<<<<<<<<<<<<<
 *     cdef public cython.uint num_authorities
 *     cdef public cython.uint num_additionals
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_11num_answers_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_11num_answers_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_11num_answers___get__(((struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_11num_answers___get__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_unsigned_int(__pyx_v_self->num_answers); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 60, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("zeroconf._protocol.incoming.DNSIncoming.num_answers.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_11num_answers_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_11num_answers_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_11num_answers_2__set__(((struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_11num_answers_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  unsigned int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_unsigned_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(1, 60, __pyx_L1_error)
  __pyx_v_self->num_answers = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("zeroconf._protocol.incoming.DNSIncoming.num_answers.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/incoming.pxd":61
 *     cdef public cython.uint num_questions
 *     cdef public cython.uint num_answers
 *     cdef public cython.uint num_authorities             # <<<<<<<<<<<<<<
 *     cdef public cython.uint num_additionals
 *     cdef public object valid
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_15num_authorities_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_15num_authorities_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_15num_authorities___get__(((struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_15num_authorities___get__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_unsigned_int(__pyx_v_self->num_authorities); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 61, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("zeroconf._protocol.incoming.DNSIncoming.num_authorities.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_15num_authorities_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_15num_authorities_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_15num_authorities_2__set__(((struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_15num_authorities_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  unsigned int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_unsigned_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(1, 61, __pyx_L1_error)
  __pyx_v_self->num_authorities = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("zeroconf._protocol.incoming.DNSIncoming.num_authorities.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/incoming.pxd":62
 *     cdef public cython.uint num_answers
 *     cdef public cython.uint num_authorities
 *     cdef public cython.uint num_additionals             # <<<<<<<<<<<<<<
 *     cdef public object valid
 *     cdef public object now
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_15num_additionals_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_15num_additionals_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_15num_additionals___get__(((struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_15num_additionals___get__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_unsigned_int(__pyx_v_self->num_additionals); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 62, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("zeroconf._protocol.incoming.DNSIncoming.num_additionals.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_15num_additionals_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_15num_additionals_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_15num_additionals_2__set__(((struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_15num_additionals_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  unsigned int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_unsigned_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(1, 62, __pyx_L1_error)
  __pyx_v_self->num_additionals = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("zeroconf._protocol.incoming.DNSIncoming.num_additionals.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/incoming.pxd":63
 *     cdef public cython.uint num_authorities
 *     cdef public cython.uint num_additionals
 *     cdef public object valid             # <<<<<<<<<<<<<<
 *     cdef public object now
 *     cdef public object scope_id
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_5valid_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_5valid_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_5valid___get__(((struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_5valid___get__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->valid);
  __pyx_r = __pyx_v_self->valid;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_5valid_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_5valid_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_5valid_2__set__(((struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_5valid_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->valid);
  __Pyx_DECREF(__pyx_v_self->valid);
  __pyx_v_self->valid = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_5valid_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_5valid_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_5valid_4__del__(((struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_5valid_4__del__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->valid);
  __Pyx_DECREF(__pyx_v_self->valid);
  __pyx_v_self->valid = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/incoming.pxd":64
 *     cdef public cython.uint num_additionals
 *     cdef public object valid
 *     cdef public object now             # <<<<<<<<<<<<<<
 *     cdef public object scope_id
 *     cdef public object source
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_3now_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_3now_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_3now___get__(((struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_3now___get__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->now);
  __pyx_r = __pyx_v_self->now;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_3now_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_3now_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_3now_2__set__(((struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_3now_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->now);
  __Pyx_DECREF(__pyx_v_self->now);
  __pyx_v_self->now = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_3now_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_3now_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_3now_4__del__(((struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_3now_4__del__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->now);
  __Pyx_DECREF(__pyx_v_self->now);
  __pyx_v_self->now = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/incoming.pxd":65
 *     cdef public object valid
 *     cdef public object now
 *     cdef public object scope_id             # <<<<<<<<<<<<<<
 *     cdef public object source
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_8scope_id_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_8scope_id_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_8scope_id___get__(((struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_8scope_id___get__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->scope_id);
  __pyx_r = __pyx_v_self->scope_id;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_8scope_id_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_8scope_id_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_8scope_id_2__set__(((struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_8scope_id_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->scope_id);
  __Pyx_DECREF(__pyx_v_self->scope_id);
  __pyx_v_self->scope_id = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_8scope_id_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_8scope_id_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_8scope_id_4__del__(((struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_8scope_id_4__del__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->scope_id);
  __Pyx_DECREF(__pyx_v_self->scope_id);
  __pyx_v_self->scope_id = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "zeroconf/_protocol/incoming.pxd":66
 *     cdef public object now
 *     cdef public object scope_id
 *     cdef public object source             # <<<<<<<<<<<<<<
 * 
 *     @cython.locals(
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_6source_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_6source_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_6source___get__(((struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_6source___get__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->source);
  __pyx_r = __pyx_v_self->source;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_6source_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_6source_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_6source_2__set__(((struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_6source_2__set__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->source);
  __Pyx_DECREF(__pyx_v_self->source);
  __pyx_v_self->source = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_6source_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_6source_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_6source_4__del__(((struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_6source_4__del__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->source);
  __Pyx_DECREF(__pyx_v_self->source);
  __pyx_v_self->source = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_11__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_8zeroconf_9_protocol_8incoming_11DNSIncoming_11__reduce_cython__ = {"__reduce_cython__", (PyCFunction)__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_11__reduce_cython__, METH_NOARGS, 0};
static PyObject *__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_11__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_10__reduce_cython__(((struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_10__reduce_cython__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  int __pyx_t_11;
  int __pyx_t_12;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._answers, self._data_len, self._did_read_others, self.data, self.flags, self.id, self.name_cache, self.now, self.num_additionals, self.num_answers, self.num_authorities, self.num_questions, self.offset, self.questions, self.scope_id, self.source, self.valid)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyInt_From_unsigned_int(__pyx_v_self->_data_len); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->_did_read_others); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_unsigned_int(__pyx_v_self->flags); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_unsigned_int(__pyx_v_self->num_additionals); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyInt_From_unsigned_int(__pyx_v_self->num_answers); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_PyInt_From_unsigned_int(__pyx_v_self->num_authorities); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_PyInt_From_unsigned_int(__pyx_v_self->num_questions); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = __Pyx_PyInt_From_unsigned_int(__pyx_v_self->offset); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_9 = PyTuple_New(17); if (unlikely(!__pyx_t_9)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_INCREF(__pyx_v_self->_answers);
  __Pyx_GIVEREF(__pyx_v_self->_answers);
  PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_v_self->_answers);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_9, 2, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->data);
  __Pyx_GIVEREF(__pyx_v_self->data);
  PyTuple_SET_ITEM(__pyx_t_9, 3, __pyx_v_self->data);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_9, 4, __pyx_t_3);
  __Pyx_INCREF(__pyx_v_self->id);
  __Pyx_GIVEREF(__pyx_v_self->id);
  PyTuple_SET_ITEM(__pyx_t_9, 5, __pyx_v_self->id);
  __Pyx_INCREF(__pyx_v_self->name_cache);
  __Pyx_GIVEREF(__pyx_v_self->name_cache);
  PyTuple_SET_ITEM(__pyx_t_9, 6, __pyx_v_self->name_cache);
  __Pyx_INCREF(__pyx_v_self->now);
  __Pyx_GIVEREF(__pyx_v_self->now);
  PyTuple_SET_ITEM(__pyx_t_9, 7, __pyx_v_self->now);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_9, 8, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_9, 9, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_9, 10, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_9, 11, __pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_8);
  PyTuple_SET_ITEM(__pyx_t_9, 12, __pyx_t_8);
  __Pyx_INCREF(__pyx_v_self->questions);
  __Pyx_GIVEREF(__pyx_v_self->questions);
  PyTuple_SET_ITEM(__pyx_t_9, 13, __pyx_v_self->questions);
  __Pyx_INCREF(__pyx_v_self->scope_id);
  __Pyx_GIVEREF(__pyx_v_self->scope_id);
  PyTuple_SET_ITEM(__pyx_t_9, 14, __pyx_v_self->scope_id);
  __Pyx_INCREF(__pyx_v_self->source);
  __Pyx_GIVEREF(__pyx_v_self->source);
  PyTuple_SET_ITEM(__pyx_t_9, 15, __pyx_v_self->source);
  __Pyx_INCREF(__pyx_v_self->valid);
  __Pyx_GIVEREF(__pyx_v_self->valid);
  PyTuple_SET_ITEM(__pyx_t_9, 16, __pyx_v_self->valid);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_t_6 = 0;
  __pyx_t_7 = 0;
  __pyx_t_8 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_9);
  __pyx_t_9 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._answers, self._data_len, self._did_read_others, self.data, self.flags, self.id, self.name_cache, self.now, self.num_additionals, self.num_answers, self.num_authorities, self.num_questions, self.offset, self.questions, self.scope_id, self.source, self.valid)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_9 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_9)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_v__dict = __pyx_t_9;
  __pyx_t_9 = 0;

  /* "(tree fragment)":7
 *     state = (self._answers, self._data_len, self._did_read_others, self.data, self.flags, self.id, self.name_cache, self.now, self.num_additionals, self.num_answers, self.num_authorities, self.num_questions, self.offset, self.questions, self.scope_id, self.source, self.valid)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_10 = (__pyx_v__dict != Py_None);
  __pyx_t_11 = (__pyx_t_10 != 0);
  if (__pyx_t_11) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_9 = PyTuple_New(1); if (unlikely(!__pyx_t_9)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_v__dict);
    __pyx_t_8 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_9); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_8));
    __pyx_t_8 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._answers is not None or self.data is not None or self.id is not None or self.name_cache is not None or self.now is not None or self.questions is not None or self.scope_id is not None or self.source is not None or self.valid is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._answers, self._data_len, self._did_read_others, self.data, self.flags, self.id, self.name_cache, self.now, self.num_additionals, self.num_answers, self.num_authorities, self.num_questions, self.offset, self.questions, self.scope_id, self.source, self.valid)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._answers is not None or self.data is not None or self.id is not None or self.name_cache is not None or self.now is not None or self.questions is not None or self.scope_id is not None or self.source is not None or self.valid is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_DNSIncoming, (type(self), 0x69f643b, None), state
 */
  /*else*/ {
    __pyx_t_10 = (__pyx_v_self->_answers != ((PyObject*)Py_None));
    __pyx_t_12 = (__pyx_t_10 != 0);
    if (!__pyx_t_12) {
    } else {
      __pyx_t_11 = __pyx_t_12;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_12 = (__pyx_v_self->data != ((PyObject*)Py_None));
    __pyx_t_10 = (__pyx_t_12 != 0);
    if (!__pyx_t_10) {
    } else {
      __pyx_t_11 = __pyx_t_10;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_10 = (__pyx_v_self->id != Py_None);
    __pyx_t_12 = (__pyx_t_10 != 0);
    if (!__pyx_t_12) {
    } else {
      __pyx_t_11 = __pyx_t_12;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_12 = (__pyx_v_self->name_cache != ((PyObject*)Py_None));
    __pyx_t_10 = (__pyx_t_12 != 0);
    if (!__pyx_t_10) {
    } else {
      __pyx_t_11 = __pyx_t_10;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_10 = (__pyx_v_self->now != Py_None);
    __pyx_t_12 = (__pyx_t_10 != 0);
    if (!__pyx_t_12) {
    } else {
      __pyx_t_11 = __pyx_t_12;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_12 = (__pyx_v_self->questions != ((PyObject*)Py_None));
    __pyx_t_10 = (__pyx_t_12 != 0);
    if (!__pyx_t_10) {
    } else {
      __pyx_t_11 = __pyx_t_10;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_10 = (__pyx_v_self->scope_id != Py_None);
    __pyx_t_12 = (__pyx_t_10 != 0);
    if (!__pyx_t_12) {
    } else {
      __pyx_t_11 = __pyx_t_12;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_12 = (__pyx_v_self->source != Py_None);
    __pyx_t_10 = (__pyx_t_12 != 0);
    if (!__pyx_t_10) {
    } else {
      __pyx_t_11 = __pyx_t_10;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_10 = (__pyx_v_self->valid != Py_None);
    __pyx_t_12 = (__pyx_t_10 != 0);
    __pyx_t_11 = __pyx_t_12;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_11;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._answers is not None or self.data is not None or self.id is not None or self.name_cache is not None or self.now is not None or self.questions is not None or self.scope_id is not None or self.source is not None or self.valid is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_DNSIncoming, (type(self), 0x69f643b, None), state
 *     else:
 */
  __pyx_t_11 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_11) {

    /* "(tree fragment)":13
 *         use_setstate = self._answers is not None or self.data is not None or self.id is not None or self.name_cache is not None or self.now is not None or self.questions is not None or self.scope_id is not None or self.source is not None or self.valid is not None
 *     if use_setstate:
 *         return __pyx_unpickle_DNSIncoming, (type(self), 0x69f643b, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_DNSIncoming, (type(self), 0x69f643b, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_n_s_pyx_unpickle_DNSIncoming); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = PyTuple_New(3); if (unlikely(!__pyx_t_9)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_9, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_111109179);
    __Pyx_GIVEREF(__pyx_int_111109179);
    PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_int_111109179);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_9, 2, Py_None);
    __pyx_t_7 = PyTuple_New(3); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_8);
    __Pyx_GIVEREF(__pyx_t_9);
    PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_t_9);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_7, 2, __pyx_v_state);
    __pyx_t_8 = 0;
    __pyx_t_9 = 0;
    __pyx_r = __pyx_t_7;
    __pyx_t_7 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._answers is not None or self.data is not None or self.id is not None or self.name_cache is not None or self.now is not None or self.questions is not None or self.scope_id is not None or self.source is not None or self.valid is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_DNSIncoming, (type(self), 0x69f643b, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_DNSIncoming, (type(self), 0x69f643b, None), state
 *     else:
 *         return __pyx_unpickle_DNSIncoming, (type(self), 0x69f643b, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_DNSIncoming__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_pyx_unpickle_DNSIncoming); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = PyTuple_New(3); if (unlikely(!__pyx_t_9)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_9, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_111109179);
    __Pyx_GIVEREF(__pyx_int_111109179);
    PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_int_111109179);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_9, 2, __pyx_v_state);
    __pyx_t_8 = PyTuple_New(2); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_7);
    __Pyx_GIVEREF(__pyx_t_9);
    PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_t_9);
    __pyx_t_7 = 0;
    __pyx_t_9 = 0;
    __pyx_r = __pyx_t_8;
    __pyx_t_8 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("zeroconf._protocol.incoming.DNSIncoming.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_DNSIncoming, (type(self), 0x69f643b, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_DNSIncoming__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_13__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyMethodDef __pyx_mdef_8zeroconf_9_protocol_8incoming_11DNSIncoming_13__setstate_cython__ = {"__setstate_cython__", (PyCFunction)__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_13__setstate_cython__, METH_O, 0};
static PyObject *__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_13__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_12__setstate_cython__(((struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8incoming_11DNSIncoming_12__setstate_cython__(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_DNSIncoming, (type(self), 0x69f643b, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_DNSIncoming__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||((void)PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_8zeroconf_9_protocol_8incoming___pyx_unpickle_DNSIncoming__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_DNSIncoming, (type(self), 0x69f643b, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_DNSIncoming__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("zeroconf._protocol.incoming.DNSIncoming.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_DNSIncoming(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_8zeroconf_9_protocol_8incoming_1__pyx_unpickle_DNSIncoming(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_8zeroconf_9_protocol_8incoming_1__pyx_unpickle_DNSIncoming = {"__pyx_unpickle_DNSIncoming", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_8zeroconf_9_protocol_8incoming_1__pyx_unpickle_DNSIncoming, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_8zeroconf_9_protocol_8incoming_1__pyx_unpickle_DNSIncoming(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_DNSIncoming (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_DNSIncoming", 1, 3, 3, 1); __PYX_ERR(2, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_DNSIncoming", 1, 3, 3, 2); __PYX_ERR(2, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_DNSIncoming") < 0)) __PYX_ERR(2, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_DNSIncoming", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("zeroconf._protocol.incoming.__pyx_unpickle_DNSIncoming", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_8zeroconf_9_protocol_8incoming___pyx_unpickle_DNSIncoming(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8zeroconf_9_protocol_8incoming___pyx_unpickle_DNSIncoming(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_DNSIncoming", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0x69f643b, 0xcd74736, 0xd8a590c):             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0x69f643b, 0xcd74736, 0xd8a590c) = (_answers, _data_len, _did_read_others, data, flags, id, name_cache, now, num_additionals, num_answers, num_authorities, num_questions, offset, questions, scope_id, source, valid))" % __pyx_checksum)
 */
  __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = (__Pyx_PySequence_ContainsTF(__pyx_t_1, __pyx_tuple__3, Py_NE)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(2, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0x69f643b, 0xcd74736, 0xd8a590c):
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0x69f643b, 0xcd74736, 0xd8a590c) = (_answers, _data_len, _did_read_others, data, flags, id, name_cache, now, num_additionals, num_answers, num_authorities, num_questions, offset, questions, scope_id, source, valid))" % __pyx_checksum)
 *     __pyx_result = DNSIncoming.__new__(__pyx_type)
 */
    __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_PickleError);
    __pyx_t_4 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_1, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_4, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_t_1);
    __pyx_v___pyx_PickleError = __pyx_t_1;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum not in (0x69f643b, 0xcd74736, 0xd8a590c):
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0x69f643b, 0xcd74736, 0xd8a590c) = (_answers, _data_len, _did_read_others, data, flags, id, name_cache, now, num_additionals, num_answers, num_authorities, num_questions, offset, questions, scope_id, source, valid))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = DNSIncoming.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_0x_x_vs_0, __pyx_t_1); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_1 = __pyx_v___pyx_PickleError; __pyx_t_6 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
      }
    }
    __pyx_t_4 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_6, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_5);
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(2, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0x69f643b, 0xcd74736, 0xd8a590c):             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0x69f643b, 0xcd74736, 0xd8a590c) = (_answers, _data_len, _did_read_others, data, flags, id, name_cache, now, num_additionals, num_answers, num_authorities, num_questions, offset, questions, scope_id, source, valid))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0x69f643b, 0xcd74736, 0xd8a590c) = (_answers, _data_len, _did_read_others, data, flags, id, name_cache, now, num_additionals, num_answers, num_authorities, num_questions, offset, questions, scope_id, source, valid))" % __pyx_checksum)
 *     __pyx_result = DNSIncoming.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_DNSIncoming__set_state(<DNSIncoming> __pyx_result, __pyx_state)
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_8zeroconf_9_protocol_8incoming_DNSIncoming), __pyx_n_s_new); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_4 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_5, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0x69f643b, 0xcd74736, 0xd8a590c) = (_answers, _data_len, _did_read_others, data, flags, id, name_cache, now, num_additionals, num_answers, num_authorities, num_questions, offset, questions, scope_id, source, valid))" % __pyx_checksum)
 *     __pyx_result = DNSIncoming.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_DNSIncoming__set_state(<DNSIncoming> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_3 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {

    /* "(tree fragment)":9
 *     __pyx_result = DNSIncoming.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_DNSIncoming__set_state(<DNSIncoming> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_DNSIncoming__set_state(DNSIncoming __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||((void)PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 9, __pyx_L1_error)
    __pyx_t_4 = __pyx_f_8zeroconf_9_protocol_8incoming___pyx_unpickle_DNSIncoming__set_state(((struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0x69f643b, 0xcd74736, 0xd8a590c) = (_answers, _data_len, _did_read_others, data, flags, id, name_cache, now, num_additionals, num_answers, num_authorities, num_questions, offset, questions, scope_id, source, valid))" % __pyx_checksum)
 *     __pyx_result = DNSIncoming.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_DNSIncoming__set_state(<DNSIncoming> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_DNSIncoming__set_state(<DNSIncoming> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_DNSIncoming__set_state(DNSIncoming __pyx_result, tuple __pyx_state):
 *     __pyx_result._answers = __pyx_state[0]; __pyx_result._data_len = __pyx_state[1]; __pyx_result._did_read_others = __pyx_state[2]; __pyx_result.data = __pyx_state[3]; __pyx_result.flags = __pyx_state[4]; __pyx_result.id = __pyx_state[5]; __pyx_result.name_cache = __pyx_state[6]; __pyx_result.now = __pyx_state[7]; __pyx_result.num_additionals = __pyx_state[8]; __pyx_result.num_answers = __pyx_state[9]; __pyx_result.num_authorities = __pyx_state[10]; __pyx_result.num_questions = __pyx_state[11]; __pyx_result.offset = __pyx_state[12]; __pyx_result.questions = __pyx_state[13]; __pyx_result.scope_id = __pyx_state[14]; __pyx_result.source = __pyx_state[15]; __pyx_result.valid = __pyx_state[16]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_DNSIncoming(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("zeroconf._protocol.incoming.__pyx_unpickle_DNSIncoming", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_DNSIncoming__set_state(<DNSIncoming> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_DNSIncoming__set_state(DNSIncoming __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._answers = __pyx_state[0]; __pyx_result._data_len = __pyx_state[1]; __pyx_result._did_read_others = __pyx_state[2]; __pyx_result.data = __pyx_state[3]; __pyx_result.flags = __pyx_state[4]; __pyx_result.id = __pyx_state[5]; __pyx_result.name_cache = __pyx_state[6]; __pyx_result.now = __pyx_state[7]; __pyx_result.num_additionals = __pyx_state[8]; __pyx_result.num_answers = __pyx_state[9]; __pyx_result.num_authorities = __pyx_state[10]; __pyx_result.num_questions = __pyx_state[11]; __pyx_result.offset = __pyx_state[12]; __pyx_result.questions = __pyx_state[13]; __pyx_result.scope_id = __pyx_state[14]; __pyx_result.source = __pyx_state[15]; __pyx_result.valid = __pyx_state[16]
 *     if len(__pyx_state) > 17 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_8zeroconf_9_protocol_8incoming___pyx_unpickle_DNSIncoming__set_state(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  unsigned int __pyx_t_2;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_DNSIncoming__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_DNSIncoming__set_state(DNSIncoming __pyx_result, tuple __pyx_state):
 *     __pyx_result._answers = __pyx_state[0]; __pyx_result._data_len = __pyx_state[1]; __pyx_result._did_read_others = __pyx_state[2]; __pyx_result.data = __pyx_state[3]; __pyx_result.flags = __pyx_state[4]; __pyx_result.id = __pyx_state[5]; __pyx_result.name_cache = __pyx_state[6]; __pyx_result.now = __pyx_state[7]; __pyx_result.num_additionals = __pyx_state[8]; __pyx_result.num_answers = __pyx_state[9]; __pyx_result.num_authorities = __pyx_state[10]; __pyx_result.num_questions = __pyx_state[11]; __pyx_result.offset = __pyx_state[12]; __pyx_result.questions = __pyx_state[13]; __pyx_result.scope_id = __pyx_state[14]; __pyx_result.source = __pyx_state[15]; __pyx_result.valid = __pyx_state[16]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 17 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[17])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||((void)PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_answers);
  __Pyx_DECREF(__pyx_v___pyx_result->_answers);
  __pyx_v___pyx_result->_answers = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_unsigned_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_data_len = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_did_read_others = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||((void)PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->data);
  __Pyx_DECREF(__pyx_v___pyx_result->data);
  __pyx_v___pyx_result->data = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_unsigned_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->flags = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->id);
  __Pyx_DECREF(__pyx_v___pyx_result->id);
  __pyx_v___pyx_result->id = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||((void)PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->name_cache);
  __Pyx_DECREF(__pyx_v___pyx_result->name_cache);
  __pyx_v___pyx_result->name_cache = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->now);
  __Pyx_DECREF(__pyx_v___pyx_result->now);
  __pyx_v___pyx_result->now = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_unsigned_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->num_additionals = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_unsigned_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->num_answers = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 10, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_unsigned_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->num_authorities = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 11, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_unsigned_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->num_questions = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 12, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_unsigned_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->offset = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 13, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||((void)PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->questions);
  __Pyx_DECREF(__pyx_v___pyx_result->questions);
  __pyx_v___pyx_result->questions = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 14, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->scope_id);
  __Pyx_DECREF(__pyx_v___pyx_result->scope_id);
  __pyx_v___pyx_result->scope_id = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 15, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->source);
  __Pyx_DECREF(__pyx_v___pyx_result->source);
  __pyx_v___pyx_result->source = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 16, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->valid);
  __Pyx_DECREF(__pyx_v___pyx_result->valid);
  __pyx_v___pyx_result->valid = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_DNSIncoming__set_state(DNSIncoming __pyx_result, tuple __pyx_state):
 *     __pyx_result._answers = __pyx_state[0]; __pyx_result._data_len = __pyx_state[1]; __pyx_result._did_read_others = __pyx_state[2]; __pyx_result.data = __pyx_state[3]; __pyx_result.flags = __pyx_state[4]; __pyx_result.id = __pyx_state[5]; __pyx_result.name_cache = __pyx_state[6]; __pyx_result.now = __pyx_state[7]; __pyx_result.num_additionals = __pyx_state[8]; __pyx_result.num_answers = __pyx_state[9]; __pyx_result.num_authorities = __pyx_state[10]; __pyx_result.num_questions = __pyx_state[11]; __pyx_result.offset = __pyx_state[12]; __pyx_result.questions = __pyx_state[13]; __pyx_result.scope_id = __pyx_state[14]; __pyx_result.source = __pyx_state[15]; __pyx_result.valid = __pyx_state[16]
 *     if len(__pyx_state) > 17 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[17])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 13, __pyx_L1_error)
  }
  __pyx_t_4 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_5 = ((__pyx_t_4 > 17) != 0);
  if (__pyx_t_5) {
  } else {
    __pyx_t_3 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_6 = (__pyx_t_5 != 0);
  __pyx_t_3 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_3) {

    /* "(tree fragment)":14
 *     __pyx_result._answers = __pyx_state[0]; __pyx_result._data_len = __pyx_state[1]; __pyx_result._did_read_others = __pyx_state[2]; __pyx_result.data = __pyx_state[3]; __pyx_result.flags = __pyx_state[4]; __pyx_result.id = __pyx_state[5]; __pyx_result.name_cache = __pyx_state[6]; __pyx_result.now = __pyx_state[7]; __pyx_result.num_additionals = __pyx_state[8]; __pyx_result.num_answers = __pyx_state[9]; __pyx_result.num_authorities = __pyx_state[10]; __pyx_result.num_questions = __pyx_state[11]; __pyx_result.offset = __pyx_state[12]; __pyx_result.questions = __pyx_state[13]; __pyx_result.scope_id = __pyx_state[14]; __pyx_result.source = __pyx_state[15]; __pyx_result.valid = __pyx_state[16]
 *     if len(__pyx_state) > 17 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[17])             # <<<<<<<<<<<<<<
 */
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_update); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 14, __pyx_L1_error)
    }
    __pyx_t_7 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 17, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
      }
    }
    __pyx_t_1 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_8, __pyx_t_9, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_7);
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_DNSIncoming__set_state(DNSIncoming __pyx_result, tuple __pyx_state):
 *     __pyx_result._answers = __pyx_state[0]; __pyx_result._data_len = __pyx_state[1]; __pyx_result._did_read_others = __pyx_state[2]; __pyx_result.data = __pyx_state[3]; __pyx_result.flags = __pyx_state[4]; __pyx_result.id = __pyx_state[5]; __pyx_result.name_cache = __pyx_state[6]; __pyx_result.now = __pyx_state[7]; __pyx_result.num_additionals = __pyx_state[8]; __pyx_result.num_answers = __pyx_state[9]; __pyx_result.num_authorities = __pyx_state[10]; __pyx_result.num_questions = __pyx_state[11]; __pyx_result.offset = __pyx_state[12]; __pyx_result.questions = __pyx_state[13]; __pyx_result.scope_id = __pyx_state[14]; __pyx_result.source = __pyx_state[15]; __pyx_result.valid = __pyx_state[16]
 *     if len(__pyx_state) > 17 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[17])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_DNSIncoming__set_state(<DNSIncoming> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_DNSIncoming__set_state(DNSIncoming __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._answers = __pyx_state[0]; __pyx_result._data_len = __pyx_state[1]; __pyx_result._did_read_others = __pyx_state[2]; __pyx_result.data = __pyx_state[3]; __pyx_result.flags = __pyx_state[4]; __pyx_result.id = __pyx_state[5]; __pyx_result.name_cache = __pyx_state[6]; __pyx_result.now = __pyx_state[7]; __pyx_result.num_additionals = __pyx_state[8]; __pyx_result.num_answers = __pyx_state[9]; __pyx_result.num_authorities = __pyx_state[10]; __pyx_result.num_questions = __pyx_state[11]; __pyx_result.offset = __pyx_state[12]; __pyx_result.questions = __pyx_state[13]; __pyx_result.scope_id = __pyx_state[14]; __pyx_result.source = __pyx_state[15]; __pyx_result.valid = __pyx_state[16]
 *     if len(__pyx_state) > 17 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("zeroconf._protocol.incoming.__pyx_unpickle_DNSIncoming__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static struct __pyx_vtabstruct_8zeroconf_9_protocol_8incoming_DNSIncoming __pyx_vtable_8zeroconf_9_protocol_8incoming_DNSIncoming;

static PyObject *__pyx_tp_new_8zeroconf_9_protocol_8incoming_DNSIncoming(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *)o);
  p->__pyx_vtab = __pyx_vtabptr_8zeroconf_9_protocol_8incoming_DNSIncoming;
  p->data = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->name_cache = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->questions = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->_answers = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->id = Py_None; Py_INCREF(Py_None);
  p->valid = Py_None; Py_INCREF(Py_None);
  p->now = Py_None; Py_INCREF(Py_None);
  p->scope_id = Py_None; Py_INCREF(Py_None);
  p->source = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_8zeroconf_9_protocol_8incoming_DNSIncoming(PyObject *o) {
  struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *p = (struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->data);
  Py_CLEAR(p->name_cache);
  Py_CLEAR(p->questions);
  Py_CLEAR(p->_answers);
  Py_CLEAR(p->id);
  Py_CLEAR(p->valid);
  Py_CLEAR(p->now);
  Py_CLEAR(p->scope_id);
  Py_CLEAR(p->source);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_8zeroconf_9_protocol_8incoming_DNSIncoming(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *p = (struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *)o;
  if (p->name_cache) {
    e = (*v)(p->name_cache, a); if (e) return e;
  }
  if (p->questions) {
    e = (*v)(p->questions, a); if (e) return e;
  }
  if (p->_answers) {
    e = (*v)(p->_answers, a); if (e) return e;
  }
  if (p->id) {
    e = (*v)(p->id, a); if (e) return e;
  }
  if (p->valid) {
    e = (*v)(p->valid, a); if (e) return e;
  }
  if (p->now) {
    e = (*v)(p->now, a); if (e) return e;
  }
  if (p->scope_id) {
    e = (*v)(p->scope_id, a); if (e) return e;
  }
  if (p->source) {
    e = (*v)(p->source, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_8zeroconf_9_protocol_8incoming_DNSIncoming(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *p = (struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *)o;
  tmp = ((PyObject*)p->name_cache);
  p->name_cache = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->questions);
  p->questions = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_answers);
  p->_answers = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->id);
  p->id = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->valid);
  p->valid = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->now);
  p->now = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->scope_id);
  p->scope_id = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->source);
  p->source = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_truncated(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_9truncated_1__get__(o);
}

static PyObject *__pyx_getprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_answers(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_7answers_1__get__(o);
}

static PyObject *__pyx_getprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_is_probe(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_8is_probe_1__get__(o);
}

static PyObject *__pyx_getprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_flags(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_5flags_1__get__(o);
}

static int __pyx_setprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_flags(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_5flags_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_data(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_4data_1__get__(o);
}

static int __pyx_setprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_data(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_4data_3__set__(o, v);
  }
  else {
    return __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_4data_5__del__(o);
  }
}

static PyObject *__pyx_getprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_name_cache(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_10name_cache_1__get__(o);
}

static int __pyx_setprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_name_cache(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_10name_cache_3__set__(o, v);
  }
  else {
    return __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_10name_cache_5__del__(o);
  }
}

static PyObject *__pyx_getprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_questions(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_9questions_1__get__(o);
}

static int __pyx_setprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_questions(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_9questions_3__set__(o, v);
  }
  else {
    return __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_9questions_5__del__(o);
  }
}

static PyObject *__pyx_getprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_id(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_2id_1__get__(o);
}

static int __pyx_setprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_id(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_2id_3__set__(o, v);
  }
  else {
    return __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_2id_5__del__(o);
  }
}

static PyObject *__pyx_getprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_num_questions(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_13num_questions_1__get__(o);
}

static int __pyx_setprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_num_questions(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_13num_questions_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_num_answers(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_11num_answers_1__get__(o);
}

static int __pyx_setprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_num_answers(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_11num_answers_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_num_authorities(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_15num_authorities_1__get__(o);
}

static int __pyx_setprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_num_authorities(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_15num_authorities_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_num_additionals(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_15num_additionals_1__get__(o);
}

static int __pyx_setprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_num_additionals(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_15num_additionals_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_valid(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_5valid_1__get__(o);
}

static int __pyx_setprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_valid(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_5valid_3__set__(o, v);
  }
  else {
    return __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_5valid_5__del__(o);
  }
}

static PyObject *__pyx_getprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_now(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_3now_1__get__(o);
}

static int __pyx_setprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_now(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_3now_3__set__(o, v);
  }
  else {
    return __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_3now_5__del__(o);
  }
}

static PyObject *__pyx_getprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_scope_id(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_8scope_id_1__get__(o);
}

static int __pyx_setprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_scope_id(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_8scope_id_3__set__(o, v);
  }
  else {
    return __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_8scope_id_5__del__(o);
  }
}

static PyObject *__pyx_getprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_source(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_6source_1__get__(o);
}

static int __pyx_setprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_source(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_6source_3__set__(o, v);
  }
  else {
    return __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_6source_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_8zeroconf_9_protocol_8incoming_DNSIncoming[] = {
  {"is_query", (PyCFunction)__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_3is_query, METH_NOARGS, __pyx_doc_8zeroconf_9_protocol_8incoming_11DNSIncoming_2is_query},
  {"is_response", (PyCFunction)__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_5is_response, METH_NOARGS, __pyx_doc_8zeroconf_9_protocol_8incoming_11DNSIncoming_4is_response},
  {"_log_exception_debug", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_7_log_exception_debug, METH_VARARGS|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_11__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_13__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_8zeroconf_9_protocol_8incoming_DNSIncoming[] = {
  {(char *)"truncated", __pyx_getprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_truncated, 0, (char *)"Returns true if this is a truncated.", 0},
  {(char *)"answers", __pyx_getprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_answers, 0, (char *)"Answers in the packet.", 0},
  {(char *)"is_probe", __pyx_getprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_is_probe, 0, (char *)"Returns true if this is a probe.", 0},
  {(char *)"flags", __pyx_getprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_flags, __pyx_setprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_flags, (char *)0, 0},
  {(char *)"data", __pyx_getprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_data, __pyx_setprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_data, (char *)0, 0},
  {(char *)"name_cache", __pyx_getprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_name_cache, __pyx_setprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_name_cache, (char *)0, 0},
  {(char *)"questions", __pyx_getprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_questions, __pyx_setprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_questions, (char *)0, 0},
  {(char *)"id", __pyx_getprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_id, __pyx_setprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_id, (char *)0, 0},
  {(char *)"num_questions", __pyx_getprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_num_questions, __pyx_setprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_num_questions, (char *)0, 0},
  {(char *)"num_answers", __pyx_getprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_num_answers, __pyx_setprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_num_answers, (char *)0, 0},
  {(char *)"num_authorities", __pyx_getprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_num_authorities, __pyx_setprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_num_authorities, (char *)0, 0},
  {(char *)"num_additionals", __pyx_getprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_num_additionals, __pyx_setprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_num_additionals, (char *)0, 0},
  {(char *)"valid", __pyx_getprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_valid, __pyx_setprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_valid, (char *)0, 0},
  {(char *)"now", __pyx_getprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_now, __pyx_setprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_now, (char *)0, 0},
  {(char *)"scope_id", __pyx_getprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_scope_id, __pyx_setprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_scope_id, (char *)0, 0},
  {(char *)"source", __pyx_getprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_source, __pyx_setprop_8zeroconf_9_protocol_8incoming_11DNSIncoming_source, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_8zeroconf_9_protocol_8incoming_DNSIncoming = {
  PyVarObject_HEAD_INIT(0, 0)
  "zeroconf._protocol.incoming.DNSIncoming", /*tp_name*/
  sizeof(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_8zeroconf_9_protocol_8incoming_DNSIncoming, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_9__repr__, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "Object representation of an incoming DNS packet", /*tp_doc*/
  __pyx_tp_traverse_8zeroconf_9_protocol_8incoming_DNSIncoming, /*tp_traverse*/
  __pyx_tp_clear_8zeroconf_9_protocol_8incoming_DNSIncoming, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_8zeroconf_9_protocol_8incoming_DNSIncoming, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_8zeroconf_9_protocol_8incoming_DNSIncoming, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_8zeroconf_9_protocol_8incoming_11DNSIncoming_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_8zeroconf_9_protocol_8incoming_DNSIncoming, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_incoming(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_incoming},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "incoming",
    __pyx_k_Multicast_DNS_Service_Discovery, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_kp_u_, __pyx_k_, sizeof(__pyx_k_), 0, 1, 0, 0},
  {&__pyx_kp_b_3H, __pyx_k_3H, sizeof(__pyx_k_3H), 0, 0, 0, 0},
  {&__pyx_kp_b_6H, __pyx_k_6H, sizeof(__pyx_k_6H), 0, 0, 0, 0},
  {&__pyx_n_s_Any, __pyx_k_Any, sizeof(__pyx_k_Any), 0, 0, 1, 1},
  {&__pyx_kp_u_Corrupt_packet_received_while_de, __pyx_k_Corrupt_packet_received_while_de, sizeof(__pyx_k_Corrupt_packet_received_while_de), 0, 1, 0, 0},
  {&__pyx_n_s_DECODE_EXCEPTIONS, __pyx_k_DECODE_EXCEPTIONS, sizeof(__pyx_k_DECODE_EXCEPTIONS), 0, 0, 1, 1},
  {&__pyx_n_s_DNSAddress, __pyx_k_DNSAddress, sizeof(__pyx_k_DNSAddress), 0, 0, 1, 1},
  {&__pyx_n_s_DNSHinfo, __pyx_k_DNSHinfo, sizeof(__pyx_k_DNSHinfo), 0, 0, 1, 1},
  {&__pyx_n_s_DNSIncoming, __pyx_k_DNSIncoming, sizeof(__pyx_k_DNSIncoming), 0, 0, 1, 1},
  {&__pyx_n_s_DNSIncoming___reduce_cython, __pyx_k_DNSIncoming___reduce_cython, sizeof(__pyx_k_DNSIncoming___reduce_cython), 0, 0, 1, 1},
  {&__pyx_n_s_DNSIncoming___setstate_cython, __pyx_k_DNSIncoming___setstate_cython, sizeof(__pyx_k_DNSIncoming___setstate_cython), 0, 0, 1, 1},
  {&__pyx_n_s_DNSIncoming__log_exception_debug, __pyx_k_DNSIncoming__log_exception_debug, sizeof(__pyx_k_DNSIncoming__log_exception_debug), 0, 0, 1, 1},
  {&__pyx_n_s_DNSIncoming_is_query, __pyx_k_DNSIncoming_is_query, sizeof(__pyx_k_DNSIncoming_is_query), 0, 0, 1, 1},
  {&__pyx_n_s_DNSIncoming_is_response, __pyx_k_DNSIncoming_is_response, sizeof(__pyx_k_DNSIncoming_is_response), 0, 0, 1, 1},
  {&__pyx_kp_u_DNSIncoming_s, __pyx_k_DNSIncoming_s, sizeof(__pyx_k_DNSIncoming_s), 0, 1, 0, 0},
  {&__pyx_n_s_DNSNsec, __pyx_k_DNSNsec, sizeof(__pyx_k_DNSNsec), 0, 0, 1, 1},
  {&__pyx_n_s_DNSPointer, __pyx_k_DNSPointer, sizeof(__pyx_k_DNSPointer), 0, 0, 1, 1},
  {&__pyx_n_s_DNSQuestion, __pyx_k_DNSQuestion, sizeof(__pyx_k_DNSQuestion), 0, 0, 1, 1},
  {&__pyx_n_s_DNSRecord, __pyx_k_DNSRecord, sizeof(__pyx_k_DNSRecord), 0, 0, 1, 1},
  {&__pyx_n_s_DNSService, __pyx_k_DNSService, sizeof(__pyx_k_DNSService), 0, 0, 1, 1},
  {&__pyx_n_s_DNSText, __pyx_k_DNSText, sizeof(__pyx_k_DNSText), 0, 0, 1, 1},
  {&__pyx_n_s_DNS_COMPRESSION_HEADER_LEN, __pyx_k_DNS_COMPRESSION_HEADER_LEN, sizeof(__pyx_k_DNS_COMPRESSION_HEADER_LEN), 0, 0, 1, 1},
  {&__pyx_n_s_DNS_COMPRESSION_POINTER_LEN, __pyx_k_DNS_COMPRESSION_POINTER_LEN, sizeof(__pyx_k_DNS_COMPRESSION_POINTER_LEN), 0, 0, 1, 1},
  {&__pyx_kp_u_DNS_compression_pointer_at, __pyx_k_DNS_compression_pointer_at, sizeof(__pyx_k_DNS_compression_pointer_at), 0, 1, 0, 0},
  {&__pyx_kp_u_DNS_compression_type, __pyx_k_DNS_compression_type, sizeof(__pyx_k_DNS_compression_type), 0, 1, 0, 0},
  {&__pyx_kp_u_DNS_name, __pyx_k_DNS_name, sizeof(__pyx_k_DNS_name), 0, 1, 0, 0},
  {&__pyx_n_s_Dict, __pyx_k_Dict, sizeof(__pyx_k_Dict), 0, 0, 1, 1},
  {&__pyx_kp_u_Exception_occurred, __pyx_k_Exception_occurred, sizeof(__pyx_k_Exception_occurred), 0, 1, 0, 0},
  {&__pyx_n_s_FLAGS_QR_MASK, __pyx_k_FLAGS_QR_MASK, sizeof(__pyx_k_FLAGS_QR_MASK), 0, 0, 1, 1},
  {&__pyx_n_s_FLAGS_QR_QUERY, __pyx_k_FLAGS_QR_QUERY, sizeof(__pyx_k_FLAGS_QR_QUERY), 0, 0, 1, 1},
  {&__pyx_n_s_FLAGS_QR_RESPONSE, __pyx_k_FLAGS_QR_RESPONSE, sizeof(__pyx_k_FLAGS_QR_RESPONSE), 0, 0, 1, 1},
  {&__pyx_n_s_FLAGS_TC, __pyx_k_FLAGS_TC, sizeof(__pyx_k_FLAGS_TC), 0, 0, 1, 1},
  {&__pyx_kp_b_HH, __pyx_k_HH, sizeof(__pyx_k_HH), 0, 0, 0, 0},
  {&__pyx_kp_b_HHiH, __pyx_k_HHiH, sizeof(__pyx_k_HHiH), 0, 0, 0, 0},
  {&__pyx_n_s_IncomingDecodeError, __pyx_k_IncomingDecodeError, sizeof(__pyx_k_IncomingDecodeError), 0, 0, 1, 1},
  {&__pyx_kp_s_Incompatible_checksums_0x_x_vs_0, __pyx_k_Incompatible_checksums_0x_x_vs_0, sizeof(__pyx_k_Incompatible_checksums_0x_x_vs_0), 0, 0, 1, 0},
  {&__pyx_n_s_IndexError, __pyx_k_IndexError, sizeof(__pyx_k_IndexError), 0, 0, 1, 1},
  {&__pyx_n_s_List, __pyx_k_List, sizeof(__pyx_k_List), 0, 0, 1, 1},
  {&__pyx_n_s_MAX_DNS_LABELS, __pyx_k_MAX_DNS_LABELS, sizeof(__pyx_k_MAX_DNS_LABELS), 0, 0, 1, 1},
  {&__pyx_n_s_MAX_NAME_LENGTH, __pyx_k_MAX_NAME_LENGTH, sizeof(__pyx_k_MAX_NAME_LENGTH), 0, 0, 1, 1},
  {&__pyx_kp_u_Maximum_dns_labels_reached_while, __pyx_k_Maximum_dns_labels_reached_while, sizeof(__pyx_k_Maximum_dns_labels_reached_while), 0, 1, 0, 0},
  {&__pyx_n_s_Optional, __pyx_k_Optional, sizeof(__pyx_k_Optional), 0, 0, 1, 1},
  {&__pyx_n_s_PickleError, __pyx_k_PickleError, sizeof(__pyx_k_PickleError), 0, 0, 1, 1},
  {&__pyx_kp_u_Received_invalid_packet_from_s_a, __pyx_k_Received_invalid_packet_from_s_a, sizeof(__pyx_k_Received_invalid_packet_from_s_a), 0, 1, 0, 0},
  {&__pyx_n_s_Set, __pyx_k_Set, sizeof(__pyx_k_Set), 0, 0, 1, 1},
  {&__pyx_n_s_Struct, __pyx_k_Struct, sizeof(__pyx_k_Struct), 0, 0, 1, 1},
  {&__pyx_n_s_TYPES, __pyx_k_TYPES, sizeof(__pyx_k_TYPES), 0, 0, 1, 1},
  {&__pyx_n_s_TYPE_A, __pyx_k_TYPE_A, sizeof(__pyx_k_TYPE_A), 0, 0, 1, 1},
  {&__pyx_n_s_TYPE_AAAA, __pyx_k_TYPE_AAAA, sizeof(__pyx_k_TYPE_AAAA), 0, 0, 1, 1},
  {&__pyx_n_s_TYPE_CNAME, __pyx_k_TYPE_CNAME, sizeof(__pyx_k_TYPE_CNAME), 0, 0, 1, 1},
  {&__pyx_n_s_TYPE_HINFO, __pyx_k_TYPE_HINFO, sizeof(__pyx_k_TYPE_HINFO), 0, 0, 1, 1},
  {&__pyx_n_s_TYPE_NSEC, __pyx_k_TYPE_NSEC, sizeof(__pyx_k_TYPE_NSEC), 0, 0, 1, 1},
  {&__pyx_n_s_TYPE_PTR, __pyx_k_TYPE_PTR, sizeof(__pyx_k_TYPE_PTR), 0, 0, 1, 1},
  {&__pyx_n_s_TYPE_SRV, __pyx_k_TYPE_SRV, sizeof(__pyx_k_TYPE_SRV), 0, 0, 1, 1},
  {&__pyx_n_s_TYPE_TXT, __pyx_k_TYPE_TXT, sizeof(__pyx_k_TYPE_TXT), 0, 0, 1, 1},
  {&__pyx_n_s_Tuple, __pyx_k_Tuple, sizeof(__pyx_k_Tuple), 0, 0, 1, 1},
  {&__pyx_n_s_UNPACK_3H, __pyx_k_UNPACK_3H, sizeof(__pyx_k_UNPACK_3H), 0, 0, 1, 1},
  {&__pyx_n_s_UNPACK_6H, __pyx_k_UNPACK_6H, sizeof(__pyx_k_UNPACK_6H), 0, 0, 1, 1},
  {&__pyx_n_s_UNPACK_HH, __pyx_k_UNPACK_HH, sizeof(__pyx_k_UNPACK_HH), 0, 0, 1, 1},
  {&__pyx_n_s_UNPACK_HHiH, __pyx_k_UNPACK_HHiH, sizeof(__pyx_k_UNPACK_HHiH), 0, 0, 1, 1},
  {&__pyx_kp_u_Unable_to_parse_skipping_record, __pyx_k_Unable_to_parse_skipping_record, sizeof(__pyx_k_Unable_to_parse_skipping_record), 0, 1, 0, 0},
  {&__pyx_n_s_Union, __pyx_k_Union, sizeof(__pyx_k_Union), 0, 0, 1, 1},
  {&__pyx_kp_u__2, __pyx_k__2, sizeof(__pyx_k__2), 0, 1, 0, 0},
  {&__pyx_n_s_answers, __pyx_k_answers, sizeof(__pyx_k_answers), 0, 0, 1, 1},
  {&__pyx_n_u_answers_2, __pyx_k_answers_2, sizeof(__pyx_k_answers_2), 0, 1, 0, 1},
  {&__pyx_kp_u_answers_s, __pyx_k_answers_s, sizeof(__pyx_k_answers_s), 0, 1, 0, 0},
  {&__pyx_kp_u_beyond_packet_from, __pyx_k_beyond_packet_from, sizeof(__pyx_k_beyond_packet_from), 0, 1, 0, 0},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_cls, __pyx_k_cls, sizeof(__pyx_k_cls), 0, 0, 1, 1},
  {&__pyx_n_s_const, __pyx_k_const, sizeof(__pyx_k_const), 0, 0, 1, 1},
  {&__pyx_n_s_current_time_millis, __pyx_k_current_time_millis, sizeof(__pyx_k_current_time_millis), 0, 0, 1, 1},
  {&__pyx_n_s_data, __pyx_k_data, sizeof(__pyx_k_data), 0, 0, 1, 1},
  {&__pyx_n_u_data, __pyx_k_data, sizeof(__pyx_k_data), 0, 1, 0, 1},
  {&__pyx_n_u_data_len, __pyx_k_data_len, sizeof(__pyx_k_data_len), 0, 1, 0, 1},
  {&__pyx_n_s_debug, __pyx_k_debug, sizeof(__pyx_k_debug), 0, 0, 1, 1},
  {&__pyx_n_s_dict, __pyx_k_dict, sizeof(__pyx_k_dict), 0, 0, 1, 1},
  {&__pyx_n_s_dict_2, __pyx_k_dict_2, sizeof(__pyx_k_dict_2), 0, 0, 1, 1},
  {&__pyx_n_u_did_read_others, __pyx_k_did_read_others, sizeof(__pyx_k_did_read_others), 0, 1, 0, 1},
  {&__pyx_n_s_dns, __pyx_k_dns, sizeof(__pyx_k_dns), 0, 0, 1, 1},
  {&__pyx_n_s_enumerate, __pyx_k_enumerate, sizeof(__pyx_k_enumerate), 0, 0, 1, 1},
  {&__pyx_n_s_error, __pyx_k_error, sizeof(__pyx_k_error), 0, 0, 1, 1},
  {&__pyx_n_s_exc_info, __pyx_k_exc_info, sizeof(__pyx_k_exc_info), 0, 0, 1, 1},
  {&__pyx_n_s_exc_str, __pyx_k_exc_str, sizeof(__pyx_k_exc_str), 0, 0, 1, 1},
  {&__pyx_kp_u_exceeds_maximum_length_of, __pyx_k_exceeds_maximum_length_of, sizeof(__pyx_k_exceeds_maximum_length_of), 0, 1, 0, 0},
  {&__pyx_n_s_exceptions, __pyx_k_exceptions, sizeof(__pyx_k_exceptions), 0, 0, 1, 1},
  {&__pyx_n_u_flags, __pyx_k_flags, sizeof(__pyx_k_flags), 0, 1, 0, 1},
  {&__pyx_kp_u_flags_s, __pyx_k_flags_s, sizeof(__pyx_k_flags_s), 0, 1, 0, 0},
  {&__pyx_kp_u_from, __pyx_k_from, sizeof(__pyx_k_from), 0, 1, 0, 0},
  {&__pyx_n_s_get, __pyx_k_get, sizeof(__pyx_k_get), 0, 0, 1, 1},
  {&__pyx_n_s_getstate, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
  {&__pyx_n_u_id, __pyx_k_id, sizeof(__pyx_k_id), 0, 1, 0, 1},
  {&__pyx_kp_u_id_s, __pyx_k_id_s, sizeof(__pyx_k_id_s), 0, 1, 0, 0},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_is_query, __pyx_k_is_query, sizeof(__pyx_k_is_query), 0, 0, 1, 1},
  {&__pyx_n_s_is_response, __pyx_k_is_response, sizeof(__pyx_k_is_response), 0, 0, 1, 1},
  {&__pyx_kp_u_is_unknown_at, __pyx_k_is_unknown_at, sizeof(__pyx_k_is_unknown_at), 0, 1, 0, 0},
  {&__pyx_n_s_log, __pyx_k_log, sizeof(__pyx_k_log), 0, 0, 1, 1},
  {&__pyx_n_s_log_exc_info, __pyx_k_log_exc_info, sizeof(__pyx_k_log_exc_info), 0, 0, 1, 1},
  {&__pyx_n_s_log_exception_debug, __pyx_k_log_exception_debug, sizeof(__pyx_k_log_exception_debug), 0, 0, 1, 1},
  {&__pyx_n_s_logger, __pyx_k_logger, sizeof(__pyx_k_logger), 0, 0, 1, 1},
  {&__pyx_n_s_logger_data, __pyx_k_logger_data, sizeof(__pyx_k_logger_data), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_kp_u_n_add_s, __pyx_k_n_add_s, sizeof(__pyx_k_n_add_s), 0, 1, 0, 0},
  {&__pyx_kp_u_n_ans_s, __pyx_k_n_ans_s, sizeof(__pyx_k_n_ans_s), 0, 1, 0, 0},
  {&__pyx_kp_u_n_auth_s, __pyx_k_n_auth_s, sizeof(__pyx_k_n_auth_s), 0, 1, 0, 0},
  {&__pyx_kp_u_n_q_s, __pyx_k_n_q_s, sizeof(__pyx_k_n_q_s), 0, 1, 0, 0},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_u_name_cache, __pyx_k_name_cache, sizeof(__pyx_k_name_cache), 0, 1, 0, 1},
  {&__pyx_n_s_new, __pyx_k_new, sizeof(__pyx_k_new), 0, 0, 1, 1},
  {&__pyx_n_s_now, __pyx_k_now, sizeof(__pyx_k_now), 0, 0, 1, 1},
  {&__pyx_n_u_now, __pyx_k_now, sizeof(__pyx_k_now), 0, 1, 0, 1},
  {&__pyx_n_u_num_additionals, __pyx_k_num_additionals, sizeof(__pyx_k_num_additionals), 0, 1, 0, 1},
  {&__pyx_n_u_num_answers, __pyx_k_num_answers, sizeof(__pyx_k_num_answers), 0, 1, 0, 1},
  {&__pyx_n_u_num_authorities, __pyx_k_num_authorities, sizeof(__pyx_k_num_authorities), 0, 1, 0, 1},
  {&__pyx_n_u_num_questions, __pyx_k_num_questions, sizeof(__pyx_k_num_questions), 0, 1, 0, 1},
  {&__pyx_n_u_offset, __pyx_k_offset, sizeof(__pyx_k_offset), 0, 1, 0, 1},
  {&__pyx_n_s_pickle, __pyx_k_pickle, sizeof(__pyx_k_pickle), 0, 0, 1, 1},
  {&__pyx_kp_u_points_to, __pyx_k_points_to, sizeof(__pyx_k_points_to), 0, 1, 0, 0},
  {&__pyx_kp_u_points_to_itself_from, __pyx_k_points_to_itself_from, sizeof(__pyx_k_points_to_itself_from), 0, 1, 0, 0},
  {&__pyx_n_s_pyx_PickleError, __pyx_k_pyx_PickleError, sizeof(__pyx_k_pyx_PickleError), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_capi, __pyx_k_pyx_capi, sizeof(__pyx_k_pyx_capi), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_checksum, __pyx_k_pyx_checksum, sizeof(__pyx_k_pyx_checksum), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_result, __pyx_k_pyx_result, sizeof(__pyx_k_pyx_result), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_state, __pyx_k_pyx_state, sizeof(__pyx_k_pyx_state), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_type, __pyx_k_pyx_type, sizeof(__pyx_k_pyx_type), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_DNSIncoming, __pyx_k_pyx_unpickle_DNSIncoming, sizeof(__pyx_k_pyx_unpickle_DNSIncoming), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_u_questions, __pyx_k_questions, sizeof(__pyx_k_questions), 0, 1, 0, 1},
  {&__pyx_kp_u_questions_s, __pyx_k_questions_s, sizeof(__pyx_k_questions_s), 0, 1, 0, 0},
  {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
  {&__pyx_n_s_reduce, __pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_cython, __pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_ex, __pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 0, 1, 1},
  {&__pyx_n_s_return, __pyx_k_return, sizeof(__pyx_k_return), 0, 0, 1, 1},
  {&__pyx_n_s_scope_id, __pyx_k_scope_id, sizeof(__pyx_k_scope_id), 0, 0, 1, 1},
  {&__pyx_n_u_scope_id, __pyx_k_scope_id, sizeof(__pyx_k_scope_id), 0, 1, 0, 1},
  {&__pyx_n_s_seen_logs, __pyx_k_seen_logs, sizeof(__pyx_k_seen_logs), 0, 0, 1, 1},
  {&__pyx_n_s_self, __pyx_k_self, sizeof(__pyx_k_self), 0, 0, 1, 1},
  {&__pyx_n_s_setstate, __pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 0, 1, 1},
  {&__pyx_n_s_setstate_cython, __pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 0, 1, 1},
  {&__pyx_n_s_slots, __pyx_k_slots, sizeof(__pyx_k_slots), 0, 0, 1, 1},
  {&__pyx_n_s_source, __pyx_k_source, sizeof(__pyx_k_source), 0, 0, 1, 1},
  {&__pyx_n_u_source, __pyx_k_source, sizeof(__pyx_k_source), 0, 1, 0, 1},
  {&__pyx_kp_s_src_zeroconf__protocol_incoming, __pyx_k_src_zeroconf__protocol_incoming, sizeof(__pyx_k_src_zeroconf__protocol_incoming), 0, 0, 1, 0},
  {&__pyx_n_s_state, __pyx_k_state, sizeof(__pyx_k_state), 0, 0, 1, 1},
  {&__pyx_n_s_str, __pyx_k_str, sizeof(__pyx_k_str), 0, 0, 1, 1},
  {&__pyx_kp_s_stringsource, __pyx_k_stringsource, sizeof(__pyx_k_stringsource), 0, 0, 1, 0},
  {&__pyx_n_s_struct, __pyx_k_struct, sizeof(__pyx_k_struct), 0, 0, 1, 1},
  {&__pyx_n_s_sys, __pyx_k_sys, sizeof(__pyx_k_sys), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_truncated, __pyx_k_truncated, sizeof(__pyx_k_truncated), 0, 0, 1, 1},
  {&__pyx_kp_u_truncated_s, __pyx_k_truncated_s, sizeof(__pyx_k_truncated_s), 0, 1, 0, 0},
  {&__pyx_n_s_typing, __pyx_k_typing, sizeof(__pyx_k_typing), 0, 0, 1, 1},
  {&__pyx_n_s_unpack_from, __pyx_k_unpack_from, sizeof(__pyx_k_unpack_from), 0, 0, 1, 1},
  {&__pyx_n_s_update, __pyx_k_update, sizeof(__pyx_k_update), 0, 0, 1, 1},
  {&__pyx_n_s_use_setstate, __pyx_k_use_setstate, sizeof(__pyx_k_use_setstate), 0, 0, 1, 1},
  {&__pyx_n_s_utils_time, __pyx_k_utils_time, sizeof(__pyx_k_utils_time), 0, 0, 1, 1},
  {&__pyx_n_u_valid, __pyx_k_valid, sizeof(__pyx_k_valid), 0, 1, 0, 1},
  {&__pyx_kp_u_was_seen_again_from, __pyx_k_was_seen_again_from, sizeof(__pyx_k_was_seen_again_from), 0, 1, 0, 0},
  {&__pyx_n_s_zeroconf__protocol_incoming, __pyx_k_zeroconf__protocol_incoming, sizeof(__pyx_k_zeroconf__protocol_incoming), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_IndexError = __Pyx_GetBuiltinName(__pyx_n_s_IndexError); if (!__pyx_builtin_IndexError) __PYX_ERR(0, 61, __pyx_L1_error)
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_range) __PYX_ERR(0, 211, __pyx_L1_error)
  __pyx_builtin_enumerate = __Pyx_GetBuiltinName(__pyx_n_s_enumerate); if (!__pyx_builtin_enumerate) __PYX_ERR(0, 327, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0x69f643b, 0xcd74736, 0xd8a590c):             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0x69f643b, 0xcd74736, 0xd8a590c) = (_answers, _data_len, _did_read_others, data, flags, id, name_cache, now, num_additionals, num_answers, num_authorities, num_questions, offset, questions, scope_id, source, valid))" % __pyx_checksum)
 */
  __pyx_tuple__3 = PyTuple_Pack(3, __pyx_int_111109179, __pyx_int_215435062, __pyx_int_227170572); if (unlikely(!__pyx_tuple__3)) __PYX_ERR(2, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);

  /* "zeroconf/_protocol/incoming.py":63
 * DECODE_EXCEPTIONS = (IndexError, struct.error, IncomingDecodeError)
 * 
 * UNPACK_3H = struct.Struct(b'!3H').unpack_from             # <<<<<<<<<<<<<<
 * UNPACK_6H = struct.Struct(b'!6H').unpack_from
 * UNPACK_HH = struct.Struct(b'!HH').unpack_from
 */
  __pyx_tuple__4 = PyTuple_Pack(1, __pyx_kp_b_3H); if (unlikely(!__pyx_tuple__4)) __PYX_ERR(0, 63, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);

  /* "zeroconf/_protocol/incoming.py":64
 * 
 * UNPACK_3H = struct.Struct(b'!3H').unpack_from
 * UNPACK_6H = struct.Struct(b'!6H').unpack_from             # <<<<<<<<<<<<<<
 * UNPACK_HH = struct.Struct(b'!HH').unpack_from
 * UNPACK_HHiH = struct.Struct(b'!HHiH').unpack_from
 */
  __pyx_tuple__5 = PyTuple_Pack(1, __pyx_kp_b_6H); if (unlikely(!__pyx_tuple__5)) __PYX_ERR(0, 64, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__5);
  __Pyx_GIVEREF(__pyx_tuple__5);

  /* "zeroconf/_protocol/incoming.py":65
 * UNPACK_3H = struct.Struct(b'!3H').unpack_from
 * UNPACK_6H = struct.Struct(b'!6H').unpack_from
 * UNPACK_HH = struct.Struct(b'!HH').unpack_from             # <<<<<<<<<<<<<<
 * UNPACK_HHiH = struct.Struct(b'!HHiH').unpack_from
 * 
 */
  __pyx_tuple__6 = PyTuple_Pack(1, __pyx_kp_b_HH); if (unlikely(!__pyx_tuple__6)) __PYX_ERR(0, 65, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);

  /* "zeroconf/_protocol/incoming.py":66
 * UNPACK_6H = struct.Struct(b'!6H').unpack_from
 * UNPACK_HH = struct.Struct(b'!HH').unpack_from
 * UNPACK_HHiH = struct.Struct(b'!HHiH').unpack_from             # <<<<<<<<<<<<<<
 * 
 * _seen_logs: Dict[str, Union[int, tuple]] = {}
 */
  __pyx_tuple__7 = PyTuple_Pack(1, __pyx_kp_b_HHiH); if (unlikely(!__pyx_tuple__7)) __PYX_ERR(0, 66, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__7);
  __Pyx_GIVEREF(__pyx_tuple__7);

  /* "zeroconf/_protocol/incoming.py":76
 * 
 *     __slots__ = (
 *         "_did_read_others",             # <<<<<<<<<<<<<<
 *         'flags',
 *         'offset',
 */
  __pyx_tuple__8 = PyTuple_Pack(17, __pyx_n_u_did_read_others, __pyx_n_u_flags, __pyx_n_u_offset, __pyx_n_u_data, __pyx_n_u_data_len, __pyx_n_u_name_cache, __pyx_n_u_questions, __pyx_n_u_answers_2, __pyx_n_u_id, __pyx_n_u_num_questions, __pyx_n_u_num_answers, __pyx_n_u_num_authorities, __pyx_n_u_num_additionals, __pyx_n_u_valid, __pyx_n_u_now, __pyx_n_u_scope_id, __pyx_n_u_source); if (unlikely(!__pyx_tuple__8)) __PYX_ERR(0, 76, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__8);
  __Pyx_GIVEREF(__pyx_tuple__8);

  /* "zeroconf/_protocol/incoming.py":130
 *             )
 * 
 *     def is_query(self) -> bool:             # <<<<<<<<<<<<<<
 *         """Returns true if this is a query."""
 *         return (self.flags & _FLAGS_QR_MASK) == _FLAGS_QR_QUERY
 */
  __pyx_tuple__9 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__9)) __PYX_ERR(0, 130, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__9);
  __Pyx_GIVEREF(__pyx_tuple__9);
  __pyx_codeobj__10 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__9, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_zeroconf__protocol_incoming, __pyx_n_s_is_query, 130, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__10)) __PYX_ERR(0, 130, __pyx_L1_error)

  /* "zeroconf/_protocol/incoming.py":134
 *         return (self.flags & _FLAGS_QR_MASK) == _FLAGS_QR_QUERY
 * 
 *     def is_response(self) -> bool:             # <<<<<<<<<<<<<<
 *         """Returns true if this is a response."""
 *         return (self.flags & _FLAGS_QR_MASK) == _FLAGS_QR_RESPONSE
 */
  __pyx_tuple__11 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__11)) __PYX_ERR(0, 134, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__11);
  __Pyx_GIVEREF(__pyx_tuple__11);
  __pyx_codeobj__12 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__11, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_zeroconf__protocol_incoming, __pyx_n_s_is_response, 134, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__12)) __PYX_ERR(0, 134, __pyx_L1_error)

  /* "zeroconf/_protocol/incoming.py":152
 * 
 *     @classmethod
 *     def _log_exception_debug(cls, *logger_data: Any) -> None:             # <<<<<<<<<<<<<<
 *         log_exc_info = False
 *         exc_info = sys.exc_info()
 */
  __pyx_tuple__13 = PyTuple_Pack(5, __pyx_n_s_cls, __pyx_n_s_logger_data, __pyx_n_s_log_exc_info, __pyx_n_s_exc_info, __pyx_n_s_exc_str); if (unlikely(!__pyx_tuple__13)) __PYX_ERR(0, 152, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__13);
  __Pyx_GIVEREF(__pyx_tuple__13);
  __pyx_codeobj__14 = (PyObject*)__Pyx_PyCode_New(1, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS|CO_VARARGS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__13, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_zeroconf__protocol_incoming, __pyx_n_s_log_exception_debug, 152, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__14)) __PYX_ERR(0, 152, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */
  __pyx_tuple__15 = PyTuple_Pack(4, __pyx_n_s_self, __pyx_n_s_state, __pyx_n_s_dict_2, __pyx_n_s_use_setstate); if (unlikely(!__pyx_tuple__15)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__15);
  __Pyx_GIVEREF(__pyx_tuple__15);
  __pyx_codeobj__16 = (PyObject*)__Pyx_PyCode_New(1, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__15, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_reduce_cython, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__16)) __PYX_ERR(2, 1, __pyx_L1_error)

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_DNSIncoming, (type(self), 0x69f643b, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_DNSIncoming__set_state(self, __pyx_state)
 */
  __pyx_tuple__17 = PyTuple_Pack(2, __pyx_n_s_self, __pyx_n_s_pyx_state); if (unlikely(!__pyx_tuple__17)) __PYX_ERR(2, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__17);
  __Pyx_GIVEREF(__pyx_tuple__17);
  __pyx_codeobj__18 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__17, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_setstate_cython, 16, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__18)) __PYX_ERR(2, 16, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __pyx_unpickle_DNSIncoming(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_tuple__19 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__19)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__19);
  __Pyx_GIVEREF(__pyx_tuple__19);
  __pyx_codeobj__20 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__19, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_DNSIncoming, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__20)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void) {
  __pyx_umethod_PyDict_Type_get.type = (PyObject*)&PyDict_Type;
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_111109179 = PyInt_FromLong(111109179L); if (unlikely(!__pyx_int_111109179)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_215435062 = PyInt_FromLong(215435062L); if (unlikely(!__pyx_int_215435062)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_227170572 = PyInt_FromLong(227170572L); if (unlikely(!__pyx_int_227170572)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(void); /*proto*/

static int __Pyx_modinit_global_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __pyx_v_8zeroconf_9_protocol_8incoming_current_time_millis = Py_None; Py_INCREF(Py_None);
  __pyx_v_8zeroconf_9_protocol_8incoming_UNPACK_3H = Py_None; Py_INCREF(Py_None);
  __pyx_v_8zeroconf_9_protocol_8incoming_UNPACK_6H = Py_None; Py_INCREF(Py_None);
  __pyx_v_8zeroconf_9_protocol_8incoming_UNPACK_HH = Py_None; Py_INCREF(Py_None);
  __pyx_v_8zeroconf_9_protocol_8incoming_UNPACK_HHiH = Py_None; Py_INCREF(Py_None);
  __pyx_v_8zeroconf_9_protocol_8incoming_DECODE_EXCEPTIONS = Py_None; Py_INCREF(Py_None);
  __pyx_v_8zeroconf_9_protocol_8incoming_IncomingDecodeError = Py_None; Py_INCREF(Py_None);
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(void) {
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  if (__Pyx_ExportVoidPtr(__pyx_n_s_DNS_COMPRESSION_HEADER_LEN, (void *)&__pyx_v_8zeroconf_9_protocol_8incoming_DNS_COMPRESSION_HEADER_LEN, "unsigned int") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_MAX_DNS_LABELS, (void *)&__pyx_v_8zeroconf_9_protocol_8incoming_MAX_DNS_LABELS, "unsigned int") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_DNS_COMPRESSION_POINTER_LEN, (void *)&__pyx_v_8zeroconf_9_protocol_8incoming_DNS_COMPRESSION_POINTER_LEN, "unsigned int") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_MAX_NAME_LENGTH, (void *)&__pyx_v_8zeroconf_9_protocol_8incoming_MAX_NAME_LENGTH, "unsigned int") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_current_time_millis, (void *)&__pyx_v_8zeroconf_9_protocol_8incoming_current_time_millis, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_TYPE_A, (void *)&__pyx_v_8zeroconf_9_protocol_8incoming__TYPE_A, "unsigned int") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_TYPE_CNAME, (void *)&__pyx_v_8zeroconf_9_protocol_8incoming__TYPE_CNAME, "unsigned int") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_TYPE_PTR, (void *)&__pyx_v_8zeroconf_9_protocol_8incoming__TYPE_PTR, "unsigned int") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_TYPE_TXT, (void *)&__pyx_v_8zeroconf_9_protocol_8incoming__TYPE_TXT, "unsigned int") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_TYPE_SRV, (void *)&__pyx_v_8zeroconf_9_protocol_8incoming__TYPE_SRV, "unsigned int") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_TYPE_HINFO, (void *)&__pyx_v_8zeroconf_9_protocol_8incoming__TYPE_HINFO, "unsigned int") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_TYPE_AAAA, (void *)&__pyx_v_8zeroconf_9_protocol_8incoming__TYPE_AAAA, "unsigned int") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_TYPE_NSEC, (void *)&__pyx_v_8zeroconf_9_protocol_8incoming__TYPE_NSEC, "unsigned int") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_FLAGS_QR_MASK, (void *)&__pyx_v_8zeroconf_9_protocol_8incoming__FLAGS_QR_MASK, "unsigned int") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_FLAGS_TC, (void *)&__pyx_v_8zeroconf_9_protocol_8incoming__FLAGS_TC, "unsigned int") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_FLAGS_QR_QUERY, (void *)&__pyx_v_8zeroconf_9_protocol_8incoming__FLAGS_QR_QUERY, "unsigned int") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_FLAGS_QR_RESPONSE, (void *)&__pyx_v_8zeroconf_9_protocol_8incoming__FLAGS_QR_RESPONSE, "unsigned int") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_UNPACK_3H, (void *)&__pyx_v_8zeroconf_9_protocol_8incoming_UNPACK_3H, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_UNPACK_6H, (void *)&__pyx_v_8zeroconf_9_protocol_8incoming_UNPACK_6H, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_UNPACK_HH, (void *)&__pyx_v_8zeroconf_9_protocol_8incoming_UNPACK_HH, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_UNPACK_HHiH, (void *)&__pyx_v_8zeroconf_9_protocol_8incoming_UNPACK_HHiH, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_DECODE_EXCEPTIONS, (void *)&__pyx_v_8zeroconf_9_protocol_8incoming_DECODE_EXCEPTIONS, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_IncomingDecodeError, (void *)&__pyx_v_8zeroconf_9_protocol_8incoming_IncomingDecodeError, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_function_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_init_code(void) {
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  __pyx_vtabptr_8zeroconf_9_protocol_8incoming_DNSIncoming = &__pyx_vtable_8zeroconf_9_protocol_8incoming_DNSIncoming;
  __pyx_vtable_8zeroconf_9_protocol_8incoming_DNSIncoming._decode_labels_at_offset = (PyObject *(*)(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *, unsigned int, PyObject *, PyObject *))__pyx_f_8zeroconf_9_protocol_8incoming_11DNSIncoming__decode_labels_at_offset;
  __pyx_vtable_8zeroconf_9_protocol_8incoming_DNSIncoming._read_header = (PyObject *(*)(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *))__pyx_f_8zeroconf_9_protocol_8incoming_11DNSIncoming__read_header;
  __pyx_vtable_8zeroconf_9_protocol_8incoming_DNSIncoming._initial_parse = (PyObject *(*)(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *))__pyx_f_8zeroconf_9_protocol_8incoming_11DNSIncoming__initial_parse;
  __pyx_vtable_8zeroconf_9_protocol_8incoming_DNSIncoming._read_others = (PyObject *(*)(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *))__pyx_f_8zeroconf_9_protocol_8incoming_11DNSIncoming__read_others;
  __pyx_vtable_8zeroconf_9_protocol_8incoming_DNSIncoming._read_questions = (PyObject *(*)(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *))__pyx_f_8zeroconf_9_protocol_8incoming_11DNSIncoming__read_questions;
  __pyx_vtable_8zeroconf_9_protocol_8incoming_DNSIncoming._read_character_string = (PyObject *(*)(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *))__pyx_f_8zeroconf_9_protocol_8incoming_11DNSIncoming__read_character_string;
  __pyx_vtable_8zeroconf_9_protocol_8incoming_DNSIncoming._read_string = (PyObject *(*)(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *, unsigned int))__pyx_f_8zeroconf_9_protocol_8incoming_11DNSIncoming__read_string;
  __pyx_vtable_8zeroconf_9_protocol_8incoming_DNSIncoming._read_record = (PyObject *(*)(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *, PyObject *, unsigned int, PyObject *, PyObject *, unsigned int))__pyx_f_8zeroconf_9_protocol_8incoming_11DNSIncoming__read_record;
  __pyx_vtable_8zeroconf_9_protocol_8incoming_DNSIncoming._read_bitmap = (PyObject *(*)(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *, unsigned int))__pyx_f_8zeroconf_9_protocol_8incoming_11DNSIncoming__read_bitmap;
  __pyx_vtable_8zeroconf_9_protocol_8incoming_DNSIncoming._read_name = (PyObject *(*)(struct __pyx_obj_8zeroconf_9_protocol_8incoming_DNSIncoming *))__pyx_f_8zeroconf_9_protocol_8incoming_11DNSIncoming__read_name;
  if (PyType_Ready(&__pyx_type_8zeroconf_9_protocol_8incoming_DNSIncoming) < 0) __PYX_ERR(0, 72, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_8zeroconf_9_protocol_8incoming_DNSIncoming.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_8zeroconf_9_protocol_8incoming_DNSIncoming.tp_dictoffset && __pyx_type_8zeroconf_9_protocol_8incoming_DNSIncoming.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_8zeroconf_9_protocol_8incoming_DNSIncoming.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  #if CYTHON_UPDATE_DESCRIPTOR_DOC
  {
    PyObject *wrapper = PyObject_GetAttrString((PyObject *)&__pyx_type_8zeroconf_9_protocol_8incoming_DNSIncoming, "__init__"); if (unlikely(!wrapper)) __PYX_ERR(0, 72, __pyx_L1_error)
    if (Py_TYPE(wrapper) == &PyWrapperDescr_Type) {
      __pyx_wrapperbase_8zeroconf_9_protocol_8incoming_11DNSIncoming___init__ = *((PyWrapperDescrObject *)wrapper)->d_base;
      __pyx_wrapperbase_8zeroconf_9_protocol_8incoming_11DNSIncoming___init__.doc = __pyx_doc_8zeroconf_9_protocol_8incoming_11DNSIncoming___init__;
      ((PyWrapperDescrObject *)wrapper)->d_base = &__pyx_wrapperbase_8zeroconf_9_protocol_8incoming_11DNSIncoming___init__;
    }
  }
  #endif
  if (__Pyx_SetVtable(__pyx_type_8zeroconf_9_protocol_8incoming_DNSIncoming.tp_dict, __pyx_vtabptr_8zeroconf_9_protocol_8incoming_DNSIncoming) < 0) __PYX_ERR(0, 72, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_DNSIncoming, (PyObject *)&__pyx_type_8zeroconf_9_protocol_8incoming_DNSIncoming) < 0) __PYX_ERR(0, 72, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_8zeroconf_9_protocol_8incoming_DNSIncoming) < 0) __PYX_ERR(0, 72, __pyx_L1_error)
  __pyx_ptype_8zeroconf_9_protocol_8incoming_DNSIncoming = &__pyx_type_8zeroconf_9_protocol_8incoming_DNSIncoming;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __pyx_t_1 = PyImport_ImportModule("zeroconf._dns"); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 21, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_8zeroconf_4_dns_DNSEntry = __Pyx_ImportType_0_29_35(__pyx_t_1, "zeroconf._dns", "DNSEntry", sizeof(struct __pyx_obj_8zeroconf_4_dns_DNSEntry), __PYX_GET_STRUCT_ALIGNMENT_0_29_35(struct __pyx_obj_8zeroconf_4_dns_DNSEntry),__Pyx_ImportType_CheckSize_Warn_0_29_35); if (!__pyx_ptype_8zeroconf_4_dns_DNSEntry) __PYX_ERR(3, 21, __pyx_L1_error)
  __pyx_vtabptr_8zeroconf_4_dns_DNSEntry = (struct __pyx_vtabstruct_8zeroconf_4_dns_DNSEntry*)__Pyx_GetVtable(__pyx_ptype_8zeroconf_4_dns_DNSEntry->tp_dict); if (unlikely(!__pyx_vtabptr_8zeroconf_4_dns_DNSEntry)) __PYX_ERR(3, 21, __pyx_L1_error)
  __pyx_ptype_8zeroconf_4_dns_DNSQuestion = __Pyx_ImportType_0_29_35(__pyx_t_1, "zeroconf._dns", "DNSQuestion", sizeof(struct __pyx_obj_8zeroconf_4_dns_DNSQuestion), __PYX_GET_STRUCT_ALIGNMENT_0_29_35(struct __pyx_obj_8zeroconf_4_dns_DNSQuestion),__Pyx_ImportType_CheckSize_Warn_0_29_35); if (!__pyx_ptype_8zeroconf_4_dns_DNSQuestion) __PYX_ERR(3, 31, __pyx_L1_error)
  __pyx_vtabptr_8zeroconf_4_dns_DNSQuestion = (struct __pyx_vtabstruct_8zeroconf_4_dns_DNSQuestion*)__Pyx_GetVtable(__pyx_ptype_8zeroconf_4_dns_DNSQuestion->tp_dict); if (unlikely(!__pyx_vtabptr_8zeroconf_4_dns_DNSQuestion)) __PYX_ERR(3, 31, __pyx_L1_error)
  __pyx_ptype_8zeroconf_4_dns_DNSRecord = __Pyx_ImportType_0_29_35(__pyx_t_1, "zeroconf._dns", "DNSRecord", sizeof(struct __pyx_obj_8zeroconf_4_dns_DNSRecord), __PYX_GET_STRUCT_ALIGNMENT_0_29_35(struct __pyx_obj_8zeroconf_4_dns_DNSRecord),__Pyx_ImportType_CheckSize_Warn_0_29_35); if (!__pyx_ptype_8zeroconf_4_dns_DNSRecord) __PYX_ERR(3, 35, __pyx_L1_error)
  __pyx_vtabptr_8zeroconf_4_dns_DNSRecord = (struct __pyx_vtabstruct_8zeroconf_4_dns_DNSRecord*)__Pyx_GetVtable(__pyx_ptype_8zeroconf_4_dns_DNSRecord->tp_dict); if (unlikely(!__pyx_vtabptr_8zeroconf_4_dns_DNSRecord)) __PYX_ERR(3, 35, __pyx_L1_error)
  __pyx_ptype_8zeroconf_4_dns_DNSAddress = __Pyx_ImportType_0_29_35(__pyx_t_1, "zeroconf._dns", "DNSAddress", sizeof(struct __pyx_obj_8zeroconf_4_dns_DNSAddress), __PYX_GET_STRUCT_ALIGNMENT_0_29_35(struct __pyx_obj_8zeroconf_4_dns_DNSAddress),__Pyx_ImportType_CheckSize_Warn_0_29_35); if (!__pyx_ptype_8zeroconf_4_dns_DNSAddress) __PYX_ERR(3, 43, __pyx_L1_error)
  __pyx_vtabptr_8zeroconf_4_dns_DNSAddress = (struct __pyx_vtabstruct_8zeroconf_4_dns_DNSAddress*)__Pyx_GetVtable(__pyx_ptype_8zeroconf_4_dns_DNSAddress->tp_dict); if (unlikely(!__pyx_vtabptr_8zeroconf_4_dns_DNSAddress)) __PYX_ERR(3, 43, __pyx_L1_error)
  __pyx_ptype_8zeroconf_4_dns_DNSHinfo = __Pyx_ImportType_0_29_35(__pyx_t_1, "zeroconf._dns", "DNSHinfo", sizeof(struct __pyx_obj_8zeroconf_4_dns_DNSHinfo), __PYX_GET_STRUCT_ALIGNMENT_0_29_35(struct __pyx_obj_8zeroconf_4_dns_DNSHinfo),__Pyx_ImportType_CheckSize_Warn_0_29_35); if (!__pyx_ptype_8zeroconf_4_dns_DNSHinfo) __PYX_ERR(3, 52, __pyx_L1_error)
  __pyx_vtabptr_8zeroconf_4_dns_DNSHinfo = (struct __pyx_vtabstruct_8zeroconf_4_dns_DNSHinfo*)__Pyx_GetVtable(__pyx_ptype_8zeroconf_4_dns_DNSHinfo->tp_dict); if (unlikely(!__pyx_vtabptr_8zeroconf_4_dns_DNSHinfo)) __PYX_ERR(3, 52, __pyx_L1_error)
  __pyx_ptype_8zeroconf_4_dns_DNSPointer = __Pyx_ImportType_0_29_35(__pyx_t_1, "zeroconf._dns", "DNSPointer", sizeof(struct __pyx_obj_8zeroconf_4_dns_DNSPointer), __PYX_GET_STRUCT_ALIGNMENT_0_29_35(struct __pyx_obj_8zeroconf_4_dns_DNSPointer),__Pyx_ImportType_CheckSize_Warn_0_29_35); if (!__pyx_ptype_8zeroconf_4_dns_DNSPointer) __PYX_ERR(3, 61, __pyx_L1_error)
  __pyx_vtabptr_8zeroconf_4_dns_DNSPointer = (struct __pyx_vtabstruct_8zeroconf_4_dns_DNSPointer*)__Pyx_GetVtable(__pyx_ptype_8zeroconf_4_dns_DNSPointer->tp_dict); if (unlikely(!__pyx_vtabptr_8zeroconf_4_dns_DNSPointer)) __PYX_ERR(3, 61, __pyx_L1_error)
  __pyx_ptype_8zeroconf_4_dns_DNSText = __Pyx_ImportType_0_29_35(__pyx_t_1, "zeroconf._dns", "DNSText", sizeof(struct __pyx_obj_8zeroconf_4_dns_DNSText), __PYX_GET_STRUCT_ALIGNMENT_0_29_35(struct __pyx_obj_8zeroconf_4_dns_DNSText),__Pyx_ImportType_CheckSize_Warn_0_29_35); if (!__pyx_ptype_8zeroconf_4_dns_DNSText) __PYX_ERR(3, 70, __pyx_L1_error)
  __pyx_vtabptr_8zeroconf_4_dns_DNSText = (struct __pyx_vtabstruct_8zeroconf_4_dns_DNSText*)__Pyx_GetVtable(__pyx_ptype_8zeroconf_4_dns_DNSText->tp_dict); if (unlikely(!__pyx_vtabptr_8zeroconf_4_dns_DNSText)) __PYX_ERR(3, 70, __pyx_L1_error)
  __pyx_ptype_8zeroconf_4_dns_DNSService = __Pyx_ImportType_0_29_35(__pyx_t_1, "zeroconf._dns", "DNSService", sizeof(struct __pyx_obj_8zeroconf_4_dns_DNSService), __PYX_GET_STRUCT_ALIGNMENT_0_29_35(struct __pyx_obj_8zeroconf_4_dns_DNSService),__Pyx_ImportType_CheckSize_Warn_0_29_35); if (!__pyx_ptype_8zeroconf_4_dns_DNSService) __PYX_ERR(3, 78, __pyx_L1_error)
  __pyx_vtabptr_8zeroconf_4_dns_DNSService = (struct __pyx_vtabstruct_8zeroconf_4_dns_DNSService*)__Pyx_GetVtable(__pyx_ptype_8zeroconf_4_dns_DNSService->tp_dict); if (unlikely(!__pyx_vtabptr_8zeroconf_4_dns_DNSService)) __PYX_ERR(3, 78, __pyx_L1_error)
  __pyx_ptype_8zeroconf_4_dns_DNSNsec = __Pyx_ImportType_0_29_35(__pyx_t_1, "zeroconf._dns", "DNSNsec", sizeof(struct __pyx_obj_8zeroconf_4_dns_DNSNsec), __PYX_GET_STRUCT_ALIGNMENT_0_29_35(struct __pyx_obj_8zeroconf_4_dns_DNSNsec),__Pyx_ImportType_CheckSize_Warn_0_29_35); if (!__pyx_ptype_8zeroconf_4_dns_DNSNsec) __PYX_ERR(3, 90, __pyx_L1_error)
  __pyx_vtabptr_8zeroconf_4_dns_DNSNsec = (struct __pyx_vtabstruct_8zeroconf_4_dns_DNSNsec*)__Pyx_GetVtable(__pyx_ptype_8zeroconf_4_dns_DNSNsec->tp_dict); if (unlikely(!__pyx_vtabptr_8zeroconf_4_dns_DNSNsec)) __PYX_ERR(3, 90, __pyx_L1_error)
  __pyx_ptype_8zeroconf_4_dns_DNSRRSet = __Pyx_ImportType_0_29_35(__pyx_t_1, "zeroconf._dns", "DNSRRSet", sizeof(struct __pyx_obj_8zeroconf_4_dns_DNSRRSet), __PYX_GET_STRUCT_ALIGNMENT_0_29_35(struct __pyx_obj_8zeroconf_4_dns_DNSRRSet),__Pyx_ImportType_CheckSize_Warn_0_29_35); if (!__pyx_ptype_8zeroconf_4_dns_DNSRRSet) __PYX_ERR(3, 99, __pyx_L1_error)
  __pyx_vtabptr_8zeroconf_4_dns_DNSRRSet = (struct __pyx_vtabstruct_8zeroconf_4_dns_DNSRRSet*)__Pyx_GetVtable(__pyx_ptype_8zeroconf_4_dns_DNSRRSet->tp_dict); if (unlikely(!__pyx_vtabptr_8zeroconf_4_dns_DNSRRSet)) __PYX_ERR(3, 99, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_variable_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __pyx_t_1 = PyImport_ImportModule("zeroconf._dns"); if (!__pyx_t_1) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_ImportVoidPtr_0_29_35(__pyx_t_1, "_LEN_BYTE", (void **)&__pyx_vp_8zeroconf_4_dns__LEN_BYTE, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_0_29_35(__pyx_t_1, "_LEN_SHORT", (void **)&__pyx_vp_8zeroconf_4_dns__LEN_SHORT, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_0_29_35(__pyx_t_1, "_LEN_INT", (void **)&__pyx_vp_8zeroconf_4_dns__LEN_INT, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_0_29_35(__pyx_t_1, "_NAME_COMPRESSION_MIN_SIZE", (void **)&__pyx_vp_8zeroconf_4_dns__NAME_COMPRESSION_MIN_SIZE, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_0_29_35(__pyx_t_1, "_BASE_MAX_SIZE", (void **)&__pyx_vp_8zeroconf_4_dns__BASE_MAX_SIZE, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_0_29_35(__pyx_t_1, "_EXPIRE_FULL_TIME_MS", (void **)&__pyx_vp_8zeroconf_4_dns__EXPIRE_FULL_TIME_MS, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_0_29_35(__pyx_t_1, "_EXPIRE_STALE_TIME_MS", (void **)&__pyx_vp_8zeroconf_4_dns__EXPIRE_STALE_TIME_MS, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_0_29_35(__pyx_t_1, "_RECENT_TIME_MS", (void **)&__pyx_vp_8zeroconf_4_dns__RECENT_TIME_MS, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_0_29_35(__pyx_t_1, "_CLASS_UNIQUE", (void **)&__pyx_vp_8zeroconf_4_dns__CLASS_UNIQUE, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_0_29_35(__pyx_t_1, "_CLASS_MASK", (void **)&__pyx_vp_8zeroconf_4_dns__CLASS_MASK, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_0_29_35(__pyx_t_1, "current_time_millis", (void **)&__pyx_vp_8zeroconf_4_dns_current_time_millis, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_function_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}


#ifndef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#elif PY_MAJOR_VERSION < 3
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" void
#else
#define __Pyx_PyMODINIT_FUNC void
#endif
#else
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" PyObject *
#else
#define __Pyx_PyMODINIT_FUNC PyObject *
#endif
#endif


#if PY_MAJOR_VERSION < 3
__Pyx_PyMODINIT_FUNC initincoming(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC initincoming(void)
#else
__Pyx_PyMODINIT_FUNC PyInit_incoming(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit_incoming(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    #if PY_VERSION_HEX >= 0x030700A1
    static PY_INT64_T main_interpreter_id = -1;
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return (unlikely(current_id == -1)) ? -1 : 0;
    } else if (unlikely(main_interpreter_id != current_id))
    #else
    static PyInterpreterState *main_interpreter = NULL;
    PyInterpreterState *current_interpreter = PyThreadState_Get()->interp;
    if (!main_interpreter) {
        main_interpreter = current_interpreter;
    } else if (unlikely(main_interpreter != current_interpreter))
    #endif
    {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
            result = PyDict_SetItemString(moddict, to_name, value);
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__Pyx_check_single_interpreter())
        return NULL;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec_incoming(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  unsigned int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module 'incoming' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #elif PY_MAJOR_VERSION >= 3
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif
  __Pyx_RefNannySetupContext("__Pyx_PyMODINIT_FUNC PyInit_incoming(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pxy_PyFrame_Initialize_Offsets
  __Pxy_PyFrame_Initialize_Offsets();
  #endif
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(WITH_THREAD) && PY_VERSION_HEX < 0x030700F0 && defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  PyEval_InitThreads();
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("incoming", __pyx_methods, __pyx_k_Multicast_DNS_Service_Discovery, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_b);
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_cython_runtime);
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_zeroconf___protocol__incoming) {
    if (PyObject_SetAttr(__pyx_m, __pyx_n_s_name, __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "zeroconf._protocol.incoming")) {
      if (unlikely(PyDict_SetItemString(modules, "zeroconf._protocol.incoming", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code();
  if (unlikely(__Pyx_modinit_variable_export_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  (void)__Pyx_modinit_function_export_code();
  if (unlikely(__Pyx_modinit_type_init_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely(__Pyx_modinit_type_import_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely(__Pyx_modinit_variable_import_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  (void)__Pyx_modinit_function_import_code();
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "zeroconf/_protocol/incoming.py":23
 * """
 * 
 * import struct             # <<<<<<<<<<<<<<
 * import sys
 * from typing import Any, Dict, List, Optional, Set, Tuple, Union
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_struct, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 23, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_struct, __pyx_t_1) < 0) __PYX_ERR(0, 23, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "zeroconf/_protocol/incoming.py":24
 * 
 * import struct
 * import sys             # <<<<<<<<<<<<<<
 * from typing import Any, Dict, List, Optional, Set, Tuple, Union
 * 
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_sys, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_sys, __pyx_t_1) < 0) __PYX_ERR(0, 24, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "zeroconf/_protocol/incoming.py":25
 * import struct
 * import sys
 * from typing import Any, Dict, List, Optional, Set, Tuple, Union             # <<<<<<<<<<<<<<
 * 
 * from .._dns import (
 */
  __pyx_t_1 = PyList_New(7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_Any);
  __Pyx_GIVEREF(__pyx_n_s_Any);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_Any);
  __Pyx_INCREF(__pyx_n_s_Dict);
  __Pyx_GIVEREF(__pyx_n_s_Dict);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_s_Dict);
  __Pyx_INCREF(__pyx_n_s_List);
  __Pyx_GIVEREF(__pyx_n_s_List);
  PyList_SET_ITEM(__pyx_t_1, 2, __pyx_n_s_List);
  __Pyx_INCREF(__pyx_n_s_Optional);
  __Pyx_GIVEREF(__pyx_n_s_Optional);
  PyList_SET_ITEM(__pyx_t_1, 3, __pyx_n_s_Optional);
  __Pyx_INCREF(__pyx_n_s_Set);
  __Pyx_GIVEREF(__pyx_n_s_Set);
  PyList_SET_ITEM(__pyx_t_1, 4, __pyx_n_s_Set);
  __Pyx_INCREF(__pyx_n_s_Tuple);
  __Pyx_GIVEREF(__pyx_n_s_Tuple);
  PyList_SET_ITEM(__pyx_t_1, 5, __pyx_n_s_Tuple);
  __Pyx_INCREF(__pyx_n_s_Union);
  __Pyx_GIVEREF(__pyx_n_s_Union);
  PyList_SET_ITEM(__pyx_t_1, 6, __pyx_n_s_Union);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_typing, __pyx_t_1, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_Any); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Any, __pyx_t_1) < 0) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_Dict); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Dict, __pyx_t_1) < 0) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_List); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_List, __pyx_t_1) < 0) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_Optional); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Optional, __pyx_t_1) < 0) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_Set); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Set, __pyx_t_1) < 0) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_Tuple); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Tuple, __pyx_t_1) < 0) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_Union); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Union, __pyx_t_1) < 0) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "zeroconf/_protocol/incoming.py":28
 * 
 * from .._dns import (
 *     DNSAddress,             # <<<<<<<<<<<<<<
 *     DNSHinfo,
 *     DNSNsec,
 */
  __pyx_t_2 = PyList_New(8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_DNSAddress);
  __Pyx_GIVEREF(__pyx_n_s_DNSAddress);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_DNSAddress);
  __Pyx_INCREF(__pyx_n_s_DNSHinfo);
  __Pyx_GIVEREF(__pyx_n_s_DNSHinfo);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_n_s_DNSHinfo);
  __Pyx_INCREF(__pyx_n_s_DNSNsec);
  __Pyx_GIVEREF(__pyx_n_s_DNSNsec);
  PyList_SET_ITEM(__pyx_t_2, 2, __pyx_n_s_DNSNsec);
  __Pyx_INCREF(__pyx_n_s_DNSPointer);
  __Pyx_GIVEREF(__pyx_n_s_DNSPointer);
  PyList_SET_ITEM(__pyx_t_2, 3, __pyx_n_s_DNSPointer);
  __Pyx_INCREF(__pyx_n_s_DNSQuestion);
  __Pyx_GIVEREF(__pyx_n_s_DNSQuestion);
  PyList_SET_ITEM(__pyx_t_2, 4, __pyx_n_s_DNSQuestion);
  __Pyx_INCREF(__pyx_n_s_DNSRecord);
  __Pyx_GIVEREF(__pyx_n_s_DNSRecord);
  PyList_SET_ITEM(__pyx_t_2, 5, __pyx_n_s_DNSRecord);
  __Pyx_INCREF(__pyx_n_s_DNSService);
  __Pyx_GIVEREF(__pyx_n_s_DNSService);
  PyList_SET_ITEM(__pyx_t_2, 6, __pyx_n_s_DNSService);
  __Pyx_INCREF(__pyx_n_s_DNSText);
  __Pyx_GIVEREF(__pyx_n_s_DNSText);
  PyList_SET_ITEM(__pyx_t_2, 7, __pyx_n_s_DNSText);

  /* "zeroconf/_protocol/incoming.py":27
 * from typing import Any, Dict, List, Optional, Set, Tuple, Union
 * 
 * from .._dns import (             # <<<<<<<<<<<<<<
 *     DNSAddress,
 *     DNSHinfo,
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_dns, __pyx_t_2, 2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "zeroconf/_protocol/incoming.py":37
 *     DNSText,
 * )
 * from .._exceptions import IncomingDecodeError             # <<<<<<<<<<<<<<
 * from .._logger import log
 * from .._utils.time import current_time_millis
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 37, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_IncomingDecodeError);
  __Pyx_GIVEREF(__pyx_n_s_IncomingDecodeError);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_IncomingDecodeError);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_exceptions, __pyx_t_1, 2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 37, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_IncomingDecodeError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 37, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_XGOTREF(__pyx_v_8zeroconf_9_protocol_8incoming_IncomingDecodeError);
  __Pyx_DECREF_SET(__pyx_v_8zeroconf_9_protocol_8incoming_IncomingDecodeError, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "zeroconf/_protocol/incoming.py":38
 * )
 * from .._exceptions import IncomingDecodeError
 * from .._logger import log             # <<<<<<<<<<<<<<
 * from .._utils.time import current_time_millis
 * from ..const import (
 */
  __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 38, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_log);
  __Pyx_GIVEREF(__pyx_n_s_log);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_log);
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_logger, __pyx_t_2, 2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 38, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_log); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 38, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_log, __pyx_t_2) < 0) __PYX_ERR(0, 38, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "zeroconf/_protocol/incoming.py":39
 * from .._exceptions import IncomingDecodeError
 * from .._logger import log
 * from .._utils.time import current_time_millis             # <<<<<<<<<<<<<<
 * from ..const import (
 *     _FLAGS_QR_MASK,
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 39, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_current_time_millis);
  __Pyx_GIVEREF(__pyx_n_s_current_time_millis);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_current_time_millis);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_utils_time, __pyx_t_1, 2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 39, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_current_time_millis); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 39, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_XGOTREF(__pyx_v_8zeroconf_9_protocol_8incoming_current_time_millis);
  __Pyx_DECREF_SET(__pyx_v_8zeroconf_9_protocol_8incoming_current_time_millis, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "zeroconf/_protocol/incoming.py":41
 * from .._utils.time import current_time_millis
 * from ..const import (
 *     _FLAGS_QR_MASK,             # <<<<<<<<<<<<<<
 *     _FLAGS_QR_QUERY,
 *     _FLAGS_QR_RESPONSE,
 */
  __pyx_t_2 = PyList_New(13); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 41, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_FLAGS_QR_MASK);
  __Pyx_GIVEREF(__pyx_n_s_FLAGS_QR_MASK);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_FLAGS_QR_MASK);
  __Pyx_INCREF(__pyx_n_s_FLAGS_QR_QUERY);
  __Pyx_GIVEREF(__pyx_n_s_FLAGS_QR_QUERY);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_n_s_FLAGS_QR_QUERY);
  __Pyx_INCREF(__pyx_n_s_FLAGS_QR_RESPONSE);
  __Pyx_GIVEREF(__pyx_n_s_FLAGS_QR_RESPONSE);
  PyList_SET_ITEM(__pyx_t_2, 2, __pyx_n_s_FLAGS_QR_RESPONSE);
  __Pyx_INCREF(__pyx_n_s_FLAGS_TC);
  __Pyx_GIVEREF(__pyx_n_s_FLAGS_TC);
  PyList_SET_ITEM(__pyx_t_2, 3, __pyx_n_s_FLAGS_TC);
  __Pyx_INCREF(__pyx_n_s_TYPE_A);
  __Pyx_GIVEREF(__pyx_n_s_TYPE_A);
  PyList_SET_ITEM(__pyx_t_2, 4, __pyx_n_s_TYPE_A);
  __Pyx_INCREF(__pyx_n_s_TYPE_AAAA);
  __Pyx_GIVEREF(__pyx_n_s_TYPE_AAAA);
  PyList_SET_ITEM(__pyx_t_2, 5, __pyx_n_s_TYPE_AAAA);
  __Pyx_INCREF(__pyx_n_s_TYPE_CNAME);
  __Pyx_GIVEREF(__pyx_n_s_TYPE_CNAME);
  PyList_SET_ITEM(__pyx_t_2, 6, __pyx_n_s_TYPE_CNAME);
  __Pyx_INCREF(__pyx_n_s_TYPE_HINFO);
  __Pyx_GIVEREF(__pyx_n_s_TYPE_HINFO);
  PyList_SET_ITEM(__pyx_t_2, 7, __pyx_n_s_TYPE_HINFO);
  __Pyx_INCREF(__pyx_n_s_TYPE_NSEC);
  __Pyx_GIVEREF(__pyx_n_s_TYPE_NSEC);
  PyList_SET_ITEM(__pyx_t_2, 8, __pyx_n_s_TYPE_NSEC);
  __Pyx_INCREF(__pyx_n_s_TYPE_PTR);
  __Pyx_GIVEREF(__pyx_n_s_TYPE_PTR);
  PyList_SET_ITEM(__pyx_t_2, 9, __pyx_n_s_TYPE_PTR);
  __Pyx_INCREF(__pyx_n_s_TYPE_SRV);
  __Pyx_GIVEREF(__pyx_n_s_TYPE_SRV);
  PyList_SET_ITEM(__pyx_t_2, 10, __pyx_n_s_TYPE_SRV);
  __Pyx_INCREF(__pyx_n_s_TYPE_TXT);
  __Pyx_GIVEREF(__pyx_n_s_TYPE_TXT);
  PyList_SET_ITEM(__pyx_t_2, 11, __pyx_n_s_TYPE_TXT);
  __Pyx_INCREF(__pyx_n_s_TYPES);
  __Pyx_GIVEREF(__pyx_n_s_TYPES);
  PyList_SET_ITEM(__pyx_t_2, 12, __pyx_n_s_TYPES);

  /* "zeroconf/_protocol/incoming.py":40
 * from .._logger import log
 * from .._utils.time import current_time_millis
 * from ..const import (             # <<<<<<<<<<<<<<
 *     _FLAGS_QR_MASK,
 *     _FLAGS_QR_QUERY,
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_const, __pyx_t_2, 2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_FLAGS_QR_MASK); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_As_unsigned_int(__pyx_t_2); if (unlikely((__pyx_t_3 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(0, 40, __pyx_L1_error)
  __pyx_v_8zeroconf_9_protocol_8incoming__FLAGS_QR_MASK = __pyx_t_3;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_FLAGS_QR_QUERY); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_As_unsigned_int(__pyx_t_2); if (unlikely((__pyx_t_3 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(0, 40, __pyx_L1_error)
  __pyx_v_8zeroconf_9_protocol_8incoming__FLAGS_QR_QUERY = __pyx_t_3;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_FLAGS_QR_RESPONSE); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_As_unsigned_int(__pyx_t_2); if (unlikely((__pyx_t_3 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(0, 40, __pyx_L1_error)
  __pyx_v_8zeroconf_9_protocol_8incoming__FLAGS_QR_RESPONSE = __pyx_t_3;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_FLAGS_TC); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_As_unsigned_int(__pyx_t_2); if (unlikely((__pyx_t_3 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(0, 40, __pyx_L1_error)
  __pyx_v_8zeroconf_9_protocol_8incoming__FLAGS_TC = __pyx_t_3;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_TYPE_A); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_As_unsigned_int(__pyx_t_2); if (unlikely((__pyx_t_3 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(0, 40, __pyx_L1_error)
  __pyx_v_8zeroconf_9_protocol_8incoming__TYPE_A = __pyx_t_3;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_TYPE_AAAA); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_As_unsigned_int(__pyx_t_2); if (unlikely((__pyx_t_3 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(0, 40, __pyx_L1_error)
  __pyx_v_8zeroconf_9_protocol_8incoming__TYPE_AAAA = __pyx_t_3;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_TYPE_CNAME); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_As_unsigned_int(__pyx_t_2); if (unlikely((__pyx_t_3 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(0, 40, __pyx_L1_error)
  __pyx_v_8zeroconf_9_protocol_8incoming__TYPE_CNAME = __pyx_t_3;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_TYPE_HINFO); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_As_unsigned_int(__pyx_t_2); if (unlikely((__pyx_t_3 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(0, 40, __pyx_L1_error)
  __pyx_v_8zeroconf_9_protocol_8incoming__TYPE_HINFO = __pyx_t_3;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_TYPE_NSEC); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_As_unsigned_int(__pyx_t_2); if (unlikely((__pyx_t_3 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(0, 40, __pyx_L1_error)
  __pyx_v_8zeroconf_9_protocol_8incoming__TYPE_NSEC = __pyx_t_3;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_TYPE_PTR); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_As_unsigned_int(__pyx_t_2); if (unlikely((__pyx_t_3 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(0, 40, __pyx_L1_error)
  __pyx_v_8zeroconf_9_protocol_8incoming__TYPE_PTR = __pyx_t_3;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_TYPE_SRV); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_As_unsigned_int(__pyx_t_2); if (unlikely((__pyx_t_3 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(0, 40, __pyx_L1_error)
  __pyx_v_8zeroconf_9_protocol_8incoming__TYPE_SRV = __pyx_t_3;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_TYPE_TXT); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_As_unsigned_int(__pyx_t_2); if (unlikely((__pyx_t_3 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(0, 40, __pyx_L1_error)
  __pyx_v_8zeroconf_9_protocol_8incoming__TYPE_TXT = __pyx_t_3;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_TYPES); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_TYPES, __pyx_t_2) < 0) __PYX_ERR(0, 53, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "zeroconf/_protocol/incoming.py":56
 * )
 * 
 * DNS_COMPRESSION_HEADER_LEN = 1             # <<<<<<<<<<<<<<
 * DNS_COMPRESSION_POINTER_LEN = 2
 * MAX_DNS_LABELS = 128
 */
  __pyx_v_8zeroconf_9_protocol_8incoming_DNS_COMPRESSION_HEADER_LEN = 1;

  /* "zeroconf/_protocol/incoming.py":57
 * 
 * DNS_COMPRESSION_HEADER_LEN = 1
 * DNS_COMPRESSION_POINTER_LEN = 2             # <<<<<<<<<<<<<<
 * MAX_DNS_LABELS = 128
 * MAX_NAME_LENGTH = 253
 */
  __pyx_v_8zeroconf_9_protocol_8incoming_DNS_COMPRESSION_POINTER_LEN = 2;

  /* "zeroconf/_protocol/incoming.py":58
 * DNS_COMPRESSION_HEADER_LEN = 1
 * DNS_COMPRESSION_POINTER_LEN = 2
 * MAX_DNS_LABELS = 128             # <<<<<<<<<<<<<<
 * MAX_NAME_LENGTH = 253
 * 
 */
  __pyx_v_8zeroconf_9_protocol_8incoming_MAX_DNS_LABELS = 0x80;

  /* "zeroconf/_protocol/incoming.py":59
 * DNS_COMPRESSION_POINTER_LEN = 2
 * MAX_DNS_LABELS = 128
 * MAX_NAME_LENGTH = 253             # <<<<<<<<<<<<<<
 * 
 * DECODE_EXCEPTIONS = (IndexError, struct.error, IncomingDecodeError)
 */
  __pyx_v_8zeroconf_9_protocol_8incoming_MAX_NAME_LENGTH = 0xFD;

  /* "zeroconf/_protocol/incoming.py":61
 * MAX_NAME_LENGTH = 253
 * 
 * DECODE_EXCEPTIONS = (IndexError, struct.error, IncomingDecodeError)             # <<<<<<<<<<<<<<
 * 
 * UNPACK_3H = struct.Struct(b'!3H').unpack_from
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_struct); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 61, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_error); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 61, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 61, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_builtin_IndexError);
  __Pyx_GIVEREF(__pyx_builtin_IndexError);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_builtin_IndexError);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_8zeroconf_9_protocol_8incoming_IncomingDecodeError);
  __Pyx_GIVEREF(__pyx_v_8zeroconf_9_protocol_8incoming_IncomingDecodeError);
  PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_8zeroconf_9_protocol_8incoming_IncomingDecodeError);
  __pyx_t_2 = 0;
  __Pyx_XGOTREF(__pyx_v_8zeroconf_9_protocol_8incoming_DECODE_EXCEPTIONS);
  __Pyx_DECREF_SET(__pyx_v_8zeroconf_9_protocol_8incoming_DECODE_EXCEPTIONS, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_t_1 = 0;

  /* "zeroconf/_protocol/incoming.py":63
 * DECODE_EXCEPTIONS = (IndexError, struct.error, IncomingDecodeError)
 * 
 * UNPACK_3H = struct.Struct(b'!3H').unpack_from             # <<<<<<<<<<<<<<
 * UNPACK_6H = struct.Struct(b'!6H').unpack_from
 * UNPACK_HH = struct.Struct(b'!HH').unpack_from
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_struct); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 63, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_Struct); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 63, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 63, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_unpack_from); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 63, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_XGOTREF(__pyx_v_8zeroconf_9_protocol_8incoming_UNPACK_3H);
  __Pyx_DECREF_SET(__pyx_v_8zeroconf_9_protocol_8incoming_UNPACK_3H, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;

  /* "zeroconf/_protocol/incoming.py":64
 * 
 * UNPACK_3H = struct.Struct(b'!3H').unpack_from
 * UNPACK_6H = struct.Struct(b'!6H').unpack_from             # <<<<<<<<<<<<<<
 * UNPACK_HH = struct.Struct(b'!HH').unpack_from
 * UNPACK_HHiH = struct.Struct(b'!HHiH').unpack_from
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_struct); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 64, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_Struct); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 64, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__5, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 64, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_unpack_from); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 64, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_XGOTREF(__pyx_v_8zeroconf_9_protocol_8incoming_UNPACK_6H);
  __Pyx_DECREF_SET(__pyx_v_8zeroconf_9_protocol_8incoming_UNPACK_6H, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_t_1 = 0;

  /* "zeroconf/_protocol/incoming.py":65
 * UNPACK_3H = struct.Struct(b'!3H').unpack_from
 * UNPACK_6H = struct.Struct(b'!6H').unpack_from
 * UNPACK_HH = struct.Struct(b'!HH').unpack_from             # <<<<<<<<<<<<<<
 * UNPACK_HHiH = struct.Struct(b'!HHiH').unpack_from
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_struct); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 65, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_Struct); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 65, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 65, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_unpack_from); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 65, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_XGOTREF(__pyx_v_8zeroconf_9_protocol_8incoming_UNPACK_HH);
  __Pyx_DECREF_SET(__pyx_v_8zeroconf_9_protocol_8incoming_UNPACK_HH, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;

  /* "zeroconf/_protocol/incoming.py":66
 * UNPACK_6H = struct.Struct(b'!6H').unpack_from
 * UNPACK_HH = struct.Struct(b'!HH').unpack_from
 * UNPACK_HHiH = struct.Struct(b'!HHiH').unpack_from             # <<<<<<<<<<<<<<
 * 
 * _seen_logs: Dict[str, Union[int, tuple]] = {}
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_struct); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 66, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_Struct); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 66, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__7, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 66, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_unpack_from); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 66, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_XGOTREF(__pyx_v_8zeroconf_9_protocol_8incoming_UNPACK_HHiH);
  __Pyx_DECREF_SET(__pyx_v_8zeroconf_9_protocol_8incoming_UNPACK_HHiH, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_t_1 = 0;

  /* "zeroconf/_protocol/incoming.py":68
 * UNPACK_HHiH = struct.Struct(b'!HHiH').unpack_from
 * 
 * _seen_logs: Dict[str, Union[int, tuple]] = {}             # <<<<<<<<<<<<<<
 * _str = str
 * 
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_seen_logs, __pyx_t_1) < 0) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "zeroconf/_protocol/incoming.py":69
 * 
 * _seen_logs: Dict[str, Union[int, tuple]] = {}
 * _str = str             # <<<<<<<<<<<<<<
 * 
 * 
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_str, ((PyObject *)(&PyUnicode_Type))) < 0) __PYX_ERR(0, 69, __pyx_L1_error)

  /* "zeroconf/_protocol/incoming.py":76
 * 
 *     __slots__ = (
 *         "_did_read_others",             # <<<<<<<<<<<<<<
 *         'flags',
 *         'offset',
 */
  if (PyDict_SetItem((PyObject *)__pyx_ptype_8zeroconf_9_protocol_8incoming_DNSIncoming->tp_dict, __pyx_n_s_slots, __pyx_tuple__8) < 0) __PYX_ERR(0, 75, __pyx_L1_error)
  PyType_Modified(__pyx_ptype_8zeroconf_9_protocol_8incoming_DNSIncoming);

  /* "zeroconf/_protocol/incoming.py":130
 *             )
 * 
 *     def is_query(self) -> bool:             # <<<<<<<<<<<<<<
 *         """Returns true if this is a query."""
 *         return (self.flags & _FLAGS_QR_MASK) == _FLAGS_QR_QUERY
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 130, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_return, ((PyObject*)&PyBool_Type)) < 0) __PYX_ERR(0, 130, __pyx_L1_error)
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_8zeroconf_9_protocol_8incoming_11DNSIncoming_3is_query, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_DNSIncoming_is_query, NULL, __pyx_n_s_zeroconf__protocol_incoming, __pyx_d, ((PyObject *)__pyx_codeobj__10)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 130, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_2, __pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_8zeroconf_9_protocol_8incoming_DNSIncoming->tp_dict, __pyx_n_s_is_query, __pyx_t_2) < 0) __PYX_ERR(0, 130, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_8zeroconf_9_protocol_8incoming_DNSIncoming);

  /* "zeroconf/_protocol/incoming.py":134
 *         return (self.flags & _FLAGS_QR_MASK) == _FLAGS_QR_QUERY
 * 
 *     def is_response(self) -> bool:             # <<<<<<<<<<<<<<
 *         """Returns true if this is a response."""
 *         return (self.flags & _FLAGS_QR_MASK) == _FLAGS_QR_RESPONSE
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 134, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_return, ((PyObject*)&PyBool_Type)) < 0) __PYX_ERR(0, 134, __pyx_L1_error)
  __pyx_t_1 = __Pyx_CyFunction_New(&__pyx_mdef_8zeroconf_9_protocol_8incoming_11DNSIncoming_5is_response, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_DNSIncoming_is_response, NULL, __pyx_n_s_zeroconf__protocol_incoming, __pyx_d, ((PyObject *)__pyx_codeobj__12)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 134, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_1, __pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_8zeroconf_9_protocol_8incoming_DNSIncoming->tp_dict, __pyx_n_s_is_response, __pyx_t_1) < 0) __PYX_ERR(0, 134, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_8zeroconf_9_protocol_8incoming_DNSIncoming);

  /* "zeroconf/_protocol/incoming.py":152
 * 
 *     @classmethod
 *     def _log_exception_debug(cls, *logger_data: Any) -> None:             # <<<<<<<<<<<<<<
 *         log_exc_info = False
 *         exc_info = sys.exc_info()
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 152, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_Any); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 152, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_logger_data, __pyx_t_2) < 0) __PYX_ERR(0, 152, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_return, Py_None) < 0) __PYX_ERR(0, 152, __pyx_L1_error)
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_8zeroconf_9_protocol_8incoming_11DNSIncoming_7_log_exception_debug, __Pyx_CYFUNCTION_CLASSMETHOD | __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_DNSIncoming__log_exception_debug, NULL, __pyx_n_s_zeroconf__protocol_incoming, __pyx_d, ((PyObject *)__pyx_codeobj__14)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 152, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_2, __pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_8zeroconf_9_protocol_8incoming_DNSIncoming->tp_dict, __pyx_n_s_log_exception_debug, __pyx_t_2) < 0) __PYX_ERR(0, 152, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_8zeroconf_9_protocol_8incoming_DNSIncoming);

  /* "zeroconf/_protocol/incoming.py":151
 *         self.valid = True
 * 
 *     @classmethod             # <<<<<<<<<<<<<<
 *     def _log_exception_debug(cls, *logger_data: Any) -> None:
 *         log_exc_info = False
 */
  __Pyx_GetNameInClass(__pyx_t_2, (PyObject *)__pyx_ptype_8zeroconf_9_protocol_8incoming_DNSIncoming, __pyx_n_s_log_exception_debug); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 152, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_Method_ClassMethod(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 151, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_8zeroconf_9_protocol_8incoming_DNSIncoming->tp_dict, __pyx_n_s_log_exception_debug, __pyx_t_1) < 0) __PYX_ERR(0, 152, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_8zeroconf_9_protocol_8incoming_DNSIncoming);

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */
  __pyx_t_1 = __Pyx_CyFunction_New(&__pyx_mdef_8zeroconf_9_protocol_8incoming_11DNSIncoming_11__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_DNSIncoming___reduce_cython, NULL, __pyx_n_s_zeroconf__protocol_incoming, __pyx_d, ((PyObject *)__pyx_codeobj__16)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_8zeroconf_9_protocol_8incoming_DNSIncoming->tp_dict, __pyx_n_s_reduce_cython, __pyx_t_1) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_8zeroconf_9_protocol_8incoming_DNSIncoming);

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_DNSIncoming, (type(self), 0x69f643b, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_DNSIncoming__set_state(self, __pyx_state)
 */
  __pyx_t_1 = __Pyx_CyFunction_New(&__pyx_mdef_8zeroconf_9_protocol_8incoming_11DNSIncoming_13__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_DNSIncoming___setstate_cython, NULL, __pyx_n_s_zeroconf__protocol_incoming, __pyx_d, ((PyObject *)__pyx_codeobj__18)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_8zeroconf_9_protocol_8incoming_DNSIncoming->tp_dict, __pyx_n_s_setstate_cython, __pyx_t_1) < 0) __PYX_ERR(2, 16, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_8zeroconf_9_protocol_8incoming_DNSIncoming);

  /* "(tree fragment)":1
 * def __pyx_unpickle_DNSIncoming(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_1 = __Pyx_CyFunction_New(&__pyx_mdef_8zeroconf_9_protocol_8incoming_1__pyx_unpickle_DNSIncoming, 0, __pyx_n_s_pyx_unpickle_DNSIncoming, NULL, __pyx_n_s_zeroconf__protocol_incoming, __pyx_d, ((PyObject *)__pyx_codeobj__20)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_DNSIncoming, __pyx_t_1) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "zeroconf/_protocol/incoming.py":1
 * """ Multicast DNS Service Discovery for Python, v0.14-wmcbrine             # <<<<<<<<<<<<<<
 *     Copyright 2003 Paul Scott-Murphy, 2014 William McBrine
 * 
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_1) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init zeroconf._protocol.incoming", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_CLEAR(__pyx_m);
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init zeroconf._protocol.incoming");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (__Pyx_PyUnicode_GET_LENGTH(**name) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (__Pyx_PyUnicode_GET_LENGTH(**argname) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* ArgTypeTest */
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (exact) {
        #if PY_MAJOR_VERSION == 2
        if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    }
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
    return 0;
}

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = __Pyx_PyFrame_GetLocalsplus(f);
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = Py_TYPE(func)->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallNoArg */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#if defined(__Pyx_CyFunction_USED) && defined(NDEBUG)
    if (likely(PyCFunction_Check(func) || __Pyx_CyFunction_Check(func)))
#else
    if (likely(PyCFunction_Check(func)))
#endif
    {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* PyCFunctionFastCall */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)(void*)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)(void*)meth)) (self, args, nargs);
    }
}
#endif

/* PyObjectCallOneArg */
#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (__Pyx_PyFastCFunction_Check(func)) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* GetTopmostException */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem *
__Pyx_PyErr_GetTopmostException(PyThreadState *tstate)
{
    _PyErr_StackItem *exc_info = tstate->exc_info;
    while ((exc_info->exc_type == NULL || exc_info->exc_type == Py_None) &&
           exc_info->previous_item != NULL)
    {
        exc_info = exc_info->previous_item;
    }
    return exc_info;
}
#endif

/* SaveResetException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    *type = exc_info->exc_type;
    *value = exc_info->exc_value;
    *tb = exc_info->exc_traceback;
    #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    #endif
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = type;
    exc_info->exc_value = value;
    exc_info->exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
#endif

/* PyErrExceptionMatches */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    PyObject *exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
    if (unlikely(PyTuple_Check(err)))
        return __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    return __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
}
#endif

/* GetException */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb)
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb)
#endif
{
    PyObject *local_type, *local_value, *local_tb;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
    #endif
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    #if CYTHON_USE_EXC_INFO_STACK
    {
        _PyErr_StackItem *exc_info = tstate->exc_info;
        tmp_type = exc_info->exc_type;
        tmp_value = exc_info->exc_value;
        tmp_tb = exc_info->exc_traceback;
        exc_info->exc_type = local_type;
        exc_info->exc_value = local_value;
        exc_info->exc_traceback = local_tb;
    }
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

/* KeywordStringCheck */
static int __Pyx_CheckKeywordStrings(
    PyObject *kwdict,
    const char* function_name,
    int kw_allowed)
{
    PyObject* key = 0;
    Py_ssize_t pos = 0;
#if CYTHON_COMPILING_IN_PYPY
    if (!kw_allowed && PyDict_Next(kwdict, &pos, &key, 0))
        goto invalid_keyword;
    return 1;
#else
    while (PyDict_Next(kwdict, &pos, &key, 0)) {
        #if PY_MAJOR_VERSION < 3
        if (unlikely(!PyString_Check(key)))
        #endif
            if (unlikely(!PyUnicode_Check(key)))
                goto invalid_keyword_type;
    }
    if ((!kw_allowed) && unlikely(key))
        goto invalid_keyword;
    return 1;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    return 0;
#endif
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
    return 0;
}

/* PyDictVersioning */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* GetModuleGlobalName */
#if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1
    result = _PyDict_GetItem_KnownHash(__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    } else if (unlikely(PyErr_Occurred())) {
        return NULL;
    }
#else
    result = PyDict_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
#endif
#else
    result = PyObject_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* PyObjectCall2Args */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2) {
    PyObject *args, *result = NULL;
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyFunction_FastCall(function, args, 2);
    }
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyCFunction_FastCall(function, args, 2);
    }
    #endif
    args = PyTuple_New(2);
    if (unlikely(!args)) goto done;
    Py_INCREF(arg1);
    PyTuple_SET_ITEM(args, 0, arg1);
    Py_INCREF(arg2);
    PyTuple_SET_ITEM(args, 1, arg2);
    Py_INCREF(function);
    result = __Pyx_PyObject_Call(function, args, NULL);
    Py_DECREF(args);
    Py_DECREF(function);
done:
    return result;
}

/* RaiseTooManyValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* RaiseNeedMoreValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* IterFinish */
static CYTHON_INLINE int __Pyx_IterFinish(void) {
#if CYTHON_FAST_THREAD_STATE
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject* exc_type = tstate->curexc_type;
    if (unlikely(exc_type)) {
        if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) {
            PyObject *exc_value, *exc_tb;
            exc_value = tstate->curexc_value;
            exc_tb = tstate->curexc_traceback;
            tstate->curexc_type = 0;
            tstate->curexc_value = 0;
            tstate->curexc_traceback = 0;
            Py_DECREF(exc_type);
            Py_XDECREF(exc_value);
            Py_XDECREF(exc_tb);
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#else
    if (unlikely(PyErr_Occurred())) {
        if (likely(PyErr_ExceptionMatches(PyExc_StopIteration))) {
            PyErr_Clear();
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#endif
}

/* UnpackItemEndCheck */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected) {
    if (unlikely(retval)) {
        Py_DECREF(retval);
        __Pyx_RaiseTooManyValuesError(expected);
        return -1;
    }
    return __Pyx_IterFinish();
}

/* decode_c_bytes */
static CYTHON_INLINE PyObject* __Pyx_decode_c_bytes(
         const char* cstring, Py_ssize_t length, Py_ssize_t start, Py_ssize_t stop,
         const char* encoding, const char* errors,
         PyObject* (*decode_func)(const char *s, Py_ssize_t size, const char *errors)) {
    if (unlikely((start < 0) | (stop < 0))) {
        if (start < 0) {
            start += length;
            if (start < 0)
                start = 0;
        }
        if (stop < 0)
            stop += length;
    }
    if (stop > length)
        stop = length;
    if (unlikely(stop <= start))
        return __Pyx_NewRef(__pyx_empty_unicode);
    length = stop - start;
    cstring += start;
    if (decode_func) {
        return decode_func(cstring, length, errors);
    } else {
        return PyUnicode_Decode(cstring, length, encoding, errors);
    }
}

/* CIntToDigits */
static const char DIGIT_PAIRS_10[2*10*10+1] = {
    "00010203040506070809"
    "10111213141516171819"
    "20212223242526272829"
    "30313233343536373839"
    "40414243444546474849"
    "50515253545556575859"
    "60616263646566676869"
    "70717273747576777879"
    "80818283848586878889"
    "90919293949596979899"
};
static const char DIGIT_PAIRS_8[2*8*8+1] = {
    "0001020304050607"
    "1011121314151617"
    "2021222324252627"
    "3031323334353637"
    "4041424344454647"
    "5051525354555657"
    "6061626364656667"
    "7071727374757677"
};
static const char DIGITS_HEX[2*16+1] = {
    "0123456789abcdef"
    "0123456789ABCDEF"
};

/* BuildPyUnicode */
static PyObject* __Pyx_PyUnicode_BuildFromAscii(Py_ssize_t ulength, char* chars, int clength,
                                                int prepend_sign, char padding_char) {
    PyObject *uval;
    Py_ssize_t uoffset = ulength - clength;
#if CYTHON_USE_UNICODE_INTERNALS
    Py_ssize_t i;
#if CYTHON_PEP393_ENABLED
    void *udata;
    uval = PyUnicode_New(ulength, 127);
    if (unlikely(!uval)) return NULL;
    udata = PyUnicode_DATA(uval);
#else
    Py_UNICODE *udata;
    uval = PyUnicode_FromUnicode(NULL, ulength);
    if (unlikely(!uval)) return NULL;
    udata = PyUnicode_AS_UNICODE(uval);
#endif
    if (uoffset > 0) {
        i = 0;
        if (prepend_sign) {
            __Pyx_PyUnicode_WRITE(PyUnicode_1BYTE_KIND, udata, 0, '-');
            i++;
        }
        for (; i < uoffset; i++) {
            __Pyx_PyUnicode_WRITE(PyUnicode_1BYTE_KIND, udata, i, padding_char);
        }
    }
    for (i=0; i < clength; i++) {
        __Pyx_PyUnicode_WRITE(PyUnicode_1BYTE_KIND, udata, uoffset+i, chars[i]);
    }
#else
    {
        PyObject *sign = NULL, *padding = NULL;
        uval = NULL;
        if (uoffset > 0) {
            prepend_sign = !!prepend_sign;
            if (uoffset > prepend_sign) {
                padding = PyUnicode_FromOrdinal(padding_char);
                if (likely(padding) && uoffset > prepend_sign + 1) {
                    PyObject *tmp;
                    PyObject *repeat = PyInt_FromSsize_t(uoffset - prepend_sign);
                    if (unlikely(!repeat)) goto done_or_error;
                    tmp = PyNumber_Multiply(padding, repeat);
                    Py_DECREF(repeat);
                    Py_DECREF(padding);
                    padding = tmp;
                }
                if (unlikely(!padding)) goto done_or_error;
            }
            if (prepend_sign) {
                sign = PyUnicode_FromOrdinal('-');
                if (unlikely(!sign)) goto done_or_error;
            }
        }
        uval = PyUnicode_DecodeASCII(chars, clength, NULL);
        if (likely(uval) && padding) {
            PyObject *tmp = PyNumber_Add(padding, uval);
            Py_DECREF(uval);
            uval = tmp;
        }
        if (likely(uval) && sign) {
            PyObject *tmp = PyNumber_Add(sign, uval);
            Py_DECREF(uval);
            uval = tmp;
        }
done_or_error:
        Py_XDECREF(padding);
        Py_XDECREF(sign);
    }
#endif
    return uval;
}

/* CIntToPyUnicode */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_From_unsigned_int(unsigned int value, Py_ssize_t width, char padding_char, char format_char) {
    char digits[sizeof(unsigned int)*3+2];
    char *dpos, *end = digits + sizeof(unsigned int)*3+2;
    const char *hex_digits = DIGITS_HEX;
    Py_ssize_t length, ulength;
    int prepend_sign, last_one_off;
    unsigned int remaining;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const unsigned int neg_one = (unsigned int) -1, const_zero = (unsigned int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (format_char == 'X') {
        hex_digits += 16;
        format_char = 'x';
    }
    remaining = value;
    last_one_off = 0;
    dpos = end;
    do {
        int digit_pos;
        switch (format_char) {
        case 'o':
            digit_pos = abs((int)(remaining % (8*8)));
            remaining = (unsigned int) (remaining / (8*8));
            dpos -= 2;
            memcpy(dpos, DIGIT_PAIRS_8 + digit_pos * 2, 2);
            last_one_off = (digit_pos < 8);
            break;
        case 'd':
            digit_pos = abs((int)(remaining % (10*10)));
            remaining = (unsigned int) (remaining / (10*10));
            dpos -= 2;
            memcpy(dpos, DIGIT_PAIRS_10 + digit_pos * 2, 2);
            last_one_off = (digit_pos < 10);
            break;
        case 'x':
            *(--dpos) = hex_digits[abs((int)(remaining % 16))];
            remaining = (unsigned int) (remaining / 16);
            break;
        default:
            assert(0);
            break;
        }
    } while (unlikely(remaining != 0));
    if (last_one_off) {
        assert(*dpos == '0');
        dpos++;
    }
    length = end - dpos;
    ulength = length;
    prepend_sign = 0;
    if (!is_unsigned && value <= neg_one) {
        if (padding_char == ' ' || width <= length + 1) {
            *(--dpos) = '-';
            ++length;
        } else {
            prepend_sign = 1;
        }
        ++ulength;
    }
    if (width > ulength) {
        ulength = width;
    }
    if (ulength == 1) {
        return PyUnicode_FromOrdinal(*dpos);
    }
    return __Pyx_PyUnicode_BuildFromAscii(ulength, dpos, (int) length, prepend_sign, padding_char);
}

/* JoinPyUnicode */
static PyObject* __Pyx_PyUnicode_Join(PyObject* value_tuple, Py_ssize_t value_count, Py_ssize_t result_ulength,
                                      CYTHON_UNUSED Py_UCS4 max_char) {
#if CYTHON_USE_UNICODE_INTERNALS && CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    PyObject *result_uval;
    int result_ukind;
    Py_ssize_t i, char_pos;
    void *result_udata;
#if CYTHON_PEP393_ENABLED
    result_uval = PyUnicode_New(result_ulength, max_char);
    if (unlikely(!result_uval)) return NULL;
    result_ukind = (max_char <= 255) ? PyUnicode_1BYTE_KIND : (max_char <= 65535) ? PyUnicode_2BYTE_KIND : PyUnicode_4BYTE_KIND;
    result_udata = PyUnicode_DATA(result_uval);
#else
    result_uval = PyUnicode_FromUnicode(NULL, result_ulength);
    if (unlikely(!result_uval)) return NULL;
    result_ukind = sizeof(Py_UNICODE);
    result_udata = PyUnicode_AS_UNICODE(result_uval);
#endif
    char_pos = 0;
    for (i=0; i < value_count; i++) {
        int ukind;
        Py_ssize_t ulength;
        void *udata;
        PyObject *uval = PyTuple_GET_ITEM(value_tuple, i);
        if (unlikely(__Pyx_PyUnicode_READY(uval)))
            goto bad;
        ulength = __Pyx_PyUnicode_GET_LENGTH(uval);
        if (unlikely(!ulength))
            continue;
        if (unlikely(char_pos + ulength < 0))
            goto overflow;
        ukind = __Pyx_PyUnicode_KIND(uval);
        udata = __Pyx_PyUnicode_DATA(uval);
        if (!CYTHON_PEP393_ENABLED || ukind == result_ukind) {
            memcpy((char *)result_udata + char_pos * result_ukind, udata, (size_t) (ulength * result_ukind));
        } else {
            #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030300F0 || defined(_PyUnicode_FastCopyCharacters)
            _PyUnicode_FastCopyCharacters(result_uval, char_pos, uval, 0, ulength);
            #else
            Py_ssize_t j;
            for (j=0; j < ulength; j++) {
                Py_UCS4 uchar = __Pyx_PyUnicode_READ(ukind, udata, j);
                __Pyx_PyUnicode_WRITE(result_ukind, result_udata, char_pos+j, uchar);
            }
            #endif
        }
        char_pos += ulength;
    }
    return result_uval;
overflow:
    PyErr_SetString(PyExc_OverflowError, "join() result is too long for a Python string");
bad:
    Py_DECREF(result_uval);
    return NULL;
#else
    result_ulength++;
    value_count++;
    return PyUnicode_Join(__pyx_empty_unicode, value_tuple);
#endif
}

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* RaiseException */
#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_FAST_THREAD_STATE
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#else
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* pyfrozenset_new */
static CYTHON_INLINE PyObject* __Pyx_PyFrozenSet_New(PyObject* it) {
    if (it) {
        PyObject* result;
#if CYTHON_COMPILING_IN_PYPY
        PyObject* args;
        args = PyTuple_Pack(1, it);
        if (unlikely(!args))
            return NULL;
        result = PyObject_Call((PyObject*)&PyFrozenSet_Type, args, NULL);
        Py_DECREF(args);
        return result;
#else
        if (PyFrozenSet_CheckExact(it)) {
            Py_INCREF(it);
            return it;
        }
        result = PyFrozenSet_New(it);
        if (unlikely(!result))
            return NULL;
        if ((PY_VERSION_HEX >= 0x031000A1) || likely(PySet_GET_SIZE(result)))
            return result;
        Py_DECREF(result);
#endif
    }
#if CYTHON_USE_TYPE_SLOTS
    return PyFrozenSet_Type.tp_new(&PyFrozenSet_Type, __pyx_empty_tuple, NULL);
#else
    return PyObject_Call((PyObject*)&PyFrozenSet_Type, __pyx_empty_tuple, NULL);
#endif
}

/* PySetContains */
static int __Pyx_PySet_ContainsUnhashable(PyObject *set, PyObject *key) {
    int result = -1;
    if (PySet_Check(key) && PyErr_ExceptionMatches(PyExc_TypeError)) {
        PyObject *tmpkey;
        PyErr_Clear();
        tmpkey = __Pyx_PyFrozenSet_New(key);
        if (tmpkey != NULL) {
            result = PySet_Contains(set, tmpkey);
            Py_DECREF(tmpkey);
        }
    }
    return result;
}
static CYTHON_INLINE int __Pyx_PySet_ContainsTF(PyObject* key, PyObject* set, int eq) {
    int result = PySet_Contains(set, key);
    if (unlikely(result < 0)) {
        result = __Pyx_PySet_ContainsUnhashable(set, key);
    }
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* UnpackUnboundCMethod */
static int __Pyx_TryUnpackUnboundCMethod(__Pyx_CachedCFunction* target) {
    PyObject *method;
    method = __Pyx_PyObject_GetAttrStr(target->type, *target->method_name);
    if (unlikely(!method))
        return -1;
    target->method = method;
#if CYTHON_COMPILING_IN_CPYTHON
    #if PY_MAJOR_VERSION >= 3
    if (likely(__Pyx_TypeCheck(method, &PyMethodDescr_Type)))
    #endif
    {
        PyMethodDescrObject *descr = (PyMethodDescrObject*) method;
        target->func = descr->d_method->ml_meth;
        target->flag = descr->d_method->ml_flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_STACKLESS);
    }
#endif
    return 0;
}

/* CallUnboundCMethod1 */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg) {
    if (likely(cfunc->func)) {
        int flag = cfunc->flag;
        if (flag == METH_O) {
            return (*(cfunc->func))(self, arg);
        } else if (PY_VERSION_HEX >= 0x030600B1 && flag == METH_FASTCALL) {
            #if PY_VERSION_HEX >= 0x030700A0
                return (*(__Pyx_PyCFunctionFast)(void*)(PyCFunction)cfunc->func)(self, &arg, 1);
            #else
                return (*(__Pyx_PyCFunctionFastWithKeywords)(void*)(PyCFunction)cfunc->func)(self, &arg, 1, NULL);
            #endif
        } else if (PY_VERSION_HEX >= 0x030700A0 && flag == (METH_FASTCALL | METH_KEYWORDS)) {
            return (*(__Pyx_PyCFunctionFastWithKeywords)(void*)(PyCFunction)cfunc->func)(self, &arg, 1, NULL);
        }
    }
    return __Pyx__CallUnboundCMethod1(cfunc, self, arg);
}
#endif
static PyObject* __Pyx__CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg){
    PyObject *args, *result = NULL;
    if (unlikely(!cfunc->func && !cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_COMPILING_IN_CPYTHON
    if (cfunc->func && (cfunc->flag & METH_VARARGS)) {
        args = PyTuple_New(1);
        if (unlikely(!args)) goto bad;
        Py_INCREF(arg);
        PyTuple_SET_ITEM(args, 0, arg);
        if (cfunc->flag & METH_KEYWORDS)
            result = (*(PyCFunctionWithKeywords)(void*)(PyCFunction)cfunc->func)(self, args, NULL);
        else
            result = (*cfunc->func)(self, args);
    } else {
        args = PyTuple_New(2);
        if (unlikely(!args)) goto bad;
        Py_INCREF(self);
        PyTuple_SET_ITEM(args, 0, self);
        Py_INCREF(arg);
        PyTuple_SET_ITEM(args, 1, arg);
        result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
    }
#else
    args = PyTuple_Pack(2, self, arg);
    if (unlikely(!args)) goto bad;
    result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
#endif
bad:
    Py_XDECREF(args);
    return result;
}

/* CallUnboundCMethod2 */
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030600B1
static CYTHON_INLINE PyObject *__Pyx_CallUnboundCMethod2(__Pyx_CachedCFunction *cfunc, PyObject *self, PyObject *arg1, PyObject *arg2) {
    if (likely(cfunc->func)) {
        PyObject *args[2] = {arg1, arg2};
        if (cfunc->flag == METH_FASTCALL) {
            #if PY_VERSION_HEX >= 0x030700A0
            return (*(__Pyx_PyCFunctionFast)(void*)(PyCFunction)cfunc->func)(self, args, 2);
            #else
            return (*(__Pyx_PyCFunctionFastWithKeywords)(void*)(PyCFunction)cfunc->func)(self, args, 2, NULL);
            #endif
        }
        #if PY_VERSION_HEX >= 0x030700A0
        if (cfunc->flag == (METH_FASTCALL | METH_KEYWORDS))
            return (*(__Pyx_PyCFunctionFastWithKeywords)(void*)(PyCFunction)cfunc->func)(self, args, 2, NULL);
        #endif
    }
    return __Pyx__CallUnboundCMethod2(cfunc, self, arg1, arg2);
}
#endif
static PyObject* __Pyx__CallUnboundCMethod2(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg1, PyObject* arg2){
    PyObject *args, *result = NULL;
    if (unlikely(!cfunc->func && !cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_COMPILING_IN_CPYTHON
    if (cfunc->func && (cfunc->flag & METH_VARARGS)) {
        args = PyTuple_New(2);
        if (unlikely(!args)) goto bad;
        Py_INCREF(arg1);
        PyTuple_SET_ITEM(args, 0, arg1);
        Py_INCREF(arg2);
        PyTuple_SET_ITEM(args, 1, arg2);
        if (cfunc->flag & METH_KEYWORDS)
            result = (*(PyCFunctionWithKeywords)(void*)(PyCFunction)cfunc->func)(self, args, NULL);
        else
            result = (*cfunc->func)(self, args);
    } else {
        args = PyTuple_New(3);
        if (unlikely(!args)) goto bad;
        Py_INCREF(self);
        PyTuple_SET_ITEM(args, 0, self);
        Py_INCREF(arg1);
        PyTuple_SET_ITEM(args, 1, arg1);
        Py_INCREF(arg2);
        PyTuple_SET_ITEM(args, 2, arg2);
        result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
    }
#else
    args = PyTuple_Pack(3, self, arg1, arg2);
    if (unlikely(!args)) goto bad;
    result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
#endif
bad:
    Py_XDECREF(args);
    return result;
}

/* dict_getitem_default */
static PyObject* __Pyx_PyDict_GetItemDefault(PyObject* d, PyObject* key, PyObject* default_value) {
    PyObject* value;
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (unlikely(PyErr_Occurred()))
            return NULL;
        value = default_value;
    }
    Py_INCREF(value);
    if ((1));
#else
    if (PyString_CheckExact(key) || PyUnicode_CheckExact(key) || PyInt_CheckExact(key)) {
        value = PyDict_GetItem(d, key);
        if (unlikely(!value)) {
            value = default_value;
        }
        Py_INCREF(value);
    }
#endif
    else {
        if (default_value == Py_None)
            value = __Pyx_CallUnboundCMethod1(&__pyx_umethod_PyDict_Type_get, d, key);
        else
            value = __Pyx_CallUnboundCMethod2(&__pyx_umethod_PyDict_Type_get, d, key, default_value);
    }
    return value;
}

/* GetAttr */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *o, PyObject *n) {
#if CYTHON_USE_TYPE_SLOTS
#if PY_MAJOR_VERSION >= 3
    if (likely(PyUnicode_Check(n)))
#else
    if (likely(PyString_Check(n)))
#endif
        return __Pyx_PyObject_GetAttrStr(o, n);
#endif
    return PyObject_GetAttr(o, n);
}

/* GetAttr3 */
static PyObject *__Pyx_GetAttr3Default(PyObject *d) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (unlikely(!__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        return NULL;
    __Pyx_PyErr_Clear();
    Py_INCREF(d);
    return d;
}
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *o, PyObject *n, PyObject *d) {
    PyObject *r = __Pyx_GetAttr(o, n);
    return (likely(r)) ? r : __Pyx_GetAttr3Default(d);
}

/* Import */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if ((1) && (strchr(__Pyx_MODULE_NAME, '.'))) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, (PyObject *)NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* ImportFrom */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* HasAttr */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *o, PyObject *n) {
    PyObject *r;
    if (unlikely(!__Pyx_PyBaseString_Check(n))) {
        PyErr_SetString(PyExc_TypeError,
                        "hasattr(): attribute name must be string");
        return -1;
    }
    r = __Pyx_GetAttr(o, n);
    if (unlikely(!r)) {
        PyErr_Clear();
        return 0;
    } else {
        Py_DECREF(r);
        return 1;
    }
}

/* PyObject_GenericGetAttrNoDict */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject *__Pyx_RaiseGenericGetAttributeError(PyTypeObject *tp, PyObject *attr_name) {
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'%.50s' object has no attribute '%U'",
                 tp->tp_name, attr_name);
#else
                 "'%.50s' object has no attribute '%.400s'",
                 tp->tp_name, PyString_AS_STRING(attr_name));
#endif
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name) {
    PyObject *descr;
    PyTypeObject *tp = Py_TYPE(obj);
    if (unlikely(!PyString_Check(attr_name))) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    assert(!tp->tp_dictoffset);
    descr = _PyType_Lookup(tp, attr_name);
    if (unlikely(!descr)) {
        return __Pyx_RaiseGenericGetAttributeError(tp, attr_name);
    }
    Py_INCREF(descr);
    #if PY_MAJOR_VERSION < 3
    if (likely(PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_HAVE_CLASS)))
    #endif
    {
        descrgetfunc f = Py_TYPE(descr)->tp_descr_get;
        if (unlikely(f)) {
            PyObject *res = f(descr, obj, (PyObject *)tp);
            Py_DECREF(descr);
            return res;
        }
    }
    return descr;
}
#endif

/* PyObject_GenericGetAttr */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name) {
    if (unlikely(Py_TYPE(obj)->tp_dictoffset)) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    return __Pyx_PyObject_GenericGetAttrNoDict(obj, attr_name);
}
#endif

/* SetVTable */
static int __Pyx_SetVtable(PyObject *dict, void *vtable) {
#if PY_VERSION_HEX >= 0x02070000
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
#else
    PyObject *ob = PyCObject_FromVoidPtr(vtable, 0);
#endif
    if (!ob)
        goto bad;
    if (PyDict_SetItem(dict, __pyx_n_s_pyx_vtable, ob) < 0)
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* PyObjectGetAttrStrNoError */
static void __Pyx_PyObject_GetAttrStr_ClearAttributeError(void) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (likely(__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        __Pyx_PyErr_Clear();
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name) {
    PyObject *result;
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_TYPE_SLOTS && PY_VERSION_HEX >= 0x030700B1
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro == PyObject_GenericGetAttr)) {
        return _PyObject_GenericGetAttrWithDict(obj, attr_name, NULL, 1);
    }
#endif
    result = __Pyx_PyObject_GetAttrStr(obj, attr_name);
    if (unlikely(!result)) {
        __Pyx_PyObject_GetAttrStr_ClearAttributeError();
    }
    return result;
}

/* SetupReduce */
static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStr(meth, __pyx_n_s_name);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_getstate = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
    PyObject *getstate = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    getstate = _PyType_Lookup((PyTypeObject*)type_obj, __pyx_n_s_getstate);
#else
    getstate = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_getstate);
    if (!getstate && PyErr_Occurred()) {
        goto __PYX_BAD;
    }
#endif
    if (getstate) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_getstate = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_getstate);
#else
        object_getstate = __Pyx_PyObject_GetAttrStrNoError((PyObject*)&PyBaseObject_Type, __pyx_n_s_getstate);
        if (!object_getstate && PyErr_Occurred()) {
            goto __PYX_BAD;
        }
#endif
        if (object_getstate != getstate) {
            goto __PYX_GOOD;
        }
    }
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_ex); if (unlikely(!reduce_ex)) goto __PYX_BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce); if (unlikely(!reduce)) goto __PYX_BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_n_s_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_reduce_cython);
            if (likely(reduce_cython)) {
                ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce, reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
            } else if (reduce == object_reduce || PyErr_Occurred()) {
                goto __PYX_BAD;
            }
            setstate = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_n_s_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_setstate_cython);
                if (likely(setstate_cython)) {
                    ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate, setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                    ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                } else if (!setstate || PyErr_Occurred()) {
                    goto __PYX_BAD;
                }
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto __PYX_GOOD;
__PYX_BAD:
    if (!PyErr_Occurred())
        PyErr_Format(PyExc_RuntimeError, "Unable to initialize pickling for %s", ((PyTypeObject*)type_obj)->tp_name);
    ret = -1;
__PYX_GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
    Py_XDECREF(object_getstate);
    Py_XDECREF(getstate);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* TypeImport */
#ifndef __PYX_HAVE_RT_ImportType_0_29_35
#define __PYX_HAVE_RT_ImportType_0_29_35
static PyTypeObject *__Pyx_ImportType_0_29_35(PyObject *module, const char *module_name, const char *class_name,
    size_t size, size_t alignment, enum __Pyx_ImportType_CheckSize_0_29_35 check_size)
{
    PyObject *result = 0;
    char warning[200];
    Py_ssize_t basicsize;
    Py_ssize_t itemsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
    PyObject *py_itemsize;
#endif
    result = PyObject_GetAttrString(module, class_name);
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
    itemsize = ((PyTypeObject *)result)->tp_itemsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
    py_itemsize = PyObject_GetAttrString(result, "__itemsize__");
    if (!py_itemsize)
        goto bad;
    itemsize = PyLong_AsSsize_t(py_itemsize);
    Py_DECREF(py_itemsize);
    py_itemsize = 0;
    if (itemsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if (itemsize) {
        if (size % alignment) {
            alignment = size % alignment;
        }
        if (itemsize < (Py_ssize_t)alignment)
            itemsize = (Py_ssize_t)alignment;
    }
    if ((size_t)(basicsize + itemsize) < size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    if (check_size == __Pyx_ImportType_CheckSize_Error_0_29_35 && (size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    else if (check_size == __Pyx_ImportType_CheckSize_Warn_0_29_35 && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(result);
    return NULL;
}
#endif

/* GetVTable */
static void* __Pyx_GetVtable(PyObject *dict) {
    void* ptr;
    PyObject *ob = PyObject_GetItem(dict, __pyx_n_s_pyx_vtable);
    if (!ob)
        goto bad;
#if PY_VERSION_HEX >= 0x02070000
    ptr = PyCapsule_GetPointer(ob, 0);
#else
    ptr = PyCObject_AsVoidPtr(ob);
#endif
    if (!ptr && !PyErr_Occurred())
        PyErr_SetString(PyExc_RuntimeError, "invalid vtable found for imported type");
    Py_DECREF(ob);
    return ptr;
bad:
    Py_XDECREF(ob);
    return NULL;
}

/* FetchCommonType */
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type) {
    PyObject* fake_module;
    PyTypeObject* cached_type = NULL;
    fake_module = PyImport_AddModule((char*) "_cython_" CYTHON_ABI);
    if (!fake_module) return NULL;
    Py_INCREF(fake_module);
    cached_type = (PyTypeObject*) PyObject_GetAttrString(fake_module, type->tp_name);
    if (cached_type) {
        if (!PyType_Check((PyObject*)cached_type)) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s is not a type object",
                type->tp_name);
            goto bad;
        }
        if (cached_type->tp_basicsize != type->tp_basicsize) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s has the wrong size, try recompiling",
                type->tp_name);
            goto bad;
        }
    } else {
        if (!PyErr_ExceptionMatches(PyExc_AttributeError)) goto bad;
        PyErr_Clear();
        if (PyType_Ready(type) < 0) goto bad;
        if (PyObject_SetAttrString(fake_module, type->tp_name, (PyObject*) type) < 0)
            goto bad;
        Py_INCREF(type);
        cached_type = type;
    }
done:
    Py_DECREF(fake_module);
    return cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}

/* CythonFunctionShared */
#include <structmember.h>
static PyObject *
__Pyx_CyFunction_get_doc(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *closure)
{
    if (unlikely(op->func_doc == NULL)) {
        if (op->func.m_ml->ml_doc) {
#if PY_MAJOR_VERSION >= 3
            op->func_doc = PyUnicode_FromString(op->func.m_ml->ml_doc);
#else
            op->func_doc = PyString_FromString(op->func.m_ml->ml_doc);
#endif
            if (unlikely(op->func_doc == NULL))
                return NULL;
        } else {
            Py_INCREF(Py_None);
            return Py_None;
        }
    }
    Py_INCREF(op->func_doc);
    return op->func_doc;
}
static int
__Pyx_CyFunction_set_doc(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp = op->func_doc;
    if (value == NULL) {
        value = Py_None;
    }
    Py_INCREF(value);
    op->func_doc = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_name(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    if (unlikely(op->func_name == NULL)) {
#if PY_MAJOR_VERSION >= 3
        op->func_name = PyUnicode_InternFromString(op->func.m_ml->ml_name);
#else
        op->func_name = PyString_InternFromString(op->func.m_ml->ml_name);
#endif
        if (unlikely(op->func_name == NULL))
            return NULL;
    }
    Py_INCREF(op->func_name);
    return op->func_name;
}
static int
__Pyx_CyFunction_set_name(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    tmp = op->func_name;
    Py_INCREF(value);
    op->func_name = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_qualname(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    Py_INCREF(op->func_qualname);
    return op->func_qualname;
}
static int
__Pyx_CyFunction_set_qualname(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    tmp = op->func_qualname;
    Py_INCREF(value);
    op->func_qualname = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_self(__pyx_CyFunctionObject *m, CYTHON_UNUSED void *closure)
{
    PyObject *self;
    self = m->func_closure;
    if (self == NULL)
        self = Py_None;
    Py_INCREF(self);
    return self;
}
static PyObject *
__Pyx_CyFunction_get_dict(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    if (unlikely(op->func_dict == NULL)) {
        op->func_dict = PyDict_New();
        if (unlikely(op->func_dict == NULL))
            return NULL;
    }
    Py_INCREF(op->func_dict);
    return op->func_dict;
}
static int
__Pyx_CyFunction_set_dict(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
    if (unlikely(value == NULL)) {
        PyErr_SetString(PyExc_TypeError,
               "function's dictionary may not be deleted");
        return -1;
    }
    if (unlikely(!PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
               "setting function's dictionary to a non-dict");
        return -1;
    }
    tmp = op->func_dict;
    Py_INCREF(value);
    op->func_dict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_globals(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    Py_INCREF(op->func_globals);
    return op->func_globals;
}
static PyObject *
__Pyx_CyFunction_get_closure(CYTHON_UNUSED __pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    Py_INCREF(Py_None);
    return Py_None;
}
static PyObject *
__Pyx_CyFunction_get_code(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    PyObject* result = (op->func_code) ? op->func_code : Py_None;
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_init_defaults(__pyx_CyFunctionObject *op) {
    int result = 0;
    PyObject *res = op->defaults_getter((PyObject *) op);
    if (unlikely(!res))
        return -1;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    op->defaults_tuple = PyTuple_GET_ITEM(res, 0);
    Py_INCREF(op->defaults_tuple);
    op->defaults_kwdict = PyTuple_GET_ITEM(res, 1);
    Py_INCREF(op->defaults_kwdict);
    #else
    op->defaults_tuple = PySequence_ITEM(res, 0);
    if (unlikely(!op->defaults_tuple)) result = -1;
    else {
        op->defaults_kwdict = PySequence_ITEM(res, 1);
        if (unlikely(!op->defaults_kwdict)) result = -1;
    }
    #endif
    Py_DECREF(res);
    return result;
}
static int
__Pyx_CyFunction_set_defaults(__pyx_CyFunctionObject *op, PyObject* value, CYTHON_UNUSED void *context) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyTuple_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__defaults__ must be set to a tuple object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_tuple;
    op->defaults_tuple = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_defaults(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context) {
    PyObject* result = op->defaults_tuple;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_tuple;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_kwdefaults(__pyx_CyFunctionObject *op, PyObject* value, CYTHON_UNUSED void *context) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__kwdefaults__ must be set to a dict object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_kwdict;
    op->defaults_kwdict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_kwdefaults(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context) {
    PyObject* result = op->defaults_kwdict;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_kwdict;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_annotations(__pyx_CyFunctionObject *op, PyObject* value, CYTHON_UNUSED void *context) {
    PyObject* tmp;
    if (!value || value == Py_None) {
        value = NULL;
    } else if (!PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__annotations__ must be set to a dict object");
        return -1;
    }
    Py_XINCREF(value);
    tmp = op->func_annotations;
    op->func_annotations = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_annotations(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context) {
    PyObject* result = op->func_annotations;
    if (unlikely(!result)) {
        result = PyDict_New();
        if (unlikely(!result)) return NULL;
        op->func_annotations = result;
    }
    Py_INCREF(result);
    return result;
}
static PyGetSetDef __pyx_CyFunction_getsets[] = {
    {(char *) "func_doc", (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "__doc__",  (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "func_name", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__name__", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__qualname__", (getter)__Pyx_CyFunction_get_qualname, (setter)__Pyx_CyFunction_set_qualname, 0, 0},
    {(char *) "__self__", (getter)__Pyx_CyFunction_get_self, 0, 0, 0},
    {(char *) "func_dict", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "__dict__", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "func_globals", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "__globals__", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "func_closure", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "__closure__", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "func_code", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "__code__", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "func_defaults", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__defaults__", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__kwdefaults__", (getter)__Pyx_CyFunction_get_kwdefaults, (setter)__Pyx_CyFunction_set_kwdefaults, 0, 0},
    {(char *) "__annotations__", (getter)__Pyx_CyFunction_get_annotations, (setter)__Pyx_CyFunction_set_annotations, 0, 0},
    {0, 0, 0, 0, 0}
};
static PyMemberDef __pyx_CyFunction_members[] = {
    {(char *) "__module__", T_OBJECT, offsetof(PyCFunctionObject, m_module), PY_WRITE_RESTRICTED, 0},
    {0, 0, 0,  0, 0}
};
static PyObject *
__Pyx_CyFunction_reduce(__pyx_CyFunctionObject *m, CYTHON_UNUSED PyObject *args)
{
#if PY_MAJOR_VERSION >= 3
    Py_INCREF(m->func_qualname);
    return m->func_qualname;
#else
    return PyString_FromString(m->func.m_ml->ml_name);
#endif
}
static PyMethodDef __pyx_CyFunction_methods[] = {
    {"__reduce__", (PyCFunction)__Pyx_CyFunction_reduce, METH_VARARGS, 0},
    {0, 0, 0, 0}
};
#if PY_VERSION_HEX < 0x030500A0
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func_weakreflist)
#else
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func.m_weakreflist)
#endif
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject *op, PyMethodDef *ml, int flags, PyObject* qualname,
                                       PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    if (unlikely(op == NULL))
        return NULL;
    op->flags = flags;
    __Pyx_CyFunction_weakreflist(op) = NULL;
    op->func.m_ml = ml;
    op->func.m_self = (PyObject *) op;
    Py_XINCREF(closure);
    op->func_closure = closure;
    Py_XINCREF(module);
    op->func.m_module = module;
    op->func_dict = NULL;
    op->func_name = NULL;
    Py_INCREF(qualname);
    op->func_qualname = qualname;
    op->func_doc = NULL;
    op->func_classobj = NULL;
    op->func_globals = globals;
    Py_INCREF(op->func_globals);
    Py_XINCREF(code);
    op->func_code = code;
    op->defaults_pyobjects = 0;
    op->defaults_size = 0;
    op->defaults = NULL;
    op->defaults_tuple = NULL;
    op->defaults_kwdict = NULL;
    op->defaults_getter = NULL;
    op->func_annotations = NULL;
    return (PyObject *) op;
}
static int
__Pyx_CyFunction_clear(__pyx_CyFunctionObject *m)
{
    Py_CLEAR(m->func_closure);
    Py_CLEAR(m->func.m_module);
    Py_CLEAR(m->func_dict);
    Py_CLEAR(m->func_name);
    Py_CLEAR(m->func_qualname);
    Py_CLEAR(m->func_doc);
    Py_CLEAR(m->func_globals);
    Py_CLEAR(m->func_code);
    Py_CLEAR(m->func_classobj);
    Py_CLEAR(m->defaults_tuple);
    Py_CLEAR(m->defaults_kwdict);
    Py_CLEAR(m->func_annotations);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_XDECREF(pydefaults[i]);
        PyObject_Free(m->defaults);
        m->defaults = NULL;
    }
    return 0;
}
static void __Pyx__CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    if (__Pyx_CyFunction_weakreflist(m) != NULL)
        PyObject_ClearWeakRefs((PyObject *) m);
    __Pyx_CyFunction_clear(m);
    PyObject_GC_Del(m);
}
static void __Pyx_CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    PyObject_GC_UnTrack(m);
    __Pyx__CyFunction_dealloc(m);
}
static int __Pyx_CyFunction_traverse(__pyx_CyFunctionObject *m, visitproc visit, void *arg)
{
    Py_VISIT(m->func_closure);
    Py_VISIT(m->func.m_module);
    Py_VISIT(m->func_dict);
    Py_VISIT(m->func_name);
    Py_VISIT(m->func_qualname);
    Py_VISIT(m->func_doc);
    Py_VISIT(m->func_globals);
    Py_VISIT(m->func_code);
    Py_VISIT(m->func_classobj);
    Py_VISIT(m->defaults_tuple);
    Py_VISIT(m->defaults_kwdict);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_VISIT(pydefaults[i]);
    }
    return 0;
}
static PyObject *__Pyx_CyFunction_descr_get(PyObject *func, PyObject *obj, PyObject *type)
{
#if PY_MAJOR_VERSION < 3
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    if (m->flags & __Pyx_CYFUNCTION_STATICMETHOD) {
        Py_INCREF(func);
        return func;
    }
    if (m->flags & __Pyx_CYFUNCTION_CLASSMETHOD) {
        if (type == NULL)
            type = (PyObject *)(Py_TYPE(obj));
        return __Pyx_PyMethod_New(func, type, (PyObject *)(Py_TYPE(type)));
    }
    if (obj == Py_None)
        obj = NULL;
#endif
    return __Pyx_PyMethod_New(func, obj, type);
}
static PyObject*
__Pyx_CyFunction_repr(__pyx_CyFunctionObject *op)
{
#if PY_MAJOR_VERSION >= 3
    return PyUnicode_FromFormat("<cyfunction %U at %p>",
                                op->func_qualname, (void *)op);
#else
    return PyString_FromFormat("<cyfunction %s at %p>",
                               PyString_AsString(op->func_qualname), (void *)op);
#endif
}
static PyObject * __Pyx_CyFunction_CallMethod(PyObject *func, PyObject *self, PyObject *arg, PyObject *kw) {
    PyCFunctionObject* f = (PyCFunctionObject*)func;
    PyCFunction meth = f->m_ml->ml_meth;
    Py_ssize_t size;
    switch (f->m_ml->ml_flags & (METH_VARARGS | METH_KEYWORDS | METH_NOARGS | METH_O)) {
    case METH_VARARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0))
            return (*meth)(self, arg);
        break;
    case METH_VARARGS | METH_KEYWORDS:
        return (*(PyCFunctionWithKeywords)(void*)meth)(self, arg, kw);
    case METH_NOARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 0))
                return (*meth)(self, NULL);
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes no arguments (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    case METH_O:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 1)) {
                PyObject *result, *arg0;
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                arg0 = PyTuple_GET_ITEM(arg, 0);
                #else
                arg0 = PySequence_ITEM(arg, 0); if (unlikely(!arg0)) return NULL;
                #endif
                result = (*meth)(self, arg0);
                #if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
                Py_DECREF(arg0);
                #endif
                return result;
            }
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes exactly one argument (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags in "
                        "__Pyx_CyFunction_Call. METH_OLDARGS is no "
                        "longer supported!");
        return NULL;
    }
    PyErr_Format(PyExc_TypeError, "%.200s() takes no keyword arguments",
                 f->m_ml->ml_name);
    return NULL;
}
static CYTHON_INLINE PyObject *__Pyx_CyFunction_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    return __Pyx_CyFunction_CallMethod(func, ((PyCFunctionObject*)func)->m_self, arg, kw);
}
static PyObject *__Pyx_CyFunction_CallAsMethod(PyObject *func, PyObject *args, PyObject *kw) {
    PyObject *result;
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *) func;
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        Py_ssize_t argc;
        PyObject *new_args;
        PyObject *self;
        argc = PyTuple_GET_SIZE(args);
        new_args = PyTuple_GetSlice(args, 1, argc);
        if (unlikely(!new_args))
            return NULL;
        self = PyTuple_GetItem(args, 0);
        if (unlikely(!self)) {
            Py_DECREF(new_args);
#if PY_MAJOR_VERSION > 2
            PyErr_Format(PyExc_TypeError,
                         "unbound method %.200S() needs an argument",
                         cyfunc->func_qualname);
#else
            PyErr_SetString(PyExc_TypeError,
                            "unbound method needs an argument");
#endif
            return NULL;
        }
        result = __Pyx_CyFunction_CallMethod(func, self, new_args, kw);
        Py_DECREF(new_args);
    } else {
        result = __Pyx_CyFunction_Call(func, args, kw);
    }
    return result;
}
static PyTypeObject __pyx_CyFunctionType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    "cython_function_or_method",
    sizeof(__pyx_CyFunctionObject),
    0,
    (destructor) __Pyx_CyFunction_dealloc,
    0,
    0,
    0,
#if PY_MAJOR_VERSION < 3
    0,
#else
    0,
#endif
    (reprfunc) __Pyx_CyFunction_repr,
    0,
    0,
    0,
    0,
    __Pyx_CyFunction_CallAsMethod,
    0,
    0,
    0,
    0,
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,
    0,
    (traverseproc) __Pyx_CyFunction_traverse,
    (inquiry) __Pyx_CyFunction_clear,
    0,
#if PY_VERSION_HEX < 0x030500A0
    offsetof(__pyx_CyFunctionObject, func_weakreflist),
#else
    offsetof(PyCFunctionObject, m_weakreflist),
#endif
    0,
    0,
    __pyx_CyFunction_methods,
    __pyx_CyFunction_members,
    __pyx_CyFunction_getsets,
    0,
    0,
    __Pyx_CyFunction_descr_get,
    0,
    offsetof(__pyx_CyFunctionObject, func_dict),
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if PY_VERSION_HEX >= 0x030400a1
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
    0,
#endif
#if PY_VERSION_HEX >= 0x030C0000
    0,
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
    0,
#endif
};
static int __pyx_CyFunction_init(void) {
    __pyx_CyFunctionType = __Pyx_FetchCommonType(&__pyx_CyFunctionType_type);
    if (unlikely(__pyx_CyFunctionType == NULL)) {
        return -1;
    }
    return 0;
}
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *func, size_t size, int pyobjects) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults = PyObject_Malloc(size);
    if (unlikely(!m->defaults))
        return PyErr_NoMemory();
    memset(m->defaults, 0, size);
    m->defaults_pyobjects = pyobjects;
    m->defaults_size = size;
    return m->defaults;
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *func, PyObject *tuple) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_tuple = tuple;
    Py_INCREF(tuple);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_kwdict = dict;
    Py_INCREF(dict);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->func_annotations = dict;
    Py_INCREF(dict);
}

/* CythonFunction */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml, int flags, PyObject* qualname,
                                      PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    PyObject *op = __Pyx_CyFunction_Init(
        PyObject_GC_New(__pyx_CyFunctionObject, __pyx_CyFunctionType),
        ml, flags, qualname, closure, module, globals, code
    );
    if (likely(op)) {
        PyObject_GC_Track(op);
    }
    return op;
}

/* ClassMethod */
static PyObject* __Pyx_Method_ClassMethod(PyObject *method) {
#if CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM <= 0x05080000
    if (PyObject_TypeCheck(method, &PyWrapperDescr_Type)) {
        return PyClassMethod_New(method);
    }
#else
#if CYTHON_COMPILING_IN_PYSTON || CYTHON_COMPILING_IN_PYPY
    if (PyMethodDescr_Check(method))
#else
    #if PY_MAJOR_VERSION == 2
    static PyTypeObject *methoddescr_type = NULL;
    if (methoddescr_type == NULL) {
       PyObject *meth = PyObject_GetAttrString((PyObject*)&PyList_Type, "append");
       if (!meth) return NULL;
       methoddescr_type = Py_TYPE(meth);
       Py_DECREF(meth);
    }
    #else
    PyTypeObject *methoddescr_type = &PyMethodDescr_Type;
    #endif
    if (__Pyx_TypeCheck(method, methoddescr_type))
#endif
    {
        PyMethodDescrObject *descr = (PyMethodDescrObject *)method;
        #if PY_VERSION_HEX < 0x03020000
        PyTypeObject *d_type = descr->d_type;
        #else
        PyTypeObject *d_type = descr->d_common.d_type;
        #endif
        return PyDescr_NewClassMethod(d_type, descr->d_method);
    }
#endif
    else if (PyMethod_Check(method)) {
        return PyClassMethod_New(PyMethod_GET_FUNCTION(method));
    }
    else {
        return PyClassMethod_New(method);
    }
}

/* GetNameInClass */
static PyObject *__Pyx_GetGlobalNameAfterAttributeLookup(PyObject *name) {
    PyObject *result;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (unlikely(!__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        return NULL;
    __Pyx_PyErr_Clear();
    __Pyx_GetModuleGlobalNameUncached(result, name);
    return result;
}
static PyObject *__Pyx__GetNameInClass(PyObject *nmspace, PyObject *name) {
    PyObject *result;
    result = __Pyx_PyObject_GetAttrStr(nmspace, name);
    if (!result) {
        result = __Pyx_GetGlobalNameAfterAttributeLookup(name);
    }
    return result;
}

/* CLineInTraceback */
#ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(CYTHON_UNUSED PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    if (unlikely(!__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, *cython_runtime_dict,
            __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_n_s_cline_in_traceback))
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        (void) PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, ((size_t)new_max) * sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
#if PY_VERSION_HEX >= 0x030b00a6
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = NULL;
    PyObject *py_funcname = NULL;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_srcfile = NULL;
    py_srcfile = PyString_FromString(filename);
    if (!py_srcfile) goto bad;
    #endif
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        funcname = PyUnicode_AsUTF8(py_funcname);
        if (!funcname) goto bad;
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        if (!py_funcname) goto bad;
        #endif
    }
    #if PY_MAJOR_VERSION < 3
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    #else
    py_code = PyCode_NewEmpty(filename, funcname, py_line);
    #endif
    Py_XDECREF(py_funcname);  // XDECREF since it's only set on Py3 if cline
    return py_code;
bad:
    Py_XDECREF(py_funcname);
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_srcfile);
    #endif
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject *ptype, *pvalue, *ptraceback;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) {
            /* If the code object creation fails, then we should clear the
               fetched exception references and propagate the new exception */
            Py_XDECREF(ptype);
            Py_XDECREF(pvalue);
            Py_XDECREF(ptraceback);
            goto bad;
        }
        __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntFromPyVerify */
#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_unsigned_int(unsigned int value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const unsigned int neg_one = (unsigned int) -1, const_zero = (unsigned int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(unsigned int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(unsigned int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(unsigned int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(unsigned int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(unsigned int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(unsigned int),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
static CYTHON_INLINE unsigned int __Pyx_PyInt_As_unsigned_int(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const unsigned int neg_one = (unsigned int) -1, const_zero = (unsigned int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(unsigned int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(unsigned int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (unsigned int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (unsigned int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(unsigned int, digit, digits[0])
                case 2:
                    if (8 * sizeof(unsigned int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) >= 2 * PyLong_SHIFT) {
                            return (unsigned int) (((((unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(unsigned int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) >= 3 * PyLong_SHIFT) {
                            return (unsigned int) (((((((unsigned int)digits[2]) << PyLong_SHIFT) | (unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(unsigned int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) >= 4 * PyLong_SHIFT) {
                            return (unsigned int) (((((((((unsigned int)digits[3]) << PyLong_SHIFT) | (unsigned int)digits[2]) << PyLong_SHIFT) | (unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (unsigned int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(unsigned int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(unsigned int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (unsigned int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(unsigned int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(unsigned int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(unsigned int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) - 1 > 2 * PyLong_SHIFT) {
                            return (unsigned int) (((unsigned int)-1)*(((((unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(unsigned int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) - 1 > 2 * PyLong_SHIFT) {
                            return (unsigned int) ((((((unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(unsigned int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) - 1 > 3 * PyLong_SHIFT) {
                            return (unsigned int) (((unsigned int)-1)*(((((((unsigned int)digits[2]) << PyLong_SHIFT) | (unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(unsigned int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) - 1 > 3 * PyLong_SHIFT) {
                            return (unsigned int) ((((((((unsigned int)digits[2]) << PyLong_SHIFT) | (unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(unsigned int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) - 1 > 4 * PyLong_SHIFT) {
                            return (unsigned int) (((unsigned int)-1)*(((((((((unsigned int)digits[3]) << PyLong_SHIFT) | (unsigned int)digits[2]) << PyLong_SHIFT) | (unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(unsigned int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) - 1 > 4 * PyLong_SHIFT) {
                            return (unsigned int) ((((((((((unsigned int)digits[3]) << PyLong_SHIFT) | (unsigned int)digits[2]) << PyLong_SHIFT) | (unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(unsigned int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(unsigned int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            unsigned int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (unsigned int) -1;
        }
    } else {
        unsigned int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (unsigned int) -1;
        val = __Pyx_PyInt_As_unsigned_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to unsigned int");
    return (unsigned int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to unsigned int");
    return (unsigned int) -1;
}

/* CIntFromPy */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* FastTypeChecks */
#if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        #if PY_MAJOR_VERSION < 3
        if (likely(exc_type == t)) return 1;
        #endif
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* CheckBinaryVersion */
static int __Pyx_check_binary_version(void) {
    char ctversion[5];
    int same=1, i, found_dot;
    const char* rt_from_call = Py_GetVersion();
    PyOS_snprintf(ctversion, 5, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    found_dot = 0;
    for (i = 0; i < 4; i++) {
        if (!ctversion[i]) {
            same = (rt_from_call[i] < '0' || rt_from_call[i] > '9');
            break;
        }
        if (rt_from_call[i] != ctversion[i]) {
            same = 0;
            break;
        }
    }
    if (!same) {
        char rtversion[5] = {'\0'};
        char message[200];
        for (i=0; i<4; ++i) {
            if (rt_from_call[i] == '.') {
                if (found_dot) break;
                found_dot = 1;
            } else if (rt_from_call[i] < '0' || rt_from_call[i] > '9') {
                break;
            }
            rtversion[i] = rt_from_call[i];
        }
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* PyObjectSetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_setattro))
        return tp->tp_setattro(obj, attr_name, value);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_setattr))
        return tp->tp_setattr(obj, PyString_AS_STRING(attr_name), value);
#endif
    return PyObject_SetAttr(obj, attr_name, value);
}
#endif

/* VoidPtrExport */
static int __Pyx_ExportVoidPtr(PyObject *name, void *p, const char *sig) {
    PyObject *d;
    PyObject *cobj = 0;
    d = PyDict_GetItem(__pyx_d, __pyx_n_s_pyx_capi);
    Py_XINCREF(d);
    if (!d) {
        d = PyDict_New();
        if (!d)
            goto bad;
        if (__Pyx_PyObject_SetAttrStr(__pyx_m, __pyx_n_s_pyx_capi, d) < 0)
            goto bad;
    }
#if PY_VERSION_HEX >= 0x02070000
    cobj = PyCapsule_New(p, sig, 0);
#else
    cobj = PyCObject_FromVoidPtrAndDesc(p, (void *)sig, 0);
#endif
    if (!cobj)
        goto bad;
    if (PyDict_SetItem(d, name, cobj) < 0)
        goto bad;
    Py_DECREF(cobj);
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(cobj);
    Py_XDECREF(d);
    return -1;
}

/* VoidPtrImport */
#ifndef __PYX_HAVE_RT_ImportVoidPtr_0_29_35
#define __PYX_HAVE_RT_ImportVoidPtr_0_29_35
static int __Pyx_ImportVoidPtr_0_29_35(PyObject *module, const char *name, void **p, const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    d = PyObject_GetAttrString(module, (char *)"__pyx_capi__");
    if (!d)
        goto bad;
    cobj = PyDict_GetItemString(d, name);
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%.200s does not export expected C variable %.200s",
                PyModule_GetName(module), name);
        goto bad;
    }
#if PY_VERSION_HEX >= 0x02070000
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C variable %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), name, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    *p = PyCapsule_GetPointer(cobj, sig);
#else
    {const char *desc, *s1, *s2;
    desc = (const char *)PyCObject_GetDesc(cobj);
    if (!desc)
        goto bad;
    s1 = desc; s2 = sig;
    while (*s1 != '\0' && *s1 == *s2) { s1++; s2++; }
    if (*s1 != *s2) {
        PyErr_Format(PyExc_TypeError,
            "C variable %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), name, sig, desc);
        goto bad;
    }
    *p = PyCObject_AsVoidPtr(cobj);}
#endif
    if (!(*p))
        goto bad;
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(d);
    return -1;
}
#endif

/* InitStrings */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(b);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject* o) {
  if (sizeof(Py_hash_t) == sizeof(Py_ssize_t)) {
    return (Py_hash_t) __Pyx_PyIndex_AsSsize_t(o);
#if PY_MAJOR_VERSION < 3
  } else if (likely(PyInt_CheckExact(o))) {
    return PyInt_AS_LONG(o);
#endif
  } else {
    Py_ssize_t ival;
    PyObject *x;
    x = PyNumber_Index(o);
    if (!x) return -1;
    ival = PyInt_AsLong(x);
    Py_DECREF(x);
    return ival;
  }
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return b ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
